﻿
(function (ko, datacontext) {
	/**
	 * Документ в том виде, в котором он представлен в редакторе пакета
	 */
	datacontext.TaskDocument = function (taskDocument) {
		var self = this;
		self._id = taskDocument.DocumentId;
		
		self.BatchId = ko.observable(taskDocument.BatchId);

		self.Id = ko.observable(taskDocument.DocumentId);
		self.DocumentId = ko.pureComputed({
			read: function () {
				return self.Id();
			},
			write: function (val) {
				self.Id(val);
			}
		});

		self.Version = ko.observable(taskDocument.Version);

		self.TemplateName = ko.observable(taskDocument.TemplateName);

		self.ChildNodes = ko.observableArray(taskDocument.ChildNodes || []);
		self.PagesCount = ko.observable(taskDocument.PagesCount);
		/**
		 * Признак неявного документа - группы страниц (PageGroup)
		 */
		self.IsImplicit = ko.observable(taskDocument.IsImplicit);
		self.HasChanges = ko.computed({
			read: function() {
				return false;
			},
			write: function(val) {

			}
		});
	};

	/**
	 * Скопировать значения из другого документа
	 * @param doc Другой документ
	 * */
	datacontext.TaskDocument.prototype.copyFrom = function (doc) {
		this.BatchId(doc.BatchId());
		this.Version(doc.Version());
		this.TemplateName(doc.TemplateName());
		this.PagesCount(doc.PagesCount());
		this.IsImplicit(doc.IsImplicit());

		// Скопируем страницы
		var currentPages = this.ChildNodes();
		var newPages = [];

		doc.ChildNodes().forEach(function (pageItem) {

			var found = false;

			for (var i = 0; i < currentPages.length; i++) {
				if (pageItem.Id() === currentPages[i].Id()) {
					currentPages[i].copyFrom(pageItem);
					newPages.push(currentPages[i]);
					found = true;
				}
			}

			if (found) {
				return;
			}

			newPages.push(pageItem);
		});

		this.ChildNodes(newPages);
	};

})(ko, batchApp.datacontext);
