﻿

window.batchApp = window.batchApp || {};

window.batchApp.datacontext = (function (ko) {

	var inited = ko.observable(false);

	function initModule(batchId) {
		scanModuleCommunicator.executeAsync('Workspace_InitImages',
			[],
			function () {
				scanModuleCommunicator.executeAsync('Workspace_UpdateRescanBatchInfos',
					[],
					function() {
						scanModuleCommunicator.executeAsync('Workspace_UpdateBatchInfos',
							[],
							function() {
								scanModuleCommunicator.executeAsync('Workspace_HasCurrentBatch',
									[],
									function(hasBatch) {
										if (hasBatch.result == '1') {
											scanModuleCommunicator.executeAsync('Workspace_CloseCurrentBatch',
												[],
												function() {
													scanModuleCommunicator.executeAsync('Workspace_OpenBatch',
														[batchId],
														function() {
															inited(true);
														});
												});
										} else {
											scanModuleCommunicator.executeAsync('Workspace_OpenBatch',
												[batchId],
												function() {
													inited(true);
												});
										}
									});
							});
					});
			});
	}

	FC.BatchEditor = FC.BatchEditor || {};
	FC.BatchEditor.batchEditorDataContext = FC.BatchEditor.batchEditorDataContext || {};
	FC.BatchEditor.batchEditorDataContext.getThumblainUrl = function (parentId, id, version) {
		return window.scanModuleCommunicator.getPluginUrl('CurrentBatch_GetPageThumbnail', [id, version]);
	}
	FC.BatchEditor.batchEditorDataContext.getProjectName = function() {
		return '';
	}
	FC.BatchEditor.batchEditorDataContext.batchId = function() {
		return $("#batchId").val();
	}

	function processOnInited(func) {
		if (inited()) {
			func();
		} else {
			inited.subscribe(function(val) {
				if (val) {
					func();
				}
			});
		}
	}

	var datacontext = {
		initModule: initModule,
		getBatchName: getBatchName,
		setBatchName: setBatchName,
		getDocuments: getDocuments,
		getDocumentName: getDocumentName,
		getPageThumbnail: getPageThumbnail,
		getPageUrl: getPageUrl,
		getPageSize: getPageSize,
		movePage: movePage,
		createDocument: createDocument,
		deleteDocument: deleteDocument,
		deletePage: deletePage,
		startFolderFilesAdding: startFolderFilesAdding,
		startScanning: startScanning,
		checkEndFilesAdding: checkEndFilesAdding,
		closeBatch: closeBatch,
		sendBatch: sendBatch,
		initScanning: initScanning,
		getBatchRegParameters: getBatchRegParameters,
		saveBatchRegParameters: saveBatchRegParameters,
		getDocRegParameters: getDocRegParameters,
		saveDocRegParameters: saveDocRegParameters,
		setDocName: setDocName,
		rotatePage: rotatePage,
		convertBWPage: convertBWPage,
		despecklePage: despecklePage,
		descewPage: descewPage,
		invertPage: invertPage,
		flipHorizontalPage: flipHorizontalPage,
		flipVerticalPage: flipVerticalPage,
		cropPage: cropPage,
		redactPage: redactPage,
		canRestorePage: canRestorePage,
		restorePage: restorePage,
		getRescanTaskId: getRescanTaskId,
		getRescanType: getRescanType,
		rescanImages: rescanImages,
		getRescanComment: getRescanComment,
		sendToException: sendToException,
		rejectTask: rejectTask,
		inited: inited,
		setComment: setComment,
		getBatchPriority: getBatchPriority,
		setBatchPriority: setBatchPriority,
		getForbidOperatorToChangeForBatchSettings: getForbidOperatorToChangeForBatchSettings,
		getForbidOperatorToChangeForDocumentSettings: getForbidOperatorToChangeForDocumentSettings
	};

	function getBatchName(batchNameObservable) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_getName',
				[],
				function(batchNameSerialized) {
					batchNameObservable(batchNameSerialized.result);
				});
		});
	}

	/**
	 * Задать новое имя пакета
	 * @param {} batchName Новое имя пакета
	 * @param {} callback Функция-продолжение
	 * @returns {} 
	 */
	function setBatchName(batchName, callback) {
		processOnInited(function () {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_Rename',
				[batchName],
				callback);
		});
	}

	function getDocuments(documentsObservable) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_GetAllPageInfos',
				[],
				function(allPagesSerialized) {
					if (allPagesSerialized.hasErrors || allPagesSerialized.result.length == 0) {
						if (allPagesSerialized.hasErrors) {
							FC.Utils.Error.Show(allPagesSerialized.errorText);
						}
						documentsObservable([]);
						return;
					}
					var pages = [];
					$.each(allPagesSerialized.result.split(";"),
						function(pageInd, pageSerialized) {
							var pageInfo = pageSerialized.split(':');
							if (pages[parseInt(pageInfo[1])] == undefined)
								pages[parseInt(pageInfo[1])] = [];
							pages[parseInt(pageInfo[1])][pageInfo[9]] = new datacontext.TaskPage({
								Id: parseInt(pageInfo[0]),
								Name: '',
								ParentDocumentId: parseInt(pageInfo[1]),	                            
								Width: parseInt(pageInfo[3]),
								Height: parseInt(pageInfo[4])
							});
						});

					window.scanModuleCommunicator.executeAsync('CurrentBatch_GetAllDocumentInfos',
						[],
						function(allDocsSerialized) {
							if (allDocsSerialized.hasErrors) {
								FC.Utils.Error.Show(allDocsSerialized.errorText);
								documentsObservable([]);
							} else {
								documentsObservable($.map(allDocsSerialized.result.split(';'),
									function(docInfoSerialized) {
										var docInfo = docInfoSerialized.split(':');
										return new datacontext.TaskDocument({
											BatchId: parseInt(docInfo[2]),
											DocumentId: parseInt(docInfo[0]),
											Comment: docInfo[6],
											Priority: docInfo[5],
											PagesCount: parseInt(docInfo[4]),
											TemplateName: scanModuleCommunicator.decodeText(docInfo[3]),
											ChildNodes: pages[parseInt(docInfo[0])],
											IsImplicit: docInfo[8] === '1'
										});
									}));
							}
						});

				});
		});
	}

	function getDocumentName(docId, callbackFunction) {
		window.scanModuleCommunicator.executeAsync('CurrentBatch_GetDocumentInfoByID',
			[docId],
			function (docInfoSerialized) {
				if (docInfoSerialized.hasErrors) {
					callbackFunction('');
				} else {
					var docInfo = docInfoSerialized.result.split(':');
					callbackFunction(scanModuleCommunicator.decodeText(docInfo[3]));	                            
				}
			});
	}

	function getPageThumbnail(page) {
		return window.scanModuleCommunicator.getPluginUrl('CurrentBatch_GetPageThumbnail',[page.Id(), page.Version()]);
	}

	function getPageUrl(pageInfo) {
		return window.scanModuleCommunicator.getPluginUrl('CurrentBatch_GetPageImage', [pageInfo.Id(), pageInfo.Version()]);
	}

	function movePage(pageId, targetId, callback) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_MovePage', [pageId, targetId], callback);
		});
	}

	function createDocument(documentPosition, isImplicit, callback) {
		if (isImplicit == undefined)
			isImplicit = false;
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_CreateDocument',
				[documentPosition, isImplicit],
				callback);
		});
	}

	function deleteDocument(documentId, callback) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_DeleteDocument', [documentId], callback);
		});
	}

	function deletePage(pageId, callback) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_DeletePage', [pageId], callback);
		});
	}

	function startFolderFilesAdding(idDoc, callback) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_ImportFiles', [idDoc, false], callback);
		});
	}

	function startScanning(idDoc, callback) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_Scan', [0, idDoc, false], callback);
		});
	}

	function checkEndFilesAdding(callbackFunc) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_AddReceivedImages',
				[],
				function(addReceivedResult) {
					if (addReceivedResult.hasErrors) {
						FC.Utils.Error.Show(addReceivedResult.errorText);
					};
					window.scanModuleCommunicator.executeAsync('Workspace_ReceiveLastEvents',
						[],
						function(eventsResponse) {
							var result = false;
							if (eventsResponse.hasErrors) {
								FC.Utils.Error.Show(addReceivedResult.errorText);
								result = true;
							} else {
								var events = eventsResponse.result.split('_');
								for (var i = 0; i < events.length; i += 2) {
									if (events[i] == '0')
										result = true;
								}
							}
							if (callbackFunc != undefined) {
								callbackFunc(result);
							}
						});
				});
		});
	}

	function closeBatch(callback) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('Workspace_CloseCurrentBatch', [], callback);
		});
	}

	function sendBatch(callback) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('Workspace_UploadBatch', [$("#batchId").val()], callback);
		});
	}

	var scanInitialized = false;
	function initScanning(callbackFunction) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('Workspace_InitScanning',
				[],
				function(result) {
					scanInitialized = !result.hasErrors;

					if (callbackFunction != undefined) {
						callbackFunction(scanInitialized);
					}
				});
		});
	}


	function getBatchRegParameters(regParamsObservable, callback) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_KeepRegParams',
				[],
				function(keepRegParamsResult) {
					if (keepRegParamsResult.hasErrors) {
						FC.Utils.Error.Show(keepRegParamsResult.errorText);
						regParamsObservable([]);
					}
					getRegParameters(keepRegParamsResult.result, regParamsObservable, callback);
				});
		});

	}
	
	function saveBatchRegParameters(currentRegParameters, callback) {
		var setNewValueFunc = function(keepRegParamsResult, index, newValue, callbackFunc) {
			scanModuleCommunicator.executeAsync('RegistrationParameters_KeepItem',
				[keepRegParamsResult, index],
				function (keepRegParameterResult) {
					if (keepRegParameterResult.hasErrors) {
						FC.Utils.Error.Show(keepRegParameterResult.errorText);
					} else {
						scanModuleCommunicator
							.executeAsync('RegistrationParameter_setValue',
							[keepRegParameterResult.result, newValue], callbackFunc);
					}
				}
			);
		}

		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_KeepRegParams',
				[],
				function(keepRegParamsResult) {
					if (keepRegParamsResult.hasErrors) {
						FC.Utils.Error.Show(keepRegParamsResult.errorText);
						return;
					}

					var actRegPars = ko.observableArray();
					getRegParameters(keepRegParamsResult.result,
						actRegPars,
						function (actualRegParameters) {
							var totalTasks = 0;
							var completedTasks = 0;
							for (var i = actualRegParameters().length - 1; i >= 0; i--) {
								var actualRegParameter = actualRegParameters()[i];
								var currentRelatedRegParam = null;
								for (var j = 0; j < currentRegParameters.length; j++) {
									if (currentRegParameters[j].Name() == actualRegParameter.Name()) {
										currentRelatedRegParam = currentRegParameters[j];
										break;
									}
								}

								if (currentRelatedRegParam.Value() !== actualRegParameter.Value()) {
									totalTasks++;
									setNewValueFunc(keepRegParamsResult.result, i, currentRelatedRegParam.Value(), function() {
										completedTasks++;
										if (completedTasks === totalTasks && callback) {
											callback();
										}
									});
								}
							}

							if (totalTasks === 0 && callback) {
								callback();
							}
						});
				});
		});
	}

	function getDocRegParameters(docId, regParamsObservable, callback) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_KeepDocumentByID',
				[docId],
				function(keepDocumentResult) {
					if (keepDocumentResult.hasErrors) {
						FC.Utils.Error.Show(keepDocumentResult.errorText);
						regParamsObservable([]);
					}
					scanModuleCommunicator.executeAsync('Document_KeepRegParams',
						[keepDocumentResult.result],
						function(keepRegParamsResult) {
							if (keepRegParamsResult.hasErrors) {
								FC.Utils.Error.Show(keepRegParamsResult.errorText);
								regParamsObservable([]);
							}
							getRegParameters(keepRegParamsResult.result, regParamsObservable, callback);
						});
				});
		});
	}

	function saveDocRegParameters(docId, currentRegParameters) {
		var setNewValueFunc = function (keepRegParamsResult, index, newValue) {
			scanModuleCommunicator.executeAsync('RegistrationParameters_KeepItem',
										[keepRegParamsResult, index],
										function (keepRegParameterResult) {
											if (keepRegParameterResult.hasErrors) {
												FC.Utils.Error.Show(keepRegParameterResult.errorText);
											} else {
												scanModuleCommunicator
													.executeAsync('RegistrationParameter_setValue',
													[keepRegParameterResult.result, newValue]);
											}
										});
		}
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_KeepDocumentByID',
				[docId],
				function(keepDocumentResult) {
					if (keepDocumentResult.hasErrors) {
						FC.Utils.Error.Show(keepDocumentResult.errorText);
						return;
					}

					scanModuleCommunicator.executeAsync('Document_KeepRegParams',
						[keepDocumentResult.result],
						function(keepRegParamsResult) {
							if (keepRegParamsResult.hasErrors) {
								FC.Utils.Error.Show(keepRegParamsResult.errorText);
								return;
							}
							var actRegPars = ko.observableArray();
							getRegParameters(keepRegParamsResult.result,
								actRegPars,
								function(actualRegParameters) {
									for (var i = actualRegParameters().length - 1; i >= 0; i--) {
										var actualRegParameter = actualRegParameters()[i];
										var currentRelatedRegParam = null;
										for (var j = 0; j < currentRegParameters.length; j++) {
											if (currentRegParameters[j].Name() == actualRegParameter.Name()) {
												currentRelatedRegParam = currentRegParameters[j];
												break;
											}
										}

										if (currentRelatedRegParam.Value() != actualRegParameter.Value()) {
											setNewValueFunc(keepRegParamsResult.result,
												i,
												currentRelatedRegParam
												.Value());
										}
									}
								});
						});
				});
		});
	}

	function getRegParameters(regParamsFixedNum, regParamsObservable, callbackFunction) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('RegistrationParameters_GetAllItemInfos',
				[regParamsFixedNum],
				function(allRegParamsResult) {
					if (allRegParamsResult.hasErrors || (allRegParamsResult.result.length == 0)) {
						if (allRegParamsResult.hasErrors) {
							FC.Utils.Error.Show(allRegParamsResult.errorText);
						}
						regParamsObservable([]);
						if (callbackFunction != undefined) {
							callbackFunction(regParamsObservable);
						}
						return;
					}

					var result = [];
					if (allRegParamsResult.result.length != 0) {
						$.each(allRegParamsResult.result.split(';'),
							function(key, value) {
								var regParamAttrs = value.split(':');
								var name = scanModuleCommunicator.decodeText(regParamAttrs[0]);
								var defaultValue = scanModuleCommunicator.decodeText(regParamAttrs[1]);
								var isRequired = regParamAttrs[2] == '1';
								var isReadOnly = regParamAttrs[3] == '1';
								var comment = scanModuleCommunicator.decodeText(regParamAttrs[4]);
								var items = [];
								for (var i = 6; i < regParamAttrs.length; i++) {
									items.push(scanModuleCommunicator.decodeText(regParamAttrs[i]));
								}
								var parameterType = items.length > 0
									? datacontext.ParameterType.SELECT
									: datacontext.ParameterType.TEXT;

								var currentRegParameter = new datacontext.RegParameter();
								currentRegParameter.Name(name);
								currentRegParameter.Value(defaultValue);
								currentRegParameter.IsRequired(isRequired);
								currentRegParameter.IsReadOnly(isReadOnly);
								currentRegParameter.ParameterType(parameterType);
								currentRegParameter.ListValues(items);
								result.push(currentRegParameter);
							});
					}
					regParamsObservable(result);
					if (callbackFunction != undefined) {
						callbackFunction(regParamsObservable);
					}
				});
		});
	}

	/**
	 * Задать новое имя для документа
	 * @param {any} docId Идентификатор документа
	 * @param {any} newName Новое имя документа
	 * @param {any} callback Callback-функция
	 */
	function setDocName(docId, newName, callback) {
		processOnInited(function () {
			scanModuleCommunicator.executeAsync('CurrentBatch_KeepDocumentByID',
				[docId],
				function (keepDocumentResult) {
					if (keepDocumentResult.hasErrors) {
						callback(keepDocumentResult);
						return;
					}

					scanModuleCommunicator.executeAsync('Document_setName',
						[keepDocumentResult.result, newName],
						callback);
				});
		});
	}

	function rotatePage(pageId, angle, onExecuted) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_RotatePage', [pageId, angle, true], onExecuted);
		});
	}

	function convertBWPage(pageId, onExecuted) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_ConvertPageToBW', [pageId, true], onExecuted);
		});
	}

	function despecklePage(pageId, index, onExecuted) {
		if (index == undefined)
			index = -1;
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_DespecklePage', [pageId, index, true], onExecuted);
		});
	}

	function descewPage(pageId, textDescew, lineDescew, squareDescew, onExecuted) {
		if (textDescew == undefined)
			textDescew = true;
		if (lineDescew == undefined)
			lineDescew = false;
		if (squareDescew == undefined)
			squareDescew = false;
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_DescewPage',
				[pageId, textDescew, lineDescew, squareDescew, true],
				onExecuted);
		});
	}

	function invertPage(pageId, onExecuted) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_InvertPage', [pageId, true], onExecuted);
		});
	}

	function flipHorizontalPage(pageId, onExecuted) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_FlipPageHorizontal', [pageId, true], onExecuted);
		});
	}

	function flipVerticalPage(pageId, onExecuted) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_FlipPageVertical', [pageId, true], onExecuted);
		});
	}
	function cropPage(pageId, left, top, right, bottom, onExecuted) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_CropPage',
				[pageId, left, top, right, bottom, true, false],
				onExecuted);
		});
	}

	function redactPage(pageId, left, top, right, bottom, onExecuted) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_RedactPage',
				[pageId, left, top, right, bottom, 0, 0, 0, true],
				onExecuted);
		});
	}

	function canRestorePage(pageId, onExecuted) {
		processOnInited(function () {
			scanModuleCommunicator.executeAsync('CurrentBatch_getCanRestorePageSourceImage',
				[pageId],
				onExecuted);
		});
	}

	function restorePage(pageId, onExecuted) {
		processOnInited(function () {
			scanModuleCommunicator.executeAsync('CurrentBatch_RestorePageSourceImage',
				[pageId],
				onExecuted);
		});
	}

	function getPageSize(pageId, pageSizeObservable, callbakFunction) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_KeepPageByID',
				[pageId],
				function(keepPageResult) {
					if (keepPageResult.hasErrors) {
						FC.Utils.Error.Show(keepPageResult.errorText);
						return;
					}
					var numOfCollected = 0;
					var onSizesRecievedFunction = function() {
						numOfCollected++;
						if (numOfCollected == 2) {
							callbakFunction();
						}
					}
					window.scanModuleCommunicator.executeAsync('Page_getWidth',
						[keepPageResult.result],
						function(result) {
							if (result.hasErrors) {
								FC.Utils.Error.Show(result.errorText);
							}
							pageSizeObservable.Width(parseInt(result.result));
							onSizesRecievedFunction();
						});
					window.scanModuleCommunicator.executeAsync('Page_getHeight',
						[keepPageResult.result],
						function(result) {
							if (result.hasErrors) {
								FC.Utils.Error.Show(result.errorText);
							}
							pageSizeObservable.Height(parseInt(result.result));
							onSizesRecievedFunction();
						});
				});
		});
	}

	function getRescanTaskId(callbackFunction) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('CurrentBatch_getRescanTaskID', [], callbackFunction);
		});
	}

	function getRescanType(rescanTypeObservable) {
		getRescanTaskId(function(result) {
			if (result.hasErrors) {
				rescanTypeObservable(datacontext.RescanType.NOTRESCAN);
				return;
			} else {
				var rescanId = parseInt(result.result);
				if (rescanId < 0) {
					rescanTypeObservable(datacontext.RescanType.NOTRESCAN);
					return;
				}
				scanModuleCommunicator.executeAsync('CurrentBatch_getNeedFullRescan', [], function (result) {
					if (result.hasErrors || result.result === '0') {
						rescanTypeObservable(datacontext.RescanType.PARTIAL);
					} else {
						rescanTypeObservable(datacontext.RescanType.FULL);
					}
				});
			}
		});
	}

	function rescanImages(pages, fromFolder, callbackFunction) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_Rescan',
				[pages.join(';'), fromFolder],
				function(result) {
					if (result.hasErrors) {
						FC.Utils.Error.Show(result.errorText);
					}
					if (callbackFunction != undefined) {
						callbackFunction(result);
					}
				});
		});
	}

	function getRescanComment(commentObservable) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_KeepInfo',
				[],
				function(infoObject) {
					if (infoObject.hasErrors) {
						FC.Utils.Error.Show(infoObject.errorText);
						commentObservable("");
					} else {
						window.scanModuleCommunicator.executeAsync('BatchInfo_getRescanComment',
							[infoObject.result],
							function(commentObject) {
								if (commentObject.hasErrors) {
									FC.Utils.Error.Show(commentObject.errorText);
									commentObservable("");
								} else {
									commentObservable(commentObject.result);
								}
							});
					}
				});
		});
	}

	function sendToException(comment, callbackFunction) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('Workspace_SendRescanTaskToExceptions',
				[-1, comment],
				function(rejectResult) {
					if (rejectResult.hasErrors) {
						FC.Utils.Error.Show(rejectResult.errorText);
					}
					scanModuleCommunicator.executeAsync('Workspace_DeleteBatch',
						[-1],
						function(deleteResult) {
							if (deleteResult.hasErrors) {
								FC.Utils.Error.Show(deleteResult.errorText);
							}
							if (callbackFunction !== undefined) {
								callbackFunction();
							}

						});
				});
		});
	}
		
	function rejectTask(callbackFunction) {
		processOnInited(function() {
			scanModuleCommunicator.executeAsync('Workspace_RejectRescanTask',
				[-1],
				function(rejectResult) {
					if (rejectResult.hasErrors) {
						FC.Utils.Error.Show(rejectResult.errorText);
					}
					scanModuleCommunicator.executeAsync('Workspace_DeleteBatch',
						[-1],
						function(deleteResult) {
							if (deleteResult.hasErrors) {
								FC.Utils.Error.Show(deleteResult.errorText);
							}
							if (callbackFunction !== undefined) {
								callbackFunction();
							}

						});
				});
		});

	}

	/**
	 * Задать комментарий к пакету
	 * @param {string} comment Текст комментария
	 * @param {} callback Callback-функция
	 */
	function setComment(comment, callback) {
		processOnInited(function () {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_KeepInfo',
				[],
				function (infoObject) {
					if (infoObject.hasErrors) {
						FC.Utils.Error.Show(infoObject.errorText);
						callback();
					} else {
						window.scanModuleCommunicator.executeAsync('BatchInfo_setComment',
							[infoObject.result, comment],
							function (commentObject) {
								if (commentObject.hasErrors) {
									FC.Utils.Error.Show(commentObject.errorText);
									callback();
								} else {
									callback();
								}
							});
					}
				});
		});
	}

	/**
	 * Получить приоритет пакета
	 * @param {} callback Callback-функция
	 */
	function getBatchPriority(callback) {
		processOnInited(function () {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_KeepInfo',
				[],
				function (infoObject) {
					if (infoObject.hasErrors) {
						callback(infoObject);
					} else {
						window.scanModuleCommunicator.executeAsync('BatchInfo_getPriority',
							[infoObject.result],
							callback);
					}
				});
		});
	}

	/**
	 * Задать приоритет пакета
	 * @param {string} priority Приоритет пакета
	 * @param {} callback Callback-функция
	 */
	function setBatchPriority(priority, callback) {
		processOnInited(function () {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_KeepInfo',
				[],
				function (infoObject) {
					if (infoObject.hasErrors) {
						callback(infoObject);
					} else {
						window.scanModuleCommunicator.executeAsync('BatchInfo_setPriority',
							[infoObject.result, priority],
							callback);
					}
				});
		});
	}

	/**
	 * Получить значение настройки пакета, запрещающей оператору изменять свойства этого пакета
	 * @param {any} callback
	 */
	function getForbidOperatorToChangeForBatchSettings(callback) {
		processOnInited(function() {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_KeepType',
				[],
				function(batchTypeObject) {
					if (batchTypeObject.hasErrors) {
						callback(batchTypeObject);
					} else {
						window.scanModuleCommunicator.executeAsync('BatchType_KeepBatchOptions',
							[batchTypeObject.result],
							function(batchOptionsObject) {
								if (batchOptionsObject.hasErrors) {
									callback(batchOptionsObject);
								} else {
									window.scanModuleCommunicator.executeAsync('BatchOptions_getForbidOperatorToChangeForBatch',
										[batchOptionsObject.result],
										callback);
								}
							});
					}
				});
		});
	}

	/**
	 * Получить значение настройки, запрещающей оператору изменять свойства документа
	 * @param {any} callback
	 */
	function getForbidOperatorToChangeForDocumentSettings(callback) {
		processOnInited(function () {
			window.scanModuleCommunicator.executeAsync('CurrentBatch_KeepType',
				[],
				function (batchTypeObject) {
					if (batchTypeObject.hasErrors) {
						callback(batchTypeObject);
					} else {
						window.scanModuleCommunicator.executeAsync('BatchType_KeepDocumentOptions',
							[batchTypeObject.result],
							function (documentOptionsObject) {
								if (documentOptionsObject.hasErrors) {
									callback(documentOptionsObject);
								} else {
									window.scanModuleCommunicator.executeAsync('DocumentOptions_getForbidOperatorToChangeForBatch',
										[documentOptionsObject.result],
										callback);
								}
							});
					}
				});
		});
	}
	
	return datacontext;


})(ko);