﻿window.batchApp = window.batchApp || {};

FC.BatchEditor.batchEditorResources = {

	// Заготовок для титула проектов ("Project: ")
	projectTitle: function () { return "Projects: "; },

	// Название для неизвестного шаблона ("Unknown")
	unprocessedTemplate: function () { return 'Unknown'; },

	// Отменить (для контекстного меню)
	undo: function () { return FC.Environment.Localization.BatchView.UndoButton; },

	// Вернуть (для контекстного меню)
	redo: function () { return FC.Environment.Localization.BatchView.RedoButton; },

	// Разделить документ (для контекстного меню)
	split: function () { return FC.Environment.Localization.BatchView.SplitButton; },

	// Отменить разделение  (для контекстного меню)
	cancelSplitting: function () { return FC.Environment.Localization.BatchView.CancelSplitting; },

	// Объединить документы (для контекстного меню)
	merge: function () { return FC.Environment.Localization.BatchView.MergeButton; },

	// Удалить  (для контекстного меню)
	deleteButton: function () { return FC.Environment.Localization.BatchView.DeleteButton; }
};

window.batchApp.extensions = (function (datacontext, layoutsDataContext) {

	var rescanType = ko.observable(datacontext.RescanType.NOTRESCAN);
	var rescanComment = ko.observable("");
	var zoomable = ko.observable(true);
	rescanType.subscribe(function (val) {
		if (val !== datacontext.RescanType.NOTRESCAN) {
			zoomable(false);
			window.batchLayout.batchLayoutViewModel.BatchEditor.zoomSettings.currentValue(window.batchLayout.batchLayoutViewModel.BatchEditor.zoomSettings.minValue());
			datacontext.getRescanComment(rescanComment);
		}
	});
	var isPartialRescan = ko.computed(function () {
		return rescanType() !== datacontext.RescanType.PARTIAL;
	});
	var isRescanTask = ko.computed(function () {
		return rescanType() !== datacontext.RescanType.NOTRESCAN;
	});

	datacontext.getRescanType(rescanType);

	var isAdminUser = document.getElementById('isAdminUser').value === 'true';

	// Можно ли переименовывать документ
	var canRenameDocument = ko.observable(isAdminUser);

	if (!isAdminUser) {
		// Загрузим значение canRenameDocument
		datacontext.getForbidOperatorToChangeForDocumentSettings(function (result) {
			if (result.hasErrors) {
				FC.Utils.Error.Show(result.errorText);
				return;
			}

			canRenameDocument(parseInt(result.result, 10) === 0);
		});
	}

	FC.BatchEditor.batchEditorSettings = {
		documentDeletingEnabled: isPartialRescan,
		documentCreatingEnabled: isPartialRescan,
		documentMergeEnabled: isPartialRescan,
		documentSplitEnabled: isPartialRescan,
		newOpenStrategyEnabled: function () { return true; }
	};

	function resources() {
		return $.extend(FC.Environment.Localization.GlobalResources, FC.Environment.Localization.NonLocalizedHotKeys, FC.Environment.Localization.BatchView);
	};

	function afterPagesDeleting(deletedPages, finished) {
		var numOfPages = deletedPages.length;
		var numOfDeletedPages = 0;
		if (numOfPages === 0 && finished) {
			finished(true);
		}
		$.each(deletedPages,
			function (index, page) {
				datacontext.deletePage(page.Id(), function () {
					numOfDeletedPages++;
					if (numOfDeletedPages >= numOfPages) {
						finished(true);
					}
				});
			});
	}

	/**
	 * Обработчик, вызываемый после перемещения страниц
	 * @param {} pages Перемещённые страницы
	 * @param {} target Цель (страница или документ), в которую перемещены страницы
	 * @param {} finished Функция, вызываемая при завершении перемещения
	 * @returns {} 
	 */
	function afterPagesMoving(pages, target, finished) {
		var insertPointId = target.Id();
		if (pages.length === 0 && finished) {
			finished(true);
			return;
		}

		var numOfPages = pages.length;
		var movePage = function (num) {
			if (num < 0 || num >= numOfPages) {
				if (finished) {
					finished(true);
				}

				return;
			}

			var page = pages[num];
			datacontext.movePage(page.Id(), insertPointId, function () {
				insertPointId = page.Id();
				movePage(num + 1);
			});
		}

		movePage(0);
	}

	function afterDocumentDeleting(doc, pagesDeleted) {
		if (!pagesDeleted) {
			datacontext.deleteDocument(doc.Id());
		} else {
			var interval = setInterval(function () {
				if (pagesDeleted()) {
					datacontext.deleteDocument(doc.Id());
					clearInterval(interval);
				}
			},
				100);
		}

	}

	function afterDocumentMoving(doc, index) {
		datacontext.createDocument(index, false, function (createResult) {
			if (createResult.hasErrors) {
				FC.Utils.Error.Show(createResult.errorText);
				return;
			}
			var newDocId = parseInt(createResult.result);
			var insertPointId = newDocId;
			var pagesToMove = doc.ChildNodes();
			var moveFunc = function (index) {
				if (pagesToMove.length == index) {
					var prevDocId = doc.Id();
					datacontext.deleteDocument(prevDocId);
					doc.Id(newDocId);
					var action = proxy.batchActionCreators.updateDocumentId(
						prevDocId,
						newDocId
					);
					proxy.store.dispatch(action);
					window.batchLayoutComponent.store.dispatch(action);
					return;
				}
				var pageToMove = pagesToMove[index];
				var currentInsertPoint = insertPointId;
				insertPointId = pageToMove.Id();
				pageToMove.ParentDocumentId = newDocId;
				datacontext.movePage(pageToMove.Id(), currentInsertPoint, function () { moveFunc(index + 1); });
			}
			moveFunc(0);
		});

	}

	var actionsEnabled = {
		basic: ko.observable(true),
		crop: ko.observable(true),
		redact: ko.observable(true)
	}
	var mainToolbarEnabled = ko.observable(true);

	var cropEditor;
	var redactEditor;

	var documentEditorLock = ko.observable(false);

	/**
	 * Сохранение изменений внесенных инструментом Redact
	 * @param {any} callback callback запускаемы по завершению сохранения
	 */
	function saveRedact(callback) {
		if (!redactEditor) {
			return
		}
		// Открываем окно "Save redaction..."
		openScanMessageWindow(FC.Environment.Localization.BatchView.SavingRedactChanges)
		redactEditor.complete(function () {
			if (callback) {
				//Вызываем переданнй callback при завершении redact
				callback();
				// Закрываем окно "Save redaction..."
				closeScanMessageWindow();
			}
		});
		proxy.store.dispatch(proxy.documentEditorActionCreators.activateCommand(-1));
		// Удаляем redactEditor
		redactEditor = null;
	}

	function modifyPages(func, params, callback) {

		params = params || [];
		var callParams = params.splice(0, params.length);
		callParams.unshift(0);
		callParams.push(null);
		var checkedPages = batchLayout.batchLayoutViewModel.CheckedPages();
		var count = 0, total = checkedPages.length;
		var errors = [];

		documentEditorLock(true);

		$.each(checkedPages, function (key, page) {
			callParams[0] = page.Id();
			callParams[callParams.length - 1] = function (result) {
				if (result.hasErrors) {
					var error = result.errorText;
					if (errors.indexOf(error) === -1) {
						errors.push(error);
						FC.Utils.Error.Show(error);
					}
					return;
				}

				datacontext.getPageSize(page.Id(),
					page,
					function () {
						page.Version(page.Version() + 1);

						var proxy = batchLayout.batchLayoutViewModel.proxy;
						var currentPage = proxy.store.getState().batch.pages[page.Id()];

						var action = proxy.batchActionCreators.updatePages(
							[
								{
									id: page.Id(),
									imageWidth: page.Width(),
									imageHeight: page.Height(),
									title: currentPage.title,
									imageUri: dependencies.batchDataContext.getPageUrl(page),
									parentDocumentId: page.ParentDocumentId
								}
							]
						)
						proxy.store.dispatch(action);
						window.batchLayoutComponent.store.dispatch(action);

						count++;

						if (count === total) {
							documentEditorLock(false);
							if (callback) {
								callback();
							}
						}
					});
			};

			func.apply(null, callParams);
		});

		batchLayout.batchLayoutViewModel.SelectedPage(batchLayout.batchLayoutViewModel.SelectedPage());
	}

	/** Получить идентификаторы выделенных страниц текущего документа */
	function getCheckedPagesIdsOfCurrentDocument() {
		var checkedPages = batchLayout.batchLayoutViewModel.CheckedPages();
		var selectedDocument = batchLayout.batchLayoutViewModel.SelectedDocument();
		var filteredByDocumentPages = checkedPages.filter(function (page) {
			return page.ParentDocumentId === selectedDocument.Id();
		});

		return filteredByDocumentPages.map(function (page) {
			return page.Id();
		});
	}

	function rotate(angle) {
		// getCurrentViewPoint и setCurrentViewPoint обеспечивают сохранение текущей видимой страницы в DocumentEditor'е
		modifyPages(datacontext.rotatePage, [angle], function () {

			var pagesIds = getCheckedPagesIdsOfCurrentDocument();
			if (cropEditor) {
				cropEditor.onRotate(angle, pagesIds);
			}

			if (redactEditor) {
				redactEditor.onRotate(angle, pagesIds);
			}
		});
	}

	function rotateCheckedPages90Clockwise() {
		rotate(90);
	}
	function rotateCheckedPages90CounterClockwise() {
		rotate(-90);
	}

	function rotateCheckedPages180() {
		rotate(180);
	}

	function convertBWCheckedPages() {
		modifyPages(datacontext.convertBWPage);
	}
	function despeckleCheckedPages() {
		modifyPages(datacontext.despecklePage, [-1]);
	}
	function descewCheckedPages() {
		modifyPages(datacontext.descewPage, [true, false, false]);
	}
	function invertCheckedPages() {
		modifyPages(datacontext.invertPage);
	}
	function restoreCheckedPages() {
		modifyPages(datacontext.restorePage, [], function () {
			var pagesIds = getCheckedPagesIdsOfCurrentDocument();
			if (cropEditor) {
				cropEditor.onRestore(pagesIds);
			}

			if (redactEditor) {
				redactEditor.onRestore(pagesIds);
			}
		});
	}
	function flipHorizontalCheckedPages() {
		modifyPages(datacontext.flipHorizontalPage, [], function () {
			var pagesIds = getCheckedPagesIdsOfCurrentDocument();
			if (cropEditor) {
				cropEditor.onFlip(0, pagesIds);
			}

			if (redactEditor) {
				redactEditor.onFlip(0, pagesIds);
			}
		});
	}
	function flipVerticalCheckedPages() {
		modifyPages(datacontext.flipVerticalPage, [], function () {
			var pagesIds = getCheckedPagesIdsOfCurrentDocument();
			if (cropEditor) {
				cropEditor.onFlip(1, pagesIds);
			}

			if (redactEditor) {
				redactEditor.onFlip(1, pagesIds);
			}
		});
	}

	function replacePages(fromFolder) {
		var pageIds = [];
		$.each(batchLayout.batchLayoutViewModel.CheckedPages(), function (key, page) {
			pageIds.push(page.Id());
		});
		if (pageIds.length === 0)
			return;

		openScanMessageWindow(FC.Environment.Localization.BatchView.LoadImagesMessage);
		datacontext.rescanImages(pageIds, fromFolder, function (initRescanRes) {
			if (initRescanRes.hasErrors) {
				closeScanMessageWindow();
			} else {
				var folderInterval = setInterval(function () {
					datacontext.checkEndFilesAdding(function (result) {
						if (result) {
							$.each(batchLayout.batchLayoutViewModel.CheckedPages(),
								function (key, page) {
									datacontext.getPageSize(page.Id(), page, function () {
										updatePageById(page.Id());
									});
								});
							closeScanMessageWindow();
							clearInterval(folderInterval);
						}
					});
				}, 1000);
			}
		});
	}

	function rescanPages() {
		replacePages(false);
	}

	function replaceFromFolder() {
		replacePages(true);
	}

	function openScanMessageWindow(message) {
		$('#ScanningLoadScreenMessage').html(message);
		$('#ScanningLoadScreen').show();
	}

	function closeScanMessageWindow() {
		$('#ScanningLoadScreen').hide();
	}

	function checkRegParams(callback) {
		var regparams = ko.observableArray();
		datacontext.getBatchRegParameters(regparams,
			function (batchParams) {
				var res = {
					Correct: true,
					Errors: []
				}
				for (var i = 0; i < batchParams().length; i++) {
					var batchParam = batchParams()[i];
					if (batchParam.IsRequired() &&
						((batchParam.Value() == undefined) || (batchParam.Value().length == 0))) {
						res.Correct = false;
						res.Errors.push({ RegParamName: batchParam.Name(), BadParamType: 'batch' });
					}
				}
				var allDocs = batchLayout.batchLayoutViewModel.Documents();
				var callExecuted = false, numberOfChecked = 0;
				var checkDocRegparam = function (index) {
					if (index >= allDocs.length) {
						return;
					}
					var curentDoc = allDocs[index];
					if (curentDoc.IsImplicit()) {
						numberOfChecked++;
						if (numberOfChecked == allDocs.length) {
							callback(res);
						}
					}
					else {
						datacontext.getDocRegParameters(curentDoc.DocumentId(),
							regparams,
							function (docParams) {
								for (var j = 0; j < docParams().length; j++) {
									var docParam = docParams()[j];
									if (docParam.IsRequired() &&
										((docParam.Value() == undefined) || (docParam.Value().length == 0))) {
										res.Correct = false;
										res.Errors.push({ RegParamName: docParam.Name(), BadParamType: 'doc', Index: index });
									}
								}
								numberOfChecked++;
								if (numberOfChecked === allDocs.length) {
									callback(res);
								}
							});
					}

					checkDocRegparam(index + 1);

				}

				if (allDocs.length == 0) {
					callback(res);
				}
				else {
					checkDocRegparam(0);
				}
			});
	}

	var toolbarItems = ko.computed(function () {
		var result = [
			{
				text: FC.Environment.Localization.BatchView.RotateGroupText,
				action: function () { return true; },
				disabled: false,
				htmlClass: 'batchEditor__imageFuncs__rotate',
				group: 'imageOperations',
				index: 20,
				subItems: [
					{
						text: FC.Environment.Localization.BatchView.FlipVerticalText,
						action: flipVerticalCheckedPages,
						htmlClass: 'batchEditor__imageFuncs__flipVertical',
						group: 'flipOperations',
						subItems: []
					},
					{
						text: FC.Environment.Localization.BatchView.FlipHorizontalText,
						action: flipHorizontalCheckedPages,
						htmlClass: 'batchEditor__imageFuncs__flipHorizontal',
						group: 'flipOperations',
						subItems: []
					},
					{
						text: FC.Environment.Localization.BatchView.Rotate90ClockText,
						action: rotateCheckedPages90Clockwise,
						htmlClass: 'batchEditor__imageFuncs__rotate__clockwise',
						group: 'rotateOperations',
						subItems: []
					},
					{
						text: FC.Environment.Localization.BatchView.Rotate90CountClockText,
						action: rotateCheckedPages90CounterClockwise,
						htmlClass: 'batchEditor__imageFuncs__rotate__counterclockwise',
						group: 'rotateOperations',
						subItems: []
					},
					{
						text: FC.Environment.Localization.BatchView.Rotate180Text,
						action: rotateCheckedPages180,
						htmlClass: 'batchEditor__imageFuncs__rotate__180',
						group: 'rotateOperations',
						subItems: []
					}
				]
			},
			{
				text: FC.Environment.Localization.BatchView.EnhanceGroupText,
				action: function () { return true; },
				htmlClass: 'batchEditor__imageFuncs__enhance',
				disabled: false,
				index: 10,
				group: 'imageOperations',
				subItems: [
					{
						text: FC.Environment.Localization.BatchView.ConverBWText,
						action: convertBWCheckedPages,
						htmlClass: 'batchEditor__imageFuncs__convertBW',
						group: 'colorOperations',
						subItems: []
					},
					{
						text: FC.Environment.Localization.BatchView.InvertColorsText,
						action: invertCheckedPages,
						htmlClass: 'batchEditor__imageFuncs__invert',
						group: 'colorOperations',
						subItems: []
					},
					{
						text: FC.Environment.Localization.BatchView.DespectleText,
						action: despeckleCheckedPages,
						htmlClass: 'batchEditor__imageFuncs__despeckle',
						group: 'dOperations',
						subItems: []
					},
					{
						text: FC.Environment.Localization.BatchView.DeskewText,
						action: descewCheckedPages,
						htmlClass: 'batchEditor__imageFuncs__descew',
						group: 'dOperations',
						subItems: []
					}
				]
			},
			{
				text: FC.Environment.Localization.BatchView.RestorePageTitle,
				action: restoreCheckedPages,
				htmlClass: 'batchEditor__imageFuncs__restore',
				subItems: [],
				index: 25
			}
		];

		if (isRescanTask()) {
			result.push({
				text: FC.Environment.Localization.BatchView.RescanText,
				action: rescanPages,
				disabled: false,
				htmlClass: 'batchEditor__imageFuncs__rescan',
				group: 'replaceOperations',
				index: 30,
				subItems: []
			});
			result.push({
				text: FC.Environment.Localization.BatchView.ReplaceText,
				action: replaceFromFolder,
				disabled: false,
				htmlClass: 'batchEditor__imageFuncs__replace',
				index: 40,
				group: 'replaceOperations',
				subItems: []
			});
		}
		return result;
	});

	var navigate = function (url) {
		if (window.parent && window.parent !== window.self) {
			var event = new FC.Utils.EventData("FC_Scanning_TaskClosed", null, null);

			window.parent.postMessage(event.ToJson(), "*");//NOSONAR
			// "Make sure this cross-domain message is being sent to the intended domain."
			// у нас используется интеграция через iframe, и по-другому не сделать
		}

		var whereToGo = FC.Utils.getParameter("returnTo");

		if (whereToGo && whereToGo.toLowerCase() === "deadend") {
			window.batchLayout.batchLayoutViewModel.Deadender().show();
		} else {
			location.replace(url);
		}
	};

	var confirmWindow;
	var confirmSaveWindow;


	$(window).resize(function () {
		fluidDialog();
	});

	$(document).on("dialogopen", ".ui-dialog", function (event, ui) {
		fluidDialog();
	});

	function fluidDialog() {
		var $visible = $(".ui-dialog:visible");
		// each open dialog
		$visible.each(function () {
			var $this = $(this);
			var dialog = $this.find(".ui-dialog-content").data("ui-dialog");
			if (dialog.options.fluid) {
				var wWidth = $(window).width();
				if (wWidth < (parseInt(dialog.options.maxWidth) + 50)) {
					$this.css("max-width", "90%");
				} else {
					$this.css("max-width", dialog.options.maxWidth + "px");
				}
				dialog.option("position", dialog.options.position);
			}
		});
	}

	$(document).ready(function () {

		confirmWindow = $('#ConfirmWindow')
			.dialog({
				modal: true,
				width: 500,
				height: 200,
				title: FC.Environment.Localization.BatchView.ToExceptionPromt,
				draggable: false,
				autoOpen: false,
				fluid: true,
				position: {
					my: "center",
					at: "center",
					of: "body"
				},
				buttons: {
					OK:
					{
						text: FC.Environment.Localization.Resource.ConfirmButton,
						click: function () {
							var action = $("#confirmAction").val();
							var callbackFunc = function () {
								navigate(layoutsDataContext.allBatchesUrl());
								confirmWindow.dialog('close');
							}
							if (action == "except") {
								datacontext.sendToException($("#confirmComment").val(), callbackFunc);
							} else {
								datacontext.rejectTask(callbackFunc);
							}

						}
					},
					CANCEL: {
						text: FC.Environment.Localization.Resource.CancelButton,
						click: function () {
							confirmWindow.dialog('close');
						}
					}
				}
			});
		confirmSaveWindow = $('#ConfirmSaveWindow')
			.dialog({
				dialogClass: 'confirmSaveWindow',
				modal: true,
				width: 600,
				height: 200,
				title: FC.Environment.Localization.BatchView.DoYouWantToSaveChanges,
				draggable: false,
				autoOpen: false,
				fluid: true,
				position: {
					my: "center",
					at: "center",
					of: window
				},
				buttons: {
					OK:
					{
						text: FC.Environment.Localization.BatchView.SaveRedact,
						click: function () {
							confirmSaveWindow.dialog('close');
							saveRedact();
							checkRegParams(function (checkResult) {

								var batchState = { error: checkResult.Correct ? "" : FC.Environment.Localization.BatchView.IncorrectRegParams };

								datacontext.setComment(JSON.stringify(batchState), function () {
									datacontext.closeBatch(function (result) {
										if (result.hasErrors) {
											FC.Utils.Error.Show(result.errorText);
										}
										navigate(layoutsDataContext.allBatchesUrl());
									});
								});
							});
						}
					},
					REJECT: {
						text: FC.Environment.Localization.BatchView.DoNotSaveRedact,
						click: function () {
							checkRegParams(function (checkResult) {

								var batchState = { error: checkResult.Correct ? "" : FC.Environment.Localization.BatchView.IncorrectRegParams };

								datacontext.setComment(JSON.stringify(batchState), function () {
									datacontext.closeBatch(function (result) {
										if (result.hasErrors) {
											FC.Utils.Error.Show(result.errorText);
										}
										confirmSaveWindow.dialog('close');
										navigate(layoutsDataContext.allBatchesUrl());
									});
								});
							});
						}
					},
					CANCLE: {
						text: FC.Environment.Localization.BatchView.Cancel,
						click: function () {
							$(this).dialog("close");
						}
					}
				}
			});

		$(".confirmSaveWindow").children(".ui-dialog-titlebar").append($('#CloseButton'));

		$('#CloseButton')
			.click(function (e) {
				$('#ConfirmSaveWindow').dialog("close");
				return false;
			});

		batchLayout.batchLayoutViewModel._SelectedPage.subscribe(function () {
			if (!actionsEnabled.basic()) {
				if (actionsEnabled.crop()) {
					setTimeout(function () { cropPage(); }, 500);
				}
				else if (actionsEnabled.redact()) {
					setTimeout(function () { redactPage(); }, 500);
				}
			}
		});
	});

	function getIdToInsert() {
		var selectedItems = batchLayout.batchLayoutViewModel.CheckedDocuments();
		if (!selectedItems || selectedItems.length == 0) {
			selectedItems = batchLayout.batchLayoutViewModel.CheckedPages();
		}

		if (!selectedItems || selectedItems.length == 0) {
			return -1;
		}

		return selectedItems[selectedItems.length - 1].Id();
	}

	function updatePage(page) {
		datacontext.getPageSize(
			page.Id(),
			page,
			function () {
				page.Version(page.Version() + 1);

				var proxy = batchLayout.batchLayoutViewModel.proxy;
				var currentPage = proxy.store.getState().batch.pages[page.Id()];

				var action = proxy.batchActionCreators.updatePages(
					[
						{
							id: page.Id(),
							imageWidth: page.Width(),
							imageHeight: page.Height(),
							title: currentPage.title,
							imageUri: dependencies.batchDataContext.getPageUrl(page),
							parentDocumentId: page.ParentDocumentId
						}
					]
				);
				proxy.store.dispatch(action);
				window.batchLayoutComponent.store.dispatch(action);
			}
		);
	}

	/**
	 * Обновить открытую страницу как в knockout, так и в redux 
	 */
	function updatePageById(pageId) {
		var selectedDocument = batchLayout.batchLayoutViewModel.SelectedDocument();
		var pages = selectedDocument.ChildNodes();
		for (var i = 0; i < pages.length; i++) {
			var page = pages[i];

			if (page.Id() !== pageId) {
				continue;
			}

			updatePage(page);
		}
	}

	var cropCss = {
		'documentEditor__nonContextOperations__crop': true
	};

	var commands = { Crop: 0, Redact: 1 };

	var alreadyLeft = false;
	return {
		onDocumentCreating: function (docPosition, batchId, isImplicit, callback) {
			datacontext.createDocument(docPosition,
				isImplicit,
				function (creationResult) {
					if (creationResult.hasErrors) {
						FC.Utils.Error.Show(creationResult.errorText);
						return;
					}
					var document = new datacontext.TaskDocument({
						DocumentId: parseInt(creationResult.result),
						IsImplicit: isImplicit,
						PagesCount: 0,
						TemplateName: '',
						BatchId: batchId
					});
					datacontext.getDocumentName(creationResult.result,
						function (docName) {
							document.TemplateName(docName);
							if (callback != undefined) {
								callback(document);
							}

						});

				});
			return
		},
		onDocumentDeleting: function (docId) {
			return !datacontext.deleteDocument(docId).hasErrors;
		},
		getGeneralControls: function () {
			var controls = [
				{
					name: FC.Environment.Localization.BatchView.CloseButton,
					title: FC.Environment.Localization.BatchView.CloseButton_Title,
					action: function () {
						if (!window.batchApp.datacontext.inited()) {
							return;
						}
						//Открываем диалог сохранения изменений только если активирован redact
						if (redactEditor) {
							confirmSaveWindow.dialog("open");
						}
						else {
							// Сохраним состояние рег.параметров в информации о пакете, чтобы знать в других местах, что пакет нельзя отправить на распознавание
							checkRegParams(function (checkResult) {

								var batchState = { error: checkResult.Correct ? "" : FC.Environment.Localization.BatchView.IncorrectRegParams };

								datacontext.setComment(JSON.stringify(batchState), function () {
									datacontext.closeBatch(function (result) {
										if (result.hasErrors) {
											FC.Utils.Error.Show(result.errorText);
										}
										confirmSaveWindow.dialog('close');
										navigate(layoutsDataContext.allBatchesUrl());
									});
								});
							});
						}
					},
					mainAction: false,
					htmlCss: "task-action--close",
					order: 20,
					visible: ko.observable(true)
				},
				{
					name: FC.Environment.Localization.BatchView.ToExceptionButton,
					title: FC.Environment.Localization.BatchView.ToExceptionButton_Title,
					action: function () {
						if (!window.batchApp.datacontext.inited()) {
							return;
						}
						$("#confirmAction").val("except");
						confirmWindow.dialog("open");
					},
					mainAction: false,
					htmlCss: "task-action--to-exception",
					order: 40,
					visible: isRescanTask
				},
				{
					name: FC.Environment.Localization.BatchView.RejectButton,
					title: FC.Environment.Localization.BatchView.RejectButton_Title,
					action: function () {
						if (!window.batchApp.datacontext.inited()) {
							return;
						}
						if (alreadyLeft) {
							return;
						}
						alreadyLeft = true;
						datacontext.rejectTask(function () {
							navigate(layoutsDataContext.allBatchesUrl());
						});
					},
					mainAction: false,
					htmlCss: "task-action--reject",
					order: 30,
					visible: isRescanTask
				},
				{
					name: FC.Environment.Localization.BatchView.SendButton,
					title: FC.Environment.Localization.BatchView.SendButton_Title,
					htmlCss: "task-action--complete task-action--main",
					order: 1,
					action: function () {
						if (
							!window.batchApp.datacontext.inited() ||
							window.batchLayout.batchLayoutViewModel.isCompliting() ||
							window.batchLayout.batchLayoutViewModel.isBatchRegParametersSaving()
						) {
							return;
						}
						if (redactEditor) {
							saveRedact();
						}
						window.batchLayout.batchLayoutViewModel.isCompliting(true);
						checkRegParams(function (checkResult) {
							closeScanMessageWindow();
							if (!checkResult.Correct) {
								window.batchLayout.batchLayoutViewModel.isCompliting(false);
								var errorDetails = checkResult.Errors[0].BadParamType == 'batch'
									? FC.Environment.Localization.BatchView.BatchRegParameterMissed
									: FC.Environment.Localization.BatchView.DocRegParameterMissed;
								errorDetails = errorDetails.replace('{0}', checkResult.Errors[0].RegParamName);
								if (checkResult.Errors[0].BadParamType == 'doc') {
									errorDetails = errorDetails.replace('{1}',
										batchLayout.batchLayoutViewModel.BatchEditor.elements()[checkResult.Errors[0].Index].documentTitleIndex());
								}
								FC.Utils.Notification.ErrorWindow(FC.Environment.Localization.BatchView.IncorrectRegParams, errorDetails);
								if (checkResult.Errors[0].BadParamType == 'doc') {
									batchLayout.batchLayoutViewModel.BatchEditor.elements()[checkResult.Errors[0].Index].selected(true);
								} else {
									batchLayout.batchLayoutViewModel
										.SelectedDocument(batchLayout.batchLayoutViewModel.Documents()[0]);
								}
							} else {
								if (alreadyLeft) {
									return;
								}
								alreadyLeft = true;
								datacontext.closeBatch(function (closeResult) {
									if (closeResult.hasErrors) {
										FC.Utils.Error.Show(closeResult.errorText);
									}
									datacontext.sendBatch(function (sendResult) {
										if (sendResult.hasErrors) {
											FC.Utils.Error.Show(sendResult.errorText);
										}
										navigate(layoutsDataContext.allBatchesUrl());
									});

								});
							}
						});
					},
					mainAction: true,
					visible: ko.observable(true)
				}
			];

			return controls;
		},
		batchEditorNonContextOperations: function () {
			return [
				new FC.NonContextFunction({
					name: FC.Environment.Localization.BatchView.AddFromScanner,
					order: 1,
					fullName: FC.Environment.Localization.BatchView.AddFromScannerTooltip,
					action: function () {
						openScanMessageWindow(FC.Environment.Localization.BatchView.InitScannersMessage);

						datacontext.initScanning(function (initScanningResult) {
							datacontext.startScanning(getIdToInsert(),
								function (startScanningResult) {
									if (startScanningResult.hasErrors) {
										closeScanMessageWindow();
										FC.Utils.Error.Show(startScanningResult.errorText);
										window.scannerOptions.viewModel.openWindow();
										return;
									}

									openScanMessageWindow(FC.Environment.Localization.BatchView.LoadImagesMessage);
									var scanningInterval = setInterval(
										function () {
											datacontext.getDocuments(function (newDocuments) {
												batchLayout.batchLayoutViewModel.copyDocumentsFrom(newDocuments, function (doc) {
													return doc;
												});
											});

											datacontext.checkEndFilesAdding(function (result) {
												if (result) {
													clearInterval(scanningInterval);
													closeScanMessageWindow();
												}
											});
										},
										1000
									);
								});

						});
					},
					subItems: [
						new FC.NonContextFunction({
							name: FC.Environment.Localization.BatchView.ScannerOptionsTooltip,
							order: 2,
							action: function () {
								openScanMessageWindow(FC.Environment.Localization.BatchView.InitScannersMessage);
								datacontext.initScanning(function () {
									window.scannerOptions.viewModel.openWindow();
									closeScanMessageWindow();
								});
							},
							htmlClass: 'batchEditor__nonContextOperations__scanOptions'
						})
					],
					htmlClass: 'batchEditor__nonContextOperations__fromScanner'
				}),
				new FC.NonContextFunction({
					name: FC.Environment.Localization.BatchView.AddFromFolder,
					order: 2,
					action: function () {
						openScanMessageWindow(FC.Environment.Localization.BatchView.LoadImagesMessage);
						datacontext.startFolderFilesAdding(getIdToInsert(),
							function (result) {
								if (result.hasErrors) {
									FC.Utils.Error.Show(result.errorText);
									return;
								}

								var updateLock = false;

								var folderInterval = setInterval(function () {
									if (!updateLock) {
										updateLock = true;

										datacontext.getDocuments(function (updateResult) {

											datacontext.checkEndFilesAdding(function (result) {
												updateLock = false;

												if (result) {
													clearInterval(folderInterval);
													closeScanMessageWindow();
												}
											});

											batchLayout.batchLayoutViewModel.copyDocumentsFrom(updateResult, function (doc) {
												return doc;
											});
										});
									}
								},
									1000);
							});

					},
					htmlClass: 'batchEditor__nonContextOperations__fromfolder'
				})
			];
		},

		init: function (proxy) {
			proxy.store.dispatch(proxy.documentEditorActionCreators.updateCommands([
				{
					id: commands.Crop,
					title: FC.Environment.Localization.BatchView.Crop + '…',
					icon: {
						viewBox: '0 0 16 16',
						id: '',
						url: '#icon--crop'
					}
				}, {
					id: commands.Redact,
					title: FC.Environment.Localization.BatchView.Redact,
					icon: {
						viewBox: '0 0 16 16',
						id: '',
						url: '#icon--redact'
					}
				}
			]));
		},

		onCommandExecute: function (proxy, commandId) {
			switch (commandId) {
				case commands.Crop:
					if (documentEditorLock()) {
						return;
					}

					if (!cropEditor) {
						cropEditor = new batchApp.datacontext.CropEditor(
							proxy.documentEditor,
							FC.Environment.Localization.BatchView.Crop,
							documentEditorLock,
							datacontext,
							function () {
								proxy.store.dispatch(proxy.documentEditorActionCreators.activateCommand(-1));
							},
							updatePageById
						);
					}

					if (redactEditor && redactEditor.isEnabled) {
						redactEditor.cancel();
					}

					var isActivated = cropEditor.toggle();
					proxy.store.dispatch(proxy.documentEditorActionCreators.activateCommand(isActivated ? commands.Crop : -1));
					break;

				case commands.Redact:
					if (documentEditorLock()) {
						return;
					}
					if (!redactEditor) {
						redactEditor = new batchApp.datacontext.RedactEditor(
							proxy.documentEditor,
							documentEditorLock,
							datacontext,
							function () {
								proxy.store.dispatch(proxy.documentEditorActionCreators.activateCommand(-1));
							},
							updatePageById
						);
					}

					if (cropEditor && cropEditor.isEnabled) {
						cropEditor.toggle();
					}
					var isActivated = redactEditor.toggle();
					proxy.store.dispatch(proxy.documentEditorActionCreators.activateCommand(isActivated ? commands.Redact : -1));
					if (!isActivated) {
						redactEditor = null;
					}
					break;
			}
		},
		redactEditor: function () {
			return redactEditor
		},
		saveRedact: saveRedact,
		openScanMessageWindow: openScanMessageWindow,
		toolbarItems: toolbarItems,
		mainToolbarEnabled: mainToolbarEnabled,
		afterPagesDeleting: afterPagesDeleting,
		afterPagesMoving: afterPagesMoving,
		afterDocumentDeleting: afterDocumentDeleting,
		afterDocumentMoving: afterDocumentMoving,
		zoomable: zoomable,
		resources: resources,
		getAdditionalContextMenuItems: function (menuType, context) {
			switch (menuType) {
				// документы
				case FC.BatchEditor.SelectionType.DOCS:
					if (!context || context.Model.IsImplicit()) {
						return [];
					}

					return [
						// Переименовать документ			
						{
							order: 5,
							title: FC.Environment.Localization.BatchView.RenameDocument,
							iconClass: 'icon  icon--edit-16',
							action: function (doc) {
								doc.tempName(doc.documentTitle());
								doc.isRenaming(true);
							},
							visible: canRenameDocument
						}, {
							order: 41,
							isSeparator: true
						}
					];

				// страницы
				case FC.BatchEditor.SelectionType.PAGES: return [];

				default: return [];
			}
		},
		renameDoc: function (docId, newName, callback) {
			datacontext.setDocName(docId, newName, callback);
		}
	};

})(window.batchApp.datacontext, window.Layouts.datacontext);
