﻿window.scannerOptions = window.scannerOptions || {};

window.scannerOptions.datacontext = (function () {
	var datacontext = {
		openNativeSettings: openNativeSettings,
		saveOptions: saveOptions,
		getAvailableScannerOptions: getAvailableScannerOptions,
        initScanner: initScanner,
		getScanningProps: getScanningProps
	};

    /**
     * Асинхронный вызов метода плагина с проверкой результата и выводом ошибки
     * @param {} methodName название метода плагина
     * @param {} params параметры метода плагина (Array)
     * @param {} continueFunc функция, которая вызывается в случае успешного выполнения. Имеет 1 параметр, в который передается результат
     * @returns {} 
     */
	function executeSuccessContinue(methodName, params, continueFunc) {
	    scanModuleCommunicator.executeAsync(methodName,
            params,
            function (methodResult) {
                if (methodResult.hasErrors) {
                    FC.Utils.Error.Show(methodResult.errorText);
                } else if (continueFunc != undefined) {
                    continueFunc(methodResult.result);
                }
            });
	}

    /**
     * Выполняет метод и присваивает полученное значение в observable переменную. 
     * @param {} opts объект параметров
     * opts.methodName название метода плагина
     * opts.params параметры метода плагина (Array)
     * opts.observable переменная для сохранения результата
     * opts.defaultValue значение в случае непудачного вызова метода
     * opts.conversionFunc функция преобразрования результата из строки
     * callbackFunction функция, вызываемая после присвоения результата
     * @returns {} 
     */
	function getPluginValue(opts) {
	    scanModuleCommunicator.executeAsync(opts.methodName,
            opts.params,
            function (methodResult) {
                if (methodResult.hasErrors) {
                    FC.Utils.Error.Show(methodResult.errorText);
                    if (opts.defaultValue != undefined) {
                        opts.observable(opts.defaultValue);
                    }
                } else {
                    if (opts.conversionFunc != undefined) opts.observable(opts.conversionFunc(methodResult.result));
                    else opts.observable(methodResult.result);
                }
                if (opts.callbackFunction != undefined) {
                    opts.callbackFunction(opts.observable);
                }
            });
	}

	function saveOptions(options) {
	    executeSuccessContinue('Workspace_KeepBatchType',
            [-2],
            function (batchTypeKeepId) {
                executeSuccessContinue('BatchType_KeepScanningOptions',
                    [batchTypeKeepId],
                    function (scanOptsId) {
                        executeSuccessContinue('ScanningOptions_KeepScanProfile',
                            [scanOptsId],
                            function (scanProfileId) {
                                var numOfSaved = 0;
                                var saveCallback = function() {
                                    numOfSaved++;
                                    if (numOfSaved === 4) {
                                        executeSuccessContinue('ScanProfile_ResetKeptScanProfile',
                                            [scanProfileId],
                                            function() {
                                                scanModuleCommunicator.executeAsync('Workspace_CommitBatchTypeChanges', [-2]);
                                            });
                                    }
                                }
                                scanModuleCommunicator.executeAsync('ScanningOptions_setDontSpecifyForBatchType',
                                [scanOptsId, false], saveCallback);

                                // TODO: убрать затычку true
                                //scanModuleCommunicator.executeAsync('ScanProfile_setUseNativeScannerInterface', [scanProfileResponse.result, options.SelectedInterface() == "native"]);
                                scanModuleCommunicator.executeAsync('ScanProfile_setUseNativeScannerInterface',
                                [scanProfileId, true], saveCallback);

                                // TODO: убрать затычку true
                                //scanModuleCommunicator.executeAsync('ScanProfile_setShowOptionsBeforeScanning', [scanProfileResponse.result, options.showWindowBeforeScan()]);
                                scanModuleCommunicator.executeAsync('ScanProfile_setShowOptionsBeforeScanning',
                                [scanProfileId, options.showWindowBeforeScan()], saveCallback);

                                scanModuleCommunicator.executeAsync('ScanProfile_setScanSourceIndex',
                                [scanProfileId, options.scannerIndex()], saveCallback);
                            });
                    });
            });

	}
    
	function getScanningProps(scanningOptionsObservable) {
	    executeSuccessContinue('Workspace_KeepBatchType',
            [-2],
            function (batchTypeKeepId) {
                executeSuccessContinue('BatchType_KeepScanningOptions',
                    [batchTypeKeepId],
                    function (scanOptsId) {
                        executeSuccessContinue('ScanningOptions_KeepScanProfile',
                            [scanOptsId],
                            function (scanProfileId) {
                                var scanProps = new datacontext.ScannerProfile();
                                getPluginValue({
                                    methodName: 'ScanProfile_getUseNativeScannerInterface',
                                    params: [scanProfileId],
                                    observable: scanProps.selectedInterface,
                                    defaultValue: "native",
                                    conversionFunc: function (val) {
                                        return val === '1' ? "native" : "abbyy";
                                    }
                                });

                                getPluginValue({
                                    methodName: 'ScanProfile_getShowOptionsBeforeScanning',
                                    params: [scanProfileId],
                                    observable: scanProps.showWindowBeforeScan,
                                    defaultValue: false,
                                    conversionFunc: function (val) { return val === '1' }
                                });

                                getPluginValue({
                                    methodName: 'ScanProfile_getScanSourceIndex',
                                    params: [scanProfileId],
                                    observable: scanProps.scannerIndex,
                                    defaultValue: 0,
                                    conversionFunc: parseInt
                                });
                                scanningOptionsObservable(scanProps);
                            });
                    });
            });
	}

    /**
     * Получение информации о подключенных сканерах. 
     * @param {} batchType Объект класса BatchType
     * @param {} scannerOptionsObservable Переменная для записи результата (observable,computed с write). Записывается объект класса ScannerOptions
     * @returns {} 
     */
	function getAvailableScannerOptions(scannerOptionsObservable) {
	    scanModuleCommunicator.executeAsync('Workspace_getScannerCount',[], function (scannersCountResponse) {
	        if (scannersCountResponse.hasErrors) {
	            FC.Utils.Error.Show(scannersCountResponse.errorText);
	            scannerOptionsObservable([]);
	        } else {
	            var scannersCount = parseInt(scannersCountResponse.result);
	            scannerOptionsObservable([]);

	            var numOfInitedScanners = 0;

                var getScannerInfo = function(index) {
                    var currentScannerOptions = new datacontext.ScannerOptions({
                        index: index,
                        isInitialized: false,
                        isNativeInterfaceAvailable: false,
                        isAbbyyInterfaceAvailable: false
                    });

                    executeSuccessContinue('Workspace_KeepScanningSource',
                        [index],
                        function (scannerKeepId) {
                                    numOfInitedScanners++;
                                    if (numOfInitedScanners === scannersCount - 1) {
                                        $(".global-locker").remove();
                                    }
                                    currentScannerOptions.scannerId(scannerKeepId);
                                    getPluginValue({
                                        methodName: 'ScanningSource_getName',
                                        params: [scannerKeepId],
                                        observable: currentScannerOptions.scannerName,
                                        defaultValue: ''
                                    });

                                    /* TODO Вернуть когда сделаем ABBYY интерфейс
                                        var scannerNIAResponse = scanModuleCommunicator.executeAsync('ScanningSource_getIsNativeScannerInterfaceAllowed', [scannerKeepResponse.result]);
                                        var scannerAIAResponse = scanModuleCommunicator.executeAsync('ScanningSource_getIsABBYYScannerInterfaceAllowed', [scannerKeepResponse.result]);
                                        */
                                    scannerOptionsObservable.push(currentScannerOptions);
                        });                    
                }

                if (scannersCount > 1) {
                    $('body').append('<div class="global-locker"></div>')
                }

	            for (var i = 1; i < parseInt(scannersCount) ; i++) {
	                getScannerInfo(i);
	            }
	        }
	    });
	}

    /**
     * Инициализация сканера
     * @param scannerId Id сканера
     * @param currentScannerOptions Переменная для записи результата (observable,computed с write). Записывается объект класса ScannerOptions
     */
    function initScanner(scannerId, currentScannerOptions) {
        scanModuleCommunicator.executeAsync('ScanningSource_Init',
            [scannerId],
            function (scannerInitSuccess) {
                if (scannerInitSuccess.hasErrors) {
                    return;
                }
                currentScannerOptions.isInitialized(true);
                getPluginValue({
                    methodName: 'ScanningSource_getIsNativeScannerSilentInterfaceAllowed',
                    params: [scannerId],
                    observable: currentScannerOptions.isNativeSilentModeAvailable,
                    defaultValue: '',
                    conversionFunc: function (val) {
                        return val === '1'
                    }
                });
            });
    }

    

    /**
     * Открытие окна настроек сканера из драйвера
     * @param {} options Объект класса ScannerOptions
     * @returns {} 
     */
	function openNativeSettings(options) {
	    executeSuccessContinue('Workspace_KeepScanningSource',
            [options.scannerIndex()],
            function (scannerKeepId) {
                executeSuccessContinue('ScanningSource_Init',
                    [scannerKeepId],
                    function () {
                        executeSuccessContinue('Workspace_KeepBatchType',
                            [-2],
                            function (keepBatchTypeId) {
                                executeSuccessContinue('BatchType_KeepScanningOptions',
                                    [keepBatchTypeId],
                                    function (keepScanOptsId) {
                                        executeSuccessContinue('ScanningOptions_KeepScanProfile',
                                            [keepScanOptsId],
                                            function (keepScanProfileId) {
                                                executeSuccessContinue('ScanProfile_setScanSourceIndex',
                                                    [keepScanProfileId, options.scannerIndex()],
                                                    function () {
                                                        executeSuccessContinue('ScanProfile_ShowNativeScanDialog',
                                                            [keepScanProfileId],
                                                            function () {
                                                                executeSuccessContinue('ScanProfile_RestoreKeptScanProfile',
                                                                    [keepScanProfileId]);
                                                            });
                                                    });
                                            });
                                    });
                            });
                    });
            });
	}

	

	return datacontext;
})();