﻿/**
* Модели типа пакета и пакета
**/

(function (ko, datacontext) {
	datacontext.Batch = Batch;
	

    /**
     * Класс пакет
     * @param {} data сериализованная информация о пакете по версии ScanningPluginBR 
     * 
     */
	function Batch(data) {
		var self = this;
		var isNewVersionPlugin = JSON.parse(localStorage.getItem('isNewVersionPlugin'));

		if(isNewVersionPlugin) {

			//Id пакета
			self.Id = ko.observable(data.id);

			//Id Типа пакета
			self.BatchTypeId = ko.observable(data.batchTypeId);


			self.ServerId = ko.observable(data.serverId);

			//Id задачи пересканирования, к которой относится пакет. -1 если не является пакетом задачи пересканирования
			self.RescanTaskID = ko.observable(data.rescanTaskId);

			//Имя пакета
			self.Name = ko.observable(data.name);

			// Количество явных документов пакета (которые являются документами при отправке на сервер обработки)
			self.ExplicitDocumentCount = ko.observable(data.explicitDocumentCount);

			// Общее количество документов пакета
			self.DocumentCount = ko.observable(data.documentCount);

			//Количество страниц в пакете
			self.PageCount = ko.observable(data.pageCount);

			//Количество страниц, загруженных на сервер обработки
			self.UploadedPageCount = ko.observable(data.uploadedPageCount);

			//Приоритет пакета (0-4)
			self.Priority = ko.observable(data.priority);

			//Описание
			self.Description = ko.observable(data.description);

			//Комментарий к задаче пересканирования (пустая строка для обычных пакетов)
			self.RescanComment = ko.observable(data.rescanComment);

			//Статус пакета ( -1 - 4)
			self.Status = ko.observable(data.status);


			var creationDate = data.creationTime;

			if (creationDate != null) {
				creationDate = new Date(data.creationTime);
			}

			//Дата создания пакета
			self.CreationTime = ko.observable(creationDate);
	
			//ВРемя загрузки пакета
			self.UploadedTime = ko.observable(data.uploadedTime);
	
			//Комментарии процесса отправки
			self.ProcessingNotes = ko.observable(data.processingNotes);
	
			//Начала периода отправки пакета
			self.UploadingTimeRangeStart = ko.observable(data.uploadingTimeRangeStart);
	
			//Конец периода отправки пакета
			self.UploadingTimeRangeStop = ko.observable(data.uploadingTimeRangeStop);
	
			//Число попыток отправки пакета
			self.RepeatFailedTaskUploadingAttempts = ko.observable(data.repeatFailedTaskUploadingAttempts);
	
			self.RegParams = ko.observableArray(data.regParams);

			//СТиль отображения времени создания пакета
			self.CreationTimeTimeFormat = ko.computed(function () {
				return (creationDate.getHours() < 10 ? '0' : '')
					+ creationDate.getHours()
					+ ':'
					+ (creationDate.getMinutes() < 10 ? '0' : '')
					+ creationDate.getMinutes();
			});

			self.CreationDateFormat = ko.computed(function() {
				var dd = creationDate.getDate();
				var mm = creationDate.getMonth() + 1; //January is 0!
				var yyyy = creationDate.getFullYear();
	
				if (dd < 10) {
					dd = '0' + dd;
				}
				if (mm < 10) {
					mm = '0' + mm;
				}
				return  dd + '.' + mm + '.' + yyyy;
			});
		} else {
			dataItems = data.split(':');

			//Id пакета
			self.Id = ko.observable(parseInt(dataItems[0]));
	
			//Id Типа пакета
			self.BatchTypeId = ko.observable(dataItems[1]);
	
			self.ServerId = ko.observable(parseInt(dataItems[2]));
	
			//Id задачи пересканирования, к которой относится пакет. -1 если не является пакетом задачи пересканирования
			self.RescanTaskID = ko.observable(parseInt(dataItems[3]));
	
			//Имя пакета
			self.Name = ko.observable(scanModuleCommunicator.decodeText(dataItems[4]));
	
			// Количество явных документов пакета (которые являются документами при отправке на сервер обработки)
			self.ExplicitDocumentCount = ko.observable(dataItems[5]);
	
			// Общее количество документов пакета
			self.DocumentCount = ko.observable(dataItems[6]);
	
			//Количество страниц в пакете
			self.PageCount = ko.observable(parseInt(dataItems[7]));
	
			//Количество страниц, загруженных на сервер обработки
			self.UploadedPageCount = ko.observable(parseInt(dataItems[8]));
	
			//Приоритет пакета (0-4)
			self.Priority = ko.observable(parseInt(dataItems[9]));
	
			//Описание
			self.Description = ko.observable(scanModuleCommunicator.decodeText(dataItems[10]));
	
			//Комментарий к задаче пересканирования (пустая строка для обычных пакетов)
			self.RescanComment = ko.observable(scanModuleCommunicator.decodeText(dataItems[11]));
	
			//Статус пакета ( -1 - 4)
			self.Status = ko.observable(parseInt(dataItems[12]));
	
	
			var creationDate = dataItems[13];

			if (creationDate != null) {
				var splitedDate = creationDate.split('_');
				creationDate = new Date(
					(parseInt(splitedDate[1]) + 1)
					+ '/'
					+ splitedDate[2]
					+ '/' + splitedDate[0]
					+ ' ' + splitedDate[3]
					+ ':' + splitedDate[4]
					+ ':' + splitedDate[5]
				);
			}
	
			//Дата создания пакета
			self.CreationTime = ko.observable(creationDate);
	
			//ВРемя загрузки пакета
			self.UploadedTime = ko.observable(dataItems[14]);
	
			//Комментарии процесса отправки
			self.ProcessingNotes = ko.observable(scanModuleCommunicator.decodeText(dataItems[15]));
	
			//Начала периода отправки пакета
			self.UploadingTimeRangeStart = ko.observable(parseInt(dataItems[16]));
	
			//Конец периода отправки пакета
			self.UploadingTimeRangeStop = ko.observable(parseInt(dataItems[17]));
	
			//Число попыток отправки пакета
			self.RepeatFailedTaskUploadingAttempts = ko.observable(parseInt(dataItems[18]));

			self.RegParams = ko.observableArray([]);

			//СТиль отображения времени создания пакета
			self.CreationTimeTimeFormat = ko.computed(function () {
				return (self.CreationTime().getHours() < 10 ? '0' : '')
					+ self.CreationTime().getHours()
					+ ':'
					+ (self.CreationTime().getMinutes() < 10 ? '0' : '')
					+ self.CreationTime().getMinutes();
			});

			self.CreationDateFormat = ko.computed(function() {
				var dd = self.CreationTime().getDate();
				var mm = self.CreationTime().getMonth() + 1; //January is 0!
				var yyyy = self.CreationTime().getFullYear();

				if (dd < 10) {
					dd = '0' + dd;
				}
				if (mm < 10) {
					mm = '0' + mm;
				}
				return  dd + '.' + mm + '.' + yyyy;
			});
		}

	    self.CreationDateFormat = ko.computed(function() {
	        var dd = creationDate.getDate();
	        var mm = creationDate.getMonth() + 1; //January is 0!
	        var yyyy = creationDate.getFullYear();

	        if (dd < 10) {
	            dd = '0' + dd;
	        }
	        if (mm < 10) {
	            mm = '0' + mm;
	        }
	        return  dd + '.' + mm + '.' + yyyy;
	    });

        //Сериализация объекта в JSON
		self.toJson = function () { return ko.toJSON(self) };

        //Текстовое представление приоритета для картинки отображения
		self.priorityToText = ko.computed(function () {
			switch(self.Priority()){
			    case 0: return FC.Environment.Localization.Batches.PriorityLowest;
			    case 1: return FC.Environment.Localization.Batches.PriorityLow;
			    case 3: return FC.Environment.Localization.Batches.PriorityHigh;
			    case 4: return FC.Environment.Localization.Batches.PriorityHighest;
			    case 2:
			    default:
			        return FC.Environment.Localization.Batches.PriorityNormal;
			}
		});

	    //Текстовое представление статуса для картинки отображения
		self.statusPic = ko.computed(function () {
			switch (self.Status()) {
				case -1:
				    return FC.Environment.Localization.NonLocalizedResources.StatusNew;
				case 0:
				case 1:
				    return FC.Environment.Localization.NonLocalizedResources.StatusPending;
				case 2:
				    return FC.Environment.Localization.NonLocalizedResources.StatusWaiting;
				case 3:
				    return FC.Environment.Localization.NonLocalizedResources.StatusUploading;
				case 4:
				    return FC.Environment.Localization.NonLocalizedResources.StatusUploaded;

			}
			return FC.Environment.Localization.NonLocalizedResources.StatusError;
		});


        //Описание статуса пакета
	    self.statusDescription = ko.computed(function() {
	        if (self.Status() === 2) {
	            if (self.ProcessingNotes().length > 0)
	                return self.ProcessingNotes();
	            if (self.UploadingTimeRangeStart() > 0 || self.UploadingTimeRangeStop() > 0) {
	                var startHours = parseInt(self.UploadingTimeRangeStart() / 60);
	                if (startHours < 10) {
	                    startHours = "0" + startHours;
	                }
	                var startMinutes = parseInt(self.UploadingTimeRangeStart() % 60);
	                if (startMinutes < 10) {
	                    startMinutes = "0" + startMinutes;
	                }
	                var stopHours = parseInt(self.UploadingTimeRangeStop() / 60);
	                if (stopHours < 10) {
	                    stopHours = "0" + stopHours;
	                }
	                var stopMinutes = parseInt(self.UploadingTimeRangeStop() % 60);
	                if (stopMinutes < 10) {
	                    stopMinutes = "0" + stopMinutes;
	                }
	                return FC.Environment.Localization.Batches.UploadedTimeRangeDescription
	                    .replace('{0}', startHours)
	                    .replace('{1}', startMinutes)
	                    .replace('{2}', stopHours)
	                    .replace('{3}', stopMinutes);
	            }
	        }
	        if (self.Status() === 5) {
	            return self.ProcessingNotes();
	        }

	        if ((self.Status() === 3) ||
                (self.Status() === 0 && self.ServerId() >= 0
				&& (self.RescanTaskID() < 0 || self.UploadedPageCount() > 0))) {
	            return FC.Environment.Localization.Batches.UploadedPageCount
                    .replace('{0}', self.UploadedPageCount())
                    .replace('{1}', self.PageCount());
	        }
	        if (self.Status() === -1 || (self.Status() === 0 && self.RescanTaskID() >= 0
				&& self.UploadedPageCount() === 0)) {
	            return self.RescanComment();
	        }

	    
	        return '';
	    });


        //Текстовое описание статуса
		self.stringStatus = ko.computed(function () {
			switch (self.Status()) {
				case -1:
				    return FC.Environment.Localization.Batches.BatchStageRescanTask;
			    case 0:
			    case 1:
			        if (self.UploadedPageCount() > 0) {
			            return FC.Environment.Localization.Batches.BatchStageCanceled;
			        }
			        else {
			            return FC.Environment.Localization.Batches.BatchStagePending;
			        }
                    
				case 2:
				    return self.ProcessingNotes().length>0 ? 
                        FC.Environment.Localization.Batches.RepeatFailedUploadingStatus.replace('{0}', self.RepeatFailedTaskUploadingAttempts() + 1) :
                        FC.Environment.Localization.Batches.BatchStageWaiting;
				case 3:
				    return FC.Environment.Localization.Batches.BatchStageUploading;
				case 4:
				    return FC.Environment.Localization.Batches.BatchStageUploaded;

			}
			return FC.Environment.Localization.Batches.BatchStageFailed;
		});
	};
	

})(ko, batchesApp.datacontext);