﻿/**
* Модели для настроек типа пакета
**/

(function (ko, datacontext) {

	datacontext.BackgroundSettings = BackgroundSettings;
	//Настройки отложенной отправки
	function BackgroundSettings(initInfo) {
		var self = this;

		initInfo = initInfo || {};

		//Использование несерверных настроек
		self.useSpec = ko.observable(true);

		//Индикатор удаления загруженных пакетов
		self.deleteUploaded = ko.observable(initInfo.deleteUploaded);

		//Включение отправки по расписанию
		self.specifyUploadTime = ko.observable(initInfo.specifyUploadTime);

		//Время начала временного промежутка отправки по расписанию(в минутах)
		self.startUploadTime = ko.observable(initInfo.startUploadTime);

		//Время конца временного промежутка отправки по расписанию(в минутах)
		self.endUploadTime = ko.observable(initInfo.endUploadTime);

		//Индикатор повторной отправки пакета с ошибками(в минутах)
		self.repeatFailedUpload = ko.observable(initInfo.repeatFailedUpload);

		//Временной промежуток повторной отправки
		self.repeatFailedUploadTime = ko.observable(initInfo.repeatFailedUploadTime);

		//Число попыток повторнйо отправки
		self.repeatFailedUploadAttempts = ko.observable(initInfo.repeatFailedUploadAttempts);


		self.startUploadTimeHourFilled = ko.observable(true);
		self.startUploadTimeMinuteFilled = ko.observable(true);
		self.endUploadTimeHourFilled = ko.observable(true);
		self.endUploadTimeMinuteFilled = ko.observable(true);
		//Час начала отправки
		self.startUploadTimeHour = ko.computed({
			read: function () {
				return parseInt(self.startUploadTime() / 60);
			},
			write: function (value) {
				self.startUploadTimeHourFilled(!!value);
				value = parseInt(value);

				if (isNaN(value))
					return;

				if (value < 0)
					value = 0;

				if (value > 23)
					value = 23;
				var startMins = self.startUploadTimeMinute();

				self.startUploadTime(startMins + 60 * value);
				self.startUploadTime.valueHasMutated();
			}
		});

		//Количество минут начала отправки с начала часа
		self.startUploadTimeMinute = ko.computed({
			read: function () {
				return self.startUploadTime() - self.startUploadTimeHour() * 60;
			},
			write: function (value) {
				self.startUploadTimeMinuteFilled(!!value);
				value = parseInt(value);
				if (isNaN(value))
					return;

				if (value < 0)
					value = 0;

				if (value > 59)
					value = 59;
				var startHours = self.startUploadTimeHour();

				self.startUploadTime(value + 60 * startHours);
				self.startUploadTime.valueHasMutated();
			}
		});

		//Час конца отправки
		self.endUploadTimeHour = ko.computed({
			read: function () {
				return parseInt(self.endUploadTime() / 60);
			},
			write: function (value) {
				self.endUploadTimeHourFilled(!!value);

				value = parseInt(value);

				if (isNaN(value))
					return;

				if (value < 0)
					value = 0;
				if (value > 23)
					value = 23;

				var endMins = self.endUploadTimeMinute();

				self.endUploadTime(endMins + 60 * value);
			}
		});

		//Количество минут конца отправки с начала часа
		self.endUploadTimeMinute = ko.computed({
			read: function () {
				return self.endUploadTime() - self.endUploadTimeHour() * 60;
			},
			write: function (value) {
				self.endUploadTimeMinuteFilled(!!value);

				value = parseInt(value);

				if (isNaN(value))
					return;

				if (value < 0)
					value = 0;

				if (value > 59)
					value = 59;
				var endHours = self.endUploadTimeHour();
				self.endUploadTime(value + 60 * endHours);
			}
		});

		self.returnToDefaults = function () {
			self.repeatFailedUpload(false);
			self.specifyUploadTime(false);
			self.deleteUploaded(true);
			self.startUploadTime(0);
			self.endUploadTime(0);
			self.repeatFailedUploadTime(5);
			self.repeatFailedUploadAttempts(3);
		}

		self.hasUploadTimeError = ko.computed(function() {
			return self.specifyUploadTime() && (
				(self.startUploadTime() === self.endUploadTime()) ||
					!self.startUploadTimeHourFilled() ||
					!self.startUploadTimeMinuteFilled() ||
					!self.endUploadTimeHourFilled() ||
					!self.endUploadTimeMinuteFilled());
		});
		

		self.toJs = function() {
			return {
				deleteUploaded: self.deleteUploaded(),
				specifyUploadTime: self.specifyUploadTime(),
				startUploadTime: self.startUploadTime(),
				endUploadTime: self.endUploadTime(),
				repeatFailedUpload: self.repeatFailedUpload(),
				repeatFailedUploadTime: self.repeatFailedUploadTime(),
				repeatFailedUploadAttempts: self.repeatFailedUploadAttempts()
			};
		}
	}
})(ko, window.batchtypeSettings.datacontext);