﻿/**
* Модели для настроек типа пакета
**/

(function (ko, datacontext) {

	datacontext.BatchProperties = BatchProperties;
	//Класс настроек пакета
	function BatchProperties(initInfo) {
		var self = this;
		initInfo = initInfo || {};
		//Использование несерверных настроек
		self.useSpec = ko.observable(true);

		//Разрешение настроивать пользователю
		self.forbidOperator = ko.observable(initInfo.forbidOperator);

		//Числовой эквивалент приоритета  (0-4)
		self.priorInd = ko.observable(initInfo.priority);

		//Список возможных приоритетов пакета
		self.availablePriorities = [
            FC.Environment.Localization.Batches.PriorityLowest,
	        FC.Environment.Localization.Batches.PriorityLow,
	        FC.Environment.Localization.Batches.PriorityNormal,
	        FC.Environment.Localization.Batches.PriorityHigh,
	        FC.Environment.Localization.Batches.PriorityHighest];

		//Схема именования пакета
		self.priority = ko.computed({
			read: function () {
				return self.availablePriorities[self.priorInd()];
			},
			write: function (val) {
				self.priorInd(self.availablePriorities.indexOf(val));
			}
		});

		//Схема именования пакета
		self.namingScheme = ko.observable(new datacontext.MaskedStringValue(initInfo.namingScheme));

		/** Имя пакета (фильтрует запрещённые символы) */
		self.name = ko.pureComputed({
			read: function () {
				return self.namingScheme().Value();
			},
			write: function (val) {
				// Нужно, чтобы в любом случае произошло оповещение о смене значения (чтобы удалить из поля запрещённые символы)
				self.namingScheme().Value(val);
				self.namingScheme().Value(val.replace(window.batchtypeSettings.DeniedNameCharsSimple, ''));
			}
		});

		self.returnToDefaults = function () {
			self.priority(FC.Environment.Localization.Batches.PriorityNormal);
			self.namingScheme(new datacontext.MaskedStringValue(''));
		}


		self.toJs = function () {
			return {
				forbidOperator: self.forbidOperator(),
				priorInd: self.priorInd(),
				namingScheme: self.namingScheme().Value()
			};
		}
	}
	
})(ko, window.batchtypeSettings.datacontext);