﻿/**
* Модели для настроек типа пакета
**/

(function (ko, datacontext) {
	
	datacontext.DocumentProperties = DocumentProperties;
	
	//Класс настроек документа
	function DocumentProperties(initInfo) {
		var self = this;
		initInfo = initInfo || {};
		//Список возможных типов баркода
		self.availableBarcodeTypes = [
            FC.Environment.Localization.Batches.BarTypesAutoDet,
            FC.Environment.Localization.NonLocalizedResources.BarTypesAustral,
			FC.Environment.Localization.NonLocalizedResources.BarTypesAztec,
			FC.Environment.Localization.NonLocalizedResources.BarTypesCoda,
			FC.Environment.Localization.NonLocalizedResources.BarTypesCC,
			FC.Environment.Localization.NonLocalizedResources.BarTypesCode128,
			FC.Environment.Localization.NonLocalizedResources.BarTypesCode32,
			FC.Environment.Localization.NonLocalizedResources.BarTypesCode39,
			FC.Environment.Localization.NonLocalizedResources.BarTypesCC39,
			FC.Environment.Localization.NonLocalizedResources.BarTypesCC39A,
			FC.Environment.Localization.NonLocalizedResources.BarTypesCode93,
			FC.Environment.Localization.NonLocalizedResources.BarTypesDataMatr,
			FC.Environment.Localization.NonLocalizedResources.BarTypesEAN13,
			FC.Environment.Localization.NonLocalizedResources.BarTypesEAN8,
			FC.Environment.Localization.NonLocalizedResources.BarTypesIATA,
			FC.Environment.Localization.NonLocalizedResources.BarTypesInd,
			FC.Environment.Localization.NonLocalizedResources.BarTypesIntel,
			FC.Environment.Localization.NonLocalizedResources.BarTypesInter,
			FC.Environment.Localization.NonLocalizedResources.BarTypesInterC,
			FC.Environment.Localization.NonLocalizedResources.BarTypesMatr,
			FC.Environment.Localization.NonLocalizedResources.BarTypesPatchcode,
			FC.Environment.Localization.NonLocalizedResources.BarTypesPDF,
			FC.Environment.Localization.NonLocalizedResources.BarTypesPostnet,
			FC.Environment.Localization.NonLocalizedResources.BarTypesQR,
			FC.Environment.Localization.NonLocalizedResources.BarTypesUCC,
			FC.Environment.Localization.NonLocalizedResources.BarTypesUPCA,
			FC.Environment.Localization.NonLocalizedResources.BarTypesUPCE
		];

		//Список возможных способов разбиения
		self.separationTypes = [
			FC.Environment.Localization.Batches.SepTypesNever,
            FC.Environment.Localization.Batches.SepTypesOnce,
            FC.Environment.Localization.Batches.SepTypesEvN,
            FC.Environment.Localization.Batches.SepTypesNoBar,
            FC.Environment.Localization.Batches.SepTypesBlank,
            FC.Environment.Localization.Batches.SepTypesImg
		];

		//Использование несерверных настроек
		self.useSpec = ko.observable(true);

		//Разрешение настроивать пользователю
		self.forbidOperator = ko.observable(initInfo.forbidOperator);

		//Схема именования документа
		self.namingScheme = ko.observable(new datacontext.MaskedStringValue(initInfo.namingScheme));

		/** Имя документа (фильтрует запрещённые символы) */
		self.name = ko.pureComputed({
			read: function () {
				return self.namingScheme().Value();
			},
			write: function (val) {
				// Нужно, чтобы в любом случае произошло оповещение о смене значения (чтобы удалить из поля запрещённые символы)
				self.namingScheme().Value(val);
				self.namingScheme().Value(val.replace(window.batchtypeSettings.DeniedNameCharsSimple, ''));
			}
		});

		//Тип баркода
		self.barTypeInd = ko.observable(initInfo.barTypeInd);


		self.barValueActivated = ko.observable(!!initInfo.barValue);
		//Значение баркода
		self.barValue = ko.observable(initInfo.barValue);

		//Индикатор удаления страниц-разделителей
		self.deleteSeparationPages = ko.observable(initInfo.deleteSeparationPages);

		//Число страниц в документе (для типа разбиения на документы по числу страниц)
		self.pagesInDocument = ko.observable(initInfo.pagesInDocument);

		//Тип разбиения на документы
		self.documentsSeparationTypeInd = ko.observable(initInfo.documentsSeparationTypeInd || 0);

		//Настройки поиска пустых страниц
		self.emptyPageDetectOpts = ko.observable(new datacontext.EmptyPageDetectOpts(initInfo.emptyPageDetectOpts));

		//Индикатор разбиения на документы
		self.createDocument = ko.observable(initInfo.documentsSeparationTypeInd !== 0);


		//Текстовая интерпретация типа разбиения на докменты
		self.documentsSeparationType = ko.computed({
			read: function () {
				return self.separationTypes[self.documentsSeparationTypeInd()];
			},
			write: function (val) {
				var typeIndex = self.separationTypes.indexOf(val);
				if (typeIndex != -1) {
					self.documentsSeparationTypeInd(typeIndex);
					self.createDocument(typeIndex > 0);
				}
			}
		});

		//Текстовая интерпретация типа баркода
		self.barType = ko.computed({
			read: function () {
				return self.availableBarcodeTypes[self.barTypeInd()];
			},
			write: function (val) {
				self.barTypeInd(self.availableBarcodeTypes.indexOf(val));
			}
		});

		self.returnToDefaults = function () {
			self.documentsSeparationTypeInd(0);
			self.namingScheme().Value('');
			self.emptyPageDetectOpts().returnToDefaults();
			self.pagesInDocument(5);
			self.barTypeInd(0);
			self.barValue('');
			self.barValueActivated(false);
			self.deleteSeparationPages(false);
		}

		self.toJs = function () {
			return {
				forbidOperator: self.forbidOperator(),
				namingScheme: self.namingScheme().Value(),
				documentsSeparationTypeInd: self.documentsSeparationTypeInd(),
				pagesInDocument: self.pagesInDocument(),
				barTypeInd: self.barTypeInd(),
				barValue: self.barValue(),
				barValueActivated: self.barValueActivated(),
				deleteSeparationPages: self.deleteSeparationPages(),
				emptyPageDetectOpts: self.emptyPageDetectOpts().toJs()
			};
		}

		
	}
})(ko, window.batchtypeSettings.datacontext);