﻿/**
* Модели для настроек типа пакета
**/

(function (ko, datacontext) {
	
	datacontext.EmptyPageDetectOpts = EmptyPageDetectOpts;

	//Класс для числовых свойств, где значение -1 означает выключение этого свойства
	function TurnableInteger(value) {
		var self = this;

		self.isActive = ko.observable(value !== -1);
		self.value = ko.observable(Math.max(0, value));
		
		self.value.subscribe(function (val) {
			if (typeof val == 'string') {
				self.value(parseInt(val.replace(/\D/g, '')));
			}


		});
		self.realIntegerValue = ko.computed({
			read: function () {
				return self.isActive() ? self.value() : -1;
			},
			write: function (val) {
				if (val < 0) {
					self.isActive(false);
					self.value(0);
				} else {
					self.isActive(true);
					self.value(val);
				}
			}
		})

	}

	//Файл настроек поиска пустых страниц
	function EmptyPageDetectOpts(initInfo) {
		var self = this;
		initInfo = initInfo || {};
		//Индикатор автоматического поиска основного содержания
		self.autoDetectKernalRect = ko.observable(initInfo.autoDetectKernalRect);

		//Размер левого отступа (прямоугольника с содержанием)
		self.leftMargin = ko.observable(initInfo.leftMargin);

		//Размер правого отступа (прямоугольника с содержанием)
		self.rightMargin = ko.observable(initInfo.rightMargin);

		//Размер отступа сверху (прямоугольника с содержанием)
		self.topMargin = ko.observable(initInfo.topMargin);

		//Размер отступа снизу (прямоугольника с содержанием)
		self.bottomMargin = ko.observable(initInfo.bottomMargin);

		//Число черных квадратов
		self.maxBlackSquaresCount = new TurnableInteger(parseInt(initInfo.maxBlackSquaresCount));
		

		//Максимальное число текстовых элементов
		self.maxTextObjectsCount = new TurnableInteger(parseInt(initInfo.maxTextObjectsCount));
		

		//Максимальное число баркодов
		self.maxBarcodesCount = new TurnableInteger(parseInt(initInfo.maxBarcodesCount));
		

		//Максимальный процент темного
		self.maxBlacknessPercentage = new TurnableInteger(parseInt(initInfo.maxBlacknessPercentage));
		

		self.returnToDefaults = function () {
			self.autoDetectKernalRect(true);
			self.leftMargin(0);
			self.rightMargin(0);
			self.topMargin(0);
			self.bottomMargin(0);
			self.maxBlackSquaresCount.value(1);
			self.maxBlackSquaresCount.isActive(true);
			self.maxTextObjectsCount.value(0);
			self.maxTextObjectsCount.isActive(true);
			self.maxBarcodesCount.value(0);
			self.maxBarcodesCount.isActive(true);
			self.maxBlacknessPercentage.value(0);
			self.maxBlacknessPercentage.isActive(false);

		}
		self.toJs = function() {
			return {
				autoDetectKernalRect: self.autoDetectKernalRect(),
				leftMargin: self.leftMargin(),
				rightMargin: self.rightMargin(),
				topMargin: self.topMargin(),
				bottomMargin: self.bottomMargin(),
				maxBlackSquaresCount: {
					value: self.maxBlackSquaresCount.value(),
					isActive: self.maxBlackSquaresCount.isActive(),
				},
				maxTextObjectsCount: {
					value: self.maxTextObjectsCount.value(),
					isActive: self.maxTextObjectsCount.isActive(),
				},
				maxBarcodesCount: {
					value: self.maxBarcodesCount.value(),
					isActive: self.maxBarcodesCount.isActive(),
				},
				maxBlacknessPercentage: {
					value: self.maxBlacknessPercentage.value(),
					isActive: self.maxBlacknessPercentage.isActive(),
				}
				
			};
		}

	}

})(ko, window.batchtypeSettings.datacontext);