﻿/**
* Модели для настроек типа пакета
**/

(function (ko, datacontext) {

	datacontext.MaskedStringValue = MaskedStringValue;
	//Класс текстового поля с поддержкой предзаданных шаблонов
	function MaskedStringValue(value) {

		var self = this;

		//Значение поля
		self.Value = ko.observable(value == undefined ? '' : value);


		//Рекурсивное удаление шаблона из значения
		function removeFromValue(pattern) {
			var oldValue;
			do {
				oldValue = self.Value();
				self.Value(self.Value().replace(pattern, ''));
			} while (oldValue !== self.Value())
		}

		//Добавление шаблона в значение
		function addToValue(pattern, position) {
			if (isFinite(position)) {
				self.Value([self.Value().slice(0, position), pattern, self.Value().slice(position)].join(''));
			} else {
				self.Value(self.Value() + pattern);
			}
			return pattern;
		}

		//Проверка наличия шаблона в значении
		function containsPatterns(pattern) {
			return self.Value().indexOf(pattern) !== -1;
		}

		/** Добавление или удаление шаблона из значения

         * 
         * @param {} addPattern -bool добавление или удаление шаблона
         * @param {} pattern - шаблон         
         */

		// Добавление шаблона оператора
		self.addOperator = function (index) {
			return addToValue(FC.Environment.Localization.NonLocalizedResources.PatternOperator, index);
		};

		self.addBarCode = function (index) {
			return addToValue(FC.Environment.Localization.NonLocalizedResources.PatternBarcode, index);
		};

		self.addNumber = function (index) {
			return addToValue(FC.Environment.Localization.NonLocalizedResources.PatternNumber, index);
		};

		self.addDate = function (index) {
			return addToValue(FC.Environment.Localization.NonLocalizedResources.PatternDate, index);
		};

		self.addTime = function (index) {
			return addToValue(FC.Environment.Localization.NonLocalizedResources.PatternTime, index);
		};

		self.addBatchType = function (index) {
			return addToValue(FC.Environment.Localization.NonLocalizedResources.PatternBatchType, index);
		};


		self.addBatchName = function (index) {
			return addToValue(FC.Environment.Localization.NonLocalizedResources.PatternBatchName, index);
		};

		self.addStation = function (index) {
			return addToValue(FC.Environment.Localization.NonLocalizedResources.PatternStation, index);
			
		};


		function tooglePattern(addPattern, pattern) {
			if (addPattern)
				addToValue(pattern);
			else
				removeFromValue(pattern);
		}

		// Наличие шаблона оператора
		self.HasOperator = ko.computed({
			read: function () {
				return containsPatterns(FC.Environment.Localization.NonLocalizedResources.PatternOperator);
			},
			write: function (value) {
				tooglePattern(value, FC.Environment.Localization.NonLocalizedResources.PatternOperator);
			}
		});

		// Наличие шаблона баркода
		self.HasBarCode = ko.computed({
			read: function () {
				return containsPatterns(FC.Environment.Localization.NonLocalizedResources.PatternBarcode);
			},
			write: function (value) {
				tooglePattern(value, FC.Environment.Localization.NonLocalizedResources.PatternBarcode);
			}
		});

		// Наличие шаблона номера
		self.HasNumber = ko.computed({
			read: function () {
				return containsPatterns(FC.Environment.Localization.NonLocalizedResources.PatternNumber);
			},
			write: function (value) {
				tooglePattern(value, FC.Environment.Localization.NonLocalizedResources.PatternNumber);
			}
		});

		// Наличие шаблона даты
		self.HasDate = ko.computed({
			read: function () {
				return containsPatterns(FC.Environment.Localization.NonLocalizedResources.PatternDate);
			},
			write: function (value) {
				tooglePattern(value, FC.Environment.Localization.NonLocalizedResources.PatternDate);
			}
		});

		// Наличие шаблона времени
		self.HasTime = ko.computed({
			read: function () {
				return containsPatterns(FC.Environment.Localization.NonLocalizedResources.PatternTime);
			},
			write: function (value) {
				tooglePattern(value, FC.Environment.Localization.NonLocalizedResources.PatternTime);
			}
		});

		// Наличие шаблона названия пакета
		self.HasBatchName = ko.computed({
			read: function () {
				return containsPatterns(FC.Environment.Localization.NonLocalizedResources.PatternBatchName);
			},
			write: function (value) {
				tooglePattern(value, FC.Environment.Localization.NonLocalizedResources.PatternBatchName);
			}
		});

		// Наличие шаблона типа пакета
		self.HasBatchType = ko.computed({
			read: function () {
				return containsPatterns(FC.Environment.Localization.NonLocalizedResources.PatternBatchType);
			},
			write: function (value) {
				tooglePattern(value, FC.Environment.Localization.NonLocalizedResources.PatternBatchType);
			}
		});

		// Наличие шаблона станции
		self.HasStation = ko.computed({
			read: function () {
				return containsPatterns(FC.Environment.Localization.NonLocalizedResources.PatternStation);
			},
			write: function (value) {
				tooglePattern(value, FC.Environment.Localization.NonLocalizedResources.PatternStation);
			}
		});

		// Копирование
		self.Clone = function () {
			return new MaskedStringValue(self.Value());
		}

	}
	
})(ko, window.batchtypeSettings.datacontext);