﻿/**
* Модели для настроек типа пакета
**/

(function (ko, datacontext) {

	//Enum принадлежности рег параметра
	datacontext.ParameterTarget = {
		//Рег параметр документа
		DOCUMENT: 1,
		//Рег параметр комплекта
		COMPLECT: 2,
		//Рег параметр пакета
		BATCH: 3
	};

	//Тип рег параметра
	datacontext.ParameterType = {
		//Текстовый рег параметр
		TEXT: 1,
		//Рег параметр с предопределенным набором значений
		SELECT: 2
	}
	
	datacontext.RegParameter = (function () {
		var nextId = 1;

		//Класс Регистрационный параметр
		return function RegParameter() {
			var self = this;

			//Локальный идентификатор регистрационного параметра 
			self.id = nextId++;

			//Имя регистрационного параметра
			self.Name = ko.observable('');

			//Значение по умолчанию
			self.DefaultValue = ko.observable(new datacontext.MaskedStringValue(''));

			//Является ли рег параметр обязательным на сервере
			self.ServerRequired = ko.observable(false);

			//Является ли рег параметр обязательным
			self.IsRequired = ko.observable(false);


			//Выгружен ли рег параметр с сервера
			self.IsServerParameter = ko.observable(false);

			//Является ли рег параметр только для чтения
			self.IsReadOnly = ko.observable(false);

			//Принадлежность  рег параметра (Enum)
			self.ParameterTarget = ko.observable(datacontext.ParameterTarget.DOCUMENT);

			//Тип регистрационного параметра(Enum)
			self.ParameterType = ko.observable(datacontext.ParameterType.TEXT);

			//Текстовое отображение типа регистрационного параметра
			self.ParameterTypeString = ko.computed(function () {
				if (self.ParameterType() == datacontext.ParameterType.SELECT)
					return FC.Environment.Localization.Batches.RPTypeDropDown;
				else
					return FC.Environment.Localization.Batches.RPTypeText;
			});

			//Проверка несовместности заданных параметров. 
			// True - если рег параметр является обязательным, только для чтения и без значения по умолчанию.
			// False в остальных случаях
			self.IsLocked = ko.computed(function () {
				return ((self.DefaultValue() == undefined) ||
					(self.DefaultValue().Value() == undefined) ||
					(self.DefaultValue().Value().length === 0)) &&
					self.IsRequired() &&
					self.IsReadOnly();
			});

			//Список значений рег параметра
			self.ListValues = ko.observableArray([]);


			//Проверка наличия значений для рег параметра с заданным список значений. 
			//Для текстового рег параметра всегда FALSE
			self.IsDropDownEmty = ko.computed(function () {
				return (self.ParameterType() == datacontext.ParameterType.SELECT) &&
				(self.ListValues() == null || self.ListValues().length === 0);
			});
			
			self.toJs = function() {
				return {
					name: self.Name(),
					defaultValue: self.DefaultValue(),
					isReadOnly: self.IsReadOnly(),
					parameterType: self.ParameterType(),
					parameterTarget: self.ParameterTarget(),
					listValues: self.ListValues()
				};
			}

			//Deep copy
			self.Clone = function () {
				var result = new RegParameter();
				result.id = self.id;
				result.Name(self.Name());
				result.DefaultValue(new datacontext.MaskedStringValue(self.DefaultValue().Value()));
				result.ServerRequired(self.ServerRequired());
				result.IsRequired(self.IsRequired());
				result.IsServerParameter(self.IsServerParameter());
				result.IsReadOnly(self.IsReadOnly());
				result.ParameterTarget(self.ParameterTarget());
				result.ParameterType(self.ParameterType());
				$.each(self.ListValues(),
					function (key, value) {
						result.ListValues.push(new datacontext.MaskedStringValue(value.Value()));
					});
				return result;
			}
		}
	})();
	
})(ko, window.batchtypeSettings.datacontext);