﻿/**
* ViewModel регпараметра
* @param {} model модель данных регпараметра
* @validateName<function> функция валидации имени регпараметра
* @createRegValue<function> функция, возвращающая новое значение регпараметра
* @needInitialValidation<bool> - нужна ли исходная валидация свойств параметра
*/

function RegParameterViewmodel(model, validateName, createRegValue, needInitialValidation, onEditHandler, regParamTypeChecker, dropDownEditor) {
	var self = this;

	self.id = RegParameterViewmodel.Counter++;
	self.model = model;

	/** 
	 * Редактор имени регпараметра
	*/
	self.nameEditor = new InlineTextEditor(
		self.model.Name, null, self.model.IsServerParameter, 90,
		(function (thisModelId) {
			return function (newName) {
				return validateName(newName, thisModelId);
			};
		})(self.model.id)
		);

	/** 
	 * Селектор типа регпараметра
	*/
	self.typeSelector = regParamTypeChecker;
	self.typeSelectorId = 'typeSelector' + self.id;
	self.valueSelectorId = 'valueSelector' + self.id;

	self.toggleType = function (data,event) {
		if (self.typeSelector.domId === self.typeSelectorId) {
			self.typeSelector.toggle();
		} else {
			self.typeSelector.close();
			self.typeSelector.domId = self.typeSelectorId;
			self.typeSelector.model(self.model);
			self.typeSelector.open(data, event);
		}
		

	}

	/** 
	 * Редактор дефолтного текста текстового регпараметра
	*/
	self.defaultValueEditor = new InlineTextEditor(
		self.model.DefaultValue().Value,
		FC.Environment.Localization.Batches.RegParam_NoTextValue,
		function () { return false },
		255,
		function (newName) {
			if (self.model.IsReadOnly() && !newName) {
				return { valid: false, errorMessage: FC.Environment.Localization.Batches.LockedRegParameterWarning };
			}

			return { valid: true, errorMessage: null };
		}, self.model.DefaultValue);

	self.isReadOnlySubscriber = self.model.IsReadOnly.subscribe(function (newValue) {
		if (newValue && !self.defaultValueEditor.text()) {
			self.defaultValueEditor.startEditing();
		} else {
			self.defaultValueEditor.validateSelf();
		}
	});

	self.defaultValueSubscriber = self.model.DefaultValue.subscribe(function (newValue) {

		self.defaultValueEditor.text = newValue.Value;
		
	});


	self.valuesEditor = new SelectRegParamsValueEditor(self.model.ListValues,
		self.model.DefaultValue,
		dropDownEditor,
		self.valueSelectorId,
		createRegValue,
		onEditHandler);


	self.parameterTypeSubscriber = self.model.ParameterType.subscribe(function (newValue) {
		if (newValue == window.batchtypeSettings.datacontext.ParameterType.SELECT &&
			(!self.model.ListValues() || !self.model.ListValues().length)) {

			// чтобы успел отрисоваться DOM-элемент с формой редактирования dropDown параметра
			self.valuesEditor.dropDownEditor.model(self.valuesEditor);
			
			var interval = setInterval(function () {
				var controlElem = $('#' + self.valueSelectorId + 'add')[0];
				if (controlElem) {
					self.valuesEditor.addDropDownValue(null,{ target: controlElem });
					clearInterval(interval);
				}
			}, 100);
		} else {
			if (self.model.IsReadOnly() && !self.defaultValueEditor.text()) {

				// чтобы успел отрисоваться DOM-элемент с формой редактирования текстового параметра
				setTimeout(self.defaultValueEditor.startEditing, 100);
			}
		}
		if (onEditHandler)
			onEditHandler();
	});

	

	if (needInitialValidation) {
		self.nameEditor.validateSelf();
		self.defaultValueEditor.validateSelf();
	}


	self.isValid = ko.computed(function () {
		var paramType = self.model.ParameterType();
		var dropDownIsEmpty = self.valuesEditor.dropDownIsEmpty();
		var nameIsValid = self.nameEditor.isValid();
		var defaultValueIsValid = self.defaultValueEditor.isValid();

		if (!nameIsValid) {
			return false;
		}

		if (paramType == window.batchtypeSettings.datacontext.ParameterType.SELECT && dropDownIsEmpty) {
			return false;
		}

		if (paramType == window.batchtypeSettings.datacontext.ParameterType.TEXT && !defaultValueIsValid) {
			return false;
		}

		return true;

	}).extend({ deferred: true });
}

RegParameterViewmodel.Counter = 0;
RegParameterViewmodel.prototype.dispose = function () {
	this.isReadOnlySubscriber.dispose();
	this.parameterTypeSubscriber.dispose();
	this.defaultValueSubscriber.dispose();
	this.isValid.dispose();
}

