﻿/**
 * ViewModel вкладки описания настроек пакета
 * @param {} batchRepParams список рег параметров пакета observable
 * @param {} datacontext ОБект работы с данными
 */
var BatchPropsViewModel = function (datacontext, batchRegParameters, userIsAdmin) {
	var self = this;
	self.userIsAdmin = userIsAdmin;
	ScrollablePanel.apply(this, ['#batch', '#batchContentFrame']);
	self.datacontext = datacontext;

	//Рег параметры пакета
	self.batchRegParameters = batchRegParameters;

	//Параметры пакета
	self.batchProperties = ko.observable();
	var sourceBatchProperties = ko.observable();

	self.batchProperties.subscribe(function (newValue) {
		if (!newValue) {
			return;
		}
		self.updatePriotitySelector();

	});

	self.priorityList = ko.observable({
		domId: ko.observable('priortyList'),
		values: ko.observableArray([])
	});

	self.updatePriotitySelector = function () {
		if (!self.batchProperties()) {
			return;
		}

		var values = ko.utils.arrayMap(self.batchProperties().availablePriorities, function (priorityName, index) {
			return {
				selected: ko.computed(function () { return self.batchProperties().priority() === priorityName; }),
				onClick: function () {
					self.batchProperties().priority(priorityName);
				},
				title: ko.observable(priorityName),
				name: ko.observable(priorityName),
				iconCss: ko.observable(null),
				nameSummary: ko.observable(null),
				hidden: ko.observable(false)
			};
		});

		self.priorityList().values(values);
	}

	//Последний построенный список рег параметров
	var batchShortRegParams = [];

	//Получения списка рег параметров со свойствами включенности в именования пакета
	self.batchSelectRegParams = ko.computed(function () {
		if (!self.batchProperties()) return [];

		//Удаление из списка и схемы именования пакета удаленных рег параметров
		//Обновление индексов в сехеме именования для других рег параметров
		for (var i = batchShortRegParams.length - 1; i >= 0; i--) {
			var found = false;
			var batchShortRegParam = batchShortRegParams[i];

			for (var j = 0; j < self.batchRegParameters().length; j++) {

				if (batchShortRegParam.id === self.batchRegParameters()[j].id) {
					found = true;
					break;
				}
			}

			if (!found) {

				if (batchShortRegParam.checked()) {
					batchShortRegParam.checked(false);
				}

				batchShortRegParams.splice(i, 1);
			}
		}

		//Добавление новых рег параметров в список
		$.each(self.batchRegParameters(),
            function (ind, bItem) {
            	ind++;
            	var found = false;

            	for (var i = 0; i < batchShortRegParams.length; i++) {
            		if (batchShortRegParams[i].id === bItem.id) {
            			batchShortRegParams[i].index(ind);
            			batchShortRegParams[i].name(bItem.Name());
            			found = true;
            			break;
            		}
            	}

            	if (!found) {
            		batchShortRegParams.push(new BatchShortRp(ind, bItem, self.batchProperties().namingScheme().Value));
            	}
            });

		//Сортировка рег параметров по индексу
		batchShortRegParams.sort(function (a, b) {
			var diff = a.index() - b.index();
			if (diff < 0) {
				return -1;
			}
			if (diff > 0) {
				return 1;
			}
			return 0;
		});

		return batchShortRegParams;
	});

	self.addTemplate = function (method) {
		var field = $('#NamingSchemeBatch');
		var caretPosition = FC.Utils.CaretTools.doGetCaretPosition(field[0]);
		var caretDiff = self.batchProperties().namingScheme()[method](caretPosition).length;
		field.focus();
		FC.Utils.CaretTools.setCaretPosition(field[0], caretPosition + caretDiff);
	}

	//Обновление данных из плагина
	self.updateInfo = function (batchType) {
		if (batchShortRegParams.length > 0)
			batchShortRegParams.splice(0, batchShortRegParams.length);
		self.datacontext.getBatchProperties(batchType, self.batchProperties);
		self.datacontext.getBatchProperties(batchType, sourceBatchProperties);
		
	};

	//Сохранение данных
	self.saveInfo = function (batchType) {
		var batchProperties = self.batchProperties();
		batchProperties.useSpec(true);
		if (batchProperties) {
			self.datacontext.setBatchProperties(batchType, batchProperties);
		}
	}


	self.checkChanges = function () {
		if (!self.batchProperties() || !sourceBatchProperties()) {
			return false;
		}
		var settingsSerialized = JSON.stringify(self.batchProperties().toJs());
		var sourceSettingsSerialized = JSON.stringify(sourceBatchProperties().toJs());
		return settingsSerialized !== sourceSettingsSerialized;
	}
}



BatchPropsViewModel.prototype = Object.create(ScrollablePanel.prototype);

BatchPropsViewModel.prototype.constructor = BatchPropsViewModel;


/**
 * Класс короткого описания рег параметров дополненные сведениями об индексе рег параметра 
 * и свойством включения рег параметра в схему именования
 * @param {} index Индекс рег параметра
 * @param {} batchRegParam Рег параметр
 * @param {} batchName схема именования типа пакета 
 * (observable для добавления удаления метки включения рег параметра в схему именования)
 */
function BatchShortRp(index, batchRegParam, batchName) {
	var self = this;
	//Предыдущее значение индекса (для исправления индекса в схеме именования при обновлении индекса рег параметра)
	//ИНдекс меняется при удалении и добавлении рег параметров
	var oldIndex = index;

	//Текущий индекс рег параметра
	self.index = ko.observable(index);

	//Id рег параметра
	self.id = batchRegParam.id;

	//Название рег параметра
	self.name = ko.observable(batchRegParam.Name());

	//Схема именования рег пакета
	self.batchName = batchName;

	//Индикатор включения рег параметра в схему именования пакета
	self.checked = ko.observable(batchName() ? batchName().indexOf('<R:' + self.index() + '>') >= 0 : false);

	//При изменении индекса меняется маска в схеме именования пакета
	self.index.subscribe(function (val) {
		batchName(batchName().replace('<R:' + oldIndex + '>', '<R:' + val + '>'));
		self.oldIndex = val;
	});

	//При включении/исключении рег параметра в схему именования пакета, 
	//добавляется/удаляется маска вида <R:индексрег параметра>
	self.checked.subscribe(function (value) {
		if (value) {
			batchName(batchName() + '<R:' + self.index() + '>');
		} else {
			var pattern = '<R:' + self.index() + '>';
			var oldValue;

			do {
				oldValue = batchName();
				batchName(batchName().replace(pattern, ''));
			} while (oldValue !== batchName());
		}
	});

	//добавление маски вида <R:индекс рег параметра>
	self.addParamToScheme = function () {
		var field = $('#NamingSchemeBatch');
		var caretPosition = FC.Utils.CaretTools.doGetCaretPosition(field[0]);
		var pattern = '<R:' + self.index() + '>';
		batchName([batchName().slice(0, caretPosition), pattern, batchName().slice(caretPosition)].join(''));
		field.focus();
		FC.Utils.CaretTools.setCaretPosition(field[0], caretPosition + pattern.length);
		return true;
	}
}