﻿window.batchesApp = window.batchesApp || {};


ko.observableArray.fn.subscribeArrayChanged = function (addCallback, deleteCallback) {
	var previousValue = undefined;
	this.subscribe(function (_previousValue) {
		previousValue = _previousValue.slice(0);
	}, undefined, 'beforeChange');
	this.subscribe(function (latestValue) {
		var editScript = ko.utils.compareArrays(previousValue, latestValue);
		for (var i = 0, j = editScript.length; i < j; i++) {
			switch (editScript[i].status) {
				case "retained":
					break;
				case "deleted":
					if (deleteCallback)
						deleteCallback(editScript[i].value);
					break;
				case "added":
					if (addCallback)
						addCallback(editScript[i].value);
					break;
			}
		}
		previousValue = undefined;
	});
};

/**
 *  ViewModel для страницы выбора пакета
 */
window.batchesApp.batchesAppViewModel = (function (ko, datacontext) {

	/**
     * Table settings
     */
	FC.Utils.setDataTablesDefaultsSettings();

	var buttonsMinimizeFunc = function () {
		setTimeout(function () {
			var funcsContainerWidth = $('.common__generalfunction').width();

			funcsContainerWidth -= $('.batchtype__selector__field').outerWidth() + 40;
			funcsContainerWidth -= $('.batches__listscontrols').outerWidth() + 40;


			var $minimizePart = $('.common__nonContext-operations ');
			var maximizeThreshold = $minimizePart.attr('data-maximize');

			if (maximizeThreshold < funcsContainerWidth) {
				$minimizePart.removeAttr('data-maximize');
				$minimizePart.removeClass('common__nonContext-operations--minimized');
				buttonsMinimizeFunc();
			} else {
				if ($minimizePart.outerWidth() > funcsContainerWidth) {
					$minimizePart.attr('data-maximize', funcsContainerWidth + 1);
					$minimizePart.addClass('common__nonContext-operations--minimized');
				}
			}
		}, 100);
	}

	var selectedRowsNum = ko.observable(0);

	var selectionState = ko.computed(function () {

		return FC.Environment.Localization.Batches.SelectedBatchesState.replace('{0}', selectedRowsNum());
	});


	var changeRowsSelector = function (e, dt, type, indexes) {

		var selectedCount = table.rows(".selected").data().count();
		var totalCount = table.rows().data().count();

		var jSelector = $("#rowSelector");
		jSelector.prop("checked", selectedCount > 0);
		if (selectedCount > 0 && selectedCount < totalCount) {
			jSelector.addClass("indeterminate")
		} else {
			jSelector.removeClass("indeterminate");
		}

		selectedRowsNum(selectedCount);

	}

	var toggleRows = function (data, event) {
		var jthis = $(event.target);
		jthis.removeClass("indeterminate");

		var checkboxes = $(table.select.selector(), '#batchesList tbody');

		if (jthis.is(':checked')) {
			checkboxes.prop('checked', true);
			table.rows().select();
		} else {
			checkboxes.prop('checked', false);
			table.rows().deselect();
		}

		return true;
	}


	var table = $("#batchesList")
        .DataTable({
        	processing: false,
        	lengthMenu: [[10, 50, 100], [10, 50, 100]],
        	pageLength: 10,
        	paging: false,
        	searching: true,
        	/*columnDefs: [
                {
                    
                    targets: "_all"
                }
            ],*/
        	columns: [
                {
                	width: "20px",
                	className: "dt-left select-checkbox",
                	defaultContent: "",
                	sortable: false
                },
                {
                	data: 'name()',
                	className: "dt-left",
                	render: function (data, type, full, meta) {
                		if (type !== "display") {
                			return data;
                		}
                		return '<a class="openBatchElement ' +
                            (full.statusCode() === 2 ? 'blocked' : '') +
                            '" data-batchId="' +
                            full.id() +
                            '" data-rescanId="' +
                            full.rescanId +
                            '">' +
                            data +
                            '</a>';
                	}
                },
                {
                	data: 'batchType()',
                	className: "dt-left",
                },
                {
                	data: 'priorityCode()',
                	className: "dt-left",
                	render: function (data, type, full, meta) {
                		if (type !== "display") {
                			return data;
                		}
                		var priorityClass = '';
                		if (data < 2) {
                			priorityClass = "green";
                		} else if (data > 2) {
                			priorityClass = "red";
                		}
                		return '<span class="' + priorityClass + '">' + full.priority() + '</span>';
                	}
                },
                {
                	data: 'statusCode()',
                	className: "dt-right",
                	width: '32px',
                	render: function (data, type, full, meta) {
                		if (data == 3) {
                			return '<a class="modalDialog__button batches__stopsending" data-batchId="' + full.id() + '">' +
								FC.Environment.Localization.Batches.StopSendingButton + '</a>';
                		}
                		return '';
                	}
                },
                {
                	data: 'status()',
                	className: "dt-left",
                },
                {
                	data: 'creationDate()',
                	className: "dt-left",
                	render: function (data, type, full, meta) {
                		if (type !== "display") {
                			return data;
                		}
                		return full.creationDateString();
                	}
                },
                {
                	data: 'docsCount()',
                	className: "dt-left",
                },
                {
                	data: 'pagesCount()',
                	className: "dt-left",
                }
        	],
        	colResize: {
        		exclude: [0]
        	},
        	drawCallback: function (settings) {

        		$('#batchesList').off('click', '.openBatchElement');
				$('#batchesList').on('click', '.openBatchElement', function (e) {
					if ($(this).hasClass('blocked')) return true;
        			if (e.which == 3) return true;
        			openBatchMethod(parseInt(this.dataset.batchid), parseInt(this.dataset.rescanid));
        		});
        		$('#batchesList').off('click', '.batches__stopsending');
        		$('#batchesList').on('click', '.batches__stopsending', function (e) {
        			if (e.which == 3) return true;
        			datacontext.cancelSendingBatch(this.dataset.batchid, reloadAllInfo);
        		});
        	},
        	order: [[1, 'asc']],
        	language: {
        		"emptyTable": FC.Environment.Localization.Batches.MsgNoBatches,
        	}

        });

	table.on('select', changeRowsSelector).on('deselect', changeRowsSelector).on('draw', changeRowsSelector);


	$("#batchesList tbody").on('click', 'td.select-checkbox', function () {
		$(this).parents("tr").eq(0).toggleClass('selected');
		changeRowsSelector();
	});



	var BatchInfo = (function () {
		var nextId = 1;

		return function BatchInfo(initInfo) {
			this.internalId = nextId++;
			this.id = ko.observable(initInfo.id);
			this.batchType = ko.observable(initInfo.batchType || '');
			this.rescanId = initInfo.rescanId;
			this.name = ko.observable(initInfo.name);
			this.priority = ko.observable(initInfo.priority);
			this.status = ko.observable(initInfo.status);
			this.creationDateString = ko.observable(initInfo.creationDateString);
			this.creationDate = ko.observable(initInfo.creationDate);
			this.docsCount = ko.observable(initInfo.docsCount);
			this.pagesCount = ko.observable(initInfo.pagesCount);
			this.statusCode = ko.observable(initInfo.statusCode);
			this.priorityCode = ko.observable(initInfo.priorityCode);
			this.regParams = ko.observable(initInfo.regParams)
			var that = this;
			$.each(['id', 'batchType', 'name', 'priority', 'status', 'creationDate', 'creationDateString', 'docsCount', 'pagesCount', 'statusCode', 'priorityCode'],
                function (i, prop) {
                	that[prop].subscribe(function (val) {
                		// Find the row in the DataTable and invalidate it, which will
                		// cause DataTables to re-read the data
                		var tableData = table.data();
                		for (var i = 0; i < tableData.length; i++) {
                			if (tableData[i].internalId == that.internalId) {
                				table.row(i).invalidate();
                				return;
                			}
                		}
                	});
                });

		}
	})();


	//Список доступных типов пакета
	var availableBatchTypes = ko.observableArray([]);

	var availableBatchTypesDictionary = ko.computed(function () {
		var res = [];
		$.each(availableBatchTypes(),
            function (key, data) {
            	res[data.Id()] = data.Name();
            });

		return res;
	});

	var batchTypeOperationDisabled = ko.computed(function () {
		var newbatchTypes = availableBatchTypes();
		return !newbatchTypes || !newbatchTypes.length;
	});

	//Выбранный тип пакета
	var selectedBatchType = ko.observable();


	//Guid текущего проекта
	var projectGuid = $("#projectGuid").val();


	//Список пакетов
	var availableBatches = ko.observableArray();

	//Список задач пересканирования
	var rescanTasks = ko.observableArray();

	//Формирование списка для отображения с учетом сортировки
	var batchesToShow = ko.observableArray([]);

	//Флаг загрузки списка пакетов
	var batchesLoaded = ko.observable(false);

	//Флаг загрузки списка задач пересканирования
	var rescansLoaded = ko.observable(false);

	//Флаг загрузки списка пакетов и задач пересканирования
	var allLoaded = ko.computed(function () { return batchesLoaded() && rescansLoaded(); });

	//Количество пакетов
	var batchesCount = ko.pureComputed(function () {
		return availableBatches().length;
	});

	//Количество задач пересканирования
	var rescansCount = ko.pureComputed(function () {
		return rescanTasks().length;
	});

	//Текущий список отрисовки(список пакетов или задач пересканирования)
	var listToShow = ko.observable(localStorage.getItem('batchesListType') ? parseInt(localStorage.getItem('batchesListType')) : 0);

	availableBatchTypes.subscribe(buttonsMinimizeFunc);

	//Сохранение выбранного типа пакета в localstorage
	selectedBatchType.subscribe(function (val) {
		if ((val == null) || (val.Id() == null)) {
			return;
		}
		localStorage.setItem('selectedBatchType', val.Id());
	});

	allLoaded.subscribe(function (val) {
		if (val) {
			setTimeout(function () {
				$("#batchesList").dataTable().fnAdjustColumnSizing(false);
			},
                100);
		}
	});

	batchesToShow.subscribeArrayChanged(
		function (addedItem) {
			table.row.add(addedItem).draw();
		},
		function (deletedItem) {
			var tableData = table.data();
			for (var i = 0; i < tableData.length; i++) {
				if (tableData[i].internalId == deletedItem.internalId) {
					table.row(i).remove().draw();
					return;
				}
			}

		}
	);


	//Запуск функции с интервальным таймером
	function setIntervalAndExecute(fn, t) {
		fn();
		return (setInterval(fn, t));
	}

	//Получение списка типов пакетов
	datacontext.getBatchTypes(function (batchesType) {
		var lastBatchType = localStorage.getItem('selectedBatchType');
		if (!lastBatchType) {
			lastBatchType = -1;
		}

		availableBatchTypes(batchesType);
		
		if (batchesType == null)
			return;

		lastBatchType = parseInt(lastBatchType);
		$.each(batchesType, function (key, value) {
			if (value.Id() === lastBatchType) {
				selectedBatchType(value);
			}
		});
		if (!selectedBatchType() && batchesType.length > 0) {
			selectedBatchType(batchesType[0]);
		}
	});

	function reloadBatchInfo() {
		datacontext.getBatches(availableBatches,
            function () {
            	batchesLoaded(true);
            	if (listToShow() == 0) updateBatchesToShow();
            });
	}

	function reloadRescanInfo() {
		datacontext.getRescanTasks(rescanTasks,
            function () {
            	rescansLoaded(true);
            	if (listToShow() == 1) updateBatchesToShow();
            });
	}

	//Функция загрузки списка пакетов и задач пересканирования
	function reloadAllInfo() {
		reloadBatchInfo();
		reloadRescanInfo();
	}

	//Обновление списка пакетов и задач пересканирования
	setIntervalAndExecute(reloadBatchInfo, 1000);
	setIntervalAndExecute(reloadRescanInfo, 30000);

	//Сохранения выбора текущего списка в Localstorage
	listToShow.subscribe(function (val) {
		localStorage.setItem('batchesListType', val);
		batchesToShow([]);
		updateBatchesToShow();
	});

	function updateBatchesToShow() {
		var result = listToShow() === 0 ? availableBatches() : rescanTasks();

		$.each(result, function (key, value) {
			var batchType = availableBatchTypesDictionary()[value.BatchTypeId()];
			var isNewVersionPlugin = JSON.parse(localStorage.getItem('isNewVersionPlugin'));

			for (var i = 0; i < batchesToShow().length; i++) {
				var batch = batchesToShow()[i];
				if ((batch.id() == -1 || batch.id() == value.Id()) &&
                (batch.rescanId == -1 || batch.rescanId == value.RescanTaskID())) {
					batch.id(value.Id());
					batch.name(value.Name());
					batch.status(value.stringStatus() + (value.statusDescription().length > 0 ? '. ' : '') + value.statusDescription());
					batch.priority(value.priorityToText());
					batch.creationDateString(value.CreationDateFormat() + ' ' + value.CreationTimeTimeFormat());
					batch.creationDate(value.CreationTime());
					batch.docsCount(value.ExplicitDocumentCount());
					batch.pagesCount(value.PageCount());
					batch.statusCode(value.Status());
					batch.priorityCode(value.Priority());
					batch.batchType(batchType || '');
					batch.regParams(isNewVersionPlugin ? value.RegParams() : null);
					return;
				}
			}
			batchesToShow.push(new BatchInfo({
				id: value.Id(),
				batchType: batchType || '',
				rescanId: value.RescanTaskID(),
				name: value.Name(),
				status: value.stringStatus() + (value.statusDescription().length > 0 ? '. ' : '') + value.statusDescription(),
				priority: value.priorityToText(),
				creationDateString: value.CreationDateFormat() + ' ' + value.CreationTimeTimeFormat(),
				creationDate: value.CreationTime(),
				docsCount: value.ExplicitDocumentCount(),
				pagesCount: value.PageCount(),
				statusCode: value.Status(),
				priorityCode: value.Priority(),
				regParams: isNewVersionPlugin?  value.RegParams() : null
			}));

		});

		for (var i = batchesToShow().length - 1; i >= 0; i--) {
			var batchToShow = batchesToShow()[i];
			var found = false;
			for (var j = 0; j < result.length; j++) {
				var batch = result[j];
				if ((batchToShow.id() == -1 || batchToShow.id() == batch.Id()) &&
                (batchToShow.rescanId == -1 || batchToShow.rescanId == batch.RescanTaskID())) {
					found = true;
					break;
				}
			}
			if (!found) {
				batchesToShow.splice(i, 1);
			}
		}

	}

	//Функция выбора списка пакетов
	var showBatches = function () {
		listToShow(0);
	}

	//Функция выбора списка задач пересканирования
	var showRescans = function () {
		listToShow(1);
	}

	//Функция удаления пакета
	var deleteBatch = function (item) {
		if (confirm(FC.Environment.Localization.Batches.BatchDeletingPrompt)) { //NOSONAR we need to use this confirm
			datacontext.deleteBatch(item.id(), reloadAllInfo);
		}
	}

	//Функция отмены отправки пакета
	var cancelSending = function (item) {
		datacontext.cancelSendingBatch(item.id(), reloadAllInfo);
	}

	//Функция возобновления отправки пакета
	var renewSending = function (item) {
		datacontext.renewSendingBatch(item.id(), reloadAllInfo);
	}

	//Функция создания пакета
	var createBatch = function (batchType, callbackFunction) {
		datacontext.createBatch(batchType, function (creationResult) {
			if (creationResult.hasErrors) {
				FC.Utils.Error.Show(creationResult.errorText);
				callbackFunction(-1);
			} else {
				callbackFunction(parseInt(creationResult.result));
			}
		});
	}

	var getBatchUrl = function(batchId) {
		var url = FC.Environment.AppPath + 'Batch/Index/' + projectGuid + '/' + batchId;

		var query = location.search.toLowerCase();
		if (query && query.indexOf('mode=mini') > -1) {
			url += "?mode=mini";
		}

		return url;
	};

	var getNewBatchUrl = function (newBatchId, fromFolder) {
		var url = getBatchUrl(newBatchId);
		url += (url.indexOf('?') > 0 ? '&' : '?') + 'openMode=';
		url += fromFolder ? 'OpenFolderDialog' : 'OpenScanDialog';

		return url;
	};

	//Функция создания пакета с последующим открытием страницы с загрузкой изображений из папки
	var addFromFolder = function () {
		if (batchTypeOperationDisabled()) {
			return false;
		}

		createBatch(selectedBatchType(), function (newBatchId) {
			if (newBatchId !== -1)
				location.replace(getNewBatchUrl(newBatchId, true));
		});

	}

	//Функция создания пакета с последующим открытием страницы с загрузкой изображений со сканера
	var addFromScanner = function () {
		if (batchTypeOperationDisabled()) {
			return false;
		}

		createBatch(selectedBatchType(), function (newBatchId) {
			if (newBatchId !== -1)
				location.replace(getNewBatchUrl(newBatchId, false));
		});
	}

	//Функция открытия пакета
	function openBatchMethod(batchId, rescanId) {
		if ((listToShow() === 1) && (batchId < 0)) {
			datacontext.openRescanBatch(rescanId,
		        function (openRescanResult) {
		        	if (openRescanResult.hasErrors) {
		        		FC.Utils.Error.Show(openRescanResult.errorText);
		        	} else {
		        		setTimeout(function () { location.replace(getBatchUrl(openRescanResult.result)); },
		                    100);
		        	}
		        });
		} else {
			setTimeout(function () { location.replace(getBatchUrl(batchId)); }, 100);
		}
	}

	//функция открытия настроек типа пакета
	var openSettings = function () {
		if (batchTypeOperationDisabled()) {
			return false;
		}

		window.batchtypeSettings.viewmodel.openWindow(availableBatchTypes(), selectedBatchType());
	}

	var listSelector = ko.observable({
		domId: ko.observable('listSelector'),
		values: ko.observableArray([
            {
            	selected: ko.computed(function () { return listToShow() == 0; }),
            	onClick: function () {
            		showBatches();
            	},
            	title: ko.computed(function () {
            		return FC.Environment.Localization.Batches.BatchListFilter.replace('{0}', batchesCount());
            	}),
            	name: ko.computed(function () {
            		return FC.Environment.Localization.Batches.BatchListFilter.replace('{0}', batchesCount());
            	}),
            	iconCss: ko.observable(null),
            	nameSummary: ko.observable(null),
            	hidden: ko.observable(false)
            },
            {
            	selected: ko.computed(function () { return listToShow() == 1; }),
            	onClick: function () {
            		showRescans();
            	},
            	title: ko.computed(function () {
            		return FC.Environment.Localization.Batches.RescanTaskListFilter.replace('{0}', rescansCount());
            	}),
            	name: ko.computed(function () {
            		return FC.Environment.Localization.Batches.RescanTaskListFilter.replace('{0}', rescansCount());
            	}),
            	iconCss: ko.observable(null),
            	nameSummary: ko.observable(null),
            	hidden: ko.observable(false)
            }
		])
	});

	var deleteSelectedBatches = function () {
		var selectedData = table.rows(".selected").data();
		if (confirm(FC.Environment.Localization.Batches.BatchDeletingPrompt)) { //NOSONAR we need to use this confirm
			$.each(selectedData,
                function (key, value) {
                	datacontext.deleteBatch(value.id(), reloadAllInfo);

                });
		}
	}

	var stopSendingSelectedBatches = function () {
		var selectedData = table.rows(".selected").data();
		$.each(selectedData,
            function (key, value) {
            	cancelSending(value);
            });
	}

	var renewSendingSelectedBatches = function () {
		var selectedData = table.rows(".selected").data();
		$.each(selectedData,
            function (key, value) {
            	renewSending(value);
            });
	}
	var selectedItems = [];
	var selectAll = ko.computed({
		read: function () {
			return batchesToShow().length == selectedRowsNum();
		},
		write: function (val) {
			if (val) {
				selectedItems = [];
				var selectedData = table.rows(".selected").data();
				$.each(selectedData,
                    function (key, value) {
                    	selectedItems.push(value.internalId);
                    });
				table.rows().select();
			} else {
				table.rows().every(function (rowIdx, tableLoop, rowLoop) {
					var data = this.data();
					if (selectedItems.indexOf(data.internalId) < 0) {
						this.deselect();
					}
				});
				selectedItems = [];
			}
		}
	});

	var deselectAll = function () {
		table.rows().deselect();
	}

	var deleteSelectedAvailable = ko.computed(function () {
		if (listToShow() == 1 || selectedRowsNum() == 0)
			return false;

		var selectedData = table.rows(".selected").data();
		for (var i = 0; i < selectedData.length; i++) {
			if ((selectedData[i].statusCode() == 2) || (selectedData[i].statusCode() == 3)) {
				return false;
			}
		}

		return true;
	});
	var stopSendingSelectedAvailable = ko.computed(function () {
		if (selectedRowsNum() == 0)
			return false;

		var selectedData = table.rows(".selected").data();
		for (var i = 0; i < selectedData.length; i++) {
			if (selectedData[i].statusCode() != 3 && selectedData[i].statusCode() != 2) {
				return false;
			}
		}

		return true;
	});

	var renewSendingSelectedAvailable = ko.computed(function () {
		if (selectedRowsNum() == 0)
			return false;

		var selectedData = table.rows(".selected").data();
		for (var i = 0; i < selectedData.length; i++) {
			if (selectedData[i].statusCode() > 1) {
				return false;
			}
		}

		return true;
	});

	$(document)
        .ready(function () {
        	var tableAdjustFnc = function () {

        		var table = $.fn.dataTable.fnTables(true);
        		if (table.length > 0) {
        			$(table).dataTable().fnAdjustColumnSizing(false);
        		}
        	}




        	$(window).resize(tableAdjustFnc);

        	$(window).resize(buttonsMinimizeFunc);
        });

	return {
		availableBatchTypes: availableBatchTypes,
		selectedBatchType: selectedBatchType,
		batchTypeOperationDisabled: batchTypeOperationDisabled,
		batchesToShow: batchesToShow,
		showBatches: showBatches,
		showRescans: showRescans,
		listToShow: listToShow,
		deleteBatch: deleteBatch,
		cancelSending: cancelSending,
		renewSending: renewSending,
		addFromFolder: addFromFolder,
		addFromScanner: addFromScanner,
		batchesCount: batchesCount,
		rescansCount: rescansCount,
		openSettings: openSettings,
		allLoaded: allLoaded,
		listSelector: listSelector,
		toggleRows: toggleRows,
		selectedRowsNum: selectedRowsNum,
		deleteSelectedBatches: deleteSelectedBatches,
		deleteSelectedAvailable: deleteSelectedAvailable,
		stopSendingSelectedBatches: stopSendingSelectedBatches,
		stopSendingSelectedAvailable: stopSendingSelectedAvailable,
		renewSendingSelectedBatches: renewSendingSelectedBatches,
		renewSendingSelectedAvailable: renewSendingSelectedAvailable,
		selectionState: selectionState,
		deselectAll: deselectAll,
		selectAll: selectAll
	};

})(ko, window.batchesApp.datacontext);


ko.applyBindings(window.batchesApp.batchesAppViewModel, document.getElementById("BatchesAppView"));

