﻿window.batchtypeSettings = window.batchtypeSettings || {};

/**
 * ViewModel окна настроек типа пакета
 */
window.batchtypeSettings.viewmodel = (function (ko, datacontext, projectGuid) {

	var availableTabs = {
		SCANNING: 1,
		BATCH: 2,
		DOCUMENT: 3,
		PARAMETERS: 4,
		UPLOADING: 5
	};


	var mainStyle = ko.observable({
		opacity: 0
	});

	var userIsAdmin = ko.observable(false);
	var isVisible = ko.observable(false);
	var confirmOpened = false;
	//Список доступных типов пакета
	var availableBatchTypes = ko.observableArray([]);

	var availableScanners = ko.observableArray(null);


	var selectedBatchType = ko.observable(null);


	// выбранный тип пакета
	var batchType = ko.observable(null);

	var currentTab = ko.observable();
	
	var batchTypeDescrViewModel = new BatchTypeDescrViewModel(datacontext, userIsAdmin);

	//ViewModel настройки сканирования
	var scannerOptionsViewModel = new ScannerOptsViewModel(datacontext, availableScanners, userIsAdmin);

	//Настройки документов пакета
	var documentPropsViewModel = new DocumentPropertiesViewmodel(datacontext, userIsAdmin);
	
	//Настройки фоновой отправки
	var backgroundSettingsViewmodel = new BackgrounSettingsViewmodel(datacontext, userIsAdmin);

	//ViewModel рег параметров 
	var regParamsViewModel = new RegParamsViewModel(datacontext, userIsAdmin);

	//ViewModel настроек пакета
	var batchPropsViewModel = new BatchPropsViewModel(datacontext, regParamsViewModel.batchRegParameters, userIsAdmin);

	var lockerVisible = ko.observable(false);

	var disableMainAction = ko.computed(function() {
		return regParamsViewModel.invalidParameters() ||
			!backgroundSettingsViewmodel.backgroundUploadProps() ||
			backgroundSettingsViewmodel.backgroundUploadProps().hasUploadTimeError();
	});

	batchType.subscribe(function () {
		regParamsViewModel.updateInfo(batchType());
		batchPropsViewModel.updateInfo(batchType());
		scannerOptionsViewModel.updateInfo(batchType());
		batchTypeDescrViewModel.updateInfo(batchType());
		documentPropsViewModel.updateInfo(batchType());
		backgroundSettingsViewmodel.updateInfo(batchType());
		
		
	});

	var updateCurentTabHeight = function () {
		switch (currentTab()) {
		case availableTabs.SCANNING:
			scannerOptionsViewModel.updateHeights();
			break;
		case availableTabs.BATCH:
			batchPropsViewModel.updateHeights();
			break;
		case availableTabs.DOCUMENT:
			documentPropsViewModel.updateHeights();
			break;
		case availableTabs.PARAMETERS:
			regParamsViewModel.updateHeights();
			break;
		case availableTabs.UPLOADING:
			backgroundSettingsViewmodel.updateHeights();
			break;
		}
	}

	//Метод откытия окна настроек типа пакета
	var openWindow = function (allBatchTypes, selectedBT) {

		if (!availableScanners() || availableScanners().length === 0) {
			datacontext.getAvailableScannerOptions(availableScanners);
		}

		availableBatchTypes(allBatchTypes);
		batchType(selectedBT);
		selectedBatchType(selectedBT);

		if (!currentTab()) {
			selectTab(availableTabs.SCANNING);
		}

		isVisible(true);
		setTimeout(function () {
			mainStyle({
				opacity: 1
			});
		}, 100);

		setTimeout(updateCurentTabHeight, 100);
	}

	var closeWindow = function() {
		mainStyle({
			opacity: 0
		});
		isVisible(false);
	}


	var selectTab = function (tabId) {
		currentTab(tabId);
		setTimeout(updateCurentTabHeight, 100);
	}

	var confirm = function() {
		

		lockerVisible(true);
		regParamsViewModel.saveInfo(batchType());
		batchPropsViewModel.saveInfo(batchType());
		scannerOptionsViewModel.saveInfo(batchType());
		documentPropsViewModel.saveInfo(batchType());
		backgroundSettingsViewmodel.saveInfo(batchType());

		setTimeout(function () {
			lockerVisible(false);
			closeWindow();
		}, 7000);
	}

	var confirmWindowProps;
	selectedBatchType.subscribe(function (val) {
		if (val) {
			if (batchType() && val != batchType()) {
				var hasChanges = regParamsViewModel.checkChanges() ||
					batchPropsViewModel.checkChanges() ||
					scannerOptionsViewModel.checkChanges() ||
					documentPropsViewModel.checkChanges() ||
					backgroundSettingsViewmodel.checkChanges();
				if (hasChanges) {
					confirmOpened = true;
					confirmWindowProps = FC.Utils.Notification.ConfirmWindowAdvanced(FC.Environment.Localization.Batches.BatchSettingsConfirmWindowHeader,
						FC.Environment.Localization.Batches.BatchSettingsConfirmWindowSubHeader,
						[
							{
								buttonText: FC.Environment.Localization.Batches.BatchSettingsConfirmWindowYes,
								action: function () {
									lockerVisible(true);
									confirmOpened = false;
									regParamsViewModel.saveInfo(batchType());
									batchPropsViewModel.saveInfo(batchType());
									scannerOptionsViewModel.saveInfo(batchType());
									documentPropsViewModel.saveInfo(batchType());
									backgroundSettingsViewmodel.saveInfo(batchType());

									setTimeout(function() {
											lockerVisible(false);
											batchType(val);
										},
										7000);
								}
							},
							{
								buttonText: FC.Environment.Localization.Batches.BatchSettingsConfirmWindowNo,
								action: function() {
									batchType(val);
									confirmOpened = false;
								}
							},
							{
								buttonText: FC.Environment.Localization.Batches.BatchSettingsConfirmWindowCancel,
								action: function () { selectedBatchType(batchType()); confirmOpened = false; }
							}
						]);
				} else {
					batchType(val);
				}
			} 
		}
	});


	datacontext.getUserCanChangeSettings(projectGuid, userIsAdmin);

	hotkeys('*',
		function (e) {
			if (hotkeys.isPressed(27)) {
				if (isVisible() && !confirmOpened && !lockerVisible()) {
					closeWindow();
				} else if (confirmOpened && !lockerVisible()) {
					selectedBatchType(batchType());
					confirmOpened = false;
					confirmWindowProps.close();
				}
			}


		});
	return {
		mainStyle: mainStyle,
		isVisible: isVisible,
		disableMainAction: disableMainAction,
		openWindow: openWindow,
		closeWindow: closeWindow,
		confirm: confirm,
		selectTab: selectTab,
		availableTabs: availableTabs,
		currentTab: currentTab,
		scannerOptionsViewModel: scannerOptionsViewModel,
		batchPropsViewModel: batchPropsViewModel,
		documentPropsViewModel: documentPropsViewModel,
		regParamsViewModel: regParamsViewModel,
		backgroundSettingsViewmodel: backgroundSettingsViewmodel,
		batchTypeDescrViewModel: batchTypeDescrViewModel,
		lockerVisible: lockerVisible,
		availableBatchTypes: availableBatchTypes,
		selectedBatchType: selectedBatchType,
		userIsAdmin: userIsAdmin

	};
})(ko, window.batchtypeSettings.datacontext, $("#projectGuid").val());

ko.applyBindings(window.batchtypeSettings.viewmodel, document.getElementById("propertiesWindow"));

