﻿
/**
 * Инлайн редактор текст
 * textObservable - текст, который будет редктироваться
 * placeholder - информация, которая будет написана, если текст textObservable пустой
 * isDisabledTextObservable - можно ли редактировать текст
 * doTextValidation - функция, проверяющая валидоность введенного текста
*/
function InlineTextEditor(textObservable, placeholder, isDisabledTextObservable, maxLength, doTextValidation, patternObject) {
	var self = this;

	/** 
 * Свойства меотды редактирования имени рег параметра
*/
	self.htmlId = 'InlineTextEditor' + InlineTextEditor.Id++;
	self.text = textObservable;
	self.placeholder = placeholder;
	self.isDisabled = isDisabledTextObservable;

	self.editingText = ko.observable();
	self.editingText(textObservable());
	self.maxLength = maxLength || 255;

	self.isValid = ko.observable(true);
	self.errorMessage = ko.observable('');

	self.editingModeOn = ko.observable(false);
	self.hasFocus = ko.observable(false);

	self.validate = function (text) {
		if (doTextValidation) {
			var result = doTextValidation(text);
			self.isValid(result.valid);
			self.errorMessage(result.errorMessage || '');
			return result.valid;
		}

		self.isValid(true);
		self.errorMessage('');
		return true;
	};

	self.validateSelf = function () {
		return self.validate(self.text());
	};

	self.startEditing = function () {
		if (!self.isDisabled()) {
			self.editingText(self.text());
			self.editingModeOn(true);
			self.hasFocus(true);
		}
		return true;
	};

	self.isValid.subscribe(function (val) {
		if (!val && !self.editingModeOn()) {
			self.startEditing();
		}
	});


	self.finishEditing = function (applyChanges) {
		var newText = applyChanges ? self.editingText() : self.text();
		if (applyChanges) {

			// если текст невалидный, то поле остается в режиме редактирование
			if (self.validate(newText)) {
				self.editingModeOn(false);
				self.text(newText);
			}

			// при этом фокус с поля всегда снимается
			self.hasFocus(false);
		}

		return true;
	};

	self.addTemplate = function (method) {
		if (patternObject) {
			var cloned = patternObject().Clone();
			cloned.Value(self.editingText());

			var field = $('#' + self.htmlId);
			var caretPosition = FC.Utils.CaretTools.doGetCaretPosition(field[0]);
			var caretDiff = cloned[method](caretPosition).length;
			self.editingText(cloned.Value());
			self.text(cloned.Value());
			self.hasFocus(true);
			FC.Utils.CaretTools.setCaretPosition(field[0], caretPosition + caretDiff);
		}
	};

	self.onBlur = function (data, event) {
		var relatedTarget = $(event.relatedTarget);
		if (!relatedTarget.hasClass('button--sm')) {
			self.finishEditing(true);
		}
		return true;
	};

	self.onKeyDown = function (data, event) {
		var keyCode = event.keyCode || event.which;

		// нажаты клавиши enter, tab
		if ([13, 9].indexOf(keyCode) !== -1) {

			self.finishEditing(true);
			return true;
		}

		// нажата клавиша esc - сделанные изменения не сохранятся
		if (keyCode === 27) {
			self.finishEditing(false);
			return true;
		}

		return true;
	};
}

InlineTextEditor.Id = 0;

