﻿/**
 * ViewModel вкладки настройки рег параметров
 * @param {} datacontext ОБект работы с данными
  */
var RegParamsViewModel = function (datacontext, userIsAdmin) {

	var self = this;
	self.userIsAdmin = userIsAdmin;
	ScrollablePanel.apply(this, ['#parameters', '#parametersContentFrame']);
	self.datacontext = datacontext;

	var sourceForbidOperator = ko.observable(false);
	self.regParamTypeChecker = new RegTypePopupSelector(datacontext.ParameterType, self.offsetY);
	self.regParamDropdownEditor = new RegDropdownValueEditor(self.offsetY);
	//Рег параметры пакета
	self.batchRegParameters = ko.observableArray([]);

	var sourceBatchRegParameters = ko.observableArray([]);
	//Рег параметры документов
	self.docsRegParameters = ko.observableArray([]);

	var sourceDocsRegParameters = ko.observableArray([]);

	//Тип списка рег парамтеров для отображения
	self.selectedRegParamsList = ko.observable(self.datacontext.ParameterTarget.BATCH);

	//Индекс выделенного рег параметра
	self.selectedRegParameterIndex = ko.observable(-1);
	
	//Enum достуных действий с рег параметрыми
	var availableRegParamsActions = {
		ADD: 1,
		EDIT: 2,
		DELETE: 3
	};

	self.batchRegParametersViewModel = ko.observableArray([]);
	self.docsRegParametersViewModel = ko.observableArray([]);

	self.regParamsToShow = ko.computed(function() {
		var batchParams = self.batchRegParametersViewModel();
		var docParams = self.docsRegParametersViewModel();
		return self.selectedRegParamsList() === self.datacontext.ParameterTarget.BATCH ? batchParams : docParams;
	});

	self.currentRegParamsListIsEmpty = ko.computed(function () {

		if (self.selectedRegParamsList() == datacontext.ParameterTarget.DOCUMENT) {
			return !self.docsRegParametersViewModel() || !self.docsRegParametersViewModel().length;
		}

		return !self.batchRegParametersViewModel() || !self.batchRegParametersViewModel().length;
		
	});

	self.getCurrentRegParamsViewModel = function () {
		return self.selectedRegParamsList() == datacontext.ParameterTarget.DOCUMENT ?
	   self.docsRegParametersViewModel : self.batchRegParametersViewModel;
	}


	self.invalidBatchRegParams = ko.computed(function () {
		var regParameters = self.batchRegParametersViewModel();

		if (!regParameters || !regParameters.length) {
			return false;
		}

		var invalidCount = ko.utils.arrayFilter(regParameters, function (regParameter) {
			return !regParameter.isValid();
		}).length;

		return invalidCount > 0;
	});

	self.invalidDocsRegParams = ko.computed(function () {
		var regParameters = self.docsRegParametersViewModel();

		if (!regParameters || !regParameters.length) {
			return false;
		}

		var invalidCount = ko.utils.arrayFilter(regParameters, function (regParameter) {
			return !regParameter.isValid();
		}).length;

		return invalidCount > 0;
	});


	self.invalidParameters = ko.computed(function () {
		return self.invalidBatchRegParams() || self.invalidDocsRegParams();
	});

	self.selectedRegParamsList.subscribe(function () {
		self.selectedRegParameterIndex(-1);
	});

	self.mapRegParametersModels = function () {
		var regParams = self.getCurrentRegParamsViewModel();
		return ko.utils.arrayMap(regParams(), function (param) {
			return param.model;
		});
	};

	function createRegValue() {
		return new datacontext.MaskedStringValue('');
	}

	function validateRegName(newName, regParamId) {
		var allRegParams = self.mapRegParametersModels();

		if (!allRegParams) {
			return { valid: true, errorMessage: '' };
		}

		if (newName.length === 0) {
			return { valid: false, errorMessage: FC.Environment.Localization.Batches.RegParamNameErrorEmpty };
		}

		for (var i = 0; i < allRegParams.length; i++) {

			if (allRegParams[i].id === regParamId) {
				continue;
			}

			if (allRegParams[i].Name() === newName) {
				return { valid: false, errorMessage: FC.Environment.Localization.Batches.RegParamNameErrorExist };
			}
		}

		return { valid: true, errorMessage: '' };
	}


	//Выделенный рег параметр
	self.selectedRegParameter = ko.computed(function () {
		if ((self.selectedRegParameterIndex() == undefined) || (self.selectedRegParameterIndex() < 0))
			return undefined;

		var regParams = self.getCurrentRegParamsViewModel();
		return regParams()[self.selectedRegParameterIndex()];
	});


	self.updateRegParamsModel = function (action, data) {
		if (!action || !data) {
			return;
		}

		var regParam = self.selectedRegParamsList() == datacontext.ParameterTarget.DOCUMENT ?
			self.docsRegParameters : self.batchRegParameters;
		switch (action) {
			case availableRegParamsActions.ADD:
				regParam.push(data);
				break;

			case availableRegParamsActions.DELETE:
				regParam.remove(data);
		}
	}

	//Метод добавления рег параметра прямо в таблице
	self.createParameter = function () {

		var newRegParameter = new datacontext.RegParameter();

		self.updateRegParamsModel(availableRegParamsActions.ADD, newRegParameter);

		var regParamsObservable = self.getCurrentRegParamsViewModel();

		regParamsObservable.push(new RegParameterViewmodel(newRegParameter, validateRegName, createRegValue, false, self.updateHeights, self.regParamTypeChecker, self.regParamDropdownEditor));

		
		
		self.updateHeights(function () {
			self.verticalSlider.end();
			regParamsObservable()[regParamsObservable().length - 1].nameEditor.startEditing();	
		});
		
		return true;

	}

	//Метод удаления рег параметра
	self.deleteParameter = function () {
		var selectedRegParameter = self.selectedRegParameter();

		if (!selectedRegParameter) {
			return;
		}

		FC_dialogs.showConfirmDialog(
			{
				environment: FC.Environment
			},
			document.getElementById('reactBTSDialogHost'),
			FC.Environment.Localization.Batches.DeleteBatchRegParameterPrompt,
			function () {
				self.updateRegParamsModel(availableRegParamsActions.DELETE, selectedRegParameter.model);

				selectedRegParameter.dispose();

				var regParamsObservable = self.getCurrentRegParamsViewModel();

				regParamsObservable.remove(selectedRegParameter);
				self.updateHeights();
			}
		);
	}

	//Выделение рег параметра
	self.selectRegParam = function (index) {
		self.selectedRegParameterIndex(index);
	}

	//Выбор списка рег параметров документов
	self.selectDocsRegParams = function () {
		self.selectedRegParamsList(datacontext.ParameterTarget.DOCUMENT);
		self.updateHeights();
	}

	//Выбор списка рег параметров пакета
	self.selectBatchRegParams = function () {
		self.selectedRegParamsList(datacontext.ParameterTarget.BATCH);
		self.updateHeights();
	}


	self.updateViewModel = function (paramsViewModel, regParameters) {
		if (paramsViewModel() && paramsViewModel().length) {
			ko.utils.arrayForEach(paramsViewModel(), function (item) {
				item.dispose();
			});

			paramsViewModel.removeAll();
		}
		paramsViewModel(ko.utils.arrayMap(regParameters, function (paramModel) {
			return new RegParameterViewmodel(paramModel, validateRegName, createRegValue, true, self.updateHeights, self.regParamTypeChecker, self.regParamDropdownEditor);
		}));

		self.updateHeights();
	}

	var lastLoadedBatchType = null;

	//Обновление данных из плагина
	self.updateInfo = function (batchType) {
		lastLoadedBatchType = batchType;
		self.datacontext.getAllRegParameters(batchType, self.datacontext.ParameterTarget.BATCH,
			function (data) {
				sourceBatchRegParameters(data.map(function(val) { return val.Clone(); }));
				self.batchRegParameters(data);
			}, function () {
			self.updateViewModel(self.batchRegParametersViewModel, self.batchRegParameters());
		});
		self.datacontext.getAllRegParameters(batchType,
			self.datacontext.ParameterTarget.DOCUMENT,
			function(data) {
				self.docsRegParameters(data);
				sourceDocsRegParameters(data.map(function(val) { return val.Clone(); }));
			},
			function() {
				self.updateViewModel(self.docsRegParametersViewModel, self.docsRegParameters());
			});
		
		self.datacontext.getRegParamsForbid(batchType, sourceForbidOperator);

		self.selectedRegParamsList(self.datacontext.ParameterTarget.BATCH);

	};

	//Сохранение данных
	self.saveInfo = function (batchType) {
		if (self.batchRegParameters()) {
			self.datacontext.saveRegParameters(batchType,
				self.batchRegParameters(),
				self.datacontext.ParameterTarget.BATCH);
		}

		if (self.docsRegParameters()) {
			self.datacontext.saveRegParameters(batchType,
				self.docsRegParameters(),
				self.datacontext.ParameterTarget.DOCUMENT);
		}
	}

	self.returnToDefaults = function() {
		self.docsRegParameters([]);
		self.updateViewModel(self.docsRegParametersViewModel, []);
		self.datacontext.loadServerRegParams(lastLoadedBatchType, function (serverRegParams) {
			self.batchRegParameters(serverRegParams);
			self.updateViewModel(self.batchRegParametersViewModel, serverRegParams);
		});
	}


	self.availableLists = datacontext.ParameterTarget;


	var chechChangesInParamsArray = function(source, current) {
		if (!source || !current) return false;

		if (source.length !== current.length) return true;

		for (var i = 0; i < source.length; i++) {
			if (JSON.stringify(source[i].toJs()) !== JSON.stringify(current[i].toJs())) return true;
		}
		return false;
	}

	self.checkChanges = function () {
		if (!chechChangesInParamsArray(sourceBatchRegParameters(), self.batchRegParameters())) {
			return chechChangesInParamsArray(sourceDocsRegParameters(), self.docsRegParameters());
		}
		return true;
	}
	
}


RegParamsViewModel.prototype = Object.create(ScrollablePanel.prototype);

RegParamsViewModel.prototype.constructor = RegParamsViewModel;
