﻿

function SelectRegParamsValueEditor(values, defaultValue, dropDownEditor, valueSelectorId, createRegValue, onEditHandler) {
	var self = this;
	self.valueSelectorId = valueSelectorId;
	self.values = values;
	self.defaultValue = defaultValue;
	self.dropDownEditor = dropDownEditor;

	self.draggedValue = ko.observable();
	self.hoveredElement = ko.observable();

	self.onDrop = function () {
		
		self.values($.map(self.valuesViewmodel(), function(val) { return val.model; }));
	}


	self.valuesViewmodel = ko.observableArray();

	
	self.values.subscribe(function (values) {
		self.valuesViewmodel($.map(values,
			function(val) {
				return new SelectRegValueViewmodel(val,
					self.onDrop,
					function() {
						var draggableElement = self.valuesViewmodel().filter(function(el) { return el.isDragged(); })[0];
						var dropPosition = self.valuesViewmodel().filter(function(el) { return el.isDropPosition(); })[0];
						if (!dropPosition) return;
						var draggablePosition = self.valuesViewmodel().indexOf(draggableElement);
						var dropIndex = self.valuesViewmodel().indexOf(dropPosition);
						
						if (dropPosition.dragAfter())
							dropIndex++;
						
						if (draggablePosition !== dropIndex) {
							if (dropIndex > draggablePosition) {
								dropIndex--;
							}
							self.valuesViewmodel.splice(draggablePosition, 1);
							self.valuesViewmodel.splice(dropIndex, 0, draggableElement);
							draggableElement.correctPosition($('#' + self.valueSelectorId + dropIndex));
						}
					}, self.draggedValue, self.hoveredElement);
			}));
	});



	/** 
	 * Редактирование значений DropDown-регпараметра 
	*/

	//Deep copy выделенного значения рег параметра для изменения
	self.editableValue = ko.observable();

	//Индекс изменяемого значения рег параметра
	self.editableValueIndex = -1;

	self.editableValueIsNew = ko.observable(true);
	self.editableValueIsDefault = ko.observable(false);


	self.dropDownIsEmpty = ko.computed(function () {
		var listValues = self.values();
		return !listValues || !listValues.length;
	});

	//Проверка наличия ошибки в значении рег параметра (проверка на повторение) 
	self.regDropDownValueIsValid = ko.computed(function () {

		if (!self.editableValue()) {
			return true;
		}
		var value = self.editableValue().Value();

		for (var i = 0; i < self.values().length; i++) {
			if (self.editableValueIndex != i
				&& self.values()[i].Value() === value) {
				return false;
			}
		}
		return true;
	}).extend({ deferred: true });

	self.saveEditableRegValue = function () {

		if (!self.regDropDownValueIsValid()) {
			return;
		}

		var editableValue = self.editableValue() || createRegValue();
		
		if (self.editableValueIndex === -1) {
			self.values.push(editableValue);
		} else {
			self.values.splice(self.editableValueIndex, 1, editableValue);
		}

		if (self.editableValueIsDefault()) {
			self.defaultValue(editableValue);
		} else {
			if (self.defaultValue().Value() === editableValue.Value()) {
				self.defaultValue(createRegValue());
			}
		}
		if (onEditHandler)
			onEditHandler();
	}



	self.toggleDropDownEditor = function (event) {
		var elemId = $(event.target).attr('id');
		if (self.dropDownEditor.domId === elemId) {
			self.dropDownEditor.toggle(null, event);
		} else {
			self.dropDownEditor.close();
			self.dropDownEditor.domId = elemId;
			self.dropDownEditor.model(self);
			self.dropDownEditor.open(null, event);
		}

		if (self.dropDownEditor.isVisible()) {
			document.getElementById('dropDownEditorInput').focus();
		}
	}

	self.editDropDownValue = function (data, event) {
		self.editableValueIndex = self.values().indexOf(data.model);
		self.editableValue(data.model.Clone());
		self.editableValueIsNew(false);
		self.editableValueIsDefault(self.defaultValue() && self.defaultValue().Value() == data.model.Value());
		self.toggleDropDownEditor(event);

		return true;
	}

	self.addDropDownValue = function (data, event) {
		if (createRegValue) {
			self.editableValueIndex = -1;
			self.editableValue(createRegValue());
			self.editableValueIsNew(true);
			self.editableValueIsDefault(false);
			self.toggleDropDownEditor(event);
		}

		return true;
	}

	self.onDeleteRegValue = function (data, event) {

		FC_dialogs.showConfirmDialog(
			{
				environment: FC.Environment
			},
			document.getElementById('reactBTSDialogHost'),
			FC.Environment.Localization.Batches.DeleteBatchRegParamValuePrompt,
			function () {
				if (self.defaultValue().Value() === self.editableValue().Value()) {
					self.defaultValue(createRegValue());
				}
				self.values.splice(self.editableValueIndex, 1);
				self.editableValueIndex = -1;
				self.editableValue(null);
				self.dropDownEditor.close();

				if (onEditHandler)
					onEditHandler();
			},
			function () {
				if (onEditHandler)
					onEditHandler();
			}
		);

		return true;
	}


	self.dragOverFirstSeparator = function() {
		event.preventDefault();
		self.valuesViewmodel()[0].dragBefore(true);
		return false;
	}
	self.dragLeaveFirstSeparator = function (data, event) {
		var target = $(event.target);
		var relatedTarget = $(event.relatedTarget);
		if (relatedTarget.hasClass('inline-list-drop-indicator') ||
			relatedTarget.hasClass('inline-list-separator__dot') ||
			target.hasClass('inline-list-drop-indicator') ||
			target.hasClass('inline-list-separator__dot')) {
			return true;
		}

		self.valuesViewmodel()[0].dragBefore(false);
		return true;
	}

	self.values.valueHasMutated();


	self.drop = function() {
		self.draggedValue(null);
	}

	self.dragEnd = function () {
		self.draggedValue(null);
	}
}



function SelectRegValueViewmodel(model, onDrop, onDropPositionChanged,draggedValue, hoveredElement) {
	var self = this;
	self.model = model;


	
	self.isHovered = ko.observable(false);

	self.dragBefore = ko.observable(false);
	self.dragAfter = ko.observable(false);
	self.isDragged = ko.observable(false);
	self.hoveredStyle = ko.observable();
	self.isDropPosition = ko.computed(function () { return self.dragBefore() || self.dragAfter(); });

	self.correctPosition = function(domElement) {
		var target = domElement;
		self.hoveredStyle({
			top: target.position().top + 'px',
			left: (target.position().left - 6) + 'px'
		});
	}
	self.mouseover = function (data, event) {
		if (draggedValue()) return true;
		self.correctPosition($(event.target));
		if (hoveredElement() && hoveredElement() != self) {
			hoveredElement().isHovered(false);
		}
		hoveredElement(self);
		self.isHovered(true);

	}

	self.mouseout = function (data, event) {
		hoveredElement(null)
		self.isHovered(false);
	}

	self.dragStart = function (data, event) {
		var crt = event.target.cloneNode(true);
		crt.style.backgroundColor = "red";
		crt.style.display = "none"; /* or visibility: hidden, or any of the above */
		document.body.appendChild(crt);
		event.originalEvent.dataTransfer.setDragImage(crt, 0, 0);
		draggedValue(self);
		self.isDragged(true);
		return true;
	}

	self.dragEnd = function () {
		draggedValue(null);
		self.isDragged(false);
		return true;
	}

	self.dragOver = function (data, event) {
		if (!draggedValue() ) {
			return true;
		}
		event.preventDefault();
		if (!self.isDragged()) {
			var target = $(event.target);
			var targetWidth = target.outerWidth();
			var position = event.originalEvent.offsetX;
			if (position < targetWidth / 2) {
				self.dragAfter(false);
				self.dragBefore(true);
			} else {
				self.dragAfter(true);
				self.dragBefore(false);
			}
			onDropPositionChanged();
		}
		return false;
	}

	self.dragLeave = function () {
		if (!draggedValue()) {
			return true;
		}
		self.dragAfter(false);
		self.dragBefore(false);
		onDropPositionChanged();
		return true;
	}

	self.drop = function (data, event) {
		if (!draggedValue()) {
			return true;
		}
		onDrop();
	}

	self.dragOverSeparator = function (data, event) {
		if (!draggedValue() || self.isDragged()) {
			return true;
		}
		event.preventDefault();
		self.dragAfter(true);
		onDropPositionChanged();
		return false;
	}

	self.dragLeaveSeparator = function (data, event) {
		if (!draggedValue()) {
			return true;
		}
		var target = $(event.target);
		var relatedTarget = $(event.relatedTarget);
		if (relatedTarget.hasClass('inline-list-drop-indicator') ||
			relatedTarget.hasClass('inline-list-separator__dot') ||
			target.hasClass('inline-list-drop-indicator') ||
			target.hasClass('inline-list-separator__dot')) {
			return true;
		}
		
		self.dragAfter(false);
		onDropPositionChanged();
		return true;
	}
}

