﻿window.batchesApp = window.batchesApp || {};

window.batchesApp.datacontext = (function () {

	var datacontext = {
		hasCurrentBatch: hasCurrentBatch,
		getBatchTypes: getBatchTypes,
		getBatches: getBatches,
		getRescanTasks: getRescanTasks,
		deleteBatch: deleteBatch,
		cancelSendingBatch: cancelSendingBatch,
		renewSendingBatch: renewSendingBatch,
		createBatch: createBatch,
		openRescanBatch: openRescanBatch
	};

	return datacontext;


    /**
     * Получение списка типов пакетов
     * @param {} batchTypesObservable Переменная для записи результата (observable,computed с write). Записывается массив объектов BatchType
     * @param {} callbackFunction CAllback функция, в качестве параметра batchTypesObservable.
     * @returns {} 
     */
	function getBatchTypes(callbackFunction) {
	    var batchTypes = [];
	    window.scanModuleCommunicator.executeAsync(
            'Workspace_getBatchTypeCount',
            [],
            function (batchTypesCountResponse) {
	            if (batchTypesCountResponse.hasErrors) {
	                FC.Utils.Error.Show(batchTypesCountResponse.errorText);
		            batchTypes = [];
	            }
                var batchesCount = parseInt(batchTypesCountResponse.result);
	            var numOfReceivedValues = 0;
	            var numOfRequiredFields = batchesCount * 2;
                var insertFunction = function(index) {
                    var newBatchType = new datacontext.BatchType({
                        Index: index
                    });
	                batchTypes.push(newBatchType);
                    window.scanModuleCommunicator.executeAsync('Workspace_KeepBatchType', [index], function (keepBatchTypeResponse) {
                        if (keepBatchTypeResponse.hasErrors) {
                            FC.Utils.Error.Show(keepBatchTypeResponse.errorText);
                        } else {

                            window.scanModuleCommunicator.executeAsync(
                                'BatchType_getID',
                                [keepBatchTypeResponse.result],
                                function (batchTypeIdResponse) {
                                    newBatchType.Id(parseInt(batchTypeIdResponse.result));
                                    numOfReceivedValues++;
                                    if ((numOfReceivedValues === numOfRequiredFields) && (callbackFunction != null)) {
                                    	callbackFunction(batchTypes);
                                    }
                                }
                            );
                            window.scanModuleCommunicator.executeAsync(
                                'BatchType_getName',
                                [keepBatchTypeResponse.result],
                                function (batchTypeNameResponse) {
                                    newBatchType.Name(batchTypeNameResponse.result);
                                    if (batchTypeNameResponse.result == 'Default') {
                                        newBatchType.Name(FC.Environment.Localization.Common.DefaultBatchTypeName);
                                    }
                                    
                                    numOfReceivedValues++;
                                    if ((numOfReceivedValues === numOfRequiredFields) && (callbackFunction != null)) {
                                    	callbackFunction(batchTypes);
                                    }
                                }
                            );
                        }
                    });
                    
                }
                for (var i = 0; i < batchesCount ; i++) {
	                insertFunction(i);
	            }
            }
        );
	}
    /**
     * Десералиация информации о пакетах из строки в массив Batch
     * @param {} serializedString строка формата плагина сканирования
     * @returns {} 
     */
	function convertBatches(batches) {
		var isNewVersionPlugin = JSON.parse(localStorage.getItem('isNewVersionPlugin'));
	    var mappedBatches = isNewVersionPlugin ? 
							batches.map(function (batchInfo) { return new datacontext.Batch(batchInfo); }) : 
							$.map(batches.split(';'), function (batchInfo) { return new datacontext.Batch(batchInfo); });
	    for (var i = mappedBatches.length - 1; i >= 0; i--) {
	        var mappedBatch = mappedBatches[i];
	        if ((mappedBatch.RescanTaskID() != null) && (mappedBatch.RescanTaskID() !== '') && (parseInt(mappedBatch.RescanTaskID()) >= 0))
	            mappedBatches.splice(i, 1);
	    }

	    return mappedBatches;
	}

	/**
	 * Имеется ли текущий открытый пакет
	 * @param {} finishedFunc Callback-функция. Передаётся объект со свойством result = "1" или "0" в качестве параметра
	 * @returns {} 
	 */
	function hasCurrentBatch(finishedFunc) {
		return window.scanModuleCommunicator.executeAsync('Workspace_UpdateBatchInfos', [], function () {
			window.scanModuleCommunicator.executeAsync('Workspace_HasCurrentBatch',
				[],
				finishedFunc);
		});
	}

    /**
     * Получение списка пакетов 
     * @param {} batchesObservable Переменная для записи результата (observable,computed с write). Записывается массив объектов Batch
     * @param {} finishedFunc CAllback функция. Передается batchesObservable в качестве параметра
     * @returns {} 
     */
	function getBatches(batchesObservable, finishedFunc) {
		hasCurrentBatch(function(hasBatch) {
	        var getBatchesFromPlugin = function () {
				var isNewVersionPlugin = JSON.parse(localStorage.getItem('isNewVersionPlugin'));
	
	            window.scanModuleCommunicator.executeAsync(
					isNewVersionPlugin ? 
						'Workspace_Json-GetAllBatchFullInfos' :
						'Workspace_GetAllBatchInfos',
	                [],
	                function(data) {
	                    if (data.hasErrors || (data.result.length === 0)) {
	                        batchesObservable([]);
	                    } else {
							var data = isNewVersionPlugin ? JSON.parse(data.result) : data.result;
	                        batchesObservable(convertBatches(data));
						}

						if (finishedFunc) {
							finishedFunc();
						}
	                });
	        }
	        if (hasBatch.result === '1') {
	            window.scanModuleCommunicator.executeAsync('Workspace_CloseCurrentBatch', [], getBatchesFromPlugin);
	        } else {
	            getBatchesFromPlugin();
	        }

	    });
	}

    /**
 * Получение списка задач пересканирования 
 * @param {} rescansObservable Переменная для записи результата (observable,computed с write). Записывается массив объектов Batch
 * @param {} finishedFunc CAllback функция. Передается rescansObservable в качестве параметра
 * @returns {} 
 */
	function getRescanTasks(rescansObservable, finishedFunc) {
		window.scanModuleCommunicator.executeAsync('Workspace_UpdateRescanBatchInfos', [], function () {
			var isNewVersionPlugin = JSON.parse(localStorage.getItem('isNewVersionPlugin'));
			if (isNewVersionPlugin) {
				window.scanModuleCommunicator.executeAsync('Workspace_Json-GetAllRescanBatchFullInfos', [], function (data) {
					var mappedRescans;
					if (data.hasErrors || (JSON.parse(data.result).length === 0))
						mappedRescans = [];
					else
						mappedRescans =  JSON.parse(data.result).map(function (batchInfo) { return new datacontext.Batch(batchInfo); });
	
	
					window.scanModuleCommunicator.executeAsync('Workspace_UpdateBatchInfos', [], function () {
						window.scanModuleCommunicator.executeAsync('Workspace_JSON-GetAllBatchFullInfos', [], function (data) {
							if (!data.hasErrors && (JSON.parse(data.result).length > 0)) {
								var mappedBatches = JSON.parse(data.result).map(function (batchInfo) { return new datacontext.Batch(batchInfo); });
								for (var i = mappedBatches.length - 1; i >= 0; i--) {
									var mappedBatch = mappedBatches[i];
									if ((mappedBatch.RescanTaskID() != null) &&
									(mappedBatch.RescanTaskID() !== '') &&
									(parseInt(mappedBatch.RescanTaskID()) >= 0)) {
										mappedRescans.push(mappedBatches[i]);
									}
								}
							}
							rescansObservable(mappedRescans);
	
							if (finishedFunc) {
								finishedFunc();
							}
						});
					});
					
				});
			} else {
				window.scanModuleCommunicator.executeAsync('Workspace_GetAllRescanBatchInfos', [], function (data) {
					var mappedRescans;
					if (data.hasErrors || (data.result.length === 0)) 
						mappedRescans = [];
					else
						mappedRescans = $.map(data.result.split(';'), function (batchInfo) { return new datacontext.Batch(batchInfo); });
	
	
					window.scanModuleCommunicator.executeAsync('Workspace_UpdateBatchInfos', [], function () {
						window.scanModuleCommunicator.executeAsync('Workspace_GetAllBatchInfos', [], function (data) {
							if (!data.hasErrors && (data.result.length > 0)) {
								var mappedBatches = $.map(data.result.split(';'), function (batchInfo) { return new datacontext.Batch(batchInfo); });
								for (var i = mappedBatches.length - 1; i >= 0; i--) {
									var mappedBatch = mappedBatches[i];
									if ((mappedBatch.RescanTaskID() != null) &&
									(mappedBatch.RescanTaskID() !== '') &&
									(parseInt(mappedBatch.RescanTaskID()) >= 0)) {
										mappedRescans.push(mappedBatches[i]);
									}
								}
							}
							rescansObservable(mappedRescans);
	
							if (finishedFunc) {
								finishedFunc();
							}
						});
					});
					
				});
			}

		});
	}

    /**
     * Функция удаления пакета
     * @param {} batchId ID пакета
     * @param {} callbackFunction Функция, вызываемая после выполнения, в качестве параметра ответ плагина
     * @returns {} 
     */
	function deleteBatch(batchId, callbackFunction) {
		return window.scanModuleCommunicator.executeAsync('Workspace_DeleteBatch', [batchId], callbackFunction);
	}

    /**
    * Функция отмены отправки пакета
    * @param {} batchId ID пакета
    * @param {} callbackFunction Функция, вызываемая после выполнения, в качестве параметра ответ плагина
    * @returns {} 
    */
	function cancelSendingBatch(batchId, callbackFunction) {
	    window.scanModuleCommunicator.executeAsync('Workspace_StopBatchUploading', [batchId], callbackFunction);
	}

    /**
    * Функция возобновления отправки пакета
    * @param {} batchId ID пакета
    * @param {} callbackFunction Функция, вызываемая после выполнения, в качестве параметра ответ плагина
    * @returns {} 
    */
	function renewSendingBatch(batchId, callbackFunction) {
	    window.scanModuleCommunicator.executeAsync('Workspace_UploadBatch', [batchId], callbackFunction);
	}
    
    /**
    * Функция получения задачи пересканирования 
    * @param {} rescanId ID Задачи пересканирования
    * @param {} callbackFunction Функция, вызываемая после выполнения, в качестве параметра ответ плагина
    * @returns {} 
    */
	function openRescanBatch(rescanId, callbackFunction) {
	    return window.scanModuleCommunicator.executeAsync('Workspace_ProcessRescanTask', [rescanId], callbackFunction);
	}
    
    /**
    * Функция создания пустого пакета
    * @param {} batchType Объект класса BatchType
    * @param {} callbackFunction Функция, вызываемая после выполнения, в качестве параметра ответ плагина
    * @returns {} 
    */
	function createBatch(batchType, callbackFunction) {
		window.batchtypeSettings.datacontext.synchronizeRegParameters(batchType, function () {
			window.scanModuleCommunicator.executeAsync('Workspace_CreateBatch', [batchType.Index()], callbackFunction);
		});
	}

    


})();