﻿window.batchtypeSettings = window.batchtypeSettings || {};

window.batchtypeSettings.datacontext = (function () {


	function getUserCanChangeSettings(projectGuid, callbackFunction) {
		$.ajax({
			url: FC.Environment.AppPath + 'Batches/CheckUserCanEditSettings',
			cache: false,
			data: {
				projectGuid: projectGuid
			},
			success: function (data) {
				if (data.ErrorType != undefined) {
					if ((data.ErrorType === 303) && (data.Url != undefined)) {
						location.replace(data.Url);
					}
				}
				callbackFunction(data);
			}
		});
	}
    /**
     * Асинхронный вызов метода плагина с проверкой результата и выводом ошибки
     * @param {} methodName название метода плагина
     * @param {} params параметры метода плагина (Array)
     * @param {} continueFunc функция, которая вызывается в случае успешного выполнения. Имеет 1 параметр, в который передается результат
     * @returns {} 
     */
    function executeSuccessContinue(methodName, params, continueFunc) {
        scanModuleCommunicator.executeAsync(methodName,
            params,
            function(methodResult) {
                if (methodResult.hasErrors) {
                    FC.Utils.Error.Show(methodResult.errorText);
                } else if (continueFunc!=undefined) {
                    continueFunc(methodResult.result);
                }
            });
    }

    /**
     * Выполняет метод и присваивает полученное значение в observable переменную. 
     * @param {} opts объект параметров
     * opts.methodName название метода плагина
     * opts.params параметры метода плагина (Array)
     * opts.observable переменная для сохранения результата
     * opts.defaultValue значение в случае непудачного вызова метода
     * opts.conversionFunc функция преобразрования результата из строки
     * callbackFunction функция, вызываемая после присвоения результата
     * @returns {} 
     */
    function getPluginValue(opts) {
        scanModuleCommunicator.executeAsync(opts.methodName,
            opts.params,
            function(methodResult) {
                if (methodResult.hasErrors) {
                    FC.Utils.Error.Show(methodResult.errorText);
                    if (opts.defaultValue != undefined) {
                        opts.observable(opts.defaultValue);
                    }
                } else {
                    if (opts.conversionFunc != undefined) opts.observable(opts.conversionFunc(methodResult.result));
                    else opts.observable(methodResult.result);
                }
                if (opts.callbackFunction != undefined) {
                    opts.callbackFunction(opts.observable);
                }
            });
    }

    /**
     *  Получение имени типа пакета
     * @param {} batchType Объект класса BatchType
     * @param {} batchTypeNameObservable Переменная для записи результата (observable,computed с write)
     * @param {} callbackFunction Функция, вызывающаяся после получения результата. Вызывается с параметром batchTypeNameObservable
     * @returns {} 
     */
    function getBatchTypeName(batchType, batchTypeNameObservable, callbackFunction) {
        scanModuleCommunicator.executeAsync('Workspace_KeepBatchType', [batchType.Index()], function (keepBatchTypeResult) {
            if (keepBatchTypeResult.hasErrors) {
                FC.Utils.Error.Show(keepBatchTypeResult.errorText);
                batchTypeNameObservable("");
                if (callbackFunction != undefined) {
                    callbackFunction(batchTypeNameObservable);
                }
            } else {
                getPluginValue({
                    methodName: 'BatchType_getName',
                    params: [keepBatchTypeResult.result],
                    observable: batchTypeNameObservable,
                    conversionFunc: function(value){
                        if (value == 'Default') {
                            return FC.Environment.Localization.Common.DefaultBatchTypeName;
                        }
                        return value;
                    },
                    defaultValue: '',
                    callbackFunction: callbackFunction
                });
            }
        });

    }

    /**
     * Получение описания типа пакета 
     * @param {} batchType  Объект класса BatchType
     * @param {} batchTypeDescrObservable Переменная для записи результата (observable,computed с write)
     * @param {} callbackFunction Функция, вызывающаяся после получения результата. Вызывается с параметром batchTypeNameObservable
     * @returns {} 
     */
    function getBatchTypeDescription(batchType, batchTypeDescrObservable, callbackFunction) {

        scanModuleCommunicator.executeAsync('Workspace_KeepBatchType', [batchType.Index()],
            function(keepBatchTypeResult) {
                if (keepBatchTypeResult.hasErrors) {
                    FC.Utils.Error.Show(keepBatchTypeResult.errorText);
                    batchTypeDescrObservable("");
                    if (callbackFunction != undefined) {
                        callbackFunction(batchTypeDescrObservable);
                    }
                } else {
                    getPluginValue({
                        methodName: 'BatchType_getDescription',
                        params: [keepBatchTypeResult.result],
                        observable: batchTypeDescrObservable,
                        defaultValue: '',
                        callbackFunction: callbackFunction
                    });
                }
            });
    }

    /**
     * Функция получения настроек сканирования
     * @param {} batchType Объект класса BatchType
     * @param {} scanningOptionsObservable Переменная для записи результата (observable,computed с write). Записывается объект класса ScanningProperties
     * @returns {} 
     */

	var datacontext = {
		loadServerRegParams: loadServerRegParams,
        getAllRegParameters: getAllRegParameters,
        saveRegParameters: saveRegParameters,
        synchronizeRegParameters: synchronizeRegParameters,
        getBatchTypeDescription: getBatchTypeDescription,
        getBatchTypeName: getBatchTypeName,
        openNativeSettings: openNativeSettings,
        getScanningProps: getScanningProps,
        setScanningProps: setScanningProps,
        getAvailableScannerOptions: getAvailableScannerOptions,
        getBatchProperties: getBatchProperties,
        setBatchProperties: setBatchProperties,
        getDocumentProperties: getDocumentProperties,
        setDocumentProperties: setDocumentProperties,
        getBackgroundUploadingProps: getBackgroundUploadingProps,
        setBackgroundUploadingProps: setBackgroundUploadingProps,
		getRegParamsForbid: getRegParamsForbid,
		setRegParamsForbid: setRegParamsForbid,
		getUserCanChangeSettings: getUserCanChangeSettings

    };

    /**
     *  Получение данных профиля сканирования
     * @param {} batchType Объект класса BatchType
     * @param {} scanningOptionsObservable Переменная для записи результата (observable,computed с write). Записывается объект класса ScanningProperties
     * @returns {} 
     */
    function getScanningProps(batchType, scanningOptionsObservable) {
        executeSuccessContinue('Workspace_KeepBatchType',
            [batchType.Index()],
            function(batchTypeKeepId) {
                executeSuccessContinue('BatchType_KeepScanningOptions',
                    [batchTypeKeepId],
                    function(scanOptsId) {
                        executeSuccessContinue('ScanningOptions_KeepScanProfile',
                            [scanOptsId],
                            function(scanProfileId) {
                                var scanProps = new datacontext.ScanningProperties();

                                getPluginValue({
                                    methodName: 'ScanningOptions_getDontSpecifyForBatchType',
                                    params: [scanOptsId],
                                    observable: scanProps.useSpec,
                                    defaultValue: false,
                                    conversionFunc: function(val) { return val !== '1' }
                                });

                                getPluginValue({
                                    methodName: 'ScanProfile_getUseNativeScannerInterface',
                                    params: [scanProfileId],
                                    observable: scanProps.selectedInterface,
                                    defaultValue: "native",
                                    conversionFunc: function (val) {
                                        return val === '1' ? "native" : "abbyy";
                                    }
                                });

                                getPluginValue({
                                    methodName: 'ScanProfile_getShowOptionsBeforeScanning',
                                    params: [scanProfileId],
                                    observable: scanProps.showWindowBeforeScan,
                                    defaultValue: false,
                                    conversionFunc: function (val) { return val === '1' }
                                });

                                getPluginValue({
                                    methodName: 'ScanProfile_getScanSourceIndex',
                                    params: [scanProfileId],
                                    observable: scanProps.scannerIndex,
                                    defaultValue: 0,
                                    conversionFunc: parseInt
                                });

                                scanningOptionsObservable(scanProps);
                            });
                    });
            });
    }

    /**
     * Сохранение настреок сканирования
     * @param {} batchType Объект класса BatchType
     * @param {} options Объект класса ScanningProperties
     * @returns {} 
     */
    function setScanningProps(batchType, options) {
        executeSuccessContinue('Workspace_KeepBatchType',
            [batchType.Index()],
            function (batchTypeKeepId) {
                executeSuccessContinue('BatchType_KeepScanningOptions',
                    [batchTypeKeepId],
                    function (scanOptsId) {
                        executeSuccessContinue('ScanningOptions_KeepScanProfile',
                            [scanOptsId],
                            function (scanProfileId) {
                                var numOfSaved = 0;
                                var saveCallback = function () {
                                    numOfSaved++;
                                    if (numOfSaved === 5) {
                                        executeSuccessContinue('ScanProfile_ResetKeptScanProfile',
                                            [scanProfileId],
                                            function () {
                                                scanModuleCommunicator.executeAsync('Workspace_CommitBatchTypeChanges', [batchType.Index()]);
                                            });
                                    }
                                }
                                scanModuleCommunicator.executeAsync('ScanningOptions_setDontSpecifyForBatchType',
                                [scanOptsId, !options.useSpec()], saveCallback);
                                scanModuleCommunicator.executeAsync('ScanningOptions_setForbidOperatorToChangeForBatch',
                                [scanOptsId, false], saveCallback);

                                // TODO: убрать затычку true
                                //scanModuleCommunicator.executeAsync('ScanProfile_setUseNativeScannerInterface', [scanProfileResponse.result, options.SelectedInterface() == "native"]);
                                scanModuleCommunicator.executeAsync('ScanProfile_setUseNativeScannerInterface',
                                [scanProfileId, true], saveCallback);

                                // TODO: убрать затычку true
                                //scanModuleCommunicator.executeAsync('ScanProfile_setShowOptionsBeforeScanning', [scanProfileResponse.result, options.showWindowBeforeScan()]);
                                scanModuleCommunicator.executeAsync('ScanProfile_setShowOptionsBeforeScanning',
                                [scanProfileId, options.showWindowBeforeScan()], saveCallback);

                                scanModuleCommunicator.executeAsync('ScanProfile_setScanSourceIndex',
                                [scanProfileId, options.scannerIndex()], saveCallback);
                            });
                    });
            });

    }

    /**
     * Получение информации о подключенных сканерах. 
     * @param {} batchType Объект класса BatchType
     * @param {} scannerOptionsObservable Переменная для записи результата (observable,computed с write). Записывается объект класса ScannerOptions
     * @returns {} 
     */
    function getAvailableScannerOptions(scannerOptionsObservable) {
        scanModuleCommunicator.executeAsync('Workspace_getScannerCount', [], function (scannersCountResponse) {
            scannerOptionsObservable([]);
            if (scannersCountResponse.hasErrors) {
                FC.Utils.Error.Show(scannersCountResponse.errorText);
            } else {
                var scannersCount = parseInt(scannersCountResponse.result);
                
                
                var numOfInitedScanners = 0;
                var addScannerFunction = function(index) {
                    executeSuccessContinue('Workspace_KeepScanningSource',
                        [index],
                        function (scannerKeepId) {
                            scanModuleCommunicator.executeAsync('ScanningSource_Init', [scannerKeepId], function (scannerInitSuccess) {
                                numOfInitedScanners++;
                                if (numOfInitedScanners === scannersCount - 1) {
                                    $(".global-locker").remove();                                    
                                }
                                if (scannerInitSuccess.hasErrors) {
                                    return;
                                }
                                
                                var currentScannerOptions = new datacontext.ScannerOptions({ Index: index });
                                    getPluginValue({
                                        methodName: 'ScanningSource_getName',
                                        params: [scannerKeepId],
                                        observable: currentScannerOptions.scannerName,
                                        defaultValue: ''
                                    });

                                    /* TODO Вернуть когда сделаем ABBYY интерфейс
                                        var scannerNIAResponse = scanModuleCommunicator.executeAsync('ScanningSource_getIsNativeScannerInterfaceAllowed', [scannerKeepResponse.result]);
                                        var scannerAIAResponse = scanModuleCommunicator.executeAsync('ScanningSource_getIsABBYYScannerInterfaceAllowed', [scannerKeepResponse.result]);
                                        */

                                    getPluginValue({
                                        methodName: 'ScanningSource_getIsNativeScannerSilentInterfaceAllowed',
                                        params: [scannerKeepId],
                                        observable: currentScannerOptions.isNativeSilentModeAvailable,
                                        defaultValue: '',
                                        conversionFunc: function (val) { return val === '1' }
                                    });
                                    scannerOptionsObservable.push(currentScannerOptions);
                                });
                        });
                }
                if (scannersCount > 1) {
                    $('body').append('<div class="global-locker"></div>')
                }
                for (var i = 1; i < scannersCount ; i++) {
                    addScannerFunction(i);
                }
            }
        });
    }

    /**
     * Открытие окна настроек сканера из драйвера
     * @param {} batchType Объект класса BatchType
     * @param {} options Объект класса ScannerOptions
     * @returns {} 
     */
    function openNativeSettings(batchType, options) {
        executeSuccessContinue('Workspace_KeepScanningSource',
            [options.Index],
            function(scannerKeepId) {
                executeSuccessContinue('ScanningSource_Init',
                    [scannerKeepId],
                    function() {
                        executeSuccessContinue('Workspace_KeepBatchType',
                            [batchType.Index()],
                            function (keepBatchTypeId) {
                                executeSuccessContinue('BatchType_KeepScanningOptions',
                                    [keepBatchTypeId],
                                    function (keepScanOptsId) {
                                        executeSuccessContinue('ScanningOptions_KeepScanProfile',
                                            [keepScanOptsId],
                                            function (keepScanProfileId) {
                                                executeSuccessContinue('ScanProfile_setScanSourceIndex',
                                                    [keepScanProfileId, options.Index],
                                                    function () {
                                                        executeSuccessContinue('ScanProfile_ShowNativeScanDialog',
                                                            [keepScanProfileId],
                                                            function () {
                                                                executeSuccessContinue('ScanProfile_RestoreKeptScanProfile',
                                                                    [keepScanProfileId]);
                                                            });
                                                    });
                                            });
                                    });
                            });
                    });
            });
    }

    /**
     * Получение настроек пакета
     * @param {} batchType  Объект класса BatchType
     * @param {} batchPropertiesObservable Переменная для записи результата (observable,computed с write). Записывается объект класса BatchProperties
     * @returns {} 
     */
    function getBatchProperties(batchType, batchPropertiesObservable) {
        executeSuccessContinue('Workspace_KeepBatchType',
            [batchType.Index()],
            function(keepBatchTypeId) {
                executeSuccessContinue('BatchType_KeepBatchOptions',
                    [keepBatchTypeId],
                    function(keepBatchOptsId) {
                        var newBatchProperties = new datacontext.BatchProperties();
                        getPluginValue({
                            methodName: 'BatchOptions_getDontSpecifyForBatchType',
                            params: [keepBatchOptsId],
                            observable: newBatchProperties.useSpec,
                            defaultValue: false,
                            conversionFunc: function(val) { return val !== 'true' }
                        });

                        getPluginValue({
                            methodName: 'BatchOptions_getForbidOperatorToChangeForBatch',
                            params: [keepBatchOptsId],
                            observable: newBatchProperties.forbidOperator,
                            defaultValue: false,
                            conversionFunc: function(val) { return val === '1' }
                        });

                        getPluginValue({
                            methodName: 'BatchOptions_getPriority',
                            params: [keepBatchOptsId],
                            observable: newBatchProperties.priorInd,
                            defaultValue: 0,
                            conversionFunc: parseInt
                        });

                        getPluginValue({
                            methodName: 'BatchOptions_getNamingSchema',
                            params: [keepBatchOptsId],
                            observable: newBatchProperties.namingScheme().Value,
                            defaultValue: ''
                        });
                        batchPropertiesObservable(newBatchProperties);
                    });
            });
    }
    
    /**
     * Сохранение настроек пакета 
     * @param {} batchType Объект класса BatchType
     * @param {} batchPropperties Объект класса BatchProperties
     * @returns {} 
     */
    function setBatchProperties(batchType, batchPropperties) {

        executeSuccessContinue('Workspace_KeepBatchType',
            [batchType.Index()],
            function(keepBatchTypeId) {
                executeSuccessContinue('BatchType_KeepBatchOptions',
                    [keepBatchTypeId],
                    function (keepBatchOptsId) {
                        var numOfOPerations = 0;
                        var commitFunction = function () {
                            numOfOPerations++;
                            if (numOfOPerations >= 4) {
                                scanModuleCommunicator
                                    .executeAsync('Workspace_CommitBatchTypeChanges', [batchType.Index()]);
                            }
                        };
                        scanModuleCommunicator.executeAsync('BatchOptions_setDontSpecifyForBatchType',
                        [keepBatchOptsId, !batchPropperties.useSpec()], commitFunction);
                        scanModuleCommunicator.executeAsync('BatchOptions_setForbidOperatorToChangeForBatch',
                        [keepBatchOptsId, batchPropperties.forbidOperator()], commitFunction);
                        scanModuleCommunicator.executeAsync('BatchOptions_setPriority',
                        [keepBatchOptsId, batchPropperties.priorInd()], commitFunction);
                        scanModuleCommunicator.executeAsync('BatchOptions_setNamingSchema',
                        [keepBatchOptsId, batchPropperties.namingScheme().Value()], commitFunction);
                    });
            });
    }
    
    /**
     * Получение настроек документа
     * @param {} batchType Объект класса BatchType
     * @param {} docPropertiesObservable Переменная для записи результата (observable,computed с write). Записывается объект класса DocumentProperties
     * @returns {} 
     */
    function getDocumentProperties(batchType, docPropertiesObservable) {

        executeSuccessContinue('Workspace_KeepBatchType', [batchType.Index()], function(keepBatchTypeId) {
            executeSuccessContinue('BatchType_KeepDocumentOptions', [keepBatchTypeId], function (keepDocOptsId) {
                var docProps = new datacontext.DocumentProperties();
                executeSuccessContinue('DocumentOptions_KeepEmptyPageDetectionOptions',
                    [keepDocOptsId],
                    function(emptyPageDetectOptsId) {
                        getPluginValue({
                            methodName: 'EmptyPageDetectionOptions_getAutodetectKernelRect',
                            params: [emptyPageDetectOptsId],
                            observable: docProps.emptyPageDetectOpts().autoDetectKernalRect,
                            defaultValue: true,
                            conversionFunc: function(val) { return val === '1'; }
                        });

                        getPluginValue({
                            methodName: 'EmptyPageDetectionOptions_getLeftMargin',
                            params: [emptyPageDetectOptsId],
                            observable: docProps.emptyPageDetectOpts().leftMargin,
                            defaultValue: 0,
                            conversionFunc: parseInt
                        });

                        getPluginValue({
                            methodName: 'EmptyPageDetectionOptions_getRightMargin',
                            params: [emptyPageDetectOptsId],
                            observable: docProps.emptyPageDetectOpts().rightMargin,
                            defaultValue: 0,
                            conversionFunc: parseInt
                        });

                        getPluginValue({
                            methodName: 'EmptyPageDetectionOptions_getTopMargin',
                            params: [emptyPageDetectOptsId],
                            observable: docProps.emptyPageDetectOpts().topMargin,
                            defaultValue: 0,
                            conversionFunc: parseInt
                        });

                        getPluginValue({
                            methodName: 'EmptyPageDetectionOptions_getBottomMargin',
                            params: [emptyPageDetectOptsId],
                            observable: docProps.emptyPageDetectOpts().bottomMargin,
                            defaultValue: 0,
                            conversionFunc: parseInt
                        });

                        getPluginValue({
                            methodName: 'EmptyPageDetectionOptions_getMaxBlackSquaresCount',
                            params: [emptyPageDetectOptsId],
                            observable: docProps.emptyPageDetectOpts().maxBlackSquaresCount.realIntegerValue,
                            defaultValue: 0,
                            conversionFunc: parseInt
                        });

                        getPluginValue({
                            methodName: 'EmptyPageDetectionOptions_getMaxTextObjectsCount',
                            params: [emptyPageDetectOptsId],
                            observable: docProps.emptyPageDetectOpts().maxTextObjectsCount.realIntegerValue,
                            defaultValue: 0,
                            conversionFunc: parseInt
                        });

                        getPluginValue({
                            methodName: 'EmptyPageDetectionOptions_getMaxBarcodesCount',
                            params: [emptyPageDetectOptsId],
                            observable: docProps.emptyPageDetectOpts().maxBarcodesCount.realIntegerValue,
                            defaultValue: 0,
                            conversionFunc: parseInt
                        });

                        getPluginValue({
                            methodName: 'EmptyPageDetectionOptions_getMaxBlacknessPercentage',
                            params: [emptyPageDetectOptsId],
                            observable: docProps.emptyPageDetectOpts().maxBlacknessPercentage.realIntegerValue,
                            defaultValue: 0,
                            conversionFunc: parseInt
                        });
                    });

                getPluginValue({
                    methodName: 'DocumentOptions_getDontSpecifyForBatchType',
                    params: [keepDocOptsId],
                    observable: docProps.useSpec,
                    defaultValue: false,
                    conversionFunc: function(val) { return val !== '1' }
                });

                getPluginValue({
                    methodName: 'DocumentOptions_getForbidOperatorToChangeForBatch',
                    params: [keepDocOptsId],
                    observable: docProps.forbidOperator,
                    defaultValue: false,
                    conversionFunc: function(val) { return val === '1' }
                });

                getPluginValue({
                    methodName: 'DocumentOptions_getNamingSchema',
                    params: [keepDocOptsId],
                    observable: docProps.namingScheme().Value,
                    defaultValue: ''
                });

                getPluginValue({
                    methodName: 'DocumentOptions_getDocumentSeparationType',
                    params: [keepDocOptsId],
                    observable: docProps.createDocument,
                    defaultValue: false,
                    conversionFunc: function(val) { return val !== '0'; }
                });

                getPluginValue({
                    methodName: 'DocumentOptions_getDocumentSeparationType',
                    params: [keepDocOptsId],
                    observable: docProps.documentsSeparationTypeInd,
                    defaultValue: 0,
                    conversionFunc: parseInt
                });

                getPluginValue({
                    methodName: 'DocumentOptions_getPagesInDocumentCount',
                    params: [keepDocOptsId],
                    observable: docProps.pagesInDocument,
                    defaultValue: 0,
                    conversionFunc: parseInt
                });

                getPluginValue({
                    methodName: 'DocumentOptions_getDeleteSeparationPages',
                    params: [keepDocOptsId],
                    observable: docProps.deleteSeparationPages,
                    defaultValue: false,
                    conversionFunc: function(val) { return val === '1' }
                });

                getPluginValue({
                    methodName: 'DocumentOptions_getSeparationBarcodeType',
                    params: [keepDocOptsId],
                    observable: docProps.barTypeInd,
                    defaultValue: 0,
                    conversionFunc: parseInt
                });

                getPluginValue({
                    methodName: 'DocumentOptions_getBarcodeValue',
                    params: [keepDocOptsId],
					observable: function (val) { docProps.barValueActivated(!!val); docProps.barValue(val) },
                    defaultValue: ''
                });
                docPropertiesObservable(docProps);
            });
        });
    }

    /**
     * Сохранение параметров документов 
     * @param {} batchType Объект класса BatchType
     * @param {} documentPropperties Объект класса DocumentProperties
     * @returns {} 
     */
    function setDocumentProperties(batchType, documentPropperties) {
        executeSuccessContinue('Workspace_KeepBatchType',
            [batchType.Index()],
            function(keepBatchTypeId) {
                executeSuccessContinue('BatchType_KeepDocumentOptions',
                    [keepBatchTypeId],
                    function(keepDocOptsId) {
                        var docSeparationTypeIndex = documentPropperties.createDocument()
                            ? documentPropperties.documentsSeparationTypeInd()
                            : 0;
                        var numOfOPerations = 0;
                        var commitFunction = function () {
                            numOfOPerations++;
                            if (numOfOPerations >= 17) {
                                scanModuleCommunicator
                                    .executeAsync('Workspace_CommitBatchTypeChanges', [batchType.Index()]);
                            }
                        };
                        scanModuleCommunicator.executeAsync('DocumentOptions_setDontSpecifyForBatchType',
                        [keepDocOptsId, !documentPropperties.useSpec()], commitFunction);
                        scanModuleCommunicator.executeAsync('DocumentOptions_setForbidOperatorToChangeForBatch',
                        [keepDocOptsId, documentPropperties.forbidOperator()], commitFunction);
                        scanModuleCommunicator.executeAsync('DocumentOptions_setNamingSchema',
                        [keepDocOptsId, documentPropperties.namingScheme().Value()], commitFunction);
                        scanModuleCommunicator.executeAsync('DocumentOptions_setDocumentSeparationType',
                        [keepDocOptsId, docSeparationTypeIndex], commitFunction);
                        scanModuleCommunicator.executeAsync('DocumentOptions_setPagesInDocumentCount',
                        [keepDocOptsId, documentPropperties.pagesInDocument()], commitFunction);
                        scanModuleCommunicator.executeAsync('DocumentOptions_setDeleteSeparationPages',
                        [keepDocOptsId, documentPropperties.deleteSeparationPages()], commitFunction);
                        scanModuleCommunicator.executeAsync('DocumentOptions_setSeparationBarcodeType',
                        [keepDocOptsId, documentPropperties.barTypeInd()], commitFunction);
                        scanModuleCommunicator.executeAsync('DocumentOptions_setBarcodeValue',
							[keepDocOptsId, documentPropperties.barValueActivated()?documentPropperties.barValue():''], commitFunction);

                        var emptyPagesDetectOpts = documentPropperties.emptyPageDetectOpts();

                        executeSuccessContinue('DocumentOptions_KeepEmptyPageDetectionOptions',
                            [keepDocOptsId],
                            function(emptyPageDetectOptsId) {
                                scanModuleCommunicator.executeAsync('EmptyPageDetectionOptions_setAutodetectKernelRect',
                                [emptyPageDetectOptsId, emptyPagesDetectOpts.autoDetectKernalRect()], commitFunction);
                                scanModuleCommunicator.executeAsync('EmptyPageDetectionOptions_setLeftMargin',
                                [emptyPageDetectOptsId, emptyPagesDetectOpts.leftMargin()], commitFunction);
                                scanModuleCommunicator.executeAsync('EmptyPageDetectionOptions_setRightMargin',
                                [emptyPageDetectOptsId, emptyPagesDetectOpts.rightMargin()], commitFunction);
                                scanModuleCommunicator.executeAsync('EmptyPageDetectionOptions_setTopMargin',
                                [emptyPageDetectOptsId, emptyPagesDetectOpts.topMargin()], commitFunction);
                                scanModuleCommunicator.executeAsync('EmptyPageDetectionOptions_setBottomMargin',
                                [emptyPageDetectOptsId, emptyPagesDetectOpts.bottomMargin()], commitFunction);
                                scanModuleCommunicator.executeAsync('EmptyPageDetectionOptions_setMaxBlackSquaresCount',
                                [emptyPageDetectOptsId, emptyPagesDetectOpts.maxBlackSquaresCount.realIntegerValue()], commitFunction);
                                scanModuleCommunicator.executeAsync('EmptyPageDetectionOptions_setMaxTextObjectsCount',
                                [emptyPageDetectOptsId, emptyPagesDetectOpts.maxTextObjectsCount.realIntegerValue()], commitFunction);
                                scanModuleCommunicator.executeAsync('EmptyPageDetectionOptions_setMaxBarcodesCount',
                                [emptyPageDetectOptsId, emptyPagesDetectOpts.maxBarcodesCount.realIntegerValue()], commitFunction);
                                scanModuleCommunicator.executeAsync('EmptyPageDetectionOptions_setMaxBlacknessPercentage',
                                [emptyPageDetectOptsId, emptyPagesDetectOpts.maxBlacknessPercentage.realIntegerValue()], commitFunction);
                            });
                    });
            });
    }

    /**
     * Получение настроек отложенной загрузки
     * @param {} batchType batchType Объект класса BatchType
     * @param {} backgroundUploadingPropsObservable Переменная для записи результата (observable,computed с write). Записывается объект класса BackgroundSettings   
     * @returns {} 
     */
    function getBackgroundUploadingProps(batchType, backgroundUploadingPropsObservable) {
        executeSuccessContinue('Workspace_KeepBatchType', [batchType.Index()], function(keepBatchTypeId) {
            executeSuccessContinue('BatchType_KeepBackgroundUploadingSettings', [keepBatchTypeId], function(keepBackgroundUploadOptsId) {
                var backgroundSettings = new datacontext.BackgroundSettings();

                getPluginValue({
                    methodName: 'BackgroundUploadingSettings_getDontSpecifyForBatchType',
                    params: [keepBackgroundUploadOptsId],
                    observable: backgroundSettings.useSpec,
                    defaultValue: false,
                    conversionFunc: function (val) { return val !== '1' }
                });

                getPluginValue({
                    methodName: 'BackgroundUploadingSettings_getRemoveUploadedLocalBatch',
                    params: [keepBackgroundUploadOptsId],
                    observable: backgroundSettings.deleteUploaded,
                    defaultValue: false,
                    conversionFunc: function (val) { return val === '1' }
                });

                getPluginValue({
                    methodName: 'BackgroundUploadingSettings_getSpecifyUploadingTimeRange',
                    params: [keepBackgroundUploadOptsId],
                    observable: backgroundSettings.specifyUploadTime,
                    defaultValue: false,
                    conversionFunc: function (val) { return val === '1' }
                });

                getPluginValue({
                    methodName: 'BackgroundUploadingSettings_getRepeatFailedTaskUploading',
                    params: [keepBackgroundUploadOptsId],
                    observable: backgroundSettings.repeatFailedUpload,
                    defaultValue: false,
                    conversionFunc: function (val) { return val === '1' }
                });

                getPluginValue({
                    methodName: 'BackgroundUploadingSettings_getUploadingTimeRangeStart',
                    params: [keepBackgroundUploadOptsId],
                    observable: backgroundSettings.startUploadTime,
                    defaultValue: 0,
                    conversionFunc: parseInt
                });

                getPluginValue({
                    methodName: 'BackgroundUploadingSettings_getUploadingTimeRangeStop',
                    params: [keepBackgroundUploadOptsId],
                    observable: backgroundSettings.endUploadTime,
                    defaultValue: 0,
                    conversionFunc: parseInt
                });

                getPluginValue({
                    methodName: 'BackgroundUploadingSettings_getRepeatFailedTaskUploadingPeriod',
                    params: [keepBackgroundUploadOptsId],
                    observable: backgroundSettings.repeatFailedUploadTime,
                    defaultValue: 0,
                    conversionFunc: function (val) { return parseInt(val) / 60; }
                });

                getPluginValue({
                    methodName: 'BackgroundUploadingSettings_getRepeatFailedTaskUploadingAttempts',
                    params: [keepBackgroundUploadOptsId],
                    observable: backgroundSettings.repeatFailedUploadAttempts,
                    defaultValue: 0,
                    conversionFunc: parseInt
                });

                backgroundUploadingPropsObservable(backgroundSettings);
            });
        });
    }

    /**
     * Сохранение настроек отложенной загрузки
     * @param {} batchType Объект класса BatchType
     * @param {} bgUploadProps Объект класса BackgroundSettings
     * @returns {} 
     */
    function setBackgroundUploadingProps(batchType, bgUploadProps) {
        executeSuccessContinue('Workspace_KeepBatchType',
            [batchType.Index()],
            function(keepBatchTypeId) {
                executeSuccessContinue('BatchType_KeepBackgroundUploadingSettings',
                    [keepBatchTypeId],
                    function (keepBackgroundUploadOptsId) {
                        var numOfOPerations = 0;
                        var commitFunction = function() {
                            numOfOPerations++;
                            if (numOfOPerations >= 9) {
                                scanModuleCommunicator
                                    .executeAsync('Workspace_CommitBatchTypeChanges', [batchType.Index()]);
                            }
                        };
                        scanModuleCommunicator.executeAsync('BackgroundUploadingSettings_setDontSpecifyForBatchType',
                        [keepBackgroundUploadOptsId, !bgUploadProps.useSpec()], commitFunction);
                        scanModuleCommunicator
                            .executeAsync('BackgroundUploadingSettings_setForbidOperatorToChangeForBatch',
                            [keepBackgroundUploadOptsId, true], commitFunction);
                        scanModuleCommunicator.executeAsync('BackgroundUploadingSettings_setRemoveUploadedLocalBatch',
                        [keepBackgroundUploadOptsId, bgUploadProps.deleteUploaded()], commitFunction);
                        scanModuleCommunicator.executeAsync('BackgroundUploadingSettings_setSpecifyUploadingTimeRange',
                        [keepBackgroundUploadOptsId, bgUploadProps.specifyUploadTime()], commitFunction);
                        scanModuleCommunicator.executeAsync('BackgroundUploadingSettings_setUploadingTimeRangeStart',
                        [keepBackgroundUploadOptsId, bgUploadProps.startUploadTime()], commitFunction);
                        scanModuleCommunicator.executeAsync('BackgroundUploadingSettings_setUploadingTimeRangeStop',
                        [keepBackgroundUploadOptsId, bgUploadProps.endUploadTime()], commitFunction);
                        scanModuleCommunicator.executeAsync('BackgroundUploadingSettings_setRepeatFailedTaskUploading',
                        [keepBackgroundUploadOptsId, bgUploadProps.repeatFailedUpload()], commitFunction);
                        scanModuleCommunicator
                            .executeAsync('BackgroundUploadingSettings_setRepeatFailedTaskUploadingPeriod',
                            [keepBackgroundUploadOptsId, bgUploadProps.repeatFailedUploadTime() * 60], commitFunction);
                        scanModuleCommunicator
                            .executeAsync('BackgroundUploadingSettings_setRepeatFailedTaskUploadingAttempts',
                            [keepBackgroundUploadOptsId, bgUploadProps.repeatFailedUploadAttempts()], commitFunction);
                    });
            });
    }

    /**
     * ПОлучение списка рег параметров из FC
     * @param {} keepBatchTypeId Объект класса BatchType
     * @param {} callbackFunction CAllback функция, с параметром список рег параметров сервера (Объекты класса RegParameter)
     * @returns {} 
     */
    function getServerRegParameters(keepBatchTypeId, callbackFunction) {
        executeSuccessContinue('BatchType_GetServerRegParamCount',
            [keepBatchTypeId,0],
            function (serverParamsCount) {
                var serverRegParams = [];
                var serverRegParamsCount = parseInt(serverParamsCount);
                if (serverRegParamsCount === 0) {
                    callbackFunction([]);
                }
                var numOfCallbacks = 0;
                var paramReceivedCallback = function() {
                    numOfCallbacks ++;
                    if (numOfCallbacks === serverRegParamsCount * 2) {
                        callbackFunction(serverRegParams);
                    }
                }

                for (var i = 0; i < parseInt(serverParamsCount); i++) {
                    var serverRegParameter = new datacontext.RegParameter();
                    serverRegParameter.IsServerParameter(true);
                    serverRegParams.push(serverRegParameter);

                    getPluginValue({
                        methodName: 'BatchType_GetServerRegParamName',
                        params: [keepBatchTypeId, 0, i],
                        observable: serverRegParameter.Name,
                        defaultValue: '',
                        callbackFunction: paramReceivedCallback
                    });

                    getPluginValue({
                        methodName: 'BatchType_GetServerRegParamIsRequired',
                        params: [keepBatchTypeId, 0, i],
                        observable: serverRegParameter.IsRequired,
                        defaultValue: '',
                        conversionFunc: function (val) { return val === '1'; },
                        callbackFunction: paramReceivedCallback
                    });
                    
                }
            });
    }


	function loadServerRegParams(batchType, callbackFunction) {
		executeSuccessContinue('Workspace_KeepBatchType',
			[batchType.Index()],
			function (keepBatchTypeId) {
				getServerRegParameters(keepBatchTypeId, callbackFunction);
			});
	}
    /**
     * Синхронизация регистрационных параметров с FC. 
     * Удаление параметров, которые были удалены или переименованы в FC, 
     * добавление ранее несуществующих
     * 
     * @param {} batchType Объект класса BatchType
     * @param {} callbackFunction Функция, вызывающаяся после синхронизации.
     * @returns {} 
     */
    function synchronizeRegParameters(batchType, callbackFunction) {
        executeSuccessContinue('Workspace_KeepBatchType',
            [batchType.Index()],
            function(keepBatchTypeId) {
                getServerRegParameters(keepBatchTypeId,
                    function(serverRegParamsList) {
                        var regParamsMap = $.map(serverRegParamsList,
                            function(value) { return { regParam: value, found: false } });

                        executeSuccessContinue('BatchType_KeepRegParamsOptions',
                            [keepBatchTypeId],
                            function(keepRpOptsId) {
                                executeSuccessContinue('RegParamsOptions_KeepBatchRegParams',
                                    [keepRpOptsId],
                                    function(keepRPsId) {
                                        executeSuccessContinue('RegistrationParameters_GetAllItemInfos',
                                            [keepRPsId],
                                            function(allRpsInfo) {
                                                
                                                var numOfDeletedParams = 0;
                                                var regParamsSerialized = allRpsInfo?allRpsInfo.split(';'):[];
                                                var numOfNewAdded = 0;
                                                var addNewServerRegParams = function(index) {
                                                    if (index >= regParamsMap.length) {
                                                        scanModuleCommunicator
                                                            .executeAsync('Workspace_CommitBatchTypeChanges',
                                                            [batchType.Index()], callbackFunction);
                                                        return;
                                                    }

                                                    var regParamPair = regParamsMap[index];
                                                    if (regParamPair.found) {
                                                        addNewServerRegParams(index + 1);
                                                        return;
                                                    }
                                                    var numOfActualParams =
                                                        regParamsSerialized.length - numOfDeletedParams + numOfNewAdded;
                                                    executeSuccessContinue('RegistrationParameters_InsertAt',
                                                        [keepRPsId, numOfActualParams, regParamPair.regParam.Name()],
                                                        function() {
                                                            executeSuccessContinue('RegistrationParameters_KeepItem',
                                                                [keepRPsId, numOfActualParams],
                                                                function(keepRpId) {
                                                                    scanModuleCommunicator
                                                                        .executeAsync('RegistrationParameter_setIsRequired',
                                                                        [keepRpId, regParamPair.regParam.IsRequired()]);
                                                                    scanModuleCommunicator
                                                                        .executeAsync('RegistrationParameter_setComment',
                                                                        [keepRpId, 1]);
                                                                    addNewServerRegParams(index + 1);
                                                                });
                                                        });
                                                }

                                                var checkLocalRegParam = function(index) {
                                                    if (index >= regParamsSerialized.length) {
                                                        addNewServerRegParams(0);
                                                        return;
                                                    }
                                                    var regParamAttrs = regParamsSerialized[index].split(':');
                                                    var name = scanModuleCommunicator.decodeText(regParamAttrs[0]);
                                                    var defaultValue = scanModuleCommunicator
                                                        .decodeText(regParamAttrs[1]);
                                                    var serverParamSymbol = scanModuleCommunicator
                                                        .decodeText(regParamAttrs[4]);
                                                    var isServerParam =
                                                        serverParamSymbol == null ? false : serverParamSymbol === '1';

                                                    var currentRegParameterOnServer;
                                                    for (var i = 0; i < regParamsMap.length; i++) {
                                                        if (regParamsMap[i].regParam.Name() === name) {
                                                            currentRegParameterOnServer = regParamsMap[i];
                                                            break;
                                                        }
                                                    }

                                                    if (currentRegParameterOnServer == undefined) {
                                                        if (isServerParam) {
                                                            scanModuleCommunicator
                                                                .executeAsync('RegistrationParameters_DeleteAt',
                                                                    [keepRPsId, index - numOfDeletedParams],
                                                                    function() {
                                                                        numOfDeletedParams++;
                                                                        checkLocalRegParam(index + 1);
                                                                    });
                                                            return;
                                                        } else {
                                                            checkLocalRegParam(index + 1);
                                                            return;
                                                        }
                                                    } else {
                                                        currentRegParameterOnServer.found = true;
                                                        executeSuccessContinue('RegistrationParameters_KeepItem',
                                                            [keepRPsId, index - numOfDeletedParams],
                                                            function(keepRpId) {
                                                                scanModuleCommunicator
                                                                    .executeAsync('RegistrationParameter_setIsRequired',
                                                                    [
                                                                        keepRpId, currentRegParameterOnServer.regParam
                                                                        .IsRequired()
                                                                    ]);
                                                                scanModuleCommunicator
                                                                    .executeAsync('RegistrationParameter_setComment',
                                                                    [keepRpId, 1]);
                                                                if (
                                                                    (defaultValue == null || defaultValue.length === 0
                                                                        ) &&
                                                                        currentRegParameterOnServer.regParam
                                                                        .IsRequired()) {
                                                                    scanModuleCommunicator
                                                                        .executeAsync('RegistrationParameter_setIsReadOnly',
                                                                        [keepRpId, false]);
                                                                }
                                                                checkLocalRegParam(index + 1);
                                                            });
                                                    }
                                                };
                                                checkLocalRegParam(0);
                                            });
                                    });
                            });
                    });
            });
    }
    
    /**
     * Получение списка регистрационных параметров для Типа пакета и типа объекта, к которым они привязываются
     * @param {} batchType тип пакета
     * @param {} regParamsTarget тип рег параметров (рег параметры пакета / рег параметры документов)
     * @param {} regParamsObservable список рег параметров (ObservableArray)
     * @param {} callbackFunction 
     * @returns {} 
     */
    function getAllRegParameters(batchType, regParamsTarget, regParamsObservable, callbackFunction) {
        executeSuccessContinue('Workspace_KeepBatchType', [batchType.Index()],function(keepBatchTypeId) {
            var getRegParams = function () {
                var result = [];
                executeSuccessContinue('BatchType_KeepRegParamsOptions', [keepBatchTypeId], function(keepRpOptsId) {
                    var methodName='';
                    switch (regParamsTarget) {
                        case datacontext.ParameterTarget.BATCH:
                            methodName = 'RegParamsOptions_KeepBatchRegParams';
                            break;
                        case datacontext.ParameterTarget.DOCUMENT:
                            methodName = 'RegParamsOptions_KeepDocumentRegParams';
                            break;
                    }
                    executeSuccessContinue(methodName,
                        [keepRpOptsId],
                        function(keepRpsId) {
                            executeSuccessContinue('RegistrationParameters_GetAllItemInfos',
                                [keepRpsId],
                                function (allRegParamsResultInfo) {
                                    if (allRegParamsResultInfo && allRegParamsResultInfo.length > 0) {
                                        $.each(allRegParamsResultInfo.split(';'),
                                            function(key, value) {
                                                var regParamAttrs = value.split(':');
                                                var name = scanModuleCommunicator.decodeText(regParamAttrs[0]);
                                                var defaultValue = new datacontext
                                                    .MaskedStringValue(scanModuleCommunicator
                                                        .decodeText(regParamAttrs[1]));
                                                var isRequired = regParamAttrs[2] === '1';
                                                var isReadOnly = regParamAttrs[3] === '1';
                                                var isServerParam =
                                                    scanModuleCommunicator.decodeText(regParamAttrs[4]) === '1';
                                                var items = [];
	                                            for (var i = 6; i < regParamAttrs.length; i++) {
		                                            items.push(new datacontext
			                                            .MaskedStringValue(scanModuleCommunicator
				                                            .decodeText(regParamAttrs[i])));
	                                            }
	                                            var parameterType = items.length > 0
                                                    ? datacontext.ParameterType.SELECT
                                                    : datacontext.ParameterType.TEXT;

                                                var currentRegParameter = new datacontext.RegParameter();
                                                currentRegParameter.Name(name);
                                                result.push(currentRegParameter);

                                                currentRegParameter.DefaultValue(defaultValue);
                                                currentRegParameter.IsServerParameter(isServerParam);
                                                currentRegParameter.IsRequired(isRequired);
                                                currentRegParameter.IsReadOnly(isReadOnly);
                                                currentRegParameter.ParameterType(parameterType);
                                                currentRegParameter.ListValues(items);
                                            });
                                    }
                                    regParamsObservable(result);
                                    if (callbackFunction) {
                                        callbackFunction(regParamsObservable);
                                    }
                                });
                        });
                });
            }
            if (regParamsTarget === datacontext.ParameterTarget.BATCH) {
                synchronizeRegParameters(batchType, getRegParams);
            } else {
                getRegParams();
            }
        });
    }

    /**
     * Обновление свойств имеющихся рег параметров
     * @param {} actualRegParameters последние сохраненные значения рег параметры
     * @param {} currentRegParameters текущие значения рег параметров
     * @param {} keepRegParamsId id Объекта рег параметров текущего типа в плагине
     * @param {} commitFunction Функция сохранения изменений
     * @param {} callbackFunction Callback функция после успешного выполнения обновления
     * @returns {} 
     */
    function updateRegParamerters(actualRegParameters, currentRegParameters, keepRegParamsId, commitFunction, callbackFunction) {
        var numOfDeletedItems = 0;
        
        var checkRegParameter = function(rpindex) {
            if (rpindex < 0) {
                callbackFunction(numOfDeletedItems);
                return;
            }

            var actualRegParameter = actualRegParameters[rpindex];
            var currentRelatedRegParam = null;
            for (var j = 0; j < currentRegParameters.length; j++) {
                if (currentRegParameters[j].Name() === actualRegParameter.Name()) {
                    currentRelatedRegParam = currentRegParameters[j];
                    break;
                }
            }

            if (currentRelatedRegParam == null) {
                scanModuleCommunicator.executeAsync('RegistrationParameters_DeleteAt',
                    [keepRegParamsId, rpindex],
                    function() {
                        numOfDeletedItems++;
                        commitFunction();
                        checkRegParameter(rpindex - 1);
                    });

                return;
            }
            executeSuccessContinue('RegistrationParameters_KeepItem',
                [keepRegParamsId, rpindex],
                function(keepRegParamId) {
                    var defValue = currentRelatedRegParam.DefaultValue();

                    if ((defValue == undefined || defValue.length === 0) &&
                    (currentRelatedRegParam.ParameterType() == datacontext.ParameterType.SELECT) &&
                    (currentRelatedRegParam.ListValues() != undefined) &&
                    (currentRelatedRegParam.ListValues().length > 0)) {
                        defValue = currentRelatedRegParam.ListValues()[0];
                    }

                    if (defValue.Value() !== actualRegParameter.DefaultValue().Value()) {
                        scanModuleCommunicator.executeAsync('RegistrationParameter_setValue',
                        [keepRegParamId, defValue.Value()], commitFunction);
                    }

                    if (currentRelatedRegParam.IsRequired() !== actualRegParameter.IsRequired()) {
                        scanModuleCommunicator.executeAsync('RegistrationParameter_setIsRequired',
                        [keepRegParamId, currentRelatedRegParam.IsRequired()], commitFunction);
                    }


                    
                    scanModuleCommunicator.executeAsync('RegistrationParameter_setIsReadOnly',
						[keepRegParamId, (currentRelatedRegParam.ParameterType() === datacontext.ParameterType.TEXT) && currentRelatedRegParam.IsReadOnly()], commitFunction);
                    


                    executeSuccessContinue('RegistrationParameter_getListValueCount',
                        [keepRegParamId],
                        function(itemsCount) {
                            var listValues = currentRelatedRegParam.ListValues();

                            var addParamsFunc = function(index) {
                                if (listValues.length <= index) return;

                                scanModuleCommunicator.executeAsync('RegistrationParameter_InsertListValueItemAt',
                                    [keepRegParamId, index, listValues[index].Value()],
                                    function () {
                                        commitFunction();
                                        addParamsFunc(index + 1);
                                        
                                    });
                            };

                            var removeFunc = function(index) {
                                if (index < 0) {
                                    if (currentRelatedRegParam.ParameterType() == datacontext.ParameterType.SELECT) {
                                        addParamsFunc(0);
                                    }
                                    return;
                                }

                                scanModuleCommunicator.executeAsync('RegistrationParameter_DeleteListValueItemAt',
                                    [keepRegParamId, index],
                                    function () {
                                        commitFunction();
                                        removeFunc(index - 1);
                                    });
                            }

                            removeFunc(parseInt(itemsCount) - 1);
                        });
                    checkRegParameter(rpindex - 1);
                });
        }
        checkRegParameter(actualRegParameters.length - 1);
    }

    /**
     * Добавление новых рег параметров
     * @param {} actualRegParameters последние сохраненные значения рег параметры
     * @param {} currentRegParameters текущие значения рег параметров
     * @param {} keepRegParamsId id Объекта рег параметров текущего типа в плагине
     * @param {} insertPoint индекс вставки новых рег параметров
     * @param {} callbackFunction Callback функция после успешного добавления новых рег параметров
     * @returns {} 
     */
    function addNewRegParameters(actualRegParameters, currentRegParameters, keepRegParamsId, insertPoint, callbackFunction) {
        var checkRpFunction = function (index) {
            if (index >= currentRegParameters.length) {
                callbackFunction();
                return;
            }

            var regParamFound = false;
            var currentRelatedRegParam = currentRegParameters[index];
            for (var j = 0; j < actualRegParameters.length; j++) {
                if (currentRelatedRegParam.Name() === actualRegParameters[j].Name()) {
                    regParamFound = true;
                    break;
                }
            }
            if (regParamFound) {
                checkRpFunction(index + 1);
            }
            else {
                executeSuccessContinue('RegistrationParameters_InsertAt', [keepRegParamsId, insertPoint, currentRelatedRegParam.Name()],function() {
                    executeSuccessContinue('RegistrationParameters_KeepItem',
                        [keepRegParamsId, insertPoint],
                        function (keepRegParamId) {
                            var defValue = currentRelatedRegParam.DefaultValue();
                            scanModuleCommunicator.executeAsync('RegistrationParameter_setValue', [keepRegParamId, defValue.Value()]);
                            scanModuleCommunicator.executeAsync('RegistrationParameter_setIsRequired', [keepRegParamId, currentRelatedRegParam.IsRequired()]);
							scanModuleCommunicator.executeAsync('RegistrationParameter_setIsReadOnly', [keepRegParamId, (currentRelatedRegParam.ParameterType() === datacontext.ParameterType.TEXT) && currentRelatedRegParam.IsReadOnly()]);
                            if (currentRelatedRegParam.ParameterType() == datacontext.ParameterType.SELECT) {
                                $.each(currentRelatedRegParam.ListValues(), function (key, value) {
                                    scanModuleCommunicator.executeAsync('RegistrationParameter_InsertListValueItemAt', [keepRegParamId, key, value.Value()]);
                                });
                            }
                            insertPoint++;
                            checkRpFunction(index + 1);
                        });
                });
                
            }
        }
        checkRpFunction(0);
    }

    /**
     * Сетод сохранения рег параметров 
     * @param {} batchType Тип пакета
     * @param {} currentRegParameters списк рег параметров 
     * @param {} regParamsTarget тип рег параметров (рег параметры пакета / рег параметры документов)
     * @returns {} 
     */
    function saveRegParameters(batchType, currentRegParameters, regParamsTarget) {
        var actualRegParameters = ko.observable();
        var commitFunction = function() {
            scanModuleCommunicator.executeAsync('Workspace_CommitBatchTypeChanges', [batchType.Index()]);
        };
        getAllRegParameters(batchType,
            regParamsTarget,
            actualRegParameters,
            function(actualRegParameters) {
                executeSuccessContinue('Workspace_KeepBatchType',
                    [batchType.Index()],
                    function(keepBatchTypeId) {
                        executeSuccessContinue('BatchType_KeepRegParamsOptions',
                            [keepBatchTypeId],
                            function(keepRpOptsId) {
                                executeSuccessContinue('RegParamsOptions_setDontSpecifyForBatchType',
                                    [keepRpOptsId, 'false'],
                                    function() {
                                        var methodName = '';
                                        switch (regParamsTarget) {
                                        case datacontext.ParameterTarget.BATCH:
                                            methodName = 'RegParamsOptions_KeepBatchRegParams';
                                            break;
                                        case datacontext.ParameterTarget.DOCUMENT:
                                            methodName = 'RegParamsOptions_KeepDocumentRegParams';
                                            break;
                                        }

                                        executeSuccessContinue(methodName,
                                            [keepRpOptsId],
                                            function(keepRegParamsId) {

                                                updateRegParamerters(actualRegParameters(),
                                                    currentRegParameters,
                                                    keepRegParamsId,
                                                    commitFunction,
                                                    function(numberOfDeletedItems) {
                                                        var regParamInsertIndex =
                                                            actualRegParameters().length - numberOfDeletedItems;
                                                        addNewRegParameters(actualRegParameters(),
                                                            currentRegParameters,
                                                            keepRegParamsId,
                                                            regParamInsertIndex,
                                                            commitFunction);
                                                    });

                                            });
                                    });
                            });
                    });
            });
    }


	function getRegParamsForbid(batchType, callbackFunction) {
		executeSuccessContinue('Workspace_KeepBatchType',
			[batchType.Index()],
			function(keepBatchTypeId) {
				executeSuccessContinue('BatchType_KeepRegParamsOptions',
					[keepBatchTypeId],
					function (keepRpOptsId) {
						getPluginValue({
							methodName: 'RegParamsOptions_getForbidOperatorToChangeForBatch',
							params: [keepRpOptsId],
							observable: callbackFunction,
							defaultValue: false,
							conversionFunc: function (val) { return val === '1' }
						});
					});
			});
	}

	function setRegParamsForbid(batchType, value) {
		executeSuccessContinue('Workspace_KeepBatchType',
			[batchType.Index()],
			function (keepBatchTypeId) {
				executeSuccessContinue('BatchType_KeepRegParamsOptions',
					[keepBatchTypeId],
					function (keepRpOptsId) {
						scanModuleCommunicator.executeAsync('RegParamsOptions_setForbidOperatorToChangeForBatch',[keepRpOptsId, value]);
					});
			});
	}
    
    return datacontext;
})();

/** Запрещенные к использованию в именах пакета и документа символы */
window.batchtypeSettings.DeniedNameCharsSimple = new RegExp('[|\\\\/?:*]', 'g'); // Для BTS