﻿window.Layouts = window.Layouts || {};

/**
 * СЛой получения данных для шапок
 */
window.Layouts.datacontext = (function() {

	var datacontext = {
		getCurrentUser: getCurrentUser,
		logout: logout,
		getLogoutPath: getLogoutPath,
        getHelpUrl: getHelpUrl,
        getLogoPath: getLogoPath,
        getLogoAlt: getLogoAlt,
        getAccSettingsPath: getAccSettingsPath,
        getSelectProjectPath: getSelectProjectPath,
        getCurrentProjectName: getCurrentProjectName,
        getProjectsInfo: getProjectsInfo,
        getProjectUrl: getProjectUrl,
        getCurrentBatchName: getCurrentBatchName,
        allProjectsUrl: allProjectsUrl,
        allBatchesUrl: allBatchesUrl,
        setProject: setProject,
        beforeShowAllProjects: beforeShowAllProjects
    };

    return datacontext;

    /**
     * Функция получения информации о текущем пользователе
     * @param {} currentUserNameObservable observable переменная для записи результата
     * @returns {} Объект ajax
     */
    function getCurrentUser(currentUserNameObservable) {
        return ajaxRequest("get", currentUserPath())
            .done(function(data) {
                currentUserNameObservable(data);
            });
    }

    /**
     * Функция получения информации о текущем проекте
     * @param {} currentProjectNameObservable observable переменная для записи результата
     * @returns {} Объект ajax
     */
    function getCurrentProjectName(currentProjectNameObservable) {

        return ajaxRequest("get", projectsInfoPath(), { projectGuid: $("#projectGuid").val() })
            .done(function(data) {
                var currentProjectGuid = $("#projectGuid").val();
                $.each(data,
                    function(key, value) {
                    	if (value.Guid.toLowerCase() === currentProjectGuid.toLowerCase()) currentProjectNameObservable(value.Name);
                    });
            });
    }

    /**
     * Функция получения списка всех проектов
     * @param {} projectsObservable observable переменная для записи результата
     * @returns {} Объект ajax
     */
    function getProjectsInfo(projectsObservable) {
        return ajaxRequest("get", projectsInfoPath(), { projectGuid: $("#projectGuid").val() })
            .done(function(data) {
                projectsObservable(
                    $.map(data,
                        function(item) {
                            return new datacontext.ProjectInfo(item);
                        }));
            });
    }

    /**
     * Функция получения информации о текущем пакете
     * @param {} batchNameObservable observable переменная для записи результата
     * @returns {} Объект ajax
     */
    function getCurrentBatchName(batchNameObservable) {
        window.batchApp.datacontext.getBatchName(batchNameObservable);
    }

    /**
     * Получение URL для перехода к проекту
     * @param {} project Объект ProjectInfo
     * @returns {} 
     */
    function getProjectUrl(project) {
        return project.Url;
    }

    function setProject(project) {
	    //scanModuleCommunicator.removeSessionId();
    	window.location = project.Url;
    	
    }

    function beforeShowAllProjects() {
    	//scanModuleCommunicator.removeSessionId();
    }

    /**
     * Служебная функция для ajax запроса
     * @param {} type тип запроса get/post
     * @param {} url URL Запроса
     * @param {} data параметры запроса
     * @param {} dataType тип данных
     * @returns {} объект ajax
     */
    function ajaxRequest(type, url, data, dataType) { // Ajax helper
        var options = {
            dataType: dataType || "json",
            contentType: "application/json",
            cache: false,
            type: type,
            data: data
        };
        var antiForgeryToken = $("#antiForgeryToken").val();
        if (antiForgeryToken) {
            options.headers = {
                'RequestVerificationToken': antiForgeryToken
            }
        }
        return $.ajax(url, options);
    }


    /**
     * @returns {} URL web api получения информации о пользователе
     */
    function currentUserPath() { return FC.Environment.AppPath + "Login/CurrentUserName"; }

    /**
     * @returns {} URL для выполнения logoff
     */
	function getLogoutPath() { return FC.Environment.AppPath + "Login/Logoff"; }

	function logout() { return FC.Utils.replaceLocation(getLogoutPath()); }

    /**
     * @returns {} URL Вызова справки
     */
    function getHelpUrl() {
    	return FC.Environment.Localization.GlobalResources.HelpPath_Scanning;
    }

    /**
     * @returns {} URL логотипа
     */
    function getLogoPath() {
    	return FC.Environment.AppPathNoTennant + FC.Environment.Localization.NonLocalizedResources.ScanningStationLogoImagePath;
    }

    /**
     * @returns {} Текст вместо логотипа
     */
    function getLogoAlt() { return ''; }

    /**
     * @returns {}  ссылку на личный кабинет
     */
	function getAccSettingsPath() {
		var tennant = FC.Environment.TennantName;
		if (FC.Environment.AppPath.toLowerCase().indexOf("winauth") > 0) {
			tennant = "WinAuth/" + tennant;
		}

        return FC.Environment.AppPathNoTennant +
            FC.Environment.Localization.NonLocalizedResources.AccSettingsUrl
			.replace("{Tennant}", tennant);
    }

    /**
     * @returns {} URL открытия проекта
     */
    function getSelectProjectPath() { return FC.Environment.AppPath + "Project"; }

    /**
     * @returns {}  URL WebApi получения списка всех проектов
     */
    function projectsInfoPath() { return FC.Environment.AppPath + "Project/GetAllProjects"; }

    /**
     * @returns {} URL страницы выбора проекта
     */
    function allProjectsUrl() { return FC.Environment.AppPath; }

    /**
     * @returns {} URL страницы выбора пакета
     */
	function allBatchesUrl() {
		var mode = FC.Utils.getParameter("mode");

		if (FC.Environment.StationType === FC.StationType.WebCapture) {
			var projectId = $("#projectId").val();

			return FC.Environment.AppPath + "Main/Index?projectId=" + projectId +
				(mode && mode.toLowerCase() === "mini" ? "&mode=mini" : "");
		}

		return FC.Environment.AppPath + "Batches/Index/" + $("#projectGuid").val() +
			(mode && mode.toLowerCase() === "mini" ? "?mode=mini" : "");
	}

})();