﻿
var PROXY_ERROR_CODE = 8;

/**
 * Модуль для работы с плагином сканирования
 */
window.scanModuleCommunicator = (function () {
    /**
     * Флаг доступности плагина
     */
    var isAvailable = true;

    /**
     * Время последнего пользовательского действия. 
     * Используется для определения времени простоя
     */
    var lastActionTime = new Date();

    /**
     * Получение данных для ининицализации плагина сканирования
     * @param {} projectGuid Уникальный идентификатор проекта
     * @param {} callbackFunction 
     * @returns {} объект Ajax
     */
    function getDataFromServer(projectGuid, errorsManager, callbackFunction) {
        if (!projectGuid) {
            errorsManager.Show("projectGuid not set");
        }

        $.ajax({
            url: FC.Environment.AppPath + 'ScanPlugin/StartPluginSessionInfo',
            cache: false,
            data: {
                projectGuid: projectGuid
            },
            success: function (data) {
                if (data.ErrorType != undefined) {
                    if ((data.ErrorType === 303) && (data.Url != undefined)) {
                        location.replace(data.Url);
                    }
                }
                callbackFunction(data);
            }
        });
    }

    /**
     * Функция редиректа на основную страницу. 
     * Вызывается в случае ошибок плагина, которые не позволяют работать дальше
     * @returns {} 
     */
    function defaultRedirectFunc() {
        var url = FC.Environment.AppPath;
        var query = location.search.toLowerCase();
        if (query && query.indexOf('mode=mini') > -1) {
            url += "?mode=mini";
        }

        location.replace(url);
    }

    /**
     * BASE64 decode по версии плагина сканирования
     * @param {} str base64 encode строка
     * @returns {} декодированная строка
     */
    function decodeText(str) {
        if (!str) {
            return "";
        }
        var b64Chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
        var b64Decoded = '';
        var chr1, chr2, chr3;
        var enc1, enc2, enc3, enc4;

        str = str.replace(/[^a-z0-9\+\/\=]/gi, '');

        for (var i = 0; i < str.length;) {
            enc1 = b64Chars.indexOf(str.charAt(i++));
            enc2 = b64Chars.indexOf(str.charAt(i++));
            enc3 = b64Chars.indexOf(str.charAt(i++));
            enc4 = b64Chars.indexOf(str.charAt(i++));

            chr1 = (enc1 << 2) | (enc2 >> 4);
            chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
            chr3 = ((enc3 & 3) << 6) | enc4;

            b64Decoded = b64Decoded + String.fromCharCode(chr1);

            if (enc3 < 64) {
                b64Decoded += String.fromCharCode(chr2);
            }
            if (enc4 < 64) {
                b64Decoded += String.fromCharCode(chr3);
            }
        }
        return decodeURIComponent(escape(b64Decoded));
    }

    /**
     * Url плагина
     */
    var moduleUrl = ko.observable(localStorage.getItem('moduleAddress'));

    /**
     * Id текущей сессии из Local storage. 
     * Туда записывается при успешной инициализации плагина
     */
    var sessId = localStorage.getItem('moduleSessId');
    if (sessId === 'null')
        sessId = null;

    /**
     * Текущий Session Id
     */
    var sessionId = ko.observable(sessId);

    /**
     * Session Id по версии WebService wsdl
     */
    var serverSessionId = localStorage.getItem("serverSessId", serverSessionId);
    if (!serverSessionId || serverSessionId.length == 0) {
        serverSessionId = -1;
    }
    else {
        serverSessionId = parseInt(serverSessionId);
    }

    /**
     * Список запросов. 
     * Не обрабатывается пока не произведена полноценная инициализация плагина
     */
    var requests = ko.observableArray([]);

    /**
     * Деинициализация плагина, когда закрывается страница
     */

    var closeSession = function () {
        if ((moduleUrl() !== null) && (sessionId() !== null)) {

            // Объяснение параметра _ см. в методе dropSession
            executeOnUnload(moduleUrl() + 'Workspace_Deinitialize?' + sessionId() + "&_");
            executeOnUnload(FC.Environment.AppPath + 'ScanPlugin/CloseSession?sessId=' + serverSessionId);
            localStorage.removeItem('moduleSessId');
            localStorage.removeItem('serverSessId');
        }
    }

    function isIE() {
        if (typeof window === 'undefined' && typeof self === 'undefined') return true;
        const windowOrSelf = typeof window !== 'undefined' ? window : self;
        const ua = windowOrSelf.navigator.userAgent;
        return ua.indexOf('Trident') > 0;
    }

    function executeOnUnload(url) {
        // https://caniuse.com/?search=beacon%20api
        if (isIE()) {
            $.ajax({
                method: 'POST', // becaon API работает через POST
                url: url,
                cache: false,
                async: false
            });
        } else {
            const navigatorProvider = window || self;
            if (navigatorProvider) {
                navigatorProvider.navigator.sendBeacon(url);
            }
        }
    }

    (function subscribeCloseSession() {
        // https://developers.google.com/web/updates/2018/07/page-lifecycle-api#legacy-lifecycle-apis-to-avoid
        const eventDispatcher = window || self;
        const terminationEventName = 'onpagehide' in eventDispatcher ? 'pagehide' : 'unload';
        eventDispatcher.addEventListener(terminationEventName, closeSession);

        console.log('Subscribed to event', terminationEventName, 'on event dispatcher', eventDispatcher);
    })();

    /**
     * Обновление времени последнего пользвательского действия
     */
    $('body')
        .mousedown(function () {
            lastActionTime = new Date();
        });

    /**
     * Функция сериализации параметров запроса
     * @param {} params список параметров
     * @returns {} 
     */
    function generateParams(params) {
        var result = "";
        for (var i = 0; i < params.length; i++) {
            if (i !== 0) {
                result += '&';
            }
            result += encodeURIComponent(params[i]);
        }
        if ((params.length !== 0) &&
            ((params[params.length - 1] == undefined) ||
                (typeof params[params.length - 1] == 'string' && params[params.length - 1].length === 0)))
            result += '&';
        return result;
    }

    /**
     * Функция получения URL Запроса к плагину по имени метода и параметрам
     * @param {} methodName имя метода
     * @param {} methodParams параметры
     * @returns {} полный URL
     */
    function getPluginUrl(methodName, methodParams) {
        var isHttps = (location.protocol === "https:");
        var params = methodParams || [];
        if (!Array.isArray(params))
            params = [params];
        params.unshift(sessionId());
        var url = moduleUrl() + (isHttps ? 's_' : '') + methodName + '?' + generateParams(params);
        return url;
    }

    var imgCounter = 0;

    /**
     * Функция вызова метода плагина по https
     * @param {} url полный URL для плагина
     * @param {} paramsCount число параметров в запросе
     * @param {} succesCallback callback функция успешного выполнения
     * @param {} failedCallback callback функция неуспешного выполнения
     * @returns {} 
     */
    function callHttpsMethod(url, paramsCount, succesCallback, failedCallback) {
        var img = new Image;

        img.onload = function () {
            var result = "";

            try {

                var canvas = document.createElement("canvas");
                canvas.height = 1;
                var context = canvas.getContext('2d');

                canvas.width = img.width;
                context.drawImage(img, 0, 0);
                var imageData = context.getImageData(0, 0, img.width, img.height);
                var data = imageData.data;
                for (var i = 0; i < img.width; i++) {
                    var red = data[4 * i];
                    var green = data[4 * i + 1];
                    var blue = data[4 * i + 2];

                    result += String.fromCharCode(blue);
                    if (green === 0) {
                        break;
                    } else {
                        result += String.fromCharCode(green);
                    }
                    if (red === 0) {
                        break;
                    } else {
                        result += String.fromCharCode(red);
                    }
                }
            } catch (e) {
                failedCallback(e);
            }

            result = decodeURIComponent(escape(result)); // Для кириллицы
            succesCallback(result);
        }

        img.onerror = function (arg) {
            failedCallback(arg);
        };

        img.crossOrigin = 'anonymous';

        var imgSrc = url + (paramsCount > 0 ? "&" : "?") + imgCounter;
        imgCounter++;
        img.src = imgSrc;
    }

    /**
     * Форматтер ответа плагина
     */
    var resultParser = {
        //Dummy
        emptyResult: function () {
            return {
                hasErrors: false,
                status: -1,
                errorText: null,
                result: null
            };
        },
        /**
         * Функция десериализации ответа плагина
         * @param {} data сериализованный ответ
         * @returns {} десерилазованный результат
         */
        parseResult: function (data) {
            var result = resultParser.emptyResult();
            result.status = parseInt(data.substr(0, 1));
            result.hasErrors = ((result.status !== 1) && (result.status !== 2));
            if (result.hasErrors) {
                result.errorText = data.substr(1);
            } else {
                result.result = data.substr(1);
            }
            return result;
        },
        /**
         * Функция формирования ответа плагина на основе текста ошибки
         * @param {} data текст ошибки
         * @returns {}  результат
         */
        moduleFailedResult: function (data) {
            var result = resultParser.emptyResult();
            result.hasErrors = true;

            try {
                result.errorText = decodeText(data);
            } catch (e) {
                result.errorText = 'unknown error';
            }

            return result;
        }
    }

    /**
     * Класс запроса
     */
    var Request = (function (sessionId, moduleUrl, requests, urlGenerator, httpsSender, resultParser) {
        var nextId = 1;

        return function Request(requestInfo) {
            var self = this;

            /**
             * Id запроса
             */
            self.id = nextId++;

            /**
             * Метод плагина
             */
            self.methodName = requestInfo.methodName;

            /**
             * Параметры запроса
             */
            self.methodParams = requestInfo.methodParams;

            /**
             * Функция вызываемая после получения ответа от плагина
             */
            self.callbackFunction = requestInfo.callbackFunction;

            /**
             * Флаш начала выполнения запроса
             */
            self.started = ko.observable(false);

            /**
             * Функция выполнения запроса по Https
             * @returns {} 
             */
            var processHttps = function () {

                httpsSender(
                    urlGenerator(self.methodName, self.methodParams),
                    (self.methodParams ? self.methodParams.length : 0),
                    function (result) {
                        result = resultParser.parseResult(result);
                        if ((result.hasErrors) && (result.status === 7)) {
                            sessionId(null);
                            self.started(false);
                        } else {
                            for (var i = 0; i < requests().length; i++) {
                                if (requests()[i].id === self.id) {
                                    requests.splice(i, 1);
                                    break;
                                }
                            }

                            if (self.callbackFunction != undefined) {
                                self.callbackFunction(result);
                            }

                        }
                    },
                    function (err) {
                        if (self.callbackFunction != undefined) {
                            self.callbackFunction(resultParser.moduleFailedResult(err));
                        }
                    });
            };

            /**
             * Функция выполнения запроса по Http
             * @returns {} 
             */
            var processHttp = function () {
                $.ajax({
                    url: urlGenerator(self.methodName, self.methodParams),
                    cache: false,
                    success: function (data) {
                        var result = resultParser.parseResult(data);
                        if ((result.hasErrors) && (result.status === 7)) {
                            sessionId(null);
                            self.started(false);
                        } else {
                            for (var i = 0; i < requests().length; i++) {
                                if (requests()[i].id === self.id) {
                                    requests.splice(i, 1);
                                    break;
                                }
                            }
                            if (self.callbackFunction != undefined) {
                                self.callbackFunction(result);
                            }

                        }
                    },
                    error: function (jqXhr, textStatus, errorThrown) {
                        var result = resultParser.moduleFailedResult(errorThrown);
                        if (self.callbackFunction != undefined) {
                            self.callbackFunction(result);
                        }
                    }
                });
            };

            /**
             * Запуск выполнения запроса
             * @returns {} 
             */
            self.process = function () {
                if (location.protocol === "https:") {
                    processHttps();
                } else {
                    processHttp();
                }
            };

        }
    })(sessionId, moduleUrl, requests, getPluginUrl, callHttpsMethod, resultParser);

    /**
     * Функция запуска необработанных запросов
     * @returns {} 
     */
    function processRequests() {
        $.each(requests(),
            function (key, value) {
                if (!value.started()) {
                    value.started(true);
                    value.process();
                }
            });
    }


    /**
     * Запуск выполнения необработанных запросов
     */
    sessionId.subscribe(function (val) {
        if (val == null)
            return;

        localStorage.setItem('moduleSessId', val);
        if (moduleUrl() != null) {
            if (requests().length > 0) {
                processRequests();
            }
        }
    });

    /**
     * Запуск выполнения необработанных запросов
     */
    moduleUrl.subscribe(function (val) {
        if (val == null)
            return;

        localStorage.setItem('moduleAddress', val);
        if (sessionId() != null) {
            if (requests().length > 0) {
                processRequests();
            }
        }
    });

    /**
     * Запуск выполнения необработанных запросов
     */
    requests.subscribe(function () {
        if (sessionId() != null && moduleUrl() != null) {
            processRequests();
        }
    });


    /**
     * Служебная функция получения id сессии
     * @param {} moduleInitInfo информация для вызова метода плагина
     * @param proxyDialog диалоговое окно для ввода данных авторизации в прокси
     * @param errorsManager
     * @param {} callbackFunction функция вызвываемая при удачном получении id сессии
     * @returns {}
     */
    function getSessionsId(moduleInitInfo, proxyDialog, errorsManager, callbackFunction) {
        var errorFunc = function (data) {
            isAvailable = false;
            if (data.status === PROXY_ERROR_CODE && moduleInitInfo.IsScanPlaginProxyRequired) {
                proxyDialog.showDialog(function (proxyData, reloadCallback) {
                    sendProxyData(proxyData, errorsManager, reloadCallback)
                });
            } else {
                errorsManager.Alert(data.errorText, defaultRedirectFunc);
            }
        };

        var successFunc = function (data) {
            var result = resultParser.parseResult(data);
            if (result.hasErrors) {
                errorFunc(result);
            } else {
                var ids = data.substr(1).split(":");
                sessionId(ids[0]);
                serverSessionId = ids[1];
                localStorage.setItem("serverSessId", serverSessionId);
                setInterval(function () { checkLicenseFunction(errorsManager); }, 600000);
                if (callbackFunction != undefined) {
                    callbackFunction();
                }
            }
        };
        var initInfo = moduleInitInfo.InitInfo.replace('localhost', window.location.hostname);

        if (location.protocol === "https:") {
            callHttpsMethod(moduleUrl() + 's_init?' + initInfo, 1, successFunc, errorFunc);
        } else {
            $.ajax({
                url: moduleUrl() + 'init?' + initInfo,
                cache: false,
                success: successFunc
            });
        }
    }

    /**
     *  Служебная функция для отправки данных авторизации для прокси
     * @param proxyData Данные для авторизации в прокси
     * @param errorsManager
     * @param reloadCallback  Функция вызываемая после установки параметров
     */
    function sendProxyData(proxyData, errorsManager, reloadCallback) {
        var errorFunc = function (data) {
            isAvailable = false;
            errorsManager.Alert(data.errorText, defaultRedirectFunc);
        };

        var successFunc = function (data) {
            var result = resultParser.parseResult(data);
            if (result.hasErrors) {
                errorFunc(result);
            }
            if (reloadCallback != undefined) {
                reloadCallback();
            }

        };

        var requestData = window.btoa(proxyData.username + ":" + proxyData.password);

        if (location.protocol === "https:") {
            callHttpsMethod(moduleUrl() + 's_SetProxyCredentials?' + requestData, 1, successFunc, errorFunc);
        } else {
            $.ajax({
                url: moduleUrl() + 'SetProxyCredentials?' + requestData,
                cache: false,
                success: successFunc,
                error: errorFunc
            });
        }
    }


    /**
     * Служебная функция проверки совместимости версии плагина
     * @param {} goodVersion Минимальная требуемая версия
     * @param {} callbackFunction Функция вызова при прохождении проверки
     */
    function checkPluginVersion(goodVersion, scanPluginDialog, callbackFunction) {
        var successFunc = function (data) {
            var result = resultParser.parseResult(data);
            var currentVersion = parseInt(result.result);
            setCheckNewVersionPlugin(currentVersion)

            if (currentVersion < goodVersion) {
                scanPluginDialog.showDialog(FC.Environment.Localization.Common.ObsoletePlugin);
                return;
            }

            callbackFunction();
        };
        var errorFunc = function () {
            isAvailable = false;
            scanPluginDialog.showDialog(FC.Environment.Localization.Common.PluginInstallPrompt);
        };
        if (location.protocol === "https:") {
            callHttpsMethod(moduleUrl() + 's_getVersion', 0, successFunc, errorFunc);
        } else {
            $.ajax({
                url: moduleUrl() + 'getVersion',
                cache: false,
                success: successFunc,
                error: errorFunc
            });
        }
    }

    /**
     * Функция проверки версии плагина 92 и выше
     * @returns {} 
     */

    function setCheckNewVersionPlugin(currentVersion) {
        var beforeVersion = 92;
        var isNewVersionPlugin = currentVersion >= beforeVersion;
        localStorage.setItem('isNewVersionPlugin', isNewVersionPlugin);
    }

    /**
     * Служебная функция инициализации плагина 
     * @returns {} 
     */
    function initializePlugin(projectGuid, scanPluginDialog, proxyDialog, errorsManager, callback) {
        requests.unshift(new Request({
            methodName: 'Workspace_InitScanning',
            methodParams: []
        }));


        getDataFromServer(projectGuid, errorsManager, function (moduleInitInfo) {
            moduleUrl(moduleInitInfo.Url);
            checkPluginVersion(moduleInitInfo.PluginVersion, scanPluginDialog,
                function () {
                    if ((sessionId() == null) || (moduleUrl() == null)) {
                        getSessionsId(moduleInitInfo, proxyDialog, errorsManager, callback);
                    }
                });
        });

    }

    /**
     * Внешняя функция вызова запроса
     * @param {} action вызываемое действие
     * @param {} params список параметров запроса
     * @param {} callbackFunction функуия обратного вызова
     * @returns {} none
     */
    var executeAsync = function (action, params, callbackFunction) {
        if (isAvailable) {
            requests.push(new Request({
                methodName: action,
                methodParams: params,
                callbackFunction: callbackFunction
            }));
        }
    };

    //Функция проверки лицензии. Внутренняя служебная функция
    function checkLicenseFunction(errorsManager) {
        var verifyLicenseFunc = function () {
            executeAsync('Workspace_VerifyLicense',
                [],
                function (result) {
                    if (result.hasErrors) {
                        closeSession();
                        errorsManager.Alert(result.errorText, defaultRedirectFunc);
                    }
                });
        };
        if (parseInt((lastActionTime - new Date()) / 60000) < 120) {
            executeAsync('Workspace_UpdateLicense',
                [],
                verifyLicenseFunc);
        } else {
            verifyLicenseFunc();
        }
    }



    return {
        executeAsync: executeAsync,
        dropSession: function () {
            console.log('Dropping session', sessionId());

            const removeSessionId = function() {
                localStorage.removeItem('moduleSessId');
            };

            $.ajax({
                method: 'POST',
                // Параметр _ - это костыль для корректной работы плагина сканирования
                // Дело в том, что плагин отрезает последний параметр при обработке GET-запроса, поскольку jQuery добавляет параметр _
                // с автоинкрементированным числом для того, чтобы запросы не кешировались браузером
                // jQuery делает так только для GET запросов, поэтому в POST-запросе мы сами должны руками написать этот (или любой другой незначимый) параметр
                // Через ajaxPrefilter сделать нельзя, т.к. beacon API им очевидно не пользуется, а писать глобальные интерсепторы для XHR - не самая хорошая идея
                url: getPluginUrl("Workspace_Deinitialize", '_'),
                cache: false,
                success: function(data, status, request) {
                    console.log('Session', sessionId(), 'dropped. Status:', status, '; data:', data, '; request:', request);
                    removeSessionId();
                },
                error: function (request, status, error) {
                    console.log('Error while attempting to drop session', sessionId(), '. Status:', status, '; error:', error, '; request:', request);
                    removeSessionId();
                }
            });
        },
        getPluginUrl: getPluginUrl,
        decodeText: decodeText,
        lastActionTime: lastActionTime,
        initializePlugin: initializePlugin,
        sendProxyData: sendProxyData
    };
})();