﻿var pages = { verify: 1, scanning: 2 };
var currentPage = Environment.CurrentPage; //pages.verify;

// Set to true to use predefined project and role (see below)
var predefineProjectAndRole = false;

// If users don't need to choose project and role
// define this variables
// Project Id and Guid can be found in Monitoring station (Settings/Projects)
var projectGuid = "DFDEDCD9-E338-4409-9825-2A2B93877941";
var projectId = 12;
// Simplest way to find out role id for verification is to open verification station
// without customization, select needed project and role. In queue list these ids will be found in URL
// i.e. http://localhost/FlexiCapture12/Verification/QueueList?projectId=12&roleId=139
var verificationRoleId = 139;

function getStationUrl(stationType) {
	if (stationType == pages.scanning) {
		// set url to scan station
		// you can preselect project for users with projectGuid paramenter
		// parameter mode=mini hides default Flexicapture headers and footers
		if (predefineProjectAndRole) {
			return Environment.ScanStation + "Batches?projectGuid=" + projectGuid + "&mode=mini";
		} else {
			return Environment.ScanStation + "?mode=mini";
		}
	}

	// set url to verification station
	// you can preselect project and role for users with projectId and roleId paramenters
	// parameter mode=mini hides default Flexicapture headers and footers	
	if (predefineProjectAndRole) {
		return Environment.VerificationStation + "QueueList?projectId=" + projectId + "&roleId=" + verificationRoleId + "&mode=mini";
	}

	return Environment.VerificationStation + "?mode=mini";
};

// sample method to open Scanning Web Station
function scan() {

	// Get IFRAME where station will be opened
	var iframe = document.getElementById("fciframe");

	// set source of iframe to scan station/
	iframe.src = getStationUrl(pages.scanning);

	iframe.style.display = "";
	currentPage = pages.scanning;
};

// sample method to open Verification Web Station
function verify() {

	// Get IFRAME where station will be opened
	var iframe = document.getElementById("fciframe");

	// set source of iframe to verification station
	iframe.src = getStationUrl(pages.verify);

	iframe.style.display = "";
	currentPage = pages.verify;
};

// sample method to log to station
function logToStation(stationType) {
	var stationUrl = getStationUrl(stationType);

	// Get IFRAME where station will be opened
	var iframe = document.getElementById("fciframe");
	iframe.src = Environment.LoginStation + "?mode=mini&returnTo=" + encodeURI(stationUrl);
};

// get current language from cookie and set it to languageToolbar
function getLanguageFromCookie() {
	var toolbar = document.getElementById('languageToolbar');
	var match = document.cookie.match(new RegExp("FCUserLang=([^;]+)"));
	if (match) {
		toolbar.value = match[1];
	} else {
		toolbar.value = "en-US";
	}
};

// get selected language from toolbar and set to cookie
function changeLanguage() {
	var toolbar = document.getElementById('languageToolbar');
	document.cookie = "FCUserLang=" + toolbar.value + "; path=/";

	// we need to reload iframe for changes to have effect
	var iframe = document.getElementById("fciframe");
	iframe.contentWindow.location.reload();
};

// this method is called, then page is loaded
function onLoad() {

	// checking if user is authentication and showing full name
	checkUser();

	if (currentPage === pages.scanning)
		scan();
	else
		verify();// opening verification station by default

	// set language value to language selector
	getLanguageFromCookie();
};

// sets userName to page
// if userName is empty, then user is not authenticated,
// and userName and logoff controls should be hidden
function setUser(userName) {
	var userLabel = document.getElementById("userName");
	var logoffLabel = document.getElementById("logoff");

	userLabel.innerText = userName;
	if (userName && userName.length > 0) {
		userLabel.style.display = "";
		logoffLabel.style.display = "";
	} else {
		userLabel.style.display = "none";
		logoffLabel.style.display = "none";
	}
};

// Send ajax request to station to findout authenticated user name
function checkUser() {
	var xmlhttp = new XMLHttpRequest();
	xmlhttp.onreadystatechange = function () {
		if (xmlhttp.readyState == XMLHttpRequest.DONE) {
			if (xmlhttp.status == 200) {
				var result = JSON.parse(xmlhttp.responseText);
				setUser(result);
			}

			// we'll check user name each second, so that when user loses it's authentication 
			// we'll know about it
			setTimeout(checkUser, 1000);
		}
	};

	// so that IE won't cache GET request
	var randomParam = "?_d=" + (Math.random() * 1000000);

	xmlhttp.open("GET", Environment.AppPath + "Custom/CurrentUserName" + randomParam, true);
	xmlhttp.send();
};

// send ajax requset to server to log user off
function logOff() {
	var xmlhttp = new XMLHttpRequest();
	xmlhttp.onreadystatechange = function () {
		if (xmlhttp.readyState == XMLHttpRequest.DONE) {
			if (xmlhttp.status == 200) {

				// we need to hide user's name and logoff control
				setUser("");

				// and open last page again.
				logToStation(currentPage);
			}
		}
	};

	// so that IE won't cache GET request
	var randomParam = "?_d=" + (Math.random() * 1000000);

	xmlhttp.open("GET", Environment.AppPath + "Server/Logoff" + randomParam, true);
	xmlhttp.send();
};

