﻿(function ($, ko) {

	/**
	 *	Получение символа 
	 * event.type должен быть keypress
	 */
	function getChar(event) {
		if (event.which == null) { // IE
			if (event.keyCode < 32) return null; // спец. символ
			return String.fromCharCode(event.keyCode)
		}

		if (event.which != 0 && event.charCode != 0) { // все кроме IE
			if (event.which < 32) return null; // спец. символ
			return String.fromCharCode(event.which); // остальные
		}

		// спец. символ
		return null;
	};	



	/**
	 *	Возращает текст выделения
	 * все <br> заменяет на \n
	 */
	function getSelectionText(element) {
		var selectionText = "";

		var selection = null;
		if (typeof window.getSelection != "undefined") {
			selection = window.getSelection();
		}

		// если есть выделение и оно не находится внтури переданного элемента element
		if (selection != null && selection.rangeCount &&
			$(selection.anchorNode).closest(element).length &&
			$(selection.focusNode).closest(element).length &&
			!selection.isCollapsed) {

			var range = selection.getRangeAt(0);
			var selectionNodes = range.cloneContents();

			$("br", selectionNodes).replaceWith("\n");
			selectionText = $(selectionNodes).text();
		}
		return selectionText;
	}


	function getNormalizedNodes(htmlString) {
		var $paragraph = $("<p/>");
		$paragraph.append(htmlString);

		// получение нормализованных DOM-узлов
		return $paragraph.contents();
	}



	ko.bindingHandlers.htmlLazy = {
		update: function (element, valueAccessor, allBindingsAccessor, viewModel) {
			var htmlString = ko.unwrap(valueAccessor());
			var $element = $(element);



			$element.empty(); // очищаем поле от текстовых узлов и html-элементов
			var nodes = getNormalizedNodes(htmlString);
			if (nodes && nodes.length) {
				$element.append(nodes);
			}

			if (viewModel.selected.peek()) {
				if (element.isContentEditable) {
					FC.Utils.CaretTools.setCaretPositionWithRange(element, viewModel.caret);
				}

				if (allBindingsAccessor.has('htmlRendered')) {
					var htmlRendered = allBindingsAccessor.get('htmlRendered');
					if (htmlRendered != null) {
						htmlRendered();
					}
				}
			}
		}
	};



	ko.bindingHandlers.contentEditable = {
		init: function (element, valueAccessor, allBindingsAccessor, viewModel) {
			var isContentEditable = ko.unwrap(valueAccessor());
			element.contentEditable = !!isContentEditable;
			
				var $element = $(element);
				var notifyOfChanges = function (pos, length, chars) {
					if (allBindingsAccessor.has('htmlChanged')) {
						var htmlChanged = allBindingsAccessor.get('htmlChanged');
						if (htmlChanged != null) {
							htmlChanged(pos, length, chars);
						}
					}
				}


				var backspaceKeyHandler = function (event) {
					event.preventDefault();
					var selection = window.selectionTools.getSelectionBoundaries(element);
					if (selection != null) {
						if (selection.collapsed) {
							if (selection.start > 0) {
								notifyOfChanges(selection.start - 1, 1, null);
							}
						}
						else {
							var deleteCount = selection.length;
							notifyOfChanges(selection.start, deleteCount, null);
						}
					}
				}

				var deleteKeyHandler = function (event) {
					event.preventDefault();
					var selection = window.selectionTools.getSelectionBoundaries(element);
					if (selection != null) {
						if (selection.collapsed) {
							if (element.lastChild && element.lastChild.localName === 'br' && parseInt(element.lastChild.dataset['index'], 10) === selection.end) {
								return; // Delete нажат в конце строки
							} 

							notifyOfChanges(selection.end, 1, null);
						}
						else {
							var deleteCount = selection.length;
							notifyOfChanges(selection.start, deleteCount, null);
						}
					}
				}

				var undoHandler = function (event) {
					event.preventDefault();
					if (allBindingsAccessor.has('undo')) {
						var undo = allBindingsAccessor.get('undo');
						if (undo != null) {
							undo();
						}
					}
				}

				var redoHandler = function (event) {
					event.preventDefault();
					if (allBindingsAccessor.has('redo')) {
						var redo = allBindingsAccessor.get('redo');
						if (redo != null) {
							redo();
						}
					}
				}

				// переопределение событий нажатия символьных клавиш и Enter
				$element.on("keypress", function (event) {
					if (event.ctrlKey || event.altKey || event.metaKey)
						return true;

					var selection = window.selectionTools.getSelectionBoundaries(this);
					var chars = "";
					if ((event.keyCode || event.which) == 13 && event.shiftKey) {
						if (selection != null) {
							if ($element.text().length == selection.end
								&& !$(element.lastChild).is("br")) {
								chars = "\n\n";
							} else {
								chars = "\n";
							}
						}

					} else {
						chars = getChar(event);
					}


					if (selection != null && chars != null && chars > "") {
						event.preventDefault();
						notifyOfChanges(selection.start, selection.length, chars);
					}

				});


				// переопределение стандартных событий для Backspace и Delete 
				$element.on("keydown", function (event) {
					var keyCode = event.keyCode;
				var contentEditableAttr = event.currentTarget.attributes.contenteditable;
				if (!contentEditableAttr || contentEditableAttr.value === 'false') return;
					switch (keyCode) {
						case 8:		// Backspace
							backspaceKeyHandler(event);
							break;
						case 46:	// Delete
							deleteKeyHandler(event);
							break;
						case 90: // комбинация ctrl+z и ctrl+shift+z 
							if (event.ctrlKey || event.metaKey) {
								if (event.shiftKey) {
									redoHandler(event);
								} else {
									undoHandler(event);
								}
							}
							break;
					}
				});

				$element.on("copy", function (event) {
					event.preventDefault();
					var copiedText = getSelectionText(this);
					if (copiedText) {
						var clipboardData = ((event.originalEvent || event).clipboardData || window.clipboardData);
						clipboardData.setData('Text', copiedText);
					}
				});

				// переопределние стандартной операции paste
				$element.on("paste", function (event) {
					event.preventDefault();
					var text = ((event.originalEvent || event).clipboardData || window.clipboardData).getData('Text');
					var selection = window.selectionTools.getSelectionBoundaries(this);

					if (selection != null && text > "") {
						notifyOfChanges(selection.start, selection.length, text);
					}
				});


				// переопределние стандартной операции cut
				$element.on("cut", function (event) {
					event.preventDefault();

					var cutText = getSelectionText(this);
					if (cutText) {
						var clipboardData = ((event.originalEvent || event).clipboardData || window.clipboardData);
						clipboardData.setData('Text', cutText);

						var selection = window.selectionTools.getSelectionBoundaries(this);
						notifyOfChanges(selection.start, selection.length, null);
					}
				});


				// Запрет выполнения drag'n'drop
				$element.on("dragstart", function (event) {
					var dataTransfer = (event.originalEvent || event).dataTransfer;
					dataTransfer.effectAllowed = 'none';
					dataTransfer.dropEffect = 'none';
				})
			.on('drop', function (event) {
				event.preventDefault();
			});



				/* 
				 * Поддержка событий утилиты ввода символов IME
				 */

				// положение курсора, с которого началась композиция
				var selection = null;

				$element.on('compositionstart', function (event) {

					// запоминаем положение курсора, с которого началась композиция
					selection = window.selectionTools.getSelectionBoundaries(this);
				})
			.on('compositionend', function (event) {

				var chars = event.originalEvent.data; // напечатанные символы-иероглифы

				if (selection != null && chars != null && chars > "") {
					notifyOfChanges(selection.start, selection.length, chars);
				}
			});

			

		},
		update: function(element, valueAccessor, allBindingsAccessor, viewModel) {
			var isContentEditable = ko.unwrap(valueAccessor());
			element.contentEditable = !!isContentEditable;
		}
	};

	ko.bindingHandlers.htmlChanged = {
		init: function (element, valueAccessor) {
			var value = valueAccessor();
			if (!$.isFunction(value)) {
				throw new Error("Invalid htmlChanged binding type");
			}
		}

	}

	ko.bindingHandlers.htmlRendered = {
		init: function (element, valueAccessor) {
			var value = valueAccessor();
			if (!$.isFunction(value)) {
				throw new Error("Invalid htmlRendered binding type");
			} else {
				value(true);
			}
		}

	}

	ko.bindingHandlers.undo = {
		init: function (element, valueAccessor) {
			var value = valueAccessor();
			if (!$.isFunction(value)) {
				throw new Error("Invalid undo binding type");
			}
		}

	}

	ko.bindingHandlers.redo = {
		init: function (element, valueAccessor) {
			var value = valueAccessor();
			if (!$.isFunction(value)) {
				throw new Error("Invalid redo binding type");
			}
		}

	}

})(jQuery, ko);