﻿FC.BusinessModels.DocumentEditor.BasicField = function (initInfo, manipulators, parent) {
	initInfo = initInfo || {};
	this.manipulators = manipulators;

	this.internalId = FC.BusinessModels.DocumentEditor.BasicField.IdScore++;
	this.id = ko.observable(initInfo.Id || -1);

	this.fieldType = initInfo.FieldType;
	this.templateId = ko.observable(initInfo.TemplateId || -1);

	this.name = ko.observable(initInfo.Name);
	this.caption = ko.observable(initInfo.Caption || -1);

	this.isReadOnly = ko.observable(initInfo.IsReadOnly || false);

	this.isDbLocked = ko.observable(false);

	this.isVisible = ko.observable(initInfo.IsVisible || false);

	this.IsSegment = ko.observable(initInfo.IsSegment || false);

	this.canHaveRegion = ko.observable(initInfo.CanHaveRegion || false);

	/**
	 * Максимальное число регионов, которое может быть у поля
	 */
	this.maxRegions = ko.observable(initInfo.MaxRegions || 0);

	this.needRecognize = ko.observable(initInfo.NeedRecognize || false);

	this.customScript = initInfo.CustomScript || '';

	this.parent = parent;
}

FC.BusinessModels.DocumentEditor.BasicField.prototype.getFieldKeyWithoutRepeateIndex = function () {
	var path = [];
	var cField = this;
	while (cField) {
		if (!(((cField.parent instanceof FC.BusinessModels.DocumentEditor.FieldsGroup &&
				cField.parent.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent)) ||
			(cField instanceof FC.BusinessModels.DocumentEditor.TableRow) ||
			(cField.parent instanceof FC.BusinessModels.DocumentEditor.SectionsGroup))) {
			path.unshift(cField.id());
		}
		cField = cField.parent;
	}

	return path;
};

FC.BusinessModels.DocumentEditor.BasicField.prototype.getFieldKey = function () {
	var path = [];
	var cField = this;
	while (cField) {
		path.unshift(cField.id());
		cField = cField.parent;
	}

	return path;
};


FC.BusinessModels.DocumentEditor.BasicField.prototype.isRepeatableField = function () {
	return (this.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent ||
		this.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.SectionsGroup || this.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Table);
}


/**
 * Построение пути до поля на основе имен полей
 * @returns {} 
 */
FC.BusinessModels.DocumentEditor.BasicField.prototype.getFieldNamePath = function () {
	var path = [];
	var cField = this;
	while (cField) {
		//Для пвоторяющихся элементов название дочерних элементов дублирует название самой повторяющейся группы и должно быть исключено
		if (cField.isRepeatableField()) {
			path.shift();
		}

		if (cField.name && cField.name()) {
			path.unshift(cField.name());
		}
		cField = cField.parent;
	}

	return path;
};

/**
 * Функция вычисления пути к полю из id групп и индексов повторяющихся элементов
 * @returns {} 
 */
FC.BusinessModels.DocumentEditor.BasicField.prototype.getFieldNamePathWithIndex = function () {
	var path = [];
	var cField = this;
	while (cField) {
		var parentField = cField.parent;
		var fieldName = cField.name ? cField.name() : '';
		if (parentField && (parentField.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent || parentField instanceof FC.BusinessModels.DocumentEditor.SectionsGroup)) {
			fieldName = fieldName + '[' + parentField.fields().indexOf(cField) + ']';
			cField = cField.parent;
		} else if (cField instanceof FC.BusinessModels.DocumentEditor.TableRow) {
			fieldName = parentField.name ? parentField.name() : '';
			fieldName = fieldName + '[' + parentField.rows().indexOf(cField) + ']';
			cField = cField.parent;
		} 
		path.unshift(fieldName);
		cField = cField.parent;
	}
	return path;
}

FC.BusinessModels.DocumentEditor.BasicField.prototype.getFieldPath = function () {
	var fieldPath = [];
	var connectedField = this;
	var parentField = connectedField.parent;
	while (connectedField) {
		if (parentField && parentField.isRepeatableField()) {
			fieldPath.unshift({
				type: FC.Viewmodels.FormEditor.IndexType.RepeateIndex,
				id: (parentField.fields ? parentField.fields() : parentField.rows()).indexOf(connectedField)
			});
		} else {
			fieldPath.unshift({
				type: FC.Viewmodels.FormEditor.IndexType.ElementId,
				id: connectedField.id()
			});
		}

		connectedField = parentField;
		if (connectedField) parentField = connectedField.parent;
	}

	return fieldPath;
}


/**
 * Получение JS Объекта 
 * @param {} fullSerialize  true - для всех свойств класса,  false для свойств класса необходимых при CHeck Rules и Apply Changes
 * @returns {} 
 */
FC.BusinessModels.DocumentEditor.BasicField.prototype.basicJs = function (fullSerialize) {
	var result = {
		Id: this.id(),
		FieldType: this.fieldType,
		Name: this.name(),
		Caption: this.caption(),
		NeedRecognize: this.needRecognize(),
		IsVisible: this.isVisible(),
		IsSegment: this.IsSegment(),
		Regions: $.map(this.manipulators.regionsManipulator.getFieldRegions(this),
			function (regionField) {
				return {
					Id: regionField.region.id(),
					Status: regionField.region.status(),
					PageId: regionField.region.pageId(),
					IsNew: regionField.region.isNew(),
					IsSegment: this.IsSegment(),
					Coordinates: $.map(regionField.region.coordinates(), function (val) {
						return {
							X1: val.x1(),
							X2: val.x2(),
							Y1: val.y1(),
							Y2: val.y2(),
						}
					}),
				}
			}),
		
	}
	if (fullSerialize) {
		result.NamePath = this.getFieldNamePathWithIndex();
		result.TemplateId = this.templateId();
		result.IsReadOnly = this.isReadOnly();
		result.CanHaveRegion = this.isReadOnly();
		result.MaxRegions = this.isReadOnly();
		result.internalId = this.internalId;

	}

	return result;
}



/**
 * Возвращает массив индексов в повторяющихся группах до текущего элемента
 * @returns {} 
 */
FC.BusinessModels.DocumentEditor.BasicField.prototype.getRepeatableContext = function() {
	var field = this;

	var result = [];
	while (field && field.parent) {
		var parentField = field.parent;
		if (parentField.isRepeatableField()) {
			result.unshift((parentField.fields ? parentField.fields() : parentField.rows()).indexOf(field));
		}
		field = parentField;
	}
	return result;
};

/**
 * Получить ссылку на поле в формате id1\id2\id3...
 * состоящую из идентификаторов родителей поля
 * @returns {} строка со ссылкой 
 */
FC.BusinessModels.DocumentEditor.BasicField.prototype.getNodeRef = function () {
	var path = this.getFieldPath();
	var ref = "";
	for (var i = 0; i < path.length; i++) {
		if (path[i].type === FC.Viewmodels.FormEditor.IndexType.RepeateIndex) {
			ref += "-";
		}
		ref += path[i].id;
		if (i < path.length - 1) {
			ref += "\\";
		}
	}

	return ref;
};

/**
 * Получить секцию, к которой принадлежит поле
 * @returns {FC.BusinessModels.DocumentEditor.Section}
 */
FC.BusinessModels.DocumentEditor.BasicField.prototype.getCurrentSection = function () {
	var field = this;
	while (field.parent) {
		field = field.parent;
	}

	return field;
};

FC.BusinessModels.DocumentEditor.BasicField.IdScore = 1;
FC.BusinessModels.DocumentEditor.BasicField.NewInstancesId = 1;