﻿FC.BusinessModels.DocumentEditor.DbCheckRulesManipulator = function(templateId,
	templateVersion,
	allFields,
	databaseRules,
	datasets,
	fields,
	regionsManipulator,
	errorsManipulator) {


	var self = this;
	self.rules = databaseRules;
	self.fields = allFields;
	self.templateId = templateId;
	self.templateVersion = templateVersion;

	self.recordIdFieldsMap = {};

	// Словарь путь_до_поля:правило 
	// в нем только поля, которые участвуют в правиле, 
	// путь к полю состоит из имен
	self.fieldsRuleMap = {};

	//Словарь путь_до_поля:правило 
	// в нем только поля, которые ограничивают запуск, 
	// путь к полю состоит из имен
	self.applyFieldsRuleMap = {};

	self.datasets = datasets;
	var joinKey = '\\';

	var ruleErrorTypes = {
		RecordNotFound: 0,
		TooManyRecords: 1,
		MissedDataset: 2,
		NotEnoughData: 3,
		RecordMismatched: 4
	}

	var ruleContextEquals = function(left, right) {
		if (!left && !right) return true;
		if (!left || !right) return false;
		return left.rule === right.rule && left.repeatableContext.equals(right.repeatableContext);
	}

	self.addRuleError = function(ruleContext, ruleErrorType, comment) {
		ruleErrorType = ruleErrorType || 0;
		var accordingFields = self.getRuleFieldsInfo(ruleContext);
		accordingFields = Object.keys(accordingFields).map(function(key) { return accordingFields[key].field; });
		var ruleText = '';
		switch (ruleErrorType) {
		case ruleErrorTypes.RecordNotFound:
			ruleText = FC.Environment.Localization.DbLookup.RuleErrorRecordNotFound;
			break;
		case ruleErrorTypes.TooManyRecords:
			ruleText = FC.Environment.Localization.DbLookup.RuleErrorTooManyRecords;
			break;
		case ruleErrorTypes.MissedDataset:
			ruleText = FC.Environment.Localization.DbLookup.RuleErrorMissedDataset;
			break;
		case ruleErrorTypes.NotEnoughData:
			ruleText = FC.Environment.Localization.DbLookup.RuleErrorNotEnoughData;
			break;
		case ruleErrorTypes.RecordMismatched:
			ruleText = FC.Environment.Localization.DbLookup.RuleErrorFieldValueMismatch.replace('%0', comment);
			break;
		}
		if (ruleContext.rule.showRuleNameInErrorMessage) {
			ruleText = ruleContext.rule.ruleName + ": " + ruleText;
		}
		errorsManipulator.addError(accordingFields,
			ruleContext.rule.isError,
			FC.BusinessModels.DocumentEditor.ErrorType.RuleError,
			ruleText,
			ruleContext.rule.ruleName);
	}

	self.removeRuleErrors = function(ruleContext) {
		var accordingFields = self.getRuleFieldsInfo(ruleContext);
		accordingFields = Object.keys(accordingFields).map(function(key) { return accordingFields[key]; });
		//Оставим ошибки у кототорых тип RuleError и совпадает название правила
		var errors = errorsManipulator.errors().filter(function(error) {
			return error.type() === FC.BusinessModels.DocumentEditor.ErrorType.RuleError &&
				error.ruleName() === ruleContext.rule.ruleName;
		});

		//Уберем ошибки, в описании которых присутствуют поля, которые не участвуют в правиле
		errorsManipulator.fieldsErrors().forEach(function(fieldError) {
			var indexOfError = errors.indexOf(fieldError.error);
			if (indexOfError < 0) return;
			if (accordingFields.filter(function(accordingField) { return accordingField.field === fieldError.field; }).length ===
				0) {
				errors.splice(indexOfError, 1);
			}
		});

		//Удалим из модели оставшиеся ошибки
		errors.forEach(errorsManipulator.removeError);
	}

	var addToMap = function(field, rule, map) {
		var docDef = field.documentDefinitionField || field.fieldRef;
		var key = docDef.join(joinKey);
		if (map[key]) {
			map[key].rules.push(rule);
		} else {
			map[key] = {
				rules: [rule],
				fieldPath: docDef
			}
		}
	}
	for (var i = 0; i < self.rules.length; i++) {
		var rule = self.rules[i];
		if (!rule.isEnabled) continue;
		var recordIdField = rule.fieldsInfo.filter(function(val) { return val.isRecordField; })[0];

		if (recordIdField) {
			addToMap(recordIdField, rule, self.recordIdFieldsMap);
		}

		rule.fieldsInfo.forEach(function(fieldInfo) {
			addToMap(fieldInfo, rule, self.fieldsRuleMap);
		});
		if (rule.applyCondition) {
			addToMap(rule.applyCondition, rule, self.applyFieldsRuleMap);
		}
	}

	self.checkFieldIsRecordId = function(field) {
		var fieldNamePath = field.getFieldNamePath();
		return !!self.recordIdFieldsMap[fieldNamePath.join(joinKey)];
	}

	// Блокирует/разблокирует текстовые поля, которые учавствуют в правилах проверки по бд, где есть recordId
	self.setLockingState = function(field, state) {
		var fieldPath = field.getFieldNamePath();
		var fieldInfo = self.recordIdFieldsMap[fieldPath.join(joinKey)];
		if (fieldInfo) {
			var fieldsToLock = [];
			for (var i = 0; i < fieldInfo.rules.length; i++) {
				var rule = fieldInfo.rules[i];
				fieldsToLock = fieldsToLock.concat(rule.fieldsInfo.map(
					function(fieldInfo) {
						return fieldInfo.documentDefinitionField.join(joinKey);
					}));
			}

			self.fields().forEach(
				function(fieldInfo) {
					if (fieldInfo.field instanceof FC.BusinessModels.DocumentEditor.TextField) {
						var fieldPath = fieldInfo.field.getFieldNamePath().join(joinKey);
						if (fieldsToLock.indexOf(fieldPath) >= 0) {
							fieldInfo.field.isDbLocked(state);
						}
					}
				});
		}
	}

	/**
	 * Проход по всем полям участвующим в правилах и запись всех точек ветвления повторяющихся элементов
	 * @param {any} rule
	 * @returns список точек ветвления для правила и флаг наличия всех повторяющихся полей
	 */
	self.getMaxRepeatableDepthForRule = function(rule) {
		var maxDepth = 0;
		var maxDepthRepeatablePoints = [];
		var fieldsMissed = false;
		var allFieldsAccordingRule = rule.fieldsInfo.slice();
		if (rule.applyCondition) {
			allFieldsAccordingRule.push(rule.applyCondition);
		}
		allFieldsAccordingRule.forEach(function(fieldInfo) {
			var fieldRef = fieldInfo.documentDefinitionField || fieldInfo.fieldRef;
			var currentNodeArray = fields();
			var foundField;
			var repeatableContextDepth = 0;
			var repeatablePoints = [];
			for (var i = 0; i < fieldRef.length; i++) {
				var levelName = fieldRef[i];
				foundField = null;
				for (var j = 0; j < currentNodeArray.length; j++) {
					var field = currentNodeArray[j];
					if (field.name() === levelName) {
						foundField = field;
						break;
					}
				}
				if (!foundField) {
					foundField = null;
					break;
				};

				if (foundField.isRepeatableField()) {
					foundField = (foundField.fields ? foundField.fields() : foundField.rows())[0];
					if (foundField) {
						repeatablePoints.push(foundField.parent);
						repeatableContextDepth++;
					} else {
						foundField = null;
						break;
					}
				}

				if (foundField.fields) {
					currentNodeArray = foundField.fields();
				} else if (foundField.cell) {
					currentNodeArray = foundField.cell();
				} else if (i !== fieldRef.length - 1) {
					foundField = null;
					break;
				}

			}

			if (foundField && (foundField instanceof FC.BusinessModels.DocumentEditor.TextField)) {
				if (maxDepth < repeatableContextDepth) {
					maxDepthRepeatablePoints = repeatablePoints;
				}
			} else {
				fieldsMissed = true;
			}
		});
		return {
			repeatablePoints: maxDepthRepeatablePoints,
			someFieldsMissing: fieldsMissed
		};
	}

	self.disableAccordingFields = function(field) {
		self.setLockingState(field, true);
	}

	self.enableAccordingFields = function(field) {
		self.setLockingState(field, false);
	}

	self.getAccordingRules = function(field) {
		var fieldNamePath = field.getFieldNamePath().join(joinKey);
		var accordingRulesModel = self.fieldsRuleMap[fieldNamePath] && self.applyFieldsRuleMap[fieldNamePath]
			? self.fieldsRuleMap[fieldNamePath].rules.concat(self.applyFieldsRuleMap[fieldNamePath].rules)
			: (self.fieldsRuleMap[fieldNamePath] || self.applyFieldsRuleMap[fieldNamePath] || {}).rules;
		return accordingRulesModel || [];
	}

	self.getRuleFieldsInfo = function(ruleContext) {
		var res = {};
		var rule = ruleContext.rule;
		var repeatableContext = ruleContext.repeatableContext;

		rule.fieldsInfo.forEach(function(fieldInfo, index) {
			var currentNodeArray = fields();
			var foundField;
			var repeatableContextIndex = 0;
			for (var i = 0; i < fieldInfo.documentDefinitionField.length; i++) {
				var levelName = fieldInfo.documentDefinitionField[i];
				foundField = null;
				for (var j = 0; j < currentNodeArray.length; j++) {
					var field = currentNodeArray[j];
					if (field.name() === levelName) {
						foundField = field;
						break;
					}
				}
				if (!foundField) {
					foundField = null;
					break;
				};

				if ((foundField.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent) ||
					(foundField.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.SectionsGroup)) {
					foundField = foundField.fields()[repeatableContext[repeatableContextIndex]];
					if (!foundField) {
						break;
					}
					repeatableContextIndex++;
				} else if (foundField.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Table) {
					foundField = foundField.rows()[repeatableContext[repeatableContextIndex]];
					if (!foundField) {
						break;
					}
					repeatableContextIndex++;
				}

				if (foundField.fields) {
					currentNodeArray = foundField.fields();
				} else if (foundField.cell) {
					currentNodeArray = foundField.cell();
				} else if (i !== fieldInfo.documentDefinitionField.length - 1) {
					foundField = null;
					break;
				}

			}

			if (foundField && (foundField instanceof FC.BusinessModels.DocumentEditor.TextField)) {
				res[fieldInfo.databaseAlias] = {
					caption: foundField.caption(),
					order: index,
					field: foundField,
					dbLookupfieldInfo: fieldInfo

				};
			} else {
				res[fieldInfo.databaseAlias] = {
					caption: '',
					order: index,
					field: null,
					dbLookupfieldInfo: fieldInfo
				};
			}
		});
		return res;
	}

	/**
	 * Запись значений для правила с применением всех настроек записи
	 * @param {any} ruleContext
	 * @param {any} values
	 * @param {bool} forced при true игнорирует настройки вставки, вставляет всегда
	 */
	self.applyRule = function(ruleContext, values, forced) {
		var accordingDataSet = self.datasets.filter(function(dataset) {
			return dataset.name === ruleContext.rule.dataSourceName;
		})[0];

		var filteringKeys = accordingDataSet.columns.filter(function(column) { return column.isFilteringKey; }).map(
			function(column) {
				return column.name;
			});

		var ruleFieldsInfo = self.getRuleFieldsInfo(ruleContext);

		var keys = Object.keys(ruleFieldsInfo);
		var updatedFields = [];

		keys.forEach(function(key) {
			var fieldCaption = ruleFieldsInfo[key];
			//Верифицируем все поля, которые были в правиле
			fieldCaption.field.isVerified(true);
			if (!fieldCaption.field) return;


			var value = values ? values[key] : '';
			if (typeof value == 'undefined') return;

			//В режиме forced игнорируем filtering key (для вставки вендора игнорируется BU)
			if (forced && filteringKeys.indexOf(key) >= 0) return;

			//Если не forced, проверяем условия вставкеи
			if (!forced) {
				var pasteAlgo = fieldCaption.dbLookupfieldInfo.pasteStrategy;

				//Если режим не вставлять то и не вставляем
				if (pasteAlgo === FC.BusinessModels.DocumentEditor.PasteStrategy.NotPaste) return;

				var currentValue = fieldCaption.field.value();

				//Если режим вставлять только при пустом, а значение есть, то мимо
				if (currentValue && pasteAlgo === FC.BusinessModels.DocumentEditor.PasteStrategy.PasteIfEmpty) return;

				//если значение совпадает, то ничего не делаем
				if (ruleFieldsInfo[key].field.value().toLowerCase().trim() === value.toLowerCase().trim()) return;
			}

			//Если мы тут, то мы будем менять значение

			//записываем измененное поле, чтобы искать связные правила
			updatedFields.push(fieldCaption.field);

			
			fieldCaption.field.value(value);
			
			fieldCaption.field.valueAuthorized = true;

			//Если удаляли значение, грозаем регион
			if (!values) {
				regionsManipulator.removeFields(ruleFieldsInfo[key].field);
			}

		});

		return updatedFields;
	}

	var generateAllRepeatableContextVariants = function(rootField, pathsArray, accordingRepeatableContext) {
		if (!pathsArray || pathsArray.length === 0) return [];
		var remainPathsArray = pathsArray.slice();
		var pathToNextRepeatable = remainPathsArray.shift();
		var nextRepeatable = rootField;
		for (var i = 0; i < pathToNextRepeatable.length; i++) {
			nextRepeatable =
				(nextRepeatable.fields ? nextRepeatable.fields() : nextRepeatable.rows()).filter(function(field) {
					return field.id() === pathToNextRepeatable[i];
				})[0];
		}


		var nextRepeatableChildren = nextRepeatable.fields ? nextRepeatable.fields() : nextRepeatable.rows();
		var result = [];
		nextRepeatableChildren.forEach(function(field, index) {
			var newAccordingRepeatableContext = accordingRepeatableContext.slice();
			newAccordingRepeatableContext.push(index);
			if (remainPathsArray.length === 0) {
				result.push(newAccordingRepeatableContext);
			} else {
				result = result.concat(generateAllRepeatableContextVariants(field, remainPathsArray, newAccordingRepeatableContext));
			}
		});

		return result;
	}


	/**
	 * Формирование контекстов для правил
	 * @param {any} rules
	 */
	var generateRuleContexts = function(rules) {
		var accordingRulesContexts = [];
		rules.forEach(function(rule) {
			var ruleDepthInfo = self.getMaxRepeatableDepthForRule(rule);

			//Промежуточные пути между точками повторения
			var pathsToNextRepeatables = [];
			for (var j = 1; j < ruleDepthInfo.repeatablePoints.length; j++) {
				var currentPoint = ruleDepthInfo.repeatablePoints[j];
				var pathToPrevRepeatable = [];
				while (!currentPoint.parent.isRepeatableField()) {
					pathToPrevRepeatable.unshift(currentPoint.id());
					currentPoint = currentPoint.parent;
				}
				pathsToNextRepeatables.push(pathToPrevRepeatable);
			}

			//В правиле не участвуют повторяющиеся поля
			if (ruleDepthInfo.repeatablePoints.length === 0) {
				accordingRulesContexts.push({
					rule: rule,
					repeatableContext: []
				});
			} else {
				var firstRepeatablePoint = ruleDepthInfo.repeatablePoints[0];
				(firstRepeatablePoint.fields
					? firstRepeatablePoint.fields()
					: firstRepeatablePoint.rows()).forEach(function(field, index) {
					if (pathsToNextRepeatables.length === 0) {
						accordingRulesContexts.push({
							rule: rule,
							repeatableContext: [index]
						});
					} else {
						var allRepeatableContexts = generateAllRepeatableContextVariants(field, pathsToNextRepeatables, [index]);
						allRepeatableContexts.forEach(function(repeatableContext) {
							accordingRulesContexts.push({
								rule: rule,
								repeatableContext: repeatableContext
							});
						});
					}
				});
			}
		});
		return accordingRulesContexts;
	}
	/**
	 * Собирает список затронутых правил, добавляет к правилам, которые уже требуют пересчета, исключает нежуное правило и инициирует пересчет
	 * @param {any} fields
	 * @param {any} rules
	 * @param {any} ignoreRule
	 */
	self.tryRecount = function(fields, rules, ignoreRuleContext) {
		rules = rules || [];
		fields = fields || [];
		fields.forEach(function(field) {
			//Получаем правила для поля
			var accordingRules = self.getAccordingRules(field);
			var fieldRepeatableContext = field.getRepeatableContext();
			//Формируем контексты на основе найденных правил 
			var accordingRulesContexts = generateRuleContexts(accordingRules);

			//Оставим только те контексты, в которых участвует исходное поле
			accordingRulesContexts = accordingRulesContexts.filter(function(ruleContext) {
				var repeatableContext = ruleContext.repeatableContext;
				for (var i = 0; i < fieldRepeatableContext.length; i++) {
					if (repeatableContext[i] !== fieldRepeatableContext[i])
						return false;
				}
				return true;
			});
			//Оставляем только новые контексты, убираем игнорящийся контекст
			accordingRulesContexts = accordingRulesContexts.filter(function(accordingRuleContext) {
				return !ruleContextEquals(accordingRuleContext, ignoreRuleContext) &&
					rules.filter(function(ruleContext) {
						return ruleContextEquals(accordingRuleContext, ruleContext);
					}).length ===
					0;

			});
			rules = rules.concat(accordingRulesContexts);
		});
		if (rules.length > 0) {
			self.startRulesRecount(rules);
		}
	}

	self.tryRecountAllRules = function() {
		var ruleContexts = generateRuleContexts(self.rules.filter(function (rule) { return rule.isEnabled; }));
		if (ruleContexts.length > 0) {
			self.startRulesRecount(ruleContexts);
		}
	}

	/**
	 * Проверка на совпадение/вхождение rightValue в leftValue
	 * @param {any} leftValue значение, которое должно содержать rightValue
	 * @param {any} rightValue
	 * @param {any} isFuzzyCompare если true, то проверяет вхождение хотя бы на 70%, если false, то точное совпадение
	 */
	var valuesEquals = function(leftValue, rightValue, isFuzzyCompare) {
		leftValue = leftValue.toLowerCase().trim();
		rightValue = rightValue.toLowerCase().trim();
		if (isFuzzyCompare) {
			return leftValue.indexOf(rightValue) >= 0 && (rightValue.length / leftValue.length * 100) >= 70;
		} else {
			return leftValue === rightValue;
		}
	}


	var checkApplyCondition = function(ruleContext) {
		var applyCondition = ruleContext.rule.applyCondition;
		if (!applyCondition) return true;

		var applyConditionField = self.fields().filter(function(field) {
			field = field.field;

			if (!(field instanceof FC.BusinessModels.DocumentEditor.SimpleField) ||
				!field.getFieldNamePath().equals(applyCondition.fieldRef)) return false;

			var fieldRepeatableContext = field.getRepeatableContext();
			for (var i = 0; i < fieldRepeatableContext.length; i++) {
				if (ruleContext.repeatableContext[i] !== fieldRepeatableContext[i]) return false;
			}
			return true;
		})[0];
		if (!applyConditionField) return false;

		applyConditionField = applyConditionField.field;
		if (applyConditionField instanceof FC.BusinessModels.DocumentEditor.TextField) {
			if (applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsInSet ||
				applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsNotInSet) {
				var valueInSet = (applyCondition.values || []).indexOf(applyConditionField.value()) >= 0;

				return (applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsInSet &&
						valueInSet) ||
					(applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsNotInSet &&
						!valueInSet);

			} else {
				return (applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsEmpty &&
						!applyConditionField.value().trim()) ||
					(applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsNotEmpty &&
						applyConditionField.value().trim());
			}
		} else if (applyConditionField instanceof FC.BusinessModels.DocumentEditor.BooleanField) {
			if (applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsInSet ||
				applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsNotInSet) {
				return (applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsInSet &&
						applyConditionField.value()) ||
					(applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsNotInSet &&
						!applyConditionField.value());
			} else {
				return (applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsEmpty &&
						!applyConditionField.value()) ||
					(applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsNotEmpty &&
						applyConditionField.value());
			}
		} else if (applyConditionField instanceof FC.BusinessModels.DocumentEditor.ChoiceField) {
			if (applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsInSet ||
				applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsNotInSet) {
				var selectedVariants = applyConditionField.selectedVariants();
				var applyConditionValues = (applyCondition.values || []);
				var selectedVariantsFromSet = selectedVariants.filter(function(selectedVariant) {
					return applyConditionValues.filter(function(value) {
							return value.toLowerCase().trim() === selectedVariant.toLowerCase().trim();
						}).length >
						0;
				});
				return ((applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsInSet &&
						selectedVariantsFromSet.length > 0) ||
					(applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsNotInSet &&
						selectedVariantsFromSet.length === 0));
			} else {
				return ((applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsEmpty &&
						applyConditionField.selectedVariants().length === 0) ||
					(applyCondition.type === FC.BusinessModels.DocumentEditor.DatabaseCheckRuleApplyConditionType.IsNotEmpty &&
						applyConditionField.selectedVariants().length > 0));
			}
		}
		return true;
	}


	/**
	 * Оценивает пригодность претендента по полям, используемым в поиске
	 * @param {any} challenger
	 * @param {any} fieldsForSearch
	 */
	var isSutableResult = function(challenger, fieldsForSearch) {
		for (var i = 0; i < fieldsForSearch.length; i++) {
			var fieldForSearch = fieldsForSearch[i];

			var currentValue = fieldForSearch.field ? fieldForSearch.field.value().toLowerCase().trim() : '';

			//Определяет критерий сравнения, если true, то значение поля совпадает со значением из бд более чем на 70%
			var isFuzzyCompare = fieldForSearch.dbLookupfieldInfo.isFuzzySearch;

			//Если true, то проверка по полю проходит даже если в поле есть значение а в бд нет, в противном случае сравнение на основе isFuzzy
			var allowEmptyValueInDb = fieldForSearch.dbLookupfieldInfo.allowEmptyValueInDb;

			//Название колонки в БД для поля
			var fieldKey = fieldForSearch.dbLookupfieldInfo.databaseAlias;
			var valuesInDb = challenger[fieldKey];

			//Если в поле нет значения и в БД нет значений, то проверка пройдена
			if (!currentValue && !valuesInDb) continue;

			//Если в БД нет значения и выставлена галка allowEmptyValueInDb, то проверка пройдена
			if (!valuesInDb && allowEmptyValueInDb) continue;

			//Других условий успешного прохождения проверки при отсутствии значения в поле или в БД нет
			if (!valuesInDb || !currentValue) return false;

			var relevantValueInDb = valuesInDb.filter(function(valueInDb) {
				return valuesEquals(valueInDb, currentValue, isFuzzyCompare);
			});
			//Если не нашли совпадений, проверка провалена.
			if (relevantValueInDb.length === 0) return false;
		}
		//Если дожили до этого места, значит все поля участвующие в поиске удовлетворяют условиям проверки
		return true;
	}


	var processCheckAgainstDatabase = function (challenger, ruleFields) {
		var mismatchedFields = [];
		//Смотрим все поля участвующие в правиле и отбираем те, у которых включена checkAgainstDb
		for (var i = 0; i < ruleFields.length; i++) {
			var accordingField = ruleFields[i];
			var currentValue = accordingField.field ? accordingField.field.value().toLowerCase().trim() : '';

			//Проверка осуществляется только для заполненных полей
			if (!currentValue || !accordingField.dbLookupfieldInfo.checkAgainstDb) continue;

			var isFuzzyCompare = accordingField.dbLookupfieldInfo.isFuzzySearch;
			var fieldKey = accordingField.dbLookupfieldInfo.databaseAlias;
			var valuesInDb = challenger[fieldKey];

			//Если тут оказалась запись с пустым списком значений из BD значит она удовлетворяла критериям поиска
			//Убираем записидля которых есть поля со значением,не соответствующим ни одной записи из бд
			if (valuesInDb &&
				valuesInDb.filter(function (valueInDb) {
					return valuesEquals(valueInDb, currentValue, isFuzzyCompare);
				}).length ===
				0) {
				mismatchedFields.push(accordingField.field);
			}
		}
		return mismatchedFields;
	}

	/**
	 * Поиск наиболее подходящих значений в случае нескольких
	 * @param {any} newValues
	 * @param {any} ruleFields
	 */
	var findBestFieldValues = function(newValues, ruleFields) {
		var bestFieldsValues = {};
		//Выбираем лучшие значения из справочника. Необходимо для колонок, с несколькими значениями
		ruleFields.forEach(function (accordingField) {

			var currentValue = accordingField.field ? accordingField.field.value().toLowerCase().trim() : '';
			var isFuzzyCompare = accordingField.dbLookupfieldInfo.isFuzzySearch;
			var fieldKey = accordingField.dbLookupfieldInfo.databaseAlias;
			var allowEmptyValueInDb = accordingField.dbLookupfieldInfo.allowEmptyValueInDb;
			var valuesInDb = newValues[fieldKey];

			//В случае когда допустимо пустое значение в бд и значения нет, то опускаем это поле
			if (!valuesInDb && allowEmptyValueInDb) {
				return;
			}

			//иначе надо перезаписать на пустое значение
			if (!valuesInDb) {
				bestFieldsValues[fieldKey] = '';
				return;
			}

			var bestValues = valuesInDb.filter(function (valueInDb) {
				return valuesEquals(valueInDb, currentValue, isFuzzyCompare);
			});
			bestFieldsValues[fieldKey] = bestValues.length > 0 ? bestValues[0] : valuesInDb[0];

		});
		return bestFieldsValues;
	}

	/**
	 * Производит пересчет первого правила из списка, по завершению инициирует обновление списков и новй пересчет
	 * @param {any} rules
	 */
	self.startRulesRecount = function(rules) {
		if (!rules || rules.length === 0) return;
		var ruleContext = rules.shift();
		var accordingFields = self.getRuleFieldsInfo(ruleContext);
		accordingFields = Object.keys(accordingFields).map(function(key) { return accordingFields[key]; });

		//Поиск еще несозданных полей. Отсекает проверку правил для повторяющихся полей без инстансов
		var nonexistentFields = accordingFields.filter(function(af) { return !af.field });
		if (nonexistentFields.length > 0) {
			self.tryRecount([], rules);
			return;
		}

		//Поиск и удаление ошибок, сгенерированных этим правилом
		self.removeRuleErrors(ruleContext);

		//Ищем непустые поля и не проверяем если все поля пусты и в правиле стоит галка Don't check if all fields are empty
		var nonEmptyFields = accordingFields.filter(function(af) { return af.field && af.field.value() });
		if (nonEmptyFields.length === 0 && ruleContext.rule.ignoreIfAllFieldsEmpty) {
			self.tryRecount([], rules);
			return;
		}

		if (!checkApplyCondition(ruleContext)) {
			self.tryRecount([], rules);
			return;
		}

		var accordingDataSet = self.datasets.filter(function(dataset) {
			return dataset.name === ruleContext.rule.dataSourceName;
		})[0];

		if (!accordingDataSet) {
			self.tryRecount([], rules);
			return;
		}

		var filteringKeys = accordingDataSet.columns
			.filter(function(column) { return column.isFilteringKey; })
			.map(function(column) { return column.name; });

		//Поля для поиска - поля, в которых включена галка поиска и имеющие значение, или с включенной галкой поиска даже при отсутствии значения
		var fieldsForSearch = accordingFields.filter(function(fieldForSearch) {
			//Убираем колонки фильтрации из поиска
			var dbColumnName = fieldForSearch.dbLookupfieldInfo.databaseAlias;
			if (filteringKeys.indexOf(dbColumnName) >= 0) return false;

			return fieldForSearch.dbLookupfieldInfo.shouldSearch &&
				((fieldForSearch.field && fieldForSearch.field.value()) || fieldForSearch.dbLookupfieldInfo.searchIfFieldEmpty);
		});

		//В сервис поиска имеет смысл передавать только те поля для которых есть значение
		var searchQueryParams = fieldsForSearch
			.filter(function(val) { return val.field && val.field.value() })
			.map(function(fieldForSearch) {
				return {
					FieldName: fieldForSearch.dbLookupfieldInfo.databaseAlias,
					FieldValue: fieldForSearch.field.value()
				}
			});

		//Если полей для поиска нет, то формируется ошибка о бессмысленности поиска
		if (fieldsForSearch.length === 0) {
			self.addRuleError(ruleContext, ruleErrorTypes.NotEnoughData);
			self.tryRecount([], rules);
		} else {
			FC.DataContext.DbCache.lookup(self.templateId(),
				self.templateVersion(),
				ruleContext.rule.dataSourceName,
				0,
				searchQueryParams,
				true,
				function(result) {
					if (!result) {
						self.tryRecount([], rules);
						return;
					}

					//Преобразуем список возможных вариантов к формату 
					// массив, где каждый вариант это map где ключ название столбца в БД а значение - множество значений из бд по этому ключу в нижнем регистре
					var variantsList = result.map(function(resultEntity) {
						var mappedResult = {};
						resultEntity.forEach(function(item) {
							if (!item.FieldValue) return;

							var values = [];
							if (mappedResult[item.FieldName]) {
								values = mappedResult[item.FieldName];
							} else {
								mappedResult[item.FieldName] = values;
							}
							values.push(item.FieldValue.trim());
						});

						return mappedResult;
					});

					//Отбираем варианты, подходящие по критерию поиска
					var suitableResults = variantsList.filter(function(variant) {
						return isSutableResult(variant, fieldsForSearch);
					});


					if (suitableResults.length === 0) {
						self.addRuleError(ruleContext, ruleErrorTypes.RecordNotFound);
						self.tryRecount([], rules);
						return;
					}

					//Check against database if completed
					var mismatchedFields = [];
					suitableResults = suitableResults.filter(function (suitableResult) {
						var badFields = processCheckAgainstDatabase(suitableResult, accordingFields);
						badFields.forEach(function(field) {
							if (mismatchedFields.indexOf(field) < 0) {
								mismatchedFields.push(field);
							}
						});
						return badFields.length === 0;
					});

					if (suitableResults.length === 0) {
						self.addRuleError(ruleContext,
							ruleErrorTypes.RecordMismatched,
							mismatchedFields.map(function(field) {
								return field.caption();
							}).join(', '));
						self.tryRecount([], rules);
						return;
					}

					if (suitableResults.length === 1) {
						var suitableResult = suitableResults[0];
						//Выбираем лучшие значения из справочника. Необходимо для колонок, с несколькими значениями
						var bestFieldsValues = findBestFieldValues(suitableResult, accordingFields);
						
						var changedFields = self.applyRule(ruleContext, bestFieldsValues);
						self.tryRecount(changedFields, rules, ruleContext);

					} else {
						self.addRuleError(ruleContext, ruleErrorTypes.TooManyRecords);
						self.tryRecount([], rules);
					}


				}
			);
		}
	}


}