﻿FC.BusinessModels.DocumentEditor.Document = function (document) {
	var self = this;
	document = document || {};

	self.id = ko.observable(document.Id);
	self.version = ko.observable(document.Version);
	self.taskId = ko.observable(document.TaskId);
	self.index = ko.observable(document.Index);
	self.externalId = ko.observable(document.ExternalId);
	self.templateName = ko.observable(document.TemplateName);
	self.projectId = ko.observable(document.ProjectId);
	self.templateId = ko.observable(document.TemplateId);
	self.templateVersion = ko.observable(document.TemplateVersion);
	self.batchId = ko.observable(document.BatchId);
	self.processingErrors = ko.observable(document.ProcessingErrors);
	self.processingWarnings = ko.observable(document.ProcessingWarnings);
	self.recognizedSymbolsCount = ko.observable(document.RecognizedSymbolsCount);
	self.verificationSymbolsCount = ko.observable(document.VerificationSymbolsCount);
	self.totalSymbolsCount = ko.observable(document.TotalSymbolsCount);
	self.symbolsForVerification = ko.observable(document.SymbolsForVerification);
	self.title = ko.observable(document.Title);
	self.comment = ko.observable(document.Comment);
	self.unchanged = ko.observable(document.Unchanged);
	self.useForTraining = ko.observable(document.UseForTraining);


	self.urlGenerator = function (pageId, pageVersion, hasColorImage) {
		return FC.Utils.getControllerUrl("DocumentPage/GetImage?"
			+ "projectId=" + self.projectId()
			+ "&batchId=" + self.batchId()
			+ "&documentId=" + self.id() + "&pageId=" + pageId +
			"&pageVersion=" + pageVersion + "&loadInColor=" + (window.batchApp.extensions.colorSwitcherMode() && hasColorImage) +
			"&sessionId="+FC.SessionInfo.SessionId);
	}

	self.errors = ko.observableArray([]);
	self.assemblingErrors = ko.observableArray([]);

	self.fieldsChangedTrigger = ko.observable();

	self.pages = ko.observableArray([]);
	self.namedProperties = ko.observableArray([]);

	//Fields tree
	self.fields = ko.observableArray([]);

	//All regions
	self.regions = ko.observableArray([]);

	var isRegionChild = function (parentId, supposedChildId, parentIsTable) {
		if (parentId.length !== supposedChildId.length - 1 && !parentIsTable ||
			parentId.length !== supposedChildId.length - 2 && parentIsTable) {

			return false;
		}

		for (var i = 0; i < parentId.length; i++) {
			if (parentId[i] !== supposedChildId[i]) {
				return false;
			}
		}

		return true;
	};

	self.documentRegions = ko.observableArray([]);

	//Array of pairs (field, region)
	self.fieldRegion = ko.observableArray([]);

	//Array of pairs (field,path)
	self.allFields = ko.observableArray([]);

	//Array of pairs (field, error)
	self.fieldErrors = ko.observableArray([]);

	self.validators = ko.observableArray([]);

	self.databaseCheckRules = ko.observableArray([]);

	self.datasets = ko.observableArray([]);

	if (document.Validators) {
		self.validators($.map(document.Validators, function (item) {
			return new FC.BusinessModels.DocumentEditor.FieldValidator(item);
		}));
	}

	if (document.DatabaseCheckRules) {
		self.databaseCheckRules($.map(document.DatabaseCheckRules, function (item) {
			return new FC.BusinessModels.DocumentEditor.DatabaseCheckRule(item);
		}));
	}

	if (document.DataSets) {
		self.datasets($.map(document.DataSets, function (item) {
			return new FC.BusinessModels.DocumentEditor.DataSet(item);
		}));
	}

	if (document.Pages) {
		self.pages($.map(document.Pages, function (item) {
			return new FC.BusinessModels.DocumentEditor.Page(item, self.urlGenerator);
		}));
	}

	if (document.NamedProperties) {
		self.namedProperties($.map(document.NamedProperties,
			function (item) {
				return new FC.BusinessModels.DocumentEditor.NamedProperty(item);
			}));
	}

	if (document.Errors) {
		self.errors($.map(document.Errors,
			function (item) {
				return new FC.BusinessModels.DocumentEditor.Error(item);
			}));
	}

	if (document.AssemblingErrors) {
		self.assemblingErrors($.map(document.AssemblingErrors,
			function (item) {
				return new FC.BusinessModels.DocumentEditor.AssemblingError(item);
			}));
	}

	self.manipulators = {
		errorsManipulator: new FC.BusinessModels.DocumentEditor
			.ErrorsManipulator(self.allFields, self.errors, self.assemblingErrors),
		regionsManipulator: new FC.BusinessModels.DocumentEditor
			.RegionsManipulator(self.allFields, self.regions, self.fieldRegion, self.documentRegions, self.fieldsChangedTrigger),
		suggestsManipulator: new FC.BusinessModels.DocumentEditor
			.SuggestsManipulator(self.fields),
		validatorsManipulator: new FC.BusinessModels.DocumentEditor.ValidatorManipulator(self.validators)
	}
	self.manipulators.dbCheckRulesManipulator =
		new FC.BusinessModels.DocumentEditor.DbCheckRulesManipulator(self.templateId, self.templateVersion, self.allFields,
		self.databaseCheckRules(),
			self.datasets(),
			self.fields,
		self.manipulators.regionsManipulator,
		self.manipulators.errorsManipulator
		);


	function processField(field, parent, parentRegions, allRegions, fieldRegions) {
		var tprocessField0 = performance.now();
		var fieldModel = null;

		var regions = [];
		if (field.Regions) {
			regions = $.map(field.Regions,
				function (reg) {
					var region = FC.BusinessModels.DocumentEditor.Region.createRegionForField(reg, field, self,
						FC.BusinessModels.DocumentEditor.Region.isRootRegion(parent));

					for (var i = 0; i < parentRegions.length; i++) {
						var parentRegion = parentRegions[i];
						if (isRegionChild(parentRegion.id(),
							region.id(),
							parentRegion instanceof FC.BusinessModels.DocumentEditor.TableRegion)) {
							parentRegion.regions.push(region);
							region.parent = parentRegion;
							break;
						}
					}
					return region;
				});
		}

		switch (field.FieldType) {
			case FC.BusinessModels.DocumentEditor.FieldTypes.Text:
				fieldModel = new FC.BusinessModels.DocumentEditor.TextField(field, self.manipulators, parent);
				break;
			case FC.BusinessModels.DocumentEditor.FieldTypes.Picture:
				fieldModel = new FC.BusinessModels.DocumentEditor.PictureField(field, self.manipulators, parent);
				break;
			case FC.BusinessModels.DocumentEditor.FieldTypes.Boolean:
				fieldModel = new FC.BusinessModels.DocumentEditor.BooleanField(field, self.manipulators, parent);
				break;
			case FC.BusinessModels.DocumentEditor.FieldTypes.Choice:
				fieldModel = new FC.BusinessModels.DocumentEditor.ChoiceField(field, self.manipulators, parent);
				if (field.Fields) {
					fieldModel.fields($.map(field.Fields,
						function (item) { return processField(item, fieldModel, regions, allRegions, fieldRegions) }));
				}
				break;
			case FC.BusinessModels.DocumentEditor.FieldTypes.GroupParent:
			case FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent:
				fieldModel = new FC.BusinessModels.DocumentEditor.FieldsGroup(field, self.manipulators, parent);
				if (field.Fields) {
					fieldModel.fields($.map(field.Fields,
						function (item) {
							if (regions.length === 0) {
								return processField(item, fieldModel, parentRegions, allRegions, fieldRegions);
							}

							return processField(item, fieldModel, regions, allRegions, fieldRegions);
						}));
				}
				break;
			case FC.BusinessModels.DocumentEditor.FieldTypes.Table:
				fieldModel = new FC.BusinessModels.DocumentEditor.Table(field, self.manipulators, parent);
				if (field.Rows) {
					fieldModel.rows($.map(field.Rows,
						function (item) {
							var row = new FC.BusinessModels.DocumentEditor.TableRow(item, self.manipulators, parent);
							row.parent = fieldModel;
							if (item.Cells) {
								row.cell($.map(item.Cells,
									function (cell) {
										return processField(cell, row, regions, allRegions, fieldRegions);
									}));
							}

							return row;
						}));
				}
				break;
		}

		if (fieldModel instanceof FC.BusinessModels.DocumentEditor.TextField && !fieldModel.isRecordFieldProcessed) {
			fieldModel.processDbChecks();
		}

		
		$.each(regions,
			function (key, val) {
				allRegions.push(val);
				fieldRegions.push({
					field: fieldModel,
					region: val
				});
			});

		FC.Utils.EventLogger.Default.addMessage("BMFUNC", "Document processField\t" + (performance.now() - tprocessField0) + "\tms");
		return fieldModel;
	}

	var createRootElement = function (field, allRegions, fieldRegions) {
		if (field.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Section) {

			var section = new FC.BusinessModels.DocumentEditor.Section(field, self, self.manipulators);
			if (field.Fields) {
				section.fields($.map(field
					.Fields,
					function (item) { return processField(item, section, [], allRegions, fieldRegions) }));
			}
			return section;
		} else {
			var sectionsGroup = new FC.BusinessModels.DocumentEditor.SectionsGroup(field, self, self.manipulators);
			if (field.Sections) {
				sectionsGroup.fields($.map(field.Sections,
					function (item) {
						var sect = new FC.BusinessModels.DocumentEditor.Section(item, null, self.manipulators);
						sect.parent = sectionsGroup;
						if (item.Fields) {
							sect.fields($.map(item
								.Fields,
								function (sectItem) { return processField(sectItem, sect, [], allRegions, fieldRegions) }));
						}

						return sect;
					}));
			}

			return sectionsGroup;
		}
	}
	var generateDocumentFields = function () {
		if (document.Fields) {
			var t0 = performance.now();
			var allRegions = [];
			var fieldRegions = [];
			self.fields($.map(document.Fields,
				function (field) {
					return createRootElement(field, allRegions, fieldRegions);
				}));


			self.documentRegions(allRegions.filter(function (val) { return val.id().length < 3 }));
			self.regions(allRegions);
			self.fieldRegion(fieldRegions);

			setTimeout(function() {
					$.each(self.allFields().filter(function(val) {
							return val.field instanceof FC.BusinessModels.DocumentEditor.TextField && val.field.isRecordIdField;
						}),
						function(index, fieldInfo) {
							var fieldValue = fieldInfo.field.value();
							if (fieldValue) {
								self.manipulators.dbCheckRulesManipulator.disableAccordingFields(fieldInfo.field);
							} else {
								self.manipulators.dbCheckRulesManipulator.enableAccordingFields(fieldInfo.field);
							}
						});
				},
				10);

			if (FC.Environment.CloneSuggestsForNewInstance) {
				self.manipulators.suggestsManipulator.updateDict();
			}

			FC.Utils.EventLogger.Default.addMessage("BMFUNC", "Document fieldGeneration\t" + (performance.now() - t0) + "\tms");
		}
	};

	generateDocumentFields();


	function analyzeField(field, path, allFieldsArray) {
		var t0 = performance.now();
		var currentPath = path.slice();
		currentPath.push(field.id());
		allFieldsArray.push({
			field: field,
			path: currentPath
		});
		if (field.fields && field.fields()) {
			$.each(field.fields(),
				function (key, subField) {
					analyzeField(subField, currentPath, allFieldsArray);
				});
		}
		if (field.rows && field.rows()) {
			$.each(field.rows(),
				function (key, subField) {
					analyzeField(subField, currentPath, allFieldsArray);
				});
		}
		if (field.cell && field.cell()) {
			$.each(field.cell(),
				function (key, subField) {
					analyzeField(subField, currentPath, allFieldsArray);
				});
		}


		FC.Utils.EventLogger.Default.addMessage("BMFUNC", "Document.analyzeField\t" + (performance.now() - t0) + "\tms");
	}


	ko.computed(function () {
		var t0 = performance.now();
		var res = [];
		if (!self.fields()) {
			return;
		}

		$.each(self.fields(),
			function (key, field) {
				analyzeField(field, [], res);
			});

		self.allFields(res);

		FC.Utils.EventLogger.Default.addMessage("BMFUNC", "Document.allFieldsComputed\t" + (performance.now() - t0) + "\tms");
	});

	/**
	 * Получение JS Объекта 
	 * @param {} fullSerialize  true - для всех свойств класса,  false для свойств класса необходимых при CHeck Rules и Apply Changes
	 * @returns {} 
	 */
	self.toJs = function (fullSerialize) {
		var t0 = performance.now();
		var res = {
			Fields: $.map(self.fields().filter(function (val) { return val; }), function (field) { return field.toJs(fullSerialize); }),
			Pages: $.map(self.pages(), function (page) { return page.toJs(); }),
			Id: self.id(),
			Version: self.version(),
			TaskId: self.taskId(),
			Index: self.index(),
			ExternalId: self.externalId(),
			TemplateName: self.templateName(),
			ProjectId: self.projectId(),
			BatchId: self.batchId(),
			TemplateId: self.templateId(),
			TemplateVersion: self.templateVersion()
		};
		if (fullSerialize) {
			res.Errors = $.map(self.errors(),
				function (error) {
					return error.toJs();
				});
			res.AssemblingErrors = $.map(self.assemblingErrors(),
				function (error) {
					return error.toJs();
				});
		}

		FC.Utils.EventLogger.Default.addMessage("BMFUNC", "Document.toJs\t" + (performance.now() - t0) + "\tms");
		return res;
	}

	var updateField = function (currentViewmodel, newField, parent, parentRegions, allRegions, fieldRegions) {
		var t0 = performance.now();
		var regions = [];
		if (newField.Regions) {
			regions = $.map(newField.Regions,
				function (reg) {
					var region = FC.BusinessModels.DocumentEditor.Region.createRegionForField(reg, newField, self,
						FC.BusinessModels.DocumentEditor.Region.isRootRegion(parent));

					for (var i = 0; i < parentRegions.length; i++) {
						var parentRegion = parentRegions[i];
						if (isRegionChild(parentRegion.id(),
							region.id(),
							parentRegion instanceof FC.BusinessModels.DocumentEditor.TableRegion)) {
							parentRegion.regions.push(region);
							region.parent = parentRegion;
							break;
						}
					}
					return region;
				});
		}
		if (((newField.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Text) &&
			(currentViewmodel instanceof FC.BusinessModels.DocumentEditor.TextField)) ||
			((newField.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Boolean) &&
				(currentViewmodel instanceof FC.BusinessModels.DocumentEditor.BooleanField)) ||
			((newField.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Choice) &&
				(currentViewmodel instanceof FC.BusinessModels.DocumentEditor.ChoiceField))) {
			currentViewmodel.updateInfo(newField);

			if ((newField.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Choice) &&
				(currentViewmodel instanceof FC.BusinessModels.DocumentEditor.ChoiceField)) {
				updateChildrenFields(currentViewmodel, currentViewmodel.fields, newField.Fields, regions, allRegions, fieldRegions);
			}
		} else if (((newField.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.GroupParent ||
			newField.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent) &&
			(currentViewmodel instanceof FC.BusinessModels.DocumentEditor.FieldsGroup)) ||
			((newField.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Section) &&
				(currentViewmodel instanceof FC.BusinessModels.DocumentEditor.Section)) ||
			((newField.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.SectionsGroup) &&
				(currentViewmodel instanceof FC.BusinessModels.DocumentEditor.SectionsGroup))) {
			if (regions.length === 0) {
				updateChildrenFields(currentViewmodel,
					currentViewmodel.fields,
					newField.Fields ? newField.Fields : newField.Sections,
					parentRegions,
					allRegions,
					fieldRegions);
			} else {
				updateChildrenFields(currentViewmodel, currentViewmodel.fields, newField.Fields, regions, allRegions, fieldRegions);
			}

			if ((newField.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Section) &&
				(currentViewmodel instanceof FC.BusinessModels.DocumentEditor.Section)) {
				currentViewmodel.pageIds(newField.PageIds);
			}

		} else if ((newField.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Table) &&
			(currentViewmodel instanceof FC.BusinessModels.DocumentEditor.Table)) {
			var newArray = [];
			if (newField.Rows) {
				$.each(newField.Rows,
					function (ind, row) {
						var rowViewmodel;
						for (var i = 0; i < currentViewmodel.rows().length; i++) {
							rowViewmodel = currentViewmodel.rows()[i];

							if (rowViewmodel.id() === row.Id) {
								updateChildrenFields(rowViewmodel, rowViewmodel.cell, row.Cells, regions, allRegions, fieldRegions);
								newArray.push(rowViewmodel);
								return;
							}
						}
						rowViewmodel = new FC.BusinessModels.DocumentEditor.TableRow(row, self.manipulators);
						rowViewmodel.parent = currentViewmodel;
						if (row.Cells) {
							rowViewmodel.cell($.map(row.Cells,
								function (cell) {
									return processField(cell, rowViewmodel, regions, allRegions, fieldRegions);
								}));
						}
						newArray.push(rowViewmodel);
					});
			}

			if (newArray.length !== currentViewmodel.rows().length) {
				currentViewmodel.rows(newArray);
			}
			for (var i = 0; i < newArray.length; i++) {
				if (newArray[i] != currentViewmodel.rows()[i]) {
					currentViewmodel.rows(newArray);
					break;
				}
			}
		}

		$.each(regions,
			function (key, val) {
				allRegions.push(val);
				fieldRegions.push({
					field: currentViewmodel,
					region: val
				});
			});

		FC.Utils.EventLogger.Default.addMessage("BMFUNC", "Document.updateField\t" + (performance.now() - t0) + "\tms");
	}

	var updateChildrenFields = function (parent, parentFieldsObservable, newFields, parentRegions, allRegions, fieldRegions) {
		var newArray = [];
		if (newFields && newFields.length > 0) {
			$.each(newFields,
				function (ind, subField) {
					for (var i = 0; i < parentFieldsObservable().length; i++) {
						var child = parentFieldsObservable()[i];
						if (child.id() === subField.Id) {
							updateField(child, subField, parent, parentRegions, allRegions, fieldRegions);
							newArray.push(child);
							return;
						}
					}
					newArray.push(processField(subField, parent, parentRegions, allRegions, fieldRegions));
				});
		}
		if (newArray.length !== parentFieldsObservable().length) {
			parentFieldsObservable(newArray);
		}
		for (var i = 0; i < newArray.length; i++) {
			if (newArray[i] != parentFieldsObservable()[i]) {
				parentFieldsObservable(newArray);
				break;
			}
		}
	};

	self.countVerificationSymbols = function () {
		var allFields = self.allFields();
		var count = 0;
		for (var i = 0; i < allFields.length; i++) {
			if (allFields[i] instanceof FC.BusinessModels.DocumentEditor.TextField) {
				count += allFields[i].verificationSymbolsCount();
			}
		}
		return count;
	};
	
	self.updateInfo = function (data) {
		var oldPages = self.pages();
		for (var pIndex = 0; pIndex < oldPages.length; pIndex++) {
			oldPages[pIndex].dispose();
		}

		if (!data) {
			self.pages([]);
			self.fields([]);
			self.errors([]);
			self.regions([]);
			self.fieldRegion([]);
			self.documentRegions([]);
			self.assemblingErrors([]);

			if (FC.Environment.CloneSuggestsForNewInstance) {
				self.manipulators.suggestsManipulator.updateDict();
			}

			return;
		}

		var t0 = performance.now();
		var allRegions = [];
		var fieldRegions = [];

		self.templateName(data.TemplateName);
		self.templateId(data.TemplateId);
		self.templateVersion(data.TemplateVersion);

		if (data.Pages) {
			self.pages($.map(data.Pages,
				function (item) {
					return new FC.BusinessModels.DocumentEditor.Page(item, self.urlGenerator);
				}));
		} else {
			self.pages([]);
		}

		var newArray = [];
		if (data.Fields) {
			$.each(data.Fields,
				function (index, field) {
					var fieldInViewmodel = null;
					for (var i = 0; i < self.fields().length; i++) {
						fieldInViewmodel = self.fields()[i];
						if (fieldInViewmodel.id() === field.Id) {
							newArray.push(fieldInViewmodel);
							updateField(fieldInViewmodel, field, null, [], allRegions, fieldRegions);
							return;
						}
					}

					var rootElement = createRootElement(field, allRegions, fieldRegions);

					newArray.push(rootElement);

				});
		}

		if (newArray.length !== self.fields().length) {
			self.fields(newArray);
		} else {
			for (var i = 0; i < newArray.length; i++) {
				if (newArray[i] != self.fields()[i]) {
					self.fields(newArray);
					break;
				}
			}
		}

		self.regions(allRegions);
		self.fieldRegion(fieldRegions);
		self.documentRegions(allRegions.filter(function (val) { return val.id().length < 3 }));

		if (data.Errors) {
			self.errors($.map(data.Errors,
				function (item) {
					return new FC.BusinessModels.DocumentEditor.Error(item);
				}));
		} else {
			self.errors([]);
		}

		if (data.AssemblingErrors) {
			self.assemblingErrors($.map(data.AssemblingErrors,
				function (item) {
					return new FC.BusinessModels.DocumentEditor.AssemblingError(item);
				}));
		} else {
			self.assemblingErrors([]);
		}

		if (FC.Environment.CloneSuggestsForNewInstance) {
			self.manipulators.suggestsManipulator.updateDict();
		}

		FC.Utils.EventLogger.Default.addMessage("BMFUNC", "Document.updateInfo\t" + (performance.now() - t0) + "\tms");
	}

	self.addClassificationError = function() {
		self.assemblingErrors.push(new FC.BusinessModels.DocumentEditor.AssemblingError({
			Type: -100,
			Description: FC.Environment.Localization.GlobalResources.ClassificationWarningText,
			DocumentNodeRef: [],
			Weight: 1,
			Position: 0,
			SectionName: ''
		}));
	}

	self.removeClassificationError = function() {
		self.assemblingErrors(self.assemblingErrors().filter(function (error) { return error.type() !== -100 }));
	}

};
