﻿FC.BusinessModels.DocumentEditor.ErrorsManipulator = function (allFields, errors, assemblingErrors) {
	var self = this;

	self.fields = allFields;
	self.errors = errors;

	self.fieldsErrors = ko.computed(function () {
		var t0 = performance.now();
		var fields = allFields();

		if (!fields || !errors() || !assemblingErrors())
			return [];

		var res = [];

		$.each(assemblingErrors(), function (key, error) {
			$.each(error.documentNodeRef(), function (dKey, path) {
				for (var i = 0; i < fields.length; i++) {
					var fieldInfo = fields[i];
					if (path.equals(fieldInfo.path)) {
						res.push({
							field: fieldInfo.field,
							error: error
						});
					}
				}
			});
		});

		$.each(errors(), function (key, error) {
			$.each(error.documentNodeRef(), function (dKey, path) {
				for (var i = 0; i < fields.length; i++) {
					var fieldInfo = fields[i];
					if (path.equals(fieldInfo.path)) {
						res.push({
							field: fieldInfo.field,
							error: error
						});
						break;
					}
				}
			});
		});

		FC.Utils.EventLogger.Default.addMessage("BMFUNC", "ErrorsManipulator fieldsErrors\t" + (performance.now() - t0) + "\tms");
		return res;
	});


	self.addError = function (fields, severity, type, message, ruleName, contextNodeRef) {
		var t0 = performance.now();
		if (!Array.isArray(fields)) {
			fields = [fields];
		}
		var nodeRef = fields.map(function(field) {
			return allFields().filter(function(val) { return val.field === field; })[0].path;
		});
		
		errors.push(new FC.BusinessModels.DocumentEditor.Error({
			DocumentNodeRef: nodeRef,
			Severity: severity,
			Type: type,
			Message: message,
			ContextNodeRef: contextNodeRef ?contextNodeRef:nodeRef[0],
			ContextNodFocusedNodeRefeRef: nodeRef[0],
			RuleName: ruleName
		}));
		FC.Utils.EventLogger.Default.addMessage("BMFUNC", "ErrorsManipulator addError\t" + (performance.now() - t0) + "\tms");
	}

	self.removeError = function (error) {
		errors.splice(errors().indexOf(error), 1);
	}

	self.removeFieldsErrors = function (fields) {
		var errorsArray = errors();

		(fields || []).forEach(function(field) {
			var fieldPath = field.getFieldKey();

			if (field.hasOwnProperty("fields") && field.fields()) {
				self.removeFieldsErrors(field.fields());
				errorsArray = errors();
			}

			errorsArray = errorsArray.filter(function(error) {
				var errorFieldsPaths = error.documentNodeRef();
				if (errorFieldsPaths.length === 1 && errorFieldsPaths[0].equals(fieldPath)) {
					return false;
				} else if ((error.type() === FC.BusinessModels.DocumentEditor.ErrorType.RuleError) &&
					(errorFieldsPaths.filter(function(ep) { return ep.equals(fieldPath) }).length > 0)) {
					return false;
				}
				return true;
			});

		});

		errors(errorsArray);
	};


}