﻿FC.BusinessModels.DocumentEditor.FieldsGroup = function (initInfo, manipulators) {
	FC.BusinessModels.DocumentEditor.BasicField.apply(this, arguments);

	initInfo = initInfo || {};

	FC.BusinessModels.DocumentEditor.FieldErrorExtend(manipulators, this);


	this.fields = ko.observableArray([]);

	this.dummy = ko.observable(initInfo.Dummy);

	this.hasTableLayout = ko.observable(initInfo.HasTableLayout || false);


	function createDummyViewmodel(field, parent) {
		var t0 = performance.now();
		var fieldModel = null;
		switch (field.FieldType) {
			case FC.BusinessModels.DocumentEditor.FieldTypes.Text:
				fieldModel = new FC.BusinessModels.DocumentEditor.TextField(field, manipulators);
				break;
			case FC.BusinessModels.DocumentEditor.FieldTypes.Picture:
				fieldModel = new FC.BusinessModels.DocumentEditor.PictureField(field, manipulators);
				break;
			case FC.BusinessModels.DocumentEditor.FieldTypes.Boolean:
				fieldModel = new FC.BusinessModels.DocumentEditor.BooleanField(field, manipulators);
				break;
			case FC.BusinessModels.DocumentEditor.FieldTypes.Choice:
				fieldModel = new FC.BusinessModels.DocumentEditor.ChoiceField(field, manipulators);
				break;
			case FC.BusinessModels.DocumentEditor.FieldTypes.GroupParent:
			case FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent:
				fieldModel = new FC.BusinessModels.DocumentEditor.FieldsGroup(field, manipulators);
				if (field.Fields) {
					fieldModel.fields($.map(field.Fields, function (item) { return createDummyViewmodel(item, fieldModel); }));
				}
				break;
			case FC.BusinessModels.DocumentEditor.FieldTypes.Table:
				fieldModel = new FC.BusinessModels.DocumentEditor.Table(field, manipulators);
				if (field.Rows) {
					fieldModel.rows($.map(field.Rows, function (item) {
						var row = new FC.BusinessModels.DocumentEditor.TableRow(item, manipulators);
						row.parent = fieldModel;
						if (item.Cells) {
							row.cell($.map(item.Cells,
								function (cell) {
									return createDummyViewmodel(cell, row);
								}));
						}

						return row;
					}));

				}

				break;
		}
		if (fieldModel) {
			fieldModel.parent = parent;
		}

		FC.Utils.EventLogger.Default.addMessage("BMFUNC", "FieldsGroup createDummyViewmodel\t" + (performance.now() - t0) + "\tms");
		return fieldModel;
	}

	this.createField = function () {
		var t0 = performance.now();
		var dummyField = this.dummy() ? createDummyViewmodel(this.dummy(), this) : null;
		if (dummyField && dummyField.id) {
			dummyField.id(-(FC.BusinessModels.DocumentEditor.BasicField.NewInstancesId++));
		}
		FC.Utils.EventLogger.Default.addMessage("BMFUNC", "FieldsGroup createField\t" + (performance.now() - t0) + "\tms");
		return dummyField;
	}
}

FC.BusinessModels.DocumentEditor.FieldsGroup.prototype = Object.create(FC.BusinessModels.DocumentEditor.BasicField.prototype);

FC.BusinessModels.DocumentEditor.FieldsGroup.prototype.constructor = FC.BusinessModels.DocumentEditor.FieldsGroup;

/**
 * Получение JS Объекта 
 * @param {} fullSerialize  true - для всех свойств класса,  false для свойств класса необходимых при CHeck Rules и Apply Changes
 * @returns {} 
 */
FC.BusinessModels.DocumentEditor.FieldsGroup.prototype.toJs = function (fullSerialize) {
	var basicJs = this.basicJs(fullSerialize);
	basicJs.Fields = $.map(this.fields().filter(function (val) { return val; }), function (field) { return field.toJs(fullSerialize); });
	if (fullSerialize) {
		basicJs.dummy = this.dummy();
		basicJs.hasTableLayout = this.hasTableLayout();
	}
	return basicJs;
}

FC.BusinessModels.DocumentEditor.FieldsGroup.prototype.addRepeatedItem = function () {
	return this.addRepeatedItemOnIndex(this.fields().length);
};

FC.BusinessModels.DocumentEditor.FieldsGroup.prototype.addRepeatedItemOnIndex = function (index) {
	var newInstance = this.createField();
	var instances = this.fields();
	if (index === 0) {
		instances.unshift(newInstance);
	} else if (index > instances.length - 1) {
		instances.push(newInstance);
	} else {
		instances.splice(index, 0, newInstance);
	}

	this.fields(instances);
	FC.Viewmodels.EventStack.addEvent();
	var getSubFields = function (field) {
		if (field instanceof FC.BusinessModels.DocumentEditor.TextField) {
			return [field];
		}
		var result = [];
		if (field.fields) {
			field.fields().forEach(function(subField) {
				var searchRes = getSubFields(subField);
				if (searchRes) {
					result = result.concat(searchRes);
				}
			});
			return result;
		}
		return null;
	}

	var fieldsInNewInstance = getSubFields(newInstance);
	this.manipulators.dbCheckRulesManipulator.tryRecount(fieldsInNewInstance, []);
	return newInstance;
};