﻿FC.BusinessModels.DocumentEditor.Region = function (initInfo, pages, isVisible, isLocked, isSectionRegion) {

	this.id = ko.observableArray(initInfo.Id);
	this.pages = pages;

	this.isNew = ko.observable(!!initInfo.IsNew);

	this.status = ko.observable(initInfo.Status);

	this.isSectionRegion = isSectionRegion;

	this.coordinates = ko.observableArray([]);
	this.pageId = ko.observable(initInfo.PageId);

	this.caption = ko.observable(initInfo.Caption);

	this.regions = ko.observableArray([]);

	this.isVisible = ko.observable(isVisible);

	this.isLocked = ko.observable(isLocked);

	this.parent = null;

	if (initInfo.Coordinates) {
		this.coordinates($.map(initInfo.Coordinates,
			function (value) {
				return new FC.BusinessModels.DocumentEditor.Coordinates(value);
			}));
	}

	this.regionPage = ko.observable(null);
	this.regionPageIndex = ko.observable(-1);

	var pageId = this.pageId();

	this.regionPage(this.pages().filter(function (page) { return page.id() === pageId; })[0]);
	this.regionPageIndex(this.pages().indexOf(this.regionPage()));
};

/**
 * Получить охватывающий прямоугольник для региона
 * @returns {}
 */
FC.BusinessModels.DocumentEditor.Region.prototype.getExternalRect = function () {
	var coords = ko.utils.unwrapObservable(this.coordinates);
	if (!coords || coords.length === 0) {
		return null;
	}

	var bigNumber = 1000000;
	var externalRect = { x1: bigNumber, y1: bigNumber, x2: -bigNumber, y2: -bigNumber };

	for (var i = 0; i < coords.length; i++) {
		var x1 = ko.utils.unwrapObservable(coords[i].x1);
		var x2 = ko.utils.unwrapObservable(coords[i].x2);
		var y1 = ko.utils.unwrapObservable(coords[i].y1);
		var y2 = ko.utils.unwrapObservable(coords[i].y2);
		if (x1 < externalRect.x1) {
			externalRect.x1 = x1;
		}
		if (x2 > externalRect.x2) {
			externalRect.x2 = x2;
		}
		if (y1 < externalRect.y1) {
			externalRect.y1 = y1;
		}
		if (y2 > externalRect.y2) {
			externalRect.y2 = y2;
		}
	}

	return externalRect;
};

/**
 * Сравнить два региона геометрически (регион выше и левее "меньше" региона, который ниже и правее)
 * @param {} r1 
 * @param {} r2 
 * @returns {} -1 если r2 больше, 0 если их вырхние углы совпадают, 1 если r1 больше
 */
FC.BusinessModels.DocumentEditor.Region.compareRegions = function (r1, r2) {
	// Если они на разных страницах, то сравниваем страницы. Чем больше id, тем ниже страница
	if (r1.pageId() !== r2.pageId()) {
		return r1.pageId() - r2.pageId();
	}

	var externalRect1 = r1.getExternalRect();
	var externalRect2 = r2.getExternalRect();

	// если прямоугольники пересекаются по вертикали (одна строка), то сравним по горизонтали
	if (externalRect1.y1 < externalRect2.y2 && externalRect2.y1 < externalRect1.y2) {
		return externalRect1.x1 - externalRect2.x1;
	}

	return externalRect1.y1 - externalRect2.y1;
};

FC.BusinessModels.DocumentEditor.Region.createRegionForField = function (regionData, field, documentModel, isRootRegion) {
	regionData.Caption = field.Caption || field.Name;

	if (field.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Table) {
		return new FC.BusinessModels.DocumentEditor.TableRegion(regionData, field, documentModel.pages, field.IsVisible, field.isSegment, isRootRegion);
	}

	if (field.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.GroupParent ||
		field.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Choice ||
		field.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent) {

		return new FC.BusinessModels.DocumentEditor.GroupRegion(regionData, documentModel.pages, field.IsVisible, field.isSegment, isRootRegion);
	}

	return new FC.BusinessModels.DocumentEditor.Region(regionData, documentModel.pages, field.IsVisible, field.isSegment, isRootRegion);
};

FC.BusinessModels.DocumentEditor.Region.isRootRegion = function (fieldParent) {
	if (!fieldParent || (fieldParent instanceof FC.BusinessModels.DocumentEditor.Section)) {
		return true;
	}

	if (fieldParent instanceof FC.BusinessModels.DocumentEditor.FieldsGroup && 
		fieldParent.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent &&
		(!fieldParent.parent || (fieldParent.parent instanceof FC.BusinessModels.DocumentEditor.Section))) {
		return true;
	}

	return false;
};
