﻿FC.BusinessModels.DocumentEditor.RegionsManipulator = function (allFields, allRegions, fieldRegionMap, documentRegions, fieldsChangedTrigger) {


	this.fieldRegionMap = fieldRegionMap;
	this.allFields = allFields;
	this.allRegions = allRegions;
	this.documentRegions = documentRegions;
	this.fieldsChangedTrigger = fieldsChangedTrigger;
	

	var fieldIdRegion = ko.computed(function() {
			var res = [];
			$.each(this.fieldRegionMap(),
				function(ind, val) {
					var regions = res[val.field.internalId];
					if (!regions) {
						regions = [];
						res[val.field.internalId] = regions;
					}
					regions.push(val);
				});

			return res;
		},
		this);

	this.getFieldRegions = function(field) {
		var res = fieldIdRegion()[field.internalId];
		return res ? res : [];
	}

	this.removeFields = function (fields, persistant) {
		var t0 = performance.now();

		if (!Array.isArray(fields)) {
			fields = [fields];
		}

		var getFieldsWithChildren = function (analyzedField) {
			var res = [];
			res.push(analyzedField);
			var analyzeSubfields = function (subfieldFunc) {
				if (analyzedField.hasOwnProperty(subfieldFunc)) {
					analyzedField[subfieldFunc]().forEach(function(analyzedSubField) {
						res = res.concat(getFieldsWithChildren(analyzedSubField));
					});
				}
			}
			analyzeSubfields('rows');
			analyzeSubfields('cell');
			analyzeSubfields('fields');

			return res;
		}

		var fieldsWithChildren = [];
		fields.forEach(function (field) { fieldsWithChildren = fieldsWithChildren.concat(getFieldsWithChildren(field)); });
		var regionsToDelete = this.fieldRegionMap().filter(function(fr) {
			return fr && fieldsWithChildren.indexOf(fr.field) >= 0;
		}).map(function (fr) { return fr.region; });

		this.deleteRegions(regionsToDelete, -1, persistant);

		this.fieldsChangedTrigger.valueHasMutated();

		FC.Utils.EventLogger.Default.addMessage("BMFUNC", "Document processField\t" + (performance.now() - t0) + "\tms");
	}

	this.deleteRegions = function (regions, paragraphIndex, persistant) {
		if (!Array.isArray(regions)) {
			regions = [regions];
		}

		var i;

		var regionFields = this.fieldRegionMap().filter(function (fr) { return regions.indexOf(fr.region)>=0; });

		for (i = 0; i < regionFields.length; i++) {
			if (regionFields[i].field instanceof FC.BusinessModels.DocumentEditor.TextField) {
				regionFields[i].field.deleteParagraph(paragraphIndex);
			} else if (regionFields[i].field instanceof FC.BusinessModels.DocumentEditor.ChoiceField) {
				var wasVerified = regionFields[i].field.isVerified();
				regionFields[i].field.values([]);
				regionFields[i].field.isVerified(wasVerified);
			}
		}

		var regionsToDelete = [];
		var regionsToMark = [];
		if (persistant) {
			regionsToDelete = regions;
		} else {
			regions.forEach(function(region) {
				if (region.isNew()) {
					regionsToDelete.push(region);
				} else {
					regionsToMark.push(region);
				}
			});
		}
		

		this.fieldRegionMap(this.fieldRegionMap().filter(function (fr) { return regionsToDelete.indexOf(fr.region) < 0; }));
		this.allRegions(this.allRegions().filter(function (r) { return regionsToDelete.indexOf(r) < 0; }));
		regionsToMark.forEach(function(region) {
			region.status(0);
			region.coordinates([]);
		});

		if (regionsToDelete.length > 0) {
			var regionsAll = this.allRegions();
			for (i = 0; i < regionsAll.length; i++) {
				var childRegions = regionsAll[i].regions();

				if (childRegions.length > 0) {
					for (var j = 0; j < regionsToDelete.length; j++) {
						var index = childRegions.indexOf(regionsToDelete[j]);
						if (index > -1) {
							childRegions.splice(index, 1);
							regionsAll[i].regions(childRegions);
							regionsToDelete.splice(j, 1);
							j--;
						}
					}

					if (regionsToDelete.length === 0) {
						break;
					}
				}
			}

			this.documentRegions(
				this.allRegions()
					.filter(function (val) { return val.isSectionRegion }));
		}
		
	};
}