﻿FC.BusinessModels.DocumentEditor.Table = function (initInfo, manipulators) {


	initInfo = initInfo || {};

	FC.BusinessModels.DocumentEditor.BasicField.apply(this, arguments);
	FC.BusinessModels.DocumentEditor.FieldErrorExtend(manipulators, this);

	this.manipulators = manipulators;


	this.dummyRow = ko.observable(initInfo.DummyRow);
	this.rows = ko.observableArray([]);
	this.hideableColumns = ko.observable(initInfo.HideableColumns);
	this.isRepeatableGroup = initInfo.IsRepeatableGroup;


}

FC.BusinessModels.DocumentEditor.Table.prototype = Object.create(FC.BusinessModels.DocumentEditor.BasicField.prototype);

FC.BusinessModels.DocumentEditor.Table.prototype.constructor = FC.BusinessModels.DocumentEditor.Table;

/**
 * Получение JS Объекта 
 * @param {} fullSerialize  true - для всех свойств класса,  false для свойств класса необходимых при CHeck Rules и Apply Changes
 * @returns {} 
 */
FC.BusinessModels.DocumentEditor.Table.prototype.toJs = function (fullSerialize) {

	var basicJs = this.basicJs(fullSerialize);

	basicJs.DummyRow = this.dummyRow();
	basicJs.Rows = $.map(this.rows(), function (row) { return row.toJs(fullSerialize); });
	basicJs.IsRepeatableGroup = this.isRepeatableGroup;
	basicJs.Regions = $.map(this.manipulators.regionsManipulator.getFieldRegions(this),
		function (regionField) {
			return {
				Id: regionField.region.id(),
				Status: regionField.region.status(),
				PageId: regionField.region.pageId(),
				IsNew: regionField.region.isNew(),
				Columns: $.map(regionField.region.columns(), function (column) {
					return {
						Id: column.id(),
						LeftPos: column.leftPos(),
						RightPos: column.rightPos()
					}
				}),
				HorizontalSeparators: ko.toJS(regionField.region.horizontalSeparators),
				Coordinates: $.map(regionField.region.coordinates(),
					function (val) {
						return {
							X1: val.x1(),
							X2: val.x2(),
							Y1: val.y1(),
							Y2: val.y2()
						}
					})
			}
		});

	if (fullSerialize) {
		basicJs.dummyRow = this.dummyRow();
		basicJs.hideableColumns = this.hideableColumns();
	}
	return basicJs;
}

FC.BusinessModels.DocumentEditor.Table.prototype.createRow = function () {
	var t0 = performance.now();
	if (!this.dummyRow()) {
		return null;
	}
	var manipulators = this.manipulators;

	var dummyRow = this.dummyRow();
	var row = new FC.BusinessModels.DocumentEditor.TableRow(dummyRow, manipulators);
	var newId = this.rows().filter(function(row) {
		return row.id() <= 0;
	}).map(function(row) {
		return row.id();
	}).sort(function(left, right) {
		if (left < right) return -1;
		return left > right ? 1 : 0;
		})[0];

	if (newId) {
		newId--;
	} else {
		newId = -1;
	}
	row.id(newId);
	row.parent = this;
	if (dummyRow.Cells) {
		row.cell($.map(dummyRow.Cells,
			function (cell) {
				var res;
				if (cell.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Text) {
					res = new FC.BusinessModels.DocumentEditor.TextField(cell, manipulators);
				} else if (cell.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Boolean) {
					res = new FC.BusinessModels.DocumentEditor.BooleanField(cell, manipulators);
				} else if (cell.FieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Picture) {
					res = new FC.BusinessModels.DocumentEditor.PictureField(cell, manipulators);
				}

				if (res) {
					res.parent = row;
				}
				return res;
			}));
	}
	FC.Utils.EventLogger.Default.addMessage("BMFUNC", "Table createRow\t" + (performance.now() - t0) + "\tms");
	return row;
}

FC.BusinessModels.DocumentEditor.Table.prototype.deleteRow = function (row) {
	this.deleteRows([row], true);
};

FC.BusinessModels.DocumentEditor.Table.prototype.deleteRows = function (rowsToDelete) {
	var rows = this.rows();
	var i;
	var j;

	for (i = rows.length-1; i >=0; i--) {
		if (rowsToDelete.indexOf(rows[i]) >= 0) {
				rows.splice(i, 1);
		}
	}

	this.rows(rows);

	var cells = [];
	rowsToDelete.forEach(function (currentRowCells) {
		if (currentRowCells) {
			currentRowCells.cell().forEach(function(currentCell) {
				cells.push(currentCell);
			});
		}
	});

	this.manipulators.errorsManipulator.removeFieldsErrors(cells);
	this.manipulators.regionsManipulator.removeFields(rowsToDelete);
	

	for (i = 0; i < cells.length; i++) {
		if (cells[i].hasOwnProperty("dispose")) {
			cells[i].dispose();
		}
	}

	FC.Viewmodels.EventStack.addEvent();
};

FC.BusinessModels.DocumentEditor.Table.prototype.addTableRowAtPosition = function (index, tableBlockId) {
	var newRow = this.createRow();
	newRow.tableBlockId(tableBlockId || []);

	var rows = this.rows();
	var tableBlockJson = JSON.stringify(tableBlockId);

	// нужно найти rowNumber и поменять rowNumber у существующих
	if (rows.length > 0 && tableBlockId) {
		var i;

		for (i = index - 1; i >= 0; i--) {
			if (JSON.stringify(rows[i].tableBlockId()) === tableBlockJson) {
				newRow.rowNumber(rows[i].rowNumber() + 1);
				break;
			}
		}
	}

	if (index > this.rows().length - 1) {
		rows.push(newRow);
	} else {
		rows.splice(index, 0, newRow);
	}


	this.rows(rows);
	this.manipulators.dbCheckRulesManipulator.tryRecount(newRow.cell(), []);
	FC.Viewmodels.EventStack.addEvent();

	return newRow;
};