﻿FC.BusinessModels.DocumentEditor.TextField = function (initInfo, manipulators) {
	FC.BusinessModels.DocumentEditor.SimpleField.apply(this, arguments);

	initInfo = initInfo || {};
	FC.BusinessModels.DocumentEditor.FieldErrorExtend(manipulators, this);


	this.value = ko.observable(FC.Utils.Serialize.htmlDecode(initInfo.Value || ''));
	this.flags = ko.observable(initInfo.Flags || '');
	this.suggests = ko.observableArray(initInfo.Suggests || []);
	this.textType = ko.observable(initInfo.TextType || false);

	// параграфы текста, взятые из разных регионов.
	this.paragraphs = ko.observableArray([]);
	if (initInfo.Paragraphs) {
		this.paragraphs(initInfo.Paragraphs.map(function (p) {
			return new FC.BusinessModels.DocumentEditor.Paragraph(p);
		}));
	}


	this.isRecordFieldProcessed = false;
	this.isRecordIdField = false;


	var fieldRef = this;
	this.hasUncertainSymbol = ko.computed(function () {
		var flags = fieldRef.flags();
		for (var i = 0; i < flags.length; i++) {
			if (flags[i] !== '0' && flags[i] !== '\n') {
				return true;
			}
		}
		return false;
	});

	this.verificationSymbolsCount = ko.computed(function () {
		if (fieldRef.isVerified()) {
			return 0;
		}

		var flags = fieldRef.flags();
		var count = 0;
		for (var i = 0; i < flags.length; i++) {
			if (flags[i] !== '0' && flags[i] !== '\n') {
				count++;
			}
		}

		return count;
	});

	this.hasRegion = ko.computed(function () {
		var fieldRegions = manipulators.regionsManipulator.fieldRegionMap();
		for (var i = 0; i < fieldRegions.length; i++) {
			if (fieldRegions[i].field === fieldRef && fieldRegions[i].region.status() !== 0) {
				return true;
			}
		}

		return false;
	});


	this.getValidator = function () {
		return manipulators.validatorsManipulator.getValidator(this);
	};

	this.manipulators = manipulators;

	//true если значение пришло из CheckRules или из датасета
	this.valueAuthorized = true;

};

FC.BusinessModels.DocumentEditor.TextField.prototype = Object.create(FC.BusinessModels.DocumentEditor.SimpleField.prototype);

FC.BusinessModels.DocumentEditor.TextField.prototype.constructor = FC.BusinessModels.DocumentEditor.TextField;

FC.BusinessModels.DocumentEditor.TextField.prototype.processDbChecks = function () {
	this.isRecordFieldProcessed = true;
	var dbCheckRulesManipulator = this.manipulators.dbCheckRulesManipulator;
	this.isRecordIdField = dbCheckRulesManipulator.checkFieldIsRecordId(this);

	if (this.isRecordIdField) {
		var field = this;
		this.value.subscribe(function (val) {
			if (val) {
				dbCheckRulesManipulator.disableAccordingFields(field);
			} else {
				dbCheckRulesManipulator.enableAccordingFields(field);
			}
		});
	}
}

/**
 * Получение JS Объекта 
 * @param {} fullSerialize  true - для всех свойств класса,  false для свойств класса необходимых при CHeck Rules и Apply Changes
 * @returns {} 
 */
FC.BusinessModels.DocumentEditor.TextField.prototype.toJs = function (fullSerialize) {
	var simpleJs = this.simpleJs(fullSerialize);
	simpleJs.Value = FC.Utils.Serialize.htmlEncode(this.value());
	simpleJs.Flags = this.flags();
	simpleJs.TextType = this.textType();
	if (fullSerialize) {
		simpleJs.Suggests = this.suggests();
	}
	return simpleJs;
}

FC.BusinessModels.DocumentEditor.TextField.prototype.dispose = function () {
	this.hasRegion.dispose();
	this.hasUncertainSymbol.dispose();
	this.fieldErrors.dispose();
};

FC.BusinessModels.DocumentEditor.TextField.prototype.getDefaultSuggests = function () {
	var computedSuggests = this.manipulators.suggestsManipulator.suggestsDict()[this.getFieldKeyWithoutRepeateIndex().join()];
	return computedSuggests ? computedSuggests : [];
};

FC.BusinessModels.DocumentEditor.TextField.prototype.setParagraphText = function (text, paragraphIndex, addNewLine) {
	var flags = this.flags();
	var newFlagPart = "";
	for (var i = 0; i < text.length; i++) {
		newFlagPart += "0";
	}

	if (paragraphIndex < 0) {
		this.value(text);
		this.flags(newFlagPart);
		return;
	}

	if (text.length === 0) {
		text = " ";
		newFlagPart = "0";
	}

	if (addNewLine) {
		text += "\n";
		newFlagPart += "\n";
	}

	var currentTextParagraph = 0;
	var indexBegin = -1;
	var indexEnd = -1;
	for (var k = 0; k < flags.length; k++) {
		if (currentTextParagraph === paragraphIndex && indexBegin < 0) {
			indexBegin = k;
			if (addNewLine) {
				indexEnd = k;
				break;
			}
		}

		if (flags[k] === '\n') {
			if (currentTextParagraph === paragraphIndex && indexBegin > -1) {
				indexEnd = k;
				break;
			}
			currentTextParagraph++;
		}
	}

	if (indexBegin > -1 && indexEnd === -1) {
		indexEnd = flags.length;
	}

	var val = this.value();
	this.value(val.substring(0, indexBegin) + text + val.substring(indexEnd));
	this.flags(flags.substring(0, indexBegin) + newFlagPart + flags.substring(indexEnd));
	if (val !== this.value) {
		this.manipulators.dbCheckRulesManipulator.tryRecount([this], []);
	}
};

FC.BusinessModels.DocumentEditor.TextField.prototype.addParagraph = function (text, paragraphIndex) {
	this.setParagraphText(text, paragraphIndex, true);
};

FC.BusinessModels.DocumentEditor.TextField.prototype.deleteParagraph = function (paragraphIndex) {
	var flags = this.flags();

	if (paragraphIndex < 0) {
		this.value("");
		this.flags("");
		return;
	}

	var currentTextParagraph = 0;
	var indexBegin = -1;
	var indexEnd = -1;
	for (var k = 0; k < flags.length; k++) {
		if (currentTextParagraph === paragraphIndex && indexBegin < 0) {
			indexBegin = k;
		}

		if (flags[k] === '\n') {
			if (currentTextParagraph === paragraphIndex && indexBegin > -1) {
				indexEnd = k + 1;
				break;
			}
			currentTextParagraph++;
		}
	}

	if (indexBegin > -1 && indexEnd === -1) {
		indexEnd = flags.length;
	}

	var val = this.value();
	this.value(val.substring(0, indexBegin) + val.substring(indexEnd));
	this.flags(flags.substring(0, indexBegin) + flags.substring(indexEnd));
};

FC.BusinessModels.DocumentEditor.TextField.prototype.updateInfo = function (newInfo) {
	this.valueAuthorized = true;
	this.value(FC.Utils.Serialize.htmlDecode(newInfo.Value || ''));
	this.flags(newInfo.Flags || '');
	this.suggests(newInfo.Suggests || []);
	this.textType(newInfo.TextType || []);
	this.isVerified(newInfo.IsVerified || false);
	this.isReadOnly(newInfo.IsReadOnly || false);
	this.needRecognize(newInfo.NeedRecognize);
	if (newInfo.Paragraphs) {
		this.paragraphs(newInfo.Paragraphs.map(function (p) {
			return new FC.BusinessModels.DocumentEditor.Paragraph(p);
		}));
	} else {
		this.paragraphs([]);
	}
};