﻿;
(function() {

	function FramesCommunicator(communicator, commands, iframeelem) {
		var self = this;
		self.iframeelement = iframeelem;
		self.communicator = communicator;
		self.sendIds = [];
		self.commands = commands;

		var getEventName = function(id) {
			return "result-" + id + "-received";
		};

		self.sendCommand = function(method, args, callbackFunction) {
			var id;

			do {
				id = parseInt(Math.random() * 999999);
			} while (self.sendIds.indexOf(id) !== -1);
			self.sendIds.push(id);
			var jsonrpcObject = {
				jsonrpc: "2.0",
				method: method,
				params: args,
				id: id
			};
			if (callbackFunction) window.addEventListener(getEventName(id), callbackFunction);

			self.communicator.postMessage(JSON.stringify(jsonrpcObject), "*");
			return id;
		};

		self.sendResponse = function(id, result, error) {
			var jsonrpcObject = {
				jsonrpc: "2.0",
				id: id
			};
			if (error) {
				jsonrpcObject.error = error;
				alert(error); //NOSONAR no expose sensitive information
			} else {
				if (!result)
					result = {};
				jsonrpcObject.result = result;
			}

			self.communicator.postMessage(JSON.stringify(jsonrpcObject), "*");
		};

		self.processResult = function(resultObject) {

			var customEvent = document.createEvent("CustomEvent");
			customEvent.initCustomEvent(getEventName(resultObject.id),
				false,
				false,
				{ result: resultObject.result, error: resultObject.error });

			window.dispatchEvent(customEvent);
			var index = self.sendIds.indexOf(resultObject.id);
			if (index >= 0) {
				self.sendIds.splice(index, 1);
			}
		};

		self.processCommand = function(commandObject) {
			try {
				self.commands[commandObject.method].call(self, commandObject, self);
			} catch (e) {
				self.sendResponse(commandObject.id, null, e.toString());
			}
		};

		self.executeAsync = function(command, args, callbackFunction) {
			self.sendCommand(command, args, callbackFunction);
		};

		var eventMethod = window.addEventListener ? "addEventListener" : "attachEvent";
		var eventer = window[eventMethod];
		var messageEvent = eventMethod === "attachEvent" ? "onmessage" : "message";

		eventer(messageEvent,
			function(e) {
				var source = e.source;
				if (source !== self.communicator)
					return;
				var key = e.message ? "message" : "data";
				var data = e[key];
				var messageObject = JSON.parse(data);
				if (messageObject.method) {
					self.processCommand(messageObject);
				} else {
					self.processResult(messageObject);
				}
			},
			false);
	}

	function customScriptLauncher(parentFunctionsObject, frameFunctionsObject, extraScripts, extraStyles) {

		var self = this;
		var iframeNode = null;
		var communicator = null;
		var parentFunctions = parentFunctionsObject;
		var frameFunctions = frameFunctionsObject;

		parentFunctionsObject.setStyles = function(commandArgs) {
			if (commandArgs.params)
				this.iframeelement.css(commandArgs.params);
			this.sendResponse(commandArgs.id, true);
		};

		var extendFrameFunctions = function() {
			frameFunctions.getWindowStyles = function(commandArgs) {
				var self = this;
				var element = !commandArgs.params.element ? "body" : commandArgs.params.element;
				var dom = $(element).get(0);
				var style = null;
				var returns = {};
				var prop;
				if (window.getComputedStyle) {
					var camelize = function(a, b) {
						return b.toUpperCase();
					};
					style = window.getComputedStyle(dom, null);
					for (var i = 0, l = style.length; i < l; i++) {
						prop = style[i];
						var camel = prop.replace(/\-([a-z])/g, camelize);
						var val = style.getPropertyValue(prop);
						returns[camel] = val;
					}
					self.sendResponse(commandArgs.id, returns);
				}
				if (style && style === dom.currentStyle) {
					for (prop in style) {
						if (style.hasOwnProperty(prop)) {
							returns[prop] = style[prop];
						}
					}
					self.sendResponse(commandArgs.id, returns);
				}
				self.sendResponse(commandArgs.id, $(element).css());
			};
		};

		function getMethods(obj) {
			var result = [];
			for (var id in obj) {
				if (obj.hasOwnProperty(id)) {
					try {
						if (typeof obj[id] === "function") {
							result.push(id + ": " + obj[id].toString());
						}
					} catch (err) {
						result.push(id + ": inaccessible");
					}
				}
			}
			return result;
		}

		var createFrameContent = function() {
			extendFrameFunctions();

			var htmlString = '';
			if (extraScripts) {
				for (var extraScriptsIndex in extraScripts) {
					if (!extraScripts.hasOwnProperty(extraScriptsIndex))
						continue;
					htmlString += '<script src="' + extraScripts[extraScriptsIndex] + '"  type="text/javascript" ></script>';
				}
			}
			if (extraStyles) {
				for (var extraStylesIndex in extraStyles) {
					if (!extraStyles.hasOwnProperty(extraStylesIndex))
						continue;
					htmlString += '<link href="' + extraStyles[extraStylesIndex] + '"  rel="stylesheet" ></link>';
				}
			}

			var communicatorFunc = FramesCommunicator.toString();
			var communicatorFuncName = FramesCommunicator.name || communicatorFunc.match(/^function\s*([^\s(]+)/)[1];

			htmlString += '<script type="text/javascript" >' + communicatorFunc + '</script>';
			htmlString += '<script type="text/javascript" >var frameFunctionsObject={' +
				getMethods(frameFunctions).join(",") +
				'};var communicator = new ' +
				communicatorFuncName +
				'(window.parent,frameFunctionsObject);</script>';
			return htmlString;
		};

		var createIFrame = function(insertPath) {

			var iframeName = "CustomScriptLauncher-";
			var numOfIframes = $("iframe[name^='" + iframeName + "']").length;
			iframeName += numOfIframes;


			var cframe = $('<iframe></iframe>');
			cframe.attr("name", iframeName);
			cframe.attr("id", iframeName);
			cframe.css('display', "none");
			$(insertPath).append(cframe);


			var iframe = cframe.get(0);
			var iframeDocument = iframe.contentWindow || iframe.contentDocument;
			if (iframeDocument.document) {
				iframeDocument = iframeDocument.document;
			}

			var htmlFrameContent = createFrameContent();
			iframeDocument.open();
			iframeDocument.write(htmlFrameContent);
			iframeDocument.close();

			cframe.attr("sandbox", "allow-scripts");

			return cframe;
		};

		self.launch = function(insertPath, callArgs, callbackSuccessed, onLaunchSuccessed) {
			iframeNode = createIFrame(insertPath);

			iframeNode.load(function() {
				var domElement = iframeNode.get(0);
				communicator = new FramesCommunicator(domElement.contentWindow, parentFunctions, iframeNode);
				communicator.executeAsync("start", callArgs, callbackSuccessed);
				if (onLaunchSuccessed) {
					onLaunchSuccessed(communicator);
				}
			});
		};

		self.destroy = function() {
			if (iframeNode) {
				iframeNode.remove();
				iframeNode = null;
			}
		};
	}

	window.CustomScriptLauncher = customScriptLauncher;
})();