﻿FC.DataContext = FC.DataContext || {};

FC.DataContext.Batch = (function () {

	var batchApiAjax = function (url, data, onAnyResult, onSuccess, onError) {
		$.ajax({
			url: url,
			type: "GET",
			data: data,
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			traditional: true,
			success: function (result) {
				if (onAnyResult) {
					onAnyResult();
				}

				if (result.Error) {
					if (onError) {
						onError(result.Error);
					} else {
						FC.Utils.HandleServerError(result.Error);
					}

					return;
				}

				if (result.RedirectUrl) {
					FC.Utils.replaceLocation(FC.Utils.prepareRedirectUrl(result.RedirectUrl));
					return;
				}

				if (onSuccess) {
					onSuccess(result.Result);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				if (onAnyResult) {
					onAnyResult();
				}

				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	};

	return {
		sendToStage: function (sendBatch, progressBar, count, batchId, userId, stageId, priority, comment, createTask, onSuccess, onFailure) {
			$.ajax({
				url: FC.Utils.getControllerUrl("BatchApi/SendToStage"),
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchId: batchId,
					userId: userId,
					stageId: stageId,
					priority: priority,
					createTask: createTask,
					comment: FC.Utils.EncodeHTML(comment)
				},
				type: "POST",
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success:
					function (result) {
						if (result.Error) {
							FC.Utils.HandleServerError(result.Error);

							if (onFailure) {
								onFailure();
							}

							return;
						}

						if (onSuccess) {
							onSuccess();
							sendBatch();

							if (count > 1) {
								progressBar();
							}
						}
					},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);

					if (onFailure) {
						onFailure();
					}
				}
			});
		},

		deleteBatches: function (batchIds, onSuccess, onError, closeSession) {
			$.ajax({
				url: FC.Utils.getControllerUrl("BatchApi/DeleteBatches"),
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchIds: batchIds,
					closeSession: closeSession
				},
				async: !closeSession,
				type: "POST",
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success:
					function (result) {
						if (result.Error) {
							FC.Utils.HandleServerError(result.Error);

							if (onError) {
								onError();
							}
							return;
						}

						if (onSuccess) {
							onSuccess();
						}
					},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);

					if (onError) {
						onError();
					}
				}
			});
		},

		setPriority: function (batchIds, priorityId, onSuccess) {
			$.ajax({
				url: FC.Utils.getControllerUrl("BatchApi/SetPriority"),
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchIds: batchIds,
					priorityId: priorityId
				},
				type: "POST",
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success:
					function (result) {
						if (result.Error) {
							FC.Utils.HandleServerError(result.Error);
							return;
						}

						if (onSuccess) {
							onSuccess();
						}
					},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		sendToExceptions: function (batchId, comment, createTask, onSuccess) {
			$.ajax({
				url: FC.Utils.getControllerUrl("BatchApi/SendToExceptions"),
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchId: batchId,
					createTask: createTask,
					comment: FC.Utils.EncodeHTML(comment)
				},
				type: "POST",
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success:
					function (result) {
						if (result.Error) {
							FC.Utils.HandleServerError(result.Error);
							return;
						}

						if (onSuccess) {
							onSuccess();
						}
					},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		getBatchDocuments: function (batchId, onSuccess) {
			$.ajax({
				url: FC.Utils.getControllerUrl("BatchApi/GetBatchDocuments"),
				type: "GET",
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchId: batchId
				},
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success: function (result) {
					if (result.Error) {
						FC.Utils.HandleServerError(result.Error);
						return;
					}

					if (result.RedirectUrl) {
						FC.Utils.replaceLocation(FC.Utils.prepareRedirectUrl(result.RedirectUrl));
						return;
					}

					if (onSuccess) {
						onSuccess(result.Result);
					}
				},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		getBatchStages: function (batchId, onSuccess) {
			$.ajax({
				url: FC.Utils.getControllerUrl("BatchApi/GetBatchStages"),
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchId: batchId
				},
				type: "GET",
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success:
					function (result) {
						if (result.Error) {
							FC.Utils.HandleServerError(result.Error);
							return;
						}

						if (onSuccess) {
							onSuccess(result.Result);
						}
					},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		getBatchUsers: function (batchId, stageId, onSuccess) {
			$.ajax({
				url: FC.Utils.getControllerUrl("BatchApi/GetBatchUsers"),
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchId: batchId,
					stageId: stageId
				},
				type: "GET",
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success:
					function (result) {
						if (result.Error) {
							FC.Utils.HandleServerError(result.Error);
							return;
						}

						if (onSuccess) {
							onSuccess(result.Result);
						}
					},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		getFilteredBatches: function (batchFilterType, firstRecord, recordsCount, getStages, getDateFrom,
			getDateTo, getFilterParameter,getFilterRegParameterName,getFilterRegParameterValue, onAnyResult, onSuccess) {

			var url = FC.Utils.getControllerUrl("BatchApi/GetBatches");

			var data = {
				FirstRecord: firstRecord,
				RecordsCount: recordsCount,
				SortExpression: ''
			};

			switch (batchFilterType) {
				case FC.BusinessModels.BatchFilterType.StageType:
					data.StageTypes = getStages();
					break;
				case FC.BusinessModels.BatchFilterType.CreationDate:
					data.StartDate = new Date(getDateFrom()).toUTCString();
					data.StopDate = new Date(getDateTo()).toUTCString();
					break;
				case FC.BusinessModels.BatchFilterType.Name:
					data.BatchName = getFilterParameter();
					break;
				case FC.BusinessModels.BatchFilterType.BatchTypeName:
					data.BatchTypeName = getFilterParameter();
					break;
				case FC.BusinessModels.BatchFilterType.BatchId:
					url = FC.Utils.getControllerUrl("BatchApi/GetBatchesById");
					data.value = getFilterParameter();
					break;
				case FC.BusinessModels.BatchFilterType.AllExceptProcessed:
					url = FC.Utils.getControllerUrl("BatchApi/GetAllBatchesExceptProcessed");
					break;
				case FC.BusinessModels.BatchFilterType.ProcessedNotLoaded:
					data.StageTypes = [FC.BusinessModels.ProcessingStageType.Processed];
					data.HideDownloadedBatches = true;
					break;
				case FC.BusinessModels.BatchFilterType.RegParameter:
					data.RegParamName = getFilterRegParameterName();
					data.RegParamValue = getFilterRegParameterValue();
			}

			data.ProjectId = FC.SessionInfo.ProjectID;
			data.RoleId = FC.SessionInfo.RoleID;

			batchApiAjax(url, data, onAnyResult, onSuccess);
		},

		/**
		* Получить все пакеты вместе с идентификаторами задач и комментариями
		*/
		getBatchesWithTaskInfo: function (firstRecord, recordsCount, onAnyResult, onSuccess) {
			var url = FC.Utils.getControllerUrl("BatchApi/GetAllBatchesExceptProcessedWithTaskInfo");

			var data = {
				FirstRecord: firstRecord,
				RecordsCount: recordsCount,
				SortExpression: ''
			};

			data.ProjectId = FC.SessionInfo.ProjectID;
			data.RoleId = FC.SessionInfo.RoleID;

			batchApiAjax(url, data, onAnyResult, onSuccess);
		},

		getMultiFilteredBatches: function (batchFilters, firstRecord, recordsCount, onAnyResult, onSuccess, onError) {

			var url = FC.Utils.getControllerUrl("BatchApi/GetBatches");

			var data = {
				FirstRecord: firstRecord,
				RecordsCount: recordsCount,
				SortExpression: ''
			};

			for (var i = 0; i < batchFilters.length; i++) {
				switch (batchFilters[i].type) {
					case FC.BusinessModels.BatchFilterType.StageType:
						data.StageTypes = batchFilters[i].getStages();
						break;
					case FC.BusinessModels.BatchFilterType.CreationDate:
						data.StartDate = new Date(batchFilters[i].getDateFrom()).toUTCString();
						data.StopDate = new Date(batchFilters[i].getDateTo()).toUTCString();
						break;
					case FC.BusinessModels.BatchFilterType.Name:
						data.BatchName = batchFilters[i].getFilterParameter();
						break;
					case FC.BusinessModels.BatchFilterType.BatchTypeName:
						data.BatchTypeName = batchFilters[i].getFilterParameter();
						break;
				}
			}

			data.ProjectId = FC.SessionInfo.ProjectID;
			data.RoleId = FC.SessionInfo.RoleID;

			batchApiAjax(url, data, onAnyResult, onSuccess, onError);
		},

		getAllowedFieldSet: function (batchId, onSuccess) {
			$.ajax({
				url: FC.Utils.getControllerUrl("BatchApi/GetAllowedFieldSet"),
				type: "GET",
				data: {
					projectId: FC.SessionInfo.ProjectID,
					batchId: batchId
				},
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				dataType: "json",
				success: function (result) {
					if (result.Error) {
						FC.Utils.HandleServerError(result.Error);
						return;
					}

					if (onSuccess) {
						onSuccess(result.Result);
					}
				},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		downloadResult: function (batchId) {
			window.open(FC.Utils.getControllerUrl("BatchApi/GetBatchResults?projectId=" + FC.SessionInfo.ProjectID + "&roleId=" + FC.SessionInfo.RoleID + "&batchId=" + batchId));
		},

		downloadResults: function (batchIds, onSuccess) {
			if (!batchIds) {
				return;
			}

			$.ajax({
				url: FC.Utils.getControllerUrl("BatchApi/SetBatchesResultsRegParam"),
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchIds: batchIds
				},
				type: "POST",
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success:
					function () {
						if (onSuccess) {
							onSuccess();
						}
					},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
				}
			});

			var url = "BatchApi/GetBatchesResults?projectId=" + FC.SessionInfo.ProjectID + "&roleId=" + FC.SessionInfo.RoleID;
			for (var i = 0; i < batchIds.length; i++) {
				url += "&batchIds[" + i + "]=" + batchIds[i];
			}

			window.location = FC.Utils.getControllerUrl(url);
		}
	}
})();