﻿FC.DataContext = FC.DataContext || {};

FC.DataContext.DbCache = {
	addEntity: function (templateId, templateVersion, cacheName, columns, values, onSuccess, onError) {
		return $.ajax({
			url: FC.Utils.getControllerUrl('DbCache/AddEntity'),
			type: "POST",
			xhrFields: {
				withCredentials: true
			},
			data: {
				projectId: FC.SessionInfo.ProjectID,
				templateid: templateId,
				templateversion: templateVersion,
				cachename: cacheName,
				columns: columns,
				values: values
			},
			success: function (data) {
				if (onSuccess) {
					onSuccess(data.Result);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				if (onError) {
					onError(errorThrown);
				}
			}
		});
	},

	editEntity: function (templateId, templateVersion, cacheName, columns, values, onSuccess, onError) {
		return $.ajax({
			url: FC.Utils.getControllerUrl('DbCache/EditEntity'),
			type: "POST",
			xhrFields: {
				withCredentials: true
			},
			data: {
				projectId: FC.SessionInfo.ProjectID,
				templateid: templateId,
				templateversion: templateVersion,
				cachename: cacheName,
				columns: columns,
				values: values
			},
			success: function (data) {
				if (onSuccess) {
					onSuccess(data.Result);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				if (onError) {
					onError(errorThrown);
				}
			}
		});
	},

	lookup: function (templateId, templateVersion, cacheName, cursorPosition, filters, allowMultipleColumnResults, onSuccess, onError) {
		(filters || []).forEach(function(filter) {
			filter.FieldValue = FC.Utils.Serialize.htmlEncode(filter.FieldValue);
		});

		cursorPosition = -1; // т.к. не указываем editableFieldName, editablefieldcursorpos всегда будет -1 (https://jira.abbyy.com/browse/FC-796)

		return $.ajax({
			url: FC.Utils.getControllerUrl('DbCache/Lookup'),
			type: "POST",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			data: {
				projectId: FC.SessionInfo.ProjectID,
				templateid: templateId,
				templateversion: templateVersion,
				cachename: cacheName,
				editablefieldcursorpos: cursorPosition,
				searchpairs: filters,
				allowMultipleColumnResults: allowMultipleColumnResults
			},
			success: function (data) {
				if (onSuccess) {
					onSuccess(data.Result);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				if (onError) {
					onError(errorThrown);
				}
			}
		});
	},

	search: function (templateId, templateVersion, cacheName, filters, onSuccess, onError) {
		return $.ajax({
			url: FC.Utils.getControllerUrl('DbCache/Search'),
			type: "POST",
			data: {
				projectId: FC.SessionInfo.ProjectID,
				templateid: templateId,
				templateversion: templateVersion,
				cachename: cacheName,
				searchpairs: filters
			},
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (data) {
				if (onSuccess) {
					onSuccess(data.Result);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				if (onError) {
					onError(errorThrown);
				}
			}
		});
	},

	getEntityStructure: function (templateId, templateVersion, cacheName, onSuccess, onError) {
		return $.ajax({
			url: FC.Utils.getControllerUrl('DbCache/GetEntityStructure'),
			data: {
				projectId: FC.SessionInfo.ProjectID,
				templateid: templateId,
				templateversion: templateVersion,
				cachename: cacheName
			},
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (data) {
				if (onSuccess) {
					onSuccess(data.Result);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				if (onError) {
					onError(errorThrown);
				}
			}
		});
	}
};
