﻿FC.DataContext = FC.DataContext || {};

FC.DataContext.Document = (function () {
	var documentsStorage = {};

	var checkSubTaskState = function (taskId, onSuccess) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Document/GetSubtaskResult"),
			type: "GET",
			data: { subtaskId: taskId },
			xhrFields: {
				withCredentials: true
			},
			dataType: "json",
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (result.Result < 1) {
					setTimeout(function () {
						checkSubTaskState(taskId, onSuccess);
					},
						1000);

					return;
				}

				if (onSuccess) {
					onSuccess(result.Result);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleProcessingError(jqXhr, textStatus, errorThrown);
			}
		});
	};

	var textStorage = {};
	return {
		sendToStage: function (batchId, documentIds, userId, stageId, priority, comment, createTask, onSuccess) {
			$.ajax({
				url: FC.Utils.getControllerUrl("Document/SendToStage"),
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchId: batchId,
					documentIds: documentIds,
					userId: userId,
					stageId: stageId,
					priority: priority,
					createTask: createTask,
					comment: FC.Utils.EncodeHTML(comment)
				},
				type: "POST",
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success:
					function (result) {
						if (result.Error) {
							FC.Utils.HandleServerError(result.Error);
							return;
						}

						if (onSuccess) {
							onSuccess();
						}
					},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		deleteDocuments: function (batchId, documentIds, onSuccess) {
			$.ajax({
				url: FC.Utils.getControllerUrl("Document/DeleteDocuments"),
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchId: batchId,
					documentIds: documentIds
				},
				type: "POST",
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success:
					function (result) {
						if (result.Error) {
							FC.Utils.HandleServerError(result.Error);
							return;
						}

						if (onSuccess) {
							onSuccess();
						}
					},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		setPriority: function (batchId, documentIds, priorityId, onSuccess) {
			$.ajax({
				url: FC.Utils.getControllerUrl("Document/SetPriority"),
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchId: batchId,
					documentIds: documentIds,
					priorityId: priorityId
				},
				type: "POST",
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success:
					function (result) {
						if (result.Error) {
							FC.Utils.HandleServerError(result.Error);
							return;
						}

						if (onSuccess) {
							onSuccess();
						}
					},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		sendToExceptions: function (batchId, documentIds, comment, createTask, onSuccess) {
			$.ajax({
				url: FC.Utils.getControllerUrl("Document/SendToExceptions"),
				data: {
					projectId: FC.SessionInfo.ProjectID,
					roleId: FC.SessionInfo.RoleID,
					batchId: batchId,
					documentIds: documentIds,
					createTask: createTask,
					comment: FC.Utils.EncodeHTML(comment)
				},
				type: "POST",
				dataType: "json",
				xhrFields: {
					withCredentials: true
				},
				beforeSend: FC.Utils.beforeAjaxSend,
				success:
					function (result) {
						if (result.Error) {
							FC.Utils.HandleServerError(result.Error);
							return;
						}

						if (onSuccess) {
							onSuccess();
						}
					},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		getDocument: function (taskId, projectId, batchId, documentId, documentVersion, loadFromTask, onSuccess, onError) {

			var storageKey = taskId + "_" + documentId;
			var resp = documentsStorage[storageKey];

			var processResponse = function (response) {
				if (response.error) {
					if (onError) {
						onError(response.error);
					} else {
						FC.Utils.HandleServerError(response.error);
					}
					return;
				}

				if (onSuccess) {
					onSuccess(response.document);
				}
			}

			var getFromServer = function () {
				$.ajax({
					url: FC.Utils.getControllerUrl("Document/GetDocument"),
					type: "GET",
					data: {
						taskId: taskId,
						projectId: projectId,
						roleId: FC.SessionInfo.RoleID,
						batchId: batchId,
						documentId: documentId,
						documentVersion: 0,
						getAsTaskAttachment: loadFromTask
					},
					dataType: "json",
					xhrFields: {
						withCredentials: true
					},
					beforeSend: FC.Utils.beforeAjaxSend,
					success: function (result) {
						var error = result.Error;

						if (resp) {
							resp.version = documentVersion;
							resp.error = error;
							if (resp.document) {
								resp.document.updateInfo(result.Result);
							} else {
								resp.document = new FC.BusinessModels.DocumentEditor.Document(result.Result);
							}

						} else {
							resp = {
								version: documentVersion,
								document: new FC.BusinessModels.DocumentEditor.Document(result.Result),
								error: error
							};
							documentsStorage[storageKey] = resp;
						}

						processResponse(resp);
					},
					error: function (jqXhr, textStatus, errorThrown) {
						FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
					}
				});
			}

			if (!loadFromTask && resp && resp.version === documentVersion) {
				processResponse(resp);
			} else {
				getFromServer();
			}
		},

		getPageText: function (projectId, batchId, documentId, pageId, pageVersion, onSuccess, templateId, templateVersion) {

			var processResult = function (result) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess(result.Result);
				}
			}
			var key = [projectId, batchId, documentId, pageId, pageVersion, templateId, templateVersion].join();
			var cachedValue = textStorage[key];
			if (cachedValue) {
				processResult(cachedValue);
			} else {
				$.ajax({
					url: FC.Utils.getControllerUrl("DocumentPage/GetText"),
					type: "GET",
					data: {
						projectId: projectId,
						roleId: FC.SessionInfo.RoleID,
						batchId: batchId,
						documentId: documentId,
						pageId: pageId,
						pageVersion: pageVersion

					},
					dataType: "json",
					beforeSend: FC.Utils.beforeAjaxSend,
					success: function (result) {
						textStorage[key] = result;
						processResult(result);
					},
					error: function (jqXhr, textStatus, errorThrown) {
						FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
					}
				});
			}


		},

		checkRules: function (doc, onlySave, onSuccess) {
			var url = FC.Utils.getControllerUrl("Document/Save");

			if (!onlySave) {
				url = FC.Utils.getControllerUrl("Document/CheckRules");
			}

			$.ajax({
				url: url,
				type: "POST",
				data: {
					document: JSON.stringify(doc),
					roleId: FC.SessionInfo.RoleID,
					taskId: doc.TaskId
				},
				xhrFields: {
					withCredentials: true
				},
				dataType: "json",
				beforeSend: FC.Utils.beforeAjaxSend,
				success: function (result, textStatus, request) {
					if (result.Error) {
						FC.Utils.HandleServerError(result.Error);
						return;
					}

					if (onlySave) {
						if (onSuccess) {
							onSuccess(result.Result);
						}
					} else {
						setTimeout(function () {
							checkSubTaskState(result.Result, onSuccess);
						},
							1000);
					}
				},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleProcessingError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		recognizeDocumentsAsync: function (allDocs, taskId, documentsToRecognize, onRecognizeDocument) {
			if (!documentsToRecognize || documentsToRecognize.length === 0) return;
			var requestProps = FC.DataContext.Task.createApplyChangesRequest(allDocs, taskId, false);

			var removeDocumentFromCache = this.removeDocumentFromCache;
			requestProps.DocumentsToRecognize = documentsToRecognize.map(function (d) {
				removeDocumentFromCache(taskId, d.Id());
				return d.Id();
			});

			var checkTaskState = function (checkProps) {
				$.ajax({
					url: FC.Utils.getControllerUrl("Task/GetRecognitionTask"),
					type: "POST",
					data: checkProps,
					xhrFields: {
						withCredentials: true
					},
					dataType: "json",
					beforeSend: FC.Utils.beforeAjaxSend,
					success: function (result, textStatus, request) {
						if (result.Error) {
							documentsToRecognize.forEach(function (doc) {
								doc.recognitionStarted(false);
							});
							FC.Utils.HandleServerError(result.Error);
							return;
						}

						if (result.Result === -1) {
							setTimeout(function () {
								checkTaskState(checkProps);
							},
								1000);

							return;
						}
						var docs = result.Result || [];
						docs.forEach(function (doc, index) {
							var currentDocument = documentsToRecognize[index];
							if (currentDocument) {
								currentDocument.copyFrom(doc);
								currentDocument.Id(doc.Id);
								currentDocument.recognitionResultUnwatched(true);
								currentDocument.recognitionStarted(false);
								currentDocument.updateTemplate(false);
							}
						});
						if (onRecognizeDocument) onRecognizeDocument(docs);
					},
					error: function (jqXhr, textStatus, errorThrown) {
						FC.Utils.HandleProcessingError(jqXhr, textStatus, errorThrown);
					}
				});
			};

			$.ajax({
				url: FC.Utils.getControllerUrl("Task/CreateRecognitionTask"),
				type: "POST",
				data: requestProps,
				xhrFields: {
					withCredentials: true
				},
				dataType: "json",
				beforeSend: FC.Utils.beforeAjaxSend,
				success: function (result, textStatus, request) {
					if (result.Error) {
						documentsToRecognize.forEach(function (doc) {
							doc.recognitionStarted(false);
						});
						FC.Utils.HandleServerError(result.Error);
						return;
					}

					if (result.Result <= 0) {
						documentsToRecognize.forEach(function (doc) {
							doc.recognitionStarted(false);
						});
						FC.Utils.HandleServerError(FC.Environment.Localization.GlobalResources.CheckRulesFailed);
						return;
					}

					requestProps.SubtaskId = result.Result;
					setTimeout(function () {
						checkTaskState(requestProps);
					},
						1000);

				},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleProcessingError(jqXhr, textStatus, errorThrown);
				}
			});
		},


		/**
		 * Экстраполировать регионы повторяющейся группы табличного типа
		 * по имеющимся регионам
		 * @param {} doc документ
		 * @param {} startRowIndex строка, с которой нужно начинать экстраполяцию
		 * @param {} useExistingRows использовать ли имеющуюся разметку строк, или разметить строки автоматически
		 * @param {} fieldPath ссылка на повторяющуюся группу в формате id1\id2\id3...
		 * @param {} onSuccess колбэк 
		 */
		extrapolateRows: function (doc, startRowIndex, useExistingRows, fieldPath, onSuccess) {

			$.ajax({
				url: FC.Utils.getControllerUrl("Document/ExtrapolateRows"),
				type: "POST",
				data: {
					document: JSON.stringify(doc),
					roleId: FC.SessionInfo.RoleID,
					rowIndex: startRowIndex,
					useExistingRows: useExistingRows,
					fieldPath: fieldPath
				},
				xhrFields: {
					withCredentials: true
				},
				dataType: "json",
				beforeSend: FC.Utils.beforeAjaxSend,
				success: function (result, textStatus, request) {
					if (result.Error) {
						FC.Utils.HandleServerError(result.Error);
						return;
					}

					if (result.Result <= 0) {
						if (onSuccess) {
							onSuccess(3);
							return;
						}

						FC.Utils.HandleServerError(FC.Environment.Localization.GlobalResources.CheckRulesFailed);
						return;
					}

					setTimeout(function () {
						checkSubTaskState(result.Result, onSuccess);
					},
						1000);
				},
				error: function (jqXhr, textStatus, errorThrown) {
					FC.Utils.HandleProcessingError(jqXhr, textStatus, errorThrown);
				}
			});
		},

		removeDocumentFromCache: function (taskId, documentId) {
			var storageKey = taskId + "_" + documentId;
			documentsStorage[storageKey] = null;
		}
	};
})();