﻿FC.DataContext = FC.DataContext || {};

function isIE() {
	if (typeof window === 'undefined' && typeof self === 'undefined') return true;
	const windowOrSelf = typeof window !== 'undefined' ? window : self;
	const ua = windowOrSelf.navigator.userAgent;
	return ua.indexOf('Trident') !== -1;
}

function isFF() {
	return navigator.userAgent.toLowerCase().indexOf('firefox')  !== -1;
}

function isChrome() {
	if (typeof window === 'undefined' && typeof self === 'undefined') return false;
	return (navigator.userAgent.toLowerCase().indexOf('chrome') > -1) &&
		(navigator.vendor.toLowerCase().indexOf('google') > -1);
}

FC.DataContext.Server =
{
	changeLanguage: function (langId) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Server/ChangeLanguage"),
			type: "POST",
			data: {
				langId: langId
			},
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
				}

				if (result.Result) {
					FC.Utils.reloadLocation();
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	closeSessionOnUnload: function() {
		// https://caniuse.com/?search=beacon%20api
		if (isIE()) {
			return this.closeSession();
		}

		return this.closeSessionViaBeacon();
	},

	closeSessionViaBeacon: function() {
		var closeSessionURL = FC.Utils.getControllerUrl('Server/Post_CloseSessionById') + '?sessionId=' + FC.SessionInfo.SessionId;
		navigator.sendBeacon(closeSessionURL);
	},

	closeSession: function (onSuccess) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Server/CloseSession"),
			async: false,
			type: "POST",
			data: {},
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function () {
				if (onSuccess) {
					onSuccess();
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	switchDataLogging: function (onSuccess) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Server/SwitchDataLogging"),
			type: "POST",
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess();
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	refreshSession: function (userIsActive, onSuccess) {
		var mode = FC.Utils.getQueryParameter("mode");
		var modeParam = mode ? "?mode=" + mode : "";

		var taskId = window.Layouts &&
			window.Layouts.ModelLoader &&
			window.Layouts.ModelLoader.viewModel &&
			window.Layouts.ModelLoader.viewModel() &&
			window.Layouts.ModelLoader.viewModel().taskId
			? window.Layouts.ModelLoader.viewModel().taskId()
			: 0;

		$.ajax({
			url: FC.Utils.getControllerUrl("Server/RefreshSession") + modeParam,
			type: "POST",
			data: {
				userIsActive: userIsActive,
				projectId: FC.SessionInfo.ProjectID,
				roleId: FC.SessionInfo.RoleID,
				taskId: taskId
			},
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result) {
				if (result.RedirectUrl) {
					FC.Utils.replaceLocation(result.RedirectUrl);
					return;
				}

				if (onSuccess) {
					onSuccess();
				}
			},
			error: function () {
			}
		});
	},

	refreshTicket: function (onSuccess) {
		var mode = FC.Utils.getQueryParameter("mode");
		var modeParam = mode ? "?mode=" + mode : "";

		$.ajax({
			url: FC.Utils.getControllerUrl("Server/RefreshTicket") + modeParam,
			type: "POST",
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result) {
				if (result.RedirectUrl) {
					FC.Utils.replaceLocation(result.RedirectUrl);
					return;
				}

				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess();
				}
			},
			error: function () {
			}
		});
	},

	getCurrentUserName: function (onSuccess) {
		onSuccess(FC.Utils.DecodeHTML(FC.Environment.UserName));
	},

	changeLoadImagesMode: function (mode) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Server/LoadImagesMode"),
			type: "POST",
			data: {
				mode: mode
			},
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	getImagesColorMode: function (onLoad) {
		FC.Utils.runAjax({
			url: FC.Utils.getControllerUrl("Server/GetImagesColorMode"),
			type: "GET",
			data: { sessionId: FC.SessionInfo.SessionId },
			beforeSend: FC.Utils.beforeAjaxSend,
			onSuccess: onLoad,
			onFail: function () { onLoad(false); },
			noRedirect: true
		});
	},

	acceptEula: function (onSuccess) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Server/AcceptEula"),
			type: "POST",
			data: {},
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			success: function (result) {
				if (result.Result && onSuccess) {
					onSuccess();
				}

				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	}
};