﻿FC.DataContext = FC.DataContext || {};

FC.DataContext.Task =
{
	createApplyChangesRequest: function (documents, taskId, excludeRecognize) {
		return {
			TaskId: taskId,
			RoleId: FC.SessionInfo.RoleID,
			ProjectId: FC.SessionInfo.ProjectID,
			DocumentsDescriptions: documents.map(function(doc, index) {
				if (doc.Status() === FC.BatchEditor.Document.Status.RecognizeStarted) return null;
				return {
					DocumentId: doc.Id(),
					TemplateName: doc.docChangedToUnknown() ||
						doc.TemplateName() === FC.BatchEditor.batchEditorResources.unprocessedTemplate() ||
						doc.TemplateName() === FC.Environment.Localization.GlobalResources.UnknownDocumentTemplate
						? ''
						: doc.TemplateName(),
					UpdateTemplate: excludeRecognize ? false : doc.updateTemplate(),
					DocPosition: index,
					ProcessTemplate: doc.docChangedToUnknown() ||
						(excludeRecognize ? false : doc.docNeedRecognize()) ||
						doc.pagesMarkedAnnex(),
					SetUnknown: doc.docChangedToUnknown(),
					IsDeleted: doc.IsDeleted(),
					HasChanges: doc.pagesMoved(),
					PagesDescriptions: doc.ChildNodes().map(function(page) {
						var setAsAnnex = page.Name() === FC.Environment.Localization.GlobalResources.AnnexPage;

						var needRecognize = !doc.docChangedToUnknown() && page.templateChanged();
						if (excludeRecognize) {
							needRecognize = setAsAnnex && needRecognize;
						}
						return {
							PageId: page.Id(),
							TemplateName: page.Name() === FC.Environment.Localization.GlobalResources.UnknownTemplate ? '' : page.Name(),
							NeedRecognize: needRecognize,
							SetAsAnnex: setAsAnnex,
							ImageTransformation: page.ImageTransformation(),
							ConfirmClassification: !page.HasClassificationWarning(),
						};
					})
				};
			}).filter(function (docDescr) { return docDescr; })
	};
	},

	countPostponed: function (onSuccess, onFail) {
		FC.Utils.runAjax({
			url: FC.Utils.getControllerUrl("Task/CountPostponed"),
			type: "GET",
			data: {
				projectId: FC.SessionInfo.ProjectID,
				roleId: FC.SessionInfo.RoleID
			},
			onSuccess: onSuccess,
			onFail: onFail
		});
	},

	getCurrentTaskDocuments: function (taskId, onSuccess) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Task/GetCurrentTaskDocuments"),
			type: "GET",
			data: {
				projectId: FC.SessionInfo.ProjectID,
				roleId: FC.SessionInfo.RoleID,
				taskId: taskId
			},
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);

					if (result.RedirectUrl) {
						FC.Utils.replaceLocation(FC.Utils.prepareRedirectUrl(result.RedirectUrl));
					}

					return;
				}

				if (result.RedirectUrl) {
					FC.Utils.replaceLocation(FC.Utils.prepareRedirectUrl(result.RedirectUrl));
					return;
				}

				if (onSuccess) {
					onSuccess(result.Result);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	finish: function (docs, taskId, onAnyResult, onSuccess) {
		var requestData = this.createApplyChangesRequest(docs, taskId);
		
		$.ajax({
			url: FC.Utils.getControllerUrl("Task/Finish"),
			type: "POST",
			data: requestData,
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result, textStatus, request) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess();
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				if (onAnyResult) {
					onAnyResult();
				}

				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	moveToStage: function (docs, taskId, userId, stageId, priority, comment, onSuccess) {
		var requestData = this.createApplyChangesRequest(docs, taskId);
		requestData.Comment = FC.Utils.EncodeHTML(comment);
		requestData.UserId = userId;
		requestData.StageId = stageId;
		requestData.Priority = priority;
		$.ajax({
			url: FC.Utils.getControllerUrl("Task/MoveToStage"),
			data: requestData,
			type: "POST",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			dataType: "json",
			success: function (result, textStatus, request) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess();
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleProcessingError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	sendToExceptions: function (docs, taskId, comment, checkRules, onSuccess) {
		var requestData = this.createApplyChangesRequest(docs, taskId);
		requestData.Comment = FC.Utils.EncodeHTML(comment);
		requestData.CheckRules = checkRules;
		$.ajax({
			url: FC.Utils.getControllerUrl("Task/SendToExceptions"),
			data: requestData,
			type: "POST",
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result, textStatus, request) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess();
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleProcessingError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	applyChanges: function (documents, taskId, excludeRecognize, onSuccess) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Task/ApplyChanges"),
			type: "POST",
			data: this.createApplyChangesRequest(documents, taskId, excludeRecognize),
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess(result.Result);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	postpone: function (docs, taskId, comment, saveChanges, onSuccess) {
		var requestData = this.createApplyChangesRequest(docs, taskId);
		requestData.Comment = FC.Utils.EncodeHTML(comment);
		requestData.SaveChanges = saveChanges;
		
		$.ajax({
			url: FC.Utils.getControllerUrl("Task/Postpone"),
			data: requestData,
			type: "POST",
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result, textStatus, request) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess();
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleProcessingError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	rescan: function (docs, taskId, userId, comment, onSuccess) {
		var requestData = this.createApplyChangesRequest(docs, taskId);
		requestData.Comment = FC.Utils.EncodeHTML(comment);
		requestData.UserId = userId;

		$.ajax({
			url: FC.Utils.getControllerUrl("Task/Rescan"),
			data: requestData,
			type: "POST",
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result, textStatus, request) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess();
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleProcessingError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	reject: function (taskId, onSuccess) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Task/Reject"),
			data: {
				projectId: FC.SessionInfo.ProjectID,
				roleId: FC.SessionInfo.RoleID,
				taskId: taskId
			},
			type: "POST",
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result) {
				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess();
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleProcessingError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	cancelTask: function (taskId, onSuccess) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Task/CancelTask"),
			data: {
				taskId: taskId
			},
			type: "POST",
			dataType: "json",
			async: false,
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function () {
				if (onSuccess) {
					onSuccess();
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	},
	getTasksByStages: function (stageIds, onAnyResult, getFilterRegParameterName, getFilterRegParameterValue, onSuccess) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Task/GetTasksOnStages"),
			type: "GET",
			data: {
				projectId: FC.SessionInfo.ProjectID,
				roleId: FC.SessionInfo.RoleID,
				stageIds: stageIds,
				RegParamName: getFilterRegParameterName(),
				RegParamValue: getFilterRegParameterValue()
			},
			traditional: true,
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result) {
				if (onAnyResult) {
					onAnyResult();
				}

				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess(result.Result);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				if (onAnyResult) {
					onAnyResult();
				}

				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	},

	getPostponedTasks: function (onAnyResult, onSuccess) {
		$.ajax({
			url: FC.Utils.getControllerUrl("Task/GetPostponed"),
			type: "GET",
			data: {
				projectId: FC.SessionInfo.ProjectID,
				roleId: FC.SessionInfo.RoleID
			},
			dataType: "json",
			xhrFields: {
				withCredentials: true
			},
			beforeSend: FC.Utils.beforeAjaxSend,
			success: function (result) {
				if (onAnyResult) {
					onAnyResult();
				}

				if (result.Error) {
					FC.Utils.HandleServerError(result.Error);
					return;
				}

				if (onSuccess) {
					onSuccess(result.Result);
				}
			},
			error: function (jqXhr, textStatus, errorThrown) {
				if (onAnyResult) {
					onAnyResult();
				}

				FC.Utils.HandleJqueryError(jqXhr, textStatus, errorThrown);
			}
		});
	}
};