﻿FC.FCGridColumn = function (columnDef) {
    var self = this;

    if (!columnDef) {
        return;
    };

    // defaults
    self.field = '';
    self.displayName = '';
    self.width = '100px';
    self.style = ko.computed(function () { return "width: " + self.width + ""; });
    self.sortable = true;
    self.sortDirectionVisible = ko.observable(false);
    self.sortDirectionIcon = ko.observable('');
    self.cellTemplate = "<div data-bind=\"text: $data['{0}']\"></div>";

    self.sortFn = function (elements, desc) {
        self.sortDirectionVisible(true);
        self.sortDirectionIcon(desc ? "▼" : "▲");

        if (desc) {
            elements.sort(function (l, r) {
                return l[self.field]() > r[self.field]() ? -1 : 1;
            });
        }
        else {
            elements.sort(function (l, r) {
                return l[self.field]() > r[self.field]() ? 1 : -1;
            });
        }
    };

    if (columnDef.hasOwnProperty('field')) {
        self.field = columnDef.field;
    }

    if (columnDef.hasOwnProperty('displayName')) {
        self.displayName = columnDef.displayName;
    }

    if (columnDef.hasOwnProperty('width')) {
        self.width = columnDef.width;
    }

    if (columnDef.hasOwnProperty('cellTemplate')) {
        self.cellTemplate = columnDef.cellTemplate;
    }

    self.sortable = !columnDef.hasOwnProperty('sortable') || columnDef['sortable'];

    if (columnDef.hasOwnProperty('sortFn')) {
        self.sortFn = columnDef.sortFn;
    }
};

FC.FCGrid = function (options) {
    var self = this;

    self.SortDirections = { asc: 1, desc: 2 };

    self._id = "grid_" + Math.floor(Math.random() * 10000);
    self.TableClass = "grid_table";
    self.RowClass = "grid_row";
    self.HeaderRowClass = "grid_header_row";
    self.HeaderCellClass = "grid_header_cell";
    self.SelectedItems = ko.observableArray([]);
    self.CanSelectRows = false;

    self.Data = ko.observableArray([]);
    self.ColumnDefinitions = ko.observableArray([]);

    self.DefaultCellTemplate = "<div data-bind=\"text: $data['{0}']\"></div>";
    self.DefaultCellClass = "grid_cell";

    self.CurrentSortField = '';
    self.CurrentSortDirection = self.SortDirections.asc;

    self.SelectAll = function () {
        if (self.AllSelected()) {
            self.SelectedItems([]);
            self.AllSelected(false);
            return;
        }

        var data = self.Data();
        self.SelectedItems(data);
        self.AllSelected(true);
    };

    self.AllSelected = ko.observable(false);
    self.Select = function (data) {
        self.AllSelected(false);

        var contains = false;
        var index = -1;
        var allselected = self.SelectedItems();

        for (var i = 0; i < allselected.length; i++) {
            if (allselected[i] === data) {
                contains = true;
                index = i;
                break;
            }
        }

        if (!contains) {
            allselected.push(data);
        } else {
            allselected.splice(index, 1);
        }

        self.SelectedItems(allselected);
        return true;
    };

    self.IsSelected = function (data) {
        var allselected = self.SelectedItems();
        for (var i = 0; i < allselected.length; i++) {
            if (allselected[i] === data) {
                return true;
            }
        }

        return false;
    };

    if (options != null) {
        if (options.data) {
            self.Data = options.data;
        }

        if (options.columnDefs && Array.isArray(options.columnDefs)) {
            self.ColumnDefinitions(ko.utils.arrayMap(options.columnDefs, function (item) {
                return new FC.FCGridColumn(item);
            }));
        }

        if (options.selectedItems) {
            self.SelectedItems = options.selectedItems;
        }

        self.CanSelectRows = options.canSelectRows;
    }

    self.Sort = function (colDef) {
        if (!colDef.sortable) {
            return;
        }

        var desc = false;
        var fieldName = colDef.field;

        if (self.CurrentSortField === fieldName) {
            if (self.CurrentSortDirection === self.SortDirections.asc) {
                desc = true;
            }
        }

        var columns = self.ColumnDefinitions();
        for (var i = 0; i < columns.length; i++) {
            if (columns[i].field !== fieldName) {
                columns[i].sortDirectionIcon('');
                columns[i].sortDirectionVisible(false);
            } else {
                columns[i].sortFn(self.Data, desc);
            }
        }

        self.CurrentSortDirection = desc ? self.SortDirections.desc : self.SortDirections.asc;
        self.CurrentSortField = fieldName;
    };

    self._generateHtml = function () {
        var html = "<table class='" + self.TableClass + "'>"
            + self._generateHeader()
            + self._generateBody();

        return html;
    };

    self._generateHeader = function () {
        var header = "<thead><tr class=\"" + self.HeaderRowClass + "\">";

        // добавляем "select all"
        if (self.CanSelectRows) {
        	header += "<th><label class=\"checkbox__label\"><input type=\"checkbox\" data-bind=\"click: SelectAll, checked: AllSelected\"/><span></span></label></th>";
        }

	    header += "<!-- ko foreach: ColumnDefinitions -->";
        header += "<th data-bind=\"click: $parent.Sort, attr: {style: style }\">";
        header += "<span class=\"header_cell_div\" data-bind=\"html : displayName\"></span>";
        header += "<span class=\"header_sorting_button\" data-bind=\"text: sortDirectionIcon, visible: sortDirectionVisible\"></span>";
        header += "</th>";
	    header += "<!-- /ko -->";
        header += "</tr></thead>";

        return header;
    };

    self._generateBody = function () {
        var columns = self.ColumnDefinitions();

        var body = "<tbody data-bind=\"foreach: Data\"><tr class='" + self.RowClass + "'>";

        // добавляем "select"
        if (self.CanSelectRows) {
        	body += "<td><label class=\"checkbox__label\"><input type=\"checkbox\" data-bind=\"click: $parent.Select, checked: $parent.IsSelected($data) \"/><span></span></label></th>";
        }

        for (var i = 0; i < columns.length; i++) {
            var column = columns[i];
            if (!column) {
                continue;
            }

            if (!column.hasOwnProperty('field')) {
                continue;
            }

            var cellTemplate = self.DefaultCellTemplate;
            if (column.hasOwnProperty('cellTemplate')) {
                cellTemplate = column['cellTemplate'];
            }

            cellTemplate = cellTemplate.replace('{0}', column['field']);

            var cellClass = self.DefaultCellClass;
            if (column.hasOwnProperty('cellClass')) {
                cellClass = column['cellClass'];
            }

            body += "<td class=\"" + cellClass + "\">" + cellTemplate + "</td>";
        }

        body += "</tr></tbody>";

        return body;
    };

    self.UpdateElement = function (elem) {
        if (!elem) {
            return;
        }

        elem.empty().append(self._generateHtml());
    };
};

ko.bindingHandlers.grid = {
    init: function (element, valueAccessor) {
        var options = valueAccessor();

        var grid = new FC.FCGrid(options);
        var elem = $(element);

        grid.UpdateElement(elem);
        //ko.cleanNode(elem[0].firstChild);
        ko.applyBindings(grid, elem[0].firstChild);

        return { controlsDescendantBindings: true };
        //FC.FCGridSettings.FCGrids.push(grid);
    },
    //update the control when the view model changes
    update: function () {
    }
};

