﻿FC.Pages.BatchesPage = function (dependencies) {
	var self = this;

	$.extend(self, new FC.Pages.BasePageModel(dependencies.serverDataContext));

	self.GetBatchFilterTypes = function () {
		var filters = [
			new FC.InfrastructureModels.FilterType(
				FC.BusinessModels.BatchFilterType.AllExceptProcessed,
				FC.Environment.Localization.GlobalResources.AllExceptProcessedFilter),
			new FC.InfrastructureModels.FilterType(
				FC.BusinessModels.BatchFilterType.StageType,
				FC.Environment.Localization.GlobalResources.StageFilter),
			new FC.InfrastructureModels.FilterType(
				FC.BusinessModels.BatchFilterType.CreationDate,
				FC.Environment.Localization.GlobalResources.CreationDateFilter),
			new FC.InfrastructureModels.FilterType(
				FC.BusinessModels.BatchFilterType.Name,
				FC.Environment.Localization.GlobalResources.NameFilter),
			new FC.InfrastructureModels.FilterType(
				FC.BusinessModels.BatchFilterType.BatchTypeName,
				FC.Environment.Localization.GlobalResources.BatchTypeNameFilter),
			new FC.InfrastructureModels.FilterType(
				FC.BusinessModels.BatchFilterType.BatchId,
				FC.Environment.Localization.GlobalResources.IdFilter),
			new FC.InfrastructureModels.FilterType(
				FC.BusinessModels.BatchFilterType.RegParameter,
				FC.Environment.Localization.GlobalResources.BatchesRegParamFilter),
			new FC.InfrastructureModels.FilterType(
				FC.BusinessModels.BatchFilterType.None,
				FC.Environment.Localization.GlobalResources.NoneBatchesFilter)
		];

		if (FC.Environment.CanDownloadBatchResult) {
			filters.push(new FC.InfrastructureModels.FilterType(
				FC.BusinessModels.BatchFilterType.ProcessedNotLoaded,
				FC.Environment.Localization.GlobalResources.ProcessesNotLoadedFitler));
		}
		return filters;
	};

	self.GetStageFilterTypes = function () {
		return [
			new FC.InfrastructureModels.FilterType(
				0,
				FC.Environment.Localization.GlobalResources.ImportStage),
			new FC.InfrastructureModels.FilterType(
				1,
				FC.Environment.Localization.GlobalResources.RecognitionStage),
			new FC.InfrastructureModels.FilterType(
				2,
				FC.Environment.Localization.GlobalResources.InteractiveStage),
			new FC.InfrastructureModels.FilterType(
				3,
				FC.Environment.Localization.GlobalResources.ExportStage),
			new FC.InfrastructureModels.FilterType(
				4,
				FC.Environment.Localization.GlobalResources.ScriptStage),
			new FC.InfrastructureModels.FilterType(
				5,
				FC.Environment.Localization.GlobalResources.ServiceStage),
			new FC.InfrastructureModels.FilterType(
				6,
				FC.Environment.Localization.GlobalResources.ProcessedStage),
			new FC.InfrastructureModels.FilterType(
				7,
				FC.Environment.Localization.GlobalResources.UserTypeStage),
			new FC.InfrastructureModels.FilterType(
				8,
				FC.Environment.Localization.GlobalResources.OutOfWorkflowStage)
		];
	};

	var tableAdjustFnc = function () {
		var table = $.fn.dataTable.fnTables(true);
		if (table.length > 0) {
			$(table).dataTable().fnAdjustColumnSizing(false);
		}
	};

	$(window).resize(tableAdjustFnc);

	self.BatchFilterValues = ko.observableArray(self.GetBatchFilterTypes());

	var lastCurrentBatchFilterTypeId = parseInt(localStorage.getItem('batches__CurrentBatchFilterType'));
	if (FC.Environment.CanDownloadBatchResult && isNaN(lastCurrentBatchFilterTypeId)) {
		lastCurrentBatchFilterTypeId = 0;
	}
	var lastCurrentBatchFilterType = self.BatchFilterValues().filter(function (item) {
		return item.Id() === lastCurrentBatchFilterTypeId;
	})[0];
	self.CurrentBatchFilterType = ko.observable(lastCurrentBatchFilterType || self.BatchFilterValues()[0]);
	self.CurrentBatchFilterType.subscribe(function (val) {
		if (val) {
			localStorage.setItem('batches__CurrentBatchFilterType', val.Id());
		} else {
			localStorage.removeItem('batches__CurrentBatchFilterType');
		}
	});

	self.loadingBatches = ko.observable(false);

	self.canDownloadBatches = ko.observable(!!FC.Environment.CanDownloadBatchResult);
	self.downloadBatchTitle = ko.observable(FC.Environment.Localization.Batches_aspx.Download_Tooltip);

	self.StageFilterValues = ko.observableArray(self.GetStageFilterTypes());

	var lastCurrentStageFilterTypeId = parseInt(localStorage.getItem('batches__CurrentStageFilterType'));
	var lastCurrentStageFilterType = self.StageFilterValues().filter(function (item) {
		return item.Id() === lastCurrentStageFilterTypeId;
	})[0];
	self.CurrentStageFilterType = ko.observable(lastCurrentStageFilterType || self.StageFilterValues()[0]);
	self.CurrentStageFilterType.subscribe(function (val) {
		if (val) {
			localStorage.setItem('batches__CurrentStageFilterType', val.Id());
		} else {
			localStorage.removeItem('batches__CurrentStageFilterType');
		}
	});

	self.BatchFilterChanged = function () {
		var currentBatchFilter = self.CurrentBatchFilterType();

		self.FilterByTextbox(false);
		self.FilterByStage(false);
		self.FilterByDate(false);
		self.FilterByRegParameter(false);

		switch (currentBatchFilter.Id()) {
			case FC.BusinessModels.BatchFilterType.StageType:
				self.FilterByStage(true);
				break;
			case FC.BusinessModels.BatchFilterType.CreationDate:
				self.FilterByDate(true);
				break;
			case FC.BusinessModels.BatchFilterType.Name:
			case FC.BusinessModels.BatchFilterType.BatchTypeName:
			case FC.BusinessModels.BatchFilterType.BatchId:
			case FC.BusinessModels.BatchFilterType.DocumentId:
				self.FilterByTextbox(true);
				break;
			case FC.BusinessModels.BatchFilterType.RegParameter:
				self.FilterByRegParameter(true);
				break;
			default: break;
		}
	};

	self.FilterByRegParameter = ko.observable(localStorage.getItem('batches__FilterByRegParameter') === 'true');
	self.FilterByRegParameter.subscribe(function (val) {
		localStorage.setItem('batches__FilterByRegParameter', val);
	});

	self.FilterRegParameterName = ko.observable(localStorage.getItem('batches__FilterRegParameterName'));
	self.FilterRegParameterName.subscribe(function (val) {
		localStorage.setItem('batches__FilterRegParameterName', val);
	});

	self.FilterRegParameterValue = ko.observable(localStorage.getItem('batches__FilterRegParameterValue'));
	self.FilterRegParameterValue.subscribe(function (val) {
		localStorage.setItem('batches__FilterRegParameterValue', val);
	});

	self.FilterByTextbox = ko.observable(localStorage.getItem('batches__FilterByTextbox') === 'true');
	self.FilterByTextbox.subscribe(function (val) {
		localStorage.setItem('batches__FilterByTextbox', val);
	});

	self.FilterByStage = ko.observable(localStorage.getItem('batches__FilterByStage') === 'true');
	self.FilterByStage.subscribe(function (val) {
		localStorage.setItem('batches__FilterByStage', val);
	});

	self.FilterByDate = ko.observable(localStorage.getItem('batches__FilterByDate') === 'true');
	self.FilterByDate.subscribe(function (val) {
		localStorage.setItem('batches__FilterByDate', val);
	});

	self.FilterParameter = ko.observable(localStorage.getItem('batches__FilterParameter'));
	self.FilterParameter.subscribe(function (val) {
		localStorage.setItem('batches__FilterParameter', val);
	});

	var lastDateFrom = parseInt(localStorage.getItem('batches__DateFrom'));
	self.DateFrom = ko.observable(isNaN(lastDateFrom) ? Date.now() : lastDateFrom);
	self.DateFrom.subscribe(function (val) {
		localStorage.setItem('batches__DateFrom', val);
	});

	var lastDateTo = parseInt(localStorage.getItem('batches__DateTo'));
	self.DateTo = ko.observable(isNaN(lastDateTo) ? Date.now() : lastDateTo);
	self.DateTo.subscribe(function (val) {
		localStorage.setItem('batches__DateTo', val);
	});

	self.DeleteVisible = ko.observable(false);
	self.PriorityChangeVisible = ko.observable(false);
	self.SendVisible = ko.observable(false);


	/** Column object constructor **/
	function column (props, render, createdCell ) {
		this.id = props.id;

		this.staticColumn = !!props.staticColumn ;

		this.name = props.name;
		this.data = props.data
		this.visibleTitle = props.visibleTitle;

		if(props.defaultContent!==undefined) {
			this.defaultContent = props.defaultContent
		}

		if(render!==undefined){
			this.render = render;
		}

		if(createdCell!==undefined) {
			this.createdCell = createdCell;
		}

		if(props.width!==undefined) {
			this.width = props.width;
		}

		if(props.className!==undefined) {
			this.className = props.className
		}

		this.visible = self.getColumnVisibility(props.id, false);
		this.showColumn = ko.observable(self.getColumnVisibility(props.id, false));
		this.showColumn.subscribe(function (newVisible) {
			this.visible = newVisible;
			self.TableContainer().table.columns(this.id).visible(newVisible);
			self.ShowColumnDropDown();
		}, this);
	}

	/** Reg column object constructor **/
	function regParamColumn (props ) {
		this.id = props.id;

		this.name = props.name;
		this.data = "RegParams";
		this.visibleTitle = props.name;

		this.render = function (data) {
			var regParameter = data.find(function (item) {
				return item.Name == props.name
			})
			return regParameter ? regParameter.Value : ""
		}
		this.visible = self.getColumnVisibility(props.id, true);
		this.showColumn = ko.observable(self.getColumnVisibility(props.id, true));
		this.showColumn.subscribe(function (newVisible) {

			this.visible = newVisible;
			var index = self.ColumnsRegParam().findIndex(function (column) { return column.id === this.id}.bind(this))
			self.TableContainer().table.columns(self.ColumnsDefault().length + index).visible(newVisible);
			self.ShowColumnDropDown();
		}, this);
	}

	/**
	 * Function for get column visibility from local storage
	 * @param id Column id
	 * @param isRegParam If true - get parameter for regParam
	 * @returns {boolean}
	 */
	self.getColumnVisibility = function (id, isRegParam) {
		var columnsFromStore = null;
		if(isRegParam) {
			var storedValue = localStorage.getItem('batches__ColumnRegParamFilter');
			columnsFromStore = !!storedValue?JSON.parse(storedValue):null
		} else {
			var storedValue = localStorage.getItem('batches__ColumnDefaultFilter')
			columnsFromStore = !!storedValue?JSON.parse(storedValue):null
		}
		if(!columnsFromStore || !columnsFromStore.length > 0) {
			return true
		} else {
			var storedVisible = columnsFromStore.find(function (column) {return column.id ===id})
			if(storedVisible===undefined) {
				return true
			} else {
				return storedVisible.visible
			}
		}
	}

	/** Get reg parameter from local storage **/
	self.getRegParamColumns = function () {
		var storedValue = localStorage.getItem('batches__ColumnRegParamFilter');
		var regParamColumnsFromStore = !!storedValue?JSON.parse(storedValue):null
		if (!regParamColumnsFromStore || !regParamColumnsFromStore.length > 0) {
			return []
		} else {
			return regParamColumnsFromStore.map(function (column) {
				return new regParamColumn({
					id: column.id,
					name: column.name
				})
			})
		}
	}

	/** Array of default columns filter **/
	self.ColumnsDefault = ko.observableArray([
		new column({
			id: 0,
			width: "20px",
			name: "check",
			defaultContent: "",
			className: "select-checkbox  batchList__expiration",
			staticColumn: true
		}),
		new column({
				id: 1,
				width: "64px",
				data: 'Priority',
				name: 'Priority',
				visibleTitle: FC.Environment.Localization.GlobalResources.PriorityColumn
			},
			function (data, type, full) {
				if (type !== "display") {
					return data;
				}
				return full.PriorityIconVisible ? "<img src=\"" + FC.Environment.AppPathNoTennant + full.PriorityIconUrl + "\"/>" : "";
			}),
		new column({
			id: 2,
			name: 'Name',
			data: 'Name',
			visibleTitle: FC.Environment.Localization.GlobalResources.BatchNameColumn
		}, function (data, type, full) {
			if (type !== "display") {
				return data;
			}
			return '<a class="link" href="' + FC.Utils.prepareRedirectUrl("Documents?projectId=" +
				FC.SessionInfo.ProjectID + "&roleId=" + FC.SessionInfo.RoleID + "&batchId=" + full.Id) + '">' + data + '</a>';
		}),
		new column({
			id: 3,
			name: 'BatchType',
			data: 'BatchType',
			visibleTitle: FC.Environment.Localization.GlobalResources.BatchTypeColumn
		}),
		new column({
			id: 4,
			name: 'StageName',
			data: 'StageName',
			visibleTitle: FC.Environment.Localization.GlobalResources.StageNameColumn
		}),
		new column({
			id: 5,
			width: "140px",
			data: 'CreationTimeString',
			name: 'CreationTimeString',
			visibleTitle: FC.Environment.Localization.GlobalResources.CreationTimeColumn
		}),
		new column({
			id: 6,
			width: "60px",
			data: 'DocumentCount',
			name: 'DocumentCount',
			visibleTitle: FC.Environment.Localization.GlobalResources.DocumentsColumn
		}),
		new column({
			id: 7,
			width: "60px",
			name: 'PageCount',
			data: 'PageCount',
			visibleTitle: FC.Environment.Localization.GlobalResources.PagesColumn
		}),
		new column({
			id: 8,
			width: "140px",
			data: 'ExpirationDateString',
			name: 'ExpirationDateString',
			visibleTitle: FC.Environment.Localization.Batches_aspx.ExpirationDate
		}),
		new column({
			id: 9,
			width: "60px",
			data: 'Id',
			name: 'Id',
			visibleTitle: FC.Environment.Localization.GlobalResources.IdColumn
		}),
		new column({
			id: 10,
			data: 'HasAttachments',
			name: 'HasAttachments',
			visibleTitle: FC.Environment.Localization.GlobalResources.AttachmentsColumn,
			staticColumn: true
		}, function (data, type) {
			if (type !== "display") {
				return data;
			}
			return data ? "<img src=\"" + FC.Environment.AppPathNoTennant + FC.Environment.Localization.NonLocalizedResources.NoAttachmentsImagePath + "\"/>" : "";
		}),
		new column({
			id: 11,
			width: FC.Environment.CanDownloadBatchResult ? "20px" : "1px",
			data: 'HasDocumentResults',
			name: 'HasDocumentResults',
			visibleTitle: FC.Environment.Localization.GlobalResources.DocumentResultColumn,
			staticColumn: true
			},
			function (data, type, full) {return type !== "display"? data : ''},
			function (td, cellData, rowData, row, col) {
			if (!cellData || !FC.Environment.CanDownloadBatchResult) {
				$(td).html('');
				return;
			}

			var elem = $('<div title="' + FC.Environment.Localization.Batches_aspx.Download_Tooltip + '" class="batch_download_button">' +
				'<svg class="batch_download-icon"><use xlink:href="#icon--download-20"></use></svg></div>');

			elem.click(function () {
				FC.DataContext.Batch.downloadResult(rowData.Id);
				setTimeout(self.TableContainer().redraw, 1000);
			});

			$(td).html(elem);
		}),
		new column({
			id: 12,
			width: FC.Environment.CanDownloadBatchResult ? "140px" : "1px",
			data: 'LastDownloadTime',
			name: 'LastDownloadTime',
			visibleTitle: FC.Environment.Localization.GlobalResources.LastDownloadTimeColumn,
			staticColumn: true
		}, function (data, type, full) {
			if (type !== "display") {
				return data;
			}

			if (!data || !FC.Environment.CanDownloadBatchResult || !full.HasDocumentResults) {
				return '';
			}

			return new Date(data.match(/[\d\-]+/)[0] * 1).toLocaleString();
		})
	])

	/** Array of registration parameters columns **/
	self.ColumnsRegParam = ko.observableArray(	self.getRegParamColumns());

	/** Condition for reg parameter columns limit **/
	self.ColumnsRegParamLimit = ko.computed(function() {
		return self.ColumnsRegParam().length<3
	});

	/** Settings column **/
	self.SettingsColumn = {
		width: "20px",
		data: null,
		name: null,
		defaultContent: ''
	}

	/** Filtered array of columns for displaying in settings drop down **/
	self.FilterColumnsDefault = ko.computed(function() {
			return ko.utils.arrayFilter(self.ColumnsDefault(), function(column) {
				return !column.staticColumn;
			});
	});

	ko.computed( function() {
		ko.toJSON(self.ColumnsDefault);
		var filters = self.ColumnsDefault().map(function (column) {
			return {
				id: column.id,
				visible: column.visible
			}
		})
		localStorage.setItem('batches__ColumnDefaultFilter', JSON.stringify(filters));
	});

	ko.computed( function() {
		ko.toJSON(self.ColumnsRegParam);
		var filters = self.ColumnsRegParam().map(function (column) {
			return {
				id: column.id,
				visible: column.visible,
				name: column.name
			}
		})
		localStorage.setItem('batches__ColumnRegParamFilter', JSON.stringify(filters));
	});

	/** Show column settings drop down **/
	self.ShowColumnDropDown = function () {
		var settingPosition = $('#settings-icon').position()
		$('#column-filter').css({
			position: 'absolute',
			top: settingPosition.top - 30,
			left: settingPosition.left - $('#column-filter').width() + 7
		}).show();
	}

	/** Toggle column settings drop down **/
	self.ToggleColumnDropDown = function () {
		var settingPosition = $('#settings-icon').position()
		$('#column-filter').css({
			position: 'absolute',
			top: settingPosition.top - 30,
			left: settingPosition.left - $('#column-filter').width() + 7
		}).toggle();
		self.HideRegParamInput();
	}

	/** Hide column settings drop down **/
	self.HideColumnDropDown = function () {
		var settingPosition = $('#settings-icon').position()
		$('#column-filter').css({
			position: 'absolute',
			top: settingPosition.top,
			left: settingPosition.left
		}).hide();
		self.HideRegParamInput();
	}

	/** Handler for filter setting menu onScroll hide **/
	$("#batch-scroll-container").on('scroll', self.HideColumnDropDown);

	/** Handler for filter setting menu onResize hide **/
	$(document).on('resize', self.HideColumnDropDown);

	/** Handler for filter setting menu outside click **/
	$(document).on('click', function (event) {
		var container = $("#column-filter");
		if (container.has(event.target).length === 0) {
			self.HideColumnDropDown();
		}
	});

	/** Value from add reg. parameter input **/
	self.RegParameterInput = ko.observable('');

	/** Add reg. parameter input visibility flag **/
	self.IsRegParamInputVisible = ko.observable(false);
	self.IsRegParamInputFocus = ko.observable(false);

	self.ShowRegParamInput = function () {
		self.IsRegParamInputVisible(true)
		self.RegParameterInput('');
		self.ShowColumnDropDown();
	}

	self.HideRegParamInput = function () {
		self.IsRegParamInputVisible(false)
		self.RegParameterInput('');
	}

	/** Prevent default for input**/
	$('#column-filter').keypress(function (event) {
		if (event.key == "Enter") {
			var elem = event.srcElement || event.target;
			self.RegParameterInput(elem.value);
			self.AddRegParamColumn();
			event.preventDefault()
		}
	});

	/** Add registration parameter to column list **/
	self.AddRegParamColumn = function () {
		var parameterName = self.RegParameterInput();
		if (!parameterName) {
			return
		}
		var regColumns = self.ColumnsRegParam()
		self.TableContainer().table.columns(self.ColumnsDefault().length + regColumns.length ).visible(true);
		self.ColumnsRegParam.push(
			new regParamColumn({
				id: regColumns.length>0 ?regColumns[regColumns.length-1].id + 1 : 0,
				name: parameterName
			}))
	};

	/** Add registration parameter to column list **/
	self.DeleteRegParamColumn = function (regColumn) {
		self.ColumnsRegParam( self.ColumnsRegParam().filter(function (column) {return column.id !== regColumn.id}));
	};


	self.ColumnsRegParam.subscribe(function () {
		var displayedColumns = self.ColumnsDefault().concat(self.ColumnsRegParam(),[self.SettingsColumn])

		self.InitTable(displayedColumns);
		self.ColumnsRegParam().forEach(function (column) {
			$('<th>' + column.name + '</th>').insertBefore("#settings-header");
		})
		self.TableContainer().reloadTable();

		self.ColumnsRegParam().forEach(function (column,index) {
			var newVisible = self.getColumnVisibility(column.id, true);
			if(self.TableContainer().table) {
				self.TableContainer().table.columns(self.ColumnsDefault().length + index).visible(newVisible);
			}
		})
	})

	self.IsRegParamInputVisible.subscribe(function (newValue) {
			self.IsRegParamInputFocus(newValue)
	})

	self.DeleteVisible.subscribe(function () {
		self.checkNonContextFunctionsMinimization();
	});
	self.PriorityChangeVisible.subscribe(function () {
		self.checkNonContextFunctionsMinimization();
	});
	self.SendVisible.subscribe(function () {
		self.checkNonContextFunctionsMinimization();
	});

	self.ApplyFilter = function () {
		self.Refresh();
	};

	self.OnDeleteBatches = function (onSuccess, onFail) {
		var batches = self.SelectedBatches();

		if (!self.DeleteVisible() || !batches || !batches.length) {
			if (onFail) {
				onFail();
			}
			return;
		}

		var batchIds = [];
		for (var i = 0; i < batches.length; i++) {
			batchIds.push(batches[i].Id);
		}

		if (batchIds.length === 0) {
			if (onFail) {
				onFail();
			}
			return;
		}

		FC.DataContext.Batch.deleteBatches(batchIds, function () {
			if (onSuccess) {
				onSuccess();
			}
			self.Refresh();
		});
	};

	self.OnPriorityChange = function (onSuccess, onFail) {
		var batches = self.SelectedBatches();
		if (!self.PriorityChangeVisible() || !batches || !batches.length) {
			if (onFail) {
				onFail();
			}
			return;
		}

		var batchIds = [];
		for (var i = 0; i < batches.length; i++) {
			batchIds.push(batches[i].Id);
		}

		if (batchIds.length === 0) {
			if (onFail) {
				onFail();
			}
			return;
		}

		FC.DataContext.Batch.setPriority(
			batchIds,
			self.PrioritySelector().selectedPriority(),
			function () {
				if (onSuccess) {
					onSuccess();
				}

				self.Refresh();
			});
	};

	self.OnSendToStage = function (onSuccess, onFail) {
		var selectedBatches = self.SelectedBatches();
		if (!self.SendVisible() || !selectedBatches || !selectedBatches.length) {
			if (onFail) {
				onFail();
			}
		}

		const count = self.TableContainer().selectedRowCount();

		let listBatchId = [];
		for (let i = 0; i < count; i++) {
			listBatchId.push(selectedBatches[i].Id)
		}

		// Прогресс бар UI
		if (count > 1) {
			$('body').append(
				'<div class="progressBarBatch__container">' +
				'<div class="progressBarBatch__box">' +
				'<div class="progressBarBatch__scale"></div>' +
				'<div class="progressBarBatch__percent">0% (0/' + count + ')</div>' +
				'</div>' +
				'</div>'
			);
		}

		// Прогресс бар
		let sendedBatch = 0;
		function progressBar() {
			sendedBatch++;
			$('.progressBarBatch__percent').text((sendedBatch * 100 / count).toFixed() + '%' + ' ' + '(' + sendedBatch + '/' + count + ')')
			$('.progressBarBatch__scale').css({ 'width': sendedBatch * 100 / count + '%' })
		}

		// Отправка батчей
		function sendBatch() {
			if (listBatchId.length === 0) {
				setTimeout(function () {
					$('.progressBarBatch__container').remove()
				}, 1000)
				return;
			}
			FC.DataContext.Batch.sendToStage(
				sendBatch,
				progressBar,
				count,
				listBatchId[listBatchId.length - 1],
				self.StageSelector().selectedUser(),
				self.StageSelector().selectedStage().SendingStageId,
				self.StageSelector().selectedPriority(),
				self.StageSelector().comment(),
				self.StageSelector().createTask(),
				function () {
					if (onSuccess) {
						onSuccess();
					}
					self.Refresh();
				}
			);
			listBatchId.pop();
		}
		sendBatch();
	};

	self.OnSendToExceptions = function (onSuccess, onFail) {

		var selectedBatches = self.SelectedBatches();
		if (!self.SendVisible() || !selectedBatches || !selectedBatches.length) {
			if (onFail) {
				onFail();
			}
		}

		FC.DataContext.Batch.sendToExceptions(
			selectedBatches[0].Id,
			self.SenderToExceptions().comment(),
			self.SenderToExceptions().createTask(),
			function () {
				if (onSuccess) {
					onSuccess();
				}

				self.Refresh();
			});
	};

	self.PrioritySelector = ko.observable(new FC.Pages.PrioritySelector(self.OnPriorityChange));
	self.BatchesDeleter = ko.observable(new FC.DialogBase(self.OnDeleteBatches, null, null, false, FC.Environment.Headless, false, false, false, "batchesDeleteDialogue"));
	self.StageSelector = ko.observable(new FC.Pages.StageSelector(true, self.OnSendToStage));
	self.SenderToExceptions = ko.observable(new FC.Pages.SenderToExceptions(true, self.OnSendToExceptions));

	self.ShowPriorityDialog = function () {
		if (!self.PriorityChangeVisible()) {
			return;
		}

		if (self.TableContainer().selectedRowCount() > 0) {
			var selectedBatches = self.SelectedBatches();
			var priority = -5;
			for (var i = 0; i < selectedBatches.length; i++) {
				if (selectedBatches[i].Priority > priority) {
					priority = selectedBatches[i].Priority;
				}
			}
			self.PrioritySelector().selectedPriority(priority);
			self.PrioritySelector().show();
		}
	};

	self.SendToExceptions = function () {
		if (!self.SendVisible()) {
			return;
		}

		if (self.TableContainer().selectedRowCount() === 1) {
			self.SenderToExceptions().show();
		}
	};

	self.ShowStagesDialog = function () {
		if (!self.SendVisible()) {
			return;
		}
		if (self.TableContainer().selectedRowCount() > 0) {
			const count = self.TableContainer().selectedRowCount();
			for (let i = 0; i < count; i++) {
				self.StageSelector().updateStages(self.SelectedBatches()[i].Id);
			}
			self.StageSelector().show();
		}
	};

	self.Delete = function () {
		if (!self.DeleteVisible()) {
			return;
		}
		if (self.TableContainer().selectedRowCount() > 0) {
			self.BatchesDeleter().show();
		}
	};

	self.Refresh = function () {
		self.TableContainer().reloadTable();
		self.pageNavigation.countPostponed();
	};


	self.SelectedBatches = function () {
		return self.TableContainer().table.rows(".selected").data();
	};

	self.GetBatches = function (onSuccess, starting, shown) {
		if (!$('.progressBarBatch__container').length) {
			self.loadingBatches(true);
		}

		FC.DataContext.Batch.getFilteredBatches(
			self.CurrentBatchFilterType().Id(),
			starting,
			shown,
			self.GetSelectedStages,
			self.DateFrom,
			self.DateTo,
			self.FilterParameter,
			self.FilterRegParameterName,
			self.FilterRegParameterValue,
			function () {
				self.loadingBatches(false);
			},
			function (batches) {
				if (onSuccess) {
					onSuccess(batches);
				}
			}
		);
	};

	self.GetActionVisibility = function (onSuccess) {
		FC.DataContext.Role.getAllowedActions(
			function (allowedActions) {
				if (!allowedActions.CanViewBatchList) {
					FC.Utils.replaceLocation(FC.Utils.prepareRedirectUrl(FC.Environment.AppPath + FC.Environment.HomeUrl));
					return;
				}

				self.DeleteVisible(allowedActions.CanDelete);
				self.PriorityChangeVisible(allowedActions.CanChangePriority);
				self.SendVisible(allowedActions.CanSendToStage);

				if (onSuccess) {
					onSuccess();
				}
			}
		);
	};

	self.GetSelectedStages = function () {
		var stagesFilterType = self.CurrentStageFilterType().Id();

		switch (stagesFilterType) {
			case 0:
				return [FC.BusinessModels.ProcessingStageType.Import];
			case 1:
				return [FC.BusinessModels.ProcessingStageType.Recognition,
				FC.BusinessModels.ProcessingStageType.Unattended];
			case 2:
				return [FC.BusinessModels.ProcessingStageType.Rescanning,
				FC.BusinessModels.ProcessingStageType.DocumentAssembly,
				FC.BusinessModels.ProcessingStageType.Verification,
				FC.BusinessModels.ProcessingStageType.DataVerification,
				FC.BusinessModels.ProcessingStageType.Exceptions,
				FC.BusinessModels.ProcessingStageType.BatchIntegrity,
				FC.BusinessModels.ProcessingStageType.ExportConfirmation];
			case 3:
				return [FC.BusinessModels.ProcessingStageType.ExportPreProcessing,
				FC.BusinessModels.ProcessingStageType.Export,
				FC.BusinessModels.ProcessingStageType.Unattended];
			case 4:
				return [FC.BusinessModels.ProcessingStageType.Script];
			case 5:
				return [FC.BusinessModels.ProcessingStageType.DataVerificationPreProcessing,
				FC.BusinessModels.ProcessingStageType.DataVerificationPostProcessing,
				FC.BusinessModels.ProcessingStageType.VerificationPreProcessing,
				FC.BusinessModels.ProcessingStageType.VerificationPostProcessing,
				FC.BusinessModels.ProcessingStageType.DoubleVerificationMerge];
			case 6:
				return [FC.BusinessModels.ProcessingStageType.Processed];
			case 7:
				return [FC.BusinessModels.ProcessingStageType.UserDefined];
			case 8:
				return [FC.BusinessModels.ProcessingStageType.Any];
		}

		return [];
	};

	self.TableContainer = ko.observable(null);

	self.batchesPermitted = ko.observable(false);
	self.pageNavigation = new FC.Pages.PageNavigator(FC.Pages.BindingPage.Batches, self.batchesPermitted);

	self.BatchFunctions = {
		nonContextOperations: ko.observableArray([
			new FC.NonContextFunction({
				name: ko.observable(FC.Environment.Localization.Documents_aspx.PriorityButton),
				order: 12,
				action: function () {
					self.ShowPriorityDialog();
				},
				htmlClass: 'batchList__nonContextOperations__priority',
				visible: self.PriorityChangeVisible,
				enabled: ko.computed(function () {
					return self.TableContainer() && self.TableContainer().selectedRowCount() > 0;
				})
			}),
			new FC.NonContextFunction({
				name: ko.observable(FC.Environment.Localization.Documents_aspx.ExceptionsButton),
				order: 13,
				action: function () {
					self.SendToExceptions();
				},
				htmlClass: 'batchList__nonContextOperations__exceptions',
				visible: self.SendVisible,
				enabled: ko.computed(function () {
					if (!self.TableContainer() || self.TableContainer().selectedRowCount() !== 1) {
						return false;
					}
					return true;
				})
			}),
			new FC.NonContextFunction({
				name: ko.observable(FC.Environment.Localization.Documents_aspx.StageButton),
				order: 14,
				action: function () {
					self.ShowStagesDialog();
				},
				htmlClass: 'batchList__nonContextOperations__sendToStage',
				visible: self.SendVisible,
				enabled: ko.computed(function () {
					return self.TableContainer() && self.TableContainer().selectedRowCount() > 0;
				})
			}),
			new FC.NonContextFunction({
				name: ko.observable(FC.Environment.Localization.Documents_aspx.DeleteButton),
				order: 15,
				action: function () {
					self.Delete();
				},
				htmlClass: 'batchList__nonContextOperations__delete',
				visible: self.DeleteVisible,
				enabled: ko.computed(function () {
					return self.TableContainer() && self.TableContainer().selectedRowCount() > 0;
				})
			}),
			new FC.NonContextFunction({
				name: ko.observable(FC.Environment.Localization.Batches_aspx.Download_Selected_Batches),
				order: 16,
				title: ko.observable(FC.Environment.Localization.Batches_aspx.Download_Selected_Batches),
				action: function () {
					if (!self.TableContainer() || self.TableContainer().selectedRowCount() === 0) {
						return;
					}

					var selectedBatches = self.SelectedBatches();
					var ids = [];
					for (var i = 0; i < selectedBatches.length; i++) {
						ids.push(selectedBatches[i].Id);
					}

					FC.DataContext.Batch.downloadResults(ids, self.Refresh);
				},
				htmlClass: 'batchEditor__function',
				index: 90,
				svgId: ko.observable('#icon--download-20'),
				svgClass: ko.observable('batches_download-icon'),
				visible: self.canDownloadBatches,
				enabled: ko.computed(function () {
					if (!self.TableContainer() || self.TableContainer().selectedRowCount() === 0) {
						return false;
					}

					var selectedBatches = self.SelectedBatches();
					return selectedBatches.filter(function (b) { return b.HasDocumentResults; }).length === selectedBatches.length;
				})
			}),
			new FC.NonContextFunction({
				name: ko.observable(''),
				title: ko.observable(FC.Environment.Localization.Batches_aspx.RefreshButton),
				order: 100,
				action: function () {
					self.Refresh();
				},
				htmlClass: 'batchList__nonContextOperations__refresh left-separator'
			})
		])
	};

	var minimizationTimeOut = null;
	self.checkNonContextFunctionsMinimization = function () {
		if (minimizationTimeOut) {
			clearTimeout(minimizationTimeOut);
		}

		minimizationTimeOut = setTimeout(function () {
			FC.checkNonContextFunctionsMinimization(
				"#BatchesPage_MenuFunctionsPanel",
				".listLayout__actions",
				".listLayout__navigation");
		}, 100);
	};


	self.tableAjax = function (data, callback) {
		var onGotBatches = function (result) {
			callback({
				recordsTotal: result.TotalCount,
				recordsFiltered: result.TotalCount,
				data: result.Items ? result.Items : []
			});

			tableAdjustFnc();
		};

		self.GetBatches(onGotBatches,
			data.start,
			data.length);

	};


	self.OnPageOpening = function () {
		var displayedColumns = self.ColumnsDefault().concat([self.SettingsColumn])
		self.InitTable(displayedColumns);
		//Init reg. parameter columns from storage
		self.ColumnsRegParam(self.getRegParamColumns())
		self.GetActionVisibility(function () {
			self.Refresh();
		});

		FC.DataContext.Role.hasAnyPermission(
			FC.DataContext.Role.permissions.BatchListBrowsing, 
			function (canSeeBatchList) { self.batchesPermitted(canSeeBatchList); }
		);
	};

	self.InitTable = function (displayedColumns) {
		self.TableContainer(new FC.DataTableContainer(
			{
				columns: displayedColumns,
				createdRow: function (row, data) {
					var jtd = $("td:eq(0)", row);

					if (data.IsExpired) {
						jtd.addClass("batchList__expiration_already");
					} else {
						if (data.ExpireSoon) {
							jtd.addClass("batchList__expiration_soon");
						}
					}
				},
				ajax: self.tableAjax,
				serverSide: true,
				ordering: false,
				processing: false
			},
			"batches_table",
			"batches_table_selectAll",
			FC.Environment.Localization.Batches_aspx.NoBatches, null, null));
	}

	$(document).ready(self.checkNonContextFunctionsMinimization);
	$(window).resize(self.checkNonContextFunctionsMinimization);



	var dialoguesElement = document.getElementById("dialogues");
	if (dialoguesElement) {
		ko.applyBindings(self, document.getElementById("dialogues"));
	}
};
