﻿FC.Pages.DocumentsPage = function (batch, dependencies) {
	var self = this;

	$.extend(self, new FC.Pages.BasePageModel(dependencies.serverDataContext));

	self.BatchName = ko.observable(batch.Name);
	self.BatchId = ko.observable(batch.Id);

	self.loadingDocuments = ko.observable(false);

	self.DeleteVisible = ko.observable(false);
	self.PriorityChangeVisible = ko.observable(false);
	self.SendVisible = ko.observable(false);

	self.DeleteVisible.subscribe(function () {
		self.checkNonContextFunctionsMinimization();
	});
	self.PriorityChangeVisible.subscribe(function () {
		self.checkNonContextFunctionsMinimization();
	});
	self.SendVisible.subscribe(function () {
		self.checkNonContextFunctionsMinimization();
	});

	self.goToBatches = function () {
		FC.Utils.replaceLocation(FC.Utils.pageUrls.batches());
		return true;
	};

	self.reload = function () {
		self.documentsLoaded(false);
		self.loadingDocuments(true);
		self.TableContainer().reloadTable();
	}

	self.OnDeleteDocuments = function () {
		var documents = self.SelectedDocuments();
		var documentIds = [];
		for (var i = 0; i < documents.length; i++) {
			documentIds.push(documents[i].Id);
		}

		if (documentIds.length === 0) {
			return;
		}

		FC.DataContext.Document.deleteDocuments(self.BatchId(), documentIds, self.reload);
	};

	self.OnPriorityChange = function () {
		var documents = self.SelectedDocuments();
		var documentIds = [];
		for (var i = 0; i < documents.length; i++) {
			documentIds.push(documents[i].Id);
		}

		if (documentIds.length === 0) {
			return;
		}

		FC.DataContext.Document.setPriority(
			self.BatchId(),
			documentIds,
			self.PrioritySelector().selectedPriority(),
			self.reload);
	};

	self.OnSendToStage = function () {
		var documents = self.SelectedDocuments();
		var documentIds = [];
		for (var i = 0; i < documents.length; i++) {
			documentIds.push(documents[i].Id);
		}

		FC.DataContext.Document.sendToStage(
			self.BatchId(),
			documentIds,
			self.StageSelector().selectedUser(),
			self.StageSelector().selectedStage().SendingStageId,
			self.StageSelector().selectedPriority(),
			self.StageSelector().comment(),
			self.StageSelector().createTask(),
			self.reload);
	};

	self.OnSendToExceptions = function () {
		var documents = self.SelectedDocuments();
		var documentIds = [];
		for (var i = 0; i < documents.length; i++) {
			documentIds.push(documents[i].Id);
		}

		FC.DataContext.Document.sendToExceptions(
			self.BatchId(),
			documentIds,
			self.SenderToExceptions().comment(),
			self.SenderToExceptions().createTask(),
			self.reload);
	};

	self.PrioritySelector = ko.observable(new FC.Pages.PrioritySelector(self.OnPriorityChange));
	self.BatchesDeleter = ko.observable(new FC.DialogBase(self.OnDeleteDocuments, null, null, false, FC.Environment.Headless, false, false, false, "documentssDeleteDialogue"));
	self.StageSelector = ko.observable(new FC.Pages.StageSelector(true, self.OnSendToStage));
	self.SenderToExceptions = ko.observable(new FC.Pages.SenderToExceptions(true, self.OnSendToExceptions));

	self.ShowPriorityDialog = function () {
		if (self.SelectedDocuments().length > 0) {
			var selected = self.SelectedDocuments();
			var priority = -5;
			for (var i = 0; i < selected.length; i++) {
				if (selected[i].Priority > priority) {
					priority = selected[i].Priority;
				}
			}
			self.PrioritySelector().selectedPriority(priority);
			self.PrioritySelector().show();
		}
	};

	self.SendToExceptions = function () {
		if (self.SelectedDocuments().length > 0) {
			self.SenderToExceptions().show();
		}
	};

	self.ShowStagesDialog = function () {
		if (self.SelectedDocuments().length > 0) {
			self.StageSelector().updateStages(self.BatchId());
			self.StageSelector().show();
		}
	};

	self.Delete = function () {
		if (self.SelectedDocuments().length > 0) {
			self.BatchesDeleter().show();
		}
	};

	self.batchesPermitted = ko.observable(false);
	self.pageNavigation = new FC.Pages.PageNavigator(FC.Pages.BindingPage.Documents, self.batchesPermitted);

	self.Refresh = function () {
		self.pageNavigation.countPostponed();
		self.documentsLoaded(false);
		self.loadingDocuments(true);
		self.TableContainer().reloadTable();
	};

	self.SelectedDocuments = function () {
		return self.TableContainer().table.rows(".selected").data();
	};

	self.GetDocuments = function (onSuccess) {
		FC.DataContext.Batch.getBatchDocuments(
			self.BatchId(),
			function (docs) {
				self.documentsLoaded(true);
				self.checkEverythingLoaded();

				if (onSuccess) {
					onSuccess(docs);
				}
			});
	};

	self.GetActionVisibility = function (onSuccess) {
		FC.DataContext.Role.getAllowedActions(
			function (allowedActions) {
				if (!allowedActions.CanViewBatchList) {
					FC.Utils.replaceLocation(FC.Utils.prepareRedirectUrl(FC.Environment.AppPath + FC.Environment.HomeUrl));
					return;
				}

				self.DeleteVisible(allowedActions.CanDelete);
				self.PriorityChangeVisible(allowedActions.CanChangePriority);
				self.SendVisible(allowedActions.CanSendToStage);

				self.actionVisibilityLoaded(true);
				self.checkEverythingLoaded();

				if (onSuccess) {
					onSuccess();
				}
			}
		);
	};

	self.actionVisibilityLoaded = ko.observable(false);
	self.documentsLoaded = ko.observable(false);

	self.checkEverythingLoaded = function () {
		if (self.actionVisibilityLoaded() &&
			self.documentsLoaded()) {

			self.loadingDocuments(false);
		}
	};

	self.everythingIsNotLoaded = function () {
		self.actionVisibilityLoaded(false);
		self.documentsLoaded(false);
		self.loadingDocuments(true);
	};

	self.TableContainer = ko.observable(null);

	self.DocumentFunctions = {
		nonContextOperations: ko.observableArray([
			new FC.NonContextFunction({
				name: ko.observable(FC.Environment.Localization.Documents_aspx.PriorityButton),
				order: 12,
				action: function () {
					self.ShowPriorityDialog();
				},
				htmlClass: 'batchList__nonContextOperations__priority',
				visible: self.PriorityChangeVisible,
				enabled: ko.computed(function () {
					if (!self.TableContainer() || self.TableContainer().selectedRowCount() === 0) {
						return false;
					}

					var selectedDocs = self.SelectedDocuments();
					return selectedDocs.filter(function (d) { return d.HasTask; }).length === 0;
				})
			}),
			new FC.NonContextFunction({
				name: ko.observable(FC.Environment.Localization.Documents_aspx.ExceptionsButton),
				order: 13,
				action: function () {
					self.SendToExceptions();
				},
				htmlClass: 'batchList__nonContextOperations__exceptions',
				visible: self.SendVisible,
				enabled: ko.computed(function () {
					if (!self.TableContainer() || self.TableContainer().selectedRowCount() === 0) {
						return false;
					}

					var selectedDocs = self.SelectedDocuments();
					return selectedDocs.filter(function (d) { return d.HasTask; }).length === 0;
				})
			}),
			new FC.NonContextFunction({
				name: ko.observable(FC.Environment.Localization.Documents_aspx.StageButton),
				order: 14,
				action: function () {
					self.ShowStagesDialog();
				},
				htmlClass: 'batchList__nonContextOperations__sendToStage',
				visible: self.SendVisible,
				enabled: ko.computed(function () {
					if (!self.TableContainer() || self.TableContainer().selectedRowCount() === 0) {
						return false;
					}

					var selectedDocs = self.SelectedDocuments();
					return selectedDocs.filter(function (d) { return d.HasTask; }).length === 0;
				})
			}),
			new FC.NonContextFunction({
				name: ko.observable(FC.Environment.Localization.Documents_aspx.DeleteButton),
				order: 15,
				action: function () {
					self.Delete();
				},
				htmlClass: 'batchList__nonContextOperations__delete',
				visible: self.DeleteVisible,
				enabled: ko.computed(function () {
					if (!self.TableContainer() || self.TableContainer().selectedRowCount() === 0) {
						return false;
					}

					var selectedDocs = self.SelectedDocuments();
					return selectedDocs.filter(function (d) { return d.HasTask; }).length === 0;
				})
			}),
			new FC.NonContextFunction({
				name: ko.observable(''),
				title: ko.observable(FC.Environment.Localization.Batches_aspx.RefreshButton),
				order: 100,
				action: function () {
					self.Refresh();
				},
				htmlClass: 'batchList__nonContextOperations__refresh left-separator'
			})
		])
	};

	var minimizationTimeOut = null;
	self.checkNonContextFunctionsMinimization = function () {
		if (minimizationTimeOut) {
			clearTimeout(minimizationTimeOut);
		}

		minimizationTimeOut = setTimeout(function () {
			FC.checkNonContextFunctionsMinimization(
				"#DocumentsPage_MenuFunctionsPanel",
				".listLayout__actions",
				".listLayout__navigation");
		}, 1);
	};

	self.taskSelected = false;

	self._tableDrawn = function () {
		var onTaskClicked = function (e) {
			if (self.taskSelected) {
				return;
			}

			var link = e.target.getAttribute("data-href");
			if (link) {
                self.taskSelected = true;
                FC.Utils.locationReplaceWrapper(link);
			}
		};

		$("tbody td .taskLink")
			.off('mousedown', onTaskClicked)
			.on('mousedown', onTaskClicked);
	};

	self.DocumentsColumns =
        [
            {
            	width: "20px",
            	className: "select-checkbox",
            	defaultContent: "",
            	sortable: false
            },
			{
				width: "64px",
				data: 'Priority',
				name: 'Priority',
				render: function (data, type, full) {
					if (type !== "display") {
						return data;
					}

					return full.PriorityIconVisible ? "<img src=\"" + FC.Environment.AppPathNoTennant + full.PriorityIconUrl + "\"/>" : "";
				}
			},
			{
				width: "60px",
				data: 'Index',
				name: 'Index'
			},
			{
				data: 'TaskId',
				name: 'TaskId',
				render: function (data, type, full) {
					if (type !== "display") {
						return data;
					}

					if (full.HasTask && full.CanOpenTask) {
						var url = FC.Utils.prepareRedirectUrl("Verify?projectId=" + FC.SessionInfo.ProjectID +
							"&roleId=" + FC.SessionInfo.RoleID + "&taskId=" + data + "&returnTo=BatchList");

						return "<span data-href=\"" + url + "\" class=\"link  taskLink\">" + full.OpenIdString + "</span>";
					}

					return data;
				}
			},
			{
				data: 'StageName',
				name: 'StageName'
			},
			{
				width: "60px",
				data: 'PageCount',
				name: 'PageCount'
			},
			{
				width: "60px",
				data: 'Characters',
				name: 'Characters'
			},
			{
				width: "60px",
				data: 'UncertainCharacters',
				name: 'UncertainCharacters'
			},
            {
            	width: "60px",
            	data: 'CharactersForVerification',
            	name: 'CharactersForVerification'
            },
            {
            	width: "20px",
            	data: 'HasErrors',
            	name: 'HasErrors',
            	render: function (data, type) {
            		if (type !== "display") {
            			return data;
            		}
            		return data ? "<img src=\"" + FC.Environment.AppPathNoTennant + FC.Environment.Localization.NonLocalizedResources.CheckImagePath + "\"/>" : "";
            	}
            },
            {
            	width: "60px",
            	data: 'Id',
            	name: 'Id'
            },
			{
				width: "20px",
				data: 'HasAttachments',
				name: 'HasAttachments',
				render: function (data, type) {
					if (type !== "display") {
						return data;
					}
					return data ? "<img src=\"" + FC.Environment.AppPathNoTennant + FC.Environment.Localization.NonLocalizedResources.NoAttachmentsImagePath + "\"/>" : "";
				}
			}
        ];

	self.OnPageOpening = function () {
		self.TableContainer(new FC.DataTableContainer(
			{
				columns: self.DocumentsColumns,
				ajax: function (data, callback) {
					var onGotDocuments = function (result) {
						callback({
							data: result ? result : []
						});
					};

					self.GetDocuments(onGotDocuments);
				},
				order: [[2, 'asc']],
				processing: false
			},
			"documents_table",
			"documents_table_selectAll",
			FC.Environment.Localization.Documents_aspx.EmptyBatch,
			null,
			self._tableDrawn));

		self.everythingIsNotLoaded();

		self.GetActionVisibility(function () {
			self.pageNavigation.countPostponed();
		});

		self.TableContainer().reloadTable();

		$(document).ready(self.checkNonContextFunctionsMinimization);
		$(window).resize(self.checkNonContextFunctionsMinimization);

		FC.DataContext.Role.hasAnyPermission(
			FC.DataContext.Role.permissions.BatchListBrowsing, 
			function (canSeeBatchList) { self.batchesPermitted(canSeeBatchList); }
		);
	};

	ko.applyBindings(self, document.getElementById("dialogues"));
};