﻿FC.Pages.PageNavigator = function (currentPage, batchesPermitted) {
	var self = this;

	self.batchesPermitted = ko.isObservable(batchesPermitted) ? batchesPermitted : ko.observable(!!batchesPermitted);
	self.currentPage = ko.isObservable(currentPage) ? currentPage : ko.observable(currentPage);

	self.hasPostponed = ko.observable(false);
	self.postponedCount = ko.observable(0);
	self.postponedUrl = ko.observable('');
	self.gotPostponed = ko.observable(false);

	self.countPostponed = function (onSuccess, onFail) {

		FC.DataContext.Task.countPostponed(function (taskCount) {
			self.hasPostponed(taskCount > 0);
			self.postponedCount(taskCount || 0);
			self.postponedUrl(FC.Utils.pageUrls.postponedTasks());
			self.gotPostponed(true);

			if (onSuccess) {
				onSuccess();
			}
		}, onFail);
	};


	self.goToBatches = function () {
		FC.Utils.replaceLocation(FC.Utils.pageUrls.batches());
		return true;
	};

	self.goToQueues = function () {
		FC.Utils.replaceLocation(FC.Utils.pageUrls.queues());
		return true;
	};

	self.goToPostponed = function () {
		FC.Utils.replaceLocation(self.postponedUrl());
		return true;
	};

	self.operations = [
		new FC.NonContextFunction({
			name: ko.observable(FC.Environment.Localization.QueueList_aspx.QueuesTitle),
			order: 1,
			action: function () {
				self.goToQueues();
			},
			htmlClass: '',
			activated: ko.computed(function () {
				return self.currentPage() === FC.Pages.BindingPage.Queues;
			})
		}),

		new FC.NonContextFunction({
			name: ko.observable(FC.Environment.Localization.Batches_aspx.BatchesTitle),
			order: 2,
			action: function () {
				self.goToBatches();
			},
			htmlClass: '',
			visible: batchesPermitted,
			activated: ko.computed(function () {
				return self.currentPage() === FC.Pages.BindingPage.Batches;
			})
		}),

		new FC.NonContextFunction({
			name: ko.observable(FC.Environment.Localization.Tasks.PostponedTasksHeader),
			summary: self.postponedCount,
			order: 3,
			action: function () {
				self.goToPostponed();
			},
			htmlClass: '',
			enabled: self.hasPostponed,
			activated: ko.computed(function () {
				return self.currentPage() === FC.Pages.BindingPage.PostponedTasks;
			})
		})
	];
}

FC.Pages.BindingPage = {
	Queues: 1,
	Batches: 2,
	Tasks: 3,
	PostponedTasks: 4,
	Documents: 5
};