﻿FC.Pages.QueueListPage = function (dependencies) {
	var self = this;

	$.extend(self, new FC.Pages.BasePageModel(dependencies.serverDataContext));

	self.isMiniMode = !!FC.Utils.getQueryParameter("mode");
	

	self.loading = ko.observable(true);
	self.HasPostponed = ko.observable(false);
	self.PostponedText = ko.observable('');
	self.PostponedUrl = ko.observable('');

	self.canExploreQueues = ko.observable(false);
	self.Queues = ko.observableArray([]);

	self.isFullWidthContainer = ko.observable((JSON.parse(localStorage.getItem('isFullWidthContainerQueues'))));

	self.ExploreQueue = function (queue, event, onSuccess, onFail) {
		if (!queue || !queue.QueueHasTasks() || !self.canExploreQueues()) {
			if (onFail) {
				onFail();
			}

			return;
		}

		if (self.refreshTimeout) {
			clearTimeout(self.refreshTimeout);
		}
		FC.Utils.replaceLocation(FC.Utils.prepareRedirectUrl(
			"Tasks?projectId=" + FC.SessionInfo.ProjectID +
			"&roleId=" + FC.SessionInfo.RoleID +
			"&postponed=false&" +
			queue.StageIdsString()));
	};

	self.Refresh = function () {
		self.GetQueues(function() {
			self.refreshTimeout = setTimeout(self.Refresh, FC.Environment.QueueRefreshPeriod);
		});
	};

	self.batchesPermitted = ko.observable(false);
	self.pageNavigation = new FC.Pages.PageNavigator(FC.Pages.BindingPage.Queues, self.batchesPermitted);

	self.HasAnyTasks = ko.observable(false);

    self.taskIsGetting = false;

	self.GetTaskFromAny = function () {
		self.autoGetModeStageIds([]);

		if (!self.HasAnyTasks()) {
			if (!self.isMiniMode) {
				self.autoGetMode(true);
			}
			return;
		}

		if (self.taskIsGetting) {
			return;
		}

		self.taskIsGetting = true;
		var url = FC.Utils.prepareRedirectUrl("Verify?projectId=" +
			FC.SessionInfo.ProjectID +
			"&roleId=" +
			FC.SessionInfo.RoleID);
		if (self.autoGetMode()) {
			url += "&autoget=true";
		}

		if (self.refreshTimeout) {
			clearTimeout(self.refreshTimeout);
		}

        FC.Utils.locationReplaceWrapper(url);
	};
	
	self.GetTaskFromQueue = function (queue) {
		if (self.taskIsGetting) {
			return;
		}

		self.taskIsGetting = true;
		var url = FC.Utils.prepareRedirectUrl("Verify?projectId=" + FC.SessionInfo.ProjectID +
            "&roleId=" + FC.SessionInfo.RoleID + "&" + queue.StageIdsString());
		if (self.autoGetMode()) {
			url += "&autoget=true";
		}

		if (self.refreshTimeout) {
			clearTimeout(self.refreshTimeout);
        }

        FC.Utils.locationReplaceWrapper(url);
	};

	self.GetQueues = function (onSuccess, onFail) {
		FC.DataContext.Queue.getQueues(function(queues) {
			var taskCount = 0;

			var newQueues = queues.map(function(item) {
				taskCount += item.FormedTaskCount;
				taskCount += item.PersonalTaskCount;

				var qp = new FC.Viewmodels.QueueAdapter(item);

				qp.MaxPriorityIconUrl(FC.Viewmodels.QueueAdapter.GetPriorityIcon(item.MaxPriority));

				return qp;
			});

			if (queues.length > 0) {
				self.pageNavigation.hasPostponed(queues[0].PostponedTaskCount > 0);
				self.pageNavigation.postponedCount(queues[0].PostponedTaskCount || 0);
				self.pageNavigation.postponedUrl(FC.Utils.pageUrls.postponedTasks());
				self.pageNavigation.gotPostponed(true);
			}

			// чтобы не мигал интерфейс, если ничего не поменялось
			if (FC.Viewmodels.QueueAdapter.QueuesEquals(newQueues, self.Queues())) {
				self.gotQueues(true);
				if (onSuccess) {
					onSuccess();
				}

				return;
			}

			self.Queues(newQueues);
			self.HasAnyTasks(taskCount > 0);

            if (self.autoGetMode()) {
                if (self.autoGetModeQueue()) {
	                if (self.autoGetModeQueue().QueueHasTasks()) {
		                self.GetTaskFromQueue(self.autoGetModeQueue());
	                }
                } else {
	                self.GetTaskFromAny();
                }
            }

			self.gotQueues(true);

			if (onSuccess) {
				onSuccess();
			}
		}, onFail);
	};

	self.CheckPermissions = function (onSuccess, onFail) {

		FC.DataContext.Role.hasAnyPermission(FC.DataContext.Role.permissions.BatchListBrowsing,
			function (canSeeBatchList) {
				self.batchesPermitted(canSeeBatchList);

				FC.DataContext.Role.hasAnyPermission(FC.DataContext.Role.permissions.TaskQueueBrowsing,
					function (canExploreQueues) {
						self.canExploreQueues(canExploreQueues);
						self.checkedPermissions(true);

						if (onSuccess) {
							onSuccess();
						}
					});
			}, onFail);
	};

	self.SetToggleIsFullWidthContainer = function () {
		var isFullWidthContainerQueues = JSON.parse(localStorage.getItem('isFullWidthContainerQueues'));
		localStorage.setItem('isFullWidthContainerQueues', !isFullWidthContainerQueues);
		self.isFullWidthContainer(!isFullWidthContainerQueues);
	}

	self.gotQueues = ko.observable(false);
	self.checkedPermissions = ko.observable(false);

	self.QueueFunctions = {
		nonContextOperations: ko.observableArray([
			new FC.NonContextFunction({
				name: ko.observable(''),
				title: ko.observable(FC.Environment.Localization.QueueList_aspx.FullWidthButton),
				order: 1,
				action: function () {
					self.SetToggleIsFullWidthContainer();
				},
				htmlClass: 'queueList__nonContextOperations__fullWidth',
				activated: ko.computed(function () {
					return self.isFullWidthContainer();
				})
			}),
			new FC.NonContextFunction({
				name: ko.observable(''),
				title: ko.observable(FC.Environment.Localization.QueueList_aspx.RefreshButton),
				order: 2,
				action: function () {
					self.Refresh();
				},
				htmlClass: 'queueList__nonContextOperations__refresh'
			}),
		])
	};

	self.refreshTimeout = null;

	self.OnPageOpening = function (onSuccess) {
		self.loading(true);

		var loadingFinished = function () {
			if (self.loading() && self.pageNavigation.gotPostponed() && self.gotQueues() && self.checkedPermissions()) {
				self.loading(false);
				if (onSuccess) {
					onSuccess();
				}
			}
		};

		self.GetQueues(function () {
			loadingFinished();
			self.CheckPermissions(loadingFinished);
			self.refreshTimeout = setTimeout(self.Refresh, FC.Pages.QueueListPage.refreshPeriod);
		});
	};

    self.autoGetMode = ko.observable(
        document.getElementById('autoget').value.toLowerCase() === 'true' &&
        !(FC.Utils.Browser.isIE() && FC.Environment.IEVerificationNewWindow)
    );

    self.autoGetModeStageIds = ko.observableArray(document.getElementById('autogetStages').value.toLowerCase().split(',')
		.map(function(stageId) {
			return parseInt(stageId);
		}));

	self.autoGetModeQueue = ko.computed(function () {
		if (!self.autoGetModeStageIds() || self.autoGetModeStageIds().length === 0) return null;
		var stageid = self.autoGetModeStageIds()[0];
		var queue = self.Queues().filter(function (queue) {
			var queueStages = queue.StageIdsString().split('&').map(function(stageParam) {
				return parseInt(stageParam.replace('stageId=', ''));
			});
			return queueStages.indexOf(stageid) >= 0;
		});
		return queue[0];
	});
	self.exitAutoGetMode = function() {
		self.autoGetMode(false);
    }


	self.autoGetMessage = ko.computed(function() {
		if (self.autoGetModeQueue()) {
			return FC.Environment.Localization.QueueList_aspx.AutoGetWithQueueMessageSecondLine.replace('{queue}',
				self.autoGetModeQueue().QueueName());
		} else {
			return FC.Environment.Localization.QueueList_aspx.AutoGetMessageSecondLine;
		}
	});

	self.getAnyTaskText = ko.computed(function() {
		return self.HasAnyTasks() || self.isMiniMode
			? FC.Environment.Localization.QueueList_aspx.GetAnyTask
			: FC.Environment.Localization.QueueList_aspx.WaitTask;
	});

	hotkeys('ctrl+g', function (event, handler) {
		event.preventDefault();
		self.GetTaskFromAny();
	});
};