﻿FC.Pages.SelectProjectAndRolePage = function (pageModel, dependencies) {
	var self = this;

	$.extend(self, new FC.Pages.BasePageModel(dependencies.serverDataContext));

	self.NoProjects = ko.observable(true);
	self.loading = ko.observable(false);
	self.Projects = ko.observableArray([]);
	self.CurrentProject = ko.observable(null);
	self.CurrentProjectId = ko.observable(pageModel.CurrentProjectId);
	self.Roles = ko.observableArray([]);
	self.CurrentRole = ko.observable(null);

	self.CanSelectRole = pageModel.CanSelectRole;

	self.selectRoleSimilarToCurrent = function () {
		if (pageModel.CurrentRoleType > 0) {
			var roles = self.Roles();
			for (var i = 0; i < roles.length; i++) {
				if (roles[i].Type() === pageModel.CurrentRoleType) {
					self.CurrentRole(roles[i]);
					break;
				}
			}
		}
	};

	self.sortRoles = function (id, roles) {
		var chosenRole = localStorage.getItem("chosenRole-" + id);
		if (chosenRole) {
			roles.sort(function (a, b) {
				return a.Name === chosenRole ? -1 : b.Name === chosenRole ? 1 : a.Permissions - b.Permissions;
			});
		} else {
			roles.sort(function (a, b) {
				return a.Permissions - b.Permissions;
			});
		}
	}

	if (pageModel && pageModel.Projects && pageModel.Projects.length > 0 && (!pageModel.CanSelectRole || pageModel.Roles && pageModel.Roles.length > 0)) {
		self.NoProjects(false);
		self.Projects(ko.utils.arrayMap(pageModel.Projects, function (item) {
			return new FC.BusinessModels.ProjectInformation(item.Id, item.UniqueId,
				item.Name, item.IsTestProject);
		}));

		var projectFound = false;

		if (self.CurrentProjectId() > 0) {
			for (var i = 0; i < pageModel.Projects.length; i++) {
				if (pageModel.Projects[i].Id === self.CurrentProjectId()) {
					self.CurrentProject(self.Projects()[i]);
					projectFound = true;
					break;
				}
			}
		}

		if (!projectFound) {
			self.CurrentProject(self.Projects()[0]);
			self.CurrentProjectId(self.Projects()[0].Id());
		}
		var currentProjectId = self.CurrentProject().Id();

		if (pageModel.Roles) {
			self.sortRoles(currentProjectId, pageModel.Roles);
		}

		self.Roles(ko.utils.arrayMap(pageModel.Roles, function (item) {
			return new FC.BusinessModels.RoleProperties(item.Id, item.Type,
				item.Name, item.Permissions);
		}));

		self.selectRoleSimilarToCurrent();
	}

	self.supressEvents = false; // Блокирует возникновение ko-событий при инициализации

	self.ChangeProject = function (data, event, onSuccess, onFail) {
		if (self.supressEvents) {
			return;
		}

		if (self.Projects() && self.Projects().length > 0 && self.CurrentProject()) {
			self.CurrentProjectId(self.CurrentProject().Id());
			self.loading(true);
			self.GetRoles(onSuccess);
		} else {
			if (onFail) {
				onFail();
			}
		}
	};

	self.Select = function (data, event, onSuccess, onFail) {

		var currentProject = self.CurrentProject();
		if (!currentProject || !self.Projects() || self.Projects().length === 0) {
			if (onFail) {
				onFail("no current project");
			}

			return;
		}

		if (dependencies.environment.StationType === FC.StationType.WebScanning) {
			FC.Utils.replaceLocation(FC.Utils.prepareRedirectUrl("Batches?projectGuid=" +
				currentProject.UniqueId().toUpperCase()));
		} else if (dependencies.environment.StationType === FC.StationType.WebCapture) {
			FC.Utils.replaceLocation(FC.Utils.prepareRedirectUrl("Main?projectId=" +
				currentProject.Id()));
		} else {
			var currentRole = self.CurrentRole();
			if (!currentRole || !self.Roles() || self.Roles().length === 0) {
				if (onFail) {
					onFail("no current role");
				}

				return;
			}

			var targetPage = "QueueList";

			localStorage.setItem("chosenRole-" + currentProject.Id(), currentRole.Name());

			FC.Utils.replaceLocation(FC.Utils.prepareRedirectUrl(targetPage + "?projectId=" +
				currentProject.Id() +
				"&roleId=" +
				currentRole.Id()));
		}
	};

	self.Refresh = function (callback) {
		window.location.reload();
	};

	self.GetRoles = function (callback) {
		if (!self.CurrentProjectId() || !self.Projects() || self.Projects().length === 0 || !self.CanSelectRole) {
			if (callback) {
				callback();
			}

			self.loading(false);
			return;
		}

		self.loading(true);
		var onSuccess = function (result) {
			if (!result || result.length === 0) {
				if (callback) {
					callback("no roles in current project " + self.CurrentProjectId());
				}
				self.loading(false);
				return;
			}

			if (result) {
				self.sortRoles(self.CurrentProjectId(), result);
			}

			self.Roles(ko.utils.arrayMap(result, function (item) {
				return new FC.BusinessModels.RoleProperties(item.Id, item.Type,
					item.Name, item.Permissions);
			}));

			self.selectRoleSimilarToCurrent();
			self.loading(false);

			if (callback) {
				callback();
			}
		}

		FC.DataContext.Role.getRoles(self.CurrentProjectId(), onSuccess);
	};

	self.OnPageOpening = function (onSuccess) {
		if (onSuccess) {
			onSuccess();
		}
	};
};