﻿FC.Pages.TasksPage = function (dependencies) {
	var self = this;

	$.extend(self, new FC.Pages.BasePageModel(dependencies.serverDataContext));

	self.loadingTasks = ko.observable(false);

	self.postponedMode = ko.observable(false);
	self.currentPageType = ko.computed(function () {
		var postponedMode = self.postponedMode();
		return postponedMode ? FC.Pages.BindingPage.PostponedTasks : FC.Pages.BindingPage.Tasks;
	});

	self.tasksTitle = ko.observable(FC.Environment.Localization.Tasks.TasksHeader);

	self.batchesPermitted = ko.observable(false);
	self.pageNavigation = new FC.Pages.PageNavigator(self.currentPageType, self.batchesPermitted);

	self.goToQueues = function () {
		FC.Utils.replaceLocation(FC.Utils.pageUrls.queues());
		return true;
	}

	var tableAdjustFnc = function () {
		var table = $.fn.dataTable.fnTables(true);
		if (table.length > 0) {
			$(table).dataTable().fnAdjustColumnSizing(false);
		}
	};

	$(window).resize(tableAdjustFnc);

	self.Refresh = function () {
		self.TableContainer().reloadTable();
		self.pageNavigation.countPostponed();
	};

	self.GetTasks = function (onSuccess) {
		self.loadingTasks(true);

		var onAnyResult = function () {
			self.loadingTasks(false);
		};

		var onSuccessResult = function (tasks) {
			if (onSuccess) {
				onSuccess(tasks);
			}
		};

		if (self.postponedMode()) {
			FC.DataContext.Task.getPostponedTasks(onAnyResult, onSuccessResult);
		} else {
			var stageIdsString = FC.Utils.getQueryParameterArray("stageId");
			var stageIds = [];
			for (var i = 0; i < stageIdsString.length; i++) {
				stageIds.push(parseInt(stageIdsString[i]));
			}

			FC.DataContext.Task.getTasksByStages(
				stageIds,
				onAnyResult,
				self.FilterRegParameterName,
				self.FilterRegParameterValue,
				onSuccessResult
				);
		}
	};

	self.GetTaskFilterTypes = function () {
		var filters = [
			new FC.InfrastructureModels.FilterType(
				FC.BusinessModels.BatchFilterType.None,
				FC.Environment.Localization.GlobalResources.NoneTasksFilter),
			new FC.InfrastructureModels.FilterType(
				FC.BusinessModels.BatchFilterType.RegParameter,
				FC.Environment.Localization.GlobalResources.TasksRegParamFilter)
		];

		return filters;
	};

	self.TaskFilterValues = ko.observableArray(self.GetTaskFilterTypes());

	var lastCurrentTaskFilterTypeId = parseInt(localStorage.getItem('tasks__CurrentTaskFilterType'));
	if (FC.Environment.CanDownloadTaskResult && isNaN(lastCurrentTaskFilterTypeId)) {
		lastCurrentTaskFilterTypeId = 0;
	}
	var lastCurrentTaskFilterType = self.TaskFilterValues().filter(function (item) {
		return item.Id() === lastCurrentTaskFilterTypeId;
	})[0];
	self.CurrentTaskFilterType = ko.observable(lastCurrentTaskFilterType || self.TaskFilterValues()[0]);
	self.CurrentTaskFilterType.subscribe(function (val) {
		if (val) {
			localStorage.setItem('tasks__CurrentTaskFilterType', val.Id());
		} else {
			localStorage.removeItem('tasks__CurrentTaskFilterType');
		}
	});


	self.TaskFilterChanged = function () {
		var currentTaskFilter = self.CurrentTaskFilterType();

		self.FilterByRegParameter(false);

		switch (currentTaskFilter.Id()) {
			case FC.BusinessModels.BatchFilterType.RegParameter:
				self.FilterByRegParameter(true);
				break;
			case FC.BusinessModels.BatchFilterType.None:
				self.FilterByRegParameter(false);
				self.FilterRegParameterName("");
				self.FilterRegParameterValue("");
			default: break;
		}
	};

	self.taskIsGetting = false;
	self.GetTask = function (data, event, onFail) {
		if (self.postponedMode() || self.noTasks()) {
			if (onFail) {
				onFail();
			}
			return;
		}

		if (self.taskIsGetting) {
			return;
		}

		self.taskIsGetting = true;
		var stageIds = FC.Utils.getQueryParameterArray("stageId");
		var stageIdsString = "";
		for (var i = 0; i < stageIds.length; i++) {
			stageIdsString += "&stageId=" + stageIds[i];
		}
		FC.Utils.replaceLocation(FC.Utils.prepareRedirectUrl("Verify?projectId=" +
			FC.SessionInfo.ProjectID +
			"&roleId=" +
			FC.SessionInfo.RoleID +
			stageIdsString +
			"&returnTo=Tasks"));
	};

	self.TableContainer = ko.observable(null);

	self.noTasks = ko.observable(true);

	self.taskFunctions = {
		nonContextOperations: ko.observableArray([
			new FC.NonContextFunction({
				name: ko.observable(''),
				title: ko.observable(FC.Environment.Localization.Tasks.RefreshButton),
				order: 100,
				action: function () {
					self.Refresh();
				},
				htmlClass: 'batchList__nonContextOperations__refresh'
			})
		])
	};

	self.tableAjax = function (data, callback) {
		self.GetTasks(function (result) {
			self.noTasks(!result || result.length == 0);
			if (callback) {
				callback({
					data: result ? result : []
				});
			}
		});
	};

	self.taskSelected = false;

	self._tableDrawn = function () {
		var onTaskClicked = function (e) {
			if (self.taskSelected) {
				return;
			}

			var link = e.target.getAttribute("data-href");
			if (link) {
				self.taskSelected = true;
                FC.Utils.locationReplaceWrapper(link);
			}
		};

		$("tbody td .taskLink")
			.off('mousedown', onTaskClicked)
			.on('mousedown', onTaskClicked);
	};

	self.ApplyFilter = function () {
		self.Refresh();
	};

	self.FilterByRegParameter = ko.observable(localStorage.getItem('tasks__FilterByRegParameter') === 'true');
	self.FilterByRegParameter.subscribe(function (val) {
		localStorage.setItem('tasks__FilterByRegParameter', val);
	});

	self.FilterRegParameterName = ko.observable(localStorage.getItem('tasks__FilterRegParameterName'));
	self.FilterRegParameterName.subscribe(function (val) {
		localStorage.setItem('tasks__FilterRegParameterName', val);
	});

	self.FilterRegParameterValue = ko.observable(localStorage.getItem('tasks__FilterRegParameterValue'));
	self.FilterRegParameterValue.subscribe(function (val) {
		localStorage.setItem('tasks__FilterRegParameterValue', val);
	});

	/** Column object constructor **/
	function column (props, render, createdCell ) {
		this.id = props.id;

		this.staticColumn = !!props.staticColumn ;

		this.name = props.name;
		this.data = props.data
		this.visibleTitle = props.visibleTitle;

		if(props.defaultContent!==undefined) {
			this.defaultContent = props.defaultContent
		}

		if(render!==undefined){
			this.render = render;
		}

		if(createdCell!==undefined) {
			this.createdCell = createdCell;
		}

		if(props.width!==undefined) {
			this.width = props.width;
		}

		if(props.className!==undefined) {
			this.className = props.className
		}

		this.visible = self.getColumnVisibility(props.id, false);
		this.showColumn = ko.observable(self.getColumnVisibility(props.id, false));
		this.showColumn.subscribe(function (newVisible) {
			this.visible = newVisible;
			self.TableContainer().table.columns(this.id).visible(newVisible);
			self.ShowColumnDropDown();
		}, this);
	}

	/** Reg column object constructor **/
	function regParamColumn (props ) {
		this.id = props.id;

		this.name = props.name;
		this.data = "RegParams";
		this.visibleTitle = props.name;

		this.render = function (data) {
			if (!data) return ''
			var regParameter = data.find(function (item) {
				return item.Name == props.name
			})
			return regParameter ? regParameter.Value : ""
		}
		this.visible = self.getColumnVisibility(props.id, true);
		this.showColumn = ko.observable(self.getColumnVisibility(props.id, true));
		this.showColumn.subscribe(function (newVisible) {

			this.visible = newVisible;
			var index = self.ColumnsRegParam().findIndex(function (column) { return column.id === this.id}.bind(this))
			self.TableContainer().table.columns(self.ColumnsDefault().length + index).visible(newVisible);
			self.ShowColumnDropDown();
		}, this);
	}

	/**
	 * Function for get column visibility from local storage
	 * @param id Column id
	 * @param isRegParam If true - get parameter for regParam
	 * @returns {boolean}
	 */
	self.getColumnVisibility = function (id, isRegParam) {
		var columnsFromStore = null;
		if(isRegParam) {
			var storedValue = localStorage.getItem('tasks__ColumnRegParamFilter');
			columnsFromStore = !!storedValue?JSON.parse(storedValue):null
		} else {
			var storedValue = localStorage.getItem('tasks__ColumnDefaultFilter')
			columnsFromStore = !!storedValue?JSON.parse(storedValue):null
		}
		if(!columnsFromStore || !columnsFromStore.length > 0) {
			return true
		} else {
			var storedVisible = columnsFromStore.find(function (column) {return column.id ===id})
			if(storedVisible===undefined) {
				return true
			} else {
				return storedVisible.visible
			}
		}
	}

	/** Get reg parameter from local storage **/
	self.getRegParamColumns = function () {
		var storedValue = localStorage.getItem('tasks__ColumnRegParamFilter');
		var regParamColumnsFromStore = !!storedValue?JSON.parse(storedValue):null
		if (!regParamColumnsFromStore || !regParamColumnsFromStore.length > 0) {
			return []
		} else {
			return regParamColumnsFromStore.map(function (column) {
				return new regParamColumn({
					id: column.id,
					name: column.name
				})
			})
		}
	}

	/** Array of default columns filter **/
	self.ColumnsDefault = ko.observableArray([
		new column({
			id: 0,
			name: 'Id',
			data: 'Id',
			minWidth: '100px',
			visibleTitle: FC.Environment.Localization.GlobalResources.TaskIdColumn
		},
		function (data, type, full) {
			if (type !== "display") {
				return data;
			}

			if (full.CanOpenTask) {

				var stageIds = FC.Utils.getQueryParameterArray("stageId");
				var stageIdsString = "";
				for (var i = 0; i < stageIds.length; i++) {
					stageIdsString += "&stageId=" + stageIds[i];
				}
				var url = FC.Utils.prepareRedirectUrl(
					"Verify?projectId=" +FC.SessionInfo.ProjectID +
					"&roleId=" + FC.SessionInfo.RoleID +
					"&taskId=" + data +
					"&returnTo=Tasks"+
					(self.postponedMode() ?  "&fromPostponed=true": stageIdsString )
					);

				return "<span data-href=\"" + url + "\" class=\"link  taskLink\">" + full.OpenIdString + "</span>";
			}

			return data;
		}),
		new column({
				id: 1,
				width: '64px',
				data: 'IsPersonal',
				name: 'IsPersonal',
				visibleTitle: FC.Environment.Localization.GlobalResources.IsPersonalColumn
			},
			function (data, type) {
				if (type !== "display") {
					return data;
				}

				return data ? "<img src=\"" + FC.Environment.AppPathNoTennant + "Content/images/PersonalTask.png\"/>" : "";
			}),
		new column({
			id: 2,
			width: '140px',
			name: 'CreationTimeString',
			data: 'CreationTimeString',
			visibleTitle: FC.Environment.Localization.GlobalResources.CreationTimeColumn
		}),
		new column({
			id: 3,
			name: 'BatchName',
			data: 'BatchName',
			visibleTitle: FC.Environment.Localization.GlobalResources.BatchNameColumn
		}),
		new column({
			id: 4,
			name: 'Comment',
			data: 'Comment',
			visibleTitle: FC.Environment.Localization.GlobalResources.CommentColumn
		},
		function (data, type) {
			if (type !== "display" || !data) {
				return data;
			}

			var res = document.createElement('div');
			var elem = document.createTextNode(data);
			res.appendChild(elem);
			return res.innerHTML;
		}),
		new column({
			id: 5,
			data: 'PostponementTime',
			name: 'PostponementTime',
			visibleTitle: FC.Environment.Localization.GlobalResources.PostponementTime
		}),
		new column({
			id: 6,
			name: 'QueueName',
			data: 'QueueName',
			visibleTitle: FC.Environment.Localization.GlobalResources.QueueColumn
		}),
		new column({
			id: 7,
			width: '64px',
			data: 'PriorityIconUrl',
			name: 'PriorityIconUrl',
			visibleTitle: FC.Environment.Localization.GlobalResources.PriorityColumn
		},
		function (data, type, full) {
			if (type !== "display") {
				return data;
			}

			return full.PriorityIconVisible ? "<img src=\"" + FC.Environment.AppPathNoTennant + data + "\"/>" : "";
		}),
	])

	/** Array of registration parameters columns **/
	self.ColumnsRegParam = ko.observableArray(	self.getRegParamColumns());

	/** Condition for reg parameter columns limit **/
	self.ColumnsRegParamLimit = ko.computed(function() {
		return self.ColumnsRegParam().length<3
	});

	/** Settings column **/
	self.SettingsColumn = {
		width: "20px",
		data: null,
		name: null,
		defaultContent: ''
	}

	/** Filtered array of columns for displaying in settings drop down **/
	self.FilterColumnsDefault = ko.computed(function() {
			return ko.utils.arrayFilter(self.ColumnsDefault(), function(column) {
				return !column.staticColumn;
			});
	});

	ko.computed( function() {
		ko.toJSON(self.ColumnsDefault);
		var filters = self.ColumnsDefault().map(function (column) {
			return {
				id: column.id,
				visible: column.visible
			}
		})
		localStorage.setItem('tasks__ColumnDefaultFilter', JSON.stringify(filters));
	});

	ko.computed( function() {
		ko.toJSON(self.ColumnsRegParam);
		var filters = self.ColumnsRegParam().map(function (column) {
			return {
				id: column.id,
				visible: column.visible,
				name: column.name
			}
		})
		localStorage.setItem('tasks__ColumnRegParamFilter', JSON.stringify(filters));
	});

	/** Show column settings drop down **/
	self.ShowColumnDropDown = function () {
		var settingPosition = $('#settings-icon').position()
		$('#column-filter').css({
			position: 'absolute',
			top: settingPosition.top - 30,
			left: settingPosition.left - $('#column-filter').width() + 7
		}).show();
	}

	/** Toggle column settings drop down **/
	self.ToggleColumnDropDown = function () {
		var settingPosition = $('#settings-icon').position()
		$('#column-filter').css({
			position: 'absolute',
			top: settingPosition.top - 30,
			left: settingPosition.left - $('#column-filter').width() + 7
		}).toggle();
		self.HideRegParamInput();
	}

	/** Hide column settings drop down **/
	self.HideColumnDropDown = function () {
		var settingPosition = $('#settings-icon').position()
		$('#column-filter').css({
			position: 'absolute',
			top: settingPosition.top,
			left: settingPosition.left
		}).hide();
		self.HideRegParamInput();
	}

	/** Handler for filter setting menu onScroll hide **/
	$("#tasks-scroll-container").on('scroll', self.HideColumnDropDown);

	/** Handler for filter setting menu onResize hide **/
	$(document).on('resize', self.HideColumnDropDown);

	/** Handler for filter setting menu outside click **/
	$(document).on('click', function (event) {
		var container = $("#column-filter");
		if (container.has(event.target).length === 0) {
			self.HideColumnDropDown();
		}
	});

	/** Value from add reg. parameter input **/
	self.RegParameterInput = ko.observable('');

	/** Add reg. parameter input visibility flag **/
	self.IsRegParamInputVisible = ko.observable(false);
	self.IsRegParamInputFocus = ko.observable(false);

	self.ShowRegParamInput = function () {
		self.IsRegParamInputVisible(true)
		self.RegParameterInput('');
		self.ShowColumnDropDown();
	}

	self.HideRegParamInput = function () {
		self.IsRegParamInputVisible(false)
		self.RegParameterInput('');
	}

	/** Prevent default for input**/
	$('#column-filter').keypress(function (event) {
		if (event.key == "Enter") {
			var elem = event.srcElement || event.target;
			self.RegParameterInput(elem.value);
			self.AddRegParamColumn();
			event.preventDefault()
		}
	});

	/** Add registration parameter to column list **/
	self.AddRegParamColumn = function () {
		var parameterName = self.RegParameterInput();
		if (!parameterName) {
			return
		}
		var regColumns = self.ColumnsRegParam()
		self.TableContainer().table.columns(self.ColumnsDefault().length + regColumns.length ).visible(true);
		self.ColumnsRegParam.push(
			new regParamColumn({
				id: regColumns.length>0 ?regColumns[regColumns.length-1].id + 1 : 0,
				name: parameterName
			}))
		self.HideColumnDropDown();
	};

	/** Add registration parameter to column list **/
	self.DeleteRegParamColumn = function (regColumn) {
		self.ColumnsRegParam( self.ColumnsRegParam().filter(function (column) {return column.id !== regColumn.id}));
	};


	self.ColumnsRegParam.subscribe(function () {
		var displayedColumns = self.ColumnsDefault().concat(self.ColumnsRegParam(),[self.SettingsColumn])

		self.InitTable(displayedColumns);
		self.ColumnsRegParam().forEach(function (column) {
			$('<th>' + column.name + '</th>').insertBefore("#settings-header");
		})
		self.TableContainer().reloadTable();

		self.ColumnsRegParam().forEach(function (column,index) {
			var newVisible = self.getColumnVisibility(column.id, true);
			if(self.TableContainer().table) {
				self.TableContainer().table.columns(self.ColumnsDefault().length + index).visible(newVisible);
			}
		})
	})

	self.IsRegParamInputVisible.subscribe(function (newValue) {
			self.IsRegParamInputFocus(newValue)
	})


	self.OnPageOpening = function () {
		var postponed = FC.Utils.getQueryParameter("postponed");
		self.postponedMode(!!(postponed && postponed === "true"));

		if (self.postponedMode()) {
			self.tasksTitle(FC.Environment.Localization.Tasks.PostponedTasksHeader);
		} else {
			self.tasksTitle(FC.Environment.Localization.Tasks.TasksHeader);
		}

		var displayedColumns = self.ColumnsDefault().concat([self.SettingsColumn])
		self.InitTable(displayedColumns);
		//Init reg. parameter columns from storage
		self.ColumnsRegParam(self.getRegParamColumns())

		//self.Refresh();
		self.pageNavigation.countPostponed();

		FC.DataContext.Role.hasAnyPermission(
			FC.DataContext.Role.permissions.BatchListBrowsing, 
			function (canSeeBatchList) { self.batchesPermitted(canSeeBatchList); }
		);
	};


	self.InitTable = function (displayedColumns) {
		var columnDefs = [{ className: "dt-left", targets: "_all" }];
		if (self.postponedMode()) {
			columnDefs.push({ visible: false, targets: [1] });
		} else {
			columnDefs.push({ visible: false, targets: [5, 6] });
		}

		self.TableContainer(new FC.DataTableContainer(
			{
				columns: displayedColumns,
				ajax: self.tableAjax,
				processing: false,
				order: [[0, 'asc']],
				columnDefs: columnDefs,
			},
			"tasks_table",
			"tasks_table_selectAll",
			FC.Environment.Localization.Tasks.NoTasks,
			null,
			self._tableDrawn));
	}
};
