﻿window.Layouts = window.Layouts || {};

window.Layouts.datacontext = (function () {

	var datacontext = {
		getCurrentUser: getCurrentUser,
		logout: logout,
		getLogoutPath: getLogoutPath,
		getHelpUrl: getHelpUrl,
		getLogoPath: getLogoPath,
		getLogoAlt: getLogoAlt,
		getAccSettingsPath: getAccSettingsPath,
		getSelectProjectPath: getSelectProjectPath,
		getCurrentProjectName: getCurrentProjectName,
		getProjectsInfo: getProjectsInfo,
		getProjectUrl: getProjectUrl,
		getCurrentBatchName: getCurrentBatchName,
		allProjectsUrl: allProjectsUrl,
		setProject: setProject,
		allBatchesUrl: allBatchesUrl
	};

	return datacontext;

	function setProject(project) {
		var viewModel = window.Layouts.ModelLoader.viewModel();
		if (viewModel && viewModel.hasOwnProperty('OnPageUnload')) {
			if (!viewModel.OnPageUnload(true)) {
				return;
			}
		}

		FC.DataContext.Project.setProject(
			project.Id(),
			function () {
				if (FC.Environment.StationType === FC.StationType.WebCapture) {
					FC.Utils.replaceLocation(FC.Utils.pageUrls.main());
				} else if (FC.Utils.getLocation().href.indexOf("QueueList") > -1) {
					FC.Utils.reloadLocation();
				} else {
					FC.Utils.replaceLocation(FC.Utils.pageUrls.queues());
				}
			});
	}

	function getCurrentUser(currentUserNameObservable, callback) {
		FC.DataContext.Server.getCurrentUserName(function (userName) {
			currentUserNameObservable(userName);

			if (callback) {
				callback();
			}
		});
	}

	function getCurrentProjectName(currentProjectNameObservable) {
		currentProjectNameObservable(FC.SessionInfo.ProjectName);
	}

	function getProjectsInfo(projectsObservable) {
		projectsObservable((ko.utils.arrayMap(FC.SessionInfo.Projects, function (item) {
			return new FC.BusinessModels.ProjectInformation(item.Id, item.UniqueId,
				item.Name, item.IsTestProject, item.Id == FC.SessionInfo.ProjectID);
		})));
	}

	function getCurrentBatchName(batchNameObservable) {
		batchNameObservable(FC.Utils.Serialize.htmlDecode(FC.Pages.VerifyPage.currentBatchName));
	}

	function getProjectUrl(project) {
		return project.Url;
	}

	// routes	
	function getLogoutPath() { return FC.Utils.getControllerUrl("Server/Logoff"); }
	function getHelpUrl() {
		switch (FC.Environment.StationType) {
			case FC.StationType.WebScanning: return FC.Environment.Localization.GlobalResources.HelpPath_Scanning;
			case FC.StationType.WebCapture: return FC.Environment.Localization.GlobalResources.HelpPath_WebCapture;
		}

		return FC.Environment.Localization.GlobalResources.HelpPath_Verification;
	}

	function logout() {
		return FC.Utils.replaceLocation(getLogoutPath() + "?sessionId=" + FC.SessionInfo.SessionId);
	}

	function getLogoPath() {
		if (FC.Environment.StationType === FC.StationType.WebCapture) {
			return FC.Environment.AppPathNoTennant + FC.Environment.Localization.NonLocalizedResources.CaptureStationLogoImagePath;
		}

		if (FC.Environment.StationType === FC.StationType.WebScanning) {
			return FC.Environment.AppPathNoTennant + FC.Environment.Localization.NonLocalizedResources.ScanningStationLogoImagePath;
		}

		return FC.Environment.AppPathNoTennant + FC.Environment.Localization.NonLocalizedResources.StationLogoImagePath;
	}
	function getLogoAlt() { return ''; }
	function getAccSettingsPath() {
		var tennant = FC.Environment.TennantName;
		if (FC.Environment.AppPath.toLowerCase().indexOf("winauth") > 0) {
			tennant = "WinAuth/" + tennant;
		}

		return FC.Environment.AppPathNoTennant + FC.Environment.Localization.NonLocalizedResources.AccSettingsUrl.replace("{Tennant}", tennant);
	}
	function getSelectProjectPath() { return FC.Utils.prepareRedirectUrl(FC.Environment.AppPath + FC.Environment.HomeUrl); }

	//FC.Environment.AppPath + "Project/GetAllProjects"; }

	function allProjectsUrl() { return FC.Utils.prepareRedirectUrl(FC.Environment.AppPath + FC.Environment.HomeUrl); }

	function allBatchesUrl() {
		if (FC.Environment.StationType === FC.StationType.WebCapture) {
			return FC.Utils.pageUrls.main();
		}
		if (window.Layouts.ModelLoader.viewModel().hasOwnProperty('canSeeBatchList') && window.Layouts.ModelLoader.viewModel().canSeeBatchList()) {
			return FC.Utils.pageUrls.batches();
		}

		return FC.Utils.pageUrls.queues();
	}

})();