﻿FC.Utils.DocumentEditor = {};
FC.Utils.DocumentEditor.jqueryElementsArrayEquals = function(left, right) {
	if (!left && right) return true;
	if (!left || !right) return false;
	if (left.length !== right.length) return false;
	for (var i = 0; i < left.length; i++) {
		if (!left[i].is(right[i])) return false;
	}

	return true;
}
FC.Utils.DocumentEditor.findBasicElements = function (element) {
	var elementParents = element.parents();
	var $parent, $parentIndex, isRepeatable = false;
	for ($parentIndex = 0; $parentIndex < elementParents.length; $parentIndex++) {
		$parent = elementParents.eq($parentIndex);
		var fieldType = $parent.attr('data-fieldType');
		isRepeatable = $parent.attr('data-isRepeatable');
		if (isRepeatable === "True") {
			isRepeatable = true;
			break;
		}
		if (fieldType === "subform") {
			break;
		}

		$parent = null;
	}
	if (!$parent) return [];

	var elements = [];
	if (isRepeatable) {
		var repeatableChildren = $parent.children();
		var repeatableElement = $parentIndex > 0 ? element.parents()[$parentIndex - 1] : element[0];
		var found = false;
		for (var i = 0; i < repeatableChildren.length; i++) {
			var elem = repeatableChildren.eq(i);
			if (elem.attr('type') == 'hidden') {
				if (found) break;
				elements = [];
			}
			if (elem[0] == repeatableElement) {
				found = true;
			}
			elements.push(elem);
		}
	} else {
		elements.push($parent);
	}
	return elements;
}

FC.Utils.DocumentEditor.findConnectedElementsToLabel = function (expanderContainer) {
	var ownerId = expanderContainer.attr('data-ownerId');

	var basicElements = FC.Utils.DocumentEditor.findBasicElements(expanderContainer);
	if (!basicElements) return;
	var res = [];
	$.each(basicElements,
		function (index, elem) {
			elem.find('*[data-fieldid="' + ownerId + '"], *[data-repeatablefieldid="' + ownerId + '"]').each(function () {
				var currentBasicElements = FC.Utils.DocumentEditor.findBasicElements($(this));
				if (FC.Utils.DocumentEditor.jqueryElementsArrayEquals(currentBasicElements,basicElements))
					res.push($(this));
			});
		});
	return res;
}

FC.Utils.DocumentEditor.findConnectedElementsToField = function (expanderContainer) {
	var fieldId = expanderContainer.attr("data-fieldid") || expanderContainer.attr("data-repeatablefieldid");

	var fieldType = expanderContainer.attr("data-fieldtype");
	var subformId = expanderContainer.attr("data-subformid");
	if (fieldType === "subform" && !subformId) {
		// секция
		var expanderParent = $("[data-sectionindex='" + fieldId + "'");
		if (expanderParent) {
			return [expanderParent];
		}
	}

	var basicElements = FC.Utils.DocumentEditor.findBasicElements(expanderContainer);
	if (!basicElements) return;
	var res = [];
	$.each(basicElements,
		function (index, elem) {
			elem.find('.groupExpanderContainer[data-ownerid="' + fieldId + '"]').each(function () {
				var currentBasicElements = FC.Utils.DocumentEditor.findBasicElements($(this));
				if (FC.Utils.DocumentEditor.jqueryElementsArrayEquals(currentBasicElements, basicElements))
					res.push($(this));
			});
		});
	return res;
}

FC.Utils.DocumentEditor.checkFieldVisible = function (field, parentViewmodel) {
	if (!field.isVisible()) {
		var fieldId = field.id();
		if (!fieldId || fieldId.length === 0)
			return null;

		if (parentViewmodel) {
			var basicElement = $('[data-jsDomId="' + parentViewmodel.UniqueIdentifier + '"]');
				
			basicElement.find('*[data-ownerid="' + fieldId + '"]').each(function() {
				var currentBasicElements = FC.Utils.DocumentEditor.findBasicElements($(this));
				if (currentBasicElements.length ===1 && currentBasicElements[0].is(basicElement))
					$(this).remove();
			});
		}
		if (field instanceof FC.Viewmodels.DocumentEditor.SectionAdapter) {
			fieldContainer.prev().remove();
		}
		fieldContainer.remove();
	}

	if (field.children && field.children()) {
		$.each(field.children(), function (index, val) { if (val) FC.Utils.DocumentEditor.checkFieldVisible(val); });
	}
}



FC.Utils.DocumentEditor.GetCustomFormKey = function (document) {
	return "customWebForm_template" +
		'-' +
		document.projectId() +
		'-' +
		document.templateId() +
		'-' +
		document.templateVersion();
}