﻿(function () {
	window.UndoEngine = {};
	var engine = window.UndoEngine;	

	engine.UndoDepth = 100;
	engine.RedoDepth = 1;

	engine.UndoStack = [];
	engine.RedoStack = [];

	engine.Undoable = function (updateFunc, getCurrentValue, currentValue) {
		var self = this;
		self.UpdateFunc = updateFunc;
		self.GetCurrentValue = getCurrentValue;
		self.CurrentValue = currentValue;
	};

	engine.MakeSnapshotKO = function (observables) {
		if (!observables || observables.length === 0) {
			return;
		}

		var ubdoables = [];
		for (var i = 0; i < observables.length; i++) {
			ubdoables.push(new engine.Undoable(observables[i], observables[i], observables[i]()));
		}

		engine.MakeSnapshot(ubdoables);
	};

	engine.MakeSnapshot = function (undoables) {
		if (!undoables || undoables.length === 0) {
			return;
		}

		if (engine.UndoStack.length > engine.UndoDepth) {
			engine.UndoStack.shift();
		}

		engine.UndoStack.push(undoables);
	};

	engine.Undo = function () {
		engine.DoUndo(engine.UndoStack, engine.RedoStack, engine.RedoDepth);
	};

	engine.DoUndo = function (stackToRemove, stackToPush, stackToPushLimit) {
		var currentUndoables = stackToRemove.pop();
		var undoed = false;

		if (currentUndoables && currentUndoables.length > 0) {
			var redoUndoables = [];
			for (var i = 0; i < currentUndoables.length; i++) {
				var currentUndoable = currentUndoables[i];

				if (!currentUndoable ||
				    !currentUndoable.hasOwnProperty('GetCurrentValue') ||
					!currentUndoable.hasOwnProperty('CurrentValue') ||
					!currentUndoable.hasOwnProperty('UpdateFunc')) {
					continue;
				}

				undoed = true;
				var redoValue = currentUndoable.GetCurrentValue();
				currentUndoable.UpdateFunc(currentUndoable.CurrentValue);
				redoUndoables.push(new engine.Undoable(currentUndoable.UpdateFunc, currentUndoable.GetCurrentValue, redoValue));
			}

			if (undoed) {
				if (stackToPush.length >= stackToPushLimit) {
					stackToPush.shift();
				}

				stackToPush.push(redoUndoables);
			}
		}
	};

	engine.Redo = function () {
		engine.DoUndo(engine.RedoStack, engine.UndoStack, engine.UndoDepth);
	};

	engine.CleanStacks = function () {
		engine.UndoStack = [];
		engine.RedoStack = [];
	};
})();
