﻿/**
 * Утилиты для обсчета полигонов
 */
FC.Utils.polygonCommon = (function () {
	/**
	 * Сравнить два observable (или нет) значения
	 * @param {any} v1
	 * @param {any} v2
	 */
	var koEqual = function (v1, v2) {
		return ko.utils.unwrapObservable(v1) === ko.utils.unwrapObservable(v2);
	};

	/**
	 * Вычесть одно observable (или нет) значение из другого
	 * @param {any} v1
	 * @param {any} v2
	 */
	var koDiff = function (v1, v2) {
		return ko.utils.unwrapObservable(v1) - ko.utils.unwrapObservable(v2);
	};

	/**
	 * Пересекаются ли прямоугольники
	 * @param {any} rect1
	 * @param {any} rect2
	 */
	var rectanglesIntersect = function (rect1, rect2) {
		return rect1.x1 < rect2.x2 && rect1.x2 > rect2.x1 &&
			rect1.y1 < rect2.y2 && rect1.y2 > rect2.y1;
	};

	/**
	 * Пересчитать два пересекающихся прямоугольника в массив не пересекающихся
	 * @param {any} rect1
	 * @param {any} rect2
	 */
	var recalculateIntersectingRectangles = function (rect1, rect2) {
		var leftPoints = [
			{ x: rect1.x1, y: rect1.y1 }, { x: rect1.x1, y: rect1.y2 }, { x: rect2.x1, y: rect2.y1 },
			{ x: rect2.x1, y: rect2.y2 }
		];
		var rightPoints = [
			{ x: rect1.x2, y: rect1.y1 }, { x: rect1.x2, y: rect1.y2 }, { x: rect2.x2, y: rect2.y1 },
			{ x: rect2.x2, y: rect2.y2 }
		];
		var pointsCompare = function (p1, p2) {
			var yDiff = p1.y - p2.y;
			if (yDiff !== 0) {
				return yDiff;
			}

			return p1.x - p2.x;
		};

		leftPoints.sort(pointsCompare);
		rightPoints.sort(pointsCompare);

		if (leftPoints[1].x > leftPoints[0].x) {
			leftPoints[1].x = leftPoints[0].x;
		}

		if (leftPoints[2].x > leftPoints[1].x) {
			leftPoints[2].x = leftPoints[1].x;
		}

		if (rightPoints[1].x < rightPoints[0].x) {
			rightPoints[1].x = rightPoints[0].x;
		}

		if (rightPoints[2].x < rightPoints[1].x) {
			rightPoints[2].x = rightPoints[1].x;
		}

		var rects = [];

		if (leftPoints[0].x === leftPoints[1].x && rightPoints[0].x === rightPoints[2].x) {
			if (leftPoints[0].y !== rightPoints[2].y) {
				rects.push({ x1: leftPoints[0].x, y1: leftPoints[0].y, x2: rightPoints[0].x, y2: rightPoints[2].y });
			}

			if (leftPoints[2].y !== rightPoints[3].y) {
				rects.push({ x1: leftPoints[3].x, y1: leftPoints[2].y, x2: rightPoints[3].x, y2: rightPoints[3].y });
			}
		} else {
			if (leftPoints[0].y !== rightPoints[1].y) {
				rects.push({ x1: leftPoints[0].x, y1: leftPoints[0].y, x2: rightPoints[0].x, y2: rightPoints[1].y });
			}

			if (leftPoints[1].x === leftPoints[3].x && rightPoints[2].x === rightPoints[3].x) {
				if (leftPoints[1].y !== rightPoints[3].y) {
					rects.push({ x1: leftPoints[1].x, y1: leftPoints[1].y, x2: rightPoints[3].x, y2: rightPoints[3].y });
				}
			} else {
				if (leftPoints[1].y !== rightPoints[2].y) {
					rects.push({ x1: leftPoints[1].x, y1: leftPoints[1].y, x2: rightPoints[2].x, y2: rightPoints[2].y });
				}
				if (leftPoints[2].y !== rightPoints[3].y) {
					rects.push({ x1: leftPoints[3].x, y1: leftPoints[2].y, x2: rightPoints[3].x, y2: rightPoints[3].y });
				}
			}
		}

		return rects;
	};

	/**
	 * Убрать пересечения из массива прямоугольников
	 * @param {any} rects
	 */
	var removeIntersections = function (rects) {
		var resultRects = rects.slice(0);

		// обработаем пересекающиеся прямоугольники
		var i = 0;
		while (i < resultRects.length) {

			var isIntersected = false;
			for (var j = i + 1; j < resultRects.length; j++) {
				if (rectanglesIntersect(resultRects[i], resultRects[j])) {
					var recalculated = recalculateIntersectingRectangles(resultRects[i], resultRects[j]);
					resultRects.splice(j, 1);
					resultRects.splice(i, 1);
					while (recalculated.length > 0) {
						var newRect = recalculated.splice(recalculated.length - 1, 1);
						resultRects.splice(i, 0, newRect[0]);
					}

					isIntersected = true;
					break;
				}
			}

			if (!isIntersected) {
				i++;
			}
		}

		return resultRects;
	}

	/**
	 * Получить из массива прямоугольников "строки" - т.е. группы прямоугольников, находящиеся на одном вертикальном уровне
	 * @param {any} rectangles
	 */
	var getIntersectingGroups = function (rectangles) {
		if (!rectangles) {
			return null;
		}

		if (rectangles.length < 2) {
			return [rectangles];
		}

		rectangles.sort(function (r1, r2) { return r1.y1 - r2.y1; });

		var intersectingGroups = [];
		var currentGroup = [rectangles[0]];
		var currentGroupMax = currentGroup[0].y2;

		var i;
		for (i = 1; i < rectangles.length; i++) {
			if (rectangles[i].y1 >= currentGroupMax) {
				intersectingGroups.push(currentGroup);
				currentGroup = [];
				currentGroupMax = 0;
			}

			if (rectangles[i].y2 > currentGroupMax) {
				currentGroupMax = rectangles[i].y2;
			}
			currentGroup.push(rectangles[i]);
		}

		intersectingGroups.push(currentGroup);
		return intersectingGroups;
	};

	/**
	 * удалить вертикальные вмятины
	 * @param {any} intersectingGroups
	 */
	var removeVerticalDents = function (intersectingGroups) {
		for (var i = 0; i < intersectingGroups.length; i++) {
			var currentGroup = intersectingGroups[i];

			if (currentGroup.length < 2) {
				continue;
			}

			currentGroup.sort(function (r1, r2) { return r1.x1 - r2.x1; });
			for (var j = 1; j < currentGroup.length; j++) {
				var k = j - 1;
				var supposedRects = [{ x2: currentGroup[j].x1, y1: currentGroup[j].y1, y2: currentGroup[j].y2, source: currentGroup[j] }];
				var y1 = currentGroup[j].y1;
				var y2 = currentGroup[j].y2;
				while (k >= 0) {
					if (y1 < currentGroup[k].y2 && y2 > currentGroup[k].y1) {

						for (var m = 0; m < supposedRects.length; m++) {
							if (currentGroup[k].y2 > supposedRects[m].y1 && currentGroup[k].y1 < supposedRects[m].y2) {

								var currentSupposed = supposedRects.splice(m, 1)[0];
								m--;

								if (currentGroup[k].y1 > currentSupposed.y1) {
									supposedRects.push({ x2: currentSupposed.x2, y1: currentSupposed.y1, y2: currentGroup[k].y1, source: currentSupposed.source });
								}

								if (currentGroup[k].y2 < currentSupposed.y2) {
									supposedRects.push({ x2: currentSupposed.x2, y1: currentGroup[k].y2, y2: currentSupposed.y2, source: currentSupposed.source });
								}

								if (currentGroup[k].x2 < currentSupposed.x2) {
									var newY1 = Math.max(currentSupposed.y1, currentGroup[k].y1);
									var newY2 = Math.min(currentSupposed.y2, currentGroup[k].y2);

									if (newY1 !== newY2) {
										if (newY1 === currentGroup[k].y1 && newY2 === currentGroup[k].y2) {
											currentGroup[k].x2 = currentSupposed.x2;
										} else if (newY1 === currentSupposed.source.y1 && newY2 === currentSupposed.source.y2) {
											currentSupposed.source.x1 = currentGroup[k].x2;
										} else {
											currentGroup.splice(k + 1,
												0,
												{
													x1: currentGroup[k].x2,
													x2: currentSupposed.x2,
													y1: newY1,
													y2: newY2
												}
											);
										}
									}
								}
							}
						}
					}
					k--;
				}
			}
		}

		return intersectingGroups;
	};

	/**
	 * Соединить группы прямоугольников тонкими полосками (однопиксельными)
	 * @param {any} intersectingGroups
	 */
	var joinIntersectingGroups = function (intersectingGroups) {
		var result = [];
		var lowestRect = null;
		var previousLowestRect = null;
		var highestRect = null;

		var cloneRect = function (rect) {
			return {
				x1: rect.x1,
				x2: rect.x2,
				y1: rect.y1,
				y2: rect.y2
			};
		};

		for (var i = 0; i < intersectingGroups.length; i++) {
			var currentGroup = intersectingGroups[i];

			for (var j = 0; j < currentGroup.length; j++) {
				if (lowestRect && currentGroup[j].y2 === lowestRect.y2) {
					if (currentGroup[j].x1 < lowestRect.x1) {
						lowestRect.x1 = currentGroup[j].x1;
					}
					if (currentGroup[j].x2 > lowestRect.x2) {
						lowestRect.x2 = currentGroup[j].x2;
					}
				}

				if (!lowestRect || currentGroup[j].y2 > lowestRect.y2) {
					lowestRect = cloneRect(currentGroup[j]);
				}

				if (highestRect && currentGroup[j].y1 === highestRect.y1) {
					if (currentGroup[j].x1 < highestRect.x1) {
						highestRect.x1 = currentGroup[j].x1;
					}
					if (currentGroup[j].x2 > highestRect.x2) {
						highestRect.x2 = currentGroup[j].x2;
					}
				}


				if (previousLowestRect && (!highestRect || currentGroup[j].y1 < highestRect.y1)) {
					highestRect = cloneRect(currentGroup[j]);
				}

				result.push(currentGroup[j]);
			}

			// соединим прямоугольники
			if (highestRect && previousLowestRect) {
				// пересекаются по горизонтали
				if (previousLowestRect.x1 < highestRect.x2 && previousLowestRect.x2 > highestRect.x1) {
					if (previousLowestRect.y2 < highestRect.y1) {
						var x = Math.max(previousLowestRect.x1, highestRect.x1);
						result.push({ y1: previousLowestRect.y2, y2: highestRect.y1, x1: x, x2: x + 1 });
					}
				} else {
					var x1;
					var x2;
					if (previousLowestRect.x1 >= highestRect.x2) {
						x1 = highestRect.x2;;
						x2 = previousLowestRect.x1;
						if (x1 !== x2) {
							if (previousLowestRect.y2 < highestRect.y1 - 1) {
								result.push({ y1: previousLowestRect.y2, y2: highestRect.y1 - 1, x1: x2, x2: x2 + 1 });
							}
							result.push({ y1: highestRect.y1 - 1, y2: highestRect.y1, x1: x1 - 1, x2: x2 + 1 });
						} else {
							result.push({ y1: previousLowestRect.y2, y2: highestRect.y1 + 1, x1: x1, x2: x2 + 1 });
						}
					} else {
						x1 = previousLowestRect.x2;
						x2 = highestRect.x1;

						if (x1 !== x2) {
							if (previousLowestRect.y2 < highestRect.y1 - 1) {
								result.push({ y1: previousLowestRect.y2, y2: highestRect.y1 - 1, x1: x1 - 1, x2: x1 });
							}
							result.push({ y1: highestRect.y1 - 1, y2: highestRect.y1, x1: x1 - 1, x2: x2 + 1 });
						} else {
							result.push({ y1: previousLowestRect.y2, y2: highestRect.y1 + 1, x1: x1 - 1, x2: x2 });
						}
					}
				}
			}

			previousLowestRect = lowestRect;
			lowestRect = null;
			highestRect = null;
		}

		return result;
	};

	/**
	 * Подготовить прямоугольники
	 * (убрать прямоугольники с нулевой шириной или высотой, поправить, чтобы x1 был < x2 и аналогично y)
	 * @param {any} rects
	 */
	var prepareRects = function (rects) {
		for (var i = 0; i < rects.length; i++) {
			if (rects[i].x1 === rects[i].x2 || rects[i].y1 === rects[i].y2) {
				rects.splice(i, 1);
				i--;
				continue;
			}

			if (rects[i].x1 > rects[i].x2) {
				var x = rects[i].x1;
				rects[i].x1 = rects[i].x2;
				rects[i].x2 = x;
			}

			if (rects[i].y1 > rects[i].y2) {
				var y = rects[i].y1;
				rects[i].y1 = rects[i].y2;
				rects[i].y2 = y;
			}
		}
	};

	/**
	 * Сделать из массива прямоугольников полигон
	 * @param {any} rects
	 */
	var createPolygonFromRects = function (rects) {
		if (!rects || rects.length === 0) {
			return [];
		}

		prepareRects(rects);

		var resultRects = removeIntersections(rects);

		if (resultRects.length < 2) {
			return resultRects;
		}

		// найдём группы прямоугольников на одном уровне по вертикали
		var intersectingGroups = getIntersectingGroups(resultRects);

		// объединим прямоугольники, находящиеся на одном уровне по вертикали
		intersectingGroups = removeVerticalDents(intersectingGroups);

		var result = joinIntersectingGroups(intersectingGroups);

		return result;
	};

	var Edge = function (x1, y1, x2, y2, source, type, isHorizontal) {
		this.x1 = x1;
		this.x2 = x2;
		this.y1 = y1;
		this.y2 = y2;
		this.isInternal = false;
		this.firstIsInternal = false;
		this.secondIsInternal = false;
		this.type = type;
		this.isHorizontal = isHorizontal;
		this.source = source;
	};

	/**
	 * Получить горизонтальные рёбра из массива прямоугольников
	 * @param {any} rectangles
	 */
	var getHorizontalEdges = function (rectangles) {
		var edgesHorizontal = [];
		var i;
		for (i = 0; i < rectangles.length; i++) {
			if (koEqual(rectangles[i].x1, rectangles[i].x2) || koEqual(rectangles[i].y1, rectangles[i].y2)) {
				continue;
			}

			var x1 = ko.utils.unwrapObservable(rectangles[i].x1) < ko.utils.unwrapObservable(rectangles[i].x2) ? rectangles[i].x1 : rectangles[i].x2;
			var y1 = ko.utils.unwrapObservable(rectangles[i].y1) < ko.utils.unwrapObservable(rectangles[i].y2) ? rectangles[i].y1 : rectangles[i].y2;
			var x2 = ko.utils.unwrapObservable(rectangles[i].x1) < ko.utils.unwrapObservable(rectangles[i].x2) ? rectangles[i].x2 : rectangles[i].x1;
			var y2 = ko.utils.unwrapObservable(rectangles[i].y1) < ko.utils.unwrapObservable(rectangles[i].y2) ? rectangles[i].y2 : rectangles[i].y1;

			edgesHorizontal.push(new Edge(x1, y1, x2, y1, rectangles[i], 1, true));
			edgesHorizontal.push(new Edge(x1, y2, x2, y2, rectangles[i], 2, true));
		}

		edgesHorizontal.sort(function (e1, e2) {
			var yDiff = koDiff(e1.y1, e2.y1);
			if (yDiff !== 0) {
				return yDiff;
			}

			return koDiff(e1.x1, e2.x1);
		});

		return edgesHorizontal;
	};

	/**
	 * Получить вертикальные рёбра из массива прямоугольников
	 * @param {any} rectangles
	 */
	var getVerticalEdges = function (rectangles) {
		var edgesVertical = [];
		var i;
		for (i = 0; i < rectangles.length; i++) {
			if (koEqual(rectangles[i].x1, rectangles[i].x2) || koEqual(rectangles[i].y1, rectangles[i].y2)) {
				continue;
			}

			var x1 = ko.utils.unwrapObservable(rectangles[i].x1) < ko.utils.unwrapObservable(rectangles[i].x2) ? rectangles[i].x1 : rectangles[i].x2;
			var y1 = ko.utils.unwrapObservable(rectangles[i].y1) < ko.utils.unwrapObservable(rectangles[i].y2) ? rectangles[i].y1 : rectangles[i].y2;
			var x2 = ko.utils.unwrapObservable(rectangles[i].x1) < ko.utils.unwrapObservable(rectangles[i].x2) ? rectangles[i].x2 : rectangles[i].x1;
			var y2 = ko.utils.unwrapObservable(rectangles[i].y1) < ko.utils.unwrapObservable(rectangles[i].y2) ? rectangles[i].y2 : rectangles[i].y1;

			edgesVertical.push(new Edge(x2, y1, x2, y2, rectangles[i], 2, false));
			edgesVertical.push(new Edge(x1, y1, x1, y2, rectangles[i], 1, false));
		}

		edgesVertical.sort(function (e1, e2) {
			var xDiff = koDiff(e1.x1, e2.x1);
			if (xDiff !== 0) {
				return xDiff;
			}

			return koDiff(e1.y1, e2.y1);
		});

		return edgesVertical;
	};

	var markEdgePointInternal = function (edgeDef) {
		if (edgeDef.pointType === 1) {
			edgeDef.edge.firstIsInternal = true;
		} else {
			edgeDef.edge.secondIsInternal = true;
		}
	};

	// отметить внутренние точки
	var markInternalPoints = function (edgesVertical, edgesHorizontal) {
		var pointsDictionary = {};
		var createKey = function (x, y) {
			return ko.utils.unwrapObservable(x) + "_" + ko.utils.unwrapObservable(y);
		};

		var putPointToDictionary = function (key, edge, pointType) {
			if (!pointsDictionary[key]) {
				pointsDictionary[key] = [{ edge: edge, pointType: pointType }];
			} else {
				pointsDictionary[key].push({ edge: edge, pointType: pointType });
			}
		};

		var putEdgeToDictionary = function (edge) {
			var key1 = createKey(edge.x1, edge.y1);
			putPointToDictionary(key1, edge, 1);

			var key2 = createKey(edge.x2, edge.y2);
			putPointToDictionary(key2, edge, 2);
		};

		edgesVertical.forEach(function (edge) { putEdgeToDictionary(edge); });
		edgesHorizontal.forEach(function (edge) { putEdgeToDictionary(edge); });

		var keys = Object.getOwnPropertyNames(pointsDictionary);
		for (var i = 0; i < keys.length; i++) {
			var edges = pointsDictionary[keys[i]];
			if (edges.length < 2) {
				continue;
			}

			// подсчитаем, какие точки прямоугольников сходятся в точке keys[i]
			// rectPointCount[0] - верхние левые точки
			// rectPointCount[1] - верхние правые точки
			// rectPointCount[2] - нижние левые точки
			// rectPointCount[3] - нижние правые точки
			var rectPointCount = [0, 0, 0, 0];
			var j;
			 
			edges.forEach(function (edge) {
				if (edge.pointType === 1) {
					if (edge.edge.type === 1) {
						rectPointCount[0]++;
					} else {
						if (edge.edge.isHorizontal) {
							rectPointCount[2]++;
						} else {
							rectPointCount[1]++;
						}
					}
				} else {
					if (edge.edge.type === 2) {
						rectPointCount[3]++;
					} else {
						if (edge.edge.isHorizontal) {
							rectPointCount[1]++;
						} else {
							rectPointCount[2]++;
						}
					}
				}
			});

			var pointsTypes = rectPointCount.reduce(function(pt, current) { return pt + (current ? 1 : 0); }, 0);

			switch (pointsTypes) {
				case 2:
					edges.forEach(markEdgePointInternal);

					// некоторые грани внутри фигуры
					if (rectPointCount[0] && rectPointCount[1] ||
						rectPointCount[2] && rectPointCount[3]) {
						// вертикальные грани - внутренние
						edges.filter(function (e) { return !e.edge.isHorizontal; })
							.forEach(function (e) { e.edge.isInternal = true; });
					} else {
						// горизонтальные грани - внутренние
						edges.filter(function (e) { return e.edge.isHorizontal; })
							.forEach(function (e) { e.edge.isInternal = true; });
					}
					break;
				case 3:
					var filterKeys = function (e, isHorizontalKey, isVerticalKey) {
						var key1 = createKey(e.edge.x1, e.edge.y1);
						var key2 = createKey(e.edge.x2, e.edge.y2);
						if (e.edge.isHorizontal && isHorizontalKey(key1, key2)) {
							return true;
						}

						if (!e.edge.isHorizontal && isVerticalKey(key1, key2)) {
							return true;
						}

						return false;
					};
					var filterEdges;

					// некоторые грани внутри фигуры
					if (rectPointCount[0]) {
						if (rectPointCount[1]) {
							if (rectPointCount[2]) {
								// прямоугольники расположены на NE, SW, SE от точки
								filterEdges = function (e) {
									return filterKeys(e,
										function (key1) { return key1 === keys[i] },
										function (key1) { return key1 === keys[i] });
								};
							} else {
								// прямоугольники расположены на NW, NE, SE от точки
								filterEdges = function (e) {
									return filterKeys(e,
										function (key1, key2) { return key2 === keys[i] },
										function (key1) { return key1 === keys[i] });
								};
							}
						} else {
							// прямоугольники расположены на NW, SW, SE от точки
							filterEdges = function (e) {
								return filterKeys(e,
									function (key1) { return key1 === keys[i] },
									function (key1, key2) { return key2 === keys[i] });
							};
						}
					} else {
						// прямоугольники расположены на NW, NE, SW от точки
						filterEdges = function (e) {
							return filterKeys(e,
								function (key1, key2) { return key2 === keys[i] },
								function (key1, key2) { return key2 === keys[i] });
						};
					}

					edges.filter(filterEdges).forEach(function (e) { e.edge.isInternal = true; });

					break;
				case 4:
					// все грани внутри фигуры
					edges.forEach(function (e) {
						markEdgePointInternal(e);
						e.edge.isInternal = true;
					});
					break;
			}
		}
	};

	/**
	 * Получить точку из ребра
	 * @param {any} edge ребро
	 * @param {any} pointType первая или вторая точка
	 */
	var getPointFromEdge = function (edge, pointType) {
		if (pointType === 2) {
			return { x: edge.x2, y: edge.y2, pointType: pointType, isInternal: edge.secondIsInternal };
		}

		return { x: edge.x1, y: edge.y1, pointType: pointType, isInternal: edge.firstIsInternal };
	};

	var createLimitedPoint = function (point, currentMode, influencedValues) {
		return new FC.Viewmodels.ImageEditor.LimitedPoint(
			point.x,
			point.y,
			null,
			null,
			null,
			null,
			influencedValues,
			currentMode);
	};

	var getNearEdges = function (edges, getValue1, getValue2, pointValue) {
		edges.sort(function (e1, e2) { return koDiff(getValue1(e1), getValue1(e2)); });

		var vMax = -1000000;
		var isCurrentGroup = false;
		var currentEdges = [];
		var i;
		var currentPointIndex = -1;
		var currentGroupHasInternal = false;
		var currentGroupHasExternal = false;
		var val1;
		var val2;

		for (i = 0; i < edges.length; i++) {
			var edge = edges[i];

			val1 = getValue1(edge);
			val2 = getValue2(edge);

			if (val1 > vMax) {

				if (isCurrentGroup) {
					break;
				}

				currentEdges = [];
				currentGroupHasInternal = false;
				currentGroupHasExternal = false;
				vMax = val2;
			}

			currentEdges.push(edge);

			if (edge.isInternal) {
				currentGroupHasInternal = true;
			} else {
				currentGroupHasExternal = true;
			}

			if (!isCurrentGroup && (val1 === pointValue || val2 === pointValue)) {
				isCurrentGroup = true;
				currentPointIndex = currentEdges.length - 1;
			}

			if (val2 > vMax) {
				vMax = val2;
			}
		}

		if (currentPointIndex > -1 && currentGroupHasInternal && currentGroupHasExternal && currentEdges.length > 2) {
			var edgeIsInternal;
			var previousVal1;
			var previousVal2;

			// Если перед текущим елементом что-то есть, то мб нужно отрезать лишнее
			if (currentPointIndex > 0) {
				edgeIsInternal = currentEdges[currentPointIndex].isInternal;
				previousVal1 = getValue1(currentEdges[currentPointIndex]);
				previousVal2 = getValue2(currentEdges[currentPointIndex]);

				i = currentPointIndex - 1;
				while (i >= 0) {
					val1 = getValue1(currentEdges[i]);
					val2 = getValue2(currentEdges[i]);

					if (edgeIsInternal && previousVal1 !== val1 && previousVal2 !== val2 && val2 !== pointValue) {
						currentEdges.splice(0, i + 1);
						break;
					}

					edgeIsInternal = currentEdges[i].isInternal;
					previousVal1 = val1;
					previousVal2 = val2;
					i--;
				}
			}

			// Если после текущего елемента что-то есть, то мб нужно отрезать лишнее
			if (currentEdges.length - currentPointIndex > 1) {
				edgeIsInternal = currentEdges[currentPointIndex].isInternal;
				previousVal1 = getValue1(currentEdges[currentPointIndex]);
				previousVal2 = getValue2(currentEdges[currentPointIndex]);

				i = currentPointIndex + 1;
				while (i < currentEdges.length) {
					val1 = getValue1(currentEdges[i]);
					val2 = getValue2(currentEdges[i]);

					if (edgeIsInternal && previousVal1 !== val1 && previousVal2 !== val2 && val1 !== pointValue) {
						currentEdges.splice(i);
						break;
					}

					edgeIsInternal = currentEdges[i].isInternal;
					previousVal1 = val1;
					previousVal2 = val2;
					i++;
				}
			}
		}

		return currentEdges;
	};

	var getLimitedPointFromEdge = function (edge, pointType, currentMode, horizontalEdges, verticalEdges, page) {
		var zero = ko.observable(0);
		var pageWidth = page ? page.width : zero;
		var pageHeight = page ? page.height : zero;

		var p = getPointFromEdge(edge, pointType);
		var influenced = [];

		// добавляем саму точку в influenced
		influenced.push(new FC.Viewmodels.DocumentEditor.LimitedValue(p.x, zero, pageWidth, true, edge.source));
		influenced.push(new FC.Viewmodels.DocumentEditor.LimitedValue(p.y, zero, pageHeight, false, edge.source));

		influenced[0].isMainCoordinate = true;
		influenced[1].isMainCoordinate = true;

		var result = createLimitedPoint(p, currentMode, influenced);

		// найдём внешние грани на том же уровне
		var sameLevelHorizontals = horizontalEdges[ko.utils.unwrapObservable(p.y)];
		var sameLevelVerticals = verticalEdges[ko.utils.unwrapObservable(p.x)];

		var i;
		var nearEdges;

		// добавим точки на том же горизонтальном уровне, которые касаются текущей
		if (sameLevelHorizontals.length > 1) {
			nearEdges = getNearEdges(sameLevelHorizontals,
				function (e) { return ko.utils.unwrapObservable(e.x1); },
				function (e) { return ko.utils.unwrapObservable(e.x2); },
				ko.utils.unwrapObservable(p.x));

			for (i = 0; i < nearEdges.length; i++) {

				if (nearEdges[i].y1 !== p.y || nearEdges[i].source !== edge.source) {
					influenced.push(new FC.Viewmodels.DocumentEditor.LimitedValue(nearEdges[i].y1, zero, pageHeight, false, nearEdges[i].source));
				}
			}
		}

		// добавим точки на том же вертикальном уровне, которые касаются текущей
		if (sameLevelVerticals.length > 1) {
			nearEdges = getNearEdges(sameLevelVerticals,
				function (e) { return ko.utils.unwrapObservable(e.y1); },
				function (e) { return ko.utils.unwrapObservable(e.y2); },
				ko.utils.unwrapObservable(p.y));

			for (i = 0; i < nearEdges.length; i++) {
				if (nearEdges[i].x1 !== p.x || nearEdges[i].source !== edge.source) {
					influenced.push(new FC.Viewmodels.DocumentEditor.LimitedValue(nearEdges[i].x1, zero, pageWidth, true, nearEdges[i].source));
				}
			}
		}


		return result;
	};

	var findNextExternalPoint = function (edges, point, isHorizontal, currentMode, horizontalEdges, verticalEdges, page) {
		var edgesOnLevel = isHorizontal ? horizontalEdges[ko.utils.unwrapObservable(point.y)] : verticalEdges[ko.utils.unwrapObservable(point.x)];

		if (!edgesOnLevel || edgesOnLevel.length === 0) {
			console.log("no edges on level!");
			return null;
		}

		if (edgesOnLevel.length === 1) {
			if (isHorizontal && koEqual(edgesOnLevel[0].x1, point.x) || !isHorizontal && koEqual(edgesOnLevel[0].y1, point.y)) {
				return getLimitedPointFromEdge(edgesOnLevel[0], 2, currentMode, horizontalEdges, verticalEdges, page);
			}

			return getLimitedPointFromEdge(edgesOnLevel[0], 1, currentMode, horizontalEdges, verticalEdges, page);
		}

		var externalPoints = [];
		var pointsDictionary = {};
		var getDictionaryKey = function(exPoint) {
			return ko.utils.unwrapObservable(exPoint.x) + "_" +  ko.utils.unwrapObservable(exPoint.y);
		};

		for (var i = 0; i < edgesOnLevel.length; i++) {
			var p;
			var key;
			if (!edgesOnLevel[i].firstIsInternal) {
				p = getLimitedPointFromEdge(edgesOnLevel[i], 1, currentMode, horizontalEdges, verticalEdges, page);
				key = getDictionaryKey(p);
				if (!pointsDictionary[key]) {
					externalPoints.push(p);
					pointsDictionary[key] = p;
				}
			}

			if (!edgesOnLevel[i].secondIsInternal) {
				p = getLimitedPointFromEdge(edgesOnLevel[i], 2, currentMode, horizontalEdges, verticalEdges, page);
				key = getDictionaryKey(p);
				if (!pointsDictionary[key]) {
					externalPoints.push(p);
					pointsDictionary[key] = p;
				}
			}
		}

		if (externalPoints.length < 3) {
			if (koEqual(externalPoints[0].x, point.x) && koEqual(externalPoints[0].y, point.y)) {
				return externalPoints[1];
			}

			return externalPoints[0];
		}

		externalPoints.sort(function (p1, p2) {
			if (isHorizontal) {
				return koDiff(p1.x, p2.x);
			}

			return koDiff(p1.y, p2.y);
		});

		var currentKey = getDictionaryKey(point);
		var currantPoint = pointsDictionary[currentKey];
		var currentIndex = externalPoints.indexOf(currantPoint);

		if (currentMode) {
			if (currentIndex === externalPoints.length - 1) {
				return externalPoints[currentIndex - 1];
			}

			return externalPoints[currentIndex + 1];
		}

		if (currentIndex === 0) {
			return externalPoints[1];
		}

		return externalPoints[currentIndex - 1];
	};

	var countExternalPoints = function (edges) {
		var externalPoints = [];
		for (var i = 0; i < edges.length; i++) {
			var p;
			if (!edges[i].firstIsInternal) {
				p = getPointFromEdge(edges[i], 1);
				if (externalPoints.filter(function (ep) { return koEqual(ep.x, p.x) && koEqual(ep.y, p.y) }).length === 0) {
					externalPoints.push(p);
				}
			}

			if (!edges[i].secondIsInternal) {
				p = getPointFromEdge(edges[i], 2);
				if (externalPoints.filter(function (ep) { return koEqual(ep.x, p.x) && koEqual(ep.y, p.y) }).length === 0) {
					externalPoints.push(p);
				}
			}
		}

		return externalPoints.length;
	};

	var createEdgesDictionary = function(edges, isHorizontal) {
		var dic = {};

		var getKey = function(e) {
			if (isHorizontal) {
				return ko.utils.unwrapObservable(e.y1);
			} 

			return ko.utils.unwrapObservable(e.x1);
		};

		for (var i = 0; i < edges.length; i++) {
			var key = getKey(edges[i]);
			if (!dic[key]) {
				dic[key] = [];
			}

			dic[key].push(edges[i]);
		}

		return dic;
	};

	var getExternalPoints = function (edgesVertical, edgesHorizontal, page) {
		// сортируем горизонтальные грани по y,x

		// начиная с самой верхней правой грани идём по часовой стрелке, добавляем в points
		// все внешние точки, промежуточные - в influenced

		edgesHorizontal.sort(function (e1, e2) {
			var yDiff = koDiff(e1.y1, e2.y1);
			if (yDiff !== 0) {
				return yDiff;
			}

			return 0 - koDiff(e1.x2, e2.x2);
		});

		var horizontalEdgesDictionary = createEdgesDictionary(edgesHorizontal, true);
		var verticalEdgesDictionary = createEdgesDictionary(edgesVertical, false);

		var pointsCount = countExternalPoints(edgesHorizontal);

		var bottomLevel = ko.utils.unwrapObservable(edgesHorizontal[edgesHorizontal.length - 1].y1);
		var startingY = ko.utils.unwrapObservable(edgesHorizontal[0].y1);
		var startingX = ko.utils.unwrapObservable(edgesHorizontal[0].x2);

		var points = [];
		var currentMode = 1;
		var currentPoint = getLimitedPointFromEdge(edgesHorizontal[0], 2, currentMode, horizontalEdgesDictionary, verticalEdgesDictionary, page);

		do {
			if (currentMode === 1 && ko.utils.unwrapObservable(currentPoint.y) === bottomLevel) {
				currentMode = 0;
				currentPoint.pointType = 0;
			}

			if (!currentPoint.isInternal) {
				points.push(currentPoint);

				// ищем следующую в вертикальных
				currentPoint = findNextExternalPoint(edgesVertical, currentPoint, false, currentMode, horizontalEdgesDictionary, verticalEdgesDictionary, page);

				if (!currentPoint) {
					console.log("no next point");
					return points;
				}

				points.push(currentPoint);

				if (points.length > pointsCount) {
					console.log("too many points");
					return points;
				}
			}

			if (ko.utils.unwrapObservable(currentPoint.x) === startingX &&
				ko.utils.unwrapObservable(currentPoint.y) === startingY) {
				break;
			}

			// ищем следующую в горизонтальных
			currentPoint = findNextExternalPoint(edgesHorizontal, currentPoint, true, currentMode, horizontalEdgesDictionary, verticalEdgesDictionary, page);
			if (!currentPoint) {
				console.log("no next point");
				return points;
			}
		} while (ko.utils.unwrapObservable(currentPoint.x) !== startingX ||
			ko.utils.unwrapObservable(currentPoint.y) !== startingY);

		return points;
	};

	/**
	 * Получить узлы полигона
	 * @param {any} rectangles прямоугольники
	 * @param {any} page текущая страница
	 */
	var getPolygonPoints = function (rectangles, page) {
		if (!rectangles || rectangles.length === 0) {
			return null;
		}

		var edgesHorizontal = getHorizontalEdges(rectangles);
		var edgesVertical = getVerticalEdges(rectangles);

		markInternalPoints(edgesVertical, edgesHorizontal);

		return getExternalPoints(edgesVertical, edgesHorizontal, page);
	};

	var normalizeRectangles = function (rectanglesObservable) {
		if (!rectanglesObservable) {
			return;
		}

		var rects = rectanglesObservable();
		if (!rects || rects.length < 2) {
			return;
		}

		var wasChanges;

		do {
			wasChanges = false;

			for (var i = 0; i < rects.length; i++) {
				for (var j = i + 1; j < rects.length; j++) {
					var rectA = rects[i];
					var rectB = rects[j];

					// нас интересуют только пересекающиеся прямоугольники
					if (rectA.x1() > rectB.x2() ||
						rectA.x2() < rectB.x1() ||
						rectA.y1() > rectB.y2() ||
						rectA.y2() < rectB.y1()) {
						continue;
					}
					var x1, x2, y1, y2;

					var indexTo2;

					if ((rectA.x1() === rectB.x2() || rectA.x2() === rectB.x1()) &&
						!(rectA.y1() === rects[j].y2() || rectA.y2() === rects[j].y1())) {
						indexTo2 = 0;

						if (rectA.y1() !== rectB.y1()) {
							if (rectA.y1() < rectB.y1()) {
								y1 = rectA.y1();
								y2 = rectB.y1();
								x1 = rectA.x1();
								x2 = rectA.x2();
								rectA.y1(rectB.y1());
							} else {
								y1 = rectB.y1();
								y2 = rectA.y1();
								x1 = rectB.x1();
								x2 = rectB.x2();
								rectB.y1(rectA.y1());
							}

							rects.splice(i, 0, new FC.BusinessModels.DocumentEditor.Coordinates({ X1: x1, X2: x2, Y1: y1, Y2: y2 }));
							indexTo2++;
						}

						if (rectA.y2() !== rectB.y2()) {
							if (rectA.y2() < rectB.y2()) {
								y1 = rectA.y2();
								y2 = rectB.y2();
								x1 = rectB.x1();
								x2 = rectB.x2();
								rectB.y2(rectA.y2());
							} else {
								y1 = rectB.y2();
								y2 = rectA.y2();
								x1 = rectA.x1();
								x2 = rectA.x2();
								rectA.y2(rectB.y2());
							}

							rects.splice(i + indexTo2,
								0,
								new FC.BusinessModels.DocumentEditor.Coordinates({ X1: x1, X2: x2, Y1: y1, Y2: y2 }));
							indexTo2++;
						}

						if (indexTo2 > 0) {
							j += indexTo2;
							wasChanges = true;
							continue;
						}
					}

					if ((rectA.y1() === rects[j].y2() || rectA.y2() === rects[j].y1()) &&
						!(rectA.x1() === rectB.x2() || rectA.x2() === rectB.x1())) {
						indexTo2 = 0;

						if (rectA.x1() !== rectB.x1()) {
							if (rectA.x1() < rectB.x1()) {
								y1 = rectA.y1();
								y2 = rectA.y2();
								x1 = rectA.x1();
								x2 = rectB.x1();
								rectA.x1(rectB.x1());
							} else {
								y1 = rectB.y1();
								y2 = rectB.y2();
								x1 = rectB.x1();
								x2 = rectA.x1();
								rectB.x1(rectA.x1());
							}

							rects.splice(i, 0, new FC.BusinessModels.DocumentEditor.Coordinates({ X1: x1, X2: x2, Y1: y1, Y2: y2 }));
							indexTo2++;
						}

						if (rectA.x2() !== rectB.x2()) {
							if (rectA.x2() < rectB.x2()) {
								y1 = rectB.y1();
								y2 = rectB.y2();
								x1 = rectA.x2();
								x2 = rectB.x2();
								rectB.x2(rectA.x2());
							} else {
								y1 = rectA.y1();
								y2 = rectA.y2();
								x1 = rectB.x2();
								x2 = rectA.x2();
								rectA.x2(rectB.x2());
							}

							rects.splice(i + indexTo2,
								0,
								new FC.BusinessModels.DocumentEditor.Coordinates({ X1: x1, X2: x2, Y1: y1, Y2: y2 }));
							indexTo2++;
						}

						if (indexTo2 > 0) {
							j += indexTo2;
							wasChanges = true;
							continue;
						}
					}
				}
			}
		} while (wasChanges);

		rectanglesObservable(rects);
	};

	return {
		createPolygonFromRects: createPolygonFromRects,
		getPolygonPoints: getPolygonPoints,
		normalizeRectangles: normalizeRectangles,
		getHorizontalEdges: getHorizontalEdges,
		getVerticalEdges: getVerticalEdges
	};
})();

// для отладки
FC.Utils.simplifyRects = function (rectangles) {
	var rects = ko.toJS(rectangles);

	var xArray = [];
	var yArray = [];
	var i;
	for (i = 0; i < rects.length; i++) {
		var x1Exist = xArray.filter(function (x) { return x === rects[i].x1; }).length > 0;
		var x2Exist = xArray.filter(function (x) { return x === rects[i].x2; }).length > 0;
		var y1Exist = yArray.filter(function (y) { return y === rects[i].y1; }).length > 0;
		var y2Exist = yArray.filter(function (y) { return y === rects[i].y2; }).length > 0;
		if (!x1Exist) {
			xArray.push(rects[i].x1);
		}

		if (!x2Exist) {
			xArray.push(rects[i].x2);
		}

		if (!y1Exist) {
			yArray.push(rects[i].y1);
		}

		if (!y2Exist) {
			yArray.push(rects[i].y2);
		}
	}

	xArray.sort(function (v1, v2) { return v1 - v2; });
	yArray.sort(function (v1, v2) { return v1 - v2; });

	var simplifiedRects = [];
	for (i = 0; i < rects.length; i++) {
		simplifiedRects.push({
			x1: xArray.indexOf(rects[i].x1) + 1,
			y1: yArray.indexOf(rects[i].y1) + 1,
			x2: xArray.indexOf(rects[i].x2) + 1,
			y2: yArray.indexOf(rects[i].y2) + 1
		});
	}
	return simplifiedRects;
};