﻿FC.Viewmodels.ActionZones = FC.Viewmodels.ActionZones || {};

FC.Viewmodels.ActionZones.BaseZone = function baseZone(rectangles, target, page, zoneActivationType) {
	this.isActive = ko.pureComputed(function () {
		switch (zoneActivationType) {
			case FC.Viewmodels.ActionZones.zoneActivationType.selected:
				return target.selected() && page.editorActive();
			default:
				return true;
		}
	}, this);
	this.target = target;
	this.cursorStyle = ko.observable(FC.Viewmodels.ActionZones.cursorTypes.crosshair);
	this.rectangles = rectangles;
	this.page = page;

	// глубина вложенности зоны - при пересечении надо выделять самую глубокую
	this.depth = ko.observable(0);
};

FC.Viewmodels.ActionZones.BaseZone.prototype.draw = function () { };
FC.Viewmodels.ActionZones.BaseZone.prototype.mouseDown = function () { return false; };
FC.Viewmodels.ActionZones.BaseZone.prototype.mouseUp = function () { return false; };
FC.Viewmodels.ActionZones.BaseZone.prototype.mouseMove = function () { return false; };
FC.Viewmodels.ActionZones.BaseZone.prototype.onHover = function () { return false; };

FC.Viewmodels.ActionZones.BaseZone.prototype.pointIsNear = function (point, delta) {
	return FC.Viewmodels.ActionZones.BaseZone.pointIsNear(this.rectangles, point, delta);
};

FC.Viewmodels.ActionZones.BaseZone.prototype.pointIsInside = function (point) {
	if (point.pageId !== this.page.id()) {
		return false;
	}

	return FC.Viewmodels.ActionZones.BaseZone.pointIsNear(this.rectangles, point, 0);
};


FC.Viewmodels.ActionZones.zoneActivationType = {
	always: 0,
	selected: 1,
	hovering: 2
};

FC.Viewmodels.ActionZones.BaseZone.pointIsNear = function (rectangles, point, delta) {
	if (!point || point.x < 0 || point.y < 0) {
		return false;
	}

	var rects = ko.utils.unwrapObservable(rectangles);
	for (var i = 0; i < rects.length; i++) {
		if (rects[i].x1() - delta <= point.x &&
			rects[i].x2() + delta >= point.x &&
			rects[i].y1() - delta <= point.y &&
			rects[i].y2() + delta >= point.y) {
			return true;
		}
	}

	return false;
};

FC.Viewmodels.ActionZones.BaseZone.prototype.selectTarget = function () {
	var notSelected = !this.target.selected();
	if (notSelected) {
		this.page.deselectRegions();
	}

	var scrolledToField = this.page.selectRegion(this.target, true, this.target.highlighted());

	if (this.target.parentRegion && this.selectParentOnSelect) {
		this.page.selectRegion(this.target.parentRegion, !scrolledToField, this.target.highlighted());
	}

	if (notSelected) {
		this.page.draw();
	}
}