﻿FC.Viewmodels.ActionZones.BorderZoneBase = function borderZoneBase(rectangles, target, sourceRectangles, page, depth) {
	if (!rectangles) {
		throw "no rectangle";
	}

	FC.Viewmodels.ActionZones.BaseZone.call(this, rectangles, target, page, FC.Viewmodels.ActionZones.zoneActivationType.selected);

	this.depth(depth + 60);
	this.sourceRectangles = sourceRectangles;
	this.startPoint = null;
};

FC.Viewmodels.ActionZones.BorderZoneBase.prototype = Object.create(FC.Viewmodels.ActionZones.BaseZone.prototype);

FC.Viewmodels.ActionZones.BorderZoneBase.prototype.draw = function (layerIndex) {
	if (!this.isActive() || layerIndex === 0 || !this.target.field()) {
		return;
	}

	var isImage = this.target.field().fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Picture;
	var isCheckboxGroup = this.target.field().fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Choice;
	var isGroup = this.target.field().fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.GroupParent;

	var cssClass = ".region-squares";

	if (isImage) {
		cssClass = ".regionImage-squares";
	} else if (isCheckboxGroup) {
		cssClass = ".regionChoice-squares";
	} else if (isGroup) {
		cssClass = ".regionGroup-squares";
	}

	var squaresColor = FC.Utils.GetCssClassValue(cssClass, "background-color");
	var layer = this.page.getLayer(1);
	var rects = ko.utils.unwrapObservable(this.rectangles);
	for (var i = 0; i < rects.length; i++) {
		layer.drawFilledRectangle(rects[i], squaresColor);
	}
};

FC.Viewmodels.ActionZones.BorderZoneBase.handlerRemovalKey = "zonesResizing";

FC.Viewmodels.ActionZones.BorderZoneBase.prototype.mouseDown = function (point) {
	var self = this;

	if (!self.isActive() || !self.pointIsInside(point)) {
		return false;
	}

	self.startPoint = point;

	var mouseMove = function (data) { self.mouseMove(self, data); };
	var mouseUp = function () { self.mouseUp(self); };

	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
	docEditor.addOnMouseUp(mouseUp.bind(this), 100, FC.Viewmodels.ActionZones.BorderZoneBase.handlerRemovalKey);
	docEditor.addOnMouseLeave(mouseUp.bind(this), 100, FC.Viewmodels.ActionZones.BorderZoneBase.handlerRemovalKey);
	docEditor.addOnMouseMove(mouseMove.bind(this), 100, FC.Viewmodels.ActionZones.BorderZoneBase.handlerRemovalKey);

	return true;
};

FC.Viewmodels.ActionZones.BorderZoneBase.prototype.mouseUp = function (zone) {
	zone.startPoint = null;

	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
	docEditor.removeOnMouseUp(FC.Viewmodels.ActionZones.BorderZoneBase.handlerRemovalKey);
	docEditor.removeOnMouseLeave(FC.Viewmodels.ActionZones.BorderZoneBase.handlerRemovalKey);
	docEditor.removeOnMouseMove(FC.Viewmodels.ActionZones.BorderZoneBase.handlerRemovalKey);

	if (zone.target.resized) {
		zone.target.updateText();
	}
};

FC.Viewmodels.ActionZones.BorderZoneBase.prototype.mouseMoveBase = function (zone, point, delta) {
	var resizeInfo = zone.target.resizeRegion(delta, false, ko.utils.unwrapObservable(this.sourceRectangles));
	if (resizeInfo && (resizeInfo.x1 !== 0 || resizeInfo.x2 !== 0 || resizeInfo.y1 !== 0 || resizeInfo.y2 !== 0)) {
		if (resizeInfo.x1 !== 0 || resizeInfo.x2 !== 0) {
			zone.startPoint.x = point.x;
		}

		if (resizeInfo.y1 !== 0 || resizeInfo.y2 !== 0) {
			zone.startPoint.y = point.y;
		}

		zone.page.draw();
	}
};