﻿FC.Viewmodels.ActionZones.ColumnHeaderZone = function сolumnHeaderZone(rectangles, target, page, depth, column) {
	var zoneRect = ko.pureComputed(function () {
		var newRects = [];
		var rects = ko.utils.unwrapObservable(rectangles);

		for (var i = 0; i < rects.length; i++) {
			newRects.push(new FC.Viewmodels.ImageEditor.Rectangle(
			{
				x1: column.leftPos(),
				y1: rects[i].y1() - FC.Viewmodels.ActionZones.zoneConstants.columnHeaderHeight / page.zoom(),
				x2: column.rightPos(),
				y2: rects[i].y1()
			}));
		}

		return newRects;
	});

	FC.Viewmodels.ActionZones.BaseZone.call(this, zoneRect, target, page, FC.Viewmodels.ActionZones.zoneActivationType.selected);

	this.sourceRectangles = rectangles;

	this.titleRectangles = ko.pureComputed(function () {
		var newRects = [];
		var rects = ko.utils.unwrapObservable(this.rectangles);

		for (var i = 0; i < rects.length; i++) {
			newRects.push(new FC.Viewmodels.ImageEditor.Rectangle(
			{
				x1: rects[i].x1(),
				y1: rects[i].y1(),
				x2: rects[i].x2() - FC.Viewmodels.ActionZones.zoneConstants.columnHeaderHeight / this.page.zoom(),
				y2: rects[i].y2()
			}));
		}

		return newRects;
	}, this);

	this.deleterRectangles = ko.pureComputed(function () {
		var newRects = [];
		var rects = ko.utils.unwrapObservable(this.rectangles);

		for (var i = 0; i < rects.length; i++) {
			newRects.push(new FC.Viewmodels.ImageEditor.Rectangle(
			{
				x1: rects[i].x2() - FC.Viewmodels.ActionZones.zoneConstants.columnHeaderHeight / this.page.zoom(),
				y1: rects[i].y1(),
				x2: rects[i].x2(),
				y2: rects[i].y2()
			}));
		}

		return newRects;
	}, this);

	this.cursorStyle(FC.Viewmodels.ActionZones.cursorTypes.pointer);
	this.column = column;

	this.depth(depth + 10);

	this.columnIsBinded = ko.pureComputed(function () {
		return !!column.id();
	});
};

FC.Viewmodels.ActionZones.ColumnHeaderZone.prototype = Object.create(FC.Viewmodels.ActionZones.BaseZone.prototype);

FC.Viewmodels.ActionZones.ColumnHeaderZone.prototype.draw = function (layerIndex) {
	if (!this.isActive() || layerIndex === 0) {
		return;
	}

	var color = FC.Utils.GetCssClassValue(".table_columnHeader", "background-color");
	var colorDeleter = FC.Utils.GetCssClassValue(".table_columnHeader-deleter", "background-color");
	var layer = this.page.getLayer(1);
	var rect = this.rectangles()[0];
	var tableRect = this.sourceRectangles()[0];

	if (this.columnIsBinded()) {
		var rectText = this.titleRectangles()[0];
		var rectDeleter = this.deleterRectangles()[0];

		layer.drawFilledRectangle(rectText, color);
		layer.drawFilledRectangle(rectDeleter, colorDeleter);

		var textPoint = new FC.Viewmodels.ImageEditor.Point(rectText.x1() + 4 / this.page.zoom(), rectText.y2() - 2 / this.page.zoom());		
		var text = this.column.caption();
        var textWidth = layer.measureText(text, 11, "SourceSansPro");

		while (text.length > 2 && textWidth + textPoint.x > rectText.x2()) {
			text = text.substr(0, text.length - 2) + "…";
            textWidth = layer.measureText(text, 11, "SourceSansPro");
		}

        layer.drawText(textPoint, text, "rgba(255,255,255,1)", 11, "SourceSansPro");
		
		textPoint = new FC.Viewmodels.ImageEditor.Point(rectDeleter.x1() + 3 / this.page.zoom(), rectDeleter.y2() - 3 / this.page.zoom());
		layer.drawText(textPoint, "\ue923", "rgba(255,255,255,1)", 10, "abbyy-icons");
	} else {
		layer.drawFilledRectangle(rect, color);

		var rectSize = 8;
		var point1 = new FC.Viewmodels.ImageEditor.Point(
			(rect.x1() + rect.x2() - rectSize / this.page.zoom()) / 2,
			(rect.y1() + rect.y2() - 2 / this.page.zoom()) / 2);
		var point2 = new FC.Viewmodels.ImageEditor.Point(
			(rect.x1() + rect.x2() + rectSize / this.page.zoom()) / 2,
			(rect.y1() + rect.y2() + 2 / this.page.zoom()) / 2);
		var point3 = new FC.Viewmodels.ImageEditor.Point(
			(rect.x1() + rect.x2() - 2 / this.page.zoom()) / 2,
			(rect.y1() + rect.y2() - rectSize / this.page.zoom()) / 2);
		var point4 = new FC.Viewmodels.ImageEditor.Point(
			(rect.x1() + rect.x2() + 2 / this.page.zoom()) / 2,
			(rect.y1() + rect.y2() + rectSize / this.page.zoom()) / 2);

		layer.drawFilledRectangle(FC.Viewmodels.ImageEditor.Rectangle.getRectangleFromPoints(point1, point2), "#ffffff");
		layer.drawFilledRectangle(FC.Viewmodels.ImageEditor.Rectangle.getRectangleFromPoints(point3, point4), "#ffffff");
	}
	
	var lineWidth = 1;
	var lineColor = FC.Utils.GetCssClassValue(".regionParent-selected", "border-color");
	if (rect.x1() > tableRect.x1()) {
		layer.drawLine(new FC.Viewmodels.ImageEditor.Point(rect.x1(), rect.y1()), new FC.Viewmodels.ImageEditor.Point(rect.x1(), rect.y2()), lineWidth, lineColor);
	}
	if (rect.x2() < tableRect.x2()) {
		layer.drawLine(new FC.Viewmodels.ImageEditor.Point(rect.x2() - 1 / this.page.zoom(), rect.y1()), new FC.Viewmodels.ImageEditor.Point(rect.x2() - 1 / this.page.zoom(), rect.y2()), lineWidth, lineColor);
	}
};

FC.Viewmodels.ActionZones.ColumnHeaderZone.prototype.mouseDown = function (point, page, event) {
	if (!this.pointIsInside(point)) {
		return false;
	}

	if (!this.columnIsBinded()) {
		this.target.bindColumn(this.column, event);
	} else {
		if (FC.Viewmodels.ActionZones.BaseZone.pointIsNear(this.deleterRectangles, point, 0)) {
			this.target.unbindColumn(this.column);
		}
	}

	return true;
};