﻿FC.Viewmodels.ActionZones.HorizontalSeparatorZone = function horizontalSeparatorZone(rectangles, target, page, depth, separator) {
	var zoneRect = ko.pureComputed(function () {
		var newRects = [];
		var rects = ko.utils.unwrapObservable(rectangles);

		var threshold = FC.Viewmodels.ActionZones.zoneConstants.horizontalSeparatorThreshold * page.zoom() < 2
			? 2 / page.zoom()
			: FC.Viewmodels.ActionZones.zoneConstants.horizontalSeparatorThreshold;

		for (var i = 0; i < rects.length; i++) {
			newRects.push(new FC.Viewmodels.ImageEditor.Rectangle(
			{
				x1: rects[i].x1(),
				y1: Math.floor(separator() - threshold),
				x2: rects[i].x2(),
				y2: Math.ceil(separator() + threshold)
			}));
		}

		return newRects;
	});

	FC.Viewmodels.ActionZones.BaseZone.call(this,
		zoneRect,
		target,
		page,
		FC.Viewmodels.ActionZones.zoneActivationType.always);

	this.sourceRectangles = rectangles;
	this.startPoint = null;
	this.separator = separator;
	this._id = FC.Viewmodels.ActionZones.HorizontalSeparatorZone.globalCounter++;
	this.isHovering = ko.observable(false);

	this.movable = ko.computed(function () {
		var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
		return !docEditor.extender
			.horizontalSeparatorDrawActivated() &&
			!docEditor.extender.verticalSeparatorDrawActivated();
	});

	this.page.subscriptions.push({
		key: this.target,
		value: this.movable
	});

	this.depth = ko.computed(function () {
		var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
		if (docEditor.extender.horizontalSeparatorDrawActivated() ||
			docEditor.extender.verticalSeparatorDrawActivated()) {
			return -1;
		} else {
			return depth + 100;
		}
	});

	this.page.subscriptions.push({
		key: this.target,
		value: this.depth
	});

	this.cursorStyle = ko.computed(function () {
		var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
		if (docEditor.extender.deleteSeparatorActivated()) {
			return FC.Viewmodels.ActionZones.cursorTypes.deleteSeparator;
		}

		return FC.Viewmodels.ActionZones.cursorTypes.moveHorizontalSeparator;
	});

	this.page.subscriptions.push({
		key: this.target,
		value: this.cursorStyle
	});
};

FC.Viewmodels.ActionZones.HorizontalSeparatorZone.prototype = Object.create(FC.Viewmodels.ActionZones.BaseZone.prototype);

FC.Viewmodels.ActionZones.HorizontalSeparatorZone.prototype.draw = function (layerIndex) {
	if (layerIndex === 0 && this.target.selected() || layerIndex === 1 && !this.target.selected()) {
		return;
	}

	var layer = this.page.getLayer(this.target.selected() ? 1 : 0);
	var cssClass = ".regionParent-default";
	if (this.target.selected() && this.movable()) {
		if (this.isHovering()) {
			cssClass = ".regionParent-selected-hover";
		} else {
			cssClass = ".regionParent-selected";
		}
	}

	var lineWidth = 2;
	var lineColor = FC.Utils.GetCssClassValue(cssClass, "border-color");

	var rects = this.sourceRectangles();
	for (var i = 0; i < rects.length; i++) {
		layer.drawLine(
			new FC.Viewmodels.ImageEditor.Point(rects[i].x1(), this.separator()),
			new FC.Viewmodels.ImageEditor.Point(rects[i].x2(), this.separator()),
			lineWidth,
			lineColor);
	}
};

FC.Viewmodels.ActionZones.HorizontalSeparatorZone.handlerRemovalKey = "separatorMove";

FC.Viewmodels.ActionZones.HorizontalSeparatorZone.prototype.mouseDown = function (point) {
	if (!this.pointIsInside(point) || !this.movable()) {
		return false;
	}

	this.selectTarget();

	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();

	if (docEditor.extender.deleteSeparatorActivated()) {
		this.target.deleteHorizontalSeparator(this);
		return true;
	}

	this.startPoint = point;
	var zone = this;

	var mouseMove = function (data) { zone.mouseMove(zone, data); };
	var mouseUp = function () { zone.mouseUp(zone); };

	docEditor.addOnMouseUp(mouseUp.bind(this), 100, FC.Viewmodels.ActionZones.HorizontalSeparatorZone.handlerRemovalKey);
	docEditor.addOnMouseLeave(mouseUp.bind(this), 100, FC.Viewmodels.ActionZones.HorizontalSeparatorZone.handlerRemovalKey);
	docEditor.addOnMouseMove(mouseMove.bind(this), 100, FC.Viewmodels.ActionZones.HorizontalSeparatorZone.handlerRemovalKey);

	return true;
};

FC.Viewmodels.ActionZones.HorizontalSeparatorZone.prototype.mouseUp = function (zone) {
	zone.startPoint = null;

	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
	docEditor.removeOnMouseUp(FC.Viewmodels.ActionZones.HorizontalSeparatorZone.handlerRemovalKey);
	docEditor.removeOnMouseLeave(FC.Viewmodels.ActionZones.HorizontalSeparatorZone.handlerRemovalKey);
	docEditor.removeOnMouseMove(FC.Viewmodels.ActionZones.HorizontalSeparatorZone.handlerRemovalKey);
};

FC.Viewmodels.ActionZones.HorizontalSeparatorZone.prototype.mouseMove = function (zone, point) {
	if (point.x < 0) {
		point.x = 0;
	}

	if (point.y < 0) {
		point.y = 0;
	}

	var deltaY = point.y - zone.startPoint.y;

	zone.startPoint = point;
	zone.moveSeparator(zone, deltaY);
	FC.Viewmodels.EventStack.addEvent();

	zone.page.draw();
};

FC.Viewmodels.ActionZones.HorizontalSeparatorZone.prototype.moveSeparator = function (zone, delta) {
	var currentSeparator = zone.separator();
	delta = FC.Viewmodels.ActionZones.HorizontalSeparatorZone.findDeltaWithRestrictions(zone.target, delta, currentSeparator);

	zone.separator(currentSeparator + delta);

	zone.target.resizeCellsY(currentSeparator, zone.separator());
};

FC.Viewmodels.ActionZones.HorizontalSeparatorZone.prototype.onHover = function (point) {
	var self = this;

	if (!self.pointIsInside(point)) {
		return;
	}

	var removalKey = "horizontalSepatator" + self._id;
	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();

	var onMouseMove = function (p) {
		if (!self.pointIsInside(p)) {
			docEditor.removeOnMouseMove(removalKey);
			self.isHovering(false);
			self.page.draw();
		}
	}

	if (!this.isHovering()) {
		this.isHovering(true);
		docEditor.addOnMouseMove(onMouseMove.bind(this), 100, removalKey);
		this.page.draw();
	}
};

FC.Viewmodels.ActionZones.HorizontalSeparatorZone.findDeltaWithRestrictions = function (tableRegion, delta, currentY) {
	var allSeparators = ko.utils.arrayMap(tableRegion.horizontalSeparators(), function (sep) { return sep.value(); });
	allSeparators.unshift(tableRegion.rectangles()[0].y1());
	allSeparators.push(tableRegion.rectangles()[0].y2());

	var previousSeparator = allSeparators[0];
	var nextSeparator = allSeparators[allSeparators.length - 1];

	for (var i = 0; i < allSeparators.length; i++) {
		if (allSeparators[i] > previousSeparator && allSeparators[i] < currentY) {
			previousSeparator = allSeparators[i];
		}

		if (allSeparators[i] < nextSeparator && allSeparators[i] > currentY) {
			nextSeparator = allSeparators[i];
		}
	}

	if (currentY + delta < previousSeparator + FC.Viewmodels.ActionZones.zoneConstants.horizontalSeparatorThreshold) {
		delta = previousSeparator + FC.Viewmodels.ActionZones.zoneConstants.horizontalSeparatorThreshold - currentY;
	}

	if (currentY + delta > nextSeparator - FC.Viewmodels.ActionZones.zoneConstants.horizontalSeparatorThreshold) {
		delta = nextSeparator - FC.Viewmodels.ActionZones.zoneConstants.horizontalSeparatorThreshold - currentY;
	}

	return delta;
};

FC.Viewmodels.ActionZones.HorizontalSeparatorZone.globalCounter = 0;