﻿/**
 * Угловая зона управления размером полигона
 * @param {} currentPoint текущая точка
 * @param {} point1 предыдущая точка
 * @param {} point2 следующая точка
 * @param {} allPoints все узлы полигона
 * @param {} currentIndex индекс текущей точки
 * @param {} target регион, с которым связана зона
 * @param {} page страница, на которой существует регион
 * @param {} depth грубина зоны 
 */
FC.Viewmodels.ActionZones.PolygonCornerZone = function (currentPoint, point1, point2, allPoints, currentIndex,
	target, page, depth) {

	// положение квадратика управления зависит от взаиморасположения трех точек.
	// и от того, в левой или в правой части фигуры находится текущая точка
	var rectPosition1 = currentPoint.pointType
		? FC.Viewmodels.ActionZones.PolygonBorderZone.RectanglePositions.NorthEast
		: FC.Viewmodels.ActionZones.PolygonBorderZone.RectanglePositions.SouthWest;

	var rectPosition2 = currentPoint.pointType ?
		FC.Viewmodels.ActionZones.PolygonBorderZone.RectanglePositions.SouthEast :
		FC.Viewmodels.ActionZones.PolygonBorderZone.RectanglePositions.NorthWest;

	var rectPosition = ko.computed(function () {
		if (ko.utils.unwrapObservable(point2.x) < ko.utils.unwrapObservable(currentPoint.x) ||
			ko.utils.unwrapObservable(point1.x) > ko.utils.unwrapObservable(currentPoint.x)) {
			return rectPosition2;
		}

		return rectPosition1;
	});

	// курсор зависит от взаиморасположения трех точек
	var cursor = ko.computed(function () {
		if (ko.utils.unwrapObservable(point2.x) < ko.utils.unwrapObservable(currentPoint.x) ||
			ko.utils.unwrapObservable(point1.x) > ko.utils.unwrapObservable(currentPoint.x)) {
			return FC.Viewmodels.ActionZones.cursorTypes.northWestSouthEast;
		}

		return FC.Viewmodels.ActionZones.cursorTypes.northEastSouthWest;
	});

	// можно ли менять горизонтальное значение
	var canChangeHorizontalValue = function (delta) {
		return FC.Viewmodels.ActionZones.PolygonBorderZone.canChangeHorizontalValue(
			currentPoint,
			allPoints,
			currentIndex,
			delta,
			page);
	};

	// можно ли менять вертикальное значение
	var canChangeVerticalValue = function (delta) {
		return FC.Viewmodels.ActionZones.PolygonBorderZone.canChangeVerticalValue(
			currentPoint,
			allPoints,
			currentIndex,
			delta,
			page);
	};

	// наследуемся от PolygonBorderZone
	FC.Viewmodels.ActionZones.PolygonBorderZone.call(this,
		currentPoint,
		rectPosition,
		currentPoint.related,
		canChangeHorizontalValue,
		canChangeVerticalValue,
		target,
		page,
		depth,
		cursor);
};

FC.Viewmodels.ActionZones.PolygonCornerZone.prototype = Object.create(FC.Viewmodels.ActionZones.PolygonBorderZone.prototype);
