﻿/**
 * Зона управления размером полигона, находящаяся в середине ребра полигона
 * @param {} currentPoint первая точка ребра
 * @param {} nextPoint вторая точка ребра
 * @param {} allPoints все узлы полигона
 * @param {} currentIndex индекс первой точки
 * @param {} target регион, с которым связана зона
 * @param {} page страница, на которой существует регион
 * @param {} depth грубина зоны
 */
FC.Viewmodels.ActionZones.PolygonMiddleZone = function (currentPoint, nextPoint, allPoints, currentIndex, target, page, depth) {

	// определим, горизонтальное ли ребро
	var isHorizontal = ko.utils.unwrapObservable(currentPoint.x) !== ko.utils.unwrapObservable(nextPoint.x);
	var rectPosition;

	// если ребро горизонтальное
	if (isHorizontal) {
		if (ko.utils.unwrapObservable(currentPoint.x) < ko.utils.unwrapObservable(nextPoint.x)) {
			// если первая точка левее второй, то квадратик, за который тянет пользователь расположен к северу от ребра
			rectPosition = FC.Viewmodels.ActionZones.PolygonBorderZone.RectanglePositions.North;
		} else {
			// иначе к югу
			rectPosition = FC.Viewmodels.ActionZones.PolygonBorderZone.RectanglePositions.South;
		}
	} else {
		// если текущая точка на левой стороне фигуры (currentPoint.pointType==0), то квадратик смотрит на восток
		rectPosition = currentPoint.pointType ?
			FC.Viewmodels.ActionZones.PolygonBorderZone.RectanglePositions.East :
			FC.Viewmodels.ActionZones.PolygonBorderZone.RectanglePositions.West;
	}

	// определим точку point1 выше (левее) и точку point2 ниже (правее)
	var point1;
	var point2;
	if (isHorizontal && ko.utils.unwrapObservable(currentPoint.x) < ko.utils.unwrapObservable(nextPoint.x) ||
		!isHorizontal && ko.utils.unwrapObservable(currentPoint.y) < ko.utils.unwrapObservable(nextPoint.y)) {
		point1 = currentPoint;
		point2 = nextPoint;
	} else {
		point1 = nextPoint;
		point2 = currentPoint;
	}

	var influencedValues = currentPoint.related;

	// отфильтруем зависимые значения в зависимости от их направления
	var currentInfluenced = isHorizontal
		? influencedValues.filter(function (v) { return !v.isHorizontal; })
		: influencedValues.filter(function (v) { return v.isHorizontal; });

	// найдём положение квадратика управления
	var coordinate = ko.pureComputed(function () {
		if (isHorizontal) {
			return (point1.x() + point2.x()) / 2;
		}
		return (point1.y() + point2.y()) / 2;
	});

	var point = isHorizontal
		? new FC.Viewmodels.ImageEditor.Point(coordinate, point1.y)
		: new FC.Viewmodels.ImageEditor.Point(point1.x, coordinate);

	// может ли меняться горизонтальное значение?
	var canChangeHorizontalValue = function (delta) {
		if (isHorizontal) {
			return false;
		}

		return FC.Viewmodels.ActionZones.PolygonBorderZone.canChangeHorizontalValue(
			currentPoint,
			allPoints,
			currentIndex,
			delta,
			page);
	};

	// может ли меняться вертикальное значение?
	var canChangeVerticalValue = function (delta) {
		if (!isHorizontal) {
			return false;
		}

		return FC.Viewmodels.ActionZones.PolygonBorderZone.canChangeVerticalValue(
			currentPoint,
			allPoints,
			currentIndex,
			delta,
			page);
	};

	// определим курсор 
	var cursor = isHorizontal
		? FC.Viewmodels.ActionZones.cursorTypes.northSouthResize
		: FC.Viewmodels.ActionZones.cursorTypes.eastWestResize;

	// наследуем от PolygonBorderZone
	FC.Viewmodels.ActionZones.PolygonBorderZone.call(this, point, rectPosition, currentInfluenced, canChangeHorizontalValue, canChangeVerticalValue, target, page, depth, cursor);

	var middleShownConstant = 5;

	// зону видно, если размер ребра больше middleShownConstant
	this.isActive = ko.pureComputed(function () {
		var defaultActive = target.selected() && page.editorActive();
		if (!defaultActive) {
			return false;
		}

		var zoom = page.zoom();

		if (isHorizontal &&
			Math.abs(point2.x() - point1.x()) * zoom < FC.Viewmodels.ActionZones.zoneConstants.borderZoneSize * middleShownConstant) {
			return false;
		}

		if (!isHorizontal &&
			Math.abs(point2.y() - point1.y()) * zoom < FC.Viewmodels.ActionZones.zoneConstants.borderZoneSize * middleShownConstant) {
			return false;
		}

		return true;
	}, this);
};

FC.Viewmodels.ActionZones.PolygonMiddleZone.prototype = Object.create(FC.Viewmodels.ActionZones.PolygonBorderZone.prototype);

