﻿/**
 * Зона для предраспознанного текста
 * @param {} rectangles прямоугольники зоны
 * @param {} target регион, с которым связана зона
 * @param {} page страница, на которой нарисован регион
 */
FC.Viewmodels.ActionZones.RecognizedTextZone = function recognizedTextZone(rectangles, target, page) {
	// наследуемся от базовой зоны
	FC.Viewmodels.ActionZones.BaseZone.call(this, rectangles, target, page, FC.Viewmodels.ActionZones.zoneActivationType.hovering);

	this.cursorStyle(FC.Viewmodels.ActionZones.cursorTypes.selectText);
	this.isRecognizedTextZone = true;
	this.regionCreator = null;
	this.parentRegion = null;
	this.modifiedCursorStyle = FC.Viewmodels.ActionZones.cursorTypes.modifyText;
};

FC.Viewmodels.ActionZones.RecognizedTextZone.prototype = Object.create(FC.Viewmodels.ActionZones.BaseZone.prototype);

FC.Viewmodels.ActionZones.RecognizedTextZone.prototype.draw = function (layerIndex) {
	if (!this.target.selected() || layerIndex === 0) {
		return;
	}

	var zoneColor = FC.Utils.GetCssClassValue(".region-text", "background-color");

	var layer = this.page.getLayer(1);

	var rects = this.rectangles();
	for (var i = 0; i < rects.length; i++) {
		layer.drawRectangle(rects[i], 0, null, zoneColor);
	}
};

FC.Viewmodels.ActionZones.RecognizedTextZone.prototype.selectTarget = function() {};

/**
 * Нажатие мыши на зону
 * @param {} point точка, в которую нажали мышь
 * @param {} p страница
 * @param {} e событие мыши
 * @param {} allZones  все зоны в месте клика
 */
FC.Viewmodels.ActionZones.RecognizedTextZone.prototype.mouseDown = function (point, p, e, allZones) {
	if (!this.pointIsInside(point)) {
		return false;
	}

	var zone = this;
	this.parentRegion = null;

	if (allZones.length > 1) {
		var tableGroupZones = allZones.filter(function (z) { return z instanceof FC.Viewmodels.ActionZones.GroupBodyEditableZone; });
		if (tableGroupZones.length > 0) {
			this.parentRegion = tableGroupZones[0].target;
		}
	}

	zone.regionCreator = zone.regionCreator || new FC.Viewmodels.DocumentEditor.RegionCreator(zone.page,
		function (rect) {
			FC.Viewmodels.ActionZones.RecognizedTextZone.selectZonesCrossingRectangle(rect, zone.page);
		},
		function (rectangle, parent, page, event) { zone.mouseUp(zone, event); });

	zone.page.deselectHovering = false;

	zone.regionCreator.startRegionDrawing(point, this);

	return true;
};

/**
 * Отжатие мыши
 * @param {} zone текущая зона
 * @param {} event событие
 */
FC.Viewmodels.ActionZones.RecognizedTextZone.prototype.mouseUp = function (zone, event) {
	// получаем полигон из выделенных текстовых зон
	var rects = FC.Viewmodels.ActionZones.RecognizedTextZone.getPolygonFromHoveringRegions(zone.page.selectedRegions());

	// колбэк, который нужно дёрнуть, чтобы вернуть текстовые зоны в нормальное состояние
	var onEnd = function () {
		zone.page.deselectHovering = true;
		zone.page.deselectRegions(true);
		zone.page.draw();
	};

	// если полигона нет
	if (!rects) {
		onEnd();
		return;
	}

	var selectedRegions = zone.page.selectedRegions();

	// если мы модифицируем регионы
	if (zone.page.regionsAreModifying()) {

		// нужно найти модифицируемый регион в активных
		var modifyable = selectedRegions.filter(function (r) { return r.modifiable; })
			.sort(function (r1, r2) {
				return r2.depth - r1.depth;
			});

		// его там может не быть - тогда просто создадим регион (см. ниже)
		if (modifyable.length > 0) {

			// модифицируем регион
			zone.page.modifyRegion(
				modifyable[0],
				rects.map(function (rect) {
					return new FC.Viewmodels.ImageEditor.Rectangle(rect);
				}));

			onEnd();
			zone.page.draw();
			return;
		}

	}
	// получаем текст из выделеннных текстовых зон
	var text = FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextFromRegions(selectedRegions);

	// и создаём регион
	zone.page.createRegion(rects, this.parentRegion, event, text, onEnd);
};

FC.Viewmodels.ActionZones.RecognizedTextZone.prototype.onHover = function () {
	if (!this.target.selected()) {
		if (this.page.deselectHovering) {
			this.page.deselectRegions(true);
		}

		this.page.selectRegion(this.target, false);
		this.page.draw(1);
	}
};

FC.Viewmodels.ActionZones.RecognizedTextZone.zoneCrossesRectangle = function (zone, rectangle) {
	var rects = zone.rectangles();
	for (var i = 0; i < rects.length; i++) {
		if (rects[i].x1() <= ko.utils.unwrapObservable(rectangle.x2) &&
			rects[i].x2() >= ko.utils.unwrapObservable(rectangle.x1) &&
			rects[i].y1() <= ko.utils.unwrapObservable(rectangle.y2) &&
			rects[i].y2() >= ko.utils.unwrapObservable(rectangle.y1)) {

			return true;
		}
	}

	return false;
};

FC.Viewmodels.ActionZones.RecognizedTextZone.zoneInsideRectangle = function (zone, rectangle) {
	var rects = zone.rectangles();

	for (var i = 0; i < rects.length; i++) {
		if (rects[i].x1() < ko.utils.unwrapObservable(rectangle.x1) ||
			rects[i].x2() > ko.utils.unwrapObservable(rectangle.x2) ||
			rects[i].y1() < ko.utils.unwrapObservable(rectangle.y1) ||
			rects[i].y2() > ko.utils.unwrapObservable(rectangle.y2)) {
			return false;
		}
	}

	return rects.length > 0;
};

FC.Viewmodels.ActionZones.RecognizedTextZone.selectZonesByRectangle = function (rectangle, page, condition) {
	var hasChanges = false;
	var zones = page.actionZones();
	for (var i = 0; i < zones.length; i++) {
		var zone = zones[i];
		if (!(zone instanceof FC.Viewmodels.ActionZones.RecognizedTextZone)) {
			continue;
		}

		if (condition(zone, rectangle)) {
			if (!zone.target.selected()) {
				hasChanges = true;
				page.selectRegion(zone.target, false);
			}
		} else {
			if (zone.target.selected()) {
				hasChanges = true;
				page.deselectRegions(false, [zone.target]);
			}
		}
	}

	if (hasChanges) {
		page.draw(1);
	}
};

FC.Viewmodels.ActionZones.RecognizedTextZone.selectZonesCrossingRectangle = function (rectangle, page) {
	FC.Viewmodels.ActionZones.RecognizedTextZone.selectZonesByRectangle(rectangle,
		page,
		FC.Viewmodels.ActionZones.RecognizedTextZone.zoneCrossesRectangle);
};

FC.Viewmodels.ActionZones.RecognizedTextZone.selectZonesInsideRectangle = function (rectangle, page) {
	FC.Viewmodels.ActionZones.RecognizedTextZone.selectZonesByRectangle(rectangle,
		page,
		FC.Viewmodels.ActionZones.RecognizedTextZone.zoneInsideRectangle);
};

FC.Viewmodels.ActionZones.RecognizedTextZone.getRegionsByRectangle = function (rectangle, page, condition) {
	var zones = page.actionZones();
	var regions = [];
	for (var i = 0; i < zones.length; i++) {
		var zone = zones[i];
		if (!(zone instanceof FC.Viewmodels.ActionZones.RecognizedTextZone)) {
			continue;
		}

		if (condition(zone, rectangle)) {
			regions.push(zone.target);
		}
	}

	return regions;
};

FC.Viewmodels.ActionZones.RecognizedTextZone.getRegionsInsideRectangle = function (rectangle, page) {
	return FC.Viewmodels.ActionZones.RecognizedTextZone.getRegionsByRectangle(rectangle,
		page,
		FC.Viewmodels.ActionZones.RecognizedTextZone.zoneInsideRectangle);
};

FC.Viewmodels.ActionZones.RecognizedTextZone.getRegionsCrossingRectangle = function (rectangle, page) {
	FC.Viewmodels.ActionZones.RecognizedTextZone.getRegionsByRectangle(rectangle,
		page,
		FC.Viewmodels.ActionZones.RecognizedTextZone.zoneCrossesRectangle);
};

FC.Viewmodels.ActionZones.RecognizedTextZone.getPolygonFromHoveringRegions = function (regions) {
	if (!regions) {
		return null;
	}

	var rects = [];
	regions.forEach(function (region) {
		if (!region.hovering) {
			return;
		}

		rects = rects.concat(ko.toJS(region.rectangles()));
	});

	if (rects.length === 0) {
		return null;
	}

	return FC.Utils.polygonCommon.createPolygonFromRects(rects);
};