﻿FC.Viewmodels.ActionZones.RegionBodyZone = function regionBodyZone(rectangles, target, page, depth) {
	FC.Viewmodels.ActionZones.BaseZone.call(this, rectangles, target, page, FC.Viewmodels.ActionZones.zoneActivationType.always);

	this.depth(depth + 50);
	this.startPoint = null;
	this.selectParentOnSelect = false;
	this.cursorStyle(FC.Viewmodels.ActionZones.cursorTypes.move);
	this.regionMoved = false;
};

FC.Viewmodels.ActionZones.RegionBodyZone.prototype = Object.create(FC.Viewmodels.ActionZones.BaseZone.prototype);

FC.Viewmodels.ActionZones.RegionBodyZone.prototype.draw = function (layerIndex) {
	if (layerIndex === 0 && this.target.selected() || layerIndex === 1 && !this.target.selected() || !this.target.field()) {
		return;
	}

	var layer = this.page.getLayer(this.target.selected() ? 1 : 0);

	var isImage = this.target.field().fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Picture;

	var selected = this.target.selected() || this.target.highlighted();
	var cssClass = isImage ? ".regionImage-default" : ".region-default";
	if (selected) {
		cssClass = isImage ? ".regionImage-selected" : ".region-selected";
	}

	var borderWidth = FC.Utils.GetCssClassValue(cssClass, "border-width");
	if (!borderWidth) {
		borderWidth = 0;
	} else {
		borderWidth = parseInt(borderWidth.replace('px', ''));
	}

	var borderColor = FC.Utils.GetCssClassValue(cssClass, "border-color");
	var backgroundColor = FC.Utils.GetCssClassValue(cssClass, "background-color");

	if (!backgroundColor) {
		backgroundColor = "rgba(0, 0, 0, 0)";
	}

	this.target.checkAndCorrectRectangles();

	layer.drawPolygon(this.rectangles(), borderWidth, borderColor, backgroundColor);
};

FC.Viewmodels.ActionZones.RegionBodyZone.handlerRemovalKey = "regionMove";

FC.Viewmodels.ActionZones.RegionBodyZone.prototype.mouseDown = function (point) {
	var zone = this;

	if (!this.pointIsInside(point)) {
		return false;
	}

	zone.selectTarget();

	zone.startPoint = point;
	zone.regionMoved = false;

	var mouseMove = function (data) { return zone.mouseMove(zone, data); };
	var mouseUp = function () { return zone.mouseUp(zone); };

	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
	docEditor.addOnMouseUp(mouseUp.bind(this), 100, FC.Viewmodels.ActionZones.RegionBodyZone.handlerRemovalKey);
	docEditor.addOnMouseLeave(mouseUp.bind(this), 100, FC.Viewmodels.ActionZones.RegionBodyZone.handlerRemovalKey);
	docEditor.addOnMouseMove(mouseMove.bind(this), 100, FC.Viewmodels.ActionZones.RegionBodyZone.handlerRemovalKey);

	return true;
};

FC.Viewmodels.ActionZones.RegionBodyZone.prototype.mouseUp = function (zone) {
	zone.startPoint = null;

	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
	docEditor.removeOnMouseUp(FC.Viewmodels.ActionZones.RegionBodyZone.handlerRemovalKey);
	docEditor.removeOnMouseLeave(FC.Viewmodels.ActionZones.RegionBodyZone.handlerRemovalKey);
	docEditor.removeOnMouseMove(FC.Viewmodels.ActionZones.RegionBodyZone.handlerRemovalKey);

	if (zone.target.resized) {
		zone.target.updateText();
	}

	return !zone.regionMoved;
};

FC.Viewmodels.ActionZones.RegionBodyZone.prototype.mouseMove = function (zone, point) {
	if (point.x < 0) {
		point.x = 0;
	}

	if (point.y < 0) {
		point.y = 0;
	}

	var deltaX = Math.round(point.x - zone.startPoint.x);
	var deltaY = Math.round(point.y - zone.startPoint.y);

	zone.startPoint = point;

	if (deltaX !== 0 || deltaY !== 0) {
		zone.regionMoved = true;
		zone.target.moveRegion(deltaX, deltaY);
	}

	zone.page.draw();

	return !zone.regionMoved;
};