﻿FC.Viewmodels.ActionZones.RowHeaderZone = function rowHeaderZone(rectangles, target, page, depth, row) {
	var maxIndex = ko.pureComputed(function () {
		return this.target.rows().length;
	}, this);

	var zoneRect = ko.pureComputed(function () {
		var newRects = [];
		var rects = ko.utils.unwrapObservable(rectangles);

		var text = "[" + maxIndex() + "]";
        var textLength = page.getLayer(1).measureText(text, 11, "SourceSansPro") +
			2 * FC.Viewmodels.ActionZones.zoneConstants.rowHeaderHorizontalPadding / page.zoom();

		for (var i = 0; i < rects.length; i++) {
			newRects.push(new FC.Viewmodels.ImageEditor.Rectangle(
			{
				x1: rects[i].x1() - textLength,
				y1: row.topSeparator(),
				x2: rects[i].x1(),
				y2: row.bottomSeparator()
			}));
		}

		return newRects;
	});

	FC.Viewmodels.ActionZones.BaseZone.call(this, zoneRect, target, page, FC.Viewmodels.ActionZones.zoneActivationType.selected);

	this.sourceRectangles = rectangles;
	this.cursorStyle(FC.Viewmodels.ActionZones.cursorTypes.pointer);
	this.row = row;
	this._id = FC.Viewmodels.ActionZones.RowHeaderZone.globalCounter++;

	this.depth(depth + 10);

	this.isHovering = ko.observable(false);
};

FC.Viewmodels.ActionZones.RowHeaderZone.prototype = Object.create(FC.Viewmodels.ActionZones.BaseZone.prototype);

FC.Viewmodels.ActionZones.RowHeaderZone.prototype.draw = function (layerIndex) {
	if (!this.isActive() || layerIndex === 0) {
		return;
	}

	var color = FC.Utils.GetCssClassValue(".table_columnHeader", "background-color");
	var colorDeleter = FC.Utils.GetCssClassValue(".table_columnHeader-deleter", "background-color");
	var layer = this.page.getLayer(1);

	var rect = this.rectangles()[0];
	var tableRect = this.sourceRectangles()[0];

	var dataBound = this.row.dataBound();
	var deleting = dataBound && this.isHovering();

	if (deleting || (!dataBound && this.isHovering())) {
		layer.drawFilledRectangle(rect, colorDeleter);
	} else {
		layer.drawFilledRectangle(rect, color);
	}

	var fontSize;
	var fontFace;
	var text;
	if (dataBound) {
		if (deleting) {
			fontSize = 10;
			fontFace = "abbyy-icons";
			text = "\ue923";
		} else {
			var rows = this.target.table().rows();
			

			text = "[" + (rows.indexOf(this.row.rowModel()) + 1) + "]";
			fontSize = 11;
            fontFace = "SourceSansPro";
		}

		var textSize = layer.measureText(text, fontSize, fontFace);

		var point = new FC.Viewmodels.ImageEditor.Point(
			(rect.x1() + rect.x2() - textSize) / 2,
			(rect.y1() + rect.y2() + 12 / this.page.zoom()) / 2);

		layer.drawText(point, text, "rgba(255,255,255,1)", fontSize, fontFace);
	} else {
		var rectSize = 8;
		var point1 = new FC.Viewmodels.ImageEditor.Point(
			(rect.x1() + rect.x2() - rectSize / this.page.zoom()) / 2,
			(rect.y1() + rect.y2() - 2 / this.page.zoom()) / 2);
		var point2 = new FC.Viewmodels.ImageEditor.Point(
			(rect.x1() + rect.x2() + rectSize / this.page.zoom()) / 2,
			(rect.y1() + rect.y2() + 2 / this.page.zoom()) / 2);
		var point3 = new FC.Viewmodels.ImageEditor.Point(
			(rect.x1() + rect.x2() - 2 / this.page.zoom()) / 2,
			(rect.y1() + rect.y2() - rectSize / this.page.zoom()) / 2);
		var point4 = new FC.Viewmodels.ImageEditor.Point(
			(rect.x1() + rect.x2() + 2 / this.page.zoom()) / 2,
			(rect.y1() + rect.y2() + rectSize / this.page.zoom()) / 2);

		layer.drawFilledRectangle(FC.Viewmodels.ImageEditor.Rectangle.getRectangleFromPoints(point1, point2), "#ffffff");
		layer.drawFilledRectangle(FC.Viewmodels.ImageEditor.Rectangle.getRectangleFromPoints(point3, point4), "#ffffff");
	}
	
	var lineWidth = 1;
	var lineColor = FC.Utils.GetCssClassValue(".regionParent-selected", "border-color");
	if (rect.y1() > tableRect.y1()) {
		layer.drawLine(new FC.Viewmodels.ImageEditor.Point(rect.x1(), rect.y1()), new FC.Viewmodels.ImageEditor.Point(rect.x2(), rect.y1()), lineWidth, lineColor);
	}
	if (rect.y2() < tableRect.y2()) {
		layer.drawLine(new FC.Viewmodels.ImageEditor.Point(rect.x1(), rect.y2() - 1 / this.page.zoom()), new FC.Viewmodels.ImageEditor.Point(rect.x2(), rect.y2() - 1 / this.page.zoom()), lineWidth, lineColor);
	}
};

FC.Viewmodels.ActionZones.RowHeaderZone.prototype.mouseDown = function (point) {
	if (!this.pointIsInside(point)) {
		return false;
	}

	if (this.row.dataBound()) {
		this.target.unbindRow(this.row, true);
	} else {
		this.target.bindRow(this.row);
	}

	return true;
};

FC.Viewmodels.ActionZones.RowHeaderZone.prototype.onHover = function (point) {
	var self = this;

	if (!self.pointIsInside(point)) {
		return;
	}

	var removalKey = "rowHeader" + self._id;
	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();

	var onMouseMove = function (p) {
		if (!self.pointIsInside(p)) {
			docEditor.removeOnMouseMove(removalKey);
			self.isHovering(false);
			self.page.draw();
		}
	}

	if (!this.isHovering()) {
		this.isHovering(true);
		docEditor.addOnMouseMove(onMouseMove.bind(this), 100, removalKey);
		this.page.draw();
	}
};

FC.Viewmodels.ActionZones.RowHeaderZone.globalCounter = 0;