﻿FC.Viewmodels.ActionZones.TableBodyEditableZone = function tableBodyEditableZone(rectangles, target, page, depth) {
	FC.Viewmodels.ActionZones.GroupBodyEditableZone.call(this, rectangles, target, page, depth);

	this.separatorCreator = new FC.Viewmodels.DocumentEditor.TableSeparatorCreator(page, target);

	this.separatorCreationForbidden = ko.observable(false);

	var self = this;
	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();

	this.cursorStyle = ko.computed(function () {
		if (docEditor.extender.horizontalSeparatorDrawActivated() && !self.separatorCreationForbidden()) {
			return FC.Viewmodels.ActionZones.cursorTypes.createHorizontalSeparator;
		}

		if (docEditor.extender.verticalSeparatorDrawActivated() && !self.separatorCreationForbidden()) {
			return FC.Viewmodels.ActionZones.cursorTypes.createVerticalSeparator;
		}

		if (self.separatorCreationForbidden() &&
			(docEditor.extender.horizontalSeparatorDrawActivated() || docEditor.extender.verticalSeparatorDrawActivated())) {

			return FC.Viewmodels.ActionZones.cursorTypes.separatorCreationForbidden;
		}

		return FC.Viewmodels.ActionZones.cursorTypes.crosshair;
	});

	this.modifiedCursorStyle = FC.Viewmodels.ActionZones.cursorTypes.modifyRegion;

	this.page.subscriptions.push({
		key: this.target,
		value: this.cursorStyle
	});

	this.page.subscriptions.push({
		key: this.target,
		value: docEditor.extender.verticalSeparatorDrawActivated.subscribe(function (newVal) {
			if (!newVal) {
				self.page.directDrawer.clear();
				self.page.draw(1);
			}
		})
	});

	this.page.subscriptions.push({
		key: this.target,
		value: docEditor.extender.horizontalSeparatorDrawActivated.subscribe(function (newVal) {
			if (!newVal) {
				self.page.directDrawer.clear();
				self.page.draw(1);
			}
		})
	});
};

FC.Viewmodels.ActionZones.TableBodyEditableZone.prototype = Object.create(FC.Viewmodels.ActionZones.GroupBodyEditableZone.prototype);

FC.Viewmodels.ActionZones.TableBodyEditableZone.prototype.mouseDown = function (point) {
	if (!this.pointIsInside(point)) {
		return false;
	}

	this.selectTarget();

	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
	if (!docEditor.extender.horizontalSeparatorDrawActivated() && !docEditor.extender.verticalSeparatorDrawActivated()) {
		this.separatorCreator.startSeparatorDrawing(point);
		return true;
	}

	if (docEditor.extender.horizontalSeparatorDrawActivated()) {
		this.separatorCreator.addHorizontalSeparator(point.y);
		this.page.draw();
		return true;
	}

	this.separatorCreator.addVerticalSeparators(point.x);
	this.page.draw();
	return true;
};

FC.Viewmodels.ActionZones.TableBodyEditableZone.prototype.onHover = function(point) {
	var self = this;

	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
	if (!(docEditor.extender.horizontalSeparatorDrawActivated() || docEditor.extender.verticalSeparatorDrawActivated()) ||
		!self.pointIsInside(point)) {
		return;
	}

	this.page.directDrawer.clear();

	var rect = this.target.getExternalRect();
	var point1 = null;
	var point2 = null;

	if (docEditor.extender.horizontalSeparatorDrawActivated()) {
		this.separatorCreationForbidden(!this.separatorCreator.isHorizontalSeparatorCreatable(point.y));

		point1 = new FC.Viewmodels.ImageEditor.Point(rect.x1, point.y);
		point2 = new FC.Viewmodels.ImageEditor.Point(rect.x2, point.y);
	}

	if (docEditor.extender.verticalSeparatorDrawActivated()) {
		this.separatorCreationForbidden(!this.separatorCreator.isVerticalSeparatorCreatable(point.x));

		point1 = new FC.Viewmodels.ImageEditor.Point(point.x, rect.y1);
		point2 = new FC.Viewmodels.ImageEditor.Point(point.x, rect.y2);
	}

	this.page.directDrawer.addDrawingCommand(function(layer) {
		layer.drawLine(point1, point2, 2, !self.separatorCreationForbidden() ? "#48ba00" : "#c60c30", [10, 20]);
	});

	this.page.draw(1);
};