﻿FC.Viewmodels.ActionZones.VerticalSeparatorZone = function (rectangles, target, page, depth, columnOnLeft, columnOnRight) {
	var columnOnLeftBorder = columnOnLeft ? columnOnLeft.rightPos : null;
	var columnOnRightBorder = columnOnRight ? columnOnRight.leftPos : null;

	var zoneRect = ko.pureComputed(function () {
		var newRects = [];
		var rects = ko.utils.unwrapObservable(rectangles);

		var threshold = FC.Viewmodels.ActionZones.zoneConstants.verticalSeparatorThreshold * page.zoom() < 2
			? 2 / page.zoom()
			: FC.Viewmodels.ActionZones.zoneConstants.verticalSeparatorThreshold;

		for (var i = 0; i < rects.length; i++) {
			newRects.push(new FC.Viewmodels.ImageEditor.Rectangle(
			{
				x1: Math.floor((columnOnLeftBorder ? columnOnLeftBorder() : columnOnRightBorder()) - threshold),
				y1: rects[i].y1(),
				x2: Math.ceil((columnOnLeftBorder ? columnOnLeftBorder() : columnOnRightBorder()) + threshold),
				y2: rects[i].y2()
			}));
		}

		return newRects;
	});

	FC.Viewmodels.ActionZones.BaseZone.call(this, zoneRect, target, page, FC.Viewmodels.ActionZones.zoneActivationType.always);

	this._id = FC.Viewmodels.ActionZones.VerticalSeparatorZone.globalCounter++;

	this.sourceRectangles = rectangles;
	this.startPoint = null;
	this.columnOnLeftBorder = columnOnLeftBorder;
	this.columnOnRightBorder = columnOnRightBorder;
	this.columnOnLeft = columnOnLeft;
	this.columnOnRight = columnOnRight;
	this.isHovering = ko.observable(false);

	this.movable = ko.computed(function () {
		var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
		return !docEditor.extender
			.horizontalSeparatorDrawActivated() &&
			!docEditor.extender.verticalSeparatorDrawActivated();
	});

	this.page.subscriptions.push({
		key: this.target,
		value: this.movable
	});

	this.depth = ko.computed(function () {
		var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
		if (docEditor.extender.horizontalSeparatorDrawActivated() ||
			docEditor.extender.verticalSeparatorDrawActivated()) {
			return -1;
		} else {
			return depth + 100;
		}
	});

	this.page.subscriptions.push({
		key: this.target,
		value: this.depth
	});

	this.cursorStyle = ko.computed(function () {
		var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
		if (docEditor.extender.deleteSeparatorActivated()) {
			return FC.Viewmodels.ActionZones.cursorTypes.deleteSeparator;
		}

		return FC.Viewmodels.ActionZones.cursorTypes.moveVerticalSeparator;
	});

	this.page.subscriptions.push({
		key: this.target,
		value: this.cursorStyle
	});
};

FC.Viewmodels.ActionZones.VerticalSeparatorZone.prototype = Object.create(FC.Viewmodels.ActionZones.BaseZone.prototype);

FC.Viewmodels.ActionZones.VerticalSeparatorZone.prototype.draw = function (layerIndex) {
	if (layerIndex === 0 && this.target.selected() || layerIndex === 1 && !this.target.selected()) {
		return;
	}

	var layer = this.page.getLayer(this.target.selected() ? 1 : 0);
	var cssClass = ".regionParent-default";
	if (this.target.selected() && this.movable()) {
		if (this.isHovering()) {
			cssClass = ".regionParent-selected-hover";
		} else {
			cssClass = ".regionParent-selected";
		}
	}

	var lineWidth = 2;
	var lineColor = FC.Utils.GetCssClassValue(cssClass, "border-color");

	var x = this.columnOnLeftBorder ? this.columnOnLeftBorder() : this.columnOnRightBorder();
	var rects = this.sourceRectangles();
	for (var i = 0; i < rects.length; i++) {
		layer.drawLine(
			new FC.Viewmodels.ImageEditor.Point(x, rects[i].y1()),
			new FC.Viewmodels.ImageEditor.Point(x, rects[i].y2()),
			lineWidth,
			lineColor);
	}
};

FC.Viewmodels.ActionZones.VerticalSeparatorZone.handlerRemovalKey = "VerticalSeparatorMove";

FC.Viewmodels.ActionZones.VerticalSeparatorZone.prototype.mouseDown = function (point) {
	if (!this.pointIsInside(point) || !this.movable()) {
		return false;
	}

	this.selectTarget();

	this.startPoint = point;
	var zone = this;

	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();

	if (docEditor.extender.deleteSeparatorActivated()) {
		this.target.deleteVerticalSeparator(this);
		return true;
	}

	var mouseMove = function (p) {
		if (p.x < 0) {
			p.x = 0;
		}

		if (p.y < 0) {
			p.y = 0;
		}

		var delta = p.x - zone.startPoint.x;

		zone.startPoint = p;
		zone.moveSeparator(zone, delta);
		FC.Viewmodels.EventStack.addEvent();

		zone.page.draw();
	};

	var mouseUp = function () {
		zone.startPoint = null;

		docEditor.removeOnMouseUp(FC.Viewmodels.ActionZones.VerticalSeparatorZone.handlerRemovalKey);
		docEditor.removeOnMouseLeave(FC.Viewmodels.ActionZones.VerticalSeparatorZone.handlerRemovalKey);
		docEditor.removeOnMouseMove(FC.Viewmodels.ActionZones.VerticalSeparatorZone.handlerRemovalKey);
	};


	docEditor.addOnMouseUp(mouseUp.bind(this), 100, FC.Viewmodels.ActionZones.VerticalSeparatorZone.handlerRemovalKey);
	docEditor.addOnMouseLeave(mouseUp.bind(this), 100, FC.Viewmodels.ActionZones.VerticalSeparatorZone.handlerRemovalKey);
	docEditor.addOnMouseMove(mouseMove.bind(this), 100, FC.Viewmodels.ActionZones.VerticalSeparatorZone.handlerRemovalKey);


	return true;
};

FC.Viewmodels.ActionZones.VerticalSeparatorZone.prototype.moveSeparator = function (zone, delta) {
	var currentSeparator = zone.columnOnLeftBorder ? zone.columnOnLeftBorder() : zone.columnOnRightBorder();
	delta = FC.Viewmodels.ActionZones.VerticalSeparatorZone.findDeltaWithRestrictions(zone.target, delta, currentSeparator);

	var newSeparator = currentSeparator + delta;
	if (zone.columnOnLeftBorder) {
		zone.columnOnLeftBorder(newSeparator);
	}

	if (zone.columnOnRightBorder) {
		zone.columnOnRightBorder(newSeparator);
	}

	zone.target.resizeCellsX(currentSeparator, newSeparator);
};

FC.Viewmodels.ActionZones.VerticalSeparatorZone.prototype.onHover = function (point) {
	var self = this;

	if (!self.pointIsInside(point)) {
		return;
	}

	var removalKey = "verticalSepatator" + self._id;
	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();

	var onMouseMove = function (p) {
		if (!self.pointIsInside(p)) {
			docEditor.removeOnMouseMove(removalKey);
			self.isHovering(false);
			self.page.draw();
		}
	}

	if (!this.isHovering()) {
		this.isHovering(true);
		docEditor.addOnMouseMove(onMouseMove.bind(this), 100, removalKey);
		this.page.draw();
	}
};

FC.Viewmodels.ActionZones.VerticalSeparatorZone.findDeltaWithRestrictions = function (tableRegion, delta, currentX) {
	var x1 = tableRegion.rectangles()[0].x1();
	var x2 = tableRegion.rectangles()[0].x2();
	var columns = tableRegion.columns();
	var allSeparators = [x1];
	var i;

	for (i = 0; i < columns.length; i++) {
		if (columns[i].leftPos() > x1) {
			allSeparators.push(columns[i].leftPos());
		}

		allSeparators.push(columns[i].rightPos());
	}

	if (allSeparators[allSeparators.length - 1] < x2) {
		allSeparators.push(x2);
	}

	var previousSeparator = allSeparators[0];
	var nextSeparator = allSeparators[allSeparators.length - 1];

	for (i = 0; i < allSeparators.length; i++) {
		if (allSeparators[i] > previousSeparator && allSeparators[i] < currentX) {
			previousSeparator = allSeparators[i];
		}

		if (allSeparators[i] < nextSeparator && allSeparators[i] > currentX) {
			nextSeparator = allSeparators[i];
		}
	}

	if (currentX + delta < previousSeparator + FC.Viewmodels.ActionZones.zoneConstants.verticalSeparatorThreshold) {
		delta = previousSeparator + FC.Viewmodels.ActionZones.zoneConstants.verticalSeparatorThreshold - currentX;
	}

	if (currentX + delta > nextSeparator - FC.Viewmodels.ActionZones.zoneConstants.verticalSeparatorThreshold) {
		delta = nextSeparator - FC.Viewmodels.ActionZones.zoneConstants.verticalSeparatorThreshold - currentX;
	}

	return delta;
};

FC.Viewmodels.ActionZones.VerticalSeparatorZone.globalCounter = 0;