﻿FC.Viewmodels.BatchEditorExtention = {
    docTemplates: null,
    hasDocumentsNeedRecognize: function () {
        var selectedDocuments = [];
        if (window.batchLayout.batchLayoutViewModel && window.batchLayout.batchLayoutViewModel.BatchEditor) {
            selectedDocuments = window.batchLayout.batchLayoutViewModel.BatchEditor.selectedDocuments() || [];
        }
        return selectedDocuments.filter(function (doc) {
            return doc.Status() === FC.BatchEditor.Document.Status.NeedRecognize;
        }).length > 0;
    },
    documentsCanUpdate: function () {
        var selectedDocuments = [];
        if (window.batchLayout.batchLayoutViewModel && window.batchLayout.batchLayoutViewModel.BatchEditor) {
            selectedDocuments = window.batchLayout.batchLayoutViewModel.BatchEditor.selectedDocuments() || [];
        }
        return selectedDocuments.filter(function (doc) {
            return doc.NewTemplateVersionAvailible();
        }).length > 0;
    },
    getDocumentToolbarItems: function () {
        var self = this;

        var selectedDocuments = [];
        if (window.batchLayout.batchLayoutViewModel && window.batchLayout.batchLayoutViewModel.BatchEditor) {
            selectedDocuments = window.batchLayout.batchLayoutViewModel.BatchEditor.selectedDocuments() || [];
        }

        var templateName = null;
        var documentsHaveSameTemplate = false;

        var i;
        if (selectedDocuments.length > 0) {

            templateName = selectedDocuments[0].TemplateName();
            documentsHaveSameTemplate = true;
            for (i = 1; i < selectedDocuments.length; i++) {
                if (selectedDocuments[i].TemplateName() !== templateName) {
                    documentsHaveSameTemplate = false;
                    break;
                }
            }
        }

        var documentTemplates = self.docTemplates || [];

        var templateSubItems = [];
        var isCurrentTemplate = documentsHaveSameTemplate && templateName === FC.BatchEditor.batchEditorResources.unprocessedTemplate();
        templateSubItems.push({
            text: FC.Environment.Localization.GlobalResources.UnknownDocumentTemplate,
            action: function () {
                self.setDocumentTemplate(FC.Environment.Localization.GlobalResources.UnknownDocumentTemplate);
            },
            htmlClass: 'italic' + (isCurrentTemplate ? ' current' : ''),
            isCurrent: isCurrentTemplate,
            group: 'changeTemplateOperation',
            subItems: []
        });

        for (i = 0; i < documentTemplates.length; i++) {
            var name = documentTemplates[i].Name();
            isCurrentTemplate = documentsHaveSameTemplate && templateName === name;

            templateSubItems.push({
                text: name,
                action: function () {
                    if (this.isCurrent) {
                        return true;
                    }

                    $('.headerTooltipAreaAbsolutizer').remove();
                    self.setDocumentTemplate(this.text);
                },
                htmlClass: '' + (isCurrentTemplate ? 'current' : ''),
                isCurrent: isCurrentTemplate,
                group: 'changeTemplateOperation',
                subItems: []
            });
        }
    },

    getPageToolbarItems: function () {

        var self = this;
        var subItems = [];

        var selectedPages = [];
        if (window.batchLayout.batchLayoutViewModel && window.batchLayout.batchLayoutViewModel.BatchEditor) {
            selectedPages = window.batchLayout.batchLayoutViewModel.BatchEditor.selectedPages() || [];
        }

        if (selectedPages.length > 0) {
            var selectedPagesDocTemplateIds = selectedPages.map(function (page) {
                var parentDoc = self.getPageParent(page);
                return parentDoc ? parentDoc.Model.TemplateId() : null;
            });
            var firstPageDocTemplateId = selectedPagesDocTemplateIds[0];
            if (selectedPagesDocTemplateIds.filter(function (id) { return id !== firstPageDocTemplateId }).length === 0) {

                var selectedPagesTemplates = selectedPages.map(function (selectedPage) { return selectedPage.TemplateName() });
                var firstSelectedPageTemplate = selectedPagesTemplates[0];
                var allPagesWithSameTemplate = selectedPagesTemplates
                    .filter(function (template) { return template !== firstSelectedPageTemplate }).length ===
                    0;
                for (var i = 0; i < self.docTemplates.length; i++) {
                    if (self.docTemplates[i].Id() === firstPageDocTemplateId) {
                        var sectionNames = self.docTemplates[i].SectionNames();

                        for (var j = 0; j < sectionNames.length; j++) {
                            var name = sectionNames[j];
                            var isCurrentTemplate = allPagesWithSameTemplate && name === firstSelectedPageTemplate;

                            subItems.push(
                                {
                                    text: name,
                                    action: function () {
                                        $('.headerTooltipAreaAbsolutizer').remove();
                                        self.setPageTemplate(this.text);
                                    },
                                    htmlClass: isCurrentTemplate ? 'current' : '',
                                    isCurrent: isCurrentTemplate,
                                    group: 'chooseSectionOperation',
                                    subItems: []
                                });
                        }
                    }
                }
            }
        }
        var result = [
            {
                text: FC.Environment.Localization.GlobalResources.ChangePageSection_Title,
                action: function () { return true; },
                htmlClass: 'batchEditor__function batchEditor__function__choose-section',
                group: 'pageSectionOperation',
                index: 3,
                mustHaveSubItems: true,
                subItems: subItems
            }
        ];

        return result;

    },

    getDocumentsContextMenu: function (self, context) {
        if (!self.docTemplates) {
            return [];
        }

        var templatesSubMenu = [
            {
                title: FC.Environment.Localization.GlobalResources.UnknownDocumentTemplate,
                action: function () {

                    // this тут - ContextMenuItem
                    self.setDocumentTemplate(FC.Environment.Localization.GlobalResources.UnknownDocumentTemplate);
                },
                enabled: self.getTemplateChangeEnabled
            }];
        for (var i = 0; i < self.docTemplates.length; i++) {
            var name = self.docTemplates[i].Name();

            templatesSubMenu.push(
                {
                    title: name,
                    action: function () {

                        // this тут - ContextMenuItem
                        self.setDocumentTemplate(this.Title());
                    },
                    enabled: self.getTemplateChangeEnabled
                });
        }

        var additionalMenuItems = [
            // отменить			
            {
                order: 10,
                title: FC.BatchEditor.batchEditorResources.undo(),
                iconClass: 'icon  icon--undo',
                action: window.batchLayout.batchLayoutViewModel.BatchEditor.extender.undo,
                hotKey: { ctrl: true, keyCode: 90, dontBubble: true, title: 'Ctrl-Z' },
                visible: ko.observable(false)
            },

            // вернуть
            {
                order: 11,
                title: FC.BatchEditor.batchEditorResources.redo(),
                iconClass: 'icon  icon--redo',
                action: window.batchLayout.batchLayoutViewModel.BatchEditor.extender.redo,
                hotKey: { ctrl: true, keyCode: 89, dontBubble: true, title: 'Ctrl-Y' },
                visible: ko.observable(false)
            },

            // объединить документы
            {
                order: 33,
                title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.BatchEditor.batchEditorResources.merge(),
                    FC.Environment.Localization.NonLocalizedHotKeys.MergeHotkey),
                iconClass: 'icon  icon--merge',
                action: window.batchLayout.batchLayoutViewModel.BatchEditor.mergeDocuments,
                enabled: function () {
                    return (window.batchLayout.batchLayoutViewModel.BatchEditor.selectedElementTypes() ==
                        FC.BatchEditor.SelectionType.DOCS) &&
                        window.batchLayout.batchLayoutViewModel.BatchEditor.selectedDocuments().length > 1;
                },
                visible: FC.BatchEditor.batchEditorSettings.documentMergeEnabled
            },
            {
                order: 41,
                isSeparator: true
            },
            {
                order: 22,
                title: FC.Environment.Localization.GlobalResources.ChangeTemplateButton,
                subMenu: templatesSubMenu,
                enabled: self.getTemplateChangeEnabled
            },
            {
                order: 23,
                title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.RemoveTemplate,
                    FC.Environment.Localization.NonLocalizedHotKeys.ClearAnalyzingResultsHotkey),
                enabled: self.getTemplateChangeEnabled,
                action: function () { self.removeTemplate(self); }
            },
            {
                order: 24,
                title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.RecognizeButton,
                    FC.Environment.Localization.NonLocalizedHotKeys.RecognizeHotKey),
                svgId: '#icon--recognize-20',
                svgClass: 'recognizeIcon',
                enabled: self.hasDocumentsNeedRecognize,
                action: self.recognizeSelectedDocuments.bind(self)
            },
            {
                order: 25,
                title: FC.Environment.Localization.GlobalResources.UpdateTemplate,
                enabled: self.documentsCanUpdate,
                action: self.updateSelectedDocumentsTemplate.bind(self)
            },
            {
                order: 28,
                title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.ConfirmDocumentDefinition,
                    FC.Environment.Localization.NonLocalizedHotKeys.ConfirmClassificationHotKey),
                svgId: '#icon--check-122',
                svgClass: 'confirmClassification',
                enabled: function () {
                    return window.batchLayout.batchLayoutViewModel.BatchEditor.selectedDocuments().filter(function (doc) {
                        return doc.HasClassificationWarning();
                    }).length >
                        0;
                },
                action: function () { self.confirmDocumentDefinition(self); }
            }
        ];

        if (context) {
            additionalMenuItems.push({
                order: 21,
                isSeparator: true
            });
        }

        return additionalMenuItems;
    },

    getPageParent: function (page) {
        var documents = window.batchLayout.batchLayoutViewModel.BatchEditor.elements();

        var parentDoc = null;
        for (var i = 0; i < documents.length; i++) {
            var childs = documents[i].ChildNodes();

            for (var j = 0; j < childs.length; j++) {
                if (childs[j].Model._id === page._id) {
                    parentDoc = documents[i];
                    break;
                }
            }

            if (parentDoc) {
                break;
            }
        }

        return parentDoc;
    },

    getPagesContextMenu: function (self) {
        var templatesSubMenu = [];

        var additionalMenuItems = [
            // удалить страницы
            {
                order: 51,
                title: FC.BatchEditor.batchEditorResources.deleteButton(),
                iconClass: 'icon  icon--delete',
                action: function () {
                    var viewModels = window.batchLayout.batchLayoutViewModel.BatchEditor.elements();
                    var docIndex;

                    if (window.batchLayout.batchLayoutViewModel.BatchEditor.selectedDocuments().length != 0) {
                        for (docIndex = 0; docIndex < viewModels.length; docIndex++) {
                            var docViewModel = viewModels[docIndex];
                            if (docViewModel.selected())
                                docViewModel.selectPages();
                        }
                    }

                    var pageCount = 0;
                    for (docIndex = 0; docIndex < viewModels.length; docIndex++) {
                        var pages = viewModels[docIndex].ChildNodes();
                        for (var j = 0; j < pages.length; j++) {
                            if (pages[j].selected()) {
                                pageCount++;
                            }
                        }
                    }

                    var message = pageCount > 1
                        ? FC.Environment.Localization.GlobalResources.DeletePages.replace("{0}", pageCount)
                        : FC.Environment.Localization.GlobalResources.DeletePage;

                    FC.Utils.Notification.ConfirmMiniWindow(
                        message,
                        "",
                        FC.Environment.Localization.GlobalResources.DeleteButton,
                        FC.Environment.Localization.GlobalResources.CancelButton,
                        window.batchLayout.batchLayoutViewModel.BatchEditor.deletePages,
                        function () { },
                        false);
                },
                enabled: function () {
                    return window.batchLayout.batchLayoutViewModel.BatchEditor.selectedElementTypes() !==
                        FC.BatchEditor.SelectionType.NONE;
                },
                visible: FC.BatchEditor.batchEditorSettings.documentDeletingEnabled
            },
            {
                order: 22,
                title: FC.Environment.Localization.GlobalResources.SelectSectionTemplate,
                action: function () { return true; },
                subMenu: templatesSubMenu,
                enabled: function () {

                    return (templatesSubMenu && templatesSubMenu.length > 0)
                        && window.batchLayout.batchLayoutViewModel.BatchEditor.selectedPages().length > 0;
                }
            },
            {
                order: 23,
                title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.RemoveTemplate,
                    FC.Environment.Localization.NonLocalizedHotKeys.ClearAnalyzingResultsHotkey),
                enabled: function () {
                    var selectedPages = window.batchLayout.batchLayoutViewModel.BatchEditor.selectedPages();
                    var documents = window.batchLayout.batchLayoutViewModel.Documents();
                    var checkedDocs = [];
                    var hasKnownDocs = false;
                    for (var i = 0; i < selectedPages.length; i++) {
                        var selectedPageDocId = selectedPages[i].ParentDocumentId;
                        if (checkedDocs.indexOf(selectedPageDocId) > 0) continue;
                        checkedDocs.push(selectedPageDocId);
                        for (var j = 0; j < documents.length; j++) {
                            var doc = documents[j];
                            if (doc.Id() === selectedPageDocId && !doc.docIsUnknown() && !doc.docIsUnprocessed()) {
                                hasKnownDocs = true;
                                break;
                            }
                        }
                        if (hasKnownDocs) break;
                    }

                    return hasKnownDocs;
                },
                action: function () { self.setPageTemplate(FC.Environment.Localization.GlobalResources.UnknownTemplate); }
            },
            {
                order: 24,
                title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.ConfirmPageSection,
                    FC.Environment.Localization.NonLocalizedHotKeys.ConfirmClassificationHotKey),
                svgId: '#icon--check-122',
                svgClass: 'confirmClassification',
                enabled: function () {
                    return window.batchLayout.batchLayoutViewModel.BatchEditor.selectedPages().filter(function (page) {
                        return page.HasClassificationWarning();
                    }).length >
                        0;
                },
                action: function () { self.confirmPageSection(); }
            },
            {
                order: 30,
                title: FC.Environment.Localization.GlobalResources.Rotate,
                action: function () {
                },
                subMenu: [{
                    order: 30,
                    title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.RotateClockwise,
                        FC.Environment.Localization.NonLocalizedHotKeys.RotateClockwiseHotkey),
                    action: function () {
                        self.rotatePage(90);
                    },
                    enabled: function () { return window.batchLayout.batchLayoutViewModel.BatchEditor.selectedPages().length > 0 }
                },
                {
                    order: 31,
                    title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.RotateCounterClockwise,
                        FC.Environment.Localization.NonLocalizedHotKeys.RotateCounterClockwiseHotkey),
                    action: function () {
                        self.rotatePage(270);
                    },
                    enabled: function () { return window.batchLayout.batchLayoutViewModel.BatchEditor.selectedPages().length > 0 }
                },
                {
                    order: 32,
                    title: FC.Environment.Localization.GlobalResources.RotateUpsideDown,
                    action: function () {
                        self.rotatePage(180);
                    },
                    enabled: function () { return window.batchLayout.batchLayoutViewModel.BatchEditor.selectedPages().length > 0 }
                }]
            }];

        if (!self.docTemplates) {
            return additionalMenuItems;
        }

        var parentDocs = window.batchLayout.batchLayoutViewModel.BatchEditor.selectedPages().map(function (p) {
            var doc = self.getPageParent(p);

            if (!doc) {
                return null;
            }

            return {
                docId: doc.Model.Id(),
                templateId: doc.Model.TemplateId(),
                templateName: doc.Model.TemplateName()
            };
        }).filter(function (dt) { return !!dt; });

        if (!parentDocs ||
            parentDocs.length === 0) {
            // нет шаблонов
            return additionalMenuItems;
        }

        // проверим, что шаблоны документов выделенных страниц одинаковые и не unknown
        for (var i = 0; i < parentDocs.length; i++) {
            for (var j = i + 1; j < parentDocs.length; j++) {
                if (parentDocs[i].docId !== parentDocs[j].docId ||
                    parentDocs[i].templateName === FC.BatchEditor.batchEditorResources.unprocessedTemplate()) {
                    return additionalMenuItems;
                }
            }
        }

        for (var i = 0; i < self.docTemplates.length; i++) {
            if (!parentDocs[0].templateName && self.docTemplates[i].Id() == parentDocs[0].templateId ||
                parentDocs[0].templateName && self.docTemplates[i].Name() == parentDocs[0].templateName) {
                var sectionNames = self.docTemplates[i].SectionNames();

                for (var j = 0; j < sectionNames.length; j++) {
                    templatesSubMenu.push(
                        {
                            title: sectionNames[j],
                            action: function () {
                                // this тут - ContextMenuItem
                                self.setPageTemplate(this.Title());
                            },
                            enabled: function () {
                                return true;
                            }
                        });
                }
            }
        }

        return additionalMenuItems;
    },

    getAdditionalContextMenuItems: function (menuType, context) {
        var self = this;

        switch (menuType) {
            // документы
            case 1: return this.getDocumentsContextMenu(self, context);

            // страницы
            case 2: return this.getPagesContextMenu(self);

            default: return [];
        }
    },

    getTemplateChangeEnabled: function () {
        var docs = window.batchLayout.batchLayoutViewModel.BatchEditor.elements();
        for (var docIndex = 0; docIndex < docs.length; docIndex++) {
            if (docs[docIndex].selected()) {
                return true;
            }
        }

        return false;
    },

    setDocumentTemplate: function (templateName) {
        // чтобы можно было отменить
        window.batchLayout.batchLayoutViewModel.BatchEditor.makeDocumentsSnapshot();

        var docs = window.batchLayout.batchLayoutViewModel.BatchEditor.elements();
        docs.forEach(function (doc) {
            if (doc.selected()) {
                doc.Model.setTemplate(templateName);
            }
        });
    },

    setPageTemplate: function (templateName) {
        // чтобы можно было отменить
        window.batchLayout.batchLayoutViewModel.BatchEditor.makeDocumentsSnapshot();
        var documents = window.batchLayout.batchLayoutViewModel.Documents();
        window.batchLayout.batchLayoutViewModel.BatchEditor.selectedPages().forEach(function (selectedPage) {
            if (selectedPage.Name() === templateName) return;
            var docId = selectedPage.ParentDocumentId;
            for (var i = 0; i < documents.length; i++) {
                var doc = documents[i];
                if (doc.Id() === docId && doc.docIsUnknown()) return;
            }
            selectedPage.Name(templateName);
        });
    },

    rotatePage: function (degree) {
        var selectedPages = window.batchLayout.batchLayoutViewModel.BatchEditor.selectedPages();

        for (var i = 0; i < selectedPages.length; i++) {
            selectedPages[i].Rotation((selectedPages[i].Rotation() + degree) % 360);
            selectedPages[i].Rotated(true);
            if (degree % 180 !== 0) {
                var width = selectedPages[i].Width();
                selectedPages[i].Width(selectedPages[i].Height());
                selectedPages[i].Height(width);
            }


            if (selectedPages[i].Name() !== FC.Environment.Localization.GlobalResources.AnnexPage) {
                selectedPages[i].Name(FC.Environment.Localization.GlobalResources.UnknownTemplate);

            }
        }

        //window.batchLayout.batchLayoutViewModel.BatchEditor.adaptateMiniatures();
    },

    removeTemplate: function () {
        // чтобы можно было отменить
        window.batchLayout.batchLayoutViewModel.BatchEditor.makeDocumentsSnapshot();

        this.setDocumentTemplate(FC.BatchEditor.batchEditorResources.unprocessedTemplate());
    },

    confirmPages: function (pages) {
        if (!pages || pages.length == 0) return;
        window.batchApp.extensions.hasChangesEver(true);

        var documents = window.batchLayout.batchLayoutViewModel.Documents();
        var processedDocids = [];
        pages.forEach(function (page) {
            var docId = page.ParentDocumentId;
            page.HasClassificationWarning(false);
            if (processedDocids.indexOf(docId) !== -1) return;
            processedDocids.push(docId);
            var document = documents.filter(function (doc) { return doc.Id() === docId })[0];
            document.classificationConfirmed(true);
        });

        var openedDocument = window.batchLayout.batchLayoutViewModel.UserEditor.documentData();
        var openedDocumentId = openedDocument ? openedDocument.id() : -1;
        var openedDocumentPreview = window.batchLayout.batchLayoutViewModel.Documents().filter(function (doc) {
            return doc.Id() === openedDocumentId;
        })[0];
        if (openedDocumentPreview && !openedDocumentPreview.HasClassificationWarning()) {
            openedDocument.removeClassificationError();
        }
    },

    confirmDocumentDefinition: function () {
        var docs = window.batchLayout.batchLayoutViewModel.BatchEditor.selectedDocuments();
        var pagesToConfirm = [];
        docs.forEach(function (doc) {
            pagesToConfirm = pagesToConfirm.concat(doc.ChildNodes().filter(function (page) {
                return page.HasClassificationWarning();
            }));
        });
        this.confirmPages(pagesToConfirm);

    },

    confirmPageSection: function () {
        var selectedPages = window.batchLayout.batchLayoutViewModel.BatchEditor.selectedPages();
        this.confirmPages(selectedPages.filter(function (page) { return page.HasClassificationWarning(); }));

    },

    recognizeSelectedDocuments: function () {
        var docsToRecognize = window.batchLayout.batchLayoutViewModel.BatchEditor.selectedDocuments().filter(function (doc) {
            return doc.Status() === FC.BatchEditor.Document.Status.NeedRecognize;
        });

        if (docsToRecognize && docsToRecognize.length > 0) {
            this.recognizedDocuments(docsToRecognize);
        }
    },

    recognizedDocuments: function (docsToRecognize) {


        var deletedDocs = window.batchLayoutViewModel.BatchEditor.deletedDocuments();
        deletedDocs.forEach(function (doc) { doc.IsDeleted(true) });
        var allDocs = window.batchLayoutViewModel.Documents();
        var docsNeedSave = allDocs.filter(function (doc) { return doc.pagesMoved() && !doc.docNeedRecognize(); }).length > 0;
        var taskId = window.Layouts.ModelLoader.viewModel().taskId();
        var recognizeDocsFunc = function () {
            dependencies.eventManager.trigger('processingStarted', { text: FC.Environment.Localization.GlobalResources.Recognizing_Title });
            FC.DataContext.Document.recognizeDocuments(
                docsToRecognize,
                taskId,
                docsToRecognize,
                function () {
                    window.batchLayoutViewModel.BatchEditor.deletedDocuments([]);
                    if (window.Layouts.ModelLoader.viewModel().documentsMoved) {
                        window.Layouts.ModelLoader.viewModel().documentsMoved(false);
                    }
                    dependencies.eventManager.trigger('processingFinished');
                });
            docsToRecognize.forEach(function (doc) { doc.recognitionStarted(true); });
        };
        if (docsNeedSave) {
            dependencies.eventManager.trigger('processingStarted',
                { text: FC.Environment.Localization.GlobalResources.SaveDocumentChanges });

            FC.DataContext.Task.applyChanges(allDocs.concat(deletedDocs), taskId, true, function (result) {
                if (window.Layouts.ModelLoader.viewModel().documentsMoved) {
                    window.Layouts.ModelLoader.viewModel().documentsMoved(false);
                }
                if (result) {
                    window.batchLayoutViewModel.copyDocumentsFrom(result, function (doc) {
                        doc.TaskId = taskId;
                        return new FC.BatchEditor.Document(doc);
                    }, true);
                }
                dependencies.eventManager.trigger('processingFinished');
                recognizeDocsFunc();

            });
        } else {
            recognizeDocsFunc();
        }
    },

    updateSelectedDocumentsTemplate: function () {
        var docsToUpdate = window.batchLayout.batchLayoutViewModel.BatchEditor.selectedDocuments().filter(function (doc) {
            return doc.NewTemplateVersionAvailible();
        });
        this.updateDocumentsTemplate(docsToUpdate);
    },

    updateDocumentsTemplate: function (docsToUpdate) {
        docsToUpdate.forEach(function (doc) { doc.updateTemplate(true); });
    }
};