﻿var FC = FC || {};
FC.Viewmodels = FC.Viewmodels || {};
FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

FC.Viewmodels.DocumentEditor.GroupRegionViewModel = function (region, parentRegion, page, depth) {
	FC.Viewmodels.DocumentEditor.RegionViewModel.call(this, region, parentRegion, page, depth);

	this.regions = ko.observableArray([]);

	this.page.subscriptions.push({
		key: this.target,
		value: this.selected
	});
};

FC.Viewmodels.DocumentEditor.GroupRegionViewModel.prototype = Object.create(FC.Viewmodels.DocumentEditor.RegionViewModel.prototype);

FC.Viewmodels.DocumentEditor.GroupRegionViewModel.prototype.updateRegions = function (regions, allRegions) {
	var oldRegions = this.regions();

	var newRegions = [];
	for (var i = 0; i < regions.length; i++) {
		var region = oldRegions.filter(function(r) { return r.model === regions[i]; })[0];

		if (region) {
			FC.Viewmodels.DocumentEditor.regionFactory.updateRegion(region, regions[i], allRegions);
		} else {
			region = FC.Viewmodels.DocumentEditor.regionFactory.createRegion(regions[i], this, this.page, allRegions);
			if (!region) {
				continue;
			}
		}

		newRegions.push(region);
	}

	this.regions(newRegions);
};

FC.Viewmodels.DocumentEditor.GroupRegionViewModel.prototype.createZones = function () {
	var zones = [];

	var childRegions = this.regions();
	var j;

	for (j = 0; j < childRegions.length; j++) {
		var childZones = childRegions[j].createZones();

		for (var k = 0; k < childZones.length; k++) {
			zones.push(childZones[k]);
		}
	}

	if (!this.isVisible()) {
		return zones;
	}

	var basicZones = this.createBasicZones();
	for (j = 0; j < basicZones.length; j++) {
		zones.push(basicZones[j]);
	}

	zones.push(new FC.Viewmodels.ActionZones.GroupBodyMovableZone(this.rectangles, this, this.page, this.depth));
	zones.push(new FC.Viewmodels.ActionZones.GroupBodyEditableZone(this.rectangles, this, this.page, this.depth));

	return zones;
};

FC.Viewmodels.DocumentEditor.GroupRegionViewModel.prototype.resizeRegion = function (delta, resizeChildren, rect) {
	var resizeInfo = this.resizeRegionBase(delta, null, rect);

	if (resizeChildren) {
		var childRegions = this.regions();
		for (var j = 0; j < childRegions.length; j++) {
			childRegions[j].resizeRegion(resizeInfo, resizeChildren);
		}
	}

	return resizeInfo;
};

FC.Viewmodels.DocumentEditor.GroupRegionViewModel.prototype.updateText = function () {
	var childRegions = this.regions();
	for (var j = 0; j < childRegions.length; j++) {
		childRegions[j].updateText();
	}

	this.updateTextBase();
};

FC.Viewmodels.DocumentEditor.GroupRegionViewModel.prototype.getExternalRect = function () {
	return FC.Viewmodels.DocumentEditor.GroupRegionViewModel.getGroupExternalRect(this);
};

FC.Viewmodels.DocumentEditor.GroupRegionViewModel.getGroupExternalRect = function(groupRegion) {
	var groupRect = groupRegion.getExternalRectBase();

	var childRegions = groupRegion.regions();
	for (var j = 0; j < childRegions.length; j++) {
		var childRect = childRegions[j].getExternalRect();
		if (groupRect.x1 > childRect.x1) {
			groupRect.x1 = childRect.x1;
		}

		if (groupRect.x2 < childRect.x2) {
			groupRect.x2 = childRect.x2;
		}

		if (groupRect.y1 > childRect.y1) {
			groupRect.y1 = childRect.y1;
		}

		if (groupRect.y2 < childRect.y2) {
			groupRect.y2 = childRect.y2;
		}
	}

	return groupRect;
};