﻿/**
 * Значение, ограниченное минимальным и максимальным значением
 * @param {} value значение (observable)
 * @param {} min минимальное значение (observable)
 * @param {} max максимальное значение (observable)
 * @param {boolean} isHorizontal значение - горизонтальное
 * @param {} source "источник" значения - прямоугольник или что-нибудь подобное 
 */
FC.Viewmodels.DocumentEditor.LimitedValue = function (value, min, max, isHorizontal, source) {
	this.value = value;
	this.minValue = min;
	this.maxValue = max;
	this.isHorizontal = isHorizontal;
	this.source = source;

	this.isMainCoordinate = false;
	
	/**
	 * Попытаться изминить значение
	 * @param {} newValue новое значение
	 * @returns {} 
	 */
	var getNewValue = function (limited, newValue) {
		var minValue = limited.minValue();
		var maxValue = limited.maxValue();

		// если не больше минимального - ставим минимальное+1
		if (newValue <= minValue) {
			return minValue + 1;
		}

		// если не меньше максимального - ставим максимальное-1
		if (newValue >= maxValue && maxValue !== minValue) {
			return maxValue - 1;
		}

		return newValue;
	};	

	/**
	 * Попытаться добавить к значению
	 * @param {} delta дельта
	 * @returns {boolean} true если значение изменилось
	 */
	this.tryAddToValue = function (delta) {
		var val = this.value();
		var newVal = getNewValue(this, val + delta);
		this.value(newVal);

		if (newVal !== val) {
			if (delta > 0) {
				if (this.isHorizontal && this.value === this.source.x1) {
					if (newVal >= this.source.x2()) {
						this.source.x2(newVal + 1);
					}
				}

				if (!this.isHorizontal && this.value === this.source.y1) {
					if (newVal >= this.source.y2()) {
						this.source.y2(newVal + 1);
					}
				}
			} else {
				if (this.isHorizontal && this.value === this.source.x2) {
					if (newVal <= this.source.x1()) {
						this.source.x1(newVal -1);
					}
				}

				if (!this.isHorizontal && this.value === this.source.y2) {
					if (newVal <= this.source.y1()) {
						this.source.y2(newVal - 1);
					}
				}
			}

		}
		return newVal !== val;
	};

	/**
	 * Добавить к значению
	 * @param {} delta дельта 
	 */
	this.addToValue = function (delta) {
		var val = this.value();
		this.value(val + delta);
	};

	/**
	 * Можно ли добавить к значению
	 * @param {} delta дельта
	 * @returns {boolean} true если можно
	 */
	this.canAdd = function (delta) {
		var newValue = this.value() + delta;

		var minValue = this.minValue();
		var maxValue = this.maxValue();

		if (newValue <= minValue || (newValue >= maxValue && maxValue !== minValue)) {
			return false;
		}

		return true;
	}
};