﻿var FC = FC || {};
FC.Viewmodels = FC.Viewmodels || {};
FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

FC.Viewmodels.DocumentEditor.PageViewModel =
	function (page, documentModel, selectedField, selectedParagraph, moveFormToField, documentDataContext, regionsPanel, readonlyMode) {
		var self = this;

		var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();

		self.subscriptions = [];

		self.selectedField = selectedField;
		self.selectedParagraph = selectedParagraph;

		self.documentModel = documentModel;
		self._id = FC.Viewmodels.DocumentEditor.PageViewModel.GlobalCounter++;
		self.id = ko.observable(page.id());

		var drawer = new FC.Viewmodels.ImageEditor.CanvasDrawer();

		var getPageEditor = function (pages) {
			for (var i = 0; i < pages.length; i++) {
				if (pages[i].id === page.id()) {

					drawer.layers = [
						new FC.Viewmodels.ImageEditor.CanvasLayer(docEditor.getPageDrawer(self.id(), 0)),
						new FC.Viewmodels.ImageEditor.CanvasLayer(docEditor.getPageDrawer(self.id(), 1))
					];

					return true;
				}
			}

			return false;
		};

		if (!getPageEditor(docEditor.pages())) {
			self.dispose = function () { };
			return;
		}

		self.subscriptions.push({
			key: null,
			value:
				{
					dispose: docEditor.onOpenedDocumentChanged(function () {
						getPageEditor(docEditor.pages());
					}
				)
}
		});

		self.zoom = ko.observable(docEditor.zoom());

		self.subscriptions.push({
			key: null,
			value:
			{
				dispose: docEditor.onZoomChanged(function(newZoom) {
					self.zoom(newZoom);
					self.draw();
				})
			}
		});

		if (readonlyMode) {
			self.subscriptions.push({
				key: null,
				value:
					readonlyMode.subscribe(function () {
						self.draw();
					})
			});
		}
		

		self.regions = ko.observableArray([]);
		self.textRegions = ko.observableArray([]);
		self.actionZones = ko.observableArray([]);
		self.width = ko.observable(page.width());
		self.height = ko.observable(page.height());
		self.version = page.modification;
		self.visible = ko.observable(false);
		self.selectedRegions = ko.observableArray([]);

		var defaultCursorStyle = FC.Viewmodels.ActionZones.cursorTypes.crosshair;
		var modifiedCursorStyle = FC.Viewmodels.ActionZones.cursorTypes.modifyRegion;
		self.currentCursor = ko.observable(defaultCursorStyle);

		self.subscriptions.push({
			key: null,
			value: self.currentCursor.subscribe(function (value) {
				docEditor.setCursor(value);
			})
		});

		self.editorActive = ko.observable(docEditor.editorActive);

		/**
		 * Модифицировать регион (добавить к нему новые прямоугольники)
		 * @param {} region текущий регион
		 * @param {} rectangles новые прямоугольники 
		 */
		self.modifyRegion = function (region, newRectangles) {
			if (!newRectangles || newRectangles.length === 0) {
				return;
			}
		
			var rects = region.rectangles();
			for (var i = 0; i < newRectangles.length; i++) {
				rects.push(newRectangles[i]);
			}

			self.recalculateRegionRectangles(region, rects);

			// обновляем текст
			region.updateText();
			region.field().needRecognize(true);

			self.directDrawer.clear();

			FC.Viewmodels.EventStack.addEvent();
		};

		self.recalculateRegionRectangles = function (region, rects) {
			// не добавляем прямоугольники, у которых нет ширины или высоты
			rects = rects.filter(function (r) { return r.y1() !== r.y2() && r.x1() !== r.x2(); });

			// пересчитываем полигон (из имеющихся прямоугольников нужно построить одну непрерывную фигуру)
			var newRects = FC.Utils.polygonCommon.createPolygonFromRects(ko.toJS(rects))
				.map(function(r) {
					return new FC.BusinessModels.DocumentEditor.Coordinates(
						{
							X1: r.x1,
							X2: r.x2,
							Y1: r.y1,
							Y2: r.y2
						});
				});

			region.rectangles(newRects);

			FC.Utils.polygonCommon.normalizeRectangles(region.rectangles);

			// пересчитываем зоны для региона
			self.replaceRegionZones(region);
		};

		/**
		 * Колбэк для рисования регионов
		 * @param {any} rectangle полученный прямоугольник
		 * @param {any} parentRegion родительский регион
		 * @param {any} p страница
		 * @param {any} event событие мыши
		 */
		var onRegionCreateMouseUp = function (rectangle, parentRegion, p, event) {
			// слишком маленький регион не создаём
			if (FC.Viewmodels.DocumentEditor.RegionCreator.regionTooSmallToCreate(rectangle)) {
				return;
			}

			// если мы не создаём регион, а модифицируем
			if (self.regionsAreModifying()) {
				var selectedRegions = self.selectedRegions()
					.filter(function (r) { return r.modifiable; })
					.sort(function (r1, r2) {
						return r2.depth - r1.depth;
					});

				if (selectedRegions.length > 0) {
					self.modifyRegion(selectedRegions[0], [rectangle]);
					self.draw();
					return;
				}
			} else {
				var text = FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextInsideRectangle(rectangle, p);
				p.createRegion(rectangle, parentRegion, event, text);
			}
		};

		var onRegionCreateMouseMove = function (rectangle, currentZone, p) {
			FC.Viewmodels.ActionZones.RecognizedTextZone.selectZonesInsideRectangle(rectangle, p);
		};

		self.regionCreator = new FC.Viewmodels.DocumentEditor.RegionCreator(self, onRegionCreateMouseMove, onRegionCreateMouseUp);

		/**
		 * Находится ли страница в режиме модификации регионов
		 */
		self.regionsAreModifying = ko.computed(function () {
			// если глобально режим не включен - то точно нет
			if (!regionsPanel.regionModifying()) {
				return false;
			}

			// если нет выделенных регионов - то нечего модифицировать
			var selectedRegions = self.selectedRegions();
			if (selectedRegions.length === 0) {
				return false;
			}

			// если хоть один выделенный регион можно модифицировать - то модифицируем
			return selectedRegions.filter(function (r) {
				return r.modifiable;
			}).length > 0;
		});

		self.subscriptions.push({
			key: null,
			value: self.regionsAreModifying
		});

		self.getScreenCoordinate = function (coordinate) {
			return coordinate * self.zoom();
		};

		self.getRealCoordinate = function (coordinate) {
			return coordinate / self.zoom() - FC.DocumentPageEditor.Margin;
		};

		self.directDrawer = new FC.Viewmodels.ImageEditor.DirectDrawer();

		self.draw = function (layerIndex) {
			var t0 = performance.now();
			
			if (typeof layerIndex == "undefined") {
				layerIndex = -1;
			}

			var displayedPages = docEditor.pages();
			if (!displayedPages.filter(function (p) { return p.id === self.id() })[0]) {
				return;
			}

			var pageDrawer;
			if (layerIndex !== 1) {
				pageDrawer = docEditor.getPageDrawer(self.id(), 0);
				pageDrawer.clearDrawCommands();
			}

			if (layerIndex !== 0) {
				pageDrawer = docEditor.getPageDrawer(self.id(), 1);
				pageDrawer.clearDrawCommands();
			}

			if (!readonlyMode()) {
				var zones = self.actionZones();
				for (var i = 0; i < zones.length; i++) {
					zones[i].draw(layerIndex);
				}

				self.directDrawer.draw(self.getLayer(1));
			}

			if (layerIndex !== 1) {
				pageDrawer = docEditor.getPageDrawer(self.id(), 0);
				pageDrawer.commit();
			}

			if (layerIndex !== 0) {
				pageDrawer = docEditor.getPageDrawer(self.id(), 1);
				pageDrawer.commit();
			}

			FC.Utils.EventLogger.Default.addMessage("RGPANEL", "pageViewModel draw\t" + (performance.now() - t0) + "\tms");
		};

		self.getLayer = function (layerIndex) {
			return drawer.getLayer(layerIndex);
		};

		var findDeepestZonesForPoint = function (point) {
			var zones = self.actionZones();
			var deepestZones = [];
			var recognizedZones = [];
			var i;
			for (i = 0; i < zones.length; i++) {
				if (!zones[i].isActive() || !zones[i].pointIsInside(point)) {
					continue;
				}

				if (zones[i] instanceof FC.Viewmodels.ActionZones.RecognizedTextZone) {
					recognizedZones.push(zones[i]);
					continue;
				}

				if (regionsPanel.textSelectingMode()) {
					continue;
				}

				if (deepestZones.length === 0) {
					deepestZones.push(zones[i]);
					continue;
				}
				if (zones[i].depth() > deepestZones[0].depth()) {
					deepestZones = [zones[i]];
					continue;
				}

				if (zones[i].depth() === deepestZones[0].depth()) {
					deepestZones.push(zones[i]);
				}
			}

			if (deepestZones.length === 0 || regionsPanel.textSelectingMode()) {
				return recognizedZones;
			}

			if (deepestZones.filter(function (z) { return z instanceof FC.Viewmodels.ActionZones.GroupBodyEditableZone; }).length) {
				for (i = 0; i < recognizedZones.length; i++) {
					deepestZones.push(recognizedZones[i]);
				}
			}

			return deepestZones;
		};

		var highlightRegions = function (regions) {
			for (var i = 0; i < regions.length; i++) {
				regions[i].highlighted(true);
			}
		};

		self.unhighlightRegions = function () {
			regionsPanel.unhighlightRegions();
		};

		self.unhighlightPageRegions = function () {
			var allRegions = self.allRegions();

			var hasChanges = false;
			for (var i = 0; i < allRegions.length; i++) {
				if (allRegions[i].highlighted()) {
					allRegions[i].highlighted(false);
					hasChanges = true;
				}
			}

			return hasChanges;
		};

		self.mouseDown = function (data, event) {
			if (readonlyMode()) {
				return true;
			}

			if (data.pageId !== self.id()) {
				return true;
			}

			var zones = findDeepestZonesForPoint(data);

			if (zones && zones.length > 0) {
				var selectedZone = zones.filter(function (z) { return z.target.selected(); });
				if (selectedZone.length > 0) {
					selectedZone.sort(function (z) { return z instanceof FC.Viewmodels.ActionZones.RecognizedTextZone ? -1 : 1; });
					var k = 0;
					while (k < selectedZone.length) {
						if (selectedZone[k].mouseDown(data, self, event, zones)) {
							return false;
						}

						k++;
					}
				}
				else if (zones[0].mouseDown(data, self, event, zones)) {
					return false;
				}
			}

			if (!self.regionsAreModifying()) {
				self.unhighlightRegions();
				self.deselectRegions();
				self.draw();
			}

			self.regionCreator.startRegionDrawing(data);
			return false;
		};

		self.createRegion = function (rectangles, parentRegion, event, text, onEnd) {
			if (regionsPanel.textSelectingMode()) {
				regionsPanel.lastSelectedText(text);
				if (onEnd) {
					onEnd();
				}
				return;
			}

			if (!Array.isArray(rectangles)) {
				rectangles = [rectangles];
			}

			var binder = new FC.Viewmodels.DocumentEditor.RegionBinder(rectangles, parentRegion, event, self, text, onEnd);
			if (binder.needShow) {
				self.setCurrentCursorAndCaption(null);
				binder.show();
			}
		};

		self.createRegionForField = function (regionData, parentRegion, field, text, status) {
			var t0 = performance.now();

			if (!regionData ||
				regionData.Coordinates[0].X1 === regionData.Coordinates[0].X2 ||
				regionData.Coordinates[0].Y1 === regionData.Coordinates[0].Y2) {
				return null;
			}

			if (typeof status === "undefined") {
				status = 1;
			}

			regionData.Id = [FC.Viewmodels.DocumentEditor.PageViewModel.NewRegionCounter++];

			var regionModel =
				FC.BusinessModels.DocumentEditor.Region.createRegionForField(regionData, field.toJs(), self.documentModel,
					FC.BusinessModels.DocumentEditor.Region.isRootRegion(field.parent));

			regionModel.isNew(true);

			FC.Viewmodels.EventStack.addEvent();

			if (field instanceof FC.BusinessModels.DocumentEditor.Table && status === 1) {
				var rowIndex = FC.Viewmodels.DocumentEditor.TableRegionViewModel.findIndexForNewRow(field,
					regionData.Id,
					ko.toJS(regionModel.coordinates()[0]),
					[],
					regionModel.regionPageIndex(),
					null,
					regionModel);

				field.addTableRowAtPosition(rowIndex.index, regionData.Id.slice(0));
			}

			if (!text || !(field instanceof FC.BusinessModels.DocumentEditor.TextField)) {
				field.needRecognize(status === 1);
				regionModel.status(status);
			}

			self.documentModel.regions.push(regionModel);
			self.documentModel.fieldRegion.push({ field: field, region: regionModel });

			var allRegions = self.allRegions();
			var region = FC.Viewmodels.DocumentEditor.regionFactory.createRegion(regionModel, parentRegion, self, allRegions);
			if (!region) {
				return null;
			}

			self.allRegions(allRegions);

			if (field instanceof FC.BusinessModels.DocumentEditor.TextField && !field.isReadOnly()) {
				field.addParagraph(text, region.getParagraphIndex());
			}

			if (parentRegion) {
				parentRegion.regions.push(region);
				parentRegion.model.regions.push(regionModel);
			} else {
				self.regions.push(region);
			}

			var newZones = region.createZones();
			for (var j = 0; j < newZones.length; j++) {
				self.actionZones.push(newZones[j]);
			}

			FC.Utils.EventLogger.Default.addMessage("RGPANEL",
				"pageViewModel createRegionForField\t" + (performance.now() - t0) + "\tms");

			// для ячеек не нужно выделять регион при создании - мы его не рисуем сами
			if (!(region instanceof FC.Viewmodels.DocumentEditor.CellRegionViewModel)) {
				setTimeout(function () {
					self.deselectRegions();
					self.selectRegion(region, true);
					self.draw();
				},
					100);
			}

			return region;
		};

		self.findRegionByModel = function (regionModel) {
			return self.allRegions().filter(function (r) { return r.model === regionModel; })[0];
		};

		var findRegionModelsByField = function (field) {
			return self.documentModel.manipulators.regionsManipulator.fieldRegionMap()
				.filter(function (val) {
					return val.field === field && val.region.status() /*&& val.region.pageId() === target.id()*/;
				})
				.map(function (fr) { return fr.region; });
		};

		var findRegionsByField = function (field) {
			return findRegionModelsByField(field)
				.map(function (fr) { return self.findRegionByModel(fr); })
				.filter(function (r) { return !!r; });
		};

		self.deselectRegions = function (onlyHovering, regionsToDeselect) {
			if (regionsToDeselect) {
				regionsPanel.deselectAllRegions(onlyHovering, regionsToDeselect, self.id());
			} else {
				regionsPanel.deselectAllRegions(onlyHovering);
			}
		};

		self.deselectPageRegions = function (onlyHovering, regionsToDeselect) {
			var regions = self.selectedRegions();

			var hasChanges = false;

			for (var i = 0; i < regions.length; i++) {
				var doDeselect = false;

				if (regionsToDeselect) {
					for (var j = 0; j < regionsToDeselect.length; j++) {
						if (regionsToDeselect[j]._id === regions[i]._id) {
							doDeselect = true;
						}
					}
				}

				if ((!!regionsToDeselect && doDeselect) || !regionsToDeselect && (!onlyHovering || regions[i].hovering)) {
					regions[i].selected(false);
					regions.splice(i, 1);
					i--;
					hasChanges = true;
				}
			}

			if (hasChanges) {
				self.selectedRegions(regions);
			}

			return hasChanges;
		};

		/**
		 * Получить положение курсора в поле для текущего региона
		 * @param {any} region вьюмодель региона
		 * @param {any} field модель поля
		 */
		var getCursorPositionForRegion = function (region, field) {
			if (!field) {
				return 0;
			}

			// получим параграф для региона
			var paragraphIndex = region.getParagraphIndex();
			if (paragraphIndex === 0) {
				return 0;
			}

			var flags = field.flags();

			// найдём начало параграфа
			var currentParagraphIndex = 0;
			for (var i = 0; i < flags.length; i++) {
				if (currentParagraphIndex === paragraphIndex) {
					return i;
				}

				if (flags[i] === "\n") {
					currentParagraphIndex++;
				}
			}

			return 0;
		};

		/**
		 * Выделить регион
		 * @param {} region вьюмодель региона
		 * @param {} scrollToField нужно ли подскроллиться к полю
		 * @param {} alreadyHighlighted регион уже подсвечен
		 * @returns {boolean} был ли подскролл
		 */
		self.selectRegion = function (region, scrollToField, alreadyHighlighted) {
			if (!region) {
				return true;
			}

			// если текущий регион не подсвечен - снимем все отметки о подсветки, они чужие
			if (!alreadyHighlighted) {
				self.unhighlightRegions();
			}

			var fieldScrolled = !scrollToField;

			// выделяем регион
			if (!region.selected()) {
				region.selected(true);
				self.selectedRegions.push(region);
			}

			// подскраливаем к полю и выделяем его
			if (scrollToField) {
				// ищем модель поля
				var fieldRegions = self.documentModel.manipulators.regionsManipulator.fieldRegionMap()
					.filter(function (val) { return val.region === region.model; });
				if (fieldRegions.length > 0) {
					var field = fieldRegions[0].field;

					// подсвечиваем все регионы поля
					var allFieldRegions = findRegionsByField(field, self);
					highlightRegions(allFieldRegions);

					FC.Viewmodels.DocumentEditor.PageViewModel.lastSelectedFieldInternalId = field.internalId;

					if (field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Text) {
						// для текстового поля
						
						self.selectedField(field);
						
						// подскраливаем
						setTimeout(
							function () {
								moveFormToField(field);

								var fieldsPanel = window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel;
								var fieldViewModel = fieldsPanel.getFieldViewmodel(field);

								// ставим курсор в положение, соответствующее региону
								fieldViewModel.selectFieldAtCursor(getCursorPositionForRegion(region, field));
							},
							100
						);
					} else if (field !== self.selectedField()) {
						// для не-текстовых

						// выделяем поле
						self.selectedField(field);

						// для чекмарки, у которой родитель - choice, выделяем и родителя
						if (field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Boolean &&
							field.parent.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Choice) {
							self.selectedField(field.parent);
						}

						// для группы или таблицы - подскроллимся
						if (field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.GroupParent ||
							field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent ||
							field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Table) {
							moveFormToField(field, true);
						}

						self.unhighlightRegions();
					} else {
						moveFormToField(field);
					}

					fieldScrolled = !(field.parent instanceof FC.BusinessModels.DocumentEditor.ChoiceField);
				}
			}

			return fieldScrolled;
		};

		/**
		 * Получить текущую вьюмодель региона для поля
		 * @param {any} fieldModel модель поля
		 * @param {any} regions вьюмодели региогов поля на текущей странице
		 * @param {any} regionModels все модели регионов поля
		 */
		var getFieldRegion = function (fieldModel, regions, regionModels, fieldWasAlreadySelected) {
			if (!regions || regions.length === 0) {
				return null;
			}

			if (!(fieldModel instanceof FC.BusinessModels.DocumentEditor.TextField) || regions.length < 2 && regionModels.length < 2) {
				if (!(fieldModel instanceof FC.BusinessModels.DocumentEditor.TextField) && fieldWasAlreadySelected) {
					return null;
				}
				return regions[0];
			}

			// отсортируем модели геометрически
			var sortedRegions = regionModels.sort(FC.BusinessModels.DocumentEditor.Region.compareRegions);

			// найдём вьюмодель региона по индексу параграфа
			var selParagraph = self.selectedParagraph();
			if (selParagraph < 0) {
				selParagraph = 0;
			}

			if (selParagraph > sortedRegions.length) {
				selParagraph = sortedRegions.length - 1;
			}

			var regionOnParagraph = sortedRegions[selParagraph];
			if (regionOnParagraph.pageId() !== self.id()) {
				return null;
			}

			return regions.filter(function (rvm) { return rvm.model === regionOnParagraph })[0];
		};

		self.subscriptions.push({
			key: null,
			value: self.selectedField.subscribe(function (field) {
				var t0 = performance.now();
				
				if (!field) {
					self.unhighlightRegions();
					self.deselectRegions();
					self.draw();
					FC.Viewmodels.DocumentEditor.PageViewModel.lastSelectedFieldInternalId = -1;
					return;
				}

				var fieldWasAlreadySelected =
					FC.Viewmodels.DocumentEditor.PageViewModel.lastSelectedFieldInternalId === field.internalId;

				if (!fieldWasAlreadySelected) {
					if (!(field instanceof FC.BusinessModels.DocumentEditor.ChoiceField) ||
						field.fields().filter(function (f) {
							return f.internalId === FC.Viewmodels.DocumentEditor.PageViewModel.lastSelectedFieldInternalId;
						}).length === 0) {
						self.unhighlightRegions();
						self.deselectRegions();
					}
				}

				var regionModels = findRegionModelsByField(field);
				if (!regionModels || regionModels.length === 0) {
					self.deselectRegions();
					self.draw();
					return;
				}

				var region = regionModels
					.map(function (r) { return self.findRegionByModel(r); })
					.filter(function (r) { return !!r; });

				FC.Viewmodels.DocumentEditor.PageViewModel.lastSelectedFieldInternalId = field.internalId;
				
				highlightRegions(region);

				var selectedRegion = getFieldRegion(field, region, regionModels, fieldWasAlreadySelected);

				if (!selectedRegion) {
					if (fieldWasAlreadySelected && field instanceof FC.BusinessModels.DocumentEditor.TextField) {
						// selectedRegion null для всех не-текстовых регионов
						// сбрасывать выделение нужно только для текстовых с продолжением
						self.deselectPageRegions();
					}

					self.draw();
					return;
				}

				if (field instanceof FC.BusinessModels.DocumentEditor.ChoiceField) {
					var selectedRegions = self.selectedRegions();
					var alreadySelectedChilds = selectedRegions.filter(function (r) { return r.field().parent === field; });
					if (alreadySelectedChilds.length === 0) {
						self.deselectPageRegions();
					} else {
						return;
					}
				} else {
					self.deselectPageRegions();
				}
				
				self.selectRegion(selectedRegion, false, true);
				var rects = selectedRegion.rectangles();
				if (rects && rects.length > 0) {
					// Без setTimeout при выделении региона активная (жёлтая) область региона рисуется некорректно.
					// Документ прокручивается к полю, а жёлтая область отрисовывается так, как если бы прокрутики не было.
					// Я так и не понял, почему это происходит.
					// Нужно лезть глубоко в недра рисования регионов, чтобы найти место, где рассчитывается положение жёлтой области.
					// Хочется верить, что переход на redux исправит эту проблему
					window.setTimeout(function () {
						docEditor.showRectangle(self.id(), rects[0].x1(), rects[0].y1(), rects[0].x2(), rects[0].y2());
					}, 100);
				}
				
				self.draw();

				FC.Utils.EventLogger.Default.addMessage("RGPANEL",
					"pageViewModel on selectedField\t" + (performance.now() - t0) + "\tms");
			})
		});

		self.deselectHovering = true;

		self.currentMousePosition = null;

		/**
		 * Задать текущий курсор и тултип у страницы
		 * @param {} currentZones зоны, в которых находится мышь 
		 */
		self.setCurrentCursorAndCaption = function (currentZones) {
			// если нет текущих зон - сбрасываем на дефолт
			if (!currentZones || currentZones.length === 0) {
				self.currentCursor(self.regionsAreModifying() ? modifiedCursorStyle : defaultCursorStyle);
				self.currentCursor.valueHasMutated();
				docEditor.textOnHover(null);
			} else {
				// определяем текущий курсор из вырхней зоны
				var cursorValue = currentZones[0].hasOwnProperty("modifiedCursorStyle") && self.regionsAreModifying()
					? currentZones[0].modifiedCursorStyle
					: currentZones[0].cursorStyle();

				self.currentCursor(cursorValue);

				// собираем текст тултипа из зон
				var text = currentZones.length > 0 ? currentZones[0].target.caption() : "";
				var k;
				for (k = 1; k < currentZones.length; k++) {
					text += "\n" + currentZones[k].target.caption();
				}

				docEditor.textOnHover(text);
			}
		};

		self.mouseMove = function (data) {
			if (readonlyMode()) {
				return true;
			}

			if (data.pageId !== self.id()) {
				return true;
			}
			
			self.currentMousePosition = data;
			var zones = findDeepestZonesForPoint(data);

			if (self.deselectHovering && (!zones || zones.length === 0 || (!zones[0].target.hovering || !zones[0].target.selected()))) {
				if (self.deselectPageRegions(true)) {
					self.draw(1);
				}
			}

			self.setCurrentCursorAndCaption(zones);

			if (zones) {
				for (var k = 0; k < zones.length; k++) {
					zones[k].onHover(data, self);
				}
			}

			return true;
		};

		self.subscriptions.push({
			key: null,
			value: regionsPanel.regionModifying.subscribe(function () {
				if (self.currentMousePosition) {
					var zones = findDeepestZonesForPoint(self.currentMousePosition);
					self.setCurrentCursorAndCaption(zones);
				}
			})
		});

		self.mouseUp = function (data) {
			if (readonlyMode()) {
				return true;
			}

			if (data.pageId !== self.id()) {
				return true;
			}

			var zones = findDeepestZonesForPoint(data);
			if (zones.length < 2) {
				return true;
			}

			var k;
			for (k = 0; k < zones.length; k++) {
				if (zones[k].target.selected()) {
					break;
				}
			}

			k++;

			if (k >= zones.length) {
				k = 0;
			}

			zones[k].selectTarget();
			return false;
		};

		self.replaceRegionZones = function(region) {
			var t0 = performance.now();

			var newZones = region.createZones();

			var zones = self.actionZones();
			for (var j = 0; j < zones.length; j++) {
				if (zones[j].target._id === region._id) {
					var removedZone = zones.splice(j, 1)[0];
					if (removedZone instanceof FC.Viewmodels.ActionZones.PolygonBorderZone && removedZone.startPoint) {
						var similarZones = newZones.filter(function(z) {
							return z.cursorStyle() === removedZone.cursorStyle();
						}).sort(function (z1, z2) {
							var diffX1 = Math.abs(z1.point.x() - removedZone.point.x());
							var diffX2= Math.abs(z2.point.x() - removedZone.point.x());
							var diffY1 = Math.abs(z1.point.y() - removedZone.point.y());
							var diffY2 = Math.abs(z2.point.y() - removedZone.point.y());

							// ищем самую ближнюю точку
							return Math.sqrt(diffX1 * diffX1 + diffY1 * diffY1) - Math.sqrt(diffX2 * diffX2 + diffY2 * diffY2);
							});

						if (similarZones[0]) {
							var spoint = removedZone.startPoint;
							removedZone.mouseUp(removedZone);
							similarZones[0].mouseDown(spoint);
						}
					}

					j--;
				}
			}

			self.actionZones(zones.concat(newZones));

			// дочерних регионов быть не должно - сложные регионы только текстовые

			FC.Utils.EventLogger.Default.addMessage("RGPANEL",
				"pageViewModel removeRegionZones\t" + (performance.now() - t0) + "\tms");
		};

		self.removeRegionZones = function (region) {
			var t0 = performance.now();

			var zones = self.actionZones();

			for (var j = 0; j < zones.length; j++) {
				if (zones[j].target._id === region._id) {
					zones.splice(j, 1);
					j--;
				}
			}

			self.actionZones(zones);

			if (region.hasOwnProperty("regions")) {
				var childRegions = region.regions();
				for (var k = 0; k < childRegions.length; k++) {
					self.removeRegionZones(childRegions[k]);
				}
			}

			FC.Utils.EventLogger.Default.addMessage("RGPANEL",
				"pageViewModel removeRegionZones\t" + (performance.now() - t0) + "\tms");
		};

		self.findRegionZoneForPoint = function (region, point) {
			var zones = self.actionZones();

			for (var j = 0; j < zones.length; j++) {
				if (zones[j].target._id === region._id && zones[j].pointIsInside(point)) {
					return zones[j];
				}
			}

			return null;
		};

		self.findRegionZones = function (region) {
			var zones = self.actionZones();
			var result = [];

			for (var j = 0; j < zones.length; j++) {
				if (zones[j].target._id === region._id) {
					result.push(zones[j]);
				}
			}

			return result;
		};

		var deleteRegionsInTree = function (root, regionsToDelete, onlyViewmodel) {
			if (!regionsToDelete || regionsToDelete.length === 0) {
				return false;
			}

			var i;
			for (i = 0; i < regionsToDelete.length; i++) {
				self.removeRegionSubscriptions(regionsToDelete[i]);
			}

			var regions = root.regions();
			var allRegions = self.allRegions();

			for (var j = 0; j < allRegions.length; j++) {
				if (regionsToDelete.indexOf(allRegions[j]) > -1) {
					allRegions.splice(j, 1);
				}
			}
			self.allRegions(allRegions);

			var modelsToDelete = [];
			var lastParagraphToDelete = -1;
			var result = false;
			for (i = 0; i < regions.length; i++) {
				var regionRemoved = false;
				for (var j = 0; j < regionsToDelete.length; j++) {
					if (regions[i]._id === regionsToDelete[j]._id) {
						if (regions[i].hasOwnProperty("regions")) {
							deleteRegionsInTree(regions[i], regions[i].regions().slice(0), onlyViewmodel);
						}

						regions.splice(i, 1);
						i--;
						root.regions(regions);
						self.removeRegionZones(regionsToDelete[j]);
						if (!onlyViewmodel) {
							modelsToDelete.push(regionsToDelete[j].model);
							lastParagraphToDelete = regionsToDelete[j].getParagraphIndex();
						}

						result = true;
						regionRemoved = true;
						break;
					}
				}

				if (regionRemoved) {
					continue;
				}

				if (regions[i].hasOwnProperty("regions")) {
					if (deleteRegionsInTree(regions[i], regionsToDelete, onlyViewmodel)) {
						result = true;
					}
				}
			}

			if (result) {
				self.draw();
			}

			if (modelsToDelete.length > 0) {
				self.documentModel.manipulators.regionsManipulator.deleteRegions(modelsToDelete, lastParagraphToDelete);
			}

			return result;
		};

		self.deleteRegion = function (regionToDelete, onlyViewmodel) {
			var t0 = performance.now();

			self.deselectRegions(false, [regionToDelete]);
			regionToDelete.beforeDeleting();
			deleteRegionsInTree(self, [regionToDelete], onlyViewmodel);

			FC.Viewmodels.EventStack.addEvent();
			FC.Utils.EventLogger.Default.addMessage("RGPANEL",
				"pageViewModel deleteRegion\t" + (performance.now() - t0) + "\tms");
		};

		self.allRegions = ko.observable([]);

		self.updateRegions = function (regions) {
			var t0 = performance.now();
			var oldRegionViewmodels = self.regions();
			var newRegions = [];
			self.actionZones([]);
			var allRegions = [];

			if (!regions) {
				self.regions([]);
				self.allRegions([]);
				return;
			}

			for (var i = 0; i < regions.length; i++) {
				var region = oldRegionViewmodels.filter(function (r) { return r.model === regions[i]; })[0];

				if (region) {
					FC.Viewmodels.DocumentEditor.regionFactory.updateRegion(region, regions[i], allRegions);
				} else {
					region = FC.Viewmodels.DocumentEditor.regionFactory.createRegion(regions[i], null, self, allRegions);
					if (!region) {
						continue;
					}
				}

				newRegions.push(region);
				var newZones = region.createZones();
				for (var j = 0; j < newZones.length; j++) {
					self.actionZones.push(newZones[j]);
				}
			}

			self.regions(newRegions);
			self.allRegions(allRegions);
			FC.Utils.EventLogger.Default.addMessage("RGPANEL",
				"pageViewModel updateRegions\t" + (performance.now() - t0) + "\tms");
		};

		var updateTextRegions = function (pageText) {
			var t0 = performance.now();

			if (!pageText || !pageText.Text) {
				return;
			}

			for (var i = 0; i < pageText.Text.length; i++) {
				var textRegion = FC.Viewmodels.DocumentEditor.regionFactory.createTextRegion(pageText.Text[i], self);

				self.textRegions.push(textRegion);
				var newZones = textRegion.createZones();
				for (var j = 0; j < newZones.length; j++) {
					self.actionZones.push(newZones[j]);
				}
			}

			FC.Utils.EventLogger.Default.addMessage("RGPANEL",
				"pageViewModel updateTextRegions\t" + (performance.now() - t0) + "\tms");
		};

		self.updateRegions(self.documentModel.documentRegions());
		self.draw();

		var removalKey = "pageMouseEvent_" + self._id;
		docEditor.addOnMouseDown(self.mouseDown.bind(this), 0, removalKey);
		docEditor.addOnMouseMove(self.mouseMove.bind(this), 0, removalKey);
		docEditor.addOnMouseUp(self.mouseUp.bind(this), 50, removalKey);
		docEditor.addOnMouseLeave(self.mouseUp.bind(this), 50, removalKey);

		docEditor.addOnMouseEnterGlobal(
			function () {
				self.editorActive(true);
				self.draw(1);
				return true;
			},
			0,
			removalKey
		);

		docEditor.addOnMouseLeaveGlobal(
			function () {
				self.editorActive(false);
				self.draw(1);
				return true;
			},
			0,
			removalKey
		);

		var getTextRegions = function () {
			documentDataContext.getPageText(
				self.documentModel.projectId(),
				self.documentModel.batchId(),
				self.documentModel.id(),
				self.id(),
				self.version(),
				updateTextRegions,
				self.documentModel.templateId(),
				self.documentModel.templateVersion());
		};

		getTextRegions();

		self.subscriptions.push({
			key: null,
			value: self.documentModel.documentRegions.subscribe(function (newDocs) {
				self.updateRegions(newDocs);
				getTextRegions();
				self.draw();
			})
		});

		self.removeRegionSubscriptions = function (region) {
			var i = 0;
			while (i < self.subscriptions.length) {
				if (self.subscriptions[i].key === region) {
					var sub = self.subscriptions.splice(i, 1);
					sub[0].value.dispose();
				} else {
					i++;
				}
			}
		};

		self.dispose = function () {
			docEditor.removePageDrawer(self.id(), 0);
			docEditor.removePageDrawer(self.id(), 1);

			docEditor.clearDrawings(self.id());

			drawer.layers[0].pageDrawer = null;
			drawer.layers[1].pageDrawer = null;

			while (self.subscriptions.length > 0) {
				var sub = self.subscriptions.pop();
				if (sub.value.dispose) {
					sub.value.dispose();
				}
			}

			docEditor.removeOnMouseDown(removalKey);
			docEditor.removeOnMouseMove(removalKey);
			docEditor.removeOnMouseUp(removalKey);
			docEditor.removeOnMouseLeave(removalKey);
			docEditor.removeOnMouseEnterGlobal(removalKey);
			docEditor.removeOnMouseLeaveGlobal(removalKey);
		};
	};

FC.Viewmodels.DocumentEditor.PageViewModel.GlobalCounter = 0;
FC.Viewmodels.DocumentEditor.PageViewModel.NewRegionCounter = 1;

FC.Viewmodels.DocumentEditor.PageViewModel.lastSelectedFieldInternalId = -1;