﻿FC.Viewmodels.DocumentEditor.RegionBinder = function (rectangles, parentRegion, event, page, newText, onBinderClose) {
	var point = { x: event.pageX, y: event.pageY };

	FC.Viewmodels.DocumentEditor.Binder.call(this, [], point, function () {
		if (onBinderClose) {
			onBinderClose();
		}

		page.directDrawer.clear();
		page.draw();
	});

	this.parentRegion = parentRegion;
	this.page = page;
	this.needShow = true;
	this.rectangles = rectangles;
	this.regionData = {
		Coordinates: rectangles.map(function (rect) {
			return {
				X1: ko.utils.unwrapObservable(rect.x1),
				X2: ko.utils.unwrapObservable(rect.x2),
				Y1: ko.utils.unwrapObservable(rect.y1),
				Y2: ko.utils.unwrapObservable(rect.y2)
			}
		}),
		Status: 2,
		PageId: page.id()
	};
	this.regionText = newText ? newText : "";

	this.onBinderClose = onBinderClose || function () { };
};


FC.Viewmodels.DocumentEditor.RegionBinder.prototype = Object.create(FC.Viewmodels.DocumentEditor.Binder.prototype);

FC.Viewmodels.DocumentEditor.RegionBinder.prototype.onSelect = function (item) {
	this.onEnd(this);

	if (item.model.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Table) {
		// не делаем таблицы сложной формы.
		if (this.regionData.Coordinates.length > 1) {
			var externalRect = { X1: 1000000, X2: -1000000, Y1: 100000, Y2: -1000000 };
			for (var j = 0; j < this.regionData.Coordinates.length; j++) {
				if (externalRect.X1 > this.regionData.Coordinates[j].X1) {
					externalRect.X1 = this.regionData.Coordinates[j].X1;
				}

				if (externalRect.Y1 > this.regionData.Coordinates[j].Y1) {
					externalRect.Y1 = this.regionData.Coordinates[j].Y1;
				}

				if (externalRect.X2 < this.regionData.Coordinates[j].X2) {
					externalRect.X2 = this.regionData.Coordinates[j].X2;
				}

				if (externalRect.Y2 < this.regionData.Coordinates[j].Y2) {
					externalRect.Y2 = this.regionData.Coordinates[j].Y2;
				}
			}
			this.regionData.Coordinates = [externalRect];
		}

		this.regionData.Columns = [{
			Id: 0,
			LeftPos: this.regionData.Coordinates[0].X1,
			RightPos: this.regionData.Coordinates[0].X2
		}];
	}

	if (item.createInstance) {
		var instance = item.model.addRepeatedItem();
		this.page.createRegionForField(this.regionData, this.parentRegion, instance, this.regionText);

	} else {
		var parent = this.parentRegion;
		if ((!parent || !(parent instanceof FC.Viewmodels.DocumentEditor.TableRegionViewModel))
			&& item.model.parent
			&& item.model.parent instanceof FC.BusinessModels.DocumentEditor.TableRow) {
			var tableBlock = JSON.parse(JSON.stringify(this.regionData));
			tableBlock.Columns = [{
				Id: 0,
				LeftPos: this.regionData.Coordinates[0].X1,
				RightPos: this.regionData.Coordinates[0].X2
			}];
			parent = this.page.createRegionForField(tableBlock, parent, item.model.parent.parent, "", 0);
		}

		this.page.createRegionForField(this.regionData, parent, item.model, this.regionText);
	}

	this.page.draw();
};

FC.Viewmodels.DocumentEditor.RegionBinder.prototype.addUnattachedItems = function (items) {
	if (!items) {
		return;
	}

	for (var i = 0; i < items.length; i++) {
		var existing = this.items.filter(function (it) { return it.model === items[i].model; });
		if (!existing || existing.length === 0) {
			if (items[i].afterSeparator) {
				var separators = this.items.filter(function (it) { return it.isSeparator; });
				if (!separators || separators.length === 0) {
					this.items.push({ isSeparator: true, text: FC.Environment.Localization.GlobalResources.OtherFields, model: null });
				}
			}

			this.items.push(items[i]);
		}
	}
};

FC.Viewmodels.DocumentEditor.RegionBinder.prototype.findUnattachedInFields = function (page, fields, afterSeparator) {
	// добавим инстансы в выборку
	var k = 0;
	fields = fields.slice(0);
	var items = [];

	var length = fields.length;
	while (k < length) {
		if (fields[k].fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent) {
			var instances = fields[k].fields();

			for (var l = 0; l < instances.length; l++) {
				fields.push(instances[l]);
			}
		}

		k++;
	}

	var fieldRegions = page.documentModel.manipulators.regionsManipulator.fieldRegionMap();
	for (var j = 0; j < fields.length; j++) {
		var field = fields[j];

		if (!field.isVisible()) {
			continue;
		}

		if (field.canHaveRegion() &&
			(field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent ||
				field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Table)) {

			items.push({
				text: field.caption(),
				model: field,
				createInstance: field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent,
				afterSeparator: afterSeparator
			});
			continue;
		}

		if ((field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Boolean ||
			field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Picture ||
			field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Text) &&
			!field.canHaveRegion()) {
			continue;
		}

		if (!field.canHaveRegion()) {
			var childItems = this.findUnattachedInFields(page, field.fields(), afterSeparator);
			for (var k = 0; k < childItems.length; k++) {
				items.push(childItems[k]);
			}
			continue;
		}

		var currentFieldRegions = fieldRegions.filter(function (f) {
			if (f.field !== field || !f.region.status()) {
				return false;
			}

			var coords = f.region.coordinates();

			return coords &&
				coords.length > 0 &&
				(coords.length > 1 ||
					coords[0].x1() > 0 ||
					coords[0].y1() > 0 ||
					coords[0].x2() < page.width() ||
					coords[0].y2() < page.height());
		});

		if (field.maxRegions() >= 0 && currentFieldRegions.length >= field.maxRegions()) {
			continue;
		}


		if (!field.parent ||
			field.parent.fieldType !== FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent) {

			items.push({ text: field.caption(), model: field, createInstance: false, afterSeparator: afterSeparator });
		} else {
			var index = field.parent.fields().indexOf(field);
			items.push({ text: field.caption() + " [" + (index + 1) + "]", model: field, createInstance: false, afterSeparator: afterSeparator });
		}

	}

	return items;
};

FC.Viewmodels.DocumentEditor.RegionBinder.prototype.findUnattachedInTableGroup = function (tableField) {
	if (!tableField) {
		return;
	}

	var rects = ko.toJS(this.rectangles);
	var yMin = 1000000, yMax = -1000000, xMin = 1000000, xMax = -10000000;
	rects.forEach(function (r) {
		if (r.y1 < yMin) {
			yMin = r.y1;
		}

		if (r.y2 > yMax) {
			yMax = r.y2;
		}

		if (r.x1 < xMin) {
			xMin = r.x1;
		}

		if (r.x2 > xMax) {
			xMax = r.x2;
		}
	});

	var rect = { y1: yMin, y2: yMax, x1: xMin, x2: xMax };

	var rowsInQuestion = this.parentRegion.rows().filter(function (r) {
		return r.topSeparator() <= rect.y2 && r.bottomSeparator() >= rect.y1 && r.rowModel();
	});

	if (rowsInQuestion.length === 0 || rowsInQuestion.length > 2) {
		return;
	}

	var row = rowsInQuestion[0];
	if (rowsInQuestion.length === 2) {
		var height1 = rowsInQuestion[0].bottomSeparator() - rect.y1;
		var height2 = rect.y2 - rowsInQuestion[1].topSeparator();

		if (height2 > height1) {
			row = rowsInQuestion[1];
		}
	}

	var items = this.findUnattachedInFields(this.page, row.rowModel().cell(), false);
	if (items.length < 2) {
		this.addUnattachedItems(items);
		return;
	}

	var columnInQuestion = this.parentRegion.columns().filter(function (c) {
		return c.leftPos() <= rect.x1 && c.rightPos() >= rect.x2;
	});

	if (columnInQuestion.length === 0) {
		this.addUnattachedItems(items);
		return;
	}
	var left = columnInQuestion[0].leftPos();
	var right = columnInQuestion[0].rightPos();
	var ids = [];
	var cells = this.parentRegion.regions();
	var i;
	for (i = 0; i < cells.length; i++) {
		var f = cells[i].field();
		if (!f) {
			continue;
		}
		var cellRect = cells[i].getExternalRect();
		if (cellRect.x1 >= left && cellRect.x2 <= right) {
			var id = f.id();
			if (ids.filter(function (idd) { return idd === id }).length === 0) {
				ids.push(id);
			}
		}
	}

	if (ids.length === 0) {
		this.addUnattachedItems(items);
		return;
	}

	var newItems = [];
	for (i = 0; i < items.length; i++) {
		if (items[i].model && ids.indexOf(items[i].model.id()) > -1) {
			items[i].boldify = true;
			newItems.unshift(items[i]);
		} else {
			newItems.push(items[i]);
		}
	}

	newItems.sort(function (item1, item2) {
		if (item1.boldify) {
			if (item2.boldify) {
				return item1.text > item2.text ? 1 : -1;
			}

			return -1;
		}

		if (item2.boldify) {
			return 1;
		}

		return item1.text > item2.text ? 1 : -1;
	});

	this.addUnattachedItems(newItems);
};

FC.Viewmodels.DocumentEditor.RegionBinder.prototype.fillItems = function () {
	var t0 = performance.now();
	var fields = [];
	var page = this.page;
	var parentRegion = this.parentRegion;
	var doc = page.documentModel;
	this.items = [];
	var hasSeparator = false;

	if (parentRegion) {
		var parentFieldRegions = doc.manipulators.regionsManipulator.fieldRegionMap()
			.filter(function (val) { return val.region === parentRegion.model; });

		if (parentFieldRegions.length === 0 || !parentFieldRegions[0].field.hasOwnProperty("fields") && !parentFieldRegions[0].field.isRepeatableGroup) {
			return;
		}

		if (!parentFieldRegions[0].field.hasOwnProperty("fields")) {
			this.findUnattachedInTableGroup(parentFieldRegions[0].field);
		} else {
			fields = parentFieldRegions[0].field.fields().slice(0);
			this.addUnattachedItems(this.findUnattachedInFields(page, fields, false));
		}

		FC.Utils.EventLogger.Default.addMessage("RGPANEL",
			"regionBider fillItems\t" + (performance.now() - t0) + "\tms");

		hasSeparator = this.items.length > 0;
	}

	fields = [];
	var sections = [];
	var secFields = doc.fields();
	var i;
	for (i = 0; i < secFields.length; i++) {
		if (secFields[i] instanceof FC.BusinessModels.DocumentEditor.SectionsGroup) {
			var secInstances = secFields[i].fields();
			for (var j = 0; j < secInstances.length; j++) {
				if (secInstances[j].pageIds() && secInstances[j].pageIds().indexOf(page.id()) >= 0) {
					sections.push(secInstances[j]);
				}
			}
		} else {
			if (secFields[i].pageIds() && secFields[i].pageIds().indexOf(page.id()) >= 0) {
				sections.push(secFields[i]);
			}
		}
	}

	for (i = 0; i < sections.length; i++) {
		var sectionFields = sections[i].fields();
		for (var j = 0; j < sectionFields.length; j++) {
			fields.push(sectionFields[j]);
		}
	}

	this.addUnattachedItems(this.findUnattachedInFields(page, fields, hasSeparator));

	FC.Utils.EventLogger.Default.addMessage("RGPANEL",
		"regionBider fillItems\t" + (performance.now() - t0) + "\tms");
};

FC.Viewmodels.DocumentEditor.RegionBinder.prototype.show = function () {
	var field = this.page.selectedField();
	if (field &&
		field.fieldType < 4 &&
		!(field.parent instanceof FC.BusinessModels.DocumentEditor.TableRow && !field.parent.parent.isRepeatableGroup)) {

		var parent = field;
		while (parent.parent != null) {
			parent = parent.parent;
		}

		if (!parent.hasOwnProperty("pageIds")) {
			console.log("bad parent");
		} else if (parent.pageIds().indexOf(this.page.id()) > -1) {
			var unattached = this.findUnattachedInFields(this.page, [field], false);
			if (unattached.length > 0) {
				this.addUnattachedItems(unattached);
				this.onSelect(this.items[0]);
				this.onBinderClose();
				this.page.directDrawer.clear();
				this.page.draw();
				return;
			}
		}
	}

	if (FC.Environment.DisableRegionBinding) {
		this.onBinderClose();
		this.page.directDrawer.clear();
		this.page.draw();
		return;
	}

	this.fillItems();
	if (this.items.length === 0) {
		this.onBinderClose();
		this.page.directDrawer.clear();
		this.page.draw();
		return;
	}

	this.showBase();
};

