﻿var FC = FC || {};
FC.Viewmodels = FC.Viewmodels || {};
FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

FC.Viewmodels.DocumentEditor.RegionCreator = function (page, onMouseMove, onMouseUp) {

	this.startPoint = null;

	var self = this;
	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
	var handlerRemovalKey = "regionDrawing";

	this.regionDrawing = function (point, currentZone) {
		if (point.x < 0 || point.y < 0 || point.x > page.width() || point.y > page.height()) {
			return true;
		}

		page.directDrawer.clear();

		var rectangle = FC.Viewmodels.ImageEditor.Rectangle.getRectangleFromPoints(point, self.startPoint);

		if (FC.Viewmodels.DocumentEditor.RegionCreator.regionTooSmallToCreate(rectangle)) {
			return true;
		}

		if (onMouseMove) {
			onMouseMove(rectangle, currentZone, page);
		}

		page.directDrawer.addDrawingCommand(function (layer) { layer.drawRectangle(rectangle, 1, "#48ba00", null, [5, 5])});
		page.draw(1);
		return false;
	};

	this.endRegionDrawing = function (point, currentZone, event) {
		docEditor.removeOnMouseUp(handlerRemovalKey);
		docEditor.removeOnMouseLeave(handlerRemovalKey);
		docEditor.removeOnMouseMove(handlerRemovalKey);

		if (point.x < 0) {
			point.x = 0;
		}

		if (point.y < 0) {
			point.y = 0;
		}

		if (point.x > page.width()) {
			point.x = page.width();
		}

		if (point.y > page.height()) {
			point.y = page.height();
		}

		var rectangle = FC.Viewmodels.ImageEditor.Rectangle.getRectangleFromPoints(point, self.startPoint);
		if (onMouseUp) {
			onMouseUp(rectangle, currentZone ? currentZone.target : null, page, event);
			return false;
		}

		return true;
	};

	this.startRegionDrawing = function (point, currentZone) {

		if (point.x < 0 || point.y < 0 || point.x > page.width() || point.y > page.height()) {
			return;
		}

		self.startPoint = point;
		var mouseMove = function (data) { self.regionDrawing(data, currentZone); };
		var mouseUp = function (data, event) { self.endRegionDrawing(data, currentZone, event); };

		docEditor.addOnMouseUp(mouseUp.bind(this), 101, handlerRemovalKey);
		docEditor.addOnMouseLeave(mouseUp.bind(this), 101, handlerRemovalKey);
		docEditor.addOnMouseMove(mouseMove.bind(this), 101, handlerRemovalKey);
	};
};

FC.Viewmodels.DocumentEditor.RegionCreator.delta = 5;

FC.Viewmodels.DocumentEditor.RegionCreator.regionTooSmallToCreate = function (rectangle) {
	return rectangle.x2() - rectangle.x1() < FC.Viewmodels.DocumentEditor.RegionCreator.delta &&
		rectangle.y2() - rectangle.y1() < FC.Viewmodels.DocumentEditor.RegionCreator.delta;
};