﻿var FC = FC || {};
FC.Viewmodels = FC.Viewmodels || {};
FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

FC.Viewmodels.DocumentEditor.RegionViewModel = function (region, parentRegion, page, depth) {

	FC.Viewmodels.DocumentEditor.RegionBase.call(this);

	this.model = region;
	this.depth = depth;
	this.parentRegion = parentRegion;
	this.page = page;
	this.rectangles = ko.observableArray([]);
	this.resized = false;

	this.id = ko.observable(region.id());
	this._id = FC.Viewmodels.DocumentEditor.RegionViewModel.GlobalCounter++;

	this.field = ko.observable();
	var fieldRegions = page.documentModel.manipulators.regionsManipulator.fieldRegionMap()
		.filter(function (val) { return val.region === region; });

	if (fieldRegions.length > 0) {
		this.field(fieldRegions[0].field);
	}

	this.rectangles = region.coordinates;

	FC.Utils.polygonCommon.normalizeRectangles(this.rectangles);

	this.highlighted = ko.observable(false);

	var self = this;
	this.caption = ko.computed(function () {
		var field = self.field();
		if (field &&
			field.parent &&
			field.parent.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent) {
			return region.caption() + "[" + (field.parent.fields().indexOf(field) + 1) + "]";
		} else {
			return region.caption();
		}
	}, this);

	this.page.subscriptions.push({ key: this, value: this.caption });

	this.page.subscriptions.push({
		key: this,
		value:
		this.model.status.subscribe(function (newStatus) {
			if (!newStatus) {
				page.deleteRegion(self, true);
			}
		})
	});

	this.points = null;

	// если поле находится в инстансе повторящейся группы,
	// то для региона доступен continue line items
	// если регион выделен
	var field = self.field();
	if (field &&
		field.parent &&
		field.parent.parent &&
		field.parent.parent.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent) {

		this.selected.subscribe(function(newValue) {
			var repeatable = field.parent.parent;
			var instance = field.parent;
			var section = repeatable.getCurrentSection();
			var sectionIsFlexible = section.isFlexible();

			var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
			var index = repeatable.fields().indexOf(instance);
			if (newValue && sectionIsFlexible) {
				docEditor.extender.selectedTableRow(index);
				docEditor.extender.selectedTableRowPath(repeatable.getNodeRef());
				docEditor.extender.selectedTableCanReanalyze(false);
			} else {
				if (docEditor.extender.selectedTableRow() === index || !sectionIsFlexible) {
					docEditor.extender.selectedTableRow(-1);
					docEditor.extender.selectedTableRowPath("");
				}
			}
		});

		this.page.subscriptions.push({
			key: this.target,
			value: this.selected
		});
	}
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype = Object.create(FC.Viewmodels.DocumentEditor.RegionBase.prototype);

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.calculatePolygonPoints = function() {
	this.points = FC.Utils.polygonCommon.getPolygonPoints(this.rectangles(), this.page);
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.createBorderZones = function () {
	var rects = this.rectangles();
	var zones = [];

	if (rects.length === 1) {
		zones.push(new FC.Viewmodels.ActionZones.BorderBottomLeftZone([rects[0]], this, this.page, this.depth));
		zones.push(new FC.Viewmodels.ActionZones.BorderBottomMiddleZone([rects[0]], this, this.page, this.depth));
		zones.push(new FC.Viewmodels.ActionZones.BorderBottomRightZone([rects[0]], this, this.page, this.depth));
		zones.push(new FC.Viewmodels.ActionZones.BorderMiddleLeftZone([rects[0]], this, this.page, this.depth));
		zones.push(new FC.Viewmodels.ActionZones.BorderMiddleRightZone([rects[0]], this, this.page, this.depth));
		zones.push(new FC.Viewmodels.ActionZones.BorderTopLeftZone([rects[0]], this, this.page, this.depth));
		zones.push(new FC.Viewmodels.ActionZones.BorderTopMiddleZone([rects[0]], this, this.page, this.depth));
		zones.push(new FC.Viewmodels.ActionZones.BorderTopRightZone([rects[0]], this, this.page, this.depth));
		return zones;
	}

	var points = this.points;

	for (var i = 0; i < points.length; i++) {
		var currentPoint = points[i];
		var previousPoint = points[i === 0 ? points.length - 1 : i - 1];
		var nextPoint = points[i === points.length - 1 ? 0 : i + 1];

		var point1 = currentPoint.pointType ? previousPoint : nextPoint;
		var point2 = currentPoint.pointType ? nextPoint : previousPoint;

		zones.push(new FC.Viewmodels.ActionZones.PolygonCornerZone(
			currentPoint,
			point1,
			point2,
			points,
			i,
			this,
			this.page,
			this.depth));

		zones.push(new FC.Viewmodels.ActionZones.PolygonMiddleZone(
			currentPoint,
			nextPoint,
			points,
			i,
			this,
			this.page,
			this.depth));
	}

	return zones;
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.createBasicZones = function () {

	this.calculatePolygonPoints();

	var zones = this.createBorderZones();
	zones.push(new FC.Viewmodels.ActionZones.DeleteZone(this.rectangles, this, this.page, this.depth));
	return zones;
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.createZones = function () {
	if (!this.isVisible()) {
		return [];
	}

	var basicZones = this.createBasicZones();
	basicZones.push(new FC.Viewmodels.ActionZones.RegionBodyZone(this.rectangles, this, this.page, this.depth));
	return basicZones;
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.resizeRegionBase = function (delta, internalBorder, rectsToChange) {
	var parent = { x1: 0, y1: 0, x2: this.page.width(), y2: this.page.height() };
	var realDelta = this.getDeltaForResize(delta, parent, internalBorder);

	var rects = this.rectangles();
	for (var i = 0; i < rects.length; i++) {
		if (!rectsToChange || rectsToChange.indexOf(rects[i]) > -1) {
			rects[i].x1(rects[i].x1() + realDelta.x1);
			rects[i].x2(rects[i].x2() + realDelta.x2);
			rects[i].y1(rects[i].y1() + realDelta.y1);
			rects[i].y2(rects[i].y2() + realDelta.y2);
		}
	}

	if (realDelta.x1 !== 0 || realDelta.x2 !== 0 || realDelta.y1 !== 0 || realDelta.y2 !== 0) {
		FC.Viewmodels.EventStack.addEvent();
		var field = this.field();
		if (field) {
			field.needRecognize(true);
		}
		this.resized = true;
	}

	return realDelta;
};

/**
 * Получить индекс текущего параграфа в связаном поле
 */
FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.getParagraphIndex = function () {
	var field = this.field();
	// параграфы есть только в текстовых полях
	if (!(field instanceof FC.BusinessModels.DocumentEditor.TextField)) {
		return -1;
	}

	// получим все модели регионов для текущего поля и отсортируем их геометрически
	var allFieldRegions = this.page.documentModel.manipulators.regionsManipulator.fieldRegionMap()
		.filter(function (val) {
			return val.field === field && val.region.status();
		})
		.map(function (fr) { return fr.region; })
		.sort(FC.BusinessModels.DocumentEditor.Region.compareRegions);

	// если регионов нет или он один - параграфы не нужны
	if (allFieldRegions.length < 2) {
		return -1;
	}

	// найдём индекс текущего региона в отсортированном списке
	for (var i = 0; i < allFieldRegions.length; i++) {
		if (allFieldRegions[i] === this.model) {
			return i;
		}
	}

	return -1;
};

/**
 * Обновить текст у связанного поля
 */
FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.updateTextBase = function () {
	var field = this.field();
	this.resized = false;

	// текст обновляем только у текстовых полей
	if (field instanceof FC.BusinessModels.DocumentEditor.TextField) {
		var rects = ko.toJS(this.rectangles());

		// сортируем прямоугольники по вертикали
		rects.sort(function (a, b) { return a.y1 - b.y1; });

		// определяем строки - т.е. если прямоугольники пересекаются по вертикали, то они собираются в одну строку
		var lines = [];
		var i;
		for (i = 0; i < rects.length; i++) {
			var currentLine = null;
			for (var j = 0; j < lines.length; j++) {
				if (lines[j].y1 < rects[i].y2 && lines[j].y2 > rects[i].y1) {
					currentLine = lines[j];
					break;
				}
			}

			if (!currentLine) {
				lines.push({ y1: rects[i].y1, y2: rects[i].y2, rects: [rects[i]] });
				continue;
			}

			currentLine.rects.push(rects[i]);
			if (currentLine.y1 > rects[i].y1) {
				currentLine.y1 = rects[i].y1;
			}

			if (currentLine.y2 < rects[i].y2) {
				currentLine.y2 = rects[i].y2;
			}
		}

		// из строк собираем текст
		var text = "";
		var separator = "\n";
		var wordsSeparator = " ";
		for (i = 0; i < lines.length; i++) {
			lines[i].rects.sort(function (r1, r2) { return r1.x1 - r2.x1; });
			for (var j = 0; j < lines[i].rects.length; j++) {
				if (text.length > 0 && text[text.length - 1] !== wordsSeparator && text[text.length - 1] !== separator) {
					text += wordsSeparator;
				}

				text += FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextInsideRectangle(lines[i].rects[j], this.page);
			}

			if (text.length > 0 && text[text.length - 1] !== separator && i < lines.length - 1) {
				text += separator;
			}
		}

		// обновляем текст в поле
		field.setParagraphText(text, this.getParagraphIndex());
	}
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.updateText = function () {
	this.updateTextBase();
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.getExternalRectBase = function () {
	var rects = this.rectangles();

	// найдём крайние точки полигона
	var bigNumber = 1000000;
	var minX = bigNumber, minY = bigNumber, maxX = -bigNumber, maxY = -bigNumber;

	for (var i = 0; i < rects.length; i++) {
		if (rects[i].x1() < minX) {
			minX = rects[i].x1();
		}

		if (rects[i].x2() > maxX) {
			maxX = rects[i].x2();
		}

		if (rects[i].y1() < minY) {
			minY = rects[i].y1();
		}

		if (rects[i].y2() > maxY) {
			maxY = rects[i].y2();
		}
	}

	return {
		x1: minX,
		x2: maxX,
		y1: minY,
		y2: maxY
	};
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.getExternalRect = function () {
	return this.getExternalRectBase();
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.getDeltaForResize = function (delta, border, internalBorder) {
	var isMoving = delta.x1 === delta.x2 && delta.y1 === delta.y2;
	var rect = isMoving ? this.getExternalRect() : this.getExternalRectBase();

	if (rect.x1 + delta.x1 < border.x1) {
		if (isMoving) {
			delta.x2 = border.x1 - rect.x1;
		}

		delta.x1 = border.x1 - rect.x1;
	}

	if (rect.x1 + delta.x1 > rect.x2 + delta.x2) {
		if (Math.abs(delta.x2) > Math.abs(delta.x1)) {
			delta.x2 = rect.x1 + FC.Viewmodels.DocumentEditor.RegionCreator.delta - rect.x2;
		} else {
			delta.x1 = rect.x2 - FC.Viewmodels.DocumentEditor.RegionCreator.delta - rect.x1;
		}
	}

	if (rect.y1 + delta.y1 < border.y1) {
		if (isMoving) {
			delta.y2 = border.y1 - rect.y1;
		}

		delta.y1 = border.y1 - rect.y1;
	}

	if (rect.y1 + delta.y1 > rect.y2 + delta.y2) {
		if (Math.abs(delta.y2) > Math.abs(delta.y1)) {
			delta.y2 = rect.y1 + FC.Viewmodels.DocumentEditor.RegionCreator.delta - rect.y2;
		} else {
			delta.y1 = rect.y2 - FC.Viewmodels.DocumentEditor.RegionCreator.delta - rect.y1;
		}

	}

	if (rect.x2 + delta.x2 > border.x2) {
		if (isMoving) {
			delta.x1 = border.x2 - rect.x2;
		}

		delta.x2 = border.x2 - rect.x2;
	}

	if (rect.y2 + delta.y2 > border.y2) {
		if (isMoving) {
			delta.y1 = border.y2 - rect.y2;
		}


		delta.y2 = border.y2 - rect.y2;
	}

	if (internalBorder && !isMoving) {
		if (rect.x1 + delta.x1 > internalBorder.x1) {
			delta.x1 = internalBorder.x1 - rect.x1;
		}

		if (rect.x2 + delta.x2 < internalBorder.x2) {
			delta.x2 = internalBorder.x2 - rect.x2;
		}

		if (rect.y1 + delta.y1 > internalBorder.y1) {
			delta.y1 = internalBorder.y1 - rect.y1;
		}

		if (rect.y2 + delta.y2 < internalBorder.y2) {
			delta.y2 = internalBorder.y2 - rect.y2;
		}
	}

	return delta;
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.resizeRegion = function (delta, resizeChildren, rect) {
	return this.resizeRegionBase(delta, null, rect);
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.moveRegion = function (deltaX, deltaY) {
	return this.resizeRegion({ x1: deltaX, y1: deltaY, x2: deltaX, y2: deltaY }, true);
};

FC.Viewmodels.DocumentEditor.RegionViewModel.prototype.checkAndCorrectRectangles = function () {
	var rectangles = this.rectangles();
	if (rectangles.length < 2) {
		return;
	}

	var rects = ko.toJS(rectangles);

	// проверим, что нет кривых прямоугольников
	var wrongDimentionsRects = rects.filter(function (r) { return r.x2 <= r.x1 || r.y2 <= r.y1; });

	if (wrongDimentionsRects.length > 0) {
		this.page.recalculateRegionRectangles(this, rectangles);
	}

	// проверим, что нет пересекающихся прямоугольников
	// и нет прямоугольников, которые не касаются других (т.е. рвут полигон)
	for (var i = 0; i < rects.length; i++) {
		var noIntersections = true;

		for (var j = 0; j < rects.length; j++) {
			if (i === j) {
				continue;
			}

			var rectA = rects[i];
			var rectB = rects[j];

			if (rectA.x1 < rectB.x2 &&
				rectB.x1 < rectA.x2 &&
				rectA.y1 < rectB.y2 &&
				rectB.y1 < rectA.y2) {
				this.page.recalculateRegionRectangles(this, rectangles);
				return;
			}

			// проверим, что прямоугольники соприкасаются
			if (!noIntersections ||
				rectA.x1 <= rectB.x2 &&
				rectB.x1 <= rectA.x2 &&
				rectA.y1 <= rectB.y2 &&
				rectB.y1 <= rectA.y2) {
				noIntersections = false;
			}
		}

		if (noIntersections) {
			this.page.recalculateRegionRectangles(this, rectangles);
			return;
		}
	}
};



FC.Viewmodels.DocumentEditor.RegionViewModel.GlobalCounter = 0;