﻿var FC = FC || {};
FC.Viewmodels = FC.Viewmodels || {};
FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

/**
 * Вьюмодель табличного региона
 * @param {} region модель региона
 * @param {} parentRegion вьюмодель региона-родителя
 * @param {} page страница, которой принадлежит регион
 * @param {} depth глубина (число)
 * @returns {} 
 */
FC.Viewmodels.DocumentEditor.TableRegionViewModel = function (region, parentRegion, page, depth) {
	// наследуем у региона группы
	FC.Viewmodels.DocumentEditor.GroupRegionViewModel.call(this, region, parentRegion, page, depth);

	var self = this;
	this.page.subscriptions.push({
		key: this,
		value: this.selected.subscribe(function (newValue) {
			if (!self.isVisible()) {
				return;
			}

			var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
			docEditor.extender.tableEditing(newValue);

			if (!newValue) {
				docEditor.extender.deleteSeparatorActivated(false);
				docEditor.extender.verticalSeparatorDrawActivated(false);
				docEditor.extender.horizontalSeparatorDrawActivated(false);
			}
		})
	});

	this.table = this.field;

	// можно ли делать из этого региона полигон
	this.modifiable = false;

	this.horizontalSeparators = ko.observableArray([]);
	this.columns = ko.observableArray([]);
	this.rows = ko.observableArray([]);
	this.cellsSizesRelateToParent = true;

	var separators = region.horizontalSeparators();
	if (separators) {
		this.horizontalSeparators(ko.utils.arrayMap(separators,
			function (value) {
				return new FC.Viewmodels.DocumentEditor.HorizontalSeparator(value);
			}));
	}

	this.page.subscriptions.push({
		key: this,
		value: this.horizontalSeparators.subscribe(function (newSeparators) {
			var values = newSeparators.map(function (s) { return s.value; });
			region.horizontalSeparators(values);
		})
	});

	this.columns(region.columns());

	var findRowModel = function (rowModels, index) {
		if (!rowModels) {
			return null;
		}

		var result = rowModels.filter(function (r) { return r.rowNumber() == index; });
		if (result && result.length > 0) {
			return result[0];
		}

		return null;
	};

	this.createRows = function () {
		var rowModels = [];
		if (this.table()) {
			rowModels = this.table().rows()
				.filter(function (r) { return JSON.stringify(r.tableBlockId()) === JSON.stringify(region.id()); });
		};

		var horizontalSeparators = this.horizontalSeparators().map(function (sep) { return sep.value; });

		if (!this.rectangles() || this.rectangles().length === 0) {
			return;
		}

		var rect = this.rectangles()[0];
		var rows = [];
		var y1 = rect.y1;
		var y2 = rect.y2;

		horizontalSeparators.push(y2);

		for (var i = 0; i < horizontalSeparators.length; i++) {
			rows.push(new FC.Viewmodels.DocumentEditor.RowViewModel(y1, horizontalSeparators[i], this, findRowModel(rowModels, i)));
			y1 = horizontalSeparators[i];
		}

		this.rows(rows);
	};

	this.createRows();

	if (this.table()) {
		this.page.subscriptions.push({
			key: this,
			value:
				this.table().rows.subscribe(function (newRows) {
					// если удалили строку
					if (newRows.length !== self.rows().length) {
						var currentRows = self.rows();
						var deletedRows = currentRows.filter(function (r) {
							return r.rowModel() && newRows.indexOf(r.rowModel()) === -1;
						});
						for (var i = 0; i < deletedRows.length; i++) {
							self.unbindRow(deletedRows[i], false);
						}

						if (deletedRows.length === 0) {
							var cells = self.regions();
							var needRedraw = false;
							for (var j = 0; j < cells.length; j++) {
								var frMap = cells[j].field().manipulators.regionsManipulator.fieldRegionMap();
								if (frMap.filter(function (fr) { return fr.field === cells[j].field(); }).length === 0) {
									needRedraw = true;
									self.page.removeRegionZones(cells[j]);
									cells.splice(j, 1);
									j--;
								}
							}

							if (needRedraw) {
								self.page.draw();
							}
						}
					}
				})
		});
	}
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype = Object.create(FC.Viewmodels.DocumentEditor.GroupRegionViewModel.prototype);

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.createBasicZones = function () {
	var zones = this.createBorderZones();
	zones.push(new FC.Viewmodels.ActionZones.DeleteTableZone(this.rectangles, this, this.page, this.depth));
	return zones;
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.createCommonTableZones = function () {
	var zones = [];

	var childRegions = this.regions();
	for (var j = 0; j < childRegions.length; j++) {
		zones = zones.concat(childRegions[j].createZones());
	}

	if (this.isVisible()) {
		zones = zones.concat(FC.Viewmodels.DocumentEditor.TableRegionViewModel.createRowSeparators(this));
		zones = zones.concat(FC.Viewmodels.DocumentEditor.TableRegionViewModel.createColumnSeparatorZones(this));
		zones = zones.concat(FC.Viewmodels.DocumentEditor.TableRegionViewModel.createColumnHeaderZones(this));
		zones = zones.concat(FC.Viewmodels.DocumentEditor.TableRegionViewModel.createRowHeaderZones(this));

		zones = zones.concat(this.createBasicZones());
		zones.push(new FC.Viewmodels.ActionZones.GroupBodyMovableZone(this.rectangles, this, this.page, this.depth + 10));
	}

	return zones;
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.createZones = function () {
	if (!this.isVisible()) {
		return [];
	}

	var zones = this.createCommonTableZones();
	zones.push(new FC.Viewmodels.ActionZones.TableBodyEditableZone(this.rectangles, this, this.page, this.depth));

	return zones;
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.recalculateColumnSeparators = function () {
	FC.Viewmodels.DocumentEditor.TableRegionViewModel.recalculateRegionZonesByType(
		this.page,
		this,
		function (zone) { return zone instanceof FC.Viewmodels.ActionZones.VerticalSeparatorZone; },
		FC.Viewmodels.DocumentEditor.TableRegionViewModel.createColumnSeparatorZones);
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.recalculateColumnHeaders = function () {
	FC.Viewmodels.DocumentEditor.TableRegionViewModel.recalculateRegionZonesByType(
		this.page,
		this,
		function (zone) { return zone instanceof FC.Viewmodels.ActionZones.ColumnHeaderZone; },
		FC.Viewmodels.DocumentEditor.TableRegionViewModel.createColumnHeaderZones);
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.recalculateRowHeaders = function () {
	FC.Viewmodels.DocumentEditor.TableRegionViewModel.recalculateRegionZonesByType(
		this.page,
		this,
		function (zone) { return zone instanceof FC.Viewmodels.ActionZones.RowHeaderZone; },
		function (region) {
			region.createRows();
			return FC.Viewmodels.DocumentEditor.TableRegionViewModel.createRowHeaderZones(region);
		});
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.recalculateRowSeparators = function () {
	FC.Viewmodels.DocumentEditor.TableRegionViewModel.recalculateRegionZonesByType(
		this.page,
		this,
		function (zone) { return zone instanceof FC.Viewmodels.ActionZones.HorizontalSeparatorZone; },
		FC.Viewmodels.DocumentEditor.TableRegionViewModel.createRowSeparators);
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.resizeRegion = function (delta, resizeChildren, rect) {
	var oldRect = ko.toJS(this.rectangles()[0]);
	var resizeInfo = this.resizeRegionBase(delta, this.getUnresizableRectangle(), rect);
	var columns = this.columns();

	if (resizeChildren) {
		var childRegions = this.regions();
		var j;
		for (j = 0; j < childRegions.length; j++) {
			childRegions[j].resizeRegion(resizeInfo, resizeChildren);
		}


		var horizontalSeparators = this.horizontalSeparators();
		for (j = 0; j < horizontalSeparators.length; j++) {
			horizontalSeparators[j].value(horizontalSeparators[j].value() + resizeInfo.y1);
		}

		for (j = 0; j < columns.length; j++) {
			columns[j].leftPos(columns[j].leftPos() + resizeInfo.x1);
			columns[j].rightPos(columns[j].rightPos() + resizeInfo.x1);
		}

		return resizeInfo;
	}

	var newRect = {
		x1: oldRect.x1 + resizeInfo.x1,
		y1: oldRect.y1 + resizeInfo.y1,
		x2: oldRect.x2 + resizeInfo.x2,
		y2: oldRect.y2 + resizeInfo.y2
	};

	if (newRect.x1 !== oldRect.x1) {
		if (columns.length > 0 && columns[0].leftPos() === oldRect.x1) {
			columns[0].leftPos(newRect.x1);
		}

		if (this.cellsSizesRelateToParent) {
			if (newRect.y1 !== oldRect.y1) {
				this.resizeCells(oldRect.x1, newRect.x1, oldRect.y1, newRect.y1);
			} else {
				this.resizeCellsX(oldRect.x1, newRect.x1);
			}
		}
	} else {
		if (this.cellsSizesRelateToParent) {
			if (newRect.y1 !== oldRect.y1) {
				this.resizeCellsY(oldRect.y1, newRect.y1);
			}
		}
	}

	if (newRect.x2 !== oldRect.x2) {
		if (columns.length > 0 && columns[columns.length - 1].rightPos() === oldRect.x2) {
			columns[columns.length - 1].rightPos(newRect.x2);
		}

		if (this.cellsSizesRelateToParent) {
			if (newRect.y2 !== oldRect.y2) {
				this.resizeCells(oldRect.x2, newRect.x2, oldRect.y2, newRect.y2);
			} else {
				this.resizeCellsX(oldRect.x2, newRect.x2);
			}
		}
	} else {
		if (this.cellsSizesRelateToParent) {
			if (newRect.y2 !== oldRect.y2) {
				this.resizeCellsY(oldRect.y2, newRect.y2);
			}
		}
	}

	return resizeInfo;
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.getUnresizableRectangle = function () {
	var minSize = 3;

	var horizontalSeparators = this.horizontalSeparators();
	var columns = this.columns();
	var externalRect = this.getExternalRectBase();

	if (columns.length === 0 ||
		columns.length === 1 && columns[0].leftPos() === externalRect.x1 && columns[0].rightPos() === externalRect.x2) {
		var x = externalRect.x2;
		externalRect.x2 = externalRect.x1 + minSize;
		externalRect.x1 = x - minSize;
	} else {
		externalRect.x1 = columns[0].rightPos() - minSize;
		if (columns.length > 1) {
			if (columns[columns.length - 1].rightPos() === externalRect.x2) {
				externalRect.x2 = columns[columns.length - 1].leftPos() + minSize;
			} else {
				externalRect.x2 = columns[columns.length - 1].rightPos() + minSize;
			}
		} else {
			externalRect.x2 = externalRect.x1 + minSize;
		}
	}

	if (horizontalSeparators.length !== 0) {
		externalRect.y1 = horizontalSeparators[0].value() - minSize;
		externalRect.y2 = horizontalSeparators[horizontalSeparators.length - 1].value() + minSize;
	} else {
		var y = externalRect.y2;
		externalRect.y2 = externalRect.y1 + minSize;
		externalRect.y1 = y - minSize;
	}

	return externalRect;
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.resizeCells = function (oldX, newX, oldY, newY) {
	var cells = this.regions();
	for (var i = 0; i < cells.length; i++) {
		var rects = cells[i].rectangles();
		var hasChanges = false;

		for (var j = 0; j < rects.length; j++) {
			if (newX >= 0 && rects[j].x1() === oldX && oldX !== newX) {
				rects[j].x1(newX);
				hasChanges = true;
			}

			if (newX >= 0 && rects[j].x2() === oldX && oldX !== newX) {
				rects[j].x2(newX);
				hasChanges = true;
			}

			if (newY >= 0 && rects[j].y1() === oldY && oldY !== newY) {
				rects[j].y1(newY);
				hasChanges = true;
			}

			if (newY >= 0 && rects[j].y2() === oldY && oldY !== newY) {
				rects[j].y2(newY);
				hasChanges = true;
			}
		}

		if (hasChanges) {
			cells[i].onCellChange();
		}
	}
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.resizeCellsX = function (oldX, newX) {
	this.resizeCells(oldX, newX, -1, -1);
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.resizeCellsY = function (oldY, newY) {
	this.resizeCells(-1, -1, oldY, newY);
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.resizeCellsOnTop = function (oldY, newY) {
	var cells = this.regions();
	for (var i = 0; i < cells.length; i++) {
		var hasChanges = false;

		var rects = cells[i].rectangles();
		for (var j = 0; j < rects.length; j++) {
			if (rects[j].y2() === oldY) {
				rects[j].y2(newY);
				hasChanges = true;
			}
		}

		if (hasChanges) {
			cells[i].onCellChange();
		}
	}
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.resizeCellsOnLeft = function (oldX, newX) {
	var cells = this.regions();
	for (var i = 0; i < cells.length; i++) {
		var hasChanges = false;

		var rects = cells[i].rectangles();
		for (var j = 0; j < rects.length; j++) {
			if (newX >= 0 && rects[j].x2() === oldX) {
				rects[j].x2(newX);
				hasChanges = true;
			}
		}

		if (hasChanges) {
			cells[i].onCellChange();
		}
	}
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.bindColumn = function (column, event) {
	this.page.setCurrentCursorAndCaption(null);
	var binder = new FC.Viewmodels.DocumentEditor.ColumnBinder(this, column, event, this.page);
	binder.show();
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.unbindColumn = function (column) {
	var t0 = performance.now();

	var i;
	var rows = this.rows();
	for (i = 0; i < rows.length; i++) {
		if (rows[i].rowModel()) {
			this.deleteRowRegions(rows[i].rowModel(), column.id());
		}
	}

	var cells = this.regions();
	i = 0;
	while (i < cells.length) {
		var field = cells[i].field();
		if (field && field.id() === column.id()) {
			this.page.deleteRegion(cells[i]);
		} else {
			i++;
		}
	}

	column.id(0);

	FC.Utils.EventLogger.Default.addMessage("RGPANEL",
		"TableRegionViewModel unbindColumn\t" + (performance.now() - t0) + "\tms");

	this.page.draw();
};

/**
 * Определить индекс новой строки
 * @param {} table модель поля таблицы
 * @param {} blockId ИД табличного региона
 * @param {} blockExternalRect охватывающий прямоугольник табличного региона
 * @param {} horizontalSeparators горизонтальные сепараторы
 * @param {} regionPageIndex индекс страницы в документе
 * @param {} bottomSeparator нижний сепаратор строки
 * @param {} regionModel модель табличного региона
 * @returns {} 
 */
FC.Viewmodels.DocumentEditor.TableRegionViewModel.findIndexForNewRow = function (table, blockId, blockExternalRect,
	horizontalSeparators, regionPageIndex, bottomSeparator, regionModel) {

	var index = -1;
	var currentIdJson = JSON.stringify(blockId);
	var currentRows = table.rows();
	var separators = horizontalSeparators.map(function (s) { return s.value(); });

	var rowNumber = 0;
	// если нет сепараторов - строка в регионе одна и по-любому считать номер по другим регионам
	if (separators.length !== 0) {
		separators.push(blockExternalRect.y2);
		rowNumber = separators.indexOf(bottomSeparator);

		// если строка - первая, то нужно искать предыдущий регион
		if (rowNumber > 0) {
			var previousRowNumber = -1;
			for (var i = 0; i < currentRows.length; i++) {
				var currentRowNumber = currentRows[i].rowNumber();

				if (currentRowNumber > previousRowNumber &&
					currentRowNumber < rowNumber &&
					JSON.stringify(currentRows[i].tableBlockId()) === currentIdJson) {

					previousRowNumber = currentRowNumber;
					index = i + 1;
				}
			}
		}
	}

	if (index > -1) {
		return { index: index, rowNumber: rowNumber }
	}

	// в этом тэйблблоке нет рядов
	var tableBlocks = table.manipulators.regionsManipulator
		.getFieldRegions(table)
		.map(function (fr) { return fr.region; });

	var pageIndexInQuestion = -1;
	var blockInQuestion = null;
	var rowsInCurrentBlock;

	for (var k = 0; k < tableBlocks.length; k++) {
		var pIndex = tableBlocks[k].regionPageIndex();
		if (pIndex > regionPageIndex || tableBlocks[k] === regionModel) {
			continue;
		}
		rowsInCurrentBlock = currentRows.filter(function (r) {
			return JSON.stringify(r.tableBlockId()) === JSON.stringify(tableBlocks[k].id());
		});

		if (rowsInCurrentBlock.length === 0) {
			continue;
		}

		if (pIndex > pageIndexInQuestion ||
			(blockInQuestion && FC.BusinessModels.DocumentEditor.TableRegion.compare(tableBlocks[k], blockInQuestion) > 0)) {
			if (pIndex < regionPageIndex || FC.BusinessModels.DocumentEditor.TableRegion.compare(tableBlocks[k], regionModel) < 0) {
				pageIndexInQuestion = pIndex;
				blockInQuestion = tableBlocks[k];
			}
		}
	}

	if (blockInQuestion) {
		rowsInCurrentBlock = currentRows.filter(function (r) {
			return JSON.stringify(r.tableBlockId()) === JSON.stringify(blockInQuestion.id());
		});
		return { index: currentRows.indexOf(rowsInCurrentBlock[rowsInCurrentBlock.length - 1]) + 1, rowNumber: 0 };
	}

	return { index: 0, rowNumber: 0 };
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.bindRow = function (row) {
	var t0 = performance.now();

	var rowInsertData = FC.Viewmodels.DocumentEditor.TableRegionViewModel.findIndexForNewRow(this.table(), this.id(),
		this.getExternalRect(), this.horizontalSeparators(), this.model.regionPageIndex(), row.bottomSeparator(), this.model);

	var index = rowInsertData.index;
	var rowModel = this.table().addTableRowAtPosition(index, this.id().slice(0));
	row.rowModel(rowModel);
	rowModel.rowNumber(rowInsertData.rowNumber);

	var cells = rowModel.cell();
	var columns = this.columns();
	for (var i = 0; i < cells.length; i++) {
		var cols = columns.filter(function (c) { return c.id() === cells[i].id(); });
		if (cols.length === 0) {
			continue;
		}

		this.createCellRegion(cells[i], row, cols[0]);
	}

	FC.Utils.EventLogger.Default.addMessage("RGPANEL",
		"TableRegionViewModel bindRow\t" + (performance.now() - t0) + "\tms");

	this.page.draw();
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.deleteRowRegions = function (rowModel, columnId) {
	var fields = rowModel.cell();
	for (var fIndex = 0; fIndex < fields.length; fIndex++) {
		var field = fields[fIndex];

		if (columnId && field.id() !== columnId) {
			continue;
		}

		var fieldRegions = field.manipulators.regionsManipulator.getFieldRegions(field);
		if (fieldRegions && fieldRegions.length > 0) {
			for (var j = 0; j < fieldRegions.length; j++) {
				var regionViewModel = this.page.findRegionByModel(fieldRegions[j].region);
				if (regionViewModel) {
					if (regionViewModel.parentRegion.isVisible()) {
						this.page.deleteRegion(regionViewModel);
					} else {
						this.page.deleteRegion(regionViewModel.parentRegion);
					}
				}
			}
		}
	}
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.unbindRow = function (row, deleteInModel) {
	var t0 = performance.now();

	var rowModel = row.rowModel();
	if (rowModel) {
		this.deleteRowRegions(rowModel);

		if (deleteInModel) {
			this.table().deleteRow(rowModel);
		}

		row.rowModel(null);
	} else {
		var cells = this.regions();

		var i = 0;
		while (i < cells.length) {
			var rects = cells[i].rectangles();
			if (rects.length > 0 && rects[0].y1() >= row.topSeparator() && rects[0].y2() <= row.bottomSeparator()) {
				this.page.removeRegionZones(cells[i]);
				cells.splice(i, 1);
				continue;
			}

			i++;
		}

		this.regions(cells);
	}

	FC.Utils.EventLogger.Default.addMessage("RGPANEL",
		"TableRegionViewModel unbindRow\t" + (performance.now() - t0) + "\tms");

	this.page.draw();
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.beforeDeleting = function () {
	var t0 = performance.now();

	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();
	if (this.field() && docEditor.extender.selectedTableRow()!==-1) {
		var fieldRef = this.field().getNodeRef();
		if (fieldRef === docEditor.extender.selectedTableRowPath()) {
			docEditor.extender.selectedTableRow(-1);
			docEditor.extender.selectedTableRowPath("");
		}
	}

	var cells = this.regions();
	var i = 0;
	while (i < cells.length) {
		this.page.removeRegionZones(cells[i]);
		cells.splice(i, 1);
	}
	this.regions(cells);

	var rows = this.rows().map(function (r) { return r.rowModel(); }).filter(function (r) { return !!r; });
	this.table().deleteRows(rows, false);
	this.rows([]);

	FC.Utils.EventLogger.Default.addMessage("RGPANEL",
		"TableRegionViewModel beforeDeleting\t" + (performance.now() - t0) + "\tms");
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.createCellRegion = function (cellModel, row, column) {
	var coord = {
		X1: column.leftPos(),
		X2: column.rightPos(),
		Y1: row.topSeparator(),
		Y2: row.bottomSeparator()
	};

	var regionData = { Coordinates: [coord], Status: 2, PageId: this.page.id() };

	var rectangle = new FC.BusinessModels.DocumentEditor.Coordinates(coord);
	var text = FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextInsideRectangle(rectangle, this.page);

	this.page.createRegionForField(regionData, this, cellModel, text);
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.recalculateCellsInColumn = function (column) {
	var currentRegions = this.regions().slice(0);
	var i = 0;
	while (i < currentRegions.length) {
		var currentCell = currentRegions[i].field();
		if (currentCell && currentCell.id() === column.id()) {
			this.page.deleteRegion(currentRegions[i]);
		}

		i++;
	}

	var currentRows = this.rows();
	var currentBlockId = JSON.stringify(this.id());

	for (i = 0; i < currentRows.length; i++) {
		var row = currentRows[i].rowModel();

		if (!row || JSON.stringify(row.tableBlockId()) !== currentBlockId) {
			continue;
		}

		var cells = row.cell();
		for (var j = 0; j < cells.length; j++) {
			if (cells[j].id() !== column.id()) {
				continue;
			}

			this.createCellRegion(cells[j], currentRows[i], column);
		}
	}


	this.page.draw();
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.deleteVerticalSeparator = function (separatorZone) {
	if (!separatorZone) {
		return;
	}

	var columns = this.columns();
	var rightBorder;
	if (separatorZone.columnOnRight) {
		rightBorder = separatorZone.columnOnRight.rightPos();

		if (separatorZone.columnOnRight.id()) {
			this.unbindColumn(separatorZone.columnOnRight);
		}

		var index = columns.indexOf(separatorZone.columnOnRight);
		if (index > -1) {
			columns.splice(index, 1);
		}


	} else {
		var extarnalRect = this.getExternalRect();
		rightBorder = extarnalRect.x2;
	}

	if (separatorZone.columnOnLeft) {
		this.resizeCellsX(separatorZone.columnOnLeft.rightPos(), rightBorder);
		separatorZone.columnOnLeft.rightPos(rightBorder);
	}

	this.recalculateColumnSeparators();
	this.recalculateColumnHeaders();

	this.page.draw();
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.deleteHorizontalSeparator = function (separatorZone) {
	var rows = this.rows();
	var bottomBorder = -1;
	var separatorValue = separatorZone.separator();
	var i;
	for (i = 0; i < rows.length; i++) {
		if (rows[i].topSeparator() === separatorValue) {
			bottomBorder = rows[i].bottomSeparator();
			this.unbindRow(rows[i], true);
			rows.splice(i, 1);
			this.rows(rows);
			i--;
		}
	}

	if (bottomBorder < 0) {
		var extarnalRect = this.getExternalRect();
		bottomBorder = extarnalRect.y2;
	}

	this.resizeCellsY(separatorValue, bottomBorder);

	var separators = this.horizontalSeparators();
	for (i = 0; i < separators.length; i++) {
		if (separators[i].value() === separatorValue) {
			separators.splice(i, 1);
			this.horizontalSeparators(separators);
			break;
		}
	}

	var regionId = JSON.stringify(this.model.id());
	var rowModels = this.table().rows()
		.filter(function (r) { return JSON.stringify(r.tableBlockId()) === regionId; });;

	for (var j = 0; j < rowModels.length; j++) {
		if (rowModels[j].rowNumber() > i) {
			rowModels[j].rowNumber(rowModels[j].rowNumber() - 1);
		}
	}

	this.recalculateRowSeparators();
	this.recalculateRowHeaders();

	this.page.draw();
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.recalculateRegionZonesByType = function (page, region, checkType, createZones) {
	var zones = page.actionZones();
	var i = 0;
	while (i < zones.length) {
		if (region._id === zones[i].target._id && checkType(zones[i])) {
			zones.splice(i, 1);
		} else {
			i++;
		}
	}

	var newZones = createZones(region);
	for (var j = 0; j < newZones.length; j++) {
		zones.push(newZones[j]);
	}

	page.actionZones(zones);
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.createColumnSeparatorZones = function (region) {
	var zones = [];

	var columns = region.columns();
	var previousRight = region.rectangles()[0].x1();
	var x2 = region.rectangles()[0].x2();
	for (var j = 0; j < columns.length; j++) {
		var rightCoord = columns[j].rightPos();
		var leftCoord = columns[j].leftPos();

		if (leftCoord !== previousRight) {
			zones.push(new FC.Viewmodels.ActionZones.VerticalSeparatorZone(region.rectangles, region, region.page, region.depth, null, columns[j]));
		}

		if (rightCoord !== x2) {
			var columnOnRight = null;
			if (j < columns.length - 1 && columns[j + 1].leftPos() === rightCoord) {
				columnOnRight = columns[j + 1];
			}

			zones.push(new FC.Viewmodels.ActionZones.VerticalSeparatorZone(region.rectangles,
				region,
				region.page,
				region.depth,
				columns[j],
				columnOnRight));
			previousRight = rightCoord;
		}
	}

	return zones;
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.createColumnHeaderZones = function (region) {
	var zones = [];

	var columns = region.columns();
	for (var j = 0; j < columns.length; j++) {
		zones.push(new FC.Viewmodels.ActionZones.ColumnHeaderZone(region.rectangles, region, region.page, region.depth, columns[j]));
	}

	return zones;
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.createRowSeparators = function (region) {
	var zones = [];

	var separators = region.horizontalSeparators();
	for (var i = 0; i < separators.length; i++) {
		zones.push(new FC.Viewmodels.ActionZones.HorizontalSeparatorZone(region.rectangles, region, region.page, region.depth, separators[i].value));
	}

	return zones;
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.createRowHeaderZones = function (region) {
	var zones = [];

	var rows = region.rows();
	for (var j = 0; j < rows.length; j++) {
		zones.push(new FC.Viewmodels.ActionZones.RowHeaderZone(region.rectangles, region, region.page, region.depth, rows[j]));
	}

	return zones;
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.updateText = function () {
	var t0 = performance.now();

	var textRegions = FC.Viewmodels.ActionZones.RecognizedTextZone.getRegionsInsideRectangle(this.rectangles()[0], this.page)
		.map(function (tr) {
			return {
				text: tr.caption(),
				x1: tr.rectangles()[0].x1(),
				x2: tr.rectangles()[0].x2(),
				y1: tr.rectangles()[0].y1(),
				y2: tr.rectangles()[0].y2()
			};
		});

	var childRegions = this.regions();

	for (var i = 0; i < childRegions.length; i++) {
		childRegions[i].resized = false;

		var field = childRegions[i].field();

		if (!field || !(field instanceof FC.BusinessModels.DocumentEditor.TextField)) {
			continue;
		}

		var rects = childRegions[i].rectangles();
		if (!rects || rects.length === 0) {
			continue;
		}

		var rect = ko.toJS(rects[0]);
		var currentTextRegions = [];
		for (var j = 0; j < textRegions.length; j++) {

			if (textRegions[j].x1 < rect.x1 ||
				textRegions[j].x2 > rect.x2 ||
				textRegions[j].y1 < rect.y1 ||
				textRegions[j].y2 > rect.y2) {
				continue;
			}

			currentTextRegions.push(textRegions[j]);
			textRegions.splice(j, 1);
			j--;
		}

		var text = FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextFromTextRegions(currentTextRegions);
		field.setParagraphText(text, -1);
	}

	FC.Utils.EventLogger.Default.addMessage("RGPANEL", "TableRegionViewModel updateText\t" + (performance.now() - t0) + "\tms");
};

FC.Viewmodels.DocumentEditor.TableRegionViewModel.prototype.getExternalRect = function () {
	return this.getExternalRectBase();
};