﻿FC.Viewmodels.DocumentEditor.TableSeparatorCreator = function (page, tableRegion, onColumnDraw, onColumnCreate, onSeparatorCreated, deltaToCreate) {
	var startPoint = null;

	var self = this;

	self.onColumnDraw = onColumnDraw;
	self.onColumnCreate = onColumnCreate;
	self.onSeparatorCreated = onSeparatorCreated;

	var defaultDelta = deltaToCreate || FC.Viewmodels.DocumentEditor.TableSeparatorCreator.delta;

	var separatorDrawing = function (point) {
		page.directDrawer.clear();
		var externalRect = tableRegion.getExternalRect();

		var deltaX = point.x - startPoint.x;
		var deltaY = point.y - startPoint.y;
		var color = "#48ba00";
		var colorColumn = "rgba(72, 186, 0, 0.1)";
		var width = 0.5;
		var delta = defaultDelta / page.zoom();

		if (Math.abs(deltaX) > delta) {
			if (Math.abs(deltaY) > delta) {
				if (self.onColumnDraw) {
					self.onColumnDraw(point);
					return true;
				}

				point.x = startPoint.x + FC.Viewmodels.ActionZones.VerticalSeparatorZone.findDeltaWithRestrictions(tableRegion, deltaX, startPoint.x);
				point.y = startPoint.y + FC.Viewmodels.ActionZones.HorizontalSeparatorZone.findDeltaWithRestrictions(tableRegion, deltaY, startPoint.y);


				var pointTopLeft = new FC.Viewmodels.ImageEditor.Point(startPoint.x > point.x ? point.x : startPoint.x, externalRect.y1);
				var pointTopRight = new FC.Viewmodels.ImageEditor.Point(startPoint.x > point.x ? startPoint.x : point.x, externalRect.y1);
				var pointBottomLeft = new FC.Viewmodels.ImageEditor.Point(startPoint.x > point.x ? point.x : startPoint.x, externalRect.y2);
				var pointBottomRight = new FC.Viewmodels.ImageEditor.Point(startPoint.x > point.x ? startPoint.x : point.x, externalRect.y2);

				page.directDrawer.addDrawingCommand(function (layer) {
					layer.drawFilledRectangle(
						FC.Viewmodels.ImageEditor.Rectangle.getRectangleFromPoints(pointTopLeft, pointBottomRight),
						colorColumn);
					layer.drawLine(pointTopLeft, pointBottomLeft, width, color);
					layer.drawLine(pointTopRight, pointBottomRight, width, color);
				});

				page.draw(1);

				return false;
			}

			page.directDrawer.addDrawingCommand(function (layer) {
				layer.drawLine(
					new FC.Viewmodels.ImageEditor.Point(externalRect.x1, startPoint.y),
					new FC.Viewmodels.ImageEditor.Point(externalRect.x2, startPoint.y),
					width,
					color);
			});
			page.draw(1);

			return false;
		}

		if (Math.abs(deltaY) > delta || Math.abs(deltaY) > Math.abs(deltaX)) {
			page.directDrawer.addDrawingCommand(function (layer) {
				layer.drawLine(
					new FC.Viewmodels.ImageEditor.Point(startPoint.x, externalRect.y1),
					new FC.Viewmodels.ImageEditor.Point(startPoint.x, externalRect.y2),
					width,
					color);
			});

			page.draw(1);
			return false;
		}

		if (self.onColumnDraw) {
			self.onColumnDraw(point);
		}
		return true;
	};

	this.addVerticalSeparators = function (x1, x2, event) {
		x2 = typeof x2 === "undefined" ? x1 : x2;

		if (x1 === x2 && !this.isVerticalSeparatorCreatable(x1)) {
			return;
		}

		var columns = tableRegion.columns();
		var i;

		var columnToSplit = null;
		var externalRect = tableRegion.getExternalRect();
		var left = externalRect.x1;
		var right = externalRect.x2;

		for (i = 0; i < columns.length; i++) {
			if (columns[i].leftPos() > x2) {
				right = columns[i].leftPos();
				break;
			}
			if (columns[i].rightPos() > x2) {
				columnToSplit = columns[i];
				break;
			}

			left = columns[i].rightPos();
		}

		if (columnToSplit) {
			right = columnToSplit.rightPos();
		}

		var delta = defaultDelta / page.zoom();
		if (x1 !== x2) {
			if (x1 - left < delta) {
				x1 = x2;
			} else if (right - x2 < delta) {
				x2 = x1;
			}
		}

		if (columnToSplit) {
			columnToSplit.rightPos(x1);
			tableRegion.resizeCellsOnLeft(right, x1);
			i++;
		}

		var newColumnLeft = columnToSplit ? null : new FC.BusinessModels.DocumentEditor.TableColumn({ LeftPos: left, RightPos: x1, Id: 0 });
		var newColumnCenter = x1 === x2 ? null : new FC.BusinessModels.DocumentEditor.TableColumn({ LeftPos: x1, RightPos: x2, Id: 0 });
		var newColumnRight = new FC.BusinessModels.DocumentEditor.TableColumn({ LeftPos: x2, RightPos: right, Id: 0 });

		if (i < columns.length) {
			if (newColumnLeft != null) {
				columns.splice(i, 0, newColumnLeft);
				i++;
			}

			if (newColumnCenter != null) {
				columns.splice(i, 0, newColumnCenter);
				i++;
			}

			columns.splice(i, 0, newColumnRight);
		} else {
			if (newColumnLeft != null) {
				columns.push(newColumnLeft);
			}

			if (newColumnCenter != null) {
				columns.push(newColumnCenter);
			}

			columns.push(newColumnRight);
		}

		tableRegion.columns(columns);
		tableRegion.recalculateColumnSeparators();
		tableRegion.recalculateColumnHeaders();

		if (newColumnCenter) {
			page.setCurrentCursorAndCaption(null);
			var binder = new FC.Viewmodels.DocumentEditor.ColumnBinder(tableRegion, newColumnCenter, event, page);
			binder.show();
		}
	};

	this.addColumn = function (x1, x2, event) {
		var minX = x1 > x2 ? x2 : x1;
		var maxX = x1 > x2 ? x1 : x2;
		this.addVerticalSeparators(minX, maxX, event);
	};

	this.isHorizontalSeparatorCreatable = function (y) {
		var separators = tableRegion.horizontalSeparators();

		// проверим, что таких сепараторов нет
		for (var i = 0; i < separators.length; i++) {
			if (Math.abs(separators[i].value() - y) < FC.Viewmodels.ActionZones.zoneConstants.horizontalSeparatorThreshold) {
				return false;
			}
		}

		return true;
	};

	this.isVerticalSeparatorCreatable = function (x) {
		var columns = tableRegion.columns();
		for (var i = 0; i < columns.length; i++) {
			if (Math.abs(columns[i].leftPos() - x) < FC.Viewmodels.ActionZones.zoneConstants.verticalSeparatorThreshold ||
				Math.abs(columns[i].rightPos() - x) < FC.Viewmodels.ActionZones.zoneConstants.verticalSeparatorThreshold) {
				// такой уже есть
				return false;
			}
		}

		return true;
	};

	this.addHorizontalSeparator = function (y) {
		if (!this.isHorizontalSeparatorCreatable(y)) {
			return;
		}

		var separators = tableRegion.horizontalSeparators();
		var i;
		var separatorModel = ko.observable(y);
		var newSeparator = new FC.Viewmodels.DocumentEditor.HorizontalSeparator(separatorModel);
		var separatorModels = tableRegion.model.horizontalSeparators();
		var externalRect = tableRegion.getExternalRect();
		var rows = tableRegion.rows();

		var newRowIndex = -1;

		if (separators.length !== 0) {
			for (i = 0; i < separators.length; i++) {
				var oldValue = separators[i].value();

				if (oldValue > y) {
					separators.splice(i, 0, newSeparator);
					separatorModels.splice(i, 0, separatorModel);

					if (tableRegion.cellsSizesRelateToParent) {
						tableRegion.resizeCellsOnTop(oldValue, y);
					}

					if (rows[i].rowModel()) {
						newRowIndex = i + 1;
					}
					break;
				}

				if (i === separators.length - 1) {
					if (rows[rows.length - 1].bottomSeparator() > y && rows[rows.length - 1].rowModel()) {
						newRowIndex = rows.length;
					};

					separators.push(newSeparator);
					separatorModels.push(separatorModel);

					if (tableRegion.cellsSizesRelateToParent) {
						tableRegion.resizeCellsOnTop(externalRect.y2, y);
					}
					i++;
					break;
				}
			}
		} else {
			i = 1;
			separators.push(newSeparator);
			separatorModels.push(separatorModel);

			if (tableRegion.cellsSizesRelateToParent) {
				tableRegion.resizeCellsOnTop(externalRect.y2, y);
			}

			if (rows.length > 0 && !!rows[0].rowModel()) {
				newRowIndex = rows.length;
			}
		}

		var regionId = JSON.stringify(tableRegion.model.id());
		var rowModels = tableRegion.table().rows()
			.filter(function (r) { return JSON.stringify(r.tableBlockId()) === regionId; });;

		for (var j = 0; j < rowModels.length; j++) {
			if (rowModels[j].rowNumber() > i) {
				rowModels[j].rowNumber(rowModels[j].rowNumber() + 1);
			}
		}

		tableRegion.horizontalSeparators(separators);
		tableRegion.recalculateRowSeparators();
		tableRegion.recalculateRowHeaders();
		tableRegion.model.horizontalSeparators(separatorModels);

		if (newRowIndex > -1) {
			rows = tableRegion.rows();
			if (newRowIndex < rows.length) {
				tableRegion.bindRow(rows[newRowIndex]);
			}
		}
	};

	var removalKey = "separatorDrawing";
	var docEditor = window.batchLayout.batchLayoutViewModel.DocumentEditor();

	var endSeparatorDrawing = function (point, event) {
		var delta = defaultDelta / page.zoom();

		var deltaX = point.x - startPoint.x;
		var deltaY = point.y - startPoint.y;
		var creatingColumn = false;

		if (Math.abs(deltaX) > delta) {
			if (Math.abs(deltaY) > delta) {
				creatingColumn = true;
				if (self.onColumnCreate) {
					self.onColumnCreate(point, event);
				} else {
					point.x = startPoint.x +
						FC.Viewmodels.ActionZones.VerticalSeparatorZone.findDeltaWithRestrictions(tableRegion, deltaX, startPoint.x);

					self.addColumn(Math.round(startPoint.x), Math.round(point.x), event);
				}
			} else {
				self.addHorizontalSeparator(Math.round(startPoint.y));
				if (self.onSeparatorCreated) {
					self.onSeparatorCreated();
				}
			}
		} else {
			if (Math.abs(deltaY) > delta || !self.onColumnCreate && Math.abs(deltaY) > Math.abs(deltaX)) {
				self.addVerticalSeparators(Math.round(startPoint.x), undefined, event);
				if (self.onSeparatorCreated) {
					self.onSeparatorCreated();
				}
			} else if (self.onColumnCreate) {
				self.onColumnCreate(point, event);
				creatingColumn = true;
			}
		}

		docEditor.removeOnMouseUp(removalKey);
		docEditor.removeOnMouseLeave(removalKey);
		docEditor.removeOnMouseMove(removalKey);

		FC.Viewmodels.EventStack.addEvent();
		startPoint = null;

		if (!creatingColumn) {
			page.directDrawer.clear();
		}

		page.draw();
	};

	this.startSeparatorDrawing = function (point) {
		startPoint = point;

		docEditor.addOnMouseUp(endSeparatorDrawing.bind(this), 100, removalKey);
		docEditor.addOnMouseLeave(endSeparatorDrawing.bind(this), 100, removalKey);
		docEditor.addOnMouseMove(separatorDrawing.bind(this), 100, removalKey);
	};
};

FC.Viewmodels.DocumentEditor.TableSeparatorCreator.delta = 20;