﻿var FC = FC || {};
FC.Viewmodels = FC.Viewmodels || {};
FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

FC.Viewmodels.DocumentEditor.TextRegionViewModel = function (textRegion, page) {
	FC.Viewmodels.DocumentEditor.RegionBase.call(this);

	this.page = page;
	this.hovering = true;
	this.rectangles = ko.observableArray([]);
	this.caption = ko.observable(textRegion.Value);
	this._id = FC.Viewmodels.DocumentEditor.RegionViewModel.GlobalCounter++;
	this.modifiable = false;

	this.rectangles([new FC.BusinessModels.DocumentEditor.Coordinates(textRegion.Coordinates)]);
};

FC.Viewmodels.DocumentEditor.TextRegionViewModel.prototype = Object.create(FC.Viewmodels.DocumentEditor.RegionBase.prototype);

FC.Viewmodels.DocumentEditor.TextRegionViewModel.prototype.createZones = function () {
	if (!this.isVisible()) {
		return [];
	}

	return [new FC.Viewmodels.ActionZones.RecognizedTextZone(this.rectangles, this, this.page)];
};

FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextFromRegions = function (regions) {
	if (!regions || regions.length === 0) {
		return "";
	}

	var textRegions = [];
	var i;
	for (i = 0; i < regions.length; i++) {
		if (!(regions[i] instanceof FC.Viewmodels.DocumentEditor.TextRegionViewModel)) {
			continue;
		}

		textRegions.push({
			text: regions[i].caption(),
			x1: regions[i].rectangles()[0].x1(),
			y1: regions[i].rectangles()[0].y1(),
			x2: regions[i].rectangles()[0].x2(),
			y2: regions[i].rectangles()[0].y2()
		});
	}

	return FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextFromTextRegions(textRegions);
};

FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextFromTextRegions = function (textRegions) {
	textRegions.sort(function (a, b) {
		if (a.y2 < b.y1) {
			return -1;
		}

		if (a.y1 > b.y2) {
			return 1;
		}

		return a.x2 - b.x1;
	});

	var text = "";
	var separator = "\n";
	var wordsSeparator = " ";
	var oldY2 = -1;
	for (var i = 0; i < textRegions.length; i++) {
		if (i > 0) {
			if (textRegions[i].y1 > oldY2) {
				text += separator;
			} else {
				text += wordsSeparator;
			}
		}

		text += textRegions[i].text;
		oldY2 = textRegions[i].y2;
	}

	return text;
};

FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextInsideRectangle = function (rectangle, page) {
	var regions = FC.Viewmodels.ActionZones.RecognizedTextZone.getRegionsInsideRectangle(rectangle, page);
	return FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextFromRegions(regions);
};

FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextCrossingRectangle = function (rectangle, page) {
	var regions = FC.Viewmodels.ActionZones.RecognizedTextZone.getRegionsCrossingRectangle(rectangle, page);
	return FC.Viewmodels.DocumentEditor.TextRegionViewModel.getTextFromRegions(regions);
};