﻿FC.Viewmodels = FC.Viewmodels || {};
FC.Viewmodels.FormEditor = FC.Viewmodels.FormEditor || {};


FC.Viewmodels.FormEditor.DbLookupViewmodel = function (panelHeight, editorWidth, textCaptureSwitcher, textCaptureStorage, onClose) {
	var self = this;
	var headerHeight = 52;
	var ignoreQueryChange = false;

	var subscription = null;
	self.textCaptureSwitcher = textCaptureSwitcher;
	self.textCaptureStorage = textCaptureStorage;

	self.document = null;
	self.mode = ko.observable(FC.Viewmodels.FormEditor.DbLookupViewmodelModes.VIEW);

	self.fieldsCaption = null;

	self.variants = ko.observableArray();
	self.selectedVariant = ko.observable();
	self.editableEntity = ko.observable();
	self.datasetDescription = ko.observable();

	self.variantsHeight = ko.observable(0);
	self.selectedVariantHeight = ko.observable(0);
	self.editWindwHeight = ko.observable(0);

	self.currentRuleContext = null;
	self.fullText = null;

	self.query = ko.observable('');
	self.queryHasFocus = ko.observable(false);
	
	self.dblookupVariantsOffsetY = ko.observable(0);
	self.dblookupSelectedVariantOffsetY = ko.observable(0);
	self.dblookupEditWindowOffsetY = ko.observable(0);
	
	self.visibleHeight = ko.computed(function() {
		var res = panelHeight() - headerHeight;
		return res < 0 ? 0 : res;
	});

	self.addEnabled = ko.computed(function() {
		var datasetDescr = self.datasetDescription();
		if (!datasetDescr) return false;
		return datasetDescr.canOperatorsAddRecords;
	});

	self.editEnabled = ko.computed(function () {
		var datasetDescr = self.datasetDescription();
		if (!datasetDescr) return false;
		return datasetDescr.canOperatorsEditRecords;
	});

	var oldValue = '';

	self.selectItem = function (data) {
		self.selectedVariant(data);
		updatePreviewHeight();
	}

	var processResult = function (result, openFirst) {
		result = result || [];
		self.variants(result.map(function(resItem) {
			return new FC.Viewmodels.FormEditor.DbLookupVariant(resItem, self.fieldsCaption);
		}));
		if (openFirst) {
			self.selectItem(self.variants()[0]);
		} else {
			self.selectItem(self.selectedVariant() ? self.variants().filter(function (item) {
				
				var sourceItem = self.selectedVariant().elementsDictionary;
				var keys = Object.keys(sourceItem);
				for (var i = 0; i < keys.length; i++) {
					var key = keys[i];
					if (sourceItem[key] !== item.elementsDictionary[key]) {
						return false;
					}
				}
				return true;
			})[0]:null);
		}

	}
	var ajax = null;
	var search = function (query, cursorPosition, openFirst) {
		if (ajax) {
			ajax.abort();
			ajax = null;
		}
		ajax = FC.DataContext.DbCache.lookup(self.document.templateId(),
			self.document.templateVersion(),
			self.currentRuleContext.rule.dataSourceName,
			(isFinite(cursorPosition) ? cursorPosition : query.length),
			[{ FieldName: '', FieldValue: query }],
			false,
			function (result) {
				ajax = null;
				processResult(result, openFirst);

			},
			function (errorThrown) {
				if (errorThrown === 'abort') return;
				ajax = null;
				self.variants([]);
			}
		);
	}
	var searchByFullText = function(openFirst) {
		search(self.fullText,-1, openFirst);
	}

	var searchByQuery = function (cursorPosition, openFirst) {
		search(self.query(), cursorPosition, openFirst);
	}
	
	self.open = function (ruleContext, document, fullText, searchValue) {
		if (!ruleContext) return;
		ignoreQueryChange = true;
		self.mode(FC.Viewmodels.FormEditor.DbLookupViewmodelModes.VIEW);
		self.fullText = fullText;
		self.document = document;
		self.currentRuleContext = ruleContext;
		self.selectedVariant(null);
		self.query(searchValue || '');
		var datasetName = self.currentRuleContext.rule.dataSourceName;
		self.fieldsCaption = self.document.manipulators.dbCheckRulesManipulator.getRuleFieldsInfo(ruleContext);
		self.variants([]);
		if (searchValue) {
			searchByQuery(-1, true);
		} else {
			searchByFullText(true);
		}

		var datasetDescr = document.datasets().filter(function(dataset) { return dataset.name === datasetName; })[0];
		self.datasetDescription(datasetDescr);
		self.dblookupVariantsOffsetY(0);
		self.dblookupSelectedVariantOffsetY(0);
		self.queryHasFocus(true);
		ignoreQueryChange = false;
	}

	var updatePreviewHeight = function() {
		setTimeout(function () {
			self.selectedVariantHeight($('.dblookup__selectedvariant__contentframe').outerHeight());
		}, 30);
	}

	var updateVariantsHeight = function () {
		setTimeout(function () {
			self.variantsHeight($('.dblookup__variants__contentframe').outerHeight());
		}, 30);
	}

	var updateEditWindowHeight = function () {
		setTimeout(function () {
			self.editWindwHeight($('.dblookup__editContainer__contentframe').outerHeight());
		}, 30);
	}
	
	self.closeLookUp = function () {
		if (subscription) {
			subscription.dispose();
			subscription = null;
		}
		onClose();
	}
	
	self.confirm = function() {
		if (self.selectedVariant()) {
			var referedFields =
				self.document.manipulators.dbCheckRulesManipulator.applyRule(self.currentRuleContext,
					self.selectedVariant().elementsDictionary, true);
			self.document.manipulators.dbCheckRulesManipulator.removeRuleErrors(self.currentRuleContext);
			self.document.manipulators.dbCheckRulesManipulator.tryRecount(referedFields, [], self.currentRuleContext);
			if (subscription) {
				subscription.dispose();
				subscription = null;
			}
			onClose();
		}
	}

	self.clearData  = function() {
		var referedFields = self.document.manipulators.dbCheckRulesManipulator.applyRule(self.currentRuleContext, null, true);
		self.document.manipulators.dbCheckRulesManipulator.removeRuleErrors(self.currentRuleContext);
		self.document.manipulators.dbCheckRulesManipulator.addRuleError(self.currentRuleContext);
		self.document.manipulators.dbCheckRulesManipulator.tryRecount(referedFields, [], self.currentRuleContext);
		if (subscription) {
			subscription.dispose();
			subscription = null;
		}
		onClose();
	}

	self.selectAndSave = function (data) {
		self.selectItem(data);
		self.confirm();
	}
	
	self.editRecord = function () {
		self.dblookupEditWindowOffsetY(0);
		var primaryKeyColumn = self.datasetDescription().columns.filter(function (val) { return val.isPrimaryKey; })[0];
		var selectedVariant = self.selectedVariant();
		if (primaryKeyColumn && selectedVariant) {
			
			FC.DataContext.DbCache.lookup(self.document.templateId(),
				self.document.templateVersion(),
				self.currentRuleContext.rule.dataSourceName,
				-1,
				[{ FieldName: primaryKeyColumn.name, FieldValue: selectedVariant.allFields[primaryKeyColumn.name] }],
				true,
				function(result) {
					if (result.length > 0) {
						self.editableEntity(
							new FC.Viewmodels.FormEditor.DbLookupEditableEntity(self.datasetDescription().columns, self.fieldsCaption, result[0], updateEditWindowHeight, FC.Viewmodels.FormEditor.DbLookupViewmodelModes.EDIT));

						subscription = self.editableEntity().valueWithFocus.subscribe(function (val) {
							if (val) {
								self.textCaptureSwitcher(true);
							} else {
								self.textCaptureSwitcher(false);
							}
						});
						self.mode(FC.Viewmodels.FormEditor.DbLookupViewmodelModes.EDIT);
						self.dblookupEditWindowOffsetY(0);
						updateEditWindowHeight();

					}
				}
			);
		}
	}

	self.addRecord = function () {
		self.dblookupEditWindowOffsetY(0);
		var defaultValue = [];
		var keys = Object.keys(self.fieldsCaption);
		for (var i = 0; i < keys.length; i++) {
			var key = keys[i];
			var fieldCaption = self.fieldsCaption[key];
			defaultValue.push({ FieldName: key, FieldValue: fieldCaption.field?fieldCaption.field.value():'' });
		}
		self.editableEntity(
			new FC.Viewmodels.FormEditor.DbLookupEditableEntity(self.datasetDescription().columns, self.fieldsCaption, defaultValue, updateEditWindowHeight, FC.Viewmodels.FormEditor.DbLookupViewmodelModes.ADD));
		subscription = self.editableEntity().valueWithFocus.subscribe(function(val) {
			if (val) {
				self.textCaptureSwitcher(true);
			} else {
				self.textCaptureSwitcher(false);
			}
		});
		self.mode(FC.Viewmodels.FormEditor.DbLookupViewmodelModes.ADD);
		self.dblookupEditWindowOffsetY(0);
		updateEditWindowHeight();
	}

	self.returnToView = function () {
		if (subscription) {
			subscription.dispose();
			subscription = null;
		}
		self.mode(FC.Viewmodels.FormEditor.DbLookupViewmodelModes.VIEW);
	}

	self.save = function () {
		var editableValue = self.editableEntity();
		if (editableValue.cannotBeSaved()) return;
		var descriptions = editableValue.entityDescription;

		var columns = [], values = [];
		for (var i = 0; i < descriptions.length; i++) {
			var descr = descriptions[i];
			var notEmptyValues = descr.values().map(function (val) { return val.value(); }).filter(function (val) { return val; });
			if (notEmptyValues.length > 0) {
				values = values.concat(notEmptyValues);
				for (var j = 0; j < notEmptyValues.length; j++) {
					columns.push(descr.columnName);
				}
			}
		}

		var saveMethod;
		if (self.mode() === FC.Viewmodels.FormEditor.DbLookupViewmodelModes.ADD) {
			saveMethod = FC.DataContext.DbCache.addEntity;
		} else {
			saveMethod = FC.DataContext.DbCache.editEntity;
		}
		saveMethod(self.document.templateId(),
			self.document.templateVersion(),
			self.currentRuleContext.rule.dataSourceName,
			columns,
			values,
			function (newId) {
				var writtenColumns = [];
				var initInfo = [];
				for (var i = 0; i < columns.length; i++) {
					var columnName = columns[i];
					if (writtenColumns.indexOf(columnName) >= 0) continue;
					writtenColumns.push(columnName);
					initInfo.push({
						FieldName: columnName,
						FieldValue: values[i]
					});
				}
				if (self.mode() === FC.Viewmodels.FormEditor.DbLookupViewmodelModes.ADD) {
					var primaryKeyColumn = self.datasetDescription().columns.filter(function (val) { return val.isPrimaryKey; })[0];
					initInfo.push({
						FieldName: primaryKeyColumn.name,
						FieldValue: newId
					});
				}
				self.selectedVariant(new FC.Viewmodels.FormEditor.DbLookupVariant(initInfo, self.fieldsCaption));
				self.confirm();
			},
			function (error) {
				FC.Utils.Error.Show(error);
			});

	}

	self.clearQuery = function () {
		self.query('');
	}

	self.focusQuery = function () {
		self.queryHasFocus(true);
	}

	self.onBasicScroll = function (data, event) {
		event.preventDefault();
		var $target = $(event.target);
		var scrollPosition = $target.scrollTop();
		$target.scrollTop(0);
		var newOffset = self.dblookupEditWindowOffsetY() + scrollPosition;
		var maxOffset = self.editWindwHeight() - self.visibleHeight();
		if (newOffset > maxOffset) {
			newOffset = maxOffset;
		}
		if (newOffset < 0) {
			newOffset = 0;
		}
		self.dblookupEditWindowOffsetY(newOffset);
	}

	self.textCaptureStorage.subscribe(function(val) {
		if (val && self.editableEntity() && self.editableEntity().valueWithFocus()) {
			var $elem = $('#'+self.editableEntity().valueWithFocus().uniqueId);
			var caretPosition = FC.Utils.CaretTools.doGetCaretPosition($elem[0]);
			var oldValue = self.editableEntity().valueWithFocus().value();
			var newValue = val;
			if (oldValue) {
				newValue = [oldValue.slice(0, caretPosition), newValue, oldValue.slice(caretPosition)].join('');
			}
			self.editableEntity().valueWithFocus().value(newValue);
			var newCaretPosition = caretPosition + val.length;
			FC.Utils.CaretTools.setCaretPosition($elem[0], newCaretPosition);
			self.editableEntity().valueWithFocus().value(newValue);
			self.textCaptureStorage('');
		}
	});

	self.query.subscribe(function (val) {
		if (ignoreQueryChange) return;
		if (!val) val = '';
		if (val.length < 3) val = '';


		if (val === oldValue) return;

		if (val.length < 3) {
			searchByFullText();
		} else {
			searchByQuery(FC.Utils.CaretTools.doGetCaretPosition($('.dblookup__queryinput')[0]));
		}
		oldValue = val;

	});

	self.dblookupVariantsScrollbar = new FC.ScrollBarNew(FC.ScrollBarNew.Orientation.Vertical,
		self.visibleHeight,
		self.dblookupVariantsOffsetY,
		self.visibleHeight,
		self.variantsHeight);

	self.dblookupSelectedVariantScrollbar = new FC.ScrollBarNew(FC.ScrollBarNew.Orientation.Vertical,
		self.visibleHeight,
		self.dblookupSelectedVariantOffsetY,
		self.visibleHeight,
		self.selectedVariantHeight);

	self.dblookupEditWindowScrollbar = new FC.ScrollBarNew(FC.ScrollBarNew.Orientation.Vertical,
		self.visibleHeight,
		self.dblookupEditWindowOffsetY,
		self.visibleHeight,
		self.editWindwHeight);

	self.variantsScrollCss = self.dblookupVariantsScrollbar.contentFrameStyle;

	self.selectedVariantScrollCss = self.dblookupSelectedVariantScrollbar.contentFrameStyle;

	self.editWindowScrollCss = self.dblookupEditWindowScrollbar.contentFrameStyle;
	
	self.variants.subscribe(updateVariantsHeight);

	editorWidth.subscribe(function() {
		updateVariantsHeight();
		updatePreviewHeight();
		updateEditWindowHeight();
	});
	
	

	self.returnToViewTitle = FC.Environment.Localization.DbLookup.ReturnButtonTooltip;
	self.confirmTitle = FC.Environment.Localization.DbLookup.SelectButtonTooltip;
	self.clearDataTitle = FC.Environment.Localization.DbLookup.ResetDataButtonTooltip;
	self.addRecordTitle = FC.Environment.Localization.DbLookup.AddRecordButtonTooltip;
	self.editRecordTitle = FC.Environment.Localization.DbLookup.EditRecordButtonTooltip;
	self.queryPlaceholder = FC.Environment.Localization.BatchView.SearchPlaceholder;
}

FC.Viewmodels.FormEditor.DbLookupViewmodelModes= {
	VIEW: 0,
	ADD: 1,
	EDIT: 2
}