﻿FC.Viewmodels.FormEditor = FC.Viewmodels.FormEditor || {};
FC.Viewmodels.FormEditor.ErrorViewmodel = function (errorInfo) {
	var self = this;

	self.id = 'erroritem-' + (++FC.Viewmodels.FormEditor.GlobalId);
	self.fields = ko.observableArray($.map(errorInfo.fields, function (item) {
		return new FC.Viewmodels.FormEditor.ErrorNameItemViewmodel(item);
	}));

	self.message = ko.observable(errorInfo.message);

	self.type = ko.observable(errorInfo.type);
	self.severity = ko.observable(errorInfo.severity);

	self.selected = ko.observable(false);
	self.connectedFields = errorInfo.connectedFields;

	self.firstConnectedFieldPath = null;

	// Кэшированная подсчитанная высота текущего элемента-ошибки
	self.cachedHeight = null;

	if (self.connectedFields && self.connectedFields.length > 0) {
		var index = 0;
		var connectedField = self.connectedFields[index];
		while (connectedField && connectedField.isVisible && !connectedField.isVisible()) {
			connectedField = self.connectedFields[++index];
		}

		if (connectedField) {
			self.firstConnectedFieldPath = connectedField.getFieldPath();		
		}
	}

	// фактический тип ошибки, высчитанный на основе severity, типа ошибки и количества полей, к которым относится .
	// используется для отображения
	self.actualType = ko.pureComputed(function () {
		var type = self.type();
		var severity = self.severity();
		var connectedFields = self.connectedFields;

		if (type === -2) {

			// ошибка сборки
			return FC.Viewmodels.FormEditor.ErrorType.assemblyError;
		} else if (type === -1) {

			// поле, требующее верификации
			return FC.Viewmodels.FormEditor.ErrorType.nonVerified;
		} else {

			// ошибка неналожения региона
			if (type === 3) {
				return FC.Viewmodels.FormEditor.ErrorType.regionError;
			} else {

				var isMultiField = connectedFields && connectedFields.length > 1;
				if (severity) {
					if (isMultiField) {

						// многополевая ошибка
						return FC.Viewmodels.FormEditor.ErrorType.multiFieldError;
					} else {

						// ошибка
						return FC.Viewmodels.FormEditor.ErrorType.error;
					}
				} else {
					if (isMultiField) {

						// многополевое предупреждение
						return FC.Viewmodels.FormEditor.ErrorType.multiFieldWarning;
					} else {

						// предупреждение
						return FC.Viewmodels.FormEditor.ErrorType.warning;
					}
				}
			}
		}

	});

	self.popupText = ko.pureComputed(function () {
		var actualType = self.actualType();
		switch (actualType) {
			case FC.Viewmodels.FormEditor.ErrorType.nonVerified:
				return FC.Environment.Localization.GlobalResources.ErrorViewMode_ToVerify;
			case FC.Viewmodels.FormEditor.ErrorType.error:
				return FC.Environment.Localization.GlobalResources.ErrorViewMode_Errors;
			case FC.Viewmodels.FormEditor.ErrorType.warning:
				return FC.Environment.Localization.GlobalResources.ErrorViewMode_Warnings;
			case FC.Viewmodels.FormEditor.ErrorType.multiFieldError:
				return FC.Environment.Localization.GlobalResources.ErrorViewMode_MultiFieldErrors;
			case FC.Viewmodels.FormEditor.ErrorType.multiFieldWarning:
				return FC.Environment.Localization.GlobalResources.ErrorViewMode_MultiFieldWarnings;
			case FC.Viewmodels.FormEditor.ErrorType.regionError:
				return FC.Environment.Localization.GlobalResources.ErrorViewMode_RegionErrors;
			case FC.Viewmodels.FormEditor.ErrorType.assemblyError:
				return FC.Environment.Localization.GlobalResources.ErrorViewMode_AssemblyErrors;
			default:
				return "";
		}
	});
	self.indicatorStyle = ko.pureComputed(function () {
		var actualType = self.actualType();
		switch (actualType) {
			case FC.Viewmodels.FormEditor.ErrorType.nonVerified:
				return "webcommon__icon webcommon__non-verified-icon";
			case FC.Viewmodels.FormEditor.ErrorType.error:
				return "webcommon__icon webcommon__error-icon";
			case FC.Viewmodels.FormEditor.ErrorType.warning:
				return "webcommon__icon webcommon__warning-icon";
			case FC.Viewmodels.FormEditor.ErrorType.multiFieldError:
				return "webcommon__icon webcommon__multi-field-error-icon";
			case FC.Viewmodels.FormEditor.ErrorType.multiFieldWarning:
				return "webcommon__icon webcommon__icon webcommon__multi-field-warning-icon";
			case FC.Viewmodels.FormEditor.ErrorType.regionError:
				return "webcommon__icon webcommon__region-error-icon";
			case FC.Viewmodels.FormEditor.ErrorType.assemblyError:
				return "webcommon__icon webcommon__assembling-error-icon";
			default:
				return "";
		}
	});
}

FC.Viewmodels.FormEditor.GlobalId = 0;

FC.Viewmodels.FormEditor.ErrorType = {

	nonVerified: -1, /* поля требует верификации */

	error: 0, /* ошибка */

	warning: 1, /* предупреждение */

	multiFieldError: 2, /* многополевая ошибка */

	multiFieldWarning: 3, /* многополевое предупреждение */

	regionError: 4, /* ошибка наложения региона */

	assemblyError: 5 /* ошибка сборки */
};