﻿FC.Viewmodels.FormEditor = FC.Viewmodels.FormEditor || {};

var SELECTED_ERRORS_KEY = 'selectedErrors';

FC.Viewmodels.FormEditor.ErrorsPanel = function (currentDocument, selectedField, selectedDocument, horizontalSeparator, editorWidth, scrollToField, dependencies) {
	var self = this;

	self.currentDocument = currentDocument;
	self.selectedDocument = selectedDocument;
	self.selectedField = selectedField;

	self.autoSelectError = 0;
	self.fieldOrder = ko.observable();

	self.multiFieldErrorsSelected = ko.observable(true);
	self.multiFieldWarningsSelected = ko.observable(true);
	self.regionErrorsSelected = ko.observable(true);
	self.assemblyErrorsSelected = ko.observable(true);
	self.errorsSelected = ko.observable(true);
	self.warningsSelected = ko.observable(true);
	self.fieldsToVerifySelected = ko.observable(true);

	// Загрузим выбранные ранее фильтры ошибок
	var previousSelections = localStorage.getItem(SELECTED_ERRORS_KEY);
	if (previousSelections) {
		selectedErrors = JSON.parse(previousSelections);

		self.multiFieldErrorsSelected(selectedErrors.multiFieldErrorsSelected);
		self.multiFieldWarningsSelected(selectedErrors.multiFieldWarningsSelected);
		self.regionErrorsSelected(selectedErrors.regionErrorsSelected);
		self.assemblyErrorsSelected(selectedErrors.assemblyErrorsSelected);
		self.errorsSelected(selectedErrors.errorsSelected);
		self.warningsSelected(selectedErrors.warningsSelected);
		self.fieldsToVerifySelected(selectedErrors.fieldsToVerifySelected);
	}

	/** Запомнить выбранные фильтры ошибок */
	function saveErrorsSelection() {
		var data = JSON.stringify({
			multiFieldErrorsSelected: self.multiFieldErrorsSelected(),
			multiFieldWarningsSelected: self.multiFieldWarningsSelected(),
			regionErrorsSelected: self.regionErrorsSelected(),
			assemblyErrorsSelected: self.assemblyErrorsSelected(),
			errorsSelected: self.errorsSelected(),
			warningsSelected: self.warningsSelected(),
			fieldsToVerifySelected: self.fieldsToVerifySelected()
		});

		localStorage.setItem(SELECTED_ERRORS_KEY, data);
	}

	self.multiFieldErrorsSelected.subscribe(saveErrorsSelection);
	self.multiFieldWarningsSelected.subscribe(saveErrorsSelection);
	self.regionErrorsSelected.subscribe(saveErrorsSelection);
	self.assemblyErrorsSelected.subscribe(saveErrorsSelection);
	self.errorsSelected.subscribe(saveErrorsSelection);
	self.warningsSelected.subscribe(saveErrorsSelection);
	self.fieldsToVerifySelected.subscribe(saveErrorsSelection);

	function analyzeNode(node) {
		var fieldIdAttr = node.attr('data-fieldId');
		var columnsInfo = node.attr('data-columnsorder');
		var isRoot = node.hasClass('customform__PagesAndCaptions');
		var subFields = [];
		if (columnsInfo) {
			$.each(columnsInfo.split(','), function (ind, val) {
				subFields.push({
					id: parseInt(val),
					children: []
				});
			});
		}
		node.children().each(function () {
			var analyzeResult = analyzeNode($(this));
			if ($.isArray(analyzeResult)) {
				subFields = subFields.concat(analyzeResult);
			} else {
				subFields.push(analyzeResult);
			}
		});
		if (fieldIdAttr && parseInt(fieldIdAttr) > 0 && !isRoot) {
			return {
				id: parseInt(fieldIdAttr),
				children: subFields
			};
		} else {
			return subFields;
		}

	}

	function selectAndScrollToField(field) {
		if (field instanceof FC.BusinessModels.DocumentEditor.Section) return;
		scrollToField(field);
		var selectedError = self.selectedError();
		self.selectedField(null);
		if (selectedError)
			selectedError.selected(true);
		self.selectedField(field);
	}

	ko.computed(function () {
		if (!self.currentDocument()) {
			return '';
		}
		return FC.Utils.DocumentEditor.GetCustomFormKey(self.currentDocument());
	}).subscribe(function (customFormKey) {
		self.fieldOrder(null);
		if (!customFormKey) return;
		var interval = setInterval(function () {
			var customFormDescription = dependencies.environment.Templates[customFormKey];
			if (!customFormDescription) return;
			clearInterval(interval);

			var $customFormTag = $(customFormDescription.text);
			self.fieldOrder({
				customFormKey: customFormKey,
				order: analyzeNode($customFormTag)
			});
		}, 100);
	});

	function compareTwoFieldsPaths(leftPath, rightPath) {
		if (!self.fieldOrder()) return 0;
		if (!leftPath && !rightPath) return 0;
		if (!leftPath) return -1;
		if (!rightPath) return 1;
		var minPathLength = Math.min(leftPath.length, rightPath.length);
		for (var i = 0; i < minPathLength; i++) {
			var leftPathElem = leftPath[i];
			var rightPathElem = rightPath[i];
			if (leftPathElem.type === rightPathElem.type && leftPathElem.id === rightPathElem.id) continue;

			if (leftPathElem.type === rightPathElem.type &&
				leftPathElem.type === FC.Viewmodels.FormEditor.IndexType.RepeateIndex) {
				return leftPathElem.id > rightPathElem.id ? 1 : -1;
			}

			var parentArray = self.fieldOrder().order;
			if (i > 0) {
				var parentPath = leftPath.slice(0, i).filter(function (val) {
					return val.type === FC.Viewmodels.FormEditor.IndexType.ElementId;
				});

				if (parentPath) {
					for (var j = 0; j < parentPath.length; j++) {
						var parentPathELement = parentPath[j];
						var prevParentArray = parentArray;
						for (var parentArrayIndex = 0; parentArrayIndex < parentArray.length; parentArrayIndex++) {
							if (parentArray[parentArrayIndex].id === parentPathELement.id) {
								parentArray = parentArray[parentArrayIndex].children;
								break;
							}
						}
						if (prevParentArray === parentArray) return 0;
					}
				}
			}
			if (parentArray && parentArray.length > 0) {
				var leftElem = parentArray.filter(function (val) { return val.id === leftPathElem.id; })[0];
				var rightElem = parentArray.filter(function (val) { return val.id === rightPathElem.id; })[0];
				if (!leftElem && !rightElem) return 0;
				if (!leftElem) return -1;
				if (!rightElem) return 1;
				var leftElemIndex = parentArray.indexOf(leftElem);
				var rightElemIndex = parentArray.indexOf(rightElem);
				return leftElemIndex < rightElemIndex ? -1 : 1;
			}
		}
		if (leftPath.length === rightPath.length) return 0;
		return leftPath.length < rightPath.length ? -1 : 1;
	}

	var errorComparer = function (left, right) {
		if (left.type() === -2 && right.type() !== -2) {
			return -1;
		}
		if (left.type() !== -2 && right.type() === -2) {
			return 1;
		}


		var pathCompareResult = compareTwoFieldsPaths(left.firstConnectedFieldPath, right.firstConnectedFieldPath);
		if (pathCompareResult === 0) {
			if (left.actualType() === FC.Viewmodels.FormEditor.ErrorType.nonVerified) return 1;
			if (right.actualType() === FC.Viewmodels.FormEditor.ErrorType.nonVerified) return -1;

			if (left.actualType() === right.actualType) return 0;
			return left.actualType() < right.actualType ? -1 : 1;
		} else {
			return pathCompareResult;
		}
	}

	self.allErrors = ko.computed(function () {

		var t0 = performance.now();

		var doc = self.currentDocument();
		if (!doc) {
			return [];
		}

		var errors = doc.errors() || [];
		var assemblingErrors = doc.assemblingErrors() || [];

		errors = assemblingErrors.concat(errors);

		if (!errors || !errors.length) {
			return [];
		}

		var res = $.map(errors,
			function (error) {
				var allFields = doc.manipulators.errorsManipulator.fieldsErrors().filter(function (val) { return val.error == error; });
				var parentFields = [
					{
						parent: null,
						fields: []
					}
				];
				$.each(allFields,
					function (ind, errorFieldPair) {
						var field = errorFieldPair.field;
						var parentField = field.parent;
						var parentFound = false;
						var obsField = field;
						while (parentField) {
							if (parentField.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent ||
								parentField.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Table) {
								var pFieldInfo;
								for (var i = 0; i < parentFields.length; i++) {
									pFieldInfo = parentFields[i];
									if (pFieldInfo.parent == parentField) {
										break;
									}
									pFieldInfo = null;
								}

								if (!pFieldInfo) {
									pFieldInfo = {
										parent: parentField,
										fields: []
									}
									parentFields.push(pFieldInfo);
								}
								pFieldInfo.fields.push({
									field: field,
									index: (parentField.rows ? parentField.rows().indexOf(obsField) : parentField.fields().indexOf(obsField)) + 1
								});
								parentFound = true;
								break;
							}
							obsField = parentField;
							parentField = parentField.parent;
						}
						if (!parentFound) {
							parentFields[0].fields.push({
								field: field
							});
						}
					});

				var fieldsToErrorHead = [];
				$.each(parentFields,
					function (ind, parentGroupInfo) {
						if (parentGroupInfo.fields.length === 0) return;
						if (!parentGroupInfo.parent) {
							fieldsToErrorHead = fieldsToErrorHead.concat(parentGroupInfo.fields.map(function (fieldInfo) {
								return {
									name: fieldInfo.field.caption(),
									isVisible: fieldInfo.field.isVisible(),
									isReadOnly: fieldInfo.field.isReadOnly && fieldInfo.field.isReadOnly(),
									select: function () {
										selectAndScrollToField(fieldInfo.field);
									}
								}
							}));
						} else {
							var fieldIdsInfo = [];
							var fieldIdsInfoKeys = [];
							$.each(parentGroupInfo.fields,
								function (index, fieldInfo) {
									var fieldIdInfo = fieldIdsInfo[fieldInfo.field.id()];
									if (!fieldIdInfo) {
										fieldIdInfo = {
											name: fieldInfo.field.caption(),
											minIndex: -1,
											targetField: null,
											indexes: []
										};
										fieldIdsInfo[fieldInfo.field.id()] = fieldIdInfo;
										fieldIdsInfoKeys.push(fieldInfo.field.id());
									}
									if (!fieldIdInfo.targetField || fieldIdInfo.minIndex > fieldInfo.index) {
										fieldIdInfo.minIndex = fieldInfo.index;
										fieldIdInfo.targetField = fieldInfo.field;
									}
									fieldIdInfo.indexes.push(fieldInfo.index);
								});

							$.each(fieldIdsInfoKeys,
								function (ind, fieldIdInfoKey) {
									var fieldIdInfo = fieldIdsInfo[fieldIdInfoKey];
									if (!fieldIdInfo) return;

									var sortNumber = function (a, b) {
										return a - b;
									};

									var indexes = fieldIdInfo.indexes.sort(sortNumber);
									var isOneByOne = true;
									for (var i = 1; i < indexes.length; i++) {
										if (indexes[i] - indexes[i - 1] > 1) {
											isOneByOne = false;
											break;
										}
									}
									var fieldGroupName = fieldIdInfo.targetField.caption() +
										'[' +
										(isOneByOne && indexes.length > 1 ? (indexes[0] + '-' + indexes[indexes.length - 1]) : indexes.join()) +
										']';
									fieldsToErrorHead.push({
										name: fieldGroupName,
										isVisible: fieldIdInfo.targetField.isVisible ? fieldIdInfo.targetField.isVisible() : true,
										isReadOnly: fieldIdInfo.targetField.isReadOnly ? fieldIdInfo.targetField.isReadOnly() : false,
										select: function () {
											selectAndScrollToField(fieldIdInfo.targetField);
										}
									});
								});
						}
					});

				var errorViewmodelParams = {
					fields: fieldsToErrorHead,
					connectedFields: $.map(allFields,
						function (val) { return val.field; })
				};

				if (error instanceof FC.BusinessModels.DocumentEditor.AssemblingError) {

					// ошибка порядка секций
					if (error.type() == 2) {
						errorViewmodelParams.message = dependencies.environment.Localization.GlobalResources.CorrectSectionOrder_ErrorPanel;

						// ошибка несоответствия ключевых полей
					} else if (error.type() == 3) {
						errorViewmodelParams.message = dependencies.environment.Localization.GlobalResources.KeyFieldMismatch_ErrorPanel;
					} else {
						errorViewmodelParams.message = error.description();
					}

					errorViewmodelParams.type = -2;
					errorViewmodelParams.severity = true;

				} else {
					errorViewmodelParams.message = error.message();
					errorViewmodelParams.severity = error.severity();
					errorViewmodelParams.type = error.type();
				}

				return new FC.Viewmodels.FormEditor.ErrorViewmodel(errorViewmodelParams);
			});


		FC.Utils.EventLogger.Default.addMessage('ErrorsPanel', "allErrors computing - " + (performance.now() - t0) + " ms");

		return res;
	});

	function getElementsCountByActualType(typeId) {
		var count = 0;

		var allErrors = self.allErrors();
		count = ko.utils.arrayFilter(allErrors, function (error) {
			return error.actualType() === typeId;
		}).length;

		return count;
	}

	function getFieldIndexRelativeToParent(field) {
		if (!field) {
			return -1;
		}

		var parentField = field.parent;
		if (!parentField) {
			return -1;
		}

		var index = -1;
		if (parentField.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent) {

			// ищем индекс повторяющегося поля
			if (parentField.hasOwnProperty('fields') && parentField.fields()) {
				index = parentField.fields().indexOf(field);
			}
		} else if (parentField instanceof FC.BusinessModels.DocumentEditor.TableRow) {

			// ищем индеус строки таблицы
			var table = parentField.parent;
			if (parentField && table.hasOwnProperty('rows') && table.rows()) {
				index = table.rows().indexOf(parentField);
			}
		} else if (parentField.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.GroupParent &&
			parentField.parent &&
			parentField.parent.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent) {

			// ищем индекс инстанса повторяющегося поля
			var instanceParent = parentField.parent;

			// ищем индекс повторяющегося поля
			if (instanceParent.hasOwnProperty('fields') && instanceParent.fields()) {
				index = instanceParent.fields().indexOf(parentField);
			}
		}

		return index;
	}

	var sortFieldsByPath = function (fields) {
		if (!fields || fields.length < 2) {
			return fields;
		}

		return fields.map(function (val) { return { field: val, fieldPath: val.getFieldPath() }; })
			.sort(function (left, right) {
				return compareTwoFieldsPaths(left.fieldPath, right.fieldPath);
			}).map(function (f) { return f.field; });
	};

	var fieldToErrorViewModel = function (field) {
		var fieldIndex = getFieldIndexRelativeToParent(field);
		var fieldName = field.caption();
		if (fieldIndex >= 0) {
			fieldName += ' [' + (fieldIndex + 1) + ']';
		}

		return new FC.Viewmodels.FormEditor.ErrorViewmodel({
			fields: [{
				name: fieldName,
				isVisible: field.isVisible(),
				select: function () {
					selectAndScrollToField(field);
				}
			}],
			message: '',
			type: -1,
			connectedFields: [field]
		});
	};

	self.fieldsWithUncertainSymbols = ko.observableArray([]);

	var calculateFieldsWithUncertainSymbols = function (doc) {
		if (!doc) return [];

		var allFields = doc.allFields();

		if (!allFields) return [];

		var fields = sortFieldsByPath($.map(allFields.filter(function (fieldWrapper) {
			var field = fieldWrapper.field;
			if (field.isVisible && !field.isVisible()) return false;
			if (field instanceof FC.BusinessModels.DocumentEditor.TextField) {
				return field.hasUncertainSymbol();
			} else return false;
		}), function (val) { return val.field }));

		var count = 0;

		if (fields) {
			for (var i = 0; i < fields.length; i++) {
				count += fields[i].verificationSymbolsCount();
			}
		}

		self.selectedDocument().VerificationSymbolsCount(count);

		return fields;
	};

	self.fieldsNeedToVerify = ko.observableArray([]);

	var calculateFieldsNeedToVerify = function (doc) {
		var t0 = performance.now();

		if (!doc) return [];

		var allFields = doc.allFields();

		if (!allFields) return [];

		var res = $.map(allFields.filter(function (fieldWrapper) {
			var field = fieldWrapper.field;
			if (field.isVisible && !field.isVisible()) return false;
			var isNotChoiceFieldChild = !(field.parent && field.parent instanceof FC.BusinessModels.DocumentEditor.ChoiceField);
			return ((field instanceof FC.BusinessModels.DocumentEditor.BooleanField ||
				field instanceof FC.BusinessModels.DocumentEditor.ChoiceField ||
				field instanceof FC.BusinessModels.DocumentEditor.TextField) && isNotChoiceFieldChild) &&
				(field.needVerification() || field.needDoubleVerification()) &&
				!field.isVerified();
		}),
			function (fieldWrapper) {
				var field = fieldWrapper.field;
				return fieldToErrorViewModel(field);
			});

		FC.Utils.EventLogger.Default.addMessage('ErrorsPanel', "fieldsNeedToVerify computing - " + (performance.now() - t0) + " ms");
		return res;
	};

	self.needVerifyAndUncertainFields = ko.observableArray([]);

	/** Имеется ли хотя бы одна ошибка */
	self.hasErrors = ko.computed(function () {
		return self.allErrors().length + self.needVerifyAndUncertainFields().length > 0;
	});

	self.recalculateFieldsNeedAttention = function (doc) {
		doc = doc || self.currentDocument();


		self.fieldsWithUncertainSymbols(calculateFieldsWithUncertainSymbols(doc));
		self.fieldsNeedToVerify(calculateFieldsNeedToVerify(doc));

		var needVerifyFields = $.map(self.fieldsNeedToVerify(), function (field) { return field.connectedFields[0]; });
		var uncertainFields = self.fieldsWithUncertainSymbols();

		var uniqueNeedVerifyFields = needVerifyFields.filter(function (item) {
			return uncertainFields.indexOf(item) === -1;
		});

		self.needVerifyAndUncertainFields(uncertainFields.concat(uniqueNeedVerifyFields).map(function (field) {
			var errorViewModel = fieldToErrorViewModel(field);
			errorViewModel.cachedHeight = 35;
			return errorViewModel;
		}));
	};

	var fieldsTriggerSubscription = null;
	self.currentDocument.subscribe(function (doc) {
		self.recalculateFieldsNeedAttention(doc);

		if (fieldsTriggerSubscription) {
			fieldsTriggerSubscription.dispose();
			fieldsTriggerSubscription = null;
		}

		if (doc) {
			fieldsTriggerSubscription = doc.fieldsChangedTrigger.subscribe(function () { self.recalculateFieldsNeedAttention() });
		}
	});

	self.fieldsNeedAttention = ko.computed(function () {
		var res = self.fieldsWithUncertainSymbols().concat($.map(self.fieldsNeedToVerify(), function (field) { return field.connectedFields[0]; }));
		var doc = self.currentDocument();
		if (!doc) {
			return [];
		}

		res = res.concat($.map(doc.manipulators.errorsManipulator.fieldsErrors(),
			function (val) { return val.field; }));

		return sortFieldsByPath(res.filter(function (value, index, s) {
			return s.indexOf(value) === index;
		}));
	});

	self.onFieldVerified = function (field) {
		if (!field) {
			return;
		}

		var removeFieldFromArray = function (arrayObservable, f) {
			var array = arrayObservable();
			if (!array) {
				return false;
			}

			for (var j = 0; j < array.length; j++) {
				if (array[j] === field) {
					array.splice(j, 1);
					arrayObservable(array);
					return true;
				}
			}

			return false;
		};

		var removeError = false;

		if (field.hasOwnProperty("isVerified")) {
			var needVerify = (field.needVerification() || field.needDoubleVerification());

			if (field.isVerified() && needVerify) {
				removeFieldFromArray(self.fieldsNeedToVerify, field);
				removeError = true;
			}

			if (field instanceof FC.BusinessModels.DocumentEditor.TextField && (!field.hasUncertainSymbol() || field.isVerified())) {
				removeFieldFromArray(self.fieldsWithUncertainSymbols, field);
				removeError = !needVerify || field.isVerified();
			}
		}

		if (removeError) {
			var errors = self.needVerifyAndUncertainFields();
			for (var j = 0; j < errors.length; j++) {
				if (errors[j].connectedFields && errors[j].connectedFields.length > 0 && errors[j].connectedFields[0] === field) {
					errors.splice(j, 1);
					self.needVerifyAndUncertainFields(errors);
					break;
				}
			}
		}
	};

	dependencies.eventManager.bind("fieldUncertaintyChanged",
		function (e, data) {
			self.onFieldVerified(data.field);
		});

	self.errorsCount = ko.computed(function () {
		return getElementsCountByActualType(FC.Viewmodels.FormEditor.ErrorType.error);
	});

	self.warningsCount = ko.computed(function () {
		return getElementsCountByActualType(FC.Viewmodels.FormEditor.ErrorType.warning);
	});

	self.multiFieldErrorsCount = ko.computed(function () {
		return getElementsCountByActualType(FC.Viewmodels.FormEditor.ErrorType.multiFieldError);
	});

	self.multiFieldWarningsCount = ko.computed(function () {
		return getElementsCountByActualType(FC.Viewmodels.FormEditor.ErrorType.multiFieldWarning);
	});

	self.regionErrorsCount = ko.computed(function () {
		return getElementsCountByActualType(FC.Viewmodels.FormEditor.ErrorType.regionError);
	});

	self.assemblyErrorsCount = ko.computed(function () {
		return getElementsCountByActualType(FC.Viewmodels.FormEditor.ErrorType.assemblyError);
	});

	self.errorViewModeData = ko.observable(new FC.FlatMultibutton([
		{
			title: ko.observable(dependencies.environment.Localization.GlobalResources.ErrorViewMode_AssemblyErrors),
			nameSummary: ko.computed(function () {
				return self.assemblyErrorsCount();
			}),
			hidden: ko.computed(function () {
				return !self.assemblyErrorsCount();
			}),
			id: FC.Viewmodels.FormEditor.ErrorType.assemblyError,
			defaultState: self.assemblyErrorsSelected,
			sticky: true,
			iconCss: "webcommon__icon webcommon__assembling-error-icon"
		},
		{
			title: ko.observable(dependencies.environment.Localization.GlobalResources.ErrorViewMode_RegionErrors),
			nameSummary: ko.computed(function () {
				return self.regionErrorsCount();
			}),
			hidden: ko.computed(function () {
				return !self.regionErrorsCount();
			}),
			id: FC.Viewmodels.FormEditor.ErrorType.regionError,
			defaultState: self.regionErrorsSelected,
			sticky: true,
			iconCss: "webcommon__icon webcommon__region-error-icon"
		},
		{
			title: ko.observable(dependencies.environment.Localization.GlobalResources.ErrorViewMode_Errors),
			nameSummary: ko.computed(function () {
				return self.errorsCount();
			}),
			hidden: ko.computed(function () {
				return !self.errorsCount();
			}),
			id: FC.Viewmodels.FormEditor.ErrorType.error,
			defaultState: self.errorsSelected,
			sticky: true,
			iconCss: "webcommon__icon webcommon__error-icon"
		},
		{
			title: ko.observable(dependencies.environment.Localization.GlobalResources.ErrorViewMode_MultiFieldErrors),
			nameSummary: ko.computed(function () {
				return self.multiFieldErrorsCount();
			}),
			hidden: ko.computed(function () {
				return !self.multiFieldErrorsCount();
			}),
			id: FC.Viewmodels.FormEditor.ErrorType.multiFieldError,
			defaultState: self.multiFieldErrorsSelected,
			sticky: true,
			iconCss: "webcommon__icon webcommon__multi-field-error-icon"
		},
		{
			title: ko.observable(dependencies.environment.Localization.GlobalResources.ErrorViewMode_Warnings),
			nameSummary: ko.computed(function () {
				return self.warningsCount();
			}),
			hidden: ko.computed(function () {
				return !self.warningsCount();
			}),
			id: FC.Viewmodels.FormEditor.ErrorType.warning,
			defaultState: self.warningsSelected,
			sticky: true,
			iconCss: "webcommon__icon webcommon__warning-icon"
		},
		{
			title: ko.observable(dependencies.environment.Localization.GlobalResources.ErrorViewMode_MultiFieldWarnings),
			nameSummary: ko.computed(function () {
				return self.multiFieldWarningsCount();
			}),
			hidden: ko.computed(function () {
				return !self.multiFieldWarningsCount();
			}),
			id: FC.Viewmodels.FormEditor.ErrorType.multiFieldWarning,
			defaultState: self.multiFieldWarningsSelected,
			sticky: true,
			iconCss: "webcommon__icon webcommon__multi-field-warning-icon"
		},
		{
			title: ko.observable(dependencies.environment.Localization.GlobalResources.ErrorViewMode_ToVerify),
			nameSummary: ko.computed(function () {
				var fields = self.needVerifyAndUncertainFields();
				return '' + fields.length;
			}),
			hidden: ko.computed(function () {
				var fields = self.needVerifyAndUncertainFields();
				return !fields || !fields.length;
			}),
			id: FC.Viewmodels.FormEditor.ErrorType.nonVerified,
			defaultState: self.fieldsToVerifySelected,
			sticky: true,
			iconCss: "webcommon__icon webcommon__non-verified-icon"
		}],
		false
	));

	self.errorViewModeData().visibleItemsCount.subscribe(function (newCount) {
		var $element = $("#ErrorViewMenuButtons");

		if ($element.length && newCount > 0) {
			//чтобы успело отрисоваться
			setTimeout(function () {
				$element.carousel('updateContent');
				self.checkErrorPanelMinimization();
			}, 100);
		}
	});

	self.errors = ko.computed(function () {
		var t0 = performance.now();

		var filtererdErrors = [];

		var doc = self.currentDocument();
		if (!doc) {
			return [];
		}
		var docCustomFormKey = FC.Utils.DocumentEditor.GetCustomFormKey(doc);
		if (!self.fieldOrder() || self.fieldOrder().customFormKey !== docCustomFormKey) {
			return [];
		}
		var errorsSelected = self.errorsSelected();
		var warningsSelected = self.warningsSelected();
		var multiFieldErrorsSelected = self.multiFieldErrorsSelected();
		var multiFieldWarningsSelected = self.multiFieldWarningsSelected();
		var regionErrorsSelected = self.regionErrorsSelected();
		var assemblyErrorsSelected = self.assemblyErrorsSelected();

		var somethingSelected = errorsSelected || warningsSelected
			|| multiFieldErrorsSelected || multiFieldWarningsSelected
			|| regionErrorsSelected || assemblyErrorsSelected;

		if (somethingSelected) {

			filtererdErrors = self.allErrors().filter(function (error) {
				var errorType = error.actualType();
				var isVisible = (errorType === FC.Viewmodels.FormEditor.ErrorType.error && errorsSelected) ||
					(errorType === FC.Viewmodels.FormEditor.ErrorType.warning && warningsSelected) ||
					(errorType === FC.Viewmodels.FormEditor.ErrorType.multiFieldError && multiFieldErrorsSelected) ||
					(errorType === FC.Viewmodels.FormEditor.ErrorType.multiFieldWarning && multiFieldWarningsSelected) ||
					(errorType === FC.Viewmodels.FormEditor.ErrorType.regionError && regionErrorsSelected) ||
					(errorType === FC.Viewmodels.FormEditor.ErrorType.assemblyError && assemblyErrorsSelected);

				if (!isVisible && error.selected()) {
					error.selected(false);
				}
				return isVisible;
			});
		}

		if (self.fieldsToVerifySelected()) {
			filtererdErrors = filtererdErrors.concat(self.needVerifyAndUncertainFields());
		} else {
			ko.utils.arrayForEach(self.needVerifyAndUncertainFields(), function (item) {
				item.selected(false);
			});
		}

		filtererdErrors = filtererdErrors.sort(errorComparer);
		FC.Utils.EventLogger.Default.addMessage('ErrorsPanel', "errors computing - " + (performance.now() - t0) + " ms");

		return filtererdErrors;
	}).extend({ deferred: true });

	self.allErrors.subscribe(function (newValues) {
		var selectedDocument = ko.unwrap(self.selectedDocument);
		if (selectedDocument && selectedDocument.hasOwnProperty('updateDocumentFormErrors')) {
			selectedDocument.updateDocumentFormErrors(newValues.filter(function (error) { return error.type() !== -2; }));
		}
	});

	self.errors.subscribe(function (val) {
		if (!val || val.length === 0) return;
		if (self.autoSelectError !== 0) {
			if (self.autoSelectError === 1) {
				self.nextError();
			} else if (self.autoSelectError === 2) {
				self.selectWorstError();
			} else {
				self.previousError();
			}
			self.autoSelectError = 0;
		}
	});

	self.selectedError = ko.computed(function () {
		var t0 = performance.now();

		var selectedErrors = ko.utils.arrayFilter(self.errors(), function (error) { return error.selected(); });

		FC.Utils.EventLogger.Default.addMessage('ErrorsPanel', "selectedError computing - " + (performance.now() - t0) + " ms");
		return selectedErrors && selectedErrors.length ? selectedErrors[0] : null;
	});

	self.selectedError.subscribe(function (val) {
		if (!val) return;

		var errors = self.errors();
		var errorIndex = errors.indexOf(val);
		self.scroller.scrollToIndex(errorIndex);
	});

	self.selectedField.subscribe(function (field) {
		var t0 = performance.now();

		if (!field) {
			if (self.selectedError()) {
				self.selectedError().selected(false);
			}
		} else {
			if (self.selectedError()) {
				if (self.selectedError().connectedFields.indexOf(field) >= 0) {
					return;
				}

				self.selectedError().selected(false);
			}

			for (var i = 0; i < self.errors().length; i++) {
				var error = self.errors()[i];
				if (error.connectedFields.indexOf(field) >= 0) {
					error.selected(true);
					return;
				}
			}
		}

		FC.Utils.EventLogger.Default.addMessage('ErrorsPanel', "selectedField subscription - " + (performance.now() - t0) + " ms");
	});

	self.selectError = function (error) {
		var t0 = performance.now();

		if (!error) {
			return;
		}

		var fields = error.fields();
		if (!fields || !fields.length) {
			return;
		}

		ko.utils.arrayForEach(self.errors(), function (err) {
			err.selected(false);
		});

		error.selected(true);
		for (var i = 0; i < fields.length; i++) {
			if (!fields[i].isReadOnly()) {
				fields[i].selectField();
				return;
			}
		}
		fields[0].selectField();
		FC.Utils.EventLogger.Default.addMessage('ErrorsPanel', "selectError - " + (performance.now() - t0) + " ms");
	}

	self.selectWorstError = function () {
		var allErrors = self.allErrors().sort(errorComparer);
		var selectedError;
		for (var i = 0; i < allErrors.length; i++) {
			var error = allErrors[i];
			if (!error.severity()) continue;

			if (error.type() < -2) {
				selectedError = error;
				break;
			}
			if (!selectedError) {
				selectedError = error;
			}
		}
		if (selectedError) {
			self.selectError(selectedError);
		} else {
			selectAndScrollToField(self.fieldsWithUncertainSymbols()[0]);
		}
	}

	self.listContainer = ko.observable(null);
	self.scrollContainer = ko.observable(null);

	self.scroller = new FC.Viewmodels.FormEditor.ErrorsScroller(self.scrollContainer, horizontalSeparator, self.errors);

	function getSelectableErrors() {
		return ko.utils.arrayFilter(self.errors(), function (error) {
			var errorFields = error.fields();
			return errorFields && errorFields.length > 0;
		});
	}

	self.nextError = function () {
		var t0 = performance.now();

		var selectableErrors = getSelectableErrors();

		var selectedError = null;
		for (var i = 0; i < selectableErrors.length; i++) {
			if (selectableErrors[i].selected()) {
				selectableErrors[i].selected(false);
				if (i < selectableErrors.length - 1) {
					selectedError = selectableErrors[i + 1];
				} else {
					self.autoSelectError = 1;
					window.batchApp.extensions.navigateToNextError();
				}
				break;
			}
		}

		if (!selectedError && selectableErrors.length > 0) {
			selectedError = selectableErrors[0];
		}

		var fields = selectedError ? selectedError.fields() : null;
		if (fields && fields.length > 0) {
			selectedError.selected(true);
			for (var i = 0; i < fields.length; i++) {
				if (!fields[i].isReadOnly()) {
					fields[i].selectField();
					FC.Utils.EventLogger.Default.addMessage('ErrorsPanel', "moving to nextError - " + (performance.now() - t0) + " ms");
					return;
				}
			}

			fields[0].selectField();
		}

		FC.Utils.EventLogger.Default.addMessage('ErrorsPanel', "moving to nextError - " + (performance.now() - t0) + " ms");
	};

	self.previousError = function () {
		var t0 = performance.now();

		var selectableErrors = getSelectableErrors();

		var selectedError = null;
		for (var i = 0; i < selectableErrors.length; i++) {
			if (selectableErrors[i].selected()) {
				selectableErrors[i].selected(false);
				if (i > 0) {
					selectedError = selectableErrors[i - 1];
				} else {
					self.autoSelectError = -1;
					window.batchApp.extensions.navigateToPreviousError();
				}
				break;
			}
		}

		if (!selectedError && selectableErrors.length > 0) {
			selectedError = selectableErrors[selectableErrors.length - 1];
		}

		var fields = selectedError ? selectedError.fields() : null;
		if (fields && fields.length > 0) {
			selectedError.selected(true);
			for (var i = 0; i < fields.length; i++) {
				if (!fields[i].isReadOnly()) {
					fields[i].selectField();
					FC.Utils.EventLogger.Default.addMessage('ErrorsPanel', "moving to previousError - " +
						(performance.now() - t0) + " ms");
					return;
				}
			}

			fields[0].selectField();
		}

		FC.Utils.EventLogger.Default.addMessage('ErrorsPanel',
			"moving to previousError - " + (performance.now() - t0) + " ms");
	};

	self.errorNavigator = {
		nonContextOperations: ko.observableArray([
			new FC.NonContextFunction({
				name: ko.observable(null),
				title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(dependencies.environment.Localization.GlobalResources.PreviousItemToVerify, dependencies.environment.Localization.NonLocalizedHotKeys.PreviousItemToVerifyHotkey),
				order: 1,
				action: self.previousError,
				htmlClass: 'batchEditor__nonContextOperations__previousError',
				enabled: ko.computed(function () {
					var errors = self.errors();
					return errors && errors.length > 0;
				})
			}),
			new FC.NonContextFunction({
				name: ko.observable(null),
				title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(dependencies.environment.Localization.GlobalResources.NextItemToVerify, dependencies.environment.Localization.NonLocalizedHotKeys.NextItemToVerifyHotkey),
				order: 2,
				action: self.nextError,
				htmlClass: 'batchEditor__nonContextOperations__nextError',
				enabled: ko.computed(function () {
					var errors = self.errors();
					return errors && errors.length > 0;
				})
			})
		])
	};

	self.checkErrorPanelMinimization = function () {

		var $menu = $('#errorViewMenu');
		if (!$menu || $menu.length === 0 || !$menu.is(":visible")) {
			return;
		}

		var $leftPart = $('#errorNavigaror');
		var $rightPart = $("#ErrorViewMenuButtons");

		if (!$leftPart.length || !$rightPart.length) {
			return;
		}

		$rightPart.carousel('init',
			{
				minShiftWidth: 50,
				carouselWidth: 'auto'
			});

		var menuWidth = $menu.width(); // ширина без полей
		var leftPartWidth = FC.Utils.getElementWidth($leftPart[0]);
		var rightPartWidth = FC.Utils.getElementWidth($rightPart[0]);
		var bufferZoneWidth = 10; // небольшой запас, чтобы правая часть никогда не наезжала на левую

		if ((leftPartWidth + rightPartWidth + bufferZoneWidth) > menuWidth) {
			var newRightPartWidth = menuWidth - leftPartWidth - bufferZoneWidth;
			$rightPart.carousel('update', newRightPartWidth);
		} else {
			if ($rightPart.carousel('canScroll')) {
				var newRightPartWidth = menuWidth - leftPartWidth - bufferZoneWidth;
				$rightPart.carousel('update', newRightPartWidth);
			}
		}

		return;
	};


	function setRange(element, start, stop) {
		window.getSelection().removeAllRanges();
		var range = document.createRange();
		range.setStart(element, start);
		range.setEnd(element, stop);
		window.getSelection().addRange(range);
	}



	function selectErrorField(field, isReversed) {
		selectAndScrollToField(field);
		if (self.selectedField() instanceof FC.BusinessModels.DocumentEditor.TextField && self.selectedField().hasUncertainSymbol()) {
			var rangeElement = $(window.getSelection().focusNode).parent();
			if (rangeElement.is('span')) {
				rangeElement = rangeElement.parent();
			}

			var spans = rangeElement.children('span');
			var nextTextElement = spans[isReversed ? spans.length - 1 : 0].firstChild;
			setRange(nextTextElement, isReversed ? nextTextElement.length - 1 : 0, isReversed ? nextTextElement.length : 1);
		}
	}
	function openNextField(fieldsSource, isReversed, searchAlgo) {

		var i = 0;
		while (fieldsSource[i] && compareTwoFieldsPaths(fieldsSource[i].getFieldPath(), self.selectedField().getFieldPath()) <= 0) {
			i++;
		}
		var nextFieldIndex = i - (isReversed ? 1 + (fieldsSource.indexOf(self.selectedField() >= 0) ? 1 : 0) : 0);
		if (fieldsSource[nextFieldIndex]) {
			selectErrorField(fieldsSource[nextFieldIndex], isReversed);

		} else {
			window.batchApp.extensions.navigateToError(isReversed ? -1 : 1, searchAlgo);
		}
	}

	function trySelectNextUncertainInField(isReversed) {
		if (!(self.selectedField() instanceof FC.BusinessModels.DocumentEditor.TextField)) return false;
		if (!self.selectedField().hasUncertainSymbol()) return false;

		var selection = window.getSelection();
		var selectedElement = $(selection.focusNode);
		var selectInTextField = selectedElement.parents('.field_value').length > 0;
		if (!selectInTextField)
			return false;

		var selectedParent = selectedElement;
		var parentSpan;
		if (selectedParent.parent().is('span')) {
			parentSpan = selectedParent;
			selectedParent = selectedParent.parent();
		}

		var prevUncertainGroups = selectedParent.prevAll('span').length;
		var uncertainGroupIndex = -1;
		var flags = self.selectedField().flags();
		var uncertainGroupStarted = false,
			previousGroupStartIndex = -1,
			previousGroupLength = -1,
			currentGroupStartIndex = -1,
			currentGroupLength = -1,
			nextGroupIndex = -1;

		for (var j = 0; j < flags.length; j++) {
			if (flags[j] !== '0' && flags[j] !== '\n') {
				if (uncertainGroupStarted) {
					if (previousGroupStartIndex !== -1 && currentGroupStartIndex === -1) {
						previousGroupLength++;
					} else if (currentGroupStartIndex !== -1 && nextGroupIndex === -1) {
						currentGroupLength++;
					}
				} else {
					uncertainGroupIndex++;
					if (prevUncertainGroups > 0 && prevUncertainGroups === uncertainGroupIndex + 1) {
						previousGroupLength = 1;
						previousGroupStartIndex = uncertainGroupIndex;
					} else if (prevUncertainGroups === uncertainGroupIndex) {
						currentGroupLength = 1;
						currentGroupStartIndex = uncertainGroupIndex;
					} else if (currentGroupStartIndex !== -1) {
						nextGroupIndex = uncertainGroupIndex;
						break;
					}
					uncertainGroupStarted = true;
				}
			} else {
				uncertainGroupStarted = false;
			}
		}


		var offsetStart = Math.min(selection.anchorOffset, selection.focusOffset);
		var offsetEnd = Math.max(selection.anchorOffset, selection.focusOffset);

		var nextTextElement;
		if (isReversed) {
			if (parentSpan && offsetStart > 0) {
				setRange(selectedElement[0], offsetStart - 1, offsetStart);
			} else if (previousGroupStartIndex !== -1) {
				nextTextElement = (selectedParent.hasClass('field_value') ? selectedParent.children('span').eq(previousGroupStartIndex) : selectedParent.prevAll('span'))[0].firstChild;
				setRange(nextTextElement, nextTextElement.length - 1, nextTextElement.length);
			} else {
				return false;
			}
		} else {
			if (parentSpan && currentGroupLength > offsetEnd) {
				setRange(selectedElement[0], offsetEnd, offsetEnd + 1);
			} else if ((!parentSpan && currentGroupStartIndex !== -1 && currentGroupStartIndex <= prevUncertainGroups) || (nextGroupIndex !== -1)) {
				nextTextElement = (selectedParent.hasClass('field_value') ? selectedParent.children('span').eq(currentGroupStartIndex) : selectedParent.nextAll('span'))[0].firstChild;
				setRange(nextTextElement, 0, 1);
			} else {
				return false;
			}
		}

		return true;
	}


	self.goToNextUncertainSymbolField = function () {
		var uncertainSymbolFields = self.fieldsWithUncertainSymbols();
		if (!uncertainSymbolFields || uncertainSymbolFields.length === 0) {
			return;
		}


		if (!self.selectedField()) {
			selectErrorField(uncertainSymbolFields[0], false);
			return;
		}

		if (!trySelectNextUncertainInField(false)) {
			window.getSelection().removeAllRanges();
			openNextField(self.fieldsWithUncertainSymbols(), false, window.batchApp.extensions.searchAlgos.UNCERTAIN);
		}

	}

	self.goToPreviousUncertainSymbolField = function () {
		var uncertainSymbolFields = self.fieldsWithUncertainSymbols();
		if (!uncertainSymbolFields || uncertainSymbolFields.length === 0) {
			return;
		}


		if (!self.selectedField()) {
			selectErrorField(uncertainSymbolFields[uncertainSymbolFields.length - 1], true);
			return;
		}

		if (!trySelectNextUncertainInField(true)) {
			window.getSelection().removeAllRanges();
			openNextField(self.fieldsWithUncertainSymbols(), true, window.batchApp.extensions.searchAlgos.UNCERTAIN);
		}
	}

	self.goToNextFieldNeedAttention = function () {
		var fieldsNeedAttention = self.fieldsNeedAttention();

		if (!self.selectedField() && fieldsNeedAttention && fieldsNeedAttention.length > 0) {
			selectErrorField(fieldsNeedAttention[0], false);

			return;
		}

		//if (!trySelectNextUncertainInField(false)) {
			//window.getSelection().removeAllRanges();
			openNextField(fieldsNeedAttention, false, window.batchApp.extensions.searchAlgos.BOTH);
		//}
	}

	self.goToPreviousFieldNeedAttention = function () {
		var fieldsNeedAttention = self.fieldsNeedAttention();
		if (!fieldsNeedAttention || fieldsNeedAttention.length === 0) {
			return;
		}


		if (!self.selectedField()) {
			selectErrorField(fieldsNeedAttention[fieldsNeedAttention.length - 1], true);
			return;
		}
		if (!trySelectNextUncertainInField(true)) {
			window.getSelection().removeAllRanges();
			openNextField(fieldsNeedAttention, true, window.batchApp.extensions.searchAlgos.BOTH);
		}
	}

	self.onResize = function () {
		self.checkErrorPanelMinimization();

		if (self.listContainer()) {
			var rect = self.listContainer().getBoundingClientRect();

			self.scroller.viewPanelWidth(rect.width);
			self.scroller.viewPanelHeight(rect.height);
		}
	};

	jQuery(document)
		.ready(function () {
			self.checkErrorPanelMinimization();
		});
}

