﻿FC.Viewmodels.FormEditor = FC.Viewmodels.FormEditor || {};
FC.Viewmodels.FormEditor.ErrorsScroller = function (scrollContainer, horizontalSeparator, errors) {
	var self = this;

	var errorsHeaderHeight = 54;
	self.errors = errors;

	self.viewPanelWidth = ko.observable();
	self.viewPanelHeight = ko.observable();

	self.listContainerHeight = ko.computed(function () {
		var viewPanelWidth = self.viewPanelWidth(); // для срабатывания пересчёта
		var viewPanelHeight = self.viewPanelHeight(); // для срабатывания пересчёта

		var elem = scrollContainer();
		var separatorHeight = horizontalSeparator();

		if (elem && separatorHeight) {
			return separatorHeight - errorsHeaderHeight;
		}

		return 0;
	});

	self.fieldTest = ko.observable();
	self.messageTest = ko.observable();

	self.totalHeight = ko.computed(function () {
		var viewPanelWidth = self.viewPanelWidth(); // для срабатывания пересчёта

		var errorsValue = errors();

		// Будем кэшировать рассчитанные высоты для одинаковых строк, чтобы не считать повторно
		var fieldsHeightCache = {};
		var messagesHeightCache = {};

		var result = 16; // padding-top: 8 + padding-bottom: 8

		for (var i = 0; i < errorsValue.length; i++) {
			if (errorsValue[i].hasOwnProperty("cachedHeight") && errorsValue[i].cachedHeight > 0) {
				result += errorsValue[i].cachedHeight;
			} else {
				result += self.calculateErrorHeight(errorsValue[i], fieldsHeightCache, messagesHeightCache);
			}
		}

		return result;
	}).extend({ deferred: true });

	self.totalTopOffset = ko.observable(0);
	self.errorVerticalScroll = new FC.ScrollBarNew(FC.ScrollBarNew.Orientation.Vertical,
		self.listContainerHeight,
		self.totalTopOffset,
		self.listContainerHeight,
		self.totalHeight);

	/**
	 * Смещение отображаемых ошибок от верха (т.к. отображение не всегда начинается с верхнего края верхней ошибки, а то скролл будет дискретным)
	 */
	self.topOffset = ko.observable(0);

	errors.subscribe(function () {
		self.totalTopOffset(0);
	});

	self.displayedErrors = ko.computed(function () {
		var errors = self.errors();

		var startOffset = self.totalTopOffset();
		var endOffset = startOffset + self.listContainerHeight();

		var currentHeight = 0;

		var fromIndex = -1, toIndex = -1;

		for (var i = 0; i < errors.length; i++) {
			var errorHeight = errors[i].cachedHeight;
			if (fromIndex === -1) {
				if (currentHeight + errorHeight >= startOffset) {
					fromIndex = i;
					self.topOffset(currentHeight - startOffset);
				}
			}

			if (fromIndex > -1 && toIndex === -1) {
				if (currentHeight + errorHeight >= endOffset) {
					toIndex = i;
					break;
				}
			}

			currentHeight += errorHeight;
		}

		if (toIndex === -1) {
			toIndex = errors.length - 1;
		}

		return self.errors().slice(fromIndex, toIndex + 1);

	}).extend({ deferred: true });

	self.wheelScrollFactor = 0.25;

	self.scroll = function (data, event) {
		self.errorVerticalScroll.wheelScroll(data, event);

		return true;
	};
};

/**
 * Подсчитать отображаемую высоту ошибки
 * @param {} error Ошибка
 * @param {} useCachedHeight Использовать ли значение, кэшированное в самой ошибке
 * @param {} fieldsHeightCache Общий кэш подсчитанных ранее высот имён полей
 * @param {} messagesHeightCache Общий кэш подсчитанных ранее высот сообщений
 * @returns {} 
 */
FC.Viewmodels.FormEditor.ErrorsScroller.prototype.calculateErrorHeight = function (error, fieldsHeightCache, messagesHeightCache) {
	var result = 16; // padding-top + padding-bottom

	var fields = error.fields();
	var fieldText = "";

	for (var i = 0; i < fields.length; i++) {
		var field = fields[i];

		if (fieldText.length > 0) {
			fieldText += " ";
		}

		fieldText += field.name();
	}

	if (fieldsHeightCache && fieldsHeightCache.hasOwnProperty(fieldText)) {
		result += fieldsHeightCache[fieldText];
	} else {
		var fieldTest = this.fieldTest();
		if (fieldTest) {
			fieldTest.innerHTML = fieldText;
			var calculatedFieldHeight = fieldTest.getBoundingClientRect().height;
			result += calculatedFieldHeight;

			if (fieldsHeightCache) {
				fieldsHeightCache[fieldText] = calculatedFieldHeight;
			}
		}
	}

	var message = error.message();
	if (message.length > 0) {
		if (messagesHeightCache && messagesHeightCache.hasOwnProperty(message)) {
			result += messagesHeightCache[message];
		} else {
			var messageTest = this.messageTest();
			if (messageTest) {
				messageTest.innerHTML = message;
				var calculatedMessageHeight = messageTest.getBoundingClientRect().height;
				result += calculatedMessageHeight;

				if (messagesHeightCache) {
					messagesHeightCache[message] = calculatedMessageHeight;
				}
			}
		}
	}

	error.cachedHeight = result;

	return result;
}

/**
 * Прокрутиться к ошибке, расположенной по заданному индексу
 * @param {} errorIndex Индекс ошибки
 * @returns {} 
 */
FC.Viewmodels.FormEditor.ErrorsScroller.prototype.scrollToIndex = function (errorIndex) {
	var errors = this.errors();
	var errorHeight = 8;

	for (var i = 0; i < errorIndex; i++) {
		errorHeight += errors[i].cachedHeight;
	}

	var startOffset = this.totalTopOffset();
	var endOffset = startOffset + this.listContainerHeight();

	var targetHeight;
	if (errorHeight <= startOffset) {
		targetHeight = errorHeight;
	} else if (errorHeight > endOffset - errors[errorIndex].cachedHeight) {
		targetHeight = errorHeight + errors[errorIndex].cachedHeight - this.listContainerHeight();
	} else {
		return;
	}

	this.totalTopOffset(targetHeight);
};