﻿
FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};
/**
 * 
 * @param {any} field model поля
 * @param {any} parent viewmodel родителя поля
 * @param {any} selectedField viewmodel выделенного поля
 * @param {any} selectedParagraph выбранный параграф (глобальный observable)
 * @param {any} offsetsInfo {verticalOffset:number, horizontalOffset: number}} offset формы
 */
FC.Viewmodels.DocumentEditor.ChoiceFieldAdapter = function (field, parent, selectedField, selectedParagraph, offsetsInfo, changeSubscribers) {
	var self = this;
	var minimumOptionsHeight = 100;
	var windowBoundMargin = 60;
	var optionsGeneralPadding = 20;
	$.extend(self, new FC.Viewmodels.DocumentEditor.FieldAdapter(field, parent, selectedField, selectedParagraph, changeSubscribers));

	self.ElementBottomLine = ko.observable(0);
	self.optionsOpened = ko.observable(false);
	self.optionsStyle = ko.observable({});
	self.focusedOption = ko.observable(null);
	self.offset = ko.observable(0);
	
	self.popupHeight = ko.observable(0);
	var getContainerNode = function() {
		return document.getElementById('formViewer_body');
	}

	var setOptionsStyles = function () {
		var elem = document.getElementById(self.DomId);
		if (!elem) return;
		var boundingRect = elem.getBoundingClientRect();
		var maxHeight = window.innerHeight - boundingRect.bottom;
		var styles = {
			width: self.optionsStyle().width,
			left: boundingRect.left + 'px'
		};
		if (maxHeight < minimumOptionsHeight && window.innerHeight > minimumOptionsHeight) {
			styles.top = null;
			styles.bottom = window.innerHeight - boundingRect.top + 'px';
			self.popupHeight(Math.min(boundingRect.top - windowBoundMargin, self.contentHeight()));
		} else {
			styles.bottom = null;
			styles.top = boundingRect.bottom + 'px';
			self.popupHeight(Math.min(maxHeight - windowBoundMargin, self.contentHeight()));
		}
		styles.height = self.popupHeight() + 'px';
		self.optionsStyle(styles);
	}

	self.optionsOpened.subscribe(function(val) {
		if (!val) {
			self.focusedOption(null);
		} else {
			var containerElement = getContainerNode();
			var containerRect = containerElement.getBoundingClientRect();
			var elem = document.getElementById(self.DomId);
			var boundingRect = elem.getBoundingClientRect();
			if (containerRect.left > boundingRect.left) {
				containerElement.scrollLeft -= (containerRect.left - boundingRect.left);
			} else if ((containerRect.right < boundingRect.right)) {
				if ((containerRect.right - containerRect.left) > (boundingRect.right - boundingRect.left)) {
					containerElement.scrollLeft += boundingRect.right - containerRect.right;
				} else {
					containerElement.scrollLeft += boundingRect.left - containerRect.left;
				}
			}
			setOptionsStyles();
		}
	});

	
	offsetsInfo.verticalOffset.subscribe(function() {
		if (self.optionsOpened()) {

			setOptionsStyles();
			//Закрываем, если комбик ушел из зоны отрисовки
			setTimeout(function() {
					var containerElement = getContainerNode();
					var containerElemRect = containerElement.getBoundingClientRect();
					var elem = document.getElementById(self.DomId);
					if (!elem) {
						self.optionsOpened(false);
						return;
					}
					var elemRect = elem.getBoundingClientRect();
					if ((elemRect.top < containerElemRect.top) || (elemRect.bottom > containerElemRect.bottom)) {
						self.optionsOpened(false);
					}
				},
				10);

		}
	});

	offsetsInfo.horizontalOffset.subscribe(function() {
		if (self.optionsOpened()) {
			setOptionsStyles();
		}
	});

	$(window).resize(function() {
		self.optionsOpened(false);
	});
	self.minSelectedCount = self.model.minSelectedCount;
	self.maxSelectedCount = self.model.maxSelectedCount;

	self.formLoaded = ko.observable(false);
	self.contentHeight = ko.computed(function () {
		if (!self.formLoaded()) return 0;
		var $elem = $('#' + self.DomId);
		var height = 0;
		$elem.parent().find('.customform__choiceitem').each(function (key, value) {
			var internalId = $(this).attr('data-internalid');
			if (internalId == -2 || (internalId == -1 && self.minSelectedCount() > 0)) return;

			var currentOuterSize = $(this).getHiddenDimensions(false);
			height += currentOuterSize.height;
		});
		return height + optionsGeneralPadding;
	});

	self.scrollBar = new FC.ScrollBarMini(self.offset, self.popupHeight, self.contentHeight);

	self.choiceValues = ko.computed(function() {
		var res = $.map(self.model.variants(),
			function(variant) {
				return {
					_id: variant.id(),
					caption: variant.name,
					selected: ko.computed({
						read: function() {
							return self.model.values().indexOf(variant.id()) >= 0;
						},
						write: function(val) {
							if (val) {
								self.model.values.push(variant.id());
							} else {
								self.model.values.splice(self.model.values().indexOf(variant.id()), 1);
							}
						}
					})
				};
			});


		res.unshift({
			_id: -1,
			caption: ko.observable(FC.Environment.Localization.GlobalResources.EmptySelection),
			selected: ko.computed({
				read: function() {
					return self.model.values().length === 0;
				},
				write: function(val) {
					self.model.values([]);
				}
			})
		});
		

		if (self.maxSelectedCount() > 1) {
			res.unshift({
				caption: ko.observable(FC.Environment.Localization.GlobalResources.MultipleSelection),
				_id: -2,
				selected: ko.observable(false)
			});
		}

		return res;
	});

	self.selectedChoiceValues = ko.computed(function() {
		var selectedVariants = self.model.selectedVariants();
		if (!selectedVariants || selectedVariants.length === 0) {
			return FC.Environment.Localization.GlobalResources.EmptySelection;
		}
		
		if (selectedVariants.length > 1) {
			return FC.Environment.Localization.GlobalResources.MultipleSelection;
		}

		return selectedVariants[0];
	});


	/* Стиль для комбобокса */
	self.ChoiceFieldValueCss = ko.pureComputed(function () {
		return self.FormFieldValueCss();
	});
	
	self.model.values.subscribe(function () {
		self.rewritableFunctions.verify(false);
		self.model.manipulators.dbCheckRulesManipulator.tryRecount([self.model], []);
		if (self.model.values().length < self.model.minSelectedCount()) {
			self.createFormatError(FC.Environment.Localization.GlobalResources.FormatErrorLessComboBox.replace("{0}", self.model.minSelectedCount()), true);
		}
		else if (self.model.values().length > self.model.maxSelectedCount()) {
			self.createFormatError(FC.Environment.Localization.GlobalResources.FormatErrorMoreComboBox.replace("{0}", self.model.minSelectedCount()), true);
		} else {
			self.removeFormatError();
		}

		self.notifyValueChanged(self.model.values());

		if (self.model.needRecognize()) {
			self.model.needRecognize(false);
		}
	});
	
	var baseVerify = self.rewritableFunctions.verify;
	self.rewritableFunctions.verify = function (goToNext) {
		if (goToNext == undefined)
			goToNext = true;

		baseVerify(null, goToNext);

	};


	self.focusElement = function(data, event) {
		self.focusedOption(data);
	}

	self.choiceClicked = function (data, event) {
		if (event.which > 1) {
			return true;
		}

		if (self.readOnly()) {
			event.preventDefault();
			event.currentTarget.focus();
		} else {
			self.optionsOpened(!self.optionsOpened());
		}
		return true;
	}

	self.optionPressed = function (item) {
		if (self.maxSelectedCount() === 1) {
			self.optionsOpened(false);
			self.deselectAll();
			if (item._id !== -1) {
				item.selected(true);
			}
		} else {
			if (item._id == -1) {
				self.deselectAll();
			} else {
				item.selected(!item.selected());
			}
		}

		self.rewritableFunctions.verify(false);
	}

	self.afterOptionsRendered = function () {
		var maxWidth = 0;
		var $elem = $('#' + self.DomId);
		var minWidth = parseInt(window.getComputedStyle(document.getElementById(self.DomId), null).getPropertyValue("min-width")
			.replace('px', ''), 10);
		if (!isNaN(minWidth)) {
			maxWidth = minWidth;
		}
		$elem.parent().find('.customform__choiceitem').each(function(key, value) {
			var currentOuterSize = $(this).getHiddenDimensions(false);
			var currentOuterWidth = currentOuterSize.width;
			if (maxWidth < currentOuterWidth) {
				maxWidth = currentOuterWidth;
			}

		});

		self.optionsStyle({ width: maxWidth+'px', top: 0, left: 0 });

		$elem.outerWidth(maxWidth);
		self.formLoaded(true);
		self.ElementBottomLine($elem.outerHeight());
	};

	self.deselectAll = function () {
		self.model.values([]);
	}

	var baseKeyPressed = self.KeyPressed;
	self.KeyPressed = function (data, event) {
		var keyCode = event.keyCode || event.which;
		if (keyCode === 40 && !self.optionsOpened()) {
				self.optionsOpened(true);
		}

		

		if (self.optionsOpened()) {
			if (keyCode === 27) {
				self.optionsOpened(false);
				return false;
			}

			var focusedIndex;
			if (keyCode === 40) {
				focusedIndex = self.choiceValues().indexOf(self.focusedOption());
				if (focusedIndex !== self.choiceValues().length - 1) {
					self.focusedOption(self.choiceValues()[focusedIndex + 1]);
				} else {
					self.focusedOption(self.choiceValues()[self.choiceValues()[0]._id === -2?1:0]);
				}
			}

			if (keyCode === 38) {
				focusedIndex = self.choiceValues().indexOf(self.focusedOption());
				if (focusedIndex !== 0) {
					self.focusedOption(self.choiceValues()[focusedIndex - 1]);
				} else {
					self.focusedOption(self.choiceValues()[self.choiceValues().length-1]);
				}
			}
			if (keyCode === 13 && self.focusedOption()) {
				if (self.maxSelectedCount() === 1) {
					
					self.deselectAll();
					if (self.focusedOption()._id !== -1) {
						self.focusedOption().selected(true);
					}
					self.optionsOpened(false);
				} else {
					if (self.focusedOption()._id == -1) {
						self.deselectAll();
					} else {
						self.focusedOption().selected(!self.focusedOption().selected());
					}
				}

				self.rewritableFunctions.verify(false);
			}
		}


		return baseKeyPressed(data, event);
	}

	self.valueHash = ko.computed(function () {
		return self.model.values().join();
	});

	$(window).on("click.formFields", function (event) {
		if ($(event.target).hasClass('abbyy_dc_customChoice')) return true;
		if ($(event.target).hasClass('formField_errorBalloon_error-message')) return true;
		var $target = $(event.target).parents('.abbyy_dc_customChoice');
		if ($target.length ===0 || $target.children('#' + self.DomId).length === 0) {
			self.optionsOpened(false);
		}
		return true;
	});
}