﻿FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};
/**
 * 
 * @param {any} allColumns список всех отображаемых колонок
 * @param {any} hideableColumns список колонок, которые необходимо скрыть в случае отсутствия в них значения
 * @param {any} offsetsInfo {verticalOffset:number, horizontalOffset: number}} offset формы
 */
FC.Viewmodels.DocumentEditor.ColumnSelector = function (allColumns, hideableColumns, offsetsInfo) {
	var self = this;

	var popupMaxHeight = 300;
	var minBorderOffset = 60;
	self.columns = allColumns;
	self.hideableColumns = hideableColumns;
	self.offset = ko.observable(0);
	self.popupHeight = ko.observable(0);
	self.contentHeight = ko.observable(0);
	self.topPosition = ko.observable(0);
	self.leftPosition = ko.observable(0);
	self.columnSelectorId = "columnSelector" + (FC.Viewmodels.DocumentEditor.ColumnSelector.GlobalCounter++);
	self.columnsSelectorVisible = ko.observable(false);


	self.scrollBar = new FC.ScrollBarMini(self.offset, self.popupHeight, self.contentHeight);

	self.allColumns = ko.computed(function() {
		return $.map(self.columns(),
			function(item) {
				return new FC.Viewmodels.DocumentEditor.ColumnSelectorViewmodel(item.id, item.name, self.hideableColumns);
			});
	});

	self.popupStyles = ko.computed(function() {
		return {
			left: self.leftPosition() + 'px',
			top: self.topPosition() + 'px',
			height: self.popupHeight() + 'px',
		}
	});

	var getControlNode = function () {
		var res = document.getElementById(self.columnSelectorId);
		return res ? res.parentElement : null;
	}

	var getContainerNode = function () {
		return document.getElementById('formViewer_body');
	}

	var countPopupProps = function () {
		var controlNode = getControlNode();
		if (!controlNode) return;
		var contentHeight = self.contentHeight();
		
		var controlBounds = controlNode.getBoundingClientRect();
		var availableHeightUndexControl = window.innerHeight - controlBounds.bottom - minBorderOffset;
		var maxAvailableHeight = Math.max(availableHeightUndexControl,
			controlBounds.top - minBorderOffset);
		var maxPopupHeight = Math.min(popupMaxHeight, contentHeight);

		var popupHeight = Math.min(maxAvailableHeight, maxPopupHeight);
		self.popupHeight(popupHeight);

		if (availableHeightUndexControl < popupHeight) {
			self.topPosition(controlBounds.top - popupHeight);
		} else {
			self.topPosition(controlBounds.bottom);
		}

		self.leftPosition(controlBounds.left);
	}
	
	self.toggleColumnsSelector = function () {
		self.columnsSelectorVisible(!self.columnsSelectorVisible());
		var contentSize = $('#' + self.columnSelectorId +' .customform__columnsselector').getHiddenDimensions(false);
		self.contentHeight(contentSize.height);
		countPopupProps();
	}
	
	$(window).resize(function() {
		self.columnsSelectorVisible(false);
	});

	offsetsInfo.verticalOffset.subscribe(function () {
		if (self.columnsSelectorVisible()) {

			countPopupProps();
			//Закрываем, если комбик ушел из зоны отрисовки
			setTimeout(function () {
				var controlElement = getControlNode();
				if (!controlElement) {
					self.columnsSelectorVisible(false);
					return;
				}
				var containerElemRect = getContainerNode().getBoundingClientRect();
					
				var elemRect = controlElement.getBoundingClientRect();
				if ((elemRect.top < containerElemRect.top) || (elemRect.bottom > containerElemRect.bottom)) {
					self.columnsSelectorVisible(false);
				}
			},
			10);
		}
	});

	$(window).on("click.ColumnSelector", function (event) {
		var $target = $(event.target);
		if (!$target.hasClass('.customform__columnsselector') &&
			$target.parents('.customform__columnsselector').length === 0) {
			self.columnsSelectorVisible(false);
		}
		return true;
	});

}

FC.Viewmodels.DocumentEditor.ColumnSelector.GlobalCounter = 0;