﻿FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};
FC.Viewmodels.DocumentEditor.FieldAdapter =
	function(field, parent, selectedField, selectedParagraph, changeSubscribers) {
		var self = this;


		self.model = field;
		self._id = FC.Viewmodels.DocumentEditor.FieldAdapter.Common.getId();

		self.selectedField = selectedField;
		self.selectedParagraph = selectedParagraph;

		self.parent = parent;

		self.changeSubscribers = changeSubscribers;

		self.disposables = [];

		self.DomId = "docEditor_field_" + self._id;
		self.UniqueIdentifier = "field" + (FC.Viewmodels.DocumentEditor.FieldAdapter.GlobalCounter++);

		self.States = {
			None: 0,
			Verified: 1,
			NeedVerification: 2,
			HasError: 4,
			HasWarning: 8,
			HasSuggestions: 16,
			HasScript: 32,
			HasMultierror: 64
		};

		self.caption = self.model.caption;

		self.fieldType = self.model.fieldType;

		self.selectField = function() {
			if (self.model !== self.selectedField()) {
				self.selectedField(self.model);
				self.selectedParagraph(null);
			}
			return true;
		}


		self.selected = ko.computed({
			read: function() {
				return self.selectedField() == self.model;
			},
			write: function(val) {
				if (val) self.selectField();
			}
		});

		self.disposables.push(self.selected);
		self.readOnly = ko.computed(function() {
			return (field.isReadOnly && field.isReadOnly()) || (field.isDbLocked && field.isDbLocked());
		});

		self.showBalloon = ko.observable(false);
		var showBallonTimeout;

		self.hasCustomAction = ko.observable(self.model.customScript && self.model.customScript.length > 0);

		self.currentState = ko.computed(function() {
			var state = 0;

			var hasError = false;
			var hasWarning = false;

			if (self.model.fieldErrors) {
				for (var errorIndex = 0; errorIndex < self.model.fieldErrors().length; errorIndex++) {
					var error = self.model.fieldErrors()[errorIndex];
					if (!error.severity || error.severity()) {
						hasError = true;
					} else {
						hasWarning = true;
					}
				}
			}


			if (hasError) {
				state += self.States.HasError;
			}

			if (hasWarning) {
				state += self.States.HasWarning;
			}


			if (self.CustomScript) {
				state += self.States.HasScript;
			}


			if (self.model.isVerified && self.model.isVerified()) {
				state += self.States.Verified;
			}
			if (self.model.needVerification && self.model.needVerification()) {
				state += self.States.NeedVerification;
			}

			return state;
		});

		self.disposables.push(self.currentState);

		/* Стиль для формы данных - отражает состояния поля */
		self.FormFieldValueCss = ko.pureComputed(function() {

			// привязан к значению CurrentState
			var state = self.currentState();

			return {
				"field-disabled": self.readOnly(),
				// поле выбрано
				"field_value-selected": self.selected(),

				// поле нуждается в верификации (и не верифицировано)
				"field_value-needVerify": ((state & self.States.NeedVerification) === self.States.NeedVerification) &&
					((state & self.States.Verified) !== self.States.Verified),

				// поле верифицировано
				"field_value-verified": (state & self.States.Verified) === self.States.Verified,

				// поле содержит ошибку
				"hasError field_value-error": ((state & self.States.HasError) === self.States.HasError),

				// поле содержит предупреждение
				"hasError field_value-warning": (state & self.States.HasWarning) === self.States.HasWarning
			};
		});

		self.createFormatError = function(errorMessage, severity) {
			var formatError = self.model.fieldErrors().filter(function(val) {
				return (val instanceof FC.BusinessModels.DocumentEditor.Error) && val.type() === 1;
			})[0];
			if (!formatError || formatError.length === 0) {
				self.model.addError(severity, FC.BusinessModels.DocumentEditor.ErrorType.FormatError, errorMessage);
			} else if (formatError.message() !== errorMessage) {
				formatError.message(errorMessage);
			}
		};

		self.removeFormatError = function() {
			var formatError = self.model.fieldErrors().filter(function(val) {
				return (val instanceof FC.BusinessModels.DocumentEditor.Error) && val.type() === 1;
			})[0];
			if (formatError) {
				self.model.removeError(formatError);
			}
		};

		self.fieldErrors = ko.computed(function() {
			if (!self.model.fieldErrors) {
				return [];
			}
			return $.map(self.model.fieldErrors(),
				function(val) {
					if (val instanceof FC.BusinessModels.DocumentEditor.AssemblingError) {
						return val.description();
					} else if (val instanceof FC.BusinessModels.DocumentEditor.Error) {
						return val.message();
					}

				});
		});

		self.disposables.push(self.fieldErrors);

		/* В поле нажата клавиша */
		self.KeyPressed = function() {
			// нужно, чтобы сработали дефолтные действия на нажатие клавиши
			return true;
		};

		self.rewritableFunctions = {
			verify: function(intermediateLevelCheck, goToNext) {
				if (goToNext == undefined)
					goToNext = true;

				FC.Viewmodels.EventStack.addEvent();

				self.model.isVerified(true);

				Sammy.apps.body.trigger("fieldUncertaintyChanged", { field: self.model });

				if (intermediateLevelCheck)
					intermediateLevelCheck();


				if (goToNext) {

					// нужна задержка, чтобы фокусировка ставилась после выполнения функции intermediateLevelCheck
					setTimeout(function() { FC.Viewmodels.FormNavigation.navigateToNextField(1, self.DomId); }, 1);
				}
			},
			selectField: function(isSelected) {
				if (isSelected && window.batchLayout.batchLayoutViewModel.rightFrameVisible()) {
					var $elem = $("#" + self.DomId);
					if ($elem.is(':hidden')) {
						$elem.show();
						$elem.parent().children('.customForm__summary').remove();
						var collapser = FC.Utils.DocumentEditor.findConnectedElementsToField($elem);
						if (collapser && collapser.length > 0) {

							collapser[0].children().forEach(function(elem) {
								FC.Utils.DomClassWrapper.removeClass(elem, 'groupExpander-down');
							});
						}
						$elem.parents().show();
						$elem.parents().each(function() {
							$(this).parent().children('.customForm__summary').remove();
							var collapser = FC.Utils.DocumentEditor.findConnectedElementsToField($(this));
							if (collapser && collapser.length > 0) {
								collapser[0].children().forEach(function(elem) {
									FC.Utils.DomClassWrapper.removeClass(elem, 'groupExpander-down');
								});
							}
						});
					}

					if ($elem.length > 0 && !$elem.is(':focus')) {
						var viewModel = ko.dataFor($elem[0]);
						viewModel.selectField();
						$elem.focus();
					}
				} else {
					$("#" + self.DomId).blur();
				}
			}
		};

		self.mouseEnterEvent = function() {
			showBallonTimeout = setTimeout(function() {
					self.showBalloon(true);
				},
				1000);
		}

		self.mouseLeaveEvent = function() {
			clearTimeout(showBallonTimeout);
			self.showBalloon(false);
		}

		self.hideBalloon = function() {
			self.showBalloon(false);
		}


		self.runDbLookup = function(data, event) {
			var parent = event.target.parentElement;
			if (event.target.dataset.islookup === 'True') {
				parent = event.target;
			}
			var ruleName = parent.dataset.rulename;
			var ruleNode = parent.dataset.rulenode;

			if (ruleName && ruleNode) {
				Sammy.apps.body.trigger('docEditor_runDbLookup',
					{
						nodeRef: ruleNode,
						ruleName: ruleName,
						repeatableContext: self.model.getRepeatableContext()
					});
			} else {
				self.runCustomScript();
			}

		};

		self.runCustomScript = function() {
			Sammy.apps.body.trigger('docEditor_runCustomScript',
				{ target: self.model, coordinates: document.getElementById(self.DomId).getBoundingClientRect(), id: self.DomId });
		};

		self.disposables.push(self.selected.subscribe(function(val) {
			self.rewritableFunctions.selectField(val);

		}));

		self.dispose = function() {
			while (self.disposables.length > 0) {
				var obj = self.disposables.pop();
				//if (obj && obj.hasOwnProperty("dispose")) {
				obj.dispose();
				//}
			}
		};


		self.pictureRequired = ko.observable(false);
		self.pictureSrc = ko.observable('');


		self.notifyValueChanged = function (newValue, notificationType) {
			var valueChangesSubscribers = self.changeSubscribers.valueChangeSubscribers;
			if (notificationType === FC.Viewmodels.DocumentEditor.FieldAdapter.NotificationTypes.RowSelectedChange) {
				valueChangesSubscribers = self.changeSubscribers.rowSelectSubscribers;
			} else if (notificationType === FC.Viewmodels.DocumentEditor.FieldAdapter.NotificationTypes.InstanceChanged) {
				valueChangesSubscribers = self.changeSubscribers.changeInstancesSubscribers;
			}
			valueChangesSubscribers = valueChangesSubscribers || [];
			
			if (!self.model.getFieldNamePathWithIndex || valueChangesSubscribers.length === 0) return;
			var fieldPath = self.model.getFieldNamePathWithIndex();
			valueChangesSubscribers.forEach(function(valueChangeSubscriber) {
				valueChangeSubscriber(fieldPath, newValue);
			});
		};

		ko.computed(function() {
			if (!self.pictureRequired()) {
				self.pictureSrc('');
				return;
			}

			var regionFieldPair = self.model.manipulators.regionsManipulator.fieldRegionMap()
				.filter(function(val) {
					return val.field == self.model &&
						val.region.status() &&
						val.region.coordinates() &&
						val.region.coordinates().length > 0;
				})[0];


			if (!regionFieldPair) {
				self.pictureSrc('');
				return;
			}

			var region = regionFieldPair.region;

			if (!region.regionPage()) {
				self.pictureSrc('');
				return;
			}


			// получим охватывающий прямоугольник 
			// и отобразим его
			var coord = region.getExternalRect();
			var width = coord.x2 - coord.x1;
			var height = coord.y2 - coord.y1;

			var url = region.regionPage().url();

			var setImage = function(imgObj) {
				self.pictureSrc(FC.Utils.PictureUtils.getImagePortion(imgObj,
					coord.x1,
					coord.y1,
					width,
					height,
					window.innerHeight / imgObj.naturalHeight));
			}

			if (window.lastLoadedImageHistory && window.lastLoadedImageHistory[url]) {
				setImage(window.lastLoadedImageHistory[url]);
			} else {
				var loadTimer;
				var imgObject = new Image();
				imgObject.src = url;

				imgObject.onLoad = onImgLoaded();

				function onImgLoaded() {
					if (loadTimer != null) clearTimeout(loadTimer);
					if (!imgObject.complete) {
						loadTimer = setTimeout(function() {
								onImgLoaded();
							},
							3);
					} else {
						window.lastLoadedImageHistory = window.lastLoadedImageHistory || {};
						window.lastLoadedImageHistory[url] = imgObject;
						setImage(imgObject);
					}
				}
			}

		});
	};

FC.Viewmodels.DocumentEditor.FieldAdapter.Common = new FC.Viewmodels.SelectionStack();

FC.Viewmodels.DocumentEditor.FieldAdapter.GlobalCounter = 0;

FC.Viewmodels.DocumentEditor.FieldAdapter.NotificationTypes = {
	ValueChange: 1,
	RowSelectedChange: 2,
	InstanceChanged: 3
};