﻿FC.Viewmodels.FormEditor = FC.Viewmodels.FormEditor || {};


FC.Viewmodels.FormEditor.FieldViewmodelAlgo = {
	generateField: function (field, parent, selectedField, selectedParagraph, offsetInfo, changeSubscribers) {
		if (!field.isVisible()) {
			return null;
		}
		if (field instanceof FC.BusinessModels.DocumentEditor.BooleanField) {
			return new FC.Viewmodels.DocumentEditor.BooleanFieldAdapter(field, parent, selectedField, selectedParagraph, changeSubscribers);
		}

		if (field instanceof FC.BusinessModels.DocumentEditor.ChoiceField) {
			return new FC.Viewmodels.DocumentEditor.ChoiceFieldAdapter(field, parent, selectedField, selectedParagraph, offsetInfo, changeSubscribers);
		}

		if (field instanceof FC.BusinessModels.DocumentEditor.TextField) {
			return new FC.Viewmodels.DocumentEditor.TextFieldAdapter(field, parent, selectedField, selectedParagraph, offsetInfo, changeSubscribers);
		}

		if (field instanceof FC.BusinessModels.DocumentEditor.PictureField) {
			return new FC.Viewmodels.DocumentEditor.PictureFieldAdapter(field, parent, selectedField, selectedParagraph, offsetInfo);
		}

		if (field instanceof FC.BusinessModels.DocumentEditor.FieldsGroup) {
			if (field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent) {
				return new FC.Viewmodels.DocumentEditor.RepeatableAdapter(field, parent, selectedField, selectedParagraph, offsetInfo, changeSubscribers);
			} else {
				return new FC.Viewmodels.DocumentEditor.FieldsGroupAdapter(field, parent, selectedField, selectedParagraph, offsetInfo, changeSubscribers);
			}
		}

		if (field instanceof FC.BusinessModels.DocumentEditor.Table)
		{
			return new FC.Viewmodels.DocumentEditor.TableAdapter(field, parent, selectedField, selectedParagraph, offsetInfo, changeSubscribers);
		}


		if (field instanceof FC.BusinessModels.DocumentEditor.Section && field.isVisible()) {
			return new FC.Viewmodels.DocumentEditor.SectionAdapter(field, parent, selectedField, selectedParagraph, offsetInfo, changeSubscribers);
		}

		if (field instanceof FC.BusinessModels.DocumentEditor.SectionsGroup) {
			return new FC.Viewmodels.DocumentEditor.RepeatableAdapter(field, null, selectedField, selectedParagraph, offsetInfo, changeSubscribers);
		}

		return null;
	},
	showField: function (field) {
		var $elem = $("#" + field.DomId);
		if ($elem.is(':hidden')) {
			$elem.show();
			$elem.parent().children('.customForm__summary').remove();
			var collapser = FC.Utils.DocumentEditor.findConnectedElementsToField($elem);
			if (collapser && collapser.length > 0) {
				collapser[0].children().forEach(function (elem) { FC.Utils.DomClassWrapper.removeClass(elem, 'groupExpander-down'); });
			}
			$elem.parents().show();
			$elem.parents().each(function () {
				$(this).parent().children('.customForm__summary').remove();
				var collapser = FC.Utils.DocumentEditor.findConnectedElementsToField($(this));
				if (collapser && collapser.length > 0) {
					collapser[0].children().forEach(function (elem) { FC.Utils.DomClassWrapper.removeClass(elem, 'groupExpander-down'); });
				}
			});
		}
	},
	moveToField: function(field, showIfHidden) {
		var $item = $('[data-jsDomId="' + field.UniqueIdentifier + '"]');
		
		if (!$item) {
			return false;
		}

		if (showIfHidden) {
			this.showField(field);
		}
		else if ($item.is(':hidden')) {
			return false;
		}

		var $container = $('#formViewer_body');

		var containerHeight = $container.outerHeight();
		var containerWidth = $container.outerWidth();

		var containerScroll = $container.scrollTop();
		var containerScrollLeft = $container.scrollLeft();

		var itemHeight = $item.outerHeight();
		var itemWidth = $item.outerWidth();

		var itemTop = ($item.offset().top - $container.offset().top);
		var itemBottom = itemTop + itemHeight + 20;

		var itemLeft = ($item.offset().left - $container.offset().left);
		var itemRight = (itemLeft + itemWidth + 20);
		
		if (itemTop < 0) {
			$container.scrollTop(containerScroll + itemTop);
		} else if (itemTop > containerHeight && itemTop > 0) {
			if (itemHeight > containerHeight) {
				$container.scrollTop(containerScroll + itemTop);
			} else {
				$container.scrollTop(containerScroll + (itemBottom - containerHeight));
			}
		} else if(itemBottom>containerHeight && itemHeight<containerHeight){
			$container.scrollTop(containerScroll + (itemBottom - containerHeight));
		}
		if (itemLeft < 0) {
			$container.scrollLeft(containerScrollLeft + itemLeft);
		} else if (itemLeft > containerWidth && itemLeft > 0) {
			if (itemWidth > containerWidth) {
				$container.scrollLeft(containerScrollLeft + itemLeft);
			} else {
				$container.scrollLeft(containerScrollLeft + (itemRight - containerWidth));
			}
		} else if (itemRight > containerWidth && itemWidth < containerWidth) {
			$container.scrollLeft(containerScrollLeft + (itemRight - containerWidth));
		}

		return true;
	},
	getViewmodelSubfields: function(analyzedField) {
		return analyzedField.children && analyzedField.children()
			? analyzedField.children()
			: ((analyzedField.rows && analyzedField.rows() ? analyzedField.rows() : (analyzedField.cells && analyzedField.cells() ? analyzedField.cells() : [])));
	},
	findSimpleFields: function (analyzedField) {
		if (!analyzedField) return [];
		if (analyzedField.model instanceof FC.BusinessModels.DocumentEditor.SimpleField) {
			return [analyzedField];
		}
		var res = [];
		var elementsToAnalyze = this.getViewmodelSubfields(analyzedField);
		for (var i = 0; i < elementsToAnalyze.length; i++) {
			var anField = elementsToAnalyze[i];
			if (anField) {
				res = res.concat(this.findSimpleFields(anField));
			}
		}
			
		return res;
	},
	selectFirstSimpleElement: function (analyzedField) {
		var simpleFields = this.findSimpleFields(analyzedField);
		if (simpleFields.length === 0) return;

		if (simpleFields.length === 1) {
			window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel.selectedField(null);
			simpleFields[0].selected(true);
			return;
		}

		var fieldsWithCoord = $.map(simpleFields,
			function(simpleField) {
				var $elem = $("#" + simpleField.DomId);
				return {
					coord: $elem.offset(),
					field: simpleField
				};
			}).filter(function(fieldWithCoord) {
			return fieldWithCoord.coord;
		});
		var firstField = fieldsWithCoord[0];
		for (var i = 1; i < fieldsWithCoord.length; i++) {
			var currentFieldWithCoord = fieldsWithCoord[i];
			if (currentFieldWithCoord.coord.top - firstField.coord.top < -5 ||
			(Math.abs(currentFieldWithCoord.coord.top - firstField.coord.top) < 5 &&
				currentFieldWithCoord.coord.left - firstField.coord.left < -5)) {
				firstField = currentFieldWithCoord;
			}
		}
		window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel.selectedField(null);
		firstField.field.selected(true);
	},
	fieldCaptionClickEvent: function(field, captionNode) {
		if (!(field.model instanceof FC.BusinessModels.DocumentEditor.SimpleField)) {
			var expander = captionNode.children('.groupExpanderContainer').children('.groupExpander');
			if (expander.length > 0 && FC.Utils.DomClassWrapper.hasClass(expander[0], 'groupExpander-down')) {
				expander.click();
				return;
			}
		}
		this.selectFirstSimpleElement(field);
	},
	getAllChildren: function(field) {
		var getFieldsWithChildren = function(analyzedField) {
			var res = [];
			res.push(analyzedField);
			var analyzeSubfields = function(subfieldFunc) {
				if (analyzedField.hasOwnProperty(subfieldFunc)) {
					analyzedField[subfieldFunc]().forEach(function(analyzedSubField) {
						res = res.concat(getFieldsWithChildren(analyzedSubField));
					});
				}
			}
			analyzeSubfields('children');
			analyzeSubfields('rows');
			analyzeSubfields('cells');

			return res;
		}

		return getFieldsWithChildren(field);
	}

}