﻿FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

FC.Viewmodels.DocumentEditor.FieldsGroupAdapter = function (field, parent, selectedField, selectedParagraph, offsetInfo, changeSubscribers) {
	var self = this;


	$.extend(self, new FC.Viewmodels.DocumentEditor.FieldAdapter(field, parent, selectedField, selectedParagraph, changeSubscribers));
	FC.Viewmodels.DocumentEditor.FieldWithChildren(self, field, true, selectedField, selectedParagraph, offsetInfo, changeSubscribers);

	self.summary = function () {
		var childrenFields = self.children();
		if (!childrenFields || childrenFields.length == 0) {
			return '';
		}
		var summaryItems = [];
		$.each(childrenFields,
			function (ind, child) {
				if (!child) return;

				if (child instanceof FC.Viewmodels.DocumentEditor.RepeatableAdapter) {
					var fields = child.model.fields();
					$.each(fields,
						function (index, field) {
							if (!field.isSummary || !field.isSummary()) return;

							if (field instanceof FC.BusinessModels.DocumentEditor.TextField) {
								var val = field.value();
								if (!val || val.length === 0) {
									summaryItems.push(FC.Environment.Localization.GlobalResources.None);
								} else {
									summaryItems.push(val);
								}
							}
							if (field instanceof FC.BusinessModels.DocumentEditor.ChoiceField) {
								var choiced = field.selectedVariants();
								if (choiced.length !== 0) {
									summaryItems.push(choiced.join('; '));
								}
							}
							if (field instanceof FC.BusinessModels.DocumentEditor.BooleanField) {
								if (field.value()) {
									summaryItems.push(FC.Environment.Localization.GlobalResources.Yes);
								} else {
									summaryItems.push(FC.Environment.Localization.GlobalResources.No);
								}
							}
						});
				}

				if (!child.model.isSummary || !child.model.isSummary()) return;

				if (child instanceof FC.Viewmodels.DocumentEditor.TextFieldAdapter) {
					var val = child.model.value();
					if (!val || val.length === 0) {
						summaryItems.push(FC.Environment.Localization.GlobalResources.None);
					} else {
						summaryItems.push(val);
					}
				}
				if (child instanceof FC.Viewmodels.DocumentEditor.ChoiceFieldAdapter) {
					var choiced = child.model.selectedVariants();
					if (choiced.length !== 0) {
						summaryItems.push(choiced.join('; '));
					}
				}
				if (child instanceof FC.Viewmodels.DocumentEditor.BooleanFieldAdapter) {
					var booleanValue = child.value();
					if (booleanValue) {
						summaryItems.push(FC.Environment.Localization.GlobalResources.Yes);
					} else {
						summaryItems.push(FC.Environment.Localization.GlobalResources.No);
					}
				}



			});

		return summaryItems.join(', ');
	};

	self.childrenToShow = ko.computed(function () {
		if (self.parent && self.parent.hasOwnProperty("hideableColumns")) {
			var res = [];

			var columnsOrder = self.parent.columnsOrder();
			if (!columnsOrder || columnsOrder.length === 0) {
				columnsOrder = [];
				$.each(self.model.fields(),
					function (ind, f) {
						columnsOrder.push(f.id());
					});
			}
			var hideableEmptyColumns = self.parent.hideableEmptyColumns();
			$.each(columnsOrder,
				function (ind, currentColumn) {
					if (hideableEmptyColumns.indexOf(currentColumn) >= 0) return;
					for (var i = 0; i < self.children().length; i++) {
						var cell = self.children()[i];
						if (cell && cell.model.id() === currentColumn) {
							res.push(cell);
							return;
						}
					}
				});
			return res;
		}

		return self.children().filter(function(val) { return val; });
	});

}