﻿FC.Viewmodels.FormEditor = FC.Viewmodels.FormEditor || {};

FC.Viewmodels.FormEditor.FieldsPanel = function (currentDocument, containerHeight, editorWidth, dependencies, formCustomScriptActions) {
	var self = this;

	self.fieldsPanelCustomScript = null;

	//Для совместимости с адаптерами
	self.horizontalSeparator = containerHeight;
	self.selectedDocument = currentDocument;
	self.customFormTemplate = ko.observable(null);
	self.customFormsInfo = {};

	self.selectedField = ko.observable();
	self.selectedParagraph = ko.observable(null);

	self.scrollableElement = ko.observable(null);
	self.containerElement = ko.observable(null);
	
	self.editorWidth = editorWidth;

	self.selectingInstances = ko.observable(false);

	self.hideables = ko.observableArray([]);

	self.formHorizontalScroll = new FC.Viewmodels.FormEditor.FormHorizontalScroll(self.containerElement, self.scrollableElement);
	self.formVerticalScroll = new FC.Viewmodels.FormEditor.FormVerticalScroll(self.containerElement, containerHeight, self.scrollableElement, self.hideables, self.formHorizontalScroll.isVisible);

	var changeSubscriptions = {
		valueChangeSubscribers: [],
		rowSelectSubscribers: [],
		changeInstancesSubscribers:[]
	};

	FC.Viewmodels.DocumentEditor.FieldWithChildren(self, self.selectedDocument, true, self.selectedField, self.selectedParagraph,
		{
			horizontalOffset: self.formHorizontalScroll.value,
			verticalOffset: self.formVerticalScroll.value
		}, changeSubscriptions);

	self.selectedDocument.subscribe(function (val) {
		if (changeSubscriptions) {
			if (changeSubscriptions.valueChangeSubscribers)
				changeSubscriptions.valueChangeSubscribers.splice(0, changeSubscriptions.valueChangeSubscribers.length);
			if (changeSubscriptions.rowSelectSubscribers)
				changeSubscriptions.rowSelectSubscribers.splice(0, changeSubscriptions.rowSelectSubscribers.length);
			if (changeSubscriptions.changeInstancesSubscribers)
				changeSubscriptions.changeInstancesSubscribers.splice(0, changeSubscriptions.changeInstancesSubscribers.length);
		}
		if (!val) {
			self.customFormTemplate(null);
			return;
		}
		self.selectedField(null);
		var customFormKey = FC.Utils.DocumentEditor.GetCustomFormKey(val);

		var activateCustomForm = function (customFormDescription) {
			FC.Environment.Templates[customFormKey] = { text: customFormDescription.Html, data: {}, css: customFormDescription.Css };
			$('head #customFormStyle').remove();
			$('head').append($('<style id="customFormStyle" type="text/css">' +
				customFormDescription.Css +
				'</style>'));
			self.customFormTemplate(customFormKey);
		}
		if (self.customFormsInfo[customFormKey]) {
			activateCustomForm(self.customFormsInfo[customFormKey]);
		} else {
			dependencies.templateDataContext.getCustomForm(val.projectId(),
				val.templateId(),
				val.templateVersion(),
				function (customFormDescription) {
					if (self.customFormsInfo[customFormKey]) {
						return;
					}
					self.customFormsInfo[customFormKey] = customFormDescription;
					var selDoc = self.selectedDocument();
					if (selDoc && FC.Utils.DocumentEditor.GetCustomFormKey(selDoc) === customFormKey) {
						activateCustomForm(customFormDescription);
					}
				});
		}


	});

	var allFields = ko.computed(function () {

		var children = self.children();
		if (!children || children.length === 0) {
			return [];
		}

		var getChildrenFields = function (field) {
			var res = [];
			if (!(field instanceof FC.Viewmodels.FormEditor.FieldsPanel)) {
				res.push(field);
			}
			if (field.children && field.children()) {
				$.each(field.children(),
					function (key, subField) {
						if (!subField) return;
						res = res.concat(getChildrenFields(subField));
					});
			}
			if (field.rows && field.rows()) {
				$.each(field.rows(),
					function (key, row) {
						res.push(row);
						if (row.cells()) {
							res = res.concat(row.cells());
						}
					});
			}
			return res;
		};

		var res = getChildrenFields(self);
		return res;
	});

	self.findField = function ($form) {
		var uniqueId = $form.attr('data-jsDomId');
		if (!uniqueId && $form.attr('data-repeatablefieldid')) {
			$form = $form.children("input[type=hidden]");
			if ($form.length > 0) {
				uniqueId = $form.attr('data-jsDomId');
			}
		}

		if (!uniqueId || uniqueId.length === 0) {
			return null;
		}

		for (var i = 0; i < allFields().length; i++) {
			var field = allFields()[i];
			if (field.UniqueIdentifier === uniqueId) {
				return field;
			}
		}
		return null;
	}

	self.hasMultipleSections = ko.computed(function () {
		if (!self.children()) return false;
		var sections = self.children().filter(function (val) { return val; });
		return sections.length > 1;
	});

	self.onGroupExpanderClicked = function () {
		var expander = $(this);
		var expanderContainer = expander.parent();
		
		
		var hideElements = FC.Utils.DomClassWrapper.toggleClass(this, 'groupExpander-down');
		var elements = FC.Utils.DocumentEditor.findConnectedElementsToLabel(expanderContainer);
		if (!elements || elements.length === 0) {
			return;
		}
		var connectedForm = elements[0];
		var description = self.findField(connectedForm);
		if (description) {
			description.collapseState = hideElements ? 0 : 1;
		}
		var heightsDiff = 0;
		if (hideElements) {
			var summary = description && description.summary ? description.summary() : null;
			heightsDiff -= connectedForm.outerHeight();
			if (summary && summary.length > 0) {
				connectedForm.children().hide();
				var $summary = $('<span class="customForm__summary">' + summary + '</span>');
				$summary.click(function () {
					expander.click();
				});
				connectedForm.append($summary);
				heightsDiff += connectedForm.outerHeight();
			} else {
				connectedForm.hide();
			}
		} else {
			connectedForm.children('.customForm__summary').remove();
			connectedForm.children().show();
			connectedForm.show();
			heightsDiff += connectedForm.outerHeight();
		}

		if (hideElements && (description.hasOwnProperty("children") || description.hasOwnProperty("rows"))) {
			(description.children && description.children() || description.rows && description.rows() || []).forEach(function (c) {
				if (c.hasOwnProperty("elementHeight")) {
					c.elementHeight = 0;
				}
			});

			if (description.hideable) {
				description.hideable.heights({
					invisibleHeightLower: 0,
					invisibleHeightUpper: 0,
					visibleHeight: 0
				});
			}
		}

		var descrParent = description.parent;
		while (descrParent) {
			if (descrParent.hasOwnProperty('elementHeight') && descrParent.elementHeight > 0) {
				descrParent.elementHeight = Math.max(descrParent.elementHeight + heightsDiff, 0);
			}
			descrParent = descrParent.parent;
		}

		self.refreshHideableOffsets();
		self.refreshScrolls();

		return false;
	};

	self.onSectionExpanderClicked = function () {
		var sectionExpander = $(this);
		sectionExpander.toggleClass('docdetails--triangle-up');
		sectionExpander.toggleClass('docdetails--triangle-down');
		var sectionId = sectionExpander.parent().attr('data-sectionindex');

		var connectedSection;
		if (sectionId.indexOf('-') > 0) {
			var sectionIdParsed = sectionId.split('-');
			connectedSection = self.children()[parseInt(sectionIdParsed[0])].children()[parseInt(sectionIdParsed[1])];
		} else {
			connectedSection = self.children()[parseInt(sectionId)];
		}

		var hideElements = sectionExpander.hasClass('docdetails--triangle-down');
		var nextElem = sectionExpander.parent().next();
		if (hideElements) {
			nextElem.addClass("customform__SubForm_hidden");
			var summary = connectedSection ? (connectedSection.summary ? connectedSection.summary() : null) : null;
			if (summary && summary.length > 0) {

				var $summary = $('<span class="customForm__sectionSummary">' + summary + '</span>');
				$summary.click(function () {
					sectionExpander.click();
				});
				$summary.insertAfter(sectionExpander.parent());
			}

		} else {
			if (nextElem.hasClass('customForm__sectionSummary')) {
				nextElem.remove();
			}
			sectionExpander.parent().next().removeClass("customform__SubForm_hidden");
		}

		self.refreshHideableOffsets();
		self.containerElement.valueHasMutated();
		self.formVerticalScroll.setScrollTop(self.formVerticalScroll.scrollTop());
		return false;
	};

	self.createGroupExpanders = function (parentJqueryElement) {
		var collapserElem =
			$('<span class="groupExpanderContainer"><svg class="groupExpander"><use xlink:href="#icon--arrow-down-202"></use></svg></span>');

		var $elems = parentJqueryElement ? parentJqueryElement.find('*[data-fieldType="statictext"]') : $('.documentEditor_formViewer_content *[data-fieldType="statictext"]');

		$elems.each(function () {
			var staticText = $(this);

			if (staticText.children().first().hasClass("groupExpanderContainer") || staticText.attr("data-binded")) {
				return;
			}

			staticText.attr("data-binded", "true");

			var connectedElements = FC.Utils.DocumentEditor.findConnectedElementsToLabel(staticText);
			if (!connectedElements || connectedElements.length === 0) {
				var ownerId = staticText.attr('data-ownerId');
				if (ownerId && ownerId.length > 0) {
					ownerId = parseInt(ownerId);
					if (ownerId === 0) return;

					var basicElements = FC.Utils.DocumentEditor.findBasicElements(staticText);

					if (basicElements.length === 0) {
						staticText.remove();
					}

					for (var i = 0; i < basicElements.length; i++) {
						var field = self.findField(basicElements[i]);
						if (field) {
							var staticTetField = field.model.fields().filter(function (val) { return val && val.id() === ownerId })[0];
							if (staticTetField && !staticTetField.isVisible()) {
								staticText.remove();
							} else {
								staticText.click(function () {
									FC.Viewmodels.FormEditor.FieldViewmodelAlgo.fieldCaptionClickEvent(staticTetField, $(this));
								});
							}
						}
					}
				}
				return;
			}

			var connectedForm = connectedElements[0];
			var description = self.findField(connectedForm);
			if (description) {
				staticText.click(function () {
					FC.Viewmodels.FormEditor.FieldViewmodelAlgo.fieldCaptionClickEvent(description, $(this));
				});
			}

			var expanded = !description || typeof (description.collapseState) === "undefined" || description.collapseState === 1;
			var collapseMode = connectedForm.attr('data-collapsemode');
			if (expanded && (!collapseMode || collapseMode.length === 0 || collapseMode == 4)) {
				return;
			}

			var groupExpander = collapserElem.clone();
			groupExpander.attr('data-ownerId', staticText.attr('data-ownerId'));
			staticText.prepend(groupExpander);

			groupExpander = groupExpander.children('.groupExpander');


			if (!description) {
				return;
			}

			var hideElement = !expanded;
			if (collapseMode == 2) {
				hideElement = true;
			}
			var analyzeField;
			if (collapseMode == 1) {
				analyzeField = function (field) {
					if (!field) {
						return false;
					}

					if (field.fieldErrors().length > 0) {
						return true;
					}

					var childrenFields = field.children && field.children()
						? field.children()
						: (field.rows && field.rows()
							? field.rows()
							: (field.cells && field.cells()
								? field.cells()
								: null));

					if (childrenFields) {
						for (var i = 0; i < childrenFields.length; i++) {
							var fieldCheckResult = analyzeField(childrenFields[i]);
							if (fieldCheckResult) return true;
						}
					}

					return false;
				};
				hideElement = !analyzeField(description);
			}

			if (collapseMode == 3) {
				analyzeField = function (field) {

					if (!field) {
						return false;
					}

					if (field instanceof FC.Viewmodels.DocumentEditor.TextFieldAdapter &&
						field.model.value() &&
						field.model.value().length > 0) {
						return true;
					}

					var childrenFields = field.children && field.children()
						? field.children()
						: (field.rows && field.rows()
							? field.rows()
							: (field.cells && field.cells()
								? field.cells()
								: null));

					if (childrenFields) {
						for (var i = 0; i < childrenFields.length; i++) {
							var fieldCheckResult = analyzeField(childrenFields[i]);
							if (fieldCheckResult) return true;
						}
					}

					return false;
				};
				hideElement = !analyzeField(description);
			}
			if (hideElement) {
				FC.Utils.DomClassWrapper.toggleClass(groupExpander[0], 'groupExpander-down');
				var summary = description.summary ? description.summary() : null;
				if (summary && summary.length > 0) {
					connectedForm.children().hide();
					var $summary = $('<span class="customForm__summary">' + summary + '</span>');
					$summary.click(function () {
						groupExpander.click();
					});
					connectedForm.append($summary);
				} else {
					connectedForm.hide();
				}
			}

		});
	};

	self.afterFormRendered = function() {
		var afterRenderStarted = performance.now();
		$('.sectionExpander').click(self.onSectionExpanderClicked);

		$('.customForm__sectionNameRow').click(function() {
			var sectionExpander = $(this).children('.sectionExpander');
			if (sectionExpander.hasClass('docdetails--triangle-down')) {
				sectionExpander.click();
			} else {

				var sectionId = $(this).attr('data-sectionindex');
				var connectedSection;
				if (sectionId.indexOf('-') > 0) {
					var sectionIdParsed = sectionId.split('-');
					connectedSection = self.children()[parseInt(sectionIdParsed[0])].children()[parseInt(sectionIdParsed[1])];
				} else {
					connectedSection = self.children()[parseInt(sectionId)];
				}
				FC.Viewmodels.FormEditor.FieldViewmodelAlgo.selectFirstSimpleElement(connectedSection);
			}
		});

		self.createGroupExpanders(null);
		$('.groupExpander').click(self.onGroupExpanderClicked);

		self.containerElement.valueHasMutated();
		setTimeout(function() { self.formVerticalScroll.setScrollTop(0); }, 1);


		FC.Utils.EventLogger.Default.addMessage("FPANEL",
			"Fields panel afterload\t" + (performance.now() - afterRenderStarted) + "\tms");
		if (!window.loadingLime.loaded) {
			FC.Utils.EventLogger.Default.addMessage("GENERAL",
				"Page Loaded\t" + (performance.now() - window.loadingLime.loadStarted) + "\tms");
			window.loadingLime.loaded = true;
		}
	};

	self.validateAllFields = function() {
		allFields().filter(function(vm) {
			return vm instanceof FC.Viewmodels.DocumentEditor.TextFieldAdapter;
		}).forEach(function(f) {
			f.leaveField();
		});
	};

	self.tryCloseFECustomScript = function() {
		if (self.fieldsPanelCustomScript) {
			self.fieldsPanelCustomScript.executeAsync('forceClose');
		}
	};

	dependencies.eventManager.bind('docEditor_runCustomScript',
		function (e, data) {
			var customScript = data.target.customScript;
			var coordinates = data.coordinates;
			var id = data.id;
			if (!customScript || customScript.length === 0) {
				return;
			}

			var inlineCustomScriptFunctions = {
				selectRow: function (tablePath, rowNumber) {
					var table = Array.isArray(tablePath) ? getFieldByName(tablePath) : tablePath;
					communicator.executeAsync('selectRow',
						{
							internalTableId: table.internalId,
							rowNumber: rowNumber
						});
				},
				deselectRow: function (tablePath, rowNumber) {
					var table = Array.isArray(tablePath) ? getFieldByName(tablePath) : tablePath;
					communicator.executeAsync('deselectRow',
						{
							internalTableId: table.internalId,
							rowNumber: rowNumber
						});
				},
				scrollFormToField: function(namesPath) {
					var field = Array.isArray(namesPath) ? getFieldByName(namesPath) : namesPath;
					communicator.executeAsync('scrollFormToField',
						{
							internalId: field.internalId
						});
				},
				resetTableCustomActions: function(tablePath) {
					var field = Array.isArray(tablePath) ? getFieldByName(tablePath) : tablePath;
					communicator.executeAsync('resetTableCustomActions',
						{
							internalTableId: field.internalId
						});
				},
				setTableCustomActions: function (tablePath, generators) {
					var field = Array.isArray(tablePath) ? getFieldByName(tablePath) : tablePath;
					
					if (!Array.isArray(generators)) generators = [generators];

					communicator.executeAsync('setTableCustomActions',
						{
							internalTableId: field.internalId,
							generators: generators.map(function(generator) {
								return {
									nameCreator: generator.nameCreator.toString(),
									eventCreator: generator.eventCreator.toString()
								};
							})
						});
				},
				setRowIcon: function (tablePath, rowNumber, svgId) {
					var field = Array.isArray(tablePath) ? getFieldByName(tablePath) : tablePath;
					communicator.executeAsync('setRowCustomIcon', { internalTableId: field.internalId, rowNumber: rowNumber, svgId: svgId });
				},
				findField: function(root, path) {
					if (path.length === 0) {
						return null;
					}

					var currentPathElem = path.shift();
					var currentPathIndex;
					if (currentPathElem.indexOf('[') > 0) {
						var currentPathDescr = currentPathElem.split('[');
						currentPathElem = currentPathDescr[0];

						currentPathIndex = parseInt(currentPathDescr[1].slice(0, -1), 10);
					}

					if (!root.Fields && !root.Cells && !root.Sections) {
						return null;
					}

					var rootChild = (root.Fields || root.Cells).filter(function(field) { return field.Name === currentPathElem; })[0];

					if (!rootChild) {
						return null;
					}

					if (typeof currentPathIndex === 'number') {
						rootChild = (rootChild.Fields || rootChild.Rows || rootChild.Sections)[currentPathIndex];
					}
					if (!rootChild) {
						return null;
					}

					if (path.length === 0) {
						return rootChild;
					}

					return findField(rootChild, path);
				},
				getFieldByName: function (namesPath) {
					namesPath = namesPath.slice();
					var doc = commandArgs.params.document;
					return findField(doc, namesPath);
				},
				getCurrentField: function() {
					return getFieldByName(commandArgs.params.currentField.NamePath);
				},
				setFieldValue: function (namesPath, value, forced, onSaved) {
					var field = Array.isArray(namesPath) ? getFieldByName(namesPath) : namesPath;
					
					if (field) {
						if (field.FieldType !== 3) {
							field.Value = value;
							field.Flags = '';
							field.NeedRecognize = false;
						} else {
							if (!Array.isArray(value)) {
								value = [value];
							}

							value = value.map(function (elem) {
								var selectedVariant = field.Variants.filter(function (variant) { return variant.Name === elem })[0];
								return selectedVariant ? selectedVariant.Id : null;
							}).filter(function (val) { return val; });

							field.Values = value;
						}
						if (forced) {
							communicator.executeAsync('updateField', { internalId: field.internalId, fieldValue: value }, onSaved);
						}
					}

					
				},
				getFieldValue: function(namesPath) {
					var field = Array.isArray(namesPath) ? getFieldByName(namesPath) : namesPath;
					if (field) {
						if (field.FieldType !== 3) {
							return field.Value;
						} else {
							return (field.Values || []).map(function(value) {
								var selectedVariant = field.Variants.filter(function(variant) { return variant.Id === value })[0];
								return selectedVariant ? selectedVariant.Name : null;
							}).filter(function(val) { return val; });
						}
					}
				},
				setCurrentFieldValue: function(value, forced) {
					setFieldValue(getCurrentField(), value, forced);
				},
				save: function() {
					this.sendResponse(commandArgs.id, commandArgs.params.document);
				},
				cancel: function() {
					this.sendResponse(commandArgs.id, null);
				}
			};

			var insertPath = "body";

			var getStartComment = function (code) {
				code = code.trim();
				if (code.indexOf('/') !== 0) return '';
				while (code.indexOf('/') === 0) {
					code = code.substr(1);
				}

				return code.substr(0, code.indexOf('*') !== 0 ? code.indexOf('\n') : code.indexOf('*/'));
			};

			var startComment = getStartComment(customScript);
			if (/mode:\s*documentEditor/.test(startComment)) {
				insertPath = '.documentEditor__bottomPanel__container';
			} else if (/mode:\s*formEditor/.test(startComment)) {
				insertPath = '.formEditorEditor__bottomPanel__container';
			}
			customScript = Object.keys(inlineCustomScriptFunctions).map(function(funcKey) {
				var funcInit = funcKey + '=' + inlineCustomScriptFunctions[funcKey].toString();
				if (funcKey === 'save' || funcKey === 'cancel') {
					funcInit += '.bind(this)';
				}
				return funcInit;
			}).join(';')+';' + customScript;

			if (insertPath !== 'body') {
				$(insertPath).empty();
			}

			var inserInFields = insertPath === '.formEditorEditor__bottomPanel__container';
			var insertInDocEditor = insertPath === '.documentEditor__bottomPanel__container';
			var document = self.selectedDocument();

			var mainCommands = {
				selectRow: function (commandArgs) {
					var internalTableId = commandArgs.params.internalTableId;
					var rowNumber = commandArgs.params.rowNumber;
					for (var i = 0; i < allFields().length; i++) {
						var field = allFields()[i];
						if (field.model && field.model.internalId === internalTableId) {
							if (field instanceof FC.Viewmodels.DocumentEditor.TableAdapter && field.rows()[rowNumber]) {
								var row = field.rows()[rowNumber];
								field.instanceMenu.selectInstance(row);
								self.scrollToField(row.model);
								this.sendResponse(commandArgs.id, true);
							}
							break;
						}
					}
					this.sendResponse(commandArgs.id, false);
				},
				deselectRow: function (commandArgs) {
					var internalTableId = commandArgs.params.internalTableId;
					var rowNumber = commandArgs.params.rowNumber;
					for (var i = 0; i < allFields().length; i++) {
						var field = allFields()[i];
						if (field.model && field.model.internalId === internalTableId) {
							if (field instanceof FC.Viewmodels.DocumentEditor.TableAdapter) {
								if (rowNumber !== undefined) {
									field.instanceMenu.unselectInstance(rowNumber);
								} else {
									field.instanceMenu.unselectInstances();
								}
									
								this.sendResponse(commandArgs.id, true);
							}
							break;
						}
					}
					this.sendResponse(commandArgs.id, false);
				},
				scrollFormToField: function (commandArgs) {
					var internalId = commandArgs.params.internalId;
					for (var i = 0; i < allFields().length; i++) {
						var field = allFields()[i];
						if (field.model && field.model.internalId === internalId) {
							self.scrollToField(field.model);
							break;
						}
					}
					this.sendResponse(commandArgs.id, false);
				},
				resetTableCustomActions: function (commandArgs) {
					var internalTableId = commandArgs.params.internalTableId;
					for (var i = 0; i < allFields().length; i++) {
						var field = allFields()[i];
						if (field.model && field.model.internalId === internalTableId) {
							if (field instanceof FC.Viewmodels.DocumentEditor.TableAdapter) {
								field.instanceCustomActionsGenerator(null);
								
								this.sendResponse(commandArgs.id, true);
							}
							break;
						}
					}
					this.sendResponse(commandArgs.id, false);
				},
				setTableCustomActions: function (commandArgs,communicator) {
					var internalTableId = commandArgs.params.internalTableId;
					var generators = commandArgs.params.generators;
					for (var i = 0; i < allFields().length; i++) {
						var field = allFields()[i];
						if (field.model && field.model.internalId === internalTableId) {
							if (field instanceof FC.Viewmodels.DocumentEditor.TableAdapter) {

								var customActionGenerator = function(rowDescription) {
									return generators.map(function (generator) {
										eval('var nameCreator =' + generator.nameCreator);
										eval('var eventCreator =' + generator.eventCreator);
										return {
											name: nameCreator(rowDescription),
											action: function() {
												communicator.executeAsync('triggerEvent', { eventName: eventCreator(rowDescription), data: rowDescription });
											}
										};
									});
								};
								field.instanceCustomActionsGenerator(customActionGenerator);
								this.sendResponse(commandArgs.id, true);
							}
							break;
						}
					}
					this.sendResponse(commandArgs.id, false);
				},
				setRowCustomIcon: function(commandArgs) {
					var internalTableId = commandArgs.params.internalTableId;
					var rowNumber = commandArgs.params.rowNumber;
					var svgId = commandArgs.params.svgId;
					for (var i = 0; i < allFields().length; i++) {
						var field = allFields()[i];
						if (field.model && field.model.internalId === internalTableId) {
							if (field instanceof FC.Viewmodels.DocumentEditor.TableAdapter && field.rows()[rowNumber]) {
								field.rows()[rowNumber].customActionIcon(svgId);
								this.sendResponse(commandArgs.id, true);
							}
							break;
						}
					}
					this.sendResponse(commandArgs.id, false);
				},
				updateField: function (commandArgs) {
					var internalId = commandArgs.params.internalId;
					var fieldValue = commandArgs.params.fieldValue;
					var document = self.selectedDocument();
					if (document && document.allFields()) {
						var allFields = document.allFields();
						for (var i = 0; i < allFields.length; i++) {
							var field = allFields[i].field;
							if (field.internalId === internalId) {
								if (field instanceof FC.BusinessModels.DocumentEditor.TextField) {
									field.value(fieldValue);
									field.flags('');
								} else if (field instanceof FC.BusinessModels.DocumentEditor.BooleanField) {
									field.value(fieldValue);
								} else if (field instanceof FC.BusinessModels.DocumentEditor.ChoiceField) {
									field.values(fieldValue);
								}
							}
						}
					}
					this.sendResponse(commandArgs.id, true);
				},
				getLocalization: function(commandArgs) {
					this.sendResponse(commandArgs.id, FC.Environment.Localization.CustomScriptLocalization);
				},
				getInvoiceStatusSettings: function (commandArgs) {
					var self = this;
					FC.DataContext.Invoice.getInvoceStatusSettings(
						document.templateId(),
						document.templateVersion(),
						function (result) {
							self.sendResponse(commandArgs.id, result);
						},
						function (error) {
							self.sendResponse(commandArgs.id, null, error);
						});
				},
				getDict: function(commandArgs) {
					var self = this;
					FC.DataContext.DbCache.lookup(
						document.templateId(),
						document.templateVersion(),
						commandArgs.params.cacheName,
						commandArgs.params.cursorPos,
						commandArgs.params.filters,
						commandArgs.params.allowMultipleColumnResults,
						function(result) {
							self.sendResponse(commandArgs.id, result);
						},
						function(error) {
							self.sendResponse(commandArgs.id, null, error);
						});
				},
				getDictAll: function(commandArgs) {
					var self = this;
					FC.DataContext.DbCache.search(
						document.templateId(),
						document.templateVersion(),
						commandArgs.params.cacheName,
						commandArgs.params.filters,
						function(result) {
							self.sendResponse(commandArgs.id, result || []);
						},
						function(error) {
							self.sendResponse(commandArgs.id, null, error);
						});
				},
				addEntity: function(commandArgs) {
					var self = this;
					FC.DataContext.DbCache.addEntity(
						document.templateId(),
						document.templateVersion(),
						commandArgs.params.cacheName,
						commandArgs.params.columns,
						commandArgs.params.values,
						function(result) {
							self.sendResponse(commandArgs.id, result);
						},
						function(error) {
							self.sendResponse(commandArgs.id, null, error);
						});
				},
				editEntity: function(commandArgs) {
					var self = this;
					FC.DataContext.DbCache.editEntity(
						document.templateId(),
						document.templateVersion(),
						commandArgs.params.cacheName,
						commandArgs.params.primaryKey,
						commandArgs.params.columns,
						commandArgs.params.values,
						function(result) {
							self.sendResponse(commandArgs.id, result);
						},
						function(error) {
							self.sendResponse(commandArgs.id, null, error);
						});
				},
				getDomElementPosition: function (commandArgs) {
					var self = this;
					var result = null;
					var domId = commandArgs.params.domId;
					if (domId) {
						var element = window.document.getElementById(domId);
						if (element) {
							var coordinates = element.getBoundingClientRect();
							if (coordinates) {
								result = {
									height: coordinates.height,
									left: coordinates.left,
									right: coordinates.right,
									top: coordinates.top,
									width: coordinates.width
								};
							}
						}
					}
					self.sendResponse(commandArgs.id, result);
				}
			};

			var extraStyles = [
				FC.Environment.AppPathNoTennant + "WebCommonNew/Styles/sourcesanspro.css",
				FC.Environment.AppPathNoTennant + "Content/customscript/formWindow.css"
			];

			var commandArgsExtender = function (commandArgs) {
				commandArgs.events = {};
				commandArgs.events.eventHandlers = {};
				commandArgs.events.addEventHandler = function (eventName, handler) {
					var handlers = this.eventHandlers[eventName];
					if (!handlers) {
						handlers = [];
						this.eventHandlers[eventName] = handlers;
					}
					handlers.push(handler);
				};
				commandArgs.events.removeEventHandler = function (eventName, handler) {
					var handlers = this.eventHandlers[eventName];
					if (handlers) {
						var handlerIndex = handlers.indexOf(handler);
						while (handlerIndex > 0) {
							handlers.splice(handlerIndex, 1);
							handlerIndex = handlers.indexOf(handler);
						}
					}

				};
				commandArgs.events.triggerHandlers = function (eventName, eventParams) {
					var handlers = this.eventHandlers[eventName];
					if (handlers) {
						for (var i = 0; i < handlers.length; i++) {
							var handlerResult = handlers[i](eventParams);
							if (handlerResult) break;
						}
					}
				};
				window.customActionContext = commandArgs;

			};
			var frameCommands = {
				start: new Function("commandArgs", "(" + commandArgsExtender.toString() + ")(commandArgs); var communicator = this;" + customScript),
				updateInstances: function (commandArgs) {
					var fieldPath = commandArgs.params.fieldPath;
					var updatedField = commandArgs.params.updatedField;

					if (fieldPath.length > 0 && updatedField) {
						var parentField = getFieldByName(fieldPath.slice(0, -1));
						if (parentField && parentField.Fields) {
							for (var i = 0; i < parentField.Fields.length; i++) {
								if (parentField.Fields[i].Id === updatedField.Id) {
									parentField.Fields[i] = updatedField;
									window.customActionContext.events.triggerHandlers("instancesCompositionChanged", commandArgs.params);
									return;
								}
							}
						}
						
					}
				},
				updateSelectedRows: function (commandArgs) {
					window.customActionContext.events.triggerHandlers("rowSelected", commandArgs.params);
				},
				updateForm: function (commandArgs) {
					setFieldValue(commandArgs.params.fieldPath, commandArgs.params.fieldValue);
					window.customActionContext.events.triggerHandlers("formChanged", commandArgs.params);
				},
				triggerEvent: function (commandArgs) {
					window.customActionContext.events.triggerHandlers(commandArgs.params.eventName, commandArgs.params.data);
				},
				forceClose: function () {
					window.customActionContext.events.triggerHandlers('onForceClose');
				}
			};

			var extraScripts = [
				FC.Environment.AppPathNoTennant + "WebCommonNew/Scripts/External/jquery-3.5.1.min.js",
				FC.Environment.AppPathNoTennant + "WebCommonNew/Scripts/External/jquery-ui.min.js"
			];


			var onValueChanged = function(fieldPath, fieldValue) {
				this.executeAsync('updateForm', { fieldPath: fieldPath, fieldValue: fieldValue });
			};

			var onRowSelectionChanged = function(fieldPath, indexes) {
				this.executeAsync('updateSelectedRows', { fieldPath: fieldPath, indexes: indexes });
			};

			var onChangeInstances = function (fieldPath, updatedField) {
				this.executeAsync('updateInstances', { fieldPath: fieldPath, updatedField: updatedField });
			};

			var scriptLauncher = new CustomScriptLauncher(mainCommands, frameCommands, extraScripts, extraStyles);
			var launchFunction = function (data) {
				var valueChangeSubscriberIndex = changeSubscriptions.valueChangeSubscribers.indexOf(onValueChanged);
				if (valueChangeSubscriberIndex >= 0) {
					changeSubscriptions.valueChangeSubscribers.splice(valueChangeSubscriberIndex, 1);
				}
				var rowSelectionChangeSubscriberIndex = changeSubscriptions.rowSelectSubscribers.indexOf(onRowSelectionChanged);
				if (rowSelectionChangeSubscriberIndex >= 0) {
					changeSubscriptions.rowSelectSubscribers.splice(rowSelectionChangeSubscriberIndex, 1);
				}

				var changeInstancesSubscribersIndex = changeSubscriptions.changeInstancesSubscribers.indexOf(onChangeInstances);
				if (changeInstancesSubscribersIndex >= 0) {
					changeSubscriptions.changeInstancesSubscribers.splice(changeInstancesSubscribersIndex, 1);
				}
				
				if ((data.detail.result !== undefined) && (data.detail.result !== "")) {
					var result = data.detail.result;
					if (result && !jQuery.isEmptyObject(result)) {
						document.updateInfo(result);
						self.validateAllFields();
						document.manipulators.dbCheckRulesManipulator.tryRecountAllRules();
					}
				}
				scriptLauncher.destroy();
				if (insertInDocEditor) {
					window.batchApp.extensions.docEditorBottomPanelVisible(false);
				} else if (inserInFields) {
					formCustomScriptActions.closeCustomScript();
				}
			};

			var launchParams = {
				document: document.toJs(true),
				currentField: data.target.toJs(true),
				appPath: window.location.origin+FC.Environment.AppPathNoTennant,
				coordinates: {
					height: coordinates.height,
					left: coordinates.left,
					right: coordinates.right,
					top: coordinates.top,
					width: coordinates.width

				},				
				currentElementDomId: id
			};

			if (insertInDocEditor) {
				window.batchApp.extensions.docEditorBottomPanelVisible(true);
			} else if (inserInFields) {
				formCustomScriptActions.openCustomScript();
			}


			var onLaunchSuccessed = function (communicator) {
				if (inserInFields) {
					self.fieldsPanelCustomScript = communicator;
				}
				changeSubscriptions.valueChangeSubscribers.push(onValueChanged.bind(communicator));
				changeSubscriptions.rowSelectSubscribers.push(onRowSelectionChanged.bind(communicator));
				changeSubscriptions.changeInstancesSubscribers.push(onChangeInstances.bind(communicator));
			};

			window.requestAnimationFrame(function () {
				scriptLauncher.launch(
					insertPath,
					launchParams,
					launchFunction,
					onLaunchSuccessed
				);
			});
		});

	self.getFieldViewmodel = function (fieldModel) {
		for (var i = 0; i < allFields().length; i++) {
			var field = allFields()[i];
			if (field.model === fieldModel) {
				return field;
			}
		}

		return null;
	};

	self.moveFormToField = function (fieldModel, showIfHidden) {
		var field = self.getFieldViewmodel(fieldModel);
		if (field) {
			return FC.Viewmodels.FormEditor.FieldViewmodelAlgo.moveToField(field, showIfHidden);
		}

		return false;
	}

	var scrollToFieldHorizontally = function ($field, $container) {
		if (!$field || $field.length === 0) {
			return false;
		}

		var delta = 50;
		var containerOffset = $container.offset();
		var fieldLeft = $field.offset().left - containerOffset.left;
		var fieldWidth = $field.outerWidth();
		if (fieldLeft + fieldWidth > $container.width() || fieldLeft < 0) {
			var container = self.containerElement();
			container.scrollLeft = container.scrollLeft + fieldLeft - delta;
			self.formHorizontalScroll.value(container.scrollLeft);
			return true;
		}

		return false;
	};

	self.scrollToField = function (fieldModel) {
		if (fieldModel instanceof FC.BusinessModels.DocumentEditor.BooleanField &&
			fieldModel.parent &&
			fieldModel.parent instanceof FC.BusinessModels.DocumentEditor.ChoiceField) {
			fieldModel = fieldModel.parent;
		}

		var field = self.getFieldViewmodel(fieldModel);

		if (!field) {
			return false;
		}

		var fieldToFocus = fieldModel;
		var fieldViewmodelToFocus = field;
		var notInHideable = false;
		while (field && !field.hasOwnProperty("elementHeight") && fieldModel.parent && !field.parent.hasOwnProperty("hideable")) {
			fieldModel = fieldModel.parent;
			field = self.getFieldViewmodel(fieldModel);
		}

		var container = $(self.containerElement());
		if (!container.is(":visible")) {
			return false;
		}

		var containerOffset = container.offset();
		var actualScrollTop = $(self.scrollableElement()).offset().top;
		var containerTop = self.formVerticalScroll.scrollTop();
		var containerHeight = container.height();
		var containerBottom = containerTop + containerHeight;

		var hideables = self.hideables()
			.sort(function (a, b) { return a.offset() - b.offset(); });

		var fieldTop;
		var fieldBottom;

		if (!field || !field.parent || !field.parent.hasOwnProperty("hideable")) {
			notInHideable = true;
			fieldModel = fieldToFocus;
			field = fieldViewmodelToFocus;
		}

		var hideableParent = null;
		if (!notInHideable) {
			hideableParent = self.getFieldViewmodel(fieldModel.parent);
			if (!hideableParent || !hideableParent.hideable) {
				return false;
			}
		}

		var i;
		var $field;

		if (field instanceof FC.Viewmodels.DocumentEditor.TableAdapter) {
			$field = $("[data-jsdomid=" + field.UniqueIdentifier + "]");
		} else {
			$field = $("#" + field.DomId);
		}

		if (notInHideable && $field.length === 0) {
			return false;
		}

		var offset = notInHideable && $field.length > 0 ? $field.offset().top - actualScrollTop : hideableParent.hideable.offset();
		var $fieldToFocus;
		var additionalOffset = 0;
		var delta = 50;

		if ($field.length === 0) {
			if (!hideableParent.hideable.visible()) {
				var $hideable = $("[data-hideableId=" + hideableParent.hideable.id());
				if ($hideable.length > 0) {
					FC.Viewmodels.FormNavigation.showElement($hideable);
				}

				var h = hideableParent;
				var summaryOffset = 0;
				while (h && (!h.hideable || !h.hideable.visible())) {
					if (h.hasOwnProperty("elementHeight")) {
						var hChildren = h.parent.children();
						for (var j = 0; j < hChildren.length; j++) {
							if (hChildren[j] === h) {
								break;
							}

							summaryOffset += hChildren[j].elementHeight;
						}
					}

					h = h.parent;
				}

				if (h && h.hideable && h.hideable.visible()) {
					var currOfset = h.hideable.offset();
					summaryOffset += h.hideable.offset();

					for (i = 0; i < hideables.length; i++) {
						if (currOfset > hideables[i].offset() && hideables[i].id() !== h.hideable.id()) {
							summaryOffset += hideables[i].totalHeight();
						}
					}

					self.formVerticalScroll.setScrollTop(summaryOffset);
					self.refreshHideableOffsets();
					return true;
				}
				return false;
			}

			for (i = 0; i < hideables.length; i++) {
				if (hideables[i].offset() < hideableParent.hideable.offset()) {
					offset += hideables[i].totalHeight();
				}
			}

			var children = hideableParent instanceof FC.Viewmodels.DocumentEditor.TableAdapter
				? hideableParent.rows()
				: hideableParent.children();

			for (i = 0; i < children.length; i++) {
				if (children[i] === field) {
					break;
				}

				offset += children[i].elementHeight;
			}

			self.formVerticalScroll.setScrollTop(offset - containerOffset.top - delta);

			var currentScrollTop = self.formVerticalScroll.scrollTop();
			$fieldToFocus = $("#" + fieldViewmodelToFocus.DomId);

			scrollToFieldHorizontally($fieldToFocus, container);
			if ($fieldToFocus.length > 0) {
				if (fieldViewmodelToFocus !== field) {
					// надо переполучить поле т.к. оно появилось
					if (field instanceof FC.Viewmodels.DocumentEditor.TableAdapter) {
						$field = $("[data-jsdomid=" + field.UniqueIdentifier + "]");
					} else {
						$field = $("#" + field.DomId);
					}
					additionalOffset = $fieldToFocus.offset().top - $field.offset().top;

					if (additionalOffset > containerHeight) {
						self.formVerticalScroll.setScrollTop(currentScrollTop + additionalOffset - delta);
					}
				}
			}
			return true;
		} else {
			FC.Viewmodels.FormNavigation.showElement($field);

			var fieldHeight;
			if (fieldViewmodelToFocus !== field) {
				if (fieldViewmodelToFocus instanceof FC.Viewmodels.DocumentEditor.TableAdapter) {
					$fieldToFocus = $("[data-jsdomid=" + fieldViewmodelToFocus.UniqueIdentifier + "]");
				} else {
					$fieldToFocus = $("#" + fieldViewmodelToFocus.DomId);
				}
				fieldHeight = $fieldToFocus.outerHeight();
				if ($fieldToFocus.length > 0) {
					additionalOffset = $fieldToFocus.offset().top - $field.offset().top;
				}
			} else {
				$fieldToFocus = $field;
				fieldHeight = $field.outerHeight();
			}

			fieldTop = $field.offset().top - actualScrollTop;
			var visibleHideable = 0;
			for (i = 0; i < hideables.length; i++) {
				if (offset >= hideables[i].offset()) {
					var hideable = $('#formViewer_body [data-hideableId="' + hideables[i].id() + '"]');
					if (hideable.is(":hidden")) {
						continue;
					}
					var heights = hideables[i].heights();

					if (heights.invisibleHeightUpper > 0) {
						fieldTop += heights.invisibleHeightUpper;
					}

					visibleHideable += heights.visibleHeight;
					if (heights.invisibleHeightLower > 0 &&
						(hideables[i].offset() + (notInHideable ? visibleHideable : heights.visibleHeight) < offset) &&
						(notInHideable || hideables[i].id() !== hideableParent.hideable.id())) {

						fieldTop += heights.invisibleHeightLower;
					}
				}
			}

			fieldTop += additionalOffset;
			fieldBottom = fieldTop + fieldHeight;
			var movedX = scrollToFieldHorizontally($fieldToFocus, container);;
			var movedY = false;

			if (fieldTop < containerTop || fieldBottom > containerBottom) {
				self.formVerticalScroll.setScrollTop(fieldTop - delta);
				movedY = true;
			}

			return movedX || movedY;
		}
	};

	self.refreshScrolls = function () {
		if (!self.containerElement()) return;
		self.containerElement.valueHasMutated();
		self.formVerticalScroll.setScrollTop(self.formVerticalScroll.scrollTop());
		var containerElement = self.containerElement();
		if (containerElement) {
			self.formHorizontalScroll.value(containerElement.scrollLeft);
		}
	};

	containerHeight.subscribe(function () {
		self.refreshHideableOffsets();
		self.refreshScrolls();
	});
	
	self.previousSelectedField = ko.observable(null);
	self.selectedField.subscribe(function (value) {
		self.previousSelectedField(value);
	}, null, "beforeChange");

	self.selectedField.subscribe(function (value) {
		if (value !== self.previousSelectedField()) {
			setTimeout(function () {
				self.scrollToField(value);
				self.refreshScrolls();
			}, 1);
		}
	});

	self.disposables = [];

	self.refreshHideableOffsets = function () {
		var hideables = self.hideables().sort(function (a, b) {
			if (a.visible()) {
				return b.visible() ? b.offset() - a.offset() : 1;
			}

			return b.visible() ? -1 : 0;
		});

		for (var i = 0; i < hideables.length; i++) {
			hideables[i].refreshOffset();
		}
	};

	self.onScroll = function (data, event) {
		event.target.scrollTop = 0;
		self.formHorizontalScroll.value(self.containerElement().scrollLeft);
	};

	self.lastInstanceMenu = null;

	self.onResize = function () {
		if (self.containerElement()) {
			self.containerElement.valueHasMutated();
			self.formHorizontalScroll.value(self.containerElement().scrollLeft);
			self.formVerticalScroll.setScrollTop(self.formVerticalScroll.scrollTop());
		}
	}

	jQuery(document)
		.ready(function () {
			window.batchLayout.batchLayoutViewModel.rightFrameVisible.subscribe(function (rightFrameVisible) {
				if (rightFrameVisible) {
					self.refreshHideableOffsets();
					self.refreshScrolls();
				}
			});
		});
};
