﻿FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

/**
 * Вьюмодель поля-картинки
 * @param {} field модель поля-картинки
 * @param {} parent родитель (вьюмодель)
 * @param {} selectedField  выбранное поле (глобальный observable)
 * @param {} selectedParagraph выбранный параграф (глобальный observable)
 * @returns {} 
 */
FC.Viewmodels.DocumentEditor.PictureFieldAdapter = function (field, parent, selectedField, selectedParagraph) {
	var self = this;

	// "унаследуем" от вьюмодели поля
	$.extend(self, new FC.Viewmodels.DocumentEditor.FieldAdapter(field, parent, selectedField, selectedParagraph));

	self.fieldSize = ko.observable({});
	self.containerSize = ko.observable();

	// обновить размер контейнера изображения
	var resizeContainer = function () {
		var getSizeInterval = setInterval(function () {
				var container = $('div[fieldid="' + self.DomId + '"]');
				if (container.length > 0) {
					clearInterval(getSizeInterval);
					var sizes = container.attr('data-sizes');
					if (sizes && sizes.length > 0) {
						self.containerSize({
							width: parseInt(sizes.split(';')[0]),
							height: parseInt(sizes.split(';')[1])
						});
					}
				}
			},
			100);
	};

	/**
	 * Стиль картинки - нужно показать на форме кусок страницы
	 */
	self.RegionWindowStyle = ko.computed(function () {
		var defaultStyle = {};

		var regionFieldPair = self.model.manipulators.regionsManipulator.fieldRegionMap()
			.filter(function (val) {
				return val.field == self.model && val.region.status() &&
					val.region.coordinates() && val.region.coordinates().length > 0;
			})[0];


		if (!regionFieldPair) {
			self.fieldSize({ width: 0, height: 0 });
			resizeContainer();
			return { width: 0, height: 0, 'background-image': "none" };
		}

		var region = regionFieldPair.region;

		if (!region.regionPage()) {
			return defaultStyle;
		}


		var defaultSize = self.containerSize() && self.containerSize().width > 0 ? self.containerSize().width : 230;
		var zoom;

		// получим охватывающий прямоугольник 
		// и отобразим его
		var coord = region.getExternalRect();
		var width = coord.x2 - coord.x1;
		var height = coord.y2 - coord.y1;
		if (height > width) {
			zoom = defaultSize / height;
		} else {
			zoom = defaultSize / width;
		}

		self.fieldSize({
			width: (width * zoom) + 'px',
			height: (height * zoom) + 'px'
		});

		return {
			width: (width * zoom) + 'px',
			height: (height * zoom) + 'px',
			'background-size': region.regionPage().width() * zoom + 'px ' + region.regionPage().height() * zoom + 'px',
			'background-image': 'url(' + region.regionPage().url() + ')',
			'background-position': '-' + (coord.x1 * zoom) + 'px -' + (coord.y1 * zoom) + 'px'
		};
	});	

	resizeContainer();

	self.valueHash = ko.computed(function () {
		var regionFieldPair = self.model.manipulators.regionsManipulator.fieldRegionMap()
			.filter(function (val) {
				return val.field == self.model && val.region.status() &&
					val.region.coordinates() && val.region.coordinates().length > 0;
			})[0];
		if (!regionFieldPair) {
			return '';
		} else {
			var region = regionFieldPair.region;
			var coord = region.coordinates()[0];
			return [coord.x1(), coord.x2(), coord.y1(), coord.y2()].join();
		}

	});
}