﻿FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

/**
 * viewmodel повторяющейся группы
 * @param {} field модель повторяющейся группы
 * @param {} parent родитель (вьюмодель)
 * @param {} selectedField выбранное поле (глобальный observable)
 * @param {} selectedParagraph выбранный параграф (глобальный observable)
 * @returns {} 
 */
FC.Viewmodels.DocumentEditor.RepeatableAdapter = function (field, parent, selectedField, selectedParagraph, offsetInfo, changeSubscribers) {
	var self = this;

	// "унаследуем" от вьюмодели поля
	$.extend(self, new FC.Viewmodels.DocumentEditor.FieldAdapter(field, parent, selectedField, selectedParagraph, changeSubscribers));
	FC.Viewmodels.DocumentEditor.FieldWithChildren(self, field, false, selectedField, selectedParagraph, offsetInfo, changeSubscribers);
	FC.Viewmodels.DocumentEditor.FieldWithColumns(self, self.model.fields, function (f) {
		return f && f.hasOwnProperty("fields") ? f.fields() : [];
	}, offsetInfo);
	var fieldsPanel = window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel;

	self.addItemText = FC.Environment.Localization.GlobalResources.AddNewItemButton.replace('{0}', field.caption());

	self.addRepeatedItem = function () {
		self.model.addRepeatedItem();
	};

	/**
	 * Удалить выбранные инстансы по их индексам (нужно для FC.Viewmodels.DocumentEditor.InstanceMenu)
	 * @param {any} selectedElements индексы выбранных инстансов
	 */
	var removeSelectedInstances = function (selectedElements) {
		var deletedFields = [], remainFields = [];
		self.model.fields().forEach(function(field, index) {
			if (selectedElements.indexOf(index) >= 0) {
				deletedFields.push(field);
			} else {
				remainFields.push(field);
			}
		});

		self.model.manipulators.errorsManipulator.removeFieldsErrors(deletedFields);
		self.model.manipulators.regionsManipulator.removeFields(deletedFields, true);
		self.model.fields(remainFields);

		self.hideable.recalculateHeights();
		fieldsPanel.refreshHideableOffsets();
		fieldsPanel.refreshScrolls();

		FC.Viewmodels.EventStack.addEvent();
	};

	/**
	* Удалить все инстансы (нужно для FC.Viewmodels.DocumentEditor.InstanceMenu)
	*/
	var removeAllInstances = function () {
		var allIndexes = [];
		for (var i = 0; i < self.model.fields().length; i++) {
			allIndexes.push(i);
		}
		removeSelectedInstances(allIndexes);
	};

	/**
	 * Меню для управления инстансами
	 */
	self.instanceMenu = new FC.Viewmodels.DocumentEditor.InstanceMenu(
		// ид
		self._id,
		// добавить инстанс на позиции
		function (index) { self.model.addRepeatedItemOnIndex(index); },
		// удалить выбранные инстансы
		removeSelectedInstances,
		// удалить все инстансы
		removeAllInstances,
		// получить индекс инстанса
		function (instance) { return self.children().indexOf(instance); },
		// выбранное поле
		selectedField,
		// получить число инстансов
		function () { return self.children().length; },
		// подскроллить форму данных к инстансу по его индексу
		function(instanceIndex) {
			var instances = self.model.fields();

			if (instanceIndex < 0 || instanceIndex >= instances.length) {
				fieldsPanel.scrollToField(self.model);
			} else {
				fieldsPanel.scrollToField(instances[instanceIndex]);
			}
		});

	function findInSubField(jsDomId, field) {
		if (field.UniqueIdentifier === jsDomId) {
			return field;
		}

		if (field.children && field.children()) {
			for (var i = 0; i < field.children().length; i++) {
				var childrenField = field.children()[i];
				if (childrenField) {
					var subfield = findInSubField(jsDomId, field.children()[i]);
					if (subfield) {
						return subfield;
					}
				}
			}
		}
		return null;

	}

	var findField = function (uniqueId, fields) {
		for (var i = 0; i < fields.length; i++) {
			var field = fields[i];
			if (!field) continue;
			if (field.UniqueIdentifier === uniqueId) {
				return field;
			}
			var subFields = FC.Viewmodels.FormEditor.FieldViewmodelAlgo.getViewmodelSubfields(field);
			if (subFields) {
				var findResult = findField(uniqueId, subFields);
				if (findResult) return findResult;
			}
		}

		return null;
	}

	self.valueHash = ko.computed(function () {
		return self.children().length;
	});

	self.displayedChildren = ko.observableArray([]);

	self.hideable = null;

	self.recalculateDisplayedChildren = function () {
		var formViewer = $(".documentEditor_formViewer_content");
		if (formViewer.length > 0 && formViewer.children().length > 0) {
			self.hideable = FC.Viewmodels.DocumentEditor.HideableItem.getOrCreateHideable(self);
		}
	};

	fieldsPanel.disposables.push(fieldsPanel.horizontalSeparator.subscribe(self.recalculateDisplayedChildren));
	fieldsPanel.disposables.push(fieldsPanel.containerElement.subscribe(self.recalculateDisplayedChildren));
	fieldsPanel.disposables.push(fieldsPanel.formVerticalScroll.scrollTop.subscribe(self.recalculateDisplayedChildren));

	self.childrenLength = ko.observable(0);
	self.children.subscribe(function (newChildren) {
		if (newChildren.length !== self.childrenLength()) {
			setTimeout(function () {
				if (self.hideable) {
					self.hideable.recalculateHeights();
				}
				fieldsPanel.refreshHideableOffsets();
				fieldsPanel.refreshScrolls();
			}, 10);
		}

		self.childrenLength(newChildren.length);
	});

	self.firstItem = ko.computed(function () {
		return self.children() && self.children().length > 0 ? self.children()[0] : null;
	});	

	
};