﻿FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

/**
 * Добавляет методы для отображения Suggestов
 */
FC.Viewmodels.DocumentEditor.SuggestMethods = function (textFieldViewmodel, offsetsInfo) {
	var self = textFieldViewmodel;


	/* Работа с саггестами */
	var suggestHeight = 25;
	var suggestsMaxHeight = 300;
	var topWindowOffset = 60;
	var bottomWindowOffset = 20;
	var leftWindowOffset = 50;
	// признак того, что список саггестов был только что виден, но его закрыл обработчик события mousedown
	// элемента окна бразуера
	
	var getContainerNode = function () {
		return document.getElementById('formViewer_body');
	}

	self.suggestsVisible = ko.observable(false);	

	// индекс выделенного в отфильтрованном списке саггеста (выделенного нажатием стрелочек на клавиатуре)
	self.selectedSuggestIndex = ko.observable(-1);

	self.offset = ko.observable(0);
	self.suggestsWidth = ko.observable(0);
	self.suggestsHeight = ko.observable(0);
	self.suggestsContentHeight = ko.observable(0);
	self.suggetsTop = ko.observable(0);
	self.suggestsLeft = ko.observable(0);

	// список отсортированных саггестов
	self.filteredSuggests = ko.observableArray([]);

	function getCurrentSuggests() {
		var suggests = self.model.suggests();

		if (suggests && suggests.length) {
			return suggests;
		}

		if (FC.Environment.CloneSuggestsForNewInstance) {
			var computedSuggests = self.model.getDefaultSuggests();
			if (computedSuggests && computedSuggests.length) {
				return computedSuggests;
			}
		}

		return [];
	}

	var getElementContainer = function() {
		var availableParents = $('#' + self.DomId).parents('.customform__FormElement');
		if (availableParents.length === 0) {
			availableParents = $('#' + self.DomId).parents('.field-isCell');
		}
		return availableParents.eq(0);
	}

	var countPopupProperties = function () {
		var contentHeight = self.filteredSuggests().length * suggestHeight;

		var $inputNode = getElementContainer();
		var inputNode = $inputNode[0];
		var boundingRect = inputNode.getBoundingClientRect();

		var popupWidth = boundingRect.right - boundingRect.left;
		var popupLeft = boundingRect.left;
		popupWidth = Math.min(popupWidth, window.innerWidth - popupLeft - leftWindowOffset);
		var popupMaxHeightUnderField = window.innerHeight - boundingRect.bottom - bottomWindowOffset;
		var popupMaxHeight = Math.max(boundingRect.top - topWindowOffset, popupMaxHeightUnderField);
		var popupMaxHeight = Math.min(popupMaxHeight, suggestsMaxHeight);
		var popupHeight = Math.min(popupMaxHeight, contentHeight);

		var popupTop = popupHeight > popupMaxHeightUnderField ? boundingRect.top - popupHeight : boundingRect.bottom;

		self.suggestsWidth(popupWidth);
		self.suggestsHeight(popupHeight);
		self.suggestsContentHeight(contentHeight);
		self.suggetsTop(popupTop);
		self.suggestsLeft(popupLeft);
		
	}
	// Доступны ли вообще саггесты
	self.suggestsSelectable = function () {

		var suggests = getCurrentSuggests();

		return suggests &&
			suggests.length > 0 &&
			!(suggests.length === 1 && suggests[0] === self.model.value());
	};


	var prepareSuggestMarkup = function (text, searchString) {
		var textLower = text.toLowerCase().trim();
		var searchStringLower = searchString.toLowerCase().trim();

		var index = textLower.indexOf(searchStringLower);
		if (index < 0 || !searchString) return text;

		var newText = '';
		if (index > 0) {
			newText += text.substring(0, index);
		}

		newText += '<span class="suggest-part--matched">' +
			text.substr(index, searchString.length) + '</span>';

		newText += text.substr(index + searchString.length);

		return newText;
	};
	
	// функция фильтрации и сортировки саггестов
	var filterSuggests = function (showingUnmatchedSuggests) {

		var suggests = getCurrentSuggests();

		if (!suggests || suggests.length === 0) {
			self.filteredSuggests([]);
		}

		var selectedInBeginning = [];
		var selectedInMiddle = [];
		var unselected = [];

		var text = self.model.value();
		var textLower = text ? text.toLowerCase().trim() : '';

		suggests.forEach(function (suggest) {
			var suggestLower = suggest.toLowerCase().trim();
			var suggesObject = { text: prepareSuggestMarkup(suggest, text), value: suggest, selected: true, hasSeparator: false };
			if (suggestLower.indexOf(textLower) === 0 && text) {
				selectedInBeginning.push(suggesObject);
			} else if (suggestLower.indexOf(textLower) !== -1 && text) {
				selectedInMiddle.push(suggesObject);
			} else if (!FC.Environment.ShowSuggestsOnTyping || showingUnmatchedSuggests) {
				suggesObject.selected = false;
				unselected.push(suggesObject);
			}
		});


		selectedInBeginning = selectedInBeginning.concat(selectedInMiddle);

		if (selectedInBeginning.length > 0 && unselected.length > 0) {
			// саджест с сепаратором
			selectedInBeginning[selectedInBeginning.length - 1].hasSeparator = true;
		}

		self.filteredSuggests(selectedInBeginning.concat(unselected));
	}
	
	// сбрасывает выделение саггеста на форме
	self.resetSuggestSelection = function () {
		self.selectedSuggestIndex(-1);
		self.offset(0);
	}

	/* подскролливает список саггестов при перемещении по нему стрелочками */
	var scrollSuggests = function () {
		var selectedSuggestIndex = self.selectedSuggestIndex();
		if(selectedSuggestIndex<0) return;
		var selectedSuggetsPosition = { top: suggestHeight * selectedSuggestIndex, bottom: (selectedSuggestIndex + 1) * suggestHeight };

		if (selectedSuggetsPosition.top < self.offset()) {
			self.offset(selectedSuggetsPosition.top);
		} else if (selectedSuggetsPosition.bottom > self.offset() + self.suggestsHeight()) {
			self.offset(selectedSuggetsPosition.bottom - self.suggestsHeight());
		}
	}

	var onMouseEventOverSuggests = function (event) {
		var $target = $(event.target);

		var availableParent = getElementContainer();

		var targetInBlock = $target.is(availableParent);
		if (!targetInBlock) {
			$target.parents().each(function() {
				if ($(this).is(availableParent)) {
					targetInBlock = true;
				}
			});
		}
		if (!targetInBlock) {
			self.hideSuggests();
		}
	}

	// выделяет саггест по списку выше
	self.selectSuggestUp = function () {
		if (self.selectedSuggestIndex() > 0) {
			self.selectedSuggestIndex(self.selectedSuggestIndex() - 1);
			scrollSuggests();
		}
	}

	// выделяет саггест по списку ниже
	self.selectSuggestDown = function () {
		var lastSuggestIndex = self.filteredSuggests().length - 1;
		var selectedSuggestIndex = self.selectedSuggestIndex();
		if (selectedSuggestIndex !== lastSuggestIndex) {			
			self.selectedSuggestIndex(selectedSuggestIndex + 1);
			scrollSuggests();
		}
	}

	// саггест стал выделеным потому что пользователь навёл на него мышью
	self.suggestHoveredByClientMouseMove = true;

	self.onArrowsKeys = function (keyCode, event) {
		switch (keyCode) {
			case 38:		// Up
				event.preventDefault();
				if (self.suggestsVisible() && self.filteredSuggests() && self.filteredSuggests().length) {
					self.selectSuggestUp();
					self.suggestHoveredByClientMouseMove = false;
				}
				break;
			case 40:	// Down
				event.preventDefault();
				if (self.suggestsVisible() && self.filteredSuggests() && self.filteredSuggests().length) {
					self.selectSuggestDown();
					self.suggestHoveredByClientMouseMove = false;
				}
				break;
		}
	}

	// движение мыши над контейнером саггестов
	self.suggestsContainerMouseMove = function (data, event) {
		self.suggestHoveredByClientMouseMove = true;
	}

	// пользователь навед мышь над саггестом
	self.suggestMouseEnter = function (data, event) {
		if (self.suggestHoveredByClientMouseMove) {
			var context = ko.contextFor(event.currentTarget);
			self.selectedSuggestIndex(context.$index());
		}
	}

	// метод отображения саггестов на экране
	self.showingSuggests = function (showingUnmatchedSuggests) {
		if (self.suggestsVisible()) {
			self.hideSuggests();
			return;
		}

		if (!self.selected() || !$("#" + self.DomId).is(":focus")) {
			self.selected();
			// нужно сфокусироваться на поле, чтобы в него встал курсор
			$("#" + self.DomId).trigger("focus");
			
		}

		// перед каждой фильтрацей сбрасываем выделение в списке саггестов
		self.resetSuggestSelection();
		filterSuggests(showingUnmatchedSuggests);

		// если отсортированный список саггестов не пуст...
		if (self.filteredSuggests() && self.filteredSuggests().length) {
			// ... и список саггестов не показан, то показываем его
			if (!self.suggestsVisible()) {
				self.suggestsVisible(true);
				$(window).on("click."+self.DomId, onMouseEventOverSuggests);				
			}
			countPopupProperties();
			
		} else {

			// Иначе нужно скрыть выпадающий список саггестов. 
			// Чтобы не болталась пустая рамка под текстовым полем.
			self.hideSuggests();
		}
	};
	
	
	$(window).resize(function () {
		self.hideSuggests();
	});
	/* Показать все саггесты для поля (Вызывается, когда пользователь нажал на "стрелочку" в поле)*/
	self.showAllSuggests = function () {		
		self.showingSuggests(true);
	}

	/* Показать только те саггесты, которые совпадают с введенным в поле текстом */
	self.showOnlyMatchedSuggests = function () {		
		self.showingSuggests(false);
	}

	// метод скрывающий саггесты с экрана
	self.hideSuggests = function () {
		if (!self.suggestsVisible()) return;

		self.resetSuggestSelection();
		
		self.suggestsVisible(false);
		$(window).off("click."+self.DomId);
	};

	// пользователь отжал мышь на "хэлпере-стрелочке"
	self.suggestsHelperMouseUp = function (data, event) {

		if (event.which !== 1) {
			return true;
		}
		
		
		self.showAllSuggests();

		
	}

	// действие по нажатию клавиши Enter на выделенном саггесте
	self.onSuggestEnterPressed = function () {


		if (self.selectedSuggestIndex() === -1) {
			return;
		}

		var suggest = self.filteredSuggests()[self.selectedSuggestIndex()];
		if (suggest && suggest.value) {

			// подставляем саггест в значение поля
			self.model.value(suggest.value);
			// верифицируем поле и остаёмся в нём же
			self.rewritableFunctions.verify(false);
		}

		self.hideSuggests();

		
	}

	// пользователь кликнул мышью на саггесте
	self.suggestMouseDown = function (data, event) {
		self.model.value(data.value);
		self.hideSuggests();
		self.rewritableFunctions.verify(false);
		return true;
	};

	self.popupStyles = ko.computed(function () {
		return {
			width: self.suggestsWidth() + 'px',
			height: self.suggestsHeight() + 'px',
			top: self.suggetsTop() + 'px',
			left: self.suggestsLeft() + 'px'
		};
	});

	offsetsInfo.verticalOffset.subscribe(function () {
		if (self.suggestsVisible()) {

			countPopupProperties();
			//Закрываем, если комбик ушел из зоны отрисовки
			setTimeout(function () {
				var containerElement = getContainerNode();
				var containerElemRect = containerElement.getBoundingClientRect();
				var elem = document.getElementById(self.DomId);
				if (!elem) {
					self.hideSuggests();
					return;
				}
				var elemRect = elem.getBoundingClientRect();
				if ((elemRect.top < containerElemRect.top) || (elemRect.bottom > containerElemRect.bottom)) {
					self.hideSuggests();
				}
			},
				10);

		}
	});

	offsetsInfo.horizontalOffset.subscribe(function () {
		if (self.suggestsVisible()) {

			countPopupProperties();
			//Закрываем, если комбик ушел из зоны отрисовки
			setTimeout(function () {
					var containerElement = getContainerNode();
					var containerElemRect = containerElement.getBoundingClientRect();
					var elem = document.getElementById(self.DomId);
					if (!elem) {
						self.hideSuggests();
						return;
					}
					var elemRect = elem.getBoundingClientRect();
					if ((elemRect.right < containerElemRect.left) || (elemRect.left > containerElemRect.right)) {
						self.hideSuggests();
					}
				},
				10);

		}
	});

	self.suggestsScrollbar = new FC.ScrollBarMini(self.offset, self.suggestsHeight, self.suggestsContentHeight);
}