﻿FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

/**
 * viewModel таблицы
 * @param {} table модель таблицы
 * @param {} parent родитель (viewModel)
 * @param {} selectedField выбранное поле (глобальный observable)
 * @param {} selectedParagraph выбранный параграф (глобальный observable)
 * @returns {} 
 */
FC.Viewmodels.DocumentEditor.TableAdapter = function (table, parent, selectedField, selectedParagraph, offsetInfo, changeSubscribers) {
	var self = this;

	// "унаследуем" от вьюмодели поля
	$.extend(self, new FC.Viewmodels.DocumentEditor.FieldAdapter(table, parent, selectedField, selectedParagraph, changeSubscribers));

	self.UniqueIdentifier = "table" + (FC.Viewmodels.DocumentEditor.TableAdapter.GlobalCounter++);
	self.addTableRowText = FC.Environment.Localization.GlobalResources.AddNewItemButton
		.replace('{0}', table.caption() ? table.caption() : table.name());

	self.rows = ko.observableArray([]);
	self.bodyDomId = "body_" + self.UniqueIdentifier;
	self.rowsLength = ko.observable(-1);

	FC.Viewmodels.DocumentEditor.FieldWithColumns(self, self.model.rows, function (row) { return row.cell(); }, offsetInfo);

	var fieldsPanel = window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel;

	ko.computed(function () {
		if (!self.model.rows()) {
			return;
		}

		var newArray = [];
		$.each(self.model.rows(),
			function (ind, row) {
				for (var i = 0; i < self.rows().length; i++) {
					var child = self.rows()[i];
					if (child.model === row) {
						newArray.push(child);
						return;
					}
				}
				newArray.push(new FC.Viewmodels.DocumentEditor.TableRowAdapter(row, self, selectedField, selectedParagraph, offsetInfo, changeSubscribers));
			});

		if (newArray.length !== self.rows().length) {
			if (self.rowsLength() < 0) {
				setTimeout(function () {
					if (self.hideable) {
						self.hideable.recalculateHeights();
					}
					fieldsPanel.refreshHideableOffsets();
					fieldsPanel.refreshScrolls();
				}, 10);

				self.rowsLength(newArray.length);
			}
			self.rows(newArray);
		}
		for (var i = 0; i < newArray.length; i++) {
			if (newArray[i] != self.rows()[i]) {
				self.rows(newArray);
				break;
			}
		}

	});

	self.firstRow = ko.computed(function () {
		return self.rows() && self.rows().length > 0 ? self.rows()[0] : null;
	});	

	self.addTableRow = function () {
		self.model.addTableRowAtPosition(self.model.rows().length);
		self.notifyValueChanged(self.model.toJs(true), FC.Viewmodels.DocumentEditor.FieldAdapter.NotificationTypes.InstanceChanged);
	};

	/**
	 * Удалить выбранные строки
	 * @param {any} selectedRows выбранные строки
	 */
	var deleteRows = function (selectedRows) {
		window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel.selectedField(null);
		window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel.previousSelectedField(null);

		self.model.deleteRows(selectedRows);
		self.notifyValueChanged(self.model.toJs(true), FC.Viewmodels.DocumentEditor.FieldAdapter.NotificationTypes.InstanceChanged);
		FC.Viewmodels.EventStack.addEvent();
	};

	/**
	 * Удалить выбранные строки по их индексам (нужно для FC.Viewmodels.DocumentEditor.InstanceMenu)
	 * @param {any} selectedElements индексы выбранных строк
	 */
	var removeSelectedInstances = function (selectedElements) {
		var rows = self.model.rows();
		var selectedRows = [];
		for (var i = 0; i < selectedElements.length; i++) {
			selectedRows.push(rows[selectedElements[i]]);
		}

		deleteRows(selectedRows);
	};

	/**
	 * Удалить все строки (нужно для FC.Viewmodels.DocumentEditor.InstanceMenu)
	 */
	var removeAllInstances = function () {
		deleteRows(self.model.rows().slice(0));
	};


	self.selectedRowsIndexes = ko.observableArray([]);

	self.instanceCustomActionsGenerator = ko.observable();

	self.customActions = ko.computed(function () {
		if (!self.selectedRowsIndexes() ||
			self.selectedRowsIndexes().length === 0 ||
			!self.instanceCustomActionsGenerator()) {
			return [];
		}
		
		
		var selectedRow = self.model.rows()[self.selectedRowsIndexes()[0]];
		return self.instanceCustomActionsGenerator()(selectedRow.toJs());
	});

	/**
	 * Меню для управления строками
	 */
	self.instanceMenu = new FC.Viewmodels.DocumentEditor.InstanceMenu(
		// ид
		self._id,
		// добавить строку на позиции
		function (index) { self.model.addTableRowAtPosition(index); self.notifyValueChanged(self.model.toJs(true), FC.Viewmodels.DocumentEditor.FieldAdapter.NotificationTypes.InstanceChanged);},
		// удалить выбранные строки
		removeSelectedInstances,
		// удалить все строки
		removeAllInstances,
		// получить индекс строки
		function(row) { return self.rows().indexOf(row); },
		// текущее выбранное поле
		selectedField,
		// получить число строк
		function() { return self.rows().length; },
		// подскроллить к строке по индексу
		function(rowIndex) {
			var rows = self.rows();

			if (rowIndex < 0 || rowIndex >= rows.length) {
				fieldsPanel.scrollToField(self.model);
			} else {
				fieldsPanel.scrollToField(rows[rowIndex].model);
			}
		},
		function(indexes) {
			self.notifyValueChanged(indexes, FC.Viewmodels.DocumentEditor.FieldAdapter.NotificationTypes.RowSelectedChange);
			self.selectedRowsIndexes(indexes);
		},
		self.customActions);


	self.deleteRow = function (data) {
		window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel.selectedField(null);
		window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel.previousSelectedField(null);
		data.dispose();
		self.model.deleteRow(data.model);
		self.notifyValueChanged(self.model.toJs(true), FC.Viewmodels.DocumentEditor.FieldAdapter.NotificationTypes.InstanceChanged);
	};

	self.afterTableRendered = function () {
		var attrSplit = function (elem, attrName) {
			var values = elem.attr(attrName);
			if (values && values.length > 0) {
				values = values.split(",");
				for (var i = 0; i < values.length; i++) {
					values[i] = +values[i];
				}
				return values;
			}
			return [];
		}
		var getTableFullId = function (table) {
			var ids = [table.id()];
			var field = table.parent;
			while (field) {
				ids.push(field.id());
				field = field.parent;
			}
			return ids.join(',');
		}


		var elem = $('#documentEditorFormViewer div[data-jsdomid="' + self.UniqueIdentifier + '"]');
		if (!elem || elem.length === 0) return;
		self.columnsOrder(attrSplit(elem, "data-columnsorder"));


		var section = self.model.parent;
		while (section.parent) {
			section = section.parent;
		}

		var doc = section.document;

		var hideableColumnsStorageKey = [
			doc.projectId(),
			doc.templateId(),
			doc.templateVersion(),
			getTableFullId(table)
		].join(',');

		var hideableColumnsFromStorage = localStorage.getItem('hidableColumns-' + hideableColumnsStorageKey);
		var hideableColumns;
		if (!hideableColumnsFromStorage || hideableColumnsFromStorage.length === 0) {
			hideableColumns = attrSplit(elem, "data-hideablecolumns");
		} else {
			hideableColumns = $.map(hideableColumnsFromStorage.split(','),
				function (data) {
					return parseInt(data);
				});
		}

		self.hideableColumns(hideableColumns);
		self.hideableColumns.subscribe(function (val) {
			localStorage.setItem('hidableColumns-' + hideableColumnsStorageKey, val.join(','));
		});

	}

	self.displayedRows = ko.observableArray([]);

	self.displayedRowsRendered = function (data, row) {
		if (row) {
			row.recalculateRowHeight();
			var rowWithChildren = FC.Viewmodels.FormEditor.FieldViewmodelAlgo.getAllChildren(row);
			rowWithChildren.filter(function (elem) { return elem.selected && elem.selected(); }).forEach(function (selectedElem) {
				document.getElementById(selectedElem.DomId).focus();
			});
		}
	};

	self.hideable = null;

	self.recalculateDisplayedChildren = function () {
		var formViewer = $(".documentEditor_formViewer_content");
		if (formViewer.length > 0 && formViewer.children().length > 0) {
			self.hideable = FC.Viewmodels.DocumentEditor.HideableItem.getOrCreateHideable(self);
		}
	};

	fieldsPanel.disposables.push(fieldsPanel.horizontalSeparator.subscribe(self.recalculateDisplayedChildren));
	fieldsPanel.disposables.push(fieldsPanel.containerElement.subscribe(self.recalculateDisplayedChildren));
	fieldsPanel.disposables.push(fieldsPanel.formVerticalScroll.scrollTop.subscribe(self.recalculateDisplayedChildren));
	

	self.rows.subscribe(function (newRows) {
		if (newRows.length !== self.rowsLength()) {
			setTimeout(function () {
				if (self.hideable) {
					self.hideable.recalculateHeights();
				}
				fieldsPanel.refreshHideableOffsets();
				fieldsPanel.refreshScrolls();
			}, 10);
		}

		self.rowsLength(newRows.length);
	});
};

FC.Viewmodels.DocumentEditor.TableAdapter.GlobalCounter = 0;