﻿FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};
FC.Viewmodels.DocumentEditor.TableRowAdapter =
	function(tableRow, parent, selectedField, selectedParagraph, offsetInfo, changeSubscribers) {
		var self = this;

		self.model = tableRow;
		self.changeSubscribers = changeSubscribers;
		self._id = FC.Viewmodels.DocumentEditor.TableRowAdapter.Common.getId();

		self.DomId = "tableRow_" + self._id;

		self.parent = parent;

		self.disposables = [];

		self.customActionIcon = ko.observable('');

		self.cells = ko.observableArray($.map(self.model.cell(),
			function(subField) {
				return FC.Viewmodels.FormEditor.FieldViewmodelAlgo.generateField(subField,
					self,
					selectedField,
					selectedParagraph,
					offsetInfo,
					changeSubscribers);
			}));

		self.cellsToShow = ko.computed(function() {
			var res = [];
			var columnsOrder = self.parent.columnsOrder();
			if (!columnsOrder || columnsOrder.length === 0) {
				columnsOrder = [];
				$.each(self.model.cell(),
					function(ind, cell) {
						columnsOrder.push(cell.id());
					});
			}
			var hideableEmptyColumns = self.parent.hideableEmptyColumns();
			$.each(columnsOrder,
				function(ind, currentColumn) {
					if (hideableEmptyColumns.indexOf(currentColumn) >= 0) return;
					for (var i = 0; i < self.cells().length; i++) {
						var cell = self.cells()[i];
						if (cell.model.id() === currentColumn) {
							res.push(cell);
							return;
						}
					}
				});
			return res;
		});

		self.disposables.push(self.cellsToShow);

		self.selected = ko.computed(function() {
			var cells = self.cells();
			if (!cells || cells.length === 0) {
				return false;
			}

			for (var i = 0; i < cells.length; i++) {
				if (cells[i].selected()) {
					return true;
				}
			}

			return false;
		});

		self.disposables.push(self.selected);

		self.dispose = function() {
			var cells = self.cells();
			for (var i = 0; i < cells.length; i++) {
				cells[i].dispose();
			}

			while (self.disposables.length > 0) {
				var obj = self.disposables.pop();
				//if (obj && obj.hasOwnProperty("dispose")) {
				obj.dispose();
				//}
			}
		};

		self.recalculateRowHeight = function() {
			var row = $("#" + self.DomId);
			if (!row || row.length === 0) {
				return;
			}

			var maxHeight = -1;
			var cells = row.find(".field-isCell:not(.field_checkbox-container):not(.field_checkbox-label):not(.field_checkbox)");

			cells.each(function(index, cell) {
				var $cell = $(cell);
				var height = $cell.children().height();

				if (height > maxHeight) {
					maxHeight = height;
				}
			});

			cells.each(function(index, cell) {
				var $cell = $(cell);
				$cell.height(maxHeight);
			});
		};
	};

FC.Viewmodels.DocumentEditor.TableRowAdapter.Common =
	{
		lastId: 0,
		getId: function () { return ++this.lastId; }
	};
