﻿FC.Viewmodels.DocumentEditor = FC.Viewmodels.DocumentEditor || {};

/**
 * viewModel для текстового поля
 * @param {} field модель поля
 * @param {} parent родитель (viewModel)
 * @param {} selectedField выбранное поле (глобальный observable)
 * @param {} selectedParagraph выбранный параграф (глобальный observable)
 * @returns {} 
 */
FC.Viewmodels.DocumentEditor.TextFieldAdapter = function (field, parent, selectedField, selectedParagraph, offsetsInfo, changeSubscribers) {
	var self = this;

	// "унаследуем" от вьюмодели поля
	$.extend(self, new FC.Viewmodels.DocumentEditor.FieldAdapter(field, parent, selectedField, selectedParagraph, changeSubscribers));

	var lastValidatedValue = self.model.value();

	self.Validator = field.Validator;

	self.ElementBottomLine = ko.observable(0);

	FC.Viewmodels.DocumentEditor.SuggestMethods(self, offsetsInfo);

	var baseFormFieldValueCss = self.FormFieldValueCss;
	var baseVerify = self.rewritableFunctions.verify;

	/**
	 * Выбрать поле
	 * @returns {} 
	 */
	self.selectField = function (data, event, paragraphOnly) {

		// найдём позицию курсора
		var selectionBoundries = null;

		// по клику нужно всегда оповещать подписчиков у selectedField/selectedParagraph
		var isClicked = false;
		if (event) {
			selectionBoundries = window.selectionTools.getSelectionBoundaries(event.currentTarget);
			isClicked = event.hasOwnProperty("type") ? event.type.indexOf("mouse") > -1 : false;
		}

		// если не нашли - ставим его на 0
		if (!selectionBoundries) {
			selectionBoundries = { start: 0 };
		}

		var flags = self.model.flags();

		// ищем текущий параграф
		var paragraphIndex = 0;
		var i = selectionBoundries.start - 1;
		while (i >= 0) {
			if (flags[i] === '\n') {
				paragraphIndex++;
			}
			i--;
		}

		// сохранием текущий параграф
		var refreshSelected = false;
		if (paragraphIndex !== self.selectedParagraph() || isClicked) {
			refreshSelected = true;
			self.selectedParagraph(paragraphIndex);
		}

		if (!paragraphOnly) {
			// сохраняем выбранное поле
			if (self.model !== self.selectedField()) {
				self.selectedField(self.model);
			} else if (refreshSelected) {
				self.selectedField.valueHasMutated();
			}
		}

		return true;
	}

	/**
	 * Выбрать поле по позиции курсора
	 * @param {} cursorPosition номер символа
	 * @returns {} 
	 */
	self.selectFieldAtCursor = function (cursorPosition) {
		var flags = self.model.flags();
		if (cursorPosition >= flags.length) {
			cursorPosition = flags.length - 1;
		}


		if (flags.length === 0) {
			self.selectField();
			return;
		}

		// считаем число групп символов и позицию курсора в группе
		// может быть группа уверенных или неуверенных символов
		// у уверенных flags[i]==0, у неуверенных не равна
		var groupCount = 0;
		var groupCharCount = 0;

		for (var i = 1; i < cursorPosition; i++) {
			if (flags[i] !== flags[i - 1]) {
				groupCount++;
				groupCharCount = 0;
			} else {
				groupCharCount++;
			}
		}

		// установка курсора по разному работает для группы неуверенных и группы уверенных символов
		var cursor;
		if (flags[cursorPosition] === "1") {
			cursor = { parentIndex: cursorPosition, nodeIndex: 0, charIndex: groupCharCount };
		} else {
			cursor = { parentIndex: -1, nodeIndex: groupCount, charIndex: groupCharCount };
		}
		var elem = document.getElementById(self.DomId);

		// если элемента нет на форме - надо подкрутить форму данных к полю
		// скорее всего, это hideable
		if (!elem) {
			setTimeout(
				function () {
					window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel.scrollToField(self.model);

					// когда появится разметка - поставим курсор
					setTimeout(function () {
						var elem = document.getElementById(self.DomId);
						if (elem) {
							FC.Utils.CaretTools.setCaretPositionWithRange(elem, cursor);
							self.selectField(null, { currentTarget: elem });
						}
					}, 2);
				},
				100
			);

			return;
		}

		// ставим курсор
		FC.Utils.CaretTools.setCaretPositionWithRange(elem, cursor);
		// выбираем поле
		setTimeout(function () { self.enterField(null, { currentTarget: elem }, true); }, 1);
	};

	
	self.updateElementBottomLine = function () {
		self.ElementBottomLine($("#" + self.DomId).outerHeight());
	}


	self.rewritableFunctions.verify = function (goToNext) {
		if (goToNext == undefined)
			goToNext = true;

		if (lastValidatedValue !== self.model.value()) {
			FC.Viewmodels.Validation.FieldValidation.Validate(self.model.getValidator(),
				self.model.value(),
				self.createFormatError,
				self.removeFormatError);
			lastValidatedValue = self.model.value();
		}

		// дeлаем отображаемый в поле текст "уверенным"
		baseVerify(function () {

			var oldFlags = self.model.flags();
			var text = self.model.value() || "";
			var newflags = "";
			for (var i = 0; i < text.length; i++) {
				if (oldFlags[i] === '\n') {
					newflags += "\n";
				} else {
					newflags += "0";
				}
			}

			self.model.flags(newflags);

			self.undoEngine.add(new FC.Viewmodels.DocumentEditor.TextFieldUndoSnapshot(
				self.model.value(), self.model.flags(),
				true, -1));

		}, goToNext);
	};

	/* Стиль для формы данных - отражает состояния поля */
	self.FormFieldValueCss = ko.pureComputed(function () {


		var css = baseFormFieldValueCss();

		css["field_text"] = true;
		css["field_hasSuggests"] = self.suggestsSelectable();
		css["field_noregion"] = !self.model.hasRegion() && (!self.model.value() || self.model.value().length === 0);

		if (self.hasCustomAction() && self.model.parent instanceof FC.BusinessModels.DocumentEditor.TableRow) {
			css["cell-with-customAction"] = true;
		}

		return css;
	});
	
	self.disposables.push(self.model.value.subscribe(function () {
		FC.Viewmodels.EventStack.addEvent();

		if (self.lastTextChangedByClient) {
			self.lastTextChangedByClient = false;

			if (self.model.needRecognize()) {
				self.model.needRecognize(false);
			}

			// показ саггестов во время пользовательского ввода
			if (FC.Environment.ShowSuggestsOnTyping) {
				self.showOnlyMatchedSuggests();
			}
		} else {
			self.undoEngine.add(new FC.Viewmodels.DocumentEditor.TextFieldUndoSnapshot(
				self.model.value(), self.model.flags(), self.model.isVerified(), -1));
			self.expectedCaretPosition = self.model.value() ? self.model.value().length : 0;
			self.notifyValueChanged(self.model.value());
		}

		setTimeout(function () {
			var fieldsPanel = window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel;
			fieldsPanel.refreshScrolls();

			if (self.parent && self.parent instanceof FC.Viewmodels.DocumentEditor.TableRowAdapter) {
				self.parent.recalculateRowHeight();
			}
		}, 1);

		return true;
	}));

	/* Движок операций Undo-Redo текстового поля */
	self.undoEngine = new FC.Viewmodels.DocumentEditor.TextFieldUndoEngine();
	self.undoEngine.add(new FC.Viewmodels.DocumentEditor.TextFieldUndoSnapshot(
		self.model.value(), self.model.flags(), self.model.isVerified(), -1));

	self.expectedCaretPosition = 0;

	self.caret = new FC.Viewmodels.DocumentEditor.CaretParameters(-1, -1, 0);

	// последнее изменение текста поля из интерфейса (не программно)
	self.lastTextChangedByClient = false;

	var changeFieldState = function (state) {
		if (!state) {
			return;
		}

		if (state.caretPosition > -1) {
			self.expectedCaretPosition = state.caretPosition;
		} else {
			self.expectedCaretPosition = state.text.length;
		}

		self.lastTextChangedByClient = true;

		if (self.model.isVerified() !== state.isVerified) {
			self.model.isVerified(state.isVerified);
		}

		self.model.value(state.text);
		self.model.flags(state.flags);

	}

	self.undo = function () {
		var fieldState = self.undoEngine.undo();
		if (fieldState != null) {
			changeFieldState(fieldState);
		}
	}

	self.redo = function () {
		var fieldState = self.undoEngine.redo();
		if (fieldState != null) {
			changeFieldState(fieldState);
		}
	}

	var baseKeyPressed = self.KeyPressed;
	self.KeyPressed = function (data, event) {

		var keyCode = event.keyCode || event.which;

		if (this.model.isDbLocked()) {
			var accordingRules = this.model.manipulators.dbCheckRulesManipulator.getAccordingRules(this.model);
			if (accordingRules.length > 0) {
				if ((keyCode >= 48 && keyCode <= 57) ||
					(event.keyCode >= 65 && event.keyCode <= 90) ||
					(event.keyCode >= 97 && event.keyCode <= 111) || keyCode === 46 || keyCode === 8) {

					Sammy.apps.body.trigger('docEditor_runDbLookupForRule',
						{
							rule: accordingRules[0],
							repeatableContext: this.model.getRepeatableContext(),
							searchValue: this.model.value()
						});
					event.preventDefault();
					return false;
				}
			}

		}

		// перенос каретки на следующую строку
		if (keyCode === 13 && event.shiftKey) {
			return true;
		}

		if ((keyCode === 40) && event.ctrlKey && !self.suggestsVisible()) {
			event.preventDefault();
			self.showAllSuggests();
			return false;
		}

		if (self.suggestsVisible()) {

			if (keyCode === 27) {
				self.suggestsVisible(false);
				event.preventDefault();
				return false;
			}
			// если нажали клавищу Enter  
			if (keyCode === 13) {

				// и при этом выбран в списке саггест
				if (self.selectedSuggestIndex() != -1) {
					self.onSuggestEnterPressed();
					return false;
				} else {

					// иначе просто закрываем список саггестов
					self.hideSuggests();
				}
			}

			// нажаты стрелочки вверх или вниз
			if ([38, 40].indexOf(keyCode) >= 0) {
				self.onArrowsKeys(keyCode, event);
			}

			// по нажатию Esc или Tab закрываем список саггестов
			if ([27, 9].indexOf(keyCode) >= 0) {
				self.hideSuggests();
			}
		}

		return baseKeyPressed(data, event);
	}

	// возвращает сбалансированные флаги по тексту
	self.getBalancedFlags = function (initialText, initialFlags) {

		var text = initialText || "";
		var flags = initialFlags ? initialFlags.split("") : [];

		// дописываем флаги
		var flagsCount;
		if (text.length > flags.length) {
			flagsCount = text.length - flags.length;
			for (var i = 0; i < flagsCount; i++) {
				flags.push("0");
			}
		}

		// отрезаем лишние флаги
		if (text.length < flags.length) {
			flagsCount = flags.length - text.length;
			for (var i = 0; i < flagsCount; i++) {
				flags.pop();
			}
		}

		return flags;
	}

	/**
	 * Html-содержимое текстового поля, редактируемого пользователем
	 * Разметка строится на основе исходного текста и флагов
	 */
	self.displayedHtml = ko.computed(function () {
		var initialText = self.model.value() || "";
		var initialFlags = self.model.flags() || "";

		var chars = initialText ? initialText.split("") : [];
		var flags = self.getBalancedFlags(initialText, initialFlags);

		var previousCharWasUncertain = false;
		var previousTagWasBr = false;
		var previousTagWasSpan = false;

		var spanOpened = false;
		var spanIndex = 0;

		var spanChildIndex = 0;
		var rootChildIndex = 0;
		var charIndex = 0;

		// сбрасываем параметры каретки
		self.caret.parentIndex = -1;
		self.caret.nodeIndex = -1;
		self.caret.charIndex = 0;

		var html = [];
		var i;

		for (i = 0; i < flags.length; i++) {
			if (i === self.expectedCaretPosition) {
				self.caret.charIndex = charIndex;
				self.caret.nodeIndex = spanOpened ? spanChildIndex : rootChildIndex;
				self.caret.parentIndex = spanOpened ? spanIndex : -1;
			}

			if (flags[i] !== '0' && flags[i] !== '\n') {
				if (!previousCharWasUncertain) {
					previousCharWasUncertain = true;
					html.push('<span class="uncertain-text" data-index="' + i + '">');
					spanOpened = true;
					spanIndex = i;
					spanChildIndex = 0;

					if (i > 0) {
						rootChildIndex++;
					}
					charIndex = 0;
				}
			} else {
				if (previousCharWasUncertain) {
					previousCharWasUncertain = false;
					previousTagWasSpan = true;
					previousTagWasBr = false;

					html.push('</span>');

					spanOpened = false;
					spanChildIndex = 0;
					charIndex = 0;
				}
			}

			var char = chars[i];
			if (char === '\n' || char === '\r') {
				html.push('<br data-index="' + i + '" />');

				if (spanOpened) {
					if (charIndex !== 0) {
						spanChildIndex += 1;
					}
				} else {
					if (i > 0) {
						rootChildIndex += 1;
					}
				}
				charIndex = 0;
				previousTagWasBr = true;
				previousTagWasSpan = false;
			} else {

				switch (char) {
					case "<": html.push("&lt;");
						break;
					case ">": html.push("&gt;");
						break;
					default: html.push(char);
				}

				if (previousTagWasBr) {
					if (spanOpened) {
						if (charIndex !== 0) {
							spanChildIndex += 1;
						}
					} else {
						if (i > 0) {
							rootChildIndex += 1;
						}
					}
				}

				if (previousTagWasSpan) {
					rootChildIndex += 1;
				}
				previousTagWasBr = false;
				previousTagWasSpan = false;
				charIndex++;
			}
		}


		if (i > 0 && i === self.expectedCaretPosition) {
			self.caret.charIndex = charIndex;
			self.caret.nodeIndex = spanOpened ? spanChildIndex : rootChildIndex;
			self.caret.parentIndex = spanOpened ? spanIndex : -1;
		}

		if (spanOpened) {
			html.push('</span>');
		}

		var res = html.join("");

		//Firefox hack
		return res.length === 0 ? '<br>' : res;

	}).extend({ deferred: true });
	self.disposables.push(self.displayedHtml);

	/**
	 * Функция, реагирующая на перерисовку содержимого текстового поля	 
	 */
	self.htmlRendered = function () {

		// пересчет нижней границы текстового поля
		self.updateElementBottomLine();
	}

	/**
	 * Функция, реагирующая на изменение содержимого текстового поля пользователем
	 * Здесь меняется текст и флаги
	 */
	self.clientHtmlChanged = function (startPosition, deletedCharsCount, insertedChars) {
		var initialText = self.model.value();
		var initialflags = self.model.flags();

		var text = initialText ? initialText.split("") : [];
		var flags = initialflags ? initialflags.split("") : [];

		var newChars = insertedChars || "";
		var newCharCount = newChars.length;
		var newFlags = "";
		for (var i = 0; i < newCharCount; i++) {
			newFlags += "0";
		}

		self.expectedCaretPosition = startPosition + newCharCount;
		if (newCharCount > 0) {
			text.splice(startPosition, deletedCharsCount, newChars);
			flags.splice(startPosition, deletedCharsCount, newFlags);
		} else {
			text.splice(startPosition, deletedCharsCount);
			flags.splice(startPosition, deletedCharsCount);
		}

		var resultText = text.join("");
		var resultFlags = flags.join("");

		self.undoEngine.add(new FC.Viewmodels.DocumentEditor.TextFieldUndoSnapshot(
			resultText, resultFlags,
			self.model.isVerified(),
			self.expectedCaretPosition));

		self.lastTextChangedByClient = true;

		self.model.value(resultText);
		self.model.flags(resultFlags);
		var resultIsUncertain = self.model.hasUncertainSymbol();

		if (!resultIsUncertain) {
			Sammy.apps.body.trigger("fieldUncertaintyChanged", { field: self.model });
		}
	}

	self.enterField = function (data, event, paragraphOnly) {
		if (self.model.valueAuthorized) {
			lastValidatedValue = self.model.value();
		}

		self.selectField(data, { currentTarget: event.currentTarget, which: 1 }, paragraphOnly);
	}

	self.leaveField = function () {
		if (lastValidatedValue !== self.model.value()) {
			self.model.valueAuthorized = false;
			self.model.manipulators.dbCheckRulesManipulator.tryRecount([self.model], []);
			FC.Viewmodels.Validation.FieldValidation.Validate(self.model.getValidator(),
				self.model.value(),
				self.createFormatError,
				self.removeFormatError);
			lastValidatedValue = self.model.value();

			self.notifyValueChanged(self.model.value());
		}
	};

	self.mouseDownEvent = function (data, event) {
		if (event.currentTarget === event.target) {
			self.enterField(data, event);
			return false;
		}

		return true;
	}

	self.valueHash = ko.computed(function () {
		return self.model.value() ? self.model.value().hashCode() : '';
	});
	self.disposables.push(self.valueHash);

	self.disposables.push(ko.computed(function () {
		var sugVisible = self.suggestsVisible();
		var selected = self.showBalloon();
		self.updateElementBottomLine();
	}));

}
