﻿/**
 * Вертикальный скролл формы данных
 * @constructor 
 * @param {} containerElement - контейнер формы данных (dom-элемент)
 * @param {} scrollableElement - скроллируемый элемент формы данных (DOM - элемень)
 * @param {} hideables  - скрываемые элементы формы данных (observable)
 * @returns {} 
 */
FC.Viewmodels.FormEditor.FormVerticalScroll = function (containerElement, containerHeight, scrollableElement, hideables, horizontalSliderVisible) {
	var scrollTop = ko.observable(0).extend({ notify: "always" });
	var scrollSize = 16;

	var scrollHeight = function () {
		var scrollable = scrollableElement();
		if (!scrollable) {
			return 0;
		}

		var invisible = 0;
		var hideablesArray = hideables();
		for (var i = 0; i < hideablesArray.length; i++) {
			if (hideablesArray[i].visible()) {
				var heights = hideablesArray[i].heights();
				invisible += heights.invisibleHeightUpper;
				invisible += heights.invisibleHeightLower;
			}
		}
		return scrollable.scrollHeight + invisible;
	};

	var verticalScrollableSize = ko.computed(function () {
		var scrollable = scrollableElement();
		var container = containerElement();
		var contHeight = containerHeight();
		if (!scrollable || !container || $(container).is(":hidden")) {
			return 0;
		}

		var invisible = 0;
		var hideablesArray = hideables();
		for (var i = 0; i < hideablesArray.length; i++) {
			if (hideablesArray[i].visible()) {
				var heights = hideablesArray[i].heights();
				invisible += heights.invisibleHeightUpper;
				invisible += heights.invisibleHeightLower;
			}
		}
		
		return scrollable.scrollHeight + invisible;
	});


	var verticalContainerSize = ko.computed(function () {
		var hasHorizontalSlider = horizontalSliderVisible();
		var scrollable = scrollableElement();
		if (!containerHeight() || !scrollable) {
			return 0;
		}

		return containerHeight() - (hasHorizontalSlider ? scrollSize:0);
	});


	scrollTop.subscribe(function (value) {
		containerElement.valueHasMutated();
		var hideablesArray = hideables()
			.filter(function (h) { return h.visible(); })
			.sort(function (a, b) { return a.offset() - b.offset(); });

		var invisibleUpper = 0;

		
		
		var hideablesTotalHeight = 0;
		var i;

		var $alreadyCounted = [];
		for (i = 0; i < hideablesArray.length; i++) {
			var currentHideable = hideablesArray[i];
			currentHideable.recalculateHeights(hideablesTotalHeight, value, containerHeight());
			var currentHideableTotalHeight = currentHideable.totalHeight();

			if (currentHideableTotalHeight > 0) {
				var $hideable = $('#formViewer_body [data-hideableId="' + currentHideable.id() + '"]');
				var notCounted = true;
				for (var j = 0; j < $alreadyCounted.length; j++) {
					if ($hideable.closest($alreadyCounted[j]).length !== 0) {
						notCounted = false;
						break;
					}
				}

				if (notCounted) {
					$alreadyCounted.push($hideable);
					hideablesTotalHeight += currentHideable.totalHeight();
					invisibleUpper += currentHideable.heights().invisibleHeightUpper;
				}
			}
		}

		var scrollable = scrollableElement();
		if (scrollable) {
			var top = invisibleUpper - value;
			top = top <= 0 ? top : 0;
			scrollable.style.top = top + "px";
		}

		containerElement.valueHasMutated();
	});


	var setScrollTop = function (supposedScrollTop) {
		
		
		var currentScrollHeight = scrollHeight();

		if (supposedScrollTop < 0) {
			scrollTop(0);
			containerElement.valueHasMutated();
		} else if (supposedScrollTop + containerHeight() >= currentScrollHeight) {
			if (currentScrollHeight > containerHeight()) {
				scrollTop(currentScrollHeight - containerHeight());
			} else {
				scrollTop(0);
			}
			containerElement.valueHasMutated();
		} else {
			scrollTop(supposedScrollTop);
			containerElement.valueHasMutated();
		}
	};

	FC.ScrollBarNew.call(this,
		FC.ScrollBarNew.Orientation.Vertical,
		verticalContainerSize,
		scrollTop,
		verticalContainerSize,
		verticalScrollableSize,
		null);


	this.scrollTop = scrollTop;
	this.setScrollTop = setScrollTop;
	this.verticalScrollableSize = verticalScrollableSize;
	this.verticalContainerSize = verticalContainerSize;
};