﻿FC.Viewmodels.FormNavigation = {
	/* Перейти к  полю
	 * direction > 0 - к следующему
	 * direction < 0 - к предыдующему
	 * onlyErrors - переходить только к полям с ошибкой
	 */
	navigateToNextField: function (direction, currentElementDomId) {
		var currentElem = null;

		if (currentElementDomId) {
			currentElem = document.getElementById(currentElementDomId);
		}

		if (!currentElem) {
			currentElem = document.activeElement;
		}

		var currentTabOrder = direction > 0 ? -1 : 100000;
		var currentSubform = null;
		var sectionDomId = null;

		if (currentElem) {
			if (!currentElem.getAttribute("data-tabable")) {
				// для чекбоксов
				currentElem = currentElem.previousElementSibling;
			}

			if (currentElem.getAttribute("data-tabable").toLowerCase() === "true") {
				sectionDomId = currentElem.getAttribute("data-sectionId");
				var subformId = currentElem.getAttribute("data-subformId");
				if (!subformId) {
					// таблица
					if (this.focusOnNextCell(currentElem, direction)) {
						return;
					}

					var tableElem = $(currentElem).parents("div[data-fieldtype='Table']")[0];
					subformId = tableElem.getAttribute("data-subformId");
					sectionDomId = tableElem.getAttribute("data-sectionId");

					var tableSection = document.getElementById(sectionDomId);
					if (tableSection.getAttribute("data-autotab") !== "True") {
						currentElem = tableElem;
					}
				}

				currentSubform = document.getElementById(subformId);
				currentTabOrder = currentElem.getAttribute("data-tabOrder");
			}
		}

		if (!currentSubform) {
			var formContainer = $("#formViewer_body");
			var subForms = formContainer.find("div[data-fieldtype='subform']");
			if (!subForms || subForms.length === 0) {
				// нет форм - нечего искать
				return;
			}

			currentSubform = subForms[0];

			if (!currentSubform) {
				return;
			}

			sectionDomId = currentSubform.id;
		}

		var nextField;
		var section = document.getElementById(sectionDomId);
		if (!section) {
			return;
		}

		while (section.getAttribute("data-autotab") === "True") {
			var tabables = $(section).find("[data-hideableid],[data-tabable=\"True\"]");
			if (this.focusOnNextTabable(tabables, currentElem, direction)) {
				return;
			}

			currentElem = null;
			var sections = $(section.parentElement).children("[data-fieldtype='subform']");
			var currentSectionIndex = sections.index(section);
			currentSectionIndex = direction > 0 ? currentSectionIndex + 1 : currentSectionIndex - 1;
			if (currentSectionIndex >= sections.length) {
				currentSectionIndex = 0;
			}

			if (currentSectionIndex < 0) {
				currentSectionIndex = sections.length - 1;
			}

			section = sections[currentSectionIndex];
			currentTabOrder = direction > 0 ? -1 : 100000;
			currentSubform = section;
		}

		do {
			if (!currentSubform) {
				return;
			}

			nextField = this.findNextFieldInSubform(currentSubform, currentTabOrder, direction, currentElem);

			if (nextField) {
				if (nextField.getAttribute("data-fieldtype") === "subform") {
					currentTabOrder = direction > 0 ? -1 : 100000;
					currentSubform = nextField;
					nextField = null;
				}
			} else {
				// дошли до самого верха
				if (!currentSubform.getAttribute("data-fieldtype")) {
					nextField = this.findNextFieldInSubform(currentSubform, direction > 0 ? -1 : 100000, direction, currentElem);
					if (!nextField) {
						return;
					}
				} else {
					var parentSubformId = currentSubform.getAttribute("data-subformId");
					currentTabOrder = parseInt(currentSubform.getAttribute("data-taborder"));
					currentSubform = parentSubformId ? document.getElementById(parentSubformId) : currentSubform.parentElement;
				}
			}

			currentElem = null;
		} while (!nextField);

		if (nextField.tagName.toLowerCase() === "input" ||
			nextField.isContentEditable ||
			$(nextField).hasClass("customForm__Combobox")) {
			this.focusOnElement(nextField, direction);
			return;
		}

		if (!nextField.hasOwnProperty("find")) {
			// таблица
			this.focusOnNextCell(null, direction, nextField);
		} else {
			this.focusOnElement(nextField.find("input,[contanteditable=\"true\"],.customForm__Combobox"), direction);
		}
	},

	findNextFieldInSubform: function (subform, currentTabIndex, direction, currentElem) {
		var tabableSelector = "[data-taborder][data-subformId=\"" + subform.id + "\"]";

		// если это самый верх
		if (!subform.getAttribute("data-fieldtype")) {
			tabableSelector = "[data-taborder][data-subformId=\"\"]";
		}

		var isRepeatable = subform.getAttribute("data-isrepeatable") === "true";
		var currentInstance = -1;
		if (isRepeatable && currentElem) {
			currentInstance = parseInt($(currentElem).parents("[data-isinstance]").first().children("input:hidden").val());
		}

		var tabables = $(subform).find(tabableSelector);

		var nextElement = null;
		var nextElementTabOrder = direction > 0 ? 1000000 : -1;
		var nextInstanceIndex = -1;
		var possibleNext = null;
		var possibleNextTaborder = direction > 0 ? 1000000 : -1;

		for (var i = 0; i < tabables.length; i++) {
			var tabOrder = parseInt(tabables[i].getAttribute("data-taborder"));

			if (isRepeatable) {
				nextInstanceIndex = parseInt($(tabables[i]).parents("[data-isinstance]").first().children("input:hidden").val());
			}

			if (!isRepeatable ||
				direction > 0 && nextInstanceIndex >= currentInstance ||
				direction < 0 && nextInstanceIndex <= currentInstance) {

				if (isRepeatable &&
					(direction > 0 && (nextInstanceIndex === currentInstance + 1) && tabOrder > -1 && tabOrder < possibleNextTaborder ||
						direction < 0 && (nextInstanceIndex === currentInstance - 1) && tabOrder < 1000000 && tabOrder > possibleNextTaborder)) {

					possibleNextTaborder = tabOrder;
					possibleNext = tabables[i];
				}

				if (direction > 0 && tabOrder > currentTabIndex && tabOrder < nextElementTabOrder ||
					direction < 0 && tabOrder < currentTabIndex && tabOrder > nextElementTabOrder) {
					nextElement = tabables[i];
					nextElementTabOrder = tabOrder;
				}
			}
		}

		if (!nextElement && isRepeatable) {
			return possibleNext;
		}

		return nextElement;
	},

	focusOnNextCell: function (currentElem, direction, table) {
		if (!table) {
			table = $(currentElem).parents("div[data-fieldtype='Table']")[0];
		}

		var fieldsPanel = window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel;
		var tableField = null;
		var currentCell = null;
		if (currentElem) {
			currentCell = fieldsPanel.findField($(currentElem));
			tableField = currentCell.parent.parent;
		} else {
			tableField = fieldsPanel.findField($(table));
		}

		var rows = tableField.rows();
		if (rows.length === 0) {
			return false;
		}

		var currentRowIndex;
		if (currentCell) {
			currentRowIndex = rows.indexOf(currentCell.parent);
		} else {
			currentRowIndex = direction > 0 ? 0 : rows.length - 1;
		}

		do {
			var currentCells = rows[currentRowIndex].cellsToShow();
			if (currentCells.length === 0) {
				return false;
			}

			var cellIndex = direction > 0 ? 0 : currentCells.length - 1;
			if (currentCell) {
				cellIndex = currentCells.indexOf(currentCell) + (direction > 0 ? 1 : -1);
			}

			for (; direction > 0 ? cellIndex < currentCells.length : cellIndex >= 0; direction > 0 ? cellIndex++ : cellIndex--) {
				if (currentCells[cellIndex].model.isVisible()) {
					currentCells[cellIndex].selectField();
					return true;
				}
			}

			currentCell = null;

			if (direction > 0) {
				currentRowIndex++;
			} else {
				currentRowIndex--;
			}
		} while (direction > 0 ? currentRowIndex < rows.length : currentRowIndex >= 0)

		return false;
	},

	focusOnNextTabable: function (tabables, currentElem, direction) {
		var i = direction > 0 ? 0 : tabables.length - 1;
		var imax = direction > 0 ? tabables.length : -1;
		if (!currentElem) {
			this.focusOnElement($(tabables[i]), direction);
			return true;
		}

		for (; direction > 0 ? i < tabables.length : i > 0; i = direction > 0 ? i + 1 : i - 1) {
			if (tabables[i] === currentElem) {
				break;
			}
		}

		if (direction > 0 && i >= tabables.length - 1) {
			return false;
		}

		if (direction < 0 && i === 0) {
			return false;
		}

		do {
			i = direction > 0 ? i + 1 : i - 1;
		}
		while ((direction > 0 ? i < imax : i > imax) && !this.focusOnElement(tabables[i], direction))

		return i !== imax;
	},

	focusOnInstance: function (instanceViewModel, direction) {
		var fieldsPanel = window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel;
		fieldsPanel.scrollToField(instanceViewModel.model);

		var tabables = $("#" + instanceViewModel.DomId).find("[data-hideableid],[data-tabable=\"True\"]");
		return this.focusOnNextTabable(tabables, null, direction);
	},

	focusOnHideable: function (fieldViewModel, direction) {
		if (!fieldViewModel) {
			return false;
		}


		var i;
		if (fieldViewModel instanceof FC.Viewmodels.DocumentEditor.FieldsGroupAdapter ||
			fieldViewModel instanceof FC.Viewmodels.DocumentEditor.SectionAdapter ||
			fieldViewModel instanceof FC.Viewmodels.DocumentEditor.RepeatableAdapter) {
			var groupFields = fieldViewModel.children();
			if (groupFields.length === 0) {
				return false;
			}

			return this.focusOnInstance(fieldViewModel, direction);
		}

		if (fieldViewModel instanceof FC.Viewmodels.DocumentEditor.TableRowAdapter) {
			var cells = fieldViewModel.cellsToShow();
			if (cells.length === 0) {
				return false;
			}

			i = direction > 0 ? 0 : cells.length - 1;
			cells[i].selectField();
			return true;
		}

		fieldViewModel.selectField();
		return true;
	},

	showElement: function ($elem) {
		if ($elem.is(':hidden')) {
			$elem.show();
			$elem.parent().children('.customForm__summary').remove();
			var collapser = FC.Utils.DocumentEditor.findConnectedElementsToField($elem);
			if (collapser && collapser.length > 0 && FC.Utils.DomClassWrapper.hasClass(collapser[0].children()[0],"groupExpander-down")) {
				collapser[0].click();
				if (FC.Utils.DomClassWrapper.hasClass(collapser[0].children()[0], "groupExpander-down")) {
					FC.Utils.DomClassWrapper.removeClass(collapser[0].children()[0],"groupExpander-down");
				}
			}

			$elem.parents().show();
			$elem.parents().each(function () {
				$(this).parent().children('.customForm__summary').remove();
				var collapser = FC.Utils.DocumentEditor.findConnectedElementsToField($(this));
				if (collapser && collapser.length > 0) {
					var children = collapser[0].children();
					var clicked = false;
					if (FC.Utils.DomClassWrapper.hasClass(children[0],"groupExpander-down")) {
						collapser[0].click();
						clicked = true;
					}

					if (!clicked && collapser[0].attr("data-sectionindex") && children.hasClass("docdetails--triangle-down")) {
						collapser[0].click();
					}
				}
			});

			return true;
		}

		return false;
	},

	focusOnElement: function (elem, direction) {
		if (!elem) {
			return false;
		}

		var $elem = elem instanceof jQuery ? elem : $(elem);
		var tabable = $elem.attr("data-tabable");
		var hasTabable = tabable && tabable.length > 0;
		var hideableId = $elem.attr("data-hideableId");

		if (!hasTabable && !hideableId) {
			return false;
		}

		var wasHidden = FC.Viewmodels.FormNavigation.showElement($elem);

		if (hasTabable) {
			var viewModel = ko.dataFor($elem[0]);
			viewModel.selectField();
			$elem.focus();
			return true;
		}

		if (hideableId) {
			var fieldsPanel = window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel;
			if (wasHidden) {
				fieldsPanel.containerElement.valueHasMutated();
			}

			var hideables = fieldsPanel.hideables().filter(function (h) { return h.id() === hideableId; });
			if (hideables.length === 0) {
				return false;
			}

			var hideableChildren = hideables[0].children();
			if (hideableChildren.length === 0) {
				return false;
			}

			if (direction < 0 && hideables[0].heights().visibleHeight > 0) {
				var heightUpper = hideables[0].heights().invisibleHeightUpper;
				if (heightUpper === 0) {
					return false;
				}


				var i = 0;
				while (heightUpper > 0) {
					var currentChild = hideableChildren[i];
					heightUpper -= currentChild.elementHeight;
					i++;
				}

				return FC.Viewmodels.FormNavigation.focusOnHideable(hideableChildren[i], direction);
			}

			var childToSelect = direction > 0 ? hideableChildren[0] : hideableChildren[hideableChildren.length - 1];
			return FC.Viewmodels.FormNavigation.focusOnHideable(childToSelect, direction);
		}
	}
};