﻿FC.Viewmodels = FC.Viewmodels || {};

FC.Viewmodels.GeneralViewmodel = function (currentDocument, currentPage, editorWidth, dependencies) {
	var self = this;
	var separatorHeight = 6;
	var documentDataContext = dependencies.documentDataContext;
	var eventManager = dependencies.eventManager;

	self.selectedDocument = currentDocument;
	self.selectedPage = currentPage;
	self.headerTemplateName = ko.observable('generalFunctions');
	self.templateName = ko.observable('formEditor_template');
	self.documentData = ko.observable();


	self.errorPanelOpened = ko.observable(true);
	self.dblookupOpened = ko.observable(false);
	self.customScriptOpened = ko.observable(false);
	self.loadFailed = ko.observable(false);
	self.documentHasChanges = ko.observable(false);
	self.bottomPartSize = ko.observable(0);
	self.bottomPanelHeight = ko.computed(function () {
		return self.bottomPartSize() - separatorHeight;
	});

	self.updateDocumentInfo = function (doc, loadAsTask, onSuccess) {
		if (!doc ||
			doc.Status() !== FC.BatchEditor.Document.Status.Normal &&
			doc.Status() !== FC.BatchEditor.Document.Status.RecognizeFinished &&
			doc.Status() !== FC.BatchEditor.Document.Status.CheckRulesFinished) {
			if (onSuccess) {
				onSuccess();
			}
			return;
		}

		documentDataContext.getDocument(
			doc.TaskId(),
			doc.ProjectId(),
			doc.BatchId(),
			doc.Id(),
			doc.Version(),
			loadAsTask,
			function (data) {
				data.removeClassificationError();

				var formErrorsSummary = [];
				data.errors().forEach(function (error) {
					var severity = error.severity() ? 1 : 0;
					var formErrorsSummaryItem = formErrorsSummary.filter(function (errSummary) {
						return errSummary.Severity == severity && errSummary.Type == error.type();
					})[0];
					if (!formErrorsSummaryItem) {
						formErrorsSummary.push({
							Severity: severity,
							Type: error.type(),
							Count: 1
						});
					} else {
						formErrorsSummaryItem.Count++;
					}
				});
				var assErrors = data.assemblingErrors().map(
					function (val) {
						return new FC.BatchEditor.AssemblingError({
							DisplayText: val.description()
						});
					});

				doc.VerificationSymbolsCount(data.countVerificationSymbols());

				doc.Version(doc.Version() + 1);
				doc.AssemblingErrors(assErrors);

				doc.FormErrors($.map(formErrorsSummary,
					function (item) {
						return new FC.BatchEditor.ErrorSummary(item);
					}));

				data.pages().forEach(function (page) {
					var docNode;
					for (var i = 0; i < doc.ChildNodes().length; i++) {
						if (doc.ChildNodes()[i].Id() === page.id()) {
							docNode = doc.ChildNodes()[i];
							break;
						}
					}

					if (docNode) {
						docNode.KeyFieldValue(page.keyValue());
						var keyFieldErrors = data.assemblingErrors().filter(function (assError) { return assError.type() === 3 });
						if (keyFieldErrors.length > 0) {
							var pageSection = data.fields().filter(function (field) {
								return field && field.pageIds().indexOf(page.id()) >= 0
							})[0];
							if (pageSection) {
								docNode.KeyFieldIsBad(keyFieldErrors.filter(function (err) {
									return err.documentNodeRef().filter(function (ref) {
										return ref[0] === pageSection.id();
									}).length >
										0;
								}).length >
									0);
							} else {
								docNode.KeyFieldIsBad(false);
							}
						} else {
							docNode.KeyFieldIsBad(false);
						}
					}
				});


				self.documentHasChanges(false);

				self.fieldsViewmodel.refreshScrolls();

				if (onSuccess) {
					onSuccess();
				}
			});
	};

	self.openErrorPanel = function () {
		self.dblookupOpened(false);
		self.customScriptOpened(false);
		self.errorPanelOpened(true);
		self.fieldsViewmodel.tryCloseFECustomScript();
	};

	self.openCustomScript = function () {
		self.errorPanelOpened(false);
		self.dblookupOpened(false);
		self.customScriptOpened(true);
	};

	self.checkRules = function (onlySave, hideMask, onSuccess, dontReloadDocument) {
		self.openErrorPanel();
		if (self.documentHasChanges() &&
			self.documentData() &&
			self.documentData().fields() &&
			self.documentData().fields().length > 0 &&
			self.documentData().templateId()) {

			var doc = self.selectedDocument();

			if (hideMask) {
				if (doc.HasChanges()) {
					Sammy.apps.body.trigger("applyChanges");
					return true;
				}
			}

			var onRequestSuccess = function (result) {
				if (result === FC.BusinessModels.CheckRulesRequestStatus.Failed) {
					FC.Utils.Error.Show(FC.Environment.Localization.GlobalResources.CheckRulesFailed);
				}

				// если уходим со страницы
				if (dontReloadDocument) {
					if (onSuccess) {
						onSuccess();
					}
					return;
				}

				doc.checkRulesStarted(false);
				doc.checkRulesResultUnwatched(true);
				doc.Version(doc.Version() + 1);

				if (doc.Id() !== self.selectedDocument().Id()) {
					return;
				}

				if (!dontReloadDocument && !onlySave) {
					self.updateDocumentInfo(doc, false, function () {
						if (onSuccess) {
							onSuccess();
						}
					});
				} else {
					if (!onlySave) {
						self.documentHasChanges(false);
					}

					if (onSuccess) {
						onSuccess();
					}
				}
			};

			doc.checkRulesStarted(true);
			documentDataContext.checkRules(self.documentData().toJs(false), onlySave, onRequestSuccess);

			return true;
		}

		return false;
	};

	self.extrapolateTable = function (rowIndex, useExistingRows, fieldPath) {

		if (self.documentData() &&
			self.documentData().fields() &&
			self.documentData().fields().length > 0 &&
			self.documentData().templateId()) {
			// сбросим выделение текущей ячейки - после экстраполяции она может измениться
			self.fieldsViewmodel.selectedField(null);

			var doc = self.selectedDocument();
			doc.checkRulesStarted(true);

			var onRequestSuccess = function (result) {
				doc.checkRulesStarted(false);
				doc.checkRulesResultUnwatched(true);
				doc.Version(doc.Version() + 1);

				if (doc.Id() !== self.selectedDocument().Id()) {
					return;
				}

				if (result === FC.BusinessModels.CheckRulesRequestStatus.Failed) {
					FC.Utils.Error.Show(FC.Environment.Localization.GlobalResources.CheckRulesFailed);
				}

				self.updateDocumentInfo(doc, true);
			};

			documentDataContext.extrapolateRows(self.documentData().toJs(false), rowIndex, useExistingRows, fieldPath, onRequestSuccess);
			return true;
		}

		return false;
	};

	self.nonContextOperations = ko.observableArray([
		new FC.NonContextFunction({
			name: FC.Environment.Localization.GlobalResources.CheckRules,
			order: 1,
			action: function () {
				self.checkRules(false, true);
			},
			htmlClass: 'batchEditor__nonContextOperations__checkRules',
			enabled: self.documentHasChanges,
			title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.CheckRules, FC.Environment.Localization.NonLocalizedHotKeys.CheckRulesHotkey)
		})
	]);

	var parentContainerId = 'documentEditorFormViewer';
	var minBottomHeight = 6;
	var minTopHeight = 50;
	var eventNamespace = 'rightPartSeparator';
	self.separatorIsMoving = ko.observable(false);

	self.readonlyMode = ko.observable(false);

	self.topPartSize = ko.computed(function () {
		var bottomPartSize = self.bottomPartSize();
		return $('#' + parentContainerId).getHiddenDimensions().height -
			((self.errorsViewmodel && self.errorsViewmodel.hasErrors()) || self.dblookupOpened() || self.customScriptOpened()
				? bottomPartSize
				: 0);
	});

	self.fieldsViewmodel = new FC.Viewmodels.FormEditor.FieldsPanel(self.documentData,
		self.topPartSize,
		editorWidth,
		dependencies,
		{
			openCustomScript: self.openCustomScript,
			closeCustomScript: function () {
				if (!self.dblookupOpened())
					self.openErrorPanel();
			}
		}
	);

	self.errorsViewmodel = new FC.Viewmodels.FormEditor.ErrorsPanel(self.documentData,
		self.fieldsViewmodel.selectedField,
		self.selectedDocument,
		self.bottomPanelHeight,
		editorWidth,
		function (field) {
			self.fieldsViewmodel.scrollToField(field);
		},
		dependencies);

	self.regionsViewmodel = new FC.Viewmodels.DocumentEditor.DocumentRegionsPanel(self.documentData, currentDocument, self.fieldsViewmodel.selectedField, self.fieldsViewmodel.selectedParagraph, self.fieldsViewmodel.scrollToField, documentDataContext, self.readonlyMode);
	self.dblookupViewmodel = new FC.Viewmodels.FormEditor.DbLookupViewmodel(self.bottomPanelHeight, editorWidth, self.regionsViewmodel.textSelectingMode, self.regionsViewmodel.lastSelectedText, self.openErrorPanel);


	self.separatorMouseMove = function (event) {
		var parentContainer = $('#' + parentContainerId);

		if (event.originalEvent.pageY < (parentContainer.offset().top + minTopHeight) ||
			event.originalEvent.pageY > (parentContainer.offset().top + parentContainer.outerHeight() - minBottomHeight)) {
			return;
		}

		var newHeight = parentContainer.offset().top + parentContainer.outerHeight() - event.originalEvent.pageY;
		if (newHeight >= minBottomHeight) {
			self.bottomPartSize(newHeight);
		}
	};

	self.separatorMouseUp = function () {
		$(window).unbind("mouseup." + eventNamespace);
		$(window).unbind("mousemove." + eventNamespace);
		self.separatorIsMoving(false);
	};

	self.separatorMouseDown = function (data, event) {
		if (self.separatorIsMoving()) {
			return;
		}

		self.separatorIsMoving(true);

		var parentContainer = $('#' + parentContainerId);
		var newHeight = parentContainer.offset().top + parentContainer.outerHeight() - event.originalEvent.pageY;
		if (newHeight >= minBottomHeight) {
			self.bottomPartSize(newHeight);
		}

		$(window).bind("mouseup." + eventNamespace, self.separatorMouseUp);
		$(window).bind("mousemove." + eventNamespace, self.separatorMouseMove);
	};

	var setBottomPanelSize = function (storageKey, defaultProportion) {
		var proportion = defaultProportion;
		if (localStorage.getItem(storageKey)) {
			proportion = parseFloat(localStorage.getItem(storageKey));
		}

		var container = $('#' + parentContainerId);

		if (container.is(":hidden")) {
			return;
		}

		var bottomHeight = Math.floor(proportion * container.height());

		self.bottomPartSize(bottomHeight > minBottomHeight ? bottomHeight : minBottomHeight);
	};

	var setErrorPanelSize = function () {
		setBottomPanelSize('horizontalSeparatorErrorPos', 1 / 4);
	};

	var setLookupPanelSize = function () {
		setBottomPanelSize('horizontalSeparatorLookupPos', 1 / 2);
	};

	var setCustomScriptPanelSize = function () {
		setBottomPanelSize('horizontalSeparatorCustomScriptPos', 1 / 4);
	};


	var oldWidth = 0;
	editorWidth.subscribe(function (val) {
		if (!oldWidth) {
			oldWidth = val;
			setTimeout(function () {
				if (self.errorPanelOpened()) {
					setErrorPanelSize();
				} else if (self.dblookupOpened()) {
					setLookupPanelSize();
				} else if (self.customScriptOpened()) {
					setCustomScriptPanelSize();
				}
			}, 1);
		}
	});

	var separatorInitInterval = setInterval(function () {
		if ($('.documentEditor__bottomPart .horizontalSeparator').length > 0) {
			clearInterval(separatorInitInterval);
			setErrorPanelSize();
		}
	}, 200);

	self.errorPanelOpened.subscribe(function (val) {
		if (val) setErrorPanelSize();
	});

	self.dblookupOpened.subscribe(function (val) {
		if (val) {
			self.fieldsViewmodel.tryCloseFECustomScript();
			setLookupPanelSize();
		}
	});

	self.customScriptOpened.subscribe(function (val) {
		if (val) setCustomScriptPanelSize();
	});

	self.bottomPartSize.subscribe(function (val) {
		var parentContainer = $('#' + parentContainerId);
		var containerHeight = parentContainer.height();
		if (containerHeight < 1 || val > containerHeight) return;
		var proportion = val / containerHeight;
		var storageKeyName = 'horizontalSeparatorErrorPos';
		if (self.dblookupOpened()) {
			storageKeyName = 'horizontalSeparatorLookupPos';
		} else if (self.customScriptOpened()) {
			storageKeyName = 'horizontalSeparatorCustomScriptPos';
		}
		localStorage.setItem(storageKeyName, proportion);
	});

	self.onResize = function (panelInfo) {
		if (panelInfo.isMinimized) {
			return;
		}

		if (self.errorPanelOpened()) {
			setErrorPanelSize();
		} else if (self.dblookupOpened()) {
			setLookupPanelSize();
		} else if (self.customScriptOpened()) {
			setCustomScriptPanelSize();
		}

		self.fieldsViewmodel.onResize();
		self.errorsViewmodel.onResize();
	};
	var previousDocId = -1;

	ko.computed(function () {
		var doc = self.selectedDocument();

		if (!doc || doc.Status() !== FC.BatchEditor.Document.Status.NeedRecognize) {
			return;
		}

		var docs = window.batchLayout.batchLayoutViewModel.Documents().filter(function (d) {
			if (d.Id() === doc.Id()) {
				return false;
			}

			return doc.Status() === FC.BatchEditor.Document.Status.NeedRecognize;
		});

		if (docs.length === 0) {
			return;
		}

		FC.Pages.VerifyPage.showRerecognizeLocker(FC.Pages.VerifyPage.formLockerModes.needRecognize);
	});

	ko.computed(function () {
		return self.selectedDocument() ? self.selectedDocument().Id() + '' + self.selectedDocument().Version() + '' +
			+ self.selectedDocument().pagesMoved() +
			(self.selectedDocument().Status() === FC.BatchEditor.Document.Status.Normal ||
				self.selectedDocument().Status() === FC.BatchEditor.Document.Status.RecognizeFinished ||
				self.selectedDocument().Status() === FC.BatchEditor.Document.Status.CheckRulesFinished ?
				FC.BatchEditor.Document.Status.Normal : self.selectedDocument().Status()) : '';

	}).subscribe(function () {
		self.openErrorPanel();
		self.documentHasChanges(false);
		var doc = self.selectedDocument();
		if (doc &&
			(doc.Status() === FC.BatchEditor.Document.Status.Normal ||
				doc.Status() === FC.BatchEditor.Document.Status.RecognizeFinished ||
				doc.Status() === FC.BatchEditor.Document.Status.CheckRulesFinished)) {
			self.readonlyMode(false);

			window.batchLayout.batchLayoutViewModel.showTwoPanelsLocker(null,
				FC.Environment.Localization.GlobalResources.DocLoadMessage,
				'');

			FC.Pages.VerifyPage.hideRerecognizeLocker();
			window.batchLayoutComponent.store.dispatch({ type: 'BATCH_LAYOUT_HIDE_LOCKER', lockerIndex: 2 });
			window.batchLayoutComponent.store.dispatch({ type: 'BATCH_LAYOUT_CANCEL_FORCED' });

			if (previousDocId !== doc.Id()) {
				self.documentData(null);
				self.fieldsViewmodel.formHorizontalScroll.windowResized(!self.fieldsViewmodel.formHorizontalScroll.windowResized());
				self.fieldsViewmodel.lastInstanceMenu = null;
				self.fieldsViewmodel.hideables([]);
				while (self.fieldsViewmodel.disposables.length > 0) {
					var disposable = self.fieldsViewmodel.disposables.pop();
					if (disposable && disposable.hasOwnProperty("dispose")) {
						disposable.dispose();
					}
				}

				$(window).off("click.formFields");
			}

			previousDocId = doc.Id();

			documentDataContext.getDocument(
				doc.TaskId(),
				doc.ProjectId(),
				doc.BatchId(),
				doc.Id(),
				doc.Version(),
				false,
				function (data) {
					data.removeClassificationError();
					if (doc.HasClassificationWarning()) {
						data.addClassificationError();
					}
					self.documentData(data);
					self.documentHasChanges(false);

					var timer = performance.now();
					var MutationObserver = window.MutationObserver || window.WebKitMutationObserver;
					var myObserver = new MutationObserver(function () { timer = performance.now(); });
					var obsConfig = { childList: true, characterData: true, attributes: true, subtree: true };
					$('#documentEditorFormViewer *[data-hash]').each(function () {
						myObserver.observe(this, obsConfig);
					});

					var intervalCheck = setInterval(function () {
						if (performance.now() - timer > 600 && intervalCheck) {
							eventManager.trigger('processingFinished');
							doc.recognitionResultUnwatched(false);
							doc.checkRulesResultUnwatched(false);
							var checkVisibleFieldExist = function (fields) {
								fields = fields || [];
								for (var i = 0; i < fields.length; i++) {
									var field = fields[i];
									if (!field.isVisible()) continue;
									if (field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Text ||
										field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Picture ||
										field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Boolean ||
										field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Choice ||
										field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.RepeatableParent ||
										field.fieldType === FC.BusinessModels.DocumentEditor.FieldTypes.Table) {
										return true;
									}
									if (field.fields) {
										if (checkVisibleFieldExist(field.fields())) {
											return true;
										}

									}
								}
								return false;
							}

							if (!checkVisibleFieldExist(data.fields())) {
								window.batchLayoutComponent.store.dispatch({ type: 'BATCH_LAYOUT_SHOW_LOCKER', lockerIndex: 2, lockerData: { noFields: true } });
							}

							myObserver.disconnect();
							clearInterval(intervalCheck);
							intervalCheck =
								null; // Если код внутри таймера выполняется медленно (например, при подключении отладчика), таймер может срабоать более одного раза. Нужно избежать этого
						}
					},
						400);
				},
				function () {
					self.loadFailed(true);
					eventManager.trigger('processingFinished');
				});
		} else {
			self.readonlyMode(true);
			window.batchLayout.batchLayoutViewModel.hideTwoPanelsLocker();
			window.batchLayoutComponent.store.dispatch({ type: 'BATCH_LAYOUT_HIDE_LOCKER', lockerIndex: 2 });
			window.batchLayoutComponent.store.dispatch({ type: 'BATCH_LAYOUT_CANCEL_FORCED' });

			var docStatus = doc ? doc.Status() : 0;
			if (docStatus === FC.BatchEditor.Document.Status.NeedRecognize && doc.pagesMoved()) {
				docStatus = FC.BatchEditor.Document.Status.NeedSave;
			}

			switch (docStatus) {
				case FC.BatchEditor.Document.Status.NeedRecognize:
					FC.Pages.VerifyPage.showRerecognizeLocker(FC.Pages.VerifyPage.formLockerModes.needRecognize);
					window.batchLayoutComponent.store.dispatch({ type: 'BATCH_LAYOUT_MAXIMIZE_PANEL', panelIndex: 2 });
					break;
				case FC.BatchEditor.Document.Status.RecognizeStarted:
					FC.Pages.VerifyPage.showRerecognizeLocker(FC.Pages.VerifyPage.formLockerModes.isRecognizing);
					window.batchLayoutComponent.store.dispatch({ type: 'BATCH_LAYOUT_MAXIMIZE_PANEL', panelIndex: 2 });
					break;
				case FC.BatchEditor.Document.Status.NeedSave:
					FC.Pages.VerifyPage.showRerecognizeLocker(FC.Pages.VerifyPage.formLockerModes.needApplyChanges);
					window.batchLayoutComponent.store.dispatch({ type: 'BATCH_LAYOUT_MAXIMIZE_PANEL', panelIndex: 2 });
					break;
				case FC.BatchEditor.Document.Status.NormalNoForm:
					FC.Pages.VerifyPage.hideRerecognizeLocker();
					window.batchLayoutComponent.store.dispatch({ type: 'BATCH_LAYOUT_SHOW_LOCKER', lockerIndex: 2 });
					window.batchLayoutComponent.store.dispatch({ type: 'BATCH_LAYOUT_MAXIMIZE_PANEL', panelIndex: 2 });
					break;

				case FC.BatchEditor.Document.Status.CheckRulesStarted:
					FC.Pages.VerifyPage.showRerecognizeLocker(FC.Pages.VerifyPage.formLockerModes.isCheckingRules);
					window.batchLayoutComponent.store.dispatch({ type: 'BATCH_LAYOUT_MAXIMIZE_PANEL', panelIndex: 2 });
					break;
				default:
					FC.Pages.VerifyPage.hideRerecognizeLocker();
					break;
			}

			if (doc && doc.Status() !== FC.BatchEditor.Document.Status.CheckRulesStarted) {
				self.documentData(null);
			}
			$(window).off("click.formFields");
		}
	});

	var runDbLookup = function (rule, searchValue, contextPath) {
		if (rule) {
			var fullText = '';
			var pages = self.regionsViewmodel.pages();
			if (pages && pages.length > 0) {
				pages.forEach(function (page) {
					var textRegions = [];
					var regions = page.textRegions();
					if (regions && regions.length > 0) {
						regions.forEach(
							function (region) {
								textRegions.push({
									text: region.caption(),
									pos: region.rectangles()[0]
								});
							});
					}
					fullText = fullText +
						' ' +
						textRegions.sort(function (left, right) {
							if (left.pos.y2 < right.pos.y1) return -1;
							if ((left.pos.y1 > right.pos.y2)) return 1;
							if (left.pos.x2 < right.pos.x1) return -1;
							if ((left.pos.x1 > right.pos.x2)) return 1;
							if (left.pos.y1 < right.pos.y1) return -1;
							if ((left.pos.y1 > right.pos.y1)) return 1;
							if (left.pos.x1 < right.pos.x1) return -1;
							if ((left.pos.x1 > right.pos.x1)) return 1;
							return 0;
						}).map(function (region) { return region.text; }).join(' ');
				});
			}

			self.errorPanelOpened(false);
			self.dblookupOpened(true);
			self.customScriptOpened(false);
			self.dblookupViewmodel.open(rule, self.documentData(), fullText, searchValue, contextPath);

		}
	};

	dependencies.eventManager.bind('docEditor_runDbLookup',
		function (e, data) {
			var nodeRef = data.nodeRef;
			var ruleName = data.ruleName;
			var repeatableContext = data.repeatableContext;
			var nodeRefArray = nodeRef.split(',').map(function (x) { return parseInt(x); });
			var docDbLookupRules = self.documentData().databaseCheckRules();
			var foundRule = null;
			//Поиск подходящего правила. Сравниватся нод реф в кнопке с путем в правиле
			for (var i = 0; i < docDbLookupRules.length; i++) {
				var rule = docDbLookupRules[i];
				if (rule.path.length !== nodeRefArray.length) continue;
				var mismatched = false;
				var j;

				for (j = 0; j < rule.path.length; j++) {
					if (rule.path[j] !== nodeRefArray[j]) {
						mismatched = true;
						break;
					}
				}

				//в старых версиях шаблонов инвертированная запись
				if (mismatched) {
					mismatched = false;
					for (j = 0; j < rule.path.length; j++) {
						if (rule.path[j] !== nodeRefArray[rule.path.length - 1 - j]) {
							mismatched = true;
							break;
						}
					}
				}

				if (!mismatched && rule.ruleName === ruleName) {
					foundRule = rule;
					break;
				}
			}
			if (foundRule) {
				runDbLookup({
					rule: foundRule,
					repeatableContext: repeatableContext
				});
			} else {
				console.error("rule not found for " + JSON.stringify({ nodeRef: nodeRef, ruleName: ruleName }));
			}

		});


	dependencies.eventManager.bind('docEditor_runDbLookupForRule', function (e, data) {
		runDbLookup({ rule: data.rule, repeatableContext: data.repeatableContext }, data.searchValue);
	});
}
