﻿window.Layouts = window.Layouts || {};

/**
 * Загрузчик ViewModel
 * @returns {} 
 */
window.Layouts.ModelLoader = (function (ko) {
	var self = {};

	self.mainViewModel = ko.observable();
	self.viewModel = ko.observable();

	self.isInitialized = false;

	self.refresher = null;

	/**
	 * Разовая инициализация загрузчика (происходит автоматом)
	 * @returns {} 
	 */
	self.init = function (dependencies) {

		FC.UI = {
			lastActivity: Date.now()
		};

		window.Layouts.RuntimeEnvironment.init(self.mainViewModel(), dependencies.serverDataContext);

		FC.Environment.Localization.Resource = FC.Environment.Localization.DataTable;

		FC.Utils.setDataTablesDefaultsSettings();

		window.Layouts.MainMasterViewModel = window.Layouts.MainMasterViewModelDef(ko, window.Layouts.datacontext);
	};

	/**
	 * Загрузить новую модель страницы
	 * @param {} pageViewModel Загружаемая модель
	 * @param {} onLoad Действие при успехе
	 * @returns {} 
	 */
	self.load = function (pageViewModel, dependencies, onLoad) {
		var page = ko.observable({ model: pageViewModel });
		self.mainViewModel(new FC.Pages.PageContext(page));

		self.viewModel(pageViewModel);

		if (!self.isInitialized) {
			self.init(dependencies);
			self.isInitialized = true;
		}

		pageViewModel.OnPageOpening(onLoad);

		$.datepicker.setDefaults($.datepicker.regional[FC.Environment.Culture]);

		if (!self.refresher && FC.Environment.StationType === 1) {
			self.refresher = setTimeout(function() {
					FC.Utils.RefreshSession(dependencies.serverDataContext);
				},
				FC.Environment.SessionRefreshInterval);
		}

		$(window).resize();
	};

	/**
	* Загрузить начальные настройки без загрузки модели
	*/
	self.loadDefault = function (dependencies) {
		var page = ko.observable({ model: null });
		self.mainViewModel(new FC.Pages.PageContext(page));

		if (!self.isInitialized) {
			self.init(dependencies);
			self.isInitialized = true;
		}

		$.datepicker.setDefaults($.datepicker.regional[FC.Environment.Culture]);

		if (!self.refresher && FC.Environment.StationType === 1) {
			self.refresher = setTimeout(function () {
				FC.Utils.RefreshSession(dependencies.serverDataContext);
			},
				FC.Environment.SessionRefreshInterval);
		}

		$(window).resize();
	};

	return {
		mainViewModel: self.mainViewModel,
		viewModel: self.viewModel,
		load: self.load,
		loadDefault: self.loadDefault
	};
})(ko);
