﻿window.Layouts = window.Layouts || {};

/**
 * Окружение для запуска станции верификации в браузере.
 * Singleton. Поддерживается только разовая инициализация
 */
window.Layouts.RuntimeEnvironment = (function () {
	var self = this;

	self.isInitialized = false;

	/**
	 * Выполнить инициализацию
	 * @param {} mainViewModel Главная VM приложения
	 * @returns {} 
	 */
	self.init = function (mainViewModel, serverDataContext) {
		if (self.isInitialized) {
			return;
		}

		try {
			var sourceAjax = $.ajax;
			$.ajax = function (params) {
				var successHandler = params.success;
				if (params.success) {
					params.success = function (result, state, xhr) {
						var newSessionId = xhr.getResponseHeader('SESSIONID');
						if (newSessionId) {
							FC.SessionInfo.SessionId = newSessionId;
						}

						successHandler(result, state, xhr);
					}
				}

				sourceAjax.call($, params);
			}

			$.ajaxSetup({
				cache: false
			});

			$.ajaxPrefilter(function (options, originalOptions) {
				if (!originalOptions.type || originalOptions.type.toUpperCase() !== 'POST' || !options.type || options.type.toUpperCase() !== 'POST') {
					options.data = $.param($.extend(originalOptions.data, { sessionId: FC.SessionInfo.SessionId }), originalOptions.traditional);
					return;
				}

				options.data = $.param($.extend(originalOptions.data, { __RequestVerificationToken: FC.Token, sessionId: FC.SessionInfo.SessionId }), originalOptions.traditional);
			});

			ko.applyBindings(mainViewModel, document.getElementById("mainContentBody"));

			var refreshTicketOnTimeout = function() {
				setTimeout(
					function() {
						FC.DataContext.Server.refreshTicket(function() {
							refreshTicketOnTimeout();
						});
					},
					900000);
			};

			refreshTicketOnTimeout();

			if (FC.Environment.StationType === FC.StationType.WebVerification) {
				$(document.body).bind("keyup",
					function(e) {
						FC.Utils.UpdateLastActivity(serverDataContext);
						//FC.Utils.SwitchDataLoggingMode(e);
					});

				$(document.body).bind("mousemove",
					function() {
						FC.Utils.UpdateLastActivity(serverDataContext);
					});

				FC.Utils.UpdateLastActivity(serverDataContext);
			}
		} finally {
			self.isInitialized = true;
		}
	}

	return {
		init: self.init
	}
})();

$(function () {
	// Для обратной совместимости обработаем хэш в адресной строке
	// Невозможно обработать на сервере, так как туда не передаётся текст после #
	var loc = FC.Utils.getLocation();
	if (loc.hash && loc.hash.length > 2) {
		var path = loc.hash;
		var query = "";
		var queryIndex = path.indexOf('?');
		if (queryIndex > -1) {
			query = path.slice(queryIndex);
			path = path.slice(0, queryIndex);
		}

		var newLocation;
		switch (path) {
			case "#/SelectProjectAndRole":
				newLocation = "";
				break;

			case "#/QueueList":
				newLocation = "QueueList";
				break;

			case "#/Verify":
				newLocation = "Verify";
				break;

			case "#/Tasks":
				newLocation = "Tasks";
				break;

			case "#/Batches":
				newLocation = "Batches";
				break;

			case "#/Documents":
				newLocation = "Documents";
				break;

		default:
			newLocation = "";
			break;
		}

		if (query.length > 0) {
			newLocation += query;
		}

		FC.Utils.replaceLocation(loc.href.replace(loc.hash, newLocation));
		return;
	}
});
