﻿FC.Viewmodels.StyleZonesFinder = {	
	findZoneForClick: function (doc, pages, x, y) {
		if (!doc) {
			return null;
		}

		// сначала проверим регионы текущего поля
		var currentField = doc.CurrentSelectedField();
		if (currentField) {
			var zone = FC.Viewmodels.StyleZonesFinder.getZoneInfoForField(currentField, x, y);
			if (zone) {
				return zone;
			}
		}


		var zoneRegions = FC.Viewmodels.StyleZonesFinder.getZoneInfoForFields(doc.Sections(), x, y);
		if (zoneRegions) {
			return zoneRegions;
		}

		return FC.Viewmodels.StyleZonesFinder.getZoneInfoForPages(pages, x, y);
	},

	getTextZonesForRectangle: function (pages, x1, y1, x2, y2) {
		if (!pages) {
			return [];
		}

		var zoneInfos = [];
		for (var pageIndex = 0; pageIndex < pages.length; pageIndex++) {
			var pageText = pages[pageIndex].PageText();
			if (!pageText) {
				continue;
			}

			var elements = pageText.TextElements();
			if (!elements) {
				continue;
			}

			for (var elemIndex = 0; elemIndex < elements.length; elemIndex++) {
				var zone = elements[elemIndex].StyleZone();
				if (zone && zone.DoesZoneCrossRectangle(x1, y1, x2, y2)) {
					zoneInfos.push({ zone: zone, textElement: elements[elemIndex] });
				}
			}
		}

		return zoneInfos;
	},

	getZoneInfoForPages: function (pages, x, y) {
		if (!pages) {
			return null;
		}

		for (var pageIndex = 0; pageIndex < pages.length; pageIndex++) {
			var pageText = pages[pageIndex].PageText();
			if (!pageText) {
				continue;
			}

			var elements = pageText.TextElements();
			if (!elements) {
				continue;
			}

			for (var elemIndex = 0; elemIndex < elements.length; elemIndex++) {
				var zone = elements[elemIndex].StyleZone();
				if (zone && zone.IsInZone(x, y)) {
					return { zone: zone, textElement: elements[elemIndex] };
				}
			}
		}

		return null;
	},

	getZoneInfoForFields: function (fields, x, y) {
		if (fields) {
			for (var i = 0; i < fields.length; i++) {
				var zone = FC.Viewmodels.StyleZonesFinder.getZoneInfoForField(fields[i], x, y);
				if (zone) {
					return zone;
				}
			}
		}

		return null;
	},

	getZoneInfoForTables: function (tables, x, y) {
		if (!tables) {
			return null;
		}

		for (var tableIndex = 0; tableIndex < tables.length; tableIndex++) {
			if (!tables[tableIndex] || tables[tableIndex].IsHidden()) {
				continue;
			}

			var rows = tables[tableIndex].Row();
			if (!rows) {
				continue;
			}

			var cellZone = null;
			for (var rowIndex = 0; rowIndex < rows.length; rowIndex++) {
				cellZone = FC.Viewmodels.StyleZonesFinder.getZoneInfoForFields(rows[rowIndex].Cell(), x, y);
				if (cellZone) {
					break;
				}
			}

			var tableBlocks = tables[tableIndex].Regions();
			if (!tableBlocks) {
				continue;
			}

			var regions = tableBlocks.Region();
			if (!regions) {
				continue;
			}

			var tableRegionsMargin = 3;

			for (var regionIndex = 0; regionIndex < regions.length; regionIndex++) {
				var styleZones = regions[regionIndex].StyleZones();

				for (var j = 0; j < styleZones.length; j++) {
					if (styleZones[j].IsNearZone(x, y, tableRegionsMargin)) {
						return { zone: styleZones[j], childZone: cellZone, region: regions[regionIndex], table: tables[tableIndex] };
					}
				}
			}

			if (cellZone) {
				return cellZone;
			}
		}

		return null;
	},

	getZoneInfoForField: function (field, x, y) {
		if (!field || field.IsHidden()) {
			return null;
		}

		// сначала смотрим дочерние регионы
		var zone = FC.Viewmodels.StyleZonesFinder.getZoneInfoForFields(field.Children().Field(), x, y);
		if (zone) {
			return zone;
		}

		zone = FC.Viewmodels.StyleZonesFinder.getZoneInfoForFields(field.Instances().Field(), x, y);
		if (zone) {
			return zone;
		}

		zone = FC.Viewmodels.StyleZonesFinder.getZoneInfoForTables(field.Children().Table(), x, y);
		if (zone) {
			return zone;
		}

		zone = FC.Viewmodels.StyleZonesFinder.getZoneInfoForTables(field.Instances().Table(), x, y);
		if (zone) {
			return zone;
		}

		// потом - текущие
		var regions = field.Regions();
		if (regions) {
			for (var i = 0; i < regions.length; i++) {
				var styleZones = regions[i].StyleZones();

				for (var j = 0; j < styleZones.length; j++) {
					if (styleZones[j].IsInZone(x, y)) {
						return { zone: styleZones[j], region: regions[i], field: field };
					}
				}
			}
		}

		return null;
	}	
};