﻿FC.Viewmodels.Validation = FC.Viewmodels.Validation || {};
/**
 * Валидатор для даты
 */
FC.Viewmodels.Validation.DatetimeValidator = function(serverValidator) {
	var self = this;

	self.serverValidator = serverValidator;

	function getDateFormatsForComponents(dateSeparators, firstComponent, secondComponent, thirdComponent, mayIncludeDayOfWeek, mayIncludeMonthName, onlyWithMonthNames) {
		
		var componentsReplacementFunc = function (separator, replacement) {
			var componentReplacer = function(component) {
				return component.indexOf("M") >= 0 ? replacement : component;
			}
			return componentReplacer(firstComponent) +
				separator +
				componentReplacer(secondComponent) +
				separator +
				componentReplacer(thirdComponent);
		}

		var dateStringFormats = [];
		$.each(dateSeparators,
			function(ind, dateSeparator) {
				if (!onlyWithMonthNames) {
					var format = firstComponent + dateSeparator + secondComponent + dateSeparator + thirdComponent;
					dateStringFormats.push(format);

					if (mayIncludeDayOfWeek) {
						dateStringFormats.push("dddd " + format);
					}
				}

				if (mayIncludeMonthName)
				{
					// короткая форма месяцев - три M
					var monthNameFormat = componentsReplacementFunc(dateSeparator, "MMM");

					dateStringFormats.push(monthNameFormat);
					if (mayIncludeDayOfWeek)
					{
						dateStringFormats.push("dddd " + monthNameFormat);
					}

					// длинная форма месяцев - четыре M
					var monthNameLongFormat = componentsReplacementFunc(dateSeparator, "MMMM");

					dateStringFormats.push(monthNameLongFormat);
					if (mayIncludeDayOfWeek)
					{
						dateStringFormats.push("dddd " + monthNameLongFormat);
					}
				}
			});


		return dateStringFormats;
	}

	function getDateFormats(dateSeparators, componentsOrder, mayIncludeDayOfWeek, mayIncludeMonthName, onlyMonthNames) {
		switch (componentsOrder) {
			case 0:
				return getDateFormatsForComponents(dateSeparators,
						"D",
						"M",
						"YY",
						mayIncludeDayOfWeek,
						mayIncludeMonthName,
						onlyMonthNames)
					.concat(getDateFormatsForComponents(dateSeparators,
						"D",
						"M",
						"YYYY",
						mayIncludeDayOfWeek,
						mayIncludeMonthName,
						onlyMonthNames));
				
			case 1:
				return getDateFormatsForComponents(dateSeparators,
						"M",
						"D",
						"YY",
						mayIncludeDayOfWeek,
						mayIncludeMonthName,
						onlyMonthNames)
					.concat(getDateFormatsForComponents(dateSeparators,
						"M",
						"D",
						"YYYY",
						mayIncludeDayOfWeek,
						mayIncludeMonthName,
						onlyMonthNames));
			case 2:
				return getDateFormatsForComponents(dateSeparators,
						"YY",
						"M",
						"D",
						mayIncludeDayOfWeek,
						mayIncludeMonthName,
						onlyMonthNames)
					.concat(getDateFormatsForComponents(dateSeparators,
						"YYYY",
						"M",
						"D",
						mayIncludeDayOfWeek,
						mayIncludeMonthName,
						onlyMonthNames));
			case 3:
				return getDateFormatsForComponents(dateSeparators,
						"YY",
						"D",
						"M",
						mayIncludeDayOfWeek,
						mayIncludeMonthName,
						onlyMonthNames)
					.concat(getDateFormatsForComponents(dateSeparators,
						"YYYY",
						"D",
						"M",
						mayIncludeDayOfWeek,
						mayIncludeMonthName,
						onlyMonthNames));
			default:
				return [];
		}
	}

	function getTimeFormats(timeSeparators) {
		var formats = [];
		$.each(timeSeparators, function(ind, timeSeparator) {
			formats.push("HH"+timeSeparator+"mm"+timeSeparator+"ss");
			formats.push("hh"+timeSeparator+"mm"+timeSeparator+"ss a");
			formats.push("HH"+timeSeparator+"mm");
			formats.push("hh"+timeSeparator+"mm a");
		});

		return formats;
	}

	function getDatetimeFormats(format) {
		if (!format) return [];
		if (!format.RecordFormat ||
			format.RecordFormat.length === 0 ||
			!format.RecordFormat[0].Languages ||
			!format.RecordFormat[0].Languages.Language ||
			format.RecordFormat[0].Languages.Language.length === 0) {
			return [];
		}
		var recordFormat = format.RecordFormat[0];
		if (!recordFormat.IncludeDate && !recordFormat.IncludeTime) {
			// ничего им не нужно
			return [];
		}

		var result = [];
		var dateSeparators = ["/", "\\\\", ".", "-", " ", ":"];
		var timeSeparators = [":", "/", "\\\\", ".", "-", ":"];
		$.each(format.RecordFormat[0].Languages.Language,
			function(ind, lcid) {
				if (lcid < 0) {
					// не может быть отрицательным
					return;
				}

				if (!recordFormat.IncludeDate && recordFormat.IncludeTime) {
					$.each(getTimeFormats(timeSeparators),
						function(i, tf) {
							result.push({
								Format: tf,
								Lcid: lcid
							});
						});
					return;
				}


				var dateFormats = [];
				var generateDateRange = function(componentOrder, mayIncludeMonthName, onlyWithMonthNames) {
					$.each(getDateFormats(dateSeparators,
							componentOrder,
							recordFormat.MayIncludeDayOfWeek,
							mayIncludeMonthName,
							onlyWithMonthNames),
						function(i, df) {
							dateFormats.push(df);
						});
				}


				generateDateRange(recordFormat.DateComponentsOrder, false, false);
				generateDateRange(0, true, true);
				generateDateRange(1, true, true);
				generateDateRange(2, true, false);
				generateDateRange(recordFormat.SecondDateComponentsOrder, false, false);
				generateDateRange(recordFormat.ThirdDateComponentsOrder, false, false);

				$.each(dateFormats,
					function(i, df) {
						result.push({
							Format: df,
							Lcid: lcid
						});
					});


				$.each(getTimeFormats(timeSeparators),
					function(i, tf) {
						$.each(dateFormats,
							function(i, df) {
								result.push({
									Format: df + ' ' + tf,
									Lcid: lcid
								});
							});
					});

			});

		return result;

	}

	if (!self.serverValidator.PossibleFormats) {
		self.serverValidator.PossibleFormats = getDatetimeFormats(self.serverValidator.DateTimeFormat);
	}

	/**
	 * Валидоровать дату.
	 * возвращает null если текст является датой
	 * или объект с полем error, в котором содержится текст ошибки, если не дата.
	 * Валидация проходит с помощью плагина moment.js
	 */
	self.Validate = function (text) {

		// если пришёл валидатор другого типа или текст пустой - не проверяем
		if (!text || !self.serverValidator || self.serverValidator.ValidatorType !== 'DatetimeValidator') {
			return null;
		}

		// если с сервера не пришли возможные форматы даты - считаем, что всё ок.
		if (!self.serverValidator.PossibleFormats || self.serverValidator.PossibleFormats.length === 0) {
			// не можем валидировать - пусть на сервере разбираются
			return null;
		}

		// ищем из всех возможных форматов хоть один, которым можно валидировать текст как дату
		for (var i = 0; i < self.serverValidator.PossibleFormats.length; i++) {
			var currentFormat = self.serverValidator.PossibleFormats[i];

			if (!currentFormat) {
				continue;
			}

			// задаём локаль для moment.js
			moment.locale(self._getMomentLocaleFromLcid(currentFormat.Lcid));
			var date = moment(text, currentFormat.Format, true);

			// если текст не валидировался в формат - пробуем следующий
			if (!date.isValid()) {
				continue;
			}

			var minAvailableDate = moment({ y: 1601, M: 0, d: 1, h: 0, m: 0, s: 0, ms: 0 });
			var maxAvailableDate = moment({ y: 3000, M: 0, d: 1, h: 0, m: 0, s: 0, ms: 0 });
			if (date < minAvailableDate || date >= maxAvailableDate) {
				break;
			}

			if (currentFormat.Format.indexOf('dddd') >= 0) {
				var textUpper = text.toUpperCase();
				var check = date.format(currentFormat.Format).toUpperCase() === textUpper;
				if (!check) check = date.format(currentFormat.Format.replace("D", "DD")).toUpperCase() === textUpper;
				if (!check) check = date.format(currentFormat.Format.replace("M", "MM")).toUpperCase() === textUpper;
				if (!check) check = date.format(currentFormat.Format.replace("D", "DD").replace("M", "MM")).toUpperCase() === textUpper;

				if(!check) continue;

			}
			// если не нужно проверять интервал - выходим
			if (!self.serverValidator.CheckInterval) {
				// всё ок
				return null;
			}

			// проверяем интервал
			var dateMin = moment(self.serverValidator.MinValue);
			var dateMax = moment(self.serverValidator.MaxValue);
			if (dateMin >= dateMax) {
				return null;
			}

			// thai
			if (currentFormat.Lcid == 1054) {
				date.subtract(543, 'years');
			}

			if (date >= dateMin && date <= dateMax) {
				return null;
			}

			// если сюда попали - значит не свалидировался интервал
			// thai
			if (currentFormat.Lcid == 1054) {
				return {
					error: FC.Environment.Localization.GlobalResources.FormatError_DateInterval
						.replace("{0}", moment(date.add(543, 'years')).format(currentFormat.Format))
						.replace("{1}", moment(dateMin.add(543, 'years')).format(currentFormat.Format))
						.replace("{2}", moment(dateMax.add(543, 'years')).format(currentFormat.Format))
				};
			}

			return {
				error: FC.Environment.Localization.GlobalResources.FormatError_DateInterval
					.replace("{0}", moment(date).format(currentFormat.Format))
					.replace("{1}", moment(dateMin).format(currentFormat.Format))
					.replace("{2}", moment(dateMax).format(currentFormat.Format))
			};
		}

		// ни один формат не подошёл - нужно вернуть стандартную ошибку
		// скажем, что подошёл первый формат (он как бы основной)
		var presentationFormat = self.serverValidator.PossibleFormats[0].Format.replace(/D+/, 'DD').replace(/M+/, 'MM');
		var basicError = FC.Environment.Localization.GlobalResources.FormatErrorDateTimeTypeMismatch.replace('%0', presentationFormat);
		return { error: basicError };
	};

	/**
	 * Получить название локали для moment.js из LCID
	 */
	self._getMomentLocaleFromLcid = function (lcid) {
		switch (lcid) {
			case 54:
			case 1078:
				return 'af';
			case 6145:
				return 'ar-ma';
			case 1025:
				return 'ar-sa';
			case 7169:
				return 'ar-tn';
			case 1:
				return 'ar';
			case 44:
			case 29740:
			case 2092:
			case 30764:
			case 1068:
				return 'az';
			case 35:
			case 1059:
				return 'be';
			case 2:
			case 1026:
				return 'bg';
			case 69:
			case 2117:
			case 1093:
				return 'bn';
			case 81:
			case 1105:
				return 'bo';
			case 126:
			case 1150:
				return 'br';
			case 30746:
			case 25626:
			case 8218:
			case 26650:
			case 5146:
				return 'bs';
			case 3:
			case 1027:
				return 'ca';
			case 5:
			case 1029:
				return 'cs';
			case 82:
			case 1106:
				return 'cy';
			case 6:
			case 1030:
				return 'da';
			case 3079:
				return 'de-at';
			case 7:
			case 2055:
			case 1031:
			case 5127:
			case 4103:
				return 'de';
			case 101:
			case 1125:
				return 'dv';
			case 8:
			case 1032:
				return 'el';
			case 3081:
				return 'en-au';
			case 4105:
				return 'en-ca';
			case 2057:
			case 9:
			case 1033:
				return 'en-gb';
			case 6153:
				return 'en-ie';
			case 5129:
				return 'en-nz';
			case 10:
			case 11274:
			case 16394:
			case 13322:
			case 9226:
			case 5130:
			case 7178:
			case 12298:
			case 3082:
			case 4106:
			case 18442:
			case 2058:
			case 19466:
			case 6154:
			case 10250:
			case 20490:
			case 15370:
			case 17418:
			case 21514:
			case 14346:
			case 8202
			:
				return 'es';
			case 37:
			case 1061:
				return 'et';
			case 45:
			case 1069:
				return 'eu';
			case 41:
			case 1065:
				return 'fa';
			case 11:
			case 1035:
				return 'fi';
			case 56:
			case 1080:
				return 'fo';
			case 3084:
				return 'fr-ca';
			case 4108:
				return 'fr-ch';
			case 12:
			case 2060:
			case 1036:
			case 5132:
			case 6156:
				return 'fr';
			case 98:
			case 1122:
				return 'fy';
			case 145:
			case 1169:
				return 'gd';
			case 86:
			case 1110:
				return 'gl';
			case 13:
			case 1037:
				return 'he';
			case 57:
			case 1081:
				return 'hi';
			case 26:
			case 4122:
			case 1050:
				return 'hr';
			case 14:
			case 1038:
				return 'hu';
			case 43:
			case 1067:
				return 'hy-am';
			case 33:
			case 1057:
				return 'id';
			case 15:
			case 1039:
				return 'is';
			case 16:
			case 2064:
			case 1040:
				return 'it';
			case 17:
			case 1041:
				return 'ja';
			case 55:
			case 1079:
				return 'ka';
			case 63:
			case 1087:
				return 'kk';
			case 83:
			case 1107:
				return 'km';
			case 18:
			case 1042:
				return 'ko';
			case 64:
			case 1088:
				return 'ky';
			case 110:
			case 1134:
				return 'lb';
			case 84:
			case 1108:
				return 'lo';
			case 39:
			case 1063:
				return 'lt';
			case 38:
			case 1062:
				return 'lv';
			case 47:
			case 1071:
				return 'mk';
			case 76:
			case 1100:
				return 'ml';
			case 78:
			case 1102:
				return 'mr';
			case 1086:
				return 'ms-my';
			case 62:
			case 2110:
				return 'ms';
			case 31764:
			case 1044:
				return 'nb';
			case 97:
			case 1121:
				return 'ne';
			case 19:
			case 2067:
			case 1043:
				return 'nl';
			case 30740:
			case 2068:
				return 'nn';
			case 70:
			case 1094:
				return 'pa-in';
			case 21:
			case 1045:
				return 'pl';
			case 1046:
				return 'pt-br';
			case 22:
			case 2070:
				return 'pt';
			case 24:
			case 1048:
				return 'ro';
			case 25:
			case 1049:
				return 'ru';
			case 59:
			case 3131:
			case 1083:
			case 2107:
				return 'se';
			case 91:
			case 1115:
				return 'si';
			case 27:
			case 1051:
				return 'sk';
			case 36:
			case 1060:
				return 'sl';
			case 28:
			case 1052:
				return 'sq';
			case 27674:
			case 7194:
			case 3098:
			case 12314:
			case 10266:
				return 'sr-cyrl';
			case 31770:
			case 28698:
			case 6170:
			case 2074:
			case 11290:
			case 9242:
				return 'sr';
			case 29:
			case 2077:
			case 1053:
				return 'sv';
			case 65:
			case 1089:
				return 'sw';
			case 73:
			case 1097:
				return 'ta';
			case 74:
			case 1098:
				return 'te';
			case 30:
			case 1054:
				return 'th';
			case 31:
			case 1055:
				return 'tr';
			case 31839:
			case 2143:
				return 'tzm-latn';
			case 95:
				return 'tzm';
			case 34:
			case 1058:
				return 'uk';
			case 67:
			case 30787:
			case 2115:
			case 31811:
			case 1091:
				return 'uz';
			case 42:
			case 1066:
				return 'vi';
			case 4:
			case 30724:
			case 31748:
			case 2052:
			case 3076:
			case 5124:
			case 4100:
				return 'zh-cn';
			case 1028:
				return 'zh-tw';
			default:
				return 'en-gb';
		}
	};
};