﻿FC.Viewmodels.Validation = FC.Viewmodels.Validation || {};
/**
 * Валидация поля
 */
FC.Viewmodels.Validation.FieldValidation = {

	/**
	 * Валидировать поле field
	 * если поле валидировалось, вызвать onSuccess
	 * если нет - onError
	 */
	Validate: function (fieldValidator, value, onError, onSuccess) {

		// если нет поля или серверных параметров валидатора
		// или не задано поведение при ошибке - то нет смысла валидировать
		if (!fieldValidator || !onError) {
			return;
		}
		

		// если текста нет, то смотрим настройку CanBeEmpty
		if (!value || value.length === 0) {
			if (fieldValidator.CanBeEmpty) {
			    // не проверяем дальше - пустое же значение
			    onSuccess();
				return;
			}

			onError(FC.Environment.Localization.GlobalResources.FieldValidation_ValueCannotBeEmpty, !fieldValidator.TreatErrorsAsWarnings);
			return;
		}

		// Создаём валидатор нужного там типа
		var validator = null;
		switch (fieldValidator.ValidatorType) {
			case 'NumberValidator':
				validator = new FC.Viewmodels.Validation.NumberValidator(fieldValidator);
				break;
			case 'CurrencyValidator':
				validator = new FC.Viewmodels.Validation.CurrencyValidator(fieldValidator);
				break;
			case 'DatetimeValidator':
				validator = new FC.Viewmodels.Validation.DatetimeValidator(fieldValidator);
				break;
		}
		
		if (validator) {
			// Валидируем поле
			var result = validator.Validate(value);

			// если пришел результат - то поле не провалидировалось, null - это успех
			if (result) {

				// текст ошибки по умолчанию
				var errorMessage = FC.Environment.Localization.GlobalResources.FormatError;

				// если пришёл специфический текст ошибки - вернём его
				if (result.hasOwnProperty('error') && result.error) {
					errorMessage = result.error;
				}

				// возвращаем ошибку и её тип - error или warning
				onError(errorMessage, !fieldValidator.TreatErrorsAsWarnings);
			} else {

				// поле успешно провалидировалось
				onSuccess();
			}
		}
	}
};