﻿window.batchApp = window.batchApp || {};
FC.BatchEditor = FC.BatchEditor || {};
FC.BatchEditor.batchEditorDataContext = FC.BatchEditor.batchEditorDataContext || {};

window.batchApp.extensions = (function () {


    FC.BatchEditor.batchEditorResources = {
        projectTitle: function () { return FC.Environment.Localization.SelectProjectAndRole_aspx.ProjectPromptLabel_Text; },
        unprocessedTemplate: function () { return FC.Environment.Localization.GlobalResources.UnprocessedTemplate; },
        undo: function () { return FC.Environment.Localization.GlobalResources.UndoButton; },
        redo: function () { return FC.Environment.Localization.GlobalResources.RedoButton; },
        split: function () { return FC.Environment.Localization.GlobalResources.SplitButton; },
        cancelSplitting: function () { return FC.Environment.Localization.GlobalResources.CancelSplitting; },
        merge: function () { return FC.Environment.Localization.GlobalResources.MergeButton; },
        deleteButton: function () { return FC.Environment.Localization.GlobalResources.DeleteButton; },
        pageSerializationType: 'Abbyy.FlexiCapture.Verification.BusinessModels.Page, Abbyy.FlexiCapture.Verification',
        documentSerializationType: 'Abbyy.FlexiCapture.Verification.BusinessModels.Document, Abbyy.FlexiCapture.Verification',
        countErrors: function () { return FC.Environment.Localization.GlobalResources.TotalErrors; },
        countWarnings: function () { return FC.Environment.Localization.GlobalResources.TotalWarnings; }
    };

    FC.BatchEditor.batchEditorDataContext.getThumblainUrl = function (parentId, id, version, angle) {
        var batchId = FC.BatchEditor.batchEditorDataContext.batchId();

        if (batchId != 0) {
            return FC.Utils.getControllerUrl("DocumentPage/GetPageThumbnail?" +
                "projectId=" + window.Layouts.ModelLoader.viewModel().projectId() +
                "&batchId=" + FC.BatchEditor.batchEditorDataContext.batchId() +
                "&documentId=" + parentId + "&pageId=" + id + "&pageVersion=" + version + "&angle=" + angle +
                "&sessionId=" + FC.SessionInfo.SessionId);
        }

        return '';
    };

    FC.BatchEditor.batchEditorDataContext.getProjectName = function () {
        if (window.Layouts.ModelLoader &&
            window.Layouts.ModelLoader.viewModel() &&
            window.Layouts.ModelLoader.hasOwnProperty('ProjectName')) {
            return window.Layouts.ModelLoader.ProjectName();
        }

        return '';
    };

    FC.BatchEditor.batchEditorDataContext.batchId = function () {
        if (window.Layouts.ModelLoader &&
            window.Layouts.ModelLoader.viewModel() &&
            window.Layouts.ModelLoader.viewModel().hasOwnProperty('BatchId')) {
            return window.Layouts.ModelLoader.viewModel().BatchId();
        }

        return 0;
    };

    FC.BatchEditor.batchEditorDataContext.ImageLoadMode = ko.observable(false);
    FC.DataContext.Server.getImagesColorMode(FC.BatchEditor.batchEditorDataContext.ImageLoadMode);


    var docEditorBottomPanelVisible = ko.observable(false);

    var deleteSeparatorActivated = ko.observable(false);
    var verticalSeparatorDrawActivated = ko.observable(false);
    var horizontalSeparatorDrawActivated = ko.observable(false);

    var tableEditing = ko.observable(false);
    var selectedTableRow = ko.observable(-1);
    var selectedTableRowPath = ko.observable("");
    var selectedTableCanReanalyze = ko.observable(false);

    var hasChanges = ko.computed(function () {
        if (window.Layouts.ModelLoader.viewModel() &&
            window.Layouts.ModelLoader.viewModel().documentsMoved &&
            window.Layouts.ModelLoader.viewModel().documentsMoved()) {
            return true;
        }

        if (!window.batchLayout) {
            return false;
        }

        if (window.batchLayout.batchLayoutViewModel) {
            var be = window.batchLayout.batchLayoutViewModel.BatchEditor;

            if (be) {
                var docs = be.elements();
                var deletedDocs = be.deletedDocuments();
                if (!docs && !deletedDocs) {
                    return false;
                }

                if (deletedDocs.length > 0) {
                    return true;
                }

                for (var i = 0; i < docs.length; i++) {
                    if (docs[i].Model.HasChanges()) {
                        return true;
                    }
                }
            }
        }

        return window.batchLayout.batchLayoutViewModel &&
            window.batchLayout.batchLayoutViewModel.UserEditor &&
            window.batchLayout.batchLayoutViewModel.UserEditor.documentHasChanges();
    });

    var documentStatusChanged = function () {
        if (!window.batchLayout ||
            !window.batchLayout.batchLayoutViewModel ||
            !window.batchLayout.batchLayoutViewModel.BatchEditor) {
            return false;
        }

        var docs = window.batchLayout.batchLayoutViewModel.BatchEditor.elements();
        if (!docs) {
            return false;
        }

        for (var i = 0; i < docs.length; i++) {
            if (docs[i].Model.Status() !== FC.BatchEditor.Document.Status.Normal && docs[i].Model.Status() !== FC.BatchEditor.Document.Status.NormalNoForm) {
                return true;
            }
        }

        return false;
    };

    var hasChangesEver = ko.observable(false);
    hasChanges.subscribe(function (val) {
        if (val) {
            hasChangesEver(true);
        }
    });

    var autoGetMode = ko.observable(false);

    autoGetMode.subscribe(function (val) {
        if (window.Layouts.ModelLoader.viewModel()) {
            FC.DataContext.Project.setAutoGetTask(window.Layouts.ModelLoader.viewModel().projectId(), val);
        }
    });

    var optionCodes = { BlackAndWhiteImages: 0 };
    var commandsCodes = { DeleteSeparator: 0, AddVerticalSeparator: 1, AddHorizontalSeparator: 2, ContinueLineItems: 3, ReanalizeLineItems: 4 };

    var validateValueBySuggests = function (value, suggests) {
        if (!value) {
            return false;
        }

        if (!suggests || suggests.length === 0) {
            return true;
        }

        for (var i = 0; i < suggests.length; i++) {
            if (value.toUpperCase() === suggests[i]) {
                return true;
            }
        }

        return false;
    };

    var checkCountryCode = function () {
        var section = window.batchLayout.batchLayoutViewModel.UserEditor.documentData().fields()[0];
        var bu = section.fields().filter(function (f) { return f.name() === "BU"; });
        var vendor = section.fields().filter(function (f) { return f.name() === "Vendor"; });

        if (bu.length === 0 || vendor.length === 0) {
            return true;
        }

        var buCountry = bu[0].fields().filter(function (f) { return f.name() === "Country"; });
        var vendorCountry = vendor[0].fields().filter(function (f) { return f.name() === "Country"; });
        var fieldsPanel = window.batchLayout.batchLayoutViewModel.UserEditor.fieldsViewmodel;

        if (buCountry.length !== 0 &&
            !validateValueBySuggests(buCountry[0].value(), buCountry[0].suggests())) {

            FC.Utils.Notification.AlertMini(
                FC.Environment.Localization.GlobalResources.EnterValidBUCountryCode,
                "",
                FC.Environment.Localization.GlobalResources.OkButton,
                function () { fieldsPanel.selectedField(buCountry[0]); });

            return false;
        }

        if (vendorCountry.length !== 0 &&
            !validateValueBySuggests(vendorCountry[0].value(), vendorCountry[0].suggests())) {

            FC.Utils.Notification.AlertMini(
                FC.Environment.Localization.GlobalResources.EnterValidVendorCountryCode,
                "",
                FC.Environment.Localization.GlobalResources.OkButton,
                function () { fieldsPanel.selectedField(vendorCountry[0]); });
            return false;
        }

        return true;
    }

    function openDocument(openedDocIndex) {
        var docViewmodels = window.batchLayout.batchLayoutViewModel.BatchEditor.elements();
        docViewmodels[openedDocIndex].selectDocument(true);
        // скроллим к документу
        window.batchApp.extensions.openDocument(docViewmodels[openedDocIndex].Model);
    }

    var generalControlsEnabled = ko.computed(function () {
        if (!window.Layouts.ModelLoader.viewModel() || !window.batchLayout) {
            return true;
        }

        var docViewmodels = window.batchLayout.batchLayoutViewModel.BatchEditor.elements();
        if (!docViewmodels || docViewmodels.length === 0) {
            return true;
        }

        return docViewmodels.filter(function (d) {
            return d.Model.Status() === FC.BatchEditor.Document.Status.RecognizeStarted ||
                d.Model.Status() === FC.BatchEditor.Document.Status.CheckRulesStarted;
        }).length === 0;
    });

    var extender = {
        resources: function () {
            return $.extend(FC.Environment.Localization.GlobalResources, FC.Environment.Localization.NonLocalizedHotKeys);
        },

        onDocumentCreating: function (docPosition, batchId, isImplicit, callback, donor) {
            var document = new FC.BatchEditor.Document({
                BatchId: batchId,
                Id: FC.BatchEditor.NewDocumentCounter.getId(),
                TemplateName: donor ? donor.TemplateName() : FC.BatchEditor.batchEditorResources.unprocessedTemplate(),
                ProjectId: window.Layouts.ModelLoader.viewModel().projectId(),
                TaskId: window.Layouts.ModelLoader.viewModel().taskId()
            });
            document.IsNew(true);

            if (callback) {
                callback(document);
            }
        },

        openDocument: function (doc, pageId, docIndex, ignoreChanges) {

            pageId = isNaN(pageId) ? -1 : pageId;
            Sammy.apps.body.trigger('showDocumentEditor');

            var openNewDoc = function (hasForm) {
                window.batchApp.extensions.docEditorBottomPanelVisible(false);

                if (hasForm) {
                    window.batchLayout.batchLayoutViewModel.showTwoPanelsLocker(null,
                        FC.Environment.Localization.GlobalResources.DocLoadMessage,
                        '');
                }

                setTimeout(function () {
                    var allDocuments = window.batchLayout.batchLayoutViewModel.BatchEditor.elements();
                    for (var i = 0; i < allDocuments.length; i++) {
                        if (allDocuments[i].Model._id === doc._id) {
                            allDocuments[i].selectDocument(true);

                            // разворачиваем документ, если он ещё не был развёрнут
                            if (!allDocuments[i].detailsVisible()) {
                                allDocuments[i].toogleDetails();
                            }

                            // скроллим к документу
                            window.batchLayout.batchLayoutViewModel.BatchEditor.scrollToDocument(i, 0);
                            break;
                        }
                    }

                    window.batchLayout.batchLayoutViewModel.SelectedDocument(doc);
                    window.batchLayout.batchLayoutViewModel.SelectedPage(doc.ChildNodes()
                        .filter(function (val) { return pageId < 0 || val.Id() === pageId })[0]);
                },
                    10);
            };

            var hasForm = doc.Status() === FC.BatchEditor.Document.Status.CheckRulesFinished ||
                doc.Status() === FC.BatchEditor.Document.Status.Normal ||
                doc.Status() === FC.BatchEditor.Document.Status.RecognizeFinished;

            if (window.batchLayout.batchLayoutViewModel.UserEditor.selectedDocument() == doc) {
                window.batchLayout.batchLayoutViewModel.SelectedPage(doc.ChildNodes().filter(function (val) {
                    return pageId < 0 || val.Id() === pageId
                })[0]);
            } else if (window.batchLayout.batchLayoutViewModel.UserEditor.documentHasChanges()) {

                FC.Pages.VerifyPage.showDocumentUnsavedConfirm(
                    window.batchLayout.batchLayoutViewModel.UserEditor.selectedDocument().TemplateName(),
                    function () {
                        window.batchLayout.batchLayoutViewModel.UserEditor.checkRules(false);
                        openNewDoc(hasForm);
                    },
                    function () {
                        FC.DataContext.Document.removeDocumentFromCache(
                            window.batchLayout.batchLayoutViewModel.UserEditor.selectedDocument().TaskId(),
                            window.batchLayout.batchLayoutViewModel.UserEditor.selectedDocument().Id());

                        openNewDoc(hasForm);
                    });
            } else {
                openNewDoc(hasForm);
            }
        },

        gotoNextDocument: function () {
            if (!window.batchLayout.batchLayoutViewModel.Documents()) {
                return;
            }
            var docViewmodels = window.batchLayout.batchLayoutViewModel.BatchEditor.elements();

            var openedDocIndex = 0;
            for (var i = 0; i < docViewmodels.length; i++) {
                if (window.batchLayout.batchLayoutViewModel.UserEditor.selectedDocument() === docViewmodels[i].Model) {
                    if (i !== (docViewmodels.length - 1)) {
                        openedDocIndex = i + 1;
                    } else {
                        openedDocIndex = 0;
                    }
                }
            }

            openDocument(openedDocIndex);
        },

        gotoPreviousDocument: function () {
            if (!window.batchLayout.batchLayoutViewModel.Documents()) {
                return;
            }
            var docs = window.batchLayout.batchLayoutViewModel.Documents();
            var openedDocIndex = 0;
            for (var i = 0; i < docs.length; i++) {
                if (window.batchLayout.batchLayoutViewModel.UserEditor.selectedDocument() === docs[i]) {
                    if (i !== 0) {
                        openedDocIndex = i - 1;
                    } else {
                        openedDocIndex = docs.length - 1;
                    }
                }
            }

            openDocument(openedDocIndex);
        },


        beforePagesDeleting: function () {
            return true;
        },

        afterDocumentDeleting: function (docModel) {
            var currentDoc = window.batchLayout.batchLayoutViewModel.SelectedDocument();
            if (!currentDoc || currentDoc.Id() != docModel.Id()) {
                return;
            }
            window.batchLayout.batchLayoutViewModel.SelectedDocument(null);
            window.batchLayout.batchLayoutViewModel.SelectedPage(null);

        },

        getAdditionalContextMenuItems: function (menuType, context) {
            return FC.Viewmodels.BatchEditorExtention.getAdditionalContextMenuItems(menuType, context);
        },

        recognizeDocuments: function (docs) {
            return FC.Viewmodels.BatchEditorExtention.recognizedDocuments(docs);
        },

        recognizeSelectedDocuments: function () {
            return FC.Viewmodels.BatchEditorExtention.recognizeSelectedDocuments();
        },

        recognizeOpenedDocument: function () {
            return FC.Viewmodels.BatchEditorExtention.recognizedDocuments([window.batchLayout.batchLayoutViewModel.SelectedDocument()]);
        },

        recognizeAllDocuments: function () {
            var docsToRecognize = window.batchLayout.batchLayoutViewModel.Documents().filter(function (doc) {
                return doc.Status() === FC.BatchEditor.Document.Status.NeedRecognize;
            });

            return FC.Viewmodels.BatchEditorExtention.recognizedDocuments(docsToRecognize);
        },

        cancelDocumentRecognition: function () {
            window.batchLayout.batchLayoutViewModel.SelectedDocument().revertTemplateChanges();
            //return FC.Viewmodels.BatchEditorExtention.recognizedDocuments(docs);
        },

        getDocumentToolbarItems: function () {
            return FC.Viewmodels.BatchEditorExtention.getDocumentToolbarItems();
        },

        getPageToolbarItems: function () {
            return FC.Viewmodels.BatchEditorExtention.getPageToolbarItems();
        },

        makeDocumentsSnapshot: function () {
            var updateFunc = function (newValue) {
                window.Layouts.ModelLoader.viewModel().documents(ko.utils.arrayMap(newValue,
                    function (item) {
                        return new FC.BatchEditor.Document(item);
                    }));
            };

            var getCurrentValue = function () {
                return FC.BatchEditor.DocumentAdapter.documentsToJs(window.Layouts.ModelLoader.viewModel().batchLayout.Documents());
            };

            UndoEngine.MakeSnapshot([new UndoEngine.Undoable(updateFunc, getCurrentValue, getCurrentValue())]);
        },

        undo: function () {
            UndoEngine.Undo();
        },

        redo: function () {
            UndoEngine.Redo();
        },

        afterDocumentMoving: function () {
            window.Layouts.ModelLoader.viewModel().documentsMoved(true);
        },

        // #region ErrorNavigation

        searchAlgos: {
            STANDART: 1, //поиск по ошибкам и ворнингам
            UNCERTAIN: 2, //поиск по неуверенным символам
            BOTH: 3, //поиск по всему
            CLASSIFICATION: 4 //поиск по ошибкам классификации
        },
        // перейти к следующей\предыдущей ошибке сборки
        navigateToError: function (direction, searchAlgo) {
            if (!searchAlgo) {
                searchAlgo = this.searchAlgos.STANDART;
            }

            if (!direction) {
                return false;
            }

            var increment = 1;

            if (direction < 0) {
                increment = -1;
            }

            if (!window.batchLayout) {
                return false;
            }

            var docs = window.batchLayout.batchLayoutViewModel.BatchEditor.elements();

            // нет документов - выходим
            if (!docs || docs.length === 0) {
                return false;
            }

            // находим выделенный документ - текущая ошибка в нём
            var currentIndex = -1;

            var i;
            var openedDocument = window.batchLayout.batchLayoutViewModel.SelectedDocument();
            if (openedDocument) {
                for (i = 0; i < docs.length; i++) {
                    if (openedDocument._id === docs[i].Model._id) {
                        currentIndex = i;
                        break;
                    }
                }
            }

            if (currentIndex === -1 && increment < 0) {
                i = docs.length;
            } else {
                i = currentIndex;
            }
            var startedFromBeginning = false;
            var stopCriteria;
            var docNotFound = false;
            do {
                i += increment;

                if (increment > 0 && i == docs.length) {
                    i = 0;
                    startedFromBeginning = true;
                }

                if (increment < 0 && i < 0) {
                    i = docs.length - 1;
                    startedFromBeginning = true;
                }

                //Дошли до документа с которого начали поиск, или дошли до конца когда нужных документов не оказалось вообще
                stopCriteria = i === currentIndex ||
                    (startedFromBeginning && currentIndex === -1 && ((i === 0 && increment > 0) || (i === docs.length - 1 && increment < 0)));


                docNotFound = stopCriteria;

                //Проверяем наличие неуверенных символов для алгоритма поиска по неуверенным символам и по смешанному
                if (!stopCriteria && (searchAlgo === this.searchAlgos.UNCERTAIN || searchAlgo === this.searchAlgos.BOTH)) {
                    stopCriteria = docs[i].Model.VerificationSymbolsCount() > 0;
                }

                //Проверяем наличие ошибок классификации для всех алгоритмов кроме поиска по неуверенным символам
                if (!stopCriteria && searchAlgo !== this.searchAlgos.UNCERTAIN) {
                    stopCriteria = docs[i].Model.HasClassificationWarning();
                }

                //Проверяем наличие неуверенных символов для алгоритма поиска по ошибкам и по смешанному
                if (!stopCriteria && (searchAlgo === this.searchAlgos.STANDART || searchAlgo === this.searchAlgos.BOTH)) {
                    stopCriteria = docs[i].Model.AssemblingErrors().length !== 0 || docs[i].Model.FormErrors().length !== 0;
                }

            } while (!stopCriteria);

            // если дошли до конца - документы не выделены

            if (docNotFound) {
                return false;
            }

            docs[i].selectDocument(true);

            // разворачиваем документ, если он ещё не был развёрнут
            if (!docs[i].detailsVisible()) {
                docs[i].toogleDetails();
            }


            this.openDocument(docs[i].Model);
            return true;

        },

        navigateToPreviousClassificationError: function () {
            this.navigateToError(-1, this.searchAlgos.CLASSIFICATION);
        },

        // перейти к следующей ошибке сборки
        navigateToNextClassificationError: function () {
            this.navigateToError(1, this.searchAlgos.CLASSIFICATION);
        },

        navigateToPreviousError: function () {
            this.navigateToError(-1);
        },

        // перейти к следующей ошибке сборки
        navigateToNextError: function () {
            this.navigateToError(1);
        },

        // #endregion

        showErrors: function () {
            return true;
        },

        defaultPageSize: { width: 140, height: 158 },

        deleteSeparatorActivated: deleteSeparatorActivated,

        verticalSeparatorDrawActivated: verticalSeparatorDrawActivated,

        horizontalSeparatorDrawActivated: horizontalSeparatorDrawActivated,

        tableEditing: tableEditing,

        selectedTableRow: selectedTableRow,

        selectedTableRowPath: selectedTableRowPath,

        selectedTableCanReanalyze: selectedTableCanReanalyze,

        performAction: function (actionChecker, actionName) {
            if (!actionChecker() || !generalControlsEnabled())
                return;

            window.batchLayout.batchLayoutViewModel.generalControlsExpanded(false);

            if (window.Layouts.ModelLoader &&
                window.Layouts.ModelLoader.viewModel() &&
                window.Layouts.ModelLoader.viewModel().hasOwnProperty(actionName)) {

                window.Layouts.ModelLoader.viewModel()[actionName]();
            }
        },

        rejectAvailable: ko.computed(function () {
            if (!window.Layouts.ModelLoader) {
                return false;
            }

            var viewModel = window.Layouts.ModelLoader.viewModel();
            return viewModel &&
                viewModel.hasOwnProperty('canReject') &&
                viewModel.canReject();
        }),

        rejectTask: function () {
            if (!this.rejectAvailable() || !generalControlsEnabled())
                return;

            window.batchLayout.batchLayoutViewModel.generalControlsExpanded(false);

            if (window.Layouts.ModelLoader &&
                window.Layouts.ModelLoader.viewModel() &&
                window.Layouts.ModelLoader.viewModel().hasOwnProperty('RejectTask')) {

                window.Layouts.ModelLoader.viewModel().ShowReject(hasChangesEver() || documentStatusChanged());
            }
        },

        postponeAvailable: ko.computed(function () {
            if (!window.Layouts.ModelLoader) {
                return false;
            }

            var viewModel = window.Layouts.ModelLoader.viewModel();

            return viewModel &&
                viewModel.hasOwnProperty('canPostpone') &&
                viewModel.canPostpone();
        }),

        postponeTask: function () {
            if (window.Layouts.ModelLoader &&
                window.Layouts.ModelLoader.viewModel() &&
                window.Layouts.ModelLoader.viewModel().hasOwnProperty('RejectTask') &&
                generalControlsEnabled()) {

                window.Layouts.ModelLoader.viewModel().ShowPostpone(hasChangesEver() || documentStatusChanged());
            }
        },

        sendToStageAvailable: ko.computed(function () {
            if (!window.Layouts.ModelLoader) {
                return false;
            }

            var viewModel = window.Layouts.ModelLoader.viewModel();

            return viewModel &&
                viewModel.hasOwnProperty('canSend') &&
                viewModel.canSend();
        }),

        sendToStageTask: function () {
            this.performAction(this.sendToStageAvailable, 'ShowStagesDialog');
        },

        sendToExceptionAvailable: ko.computed(function () {
            if (!window.Layouts.ModelLoader) {
                return false;
            }

            var viewModel = window.Layouts.ModelLoader.viewModel();

            return viewModel &&
                viewModel.hasOwnProperty('canSendToExceptions') &&
                viewModel.canSendToExceptions();
        }),

        sendToExceptionTask: function () {
            this.performAction(this.sendToExceptionAvailable, 'SendToExceptions');
        },

        sendToRescanAvailable: ko.computed(function () {
            if (!window.Layouts.ModelLoader) {
                return false;
            }
            var viewModel = window.Layouts.ModelLoader.viewModel();
            return viewModel && viewModel.hasOwnProperty('canRescan') &&
                viewModel.canRescan();
        }),

        sendToRescanTask: function () {
            this.performAction(this.sendToRescanAvailable, 'ShowRescan');
        },


        completeTaskAvailable: ko.computed(function () {
            if (!window.Layouts.ModelLoader) {
                return false;
            }

            var viewModel = window.Layouts.ModelLoader.viewModel();

            return viewModel &&
                viewModel.hasOwnProperty('canClose') &&
                viewModel.canClose();
        }),

        completeTask: function () {
            this.performAction(this.completeTaskAvailable, 'FinishTask');
        },

        getGeneralControls: function () {

            var finishTaskOrder = 30;
            var recognizeOrder = 35;

            switch (FC.Environment.TaskMenuLocation) {
                case FC.InfrastructureModels.TaskMenuLocation.DockingRight:
                    recognizeOrder = 1;
                    finishTaskOrder = 2;
                    break;
                case FC.InfrastructureModels.TaskMenuLocation.Floating:
                    finishTaskOrder = 30;
                    recognizeOrder = 35;
                    break;
            }

            var controls = [
                {
                    name: FC.Environment.Localization.GlobalResources.RejectTask,
                    title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.RejectTaskButton_Title, FC.Environment.Localization.NonLocalizedHotKeys.RejectTaskHotkey),
                    action: this.rejectTask.bind(this),
                    mainAction: false,
                    htmlCss: "task-action--reject",
                    order: 5,
                    visible: this.rejectAvailable,
                    enabled: generalControlsEnabled,
                    disabledTitle: FC.Environment.Localization.GlobalResources.TaskOperationsDisabledWhenRecognize
                },
                {
                    name: FC.Environment.Localization.GlobalResources.PostponeTask,
                    title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.PostponeTaskButton_Title, FC.Environment.Localization.NonLocalizedHotKeys.PostponeTaskHotkey),
                    action: this.postponeTask.bind(this),
                    mainAction: false,
                    htmlCss: "task-action--postpone",
                    order: 10,
                    visible: this.postponeAvailable,
                    enabled: generalControlsEnabled,
                    disabledTitle: FC.Environment.Localization.GlobalResources.TaskOperationsDisabledWhenRecognize
                },
                {
                    name: FC.Environment.Localization.GlobalResources.SendTaskToStage,
                    title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.SendTaskToStageButton_Title, FC.Environment.Localization.NonLocalizedHotKeys.SendToStageHotkey),
                    htmlCss: "task-action--to-stage",
                    order: 15,
                    action: this.sendToStageTask.bind(this),
                    mainAction: ko.computed(function () {
                        return window.Layouts.ModelLoader &&
                            window.Layouts.ModelLoader.viewModel() &&
                            window.Layouts.ModelLoader.viewModel().hasOwnProperty('canClose') &&
                            !window.Layouts.ModelLoader.viewModel().canClose();
                    }),
                    visible: this.sendToStageAvailable,
                    enabled: generalControlsEnabled,
                    disabledTitle: FC.Environment.Localization.GlobalResources.TaskOperationsDisabledWhenRecognize
                },
                {
                    name: FC.Environment.Localization.GlobalResources.SendTaskToRescan,
                    title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.SendTaskToRescanButton_Title, FC.Environment.Localization.NonLocalizedHotKeys.SendToRescanHotkey),
                    action: this.sendToRescanTask.bind(this),
                    mainAction: false,
                    htmlCss: "task-action--rescan",
                    order: 20,
                    visible: this.sendToRescanAvailable,
                    enabled: generalControlsEnabled,
                    disabledTitle: FC.Environment.Localization.GlobalResources.TaskOperationsDisabledWhenRecognize
                },
                {
                    name: FC.Environment.Localization.GlobalResources.SendTaskToExceptions,
                    title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.SendTaskToExceptionsButton_Title, FC.Environment.Localization.NonLocalizedHotKeys.SendToExceptionHotkey),
                    action: this.sendToExceptionTask.bind(this),
                    mainAction: false,
                    htmlCss: "task-action--to-exception",
                    order: 25,
                    visible: this.sendToExceptionAvailable,
                    enabled: generalControlsEnabled,
                    disabledTitle: FC.Environment.Localization.GlobalResources.TaskOperationsDisabledWhenRecognize
                },
                {
                    name: FC.Environment.Localization.GlobalResources.TaskComplete,
                    title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.TaskCompleteButton_Title, FC.Environment.Localization.NonLocalizedHotKeys.CompleteTaskHotkey),
                    htmlCss: 'task-action--complete task-action--main',
                    order: finishTaskOrder,
                    action: this.completeTask.bind(this),
                    mainAction: ko.computed(function () {
                        return window.Layouts.ModelLoader &&
                            window.Layouts.ModelLoader.viewModel() &&
                            window.Layouts.ModelLoader.viewModel().hasOwnProperty('canClose') &&
                            window.Layouts.ModelLoader.viewModel().canClose();
                    }),
                    visible: this.completeTaskAvailable,
                    enabled: generalControlsEnabled,
                    disabledTitle: FC.Environment.Localization.GlobalResources.TaskOperationsDisabledWhenRecognize
                }
            ];

            return controls;
        },

        batchEditorNonContextOperations: function (be) {
            return [new FC.NonContextFunction({
                name: FC.Environment.Localization.GlobalResources.ApplyChangesButton,
                order: 1,
                action: function () { Sammy.apps.body.trigger('applyChanges'); },
                htmlClass: 'batchEditor__nonContextOperations__applyChanges',
                visible: ko.computed(function () {
                    if (!window.Layouts.ModelLoader) {
                        return false;
                    }

                    return true;
                }),
                title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.ApplyChangesButton, FC.Environment.Localization.NonLocalizedHotKeys.ApplyChangesHotkey),
                enabled: hasChanges
            })];
        },

        batchEditorErrorNavigator: function (batchEditorHasErrors) {
            return [
                new FC.NonContextFunction({
                    name: null,
                    title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.GoToPreviousBatchError, FC.Environment.Localization.NonLocalizedHotKeys.PreviousErrorDocument),
                    order: 1,
                    action: function () { window.batchApp.extensions.navigateToPreviousError() },
                    htmlClass: 'batchEditor__nonContextOperations__previousBatchEditorError',
                    enabled: typeof batchEditorHasErrors != 'undefined' ? batchEditorHasErrors : true
                }),
                new FC.NonContextFunction({
                    name: null,
                    title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.GoToNextBatchError, FC.Environment.Localization.NonLocalizedHotKeys.NextErrorDocument),
                    order: 2,
                    action: function () { window.batchApp.extensions.navigateToNextError() },
                    htmlClass: 'batchEditor__nonContextOperations__nextBatchEditorError',
                    enabled: typeof batchEditorHasErrors != 'undefined' ? batchEditorHasErrors : true
                })
            ];
        },

        toggleDeleteSeparatorMode: function () {
            if (!tableEditing()) {
                return;
            }

            if (!deleteSeparatorActivated()) {
                verticalSeparatorDrawActivated(false);
                horizontalSeparatorDrawActivated(false);
            }

            deleteSeparatorActivated(!deleteSeparatorActivated());
        },

        toggleAddVerticalSeparatorMode: function () {
            if (!tableEditing()) {
                return;
            }

            if (!verticalSeparatorDrawActivated()) {
                deleteSeparatorActivated(false);
                horizontalSeparatorDrawActivated(false);
            }

            verticalSeparatorDrawActivated(!verticalSeparatorDrawActivated());
        },

        toggleAddHorizontalSeparatorMode: function () {
            if (!tableEditing()) {
                return;
            }

            if (!horizontalSeparatorDrawActivated()) {
                verticalSeparatorDrawActivated(false);
                deleteSeparatorActivated(false);
            }

            horizontalSeparatorDrawActivated(!horizontalSeparatorDrawActivated());
        },

        documentEditorNonContextOperations: function (de) {
            var deleteSeparatorFunc = new FC.NonContextFunction({
                name: FC.Environment.Localization.GlobalResources.DelSeparatorToolTip,
                title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.DelSeparatorToolTip,
                    FC.Environment.Localization.NonLocalizedHotKeys.RemoveSeparatorHotkey),
                order: 3,
                action: this.toggleDeleteSeparatorMode,
                htmlClass: "docEditor_operations_deleteSeparator docEditor_operations_minimized",
                visible: tableEditing,
                enabled: ko.observable(true),
                activated: deleteSeparatorActivated,
                textVisible: false
            });

            var addVerticalSeparatorFunc = new FC.NonContextFunction({
                name: FC.Environment.Localization.GlobalResources.AddVerticalSeparator,
                title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.AddVerticalSeparator,
                    FC.Environment.Localization.NonLocalizedHotKeys.AddVerticalSeparatorHotkey),
                order: 1,
                action: this.toggleAddVerticalSeparatorMode,
                htmlClass: "docEditor_operations_addVerticalSeparator  docEditor_operations_minimized",
                visible: tableEditing,
                enabled: ko.observable(true),
                activated: verticalSeparatorDrawActivated,
                textVisible: false
            });

            var addHorizontalSeparatorFunc = new FC.NonContextFunction({
                name: FC.Environment.Localization.GlobalResources.AddHorizontalSeparator,
                title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.AddHorizontalSeparator,
                    FC.Environment.Localization.NonLocalizedHotKeys.AddHorizontalSeparatorHotkey),
                order: 2,
                action: this.toggleAddHorizontalSeparatorMode,
                htmlClass: "docEditor_operations_addHorizontalSeparator  docEditor_operations_minimized",
                visible: tableEditing,
                enabled: ko.observable(true),
                activated: horizontalSeparatorDrawActivated,
                textVisible: false
            });

            var continueLineItemsFunc = new FC.NonContextFunction({
                name: FC.Environment.Localization.GlobalResources.ContinueLineItems,
                title: FC.Environment.Localization.GlobalResources.ContinueLineItems,
                order: 4,
                action: function () {
                    if (selectedTableRow() > -1 && checkCountryCode()) {
                        window.batchLayout.batchLayoutViewModel.UserEditor.extrapolateTable(selectedTableRow(), false, selectedTableRowPath());
                    }
                },
                htmlClass: "docEditor_operations_continueLineItems  docEditor_operations_minimized",
                visible: ko.computed(function () { return selectedTableRow() > -1; }),
                enabled: ko.observable(true),
                textVisible: false
            });

            var reanalyzeLineItemsFunc = new FC.NonContextFunction({
                name: FC.Environment.Localization.GlobalResources.ReanalizeLineItems,
                title: FC.Environment.Localization.GlobalResources.ReanalizeLineItems,
                order: 5,
                action: function () {
                    if (selectedTableCanReanalyze() && selectedTableRow() > -1 && checkCountryCode()) {
                        window.batchLayout.batchLayoutViewModel.UserEditor.extrapolateTable(selectedTableRow(), true, selectedTableRowPath());
                    }
                },
                htmlClass: "docEditor_operations_reanalyzeLineItems  docEditor_operations_minimized",
                visible: ko.computed(function () { return selectedTableCanReanalyze() && selectedTableRow() > -1; }),
                enabled: ko.observable(true),
                textVisible: false
            });

            return [addVerticalSeparatorFunc, addHorizontalSeparatorFunc, deleteSeparatorFunc, continueLineItemsFunc, reanalyzeLineItemsFunc];
        },
        docEditorBottomPanelVisible: docEditorBottomPanelVisible,
        zoomable: function () {
            return false;
        },

        createDocumentEditor: function () {
            return null;
        },

        openRightAutomatically: true,
        autoGetModeSelectable: FC.Environment.StationType === FC.StationType.WebVerification && !FC.Utils.getQueryParameter("mode"),
        autoGetMode: autoGetMode,

        zoomFitWidth: function () { return FC.Environment.Localization.GlobalResources.ZoomFitWidth; },

        zoomFitHeight: function () { return FC.Environment.Localization.GlobalResources.ZoomFitHeight; },

        imageZoomIn: function () { FC.Viewmodels.DocumentEditor.current.ZoomIn(); },
        hasChangesEver: hasChangesEver,
        imageZoomOut: function () { FC.Viewmodels.DocumentEditor.current.ZoomOut(); },
        UserEditor: FC.Viewmodels.GeneralViewmodel,
        hasChanges: hasChanges,

        colorSwitcherMode: FC.BatchEditor.batchEditorDataContext.ImageLoadMode,
        switchLoadImagesMode: function () {
            var newLoadMode = !FC.BatchEditor.batchEditorDataContext.ImageLoadMode();
            FC.BatchEditor.batchEditorDataContext.ImageLoadMode(newLoadMode);
            FC.DataContext.Server.changeLoadImagesMode(newLoadMode
                ? FC.BusinessModels.LoadImagesModes.Color
                : FC.BusinessModels.LoadImagesModes.BlackAndWhite);
        },

        init: function (proxy) {
            proxy.documentEditor.extender = this; // TODO: зависимость от extender лучше заменить DI

            proxy.store.dispatch(proxy.documentEditorActionCreators.updateOptions([{
                option: {
                    id: optionCodes.BlackAndWhiteImages,
                    name: FC.Environment.Localization.GlobalResources.ShowBlackWhiteImagesTitle
                },
                isEnabled: !FC.BatchEditor.batchEditorDataContext.ImageLoadMode()
            }]));

            proxy.store.subscribe(function () {
                var state = proxy.store.getState();
                var options = state.documentEditor.enabledOptions;

                var newLoadMode = !options[optionCodes.BlackAndWhiteImages];

                if (newLoadMode !== FC.BatchEditor.batchEditorDataContext.ImageLoadMode()) {

                    FC.BatchEditor.batchEditorDataContext.ImageLoadMode(newLoadMode);
                    FC.DataContext.Server.changeLoadImagesMode(newLoadMode
                        ? FC.BusinessModels.LoadImagesModes.Color
                        : FC.BusinessModels.LoadImagesModes.BlackAndWhite);

                    var action = proxy.batchActionCreators.updatePages(
                        state.batch.pages.values().map(function (page) {
                            return {
                                id: page.id,
                                imageWidth: page.imageWidth,
                                imageHeight: page.imageHeight,
                                title: page.title,
                                imageUri: window.batchApp.datacontext.getPageUrl(page, newLoadMode && page.hasColorImage),
                                parentDocumentId: page.parentDocumentId,
                                hasColorImage: page.hasColorImage,
                                version: page.version,
                                rotation: page.rotation
                            };
                        })
                    );
                    proxy.store.dispatch(action);
                    window.batchLayoutComponent.store.dispatch(action);
                }
            });

            var updateCommands = function () {
                var isTableEditing = tableEditing();
                var rowSelected = selectedTableRow() > -1;
                var commands = [];

                if (isTableEditing) {
                    commands = commands.concat([
                        {
                            id: commandsCodes.AddVerticalSeparator,
                            title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.AddVerticalSeparator,
                                FC.Environment.Localization.NonLocalizedHotKeys.AddVerticalSeparatorHotkey),
                            imageClassName: 'documentEditor__icon docEditor_operations_addVerticalSeparator_icon'
                        }, {
                            id: commandsCodes.AddHorizontalSeparator,
                            title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.AddHorizontalSeparator,
                                FC.Environment.Localization.NonLocalizedHotKeys.AddHorizontalSeparatorHotkey),
                            imageClassName: 'documentEditor__icon docEditor_operations_addHorizontalSeparator_icon'
                        }, {
                            id: commandsCodes.DeleteSeparator,
                            title: FC.Utils.HotkeyTooltips.GenerateTooltipWithHotkey(FC.Environment.Localization.GlobalResources.DelSeparatorToolTip,
                                FC.Environment.Localization.NonLocalizedHotKeys.RemoveSeparatorHotkey),
                            imageClassName: 'documentEditor__icon docEditor_operations_deleteSeparator_icon'
                        }
                    ]);
                } else {
                    proxy.store.dispatch(proxy.documentEditorActionCreators.activateCommand(-1));
                }

                if (rowSelected) {
                    var additionalCommands = [{
                        id: commandsCodes.ContinueLineItems,
                        title: FC.Environment.Localization.GlobalResources.ContinueLineItems,
                        imageClassName: 'documentEditor__icon docEditor_operations_continueLineItems_icon'
                    }];

                    if (selectedTableCanReanalyze()) {
                        additionalCommands.push({
                            id: commandsCodes.ReanalizeLineItems,
                            title: FC.Environment.Localization.GlobalResources.ReanalizeLineItems,
                            imageClassName: 'documentEditor__icon docEditor_operations_reanalyzeLineItems_icon'
                        });
                    }

                    commands = commands.concat(additionalCommands);
                }

                proxy.store.dispatch(proxy.documentEditorActionCreators.updateCommands(commands));
            }

            tableEditing.subscribe(updateCommands);
            selectedTableRow.subscribe(updateCommands);

            docEditorBottomPanelVisible.subscribe(function (isVisible) {
                if (isVisible) {
                    proxy.store.dispatch(proxy.layoutActionCreators.showBottomPanel());
                } else {
                    proxy.store.dispatch(proxy.layoutActionCreators.hideBottomPanel());
                }
            });
        },

        onCommandExecute: function (proxy, commandId) {
            switch (commandId) {
                case commandsCodes.AddVerticalSeparator:
                    this.toggleAddVerticalSeparatorMode();
                    proxy.store.dispatch(proxy.documentEditorActionCreators.activateCommand(verticalSeparatorDrawActivated() ? commandsCodes.AddVerticalSeparator : -1));
                    break;

                case commandsCodes.AddHorizontalSeparator:
                    this.toggleAddHorizontalSeparatorMode();
                    proxy.store.dispatch(proxy.documentEditorActionCreators.activateCommand(horizontalSeparatorDrawActivated() ? commandsCodes.AddHorizontalSeparator : -1));
                    break;

                case commandsCodes.DeleteSeparator:
                    this.toggleDeleteSeparatorMode();
                    proxy.store.dispatch(proxy.documentEditorActionCreators.activateCommand(deleteSeparatorActivated() ? commandsCodes.DeleteSeparator : -1));
                    break;

                case commandsCodes.ContinueLineItems:
                    if (selectedTableRow() > -1 && checkCountryCode()) {
                        window.batchLayout.batchLayoutViewModel.UserEditor.extrapolateTable(selectedTableRow(), false, selectedTableRowPath());
                    }
                    break;

                case commandsCodes.ReanalizeLineItems:
                    if (selectedTableRow() > -1 && checkCountryCode()) {
                        window.batchLayout.batchLayoutViewModel.UserEditor.extrapolateTable(selectedTableRow(), true, selectedTableRowPath());
                    }
                    break;
            }
        }
    };

    return extender;
})();
