﻿FC.Viewmodels.documentDefinitionsContext = {
	documentDefinitions: ko.observableArray([]),

	fullTemplates: {},

	getDocumentDefinitionByName: function (templateName) {
		var currentDefinitions = this.documentDefinitions();
		for (var i = 0; i < currentDefinitions.length; i++) {
			if (currentDefinitions[i].Name === templateName) {
				return currentDefinitions[i];
			}
		}

		return null;
	},

	_generateLocalId: function (templateId, templateVersion) {
		return templateId + '_' + templateVersion;
	},

	getDocumentDefinitionFullInfo: function (templateId, templateVersion, templateDataContext, onSuccess, forceLoad) {
		var localId = this._generateLocalId(templateId, templateVersion);

		if (this.fullTemplates.hasOwnProperty(localId) && !forceLoad) {
			if (onSuccess) {
				onSuccess(this.fullTemplates[localId]);
			}

			return;
		}

		templateDataContext.getDocumentDefinition(
			templateId,
			templateVersion,
			function (template) {
				FC.Viewmodels.documentDefinitionsContext.fullTemplates[localId] = template;

				if (onSuccess) {
					onSuccess(template);
				}
			});
	},

	loadDocumentDefinitions: function (batchId, templateDataContext, onSuccess) {
		templateDataContext.getAllDocumentDefinitions(
			batchId,
			function (templates) {
				FC.Viewmodels.documentDefinitionsContext.documentDefinitions(ko.utils.arrayMap(templates,
					function (item) {
						return new FC.BusinessModels.DocumentTemplateInfo(item);
					}));

				if (onSuccess) {
					onSuccess();
				}
			});
	},

	createFieldFromTemplate: function (fieldName, docTemplate) {
		var fieldTemplate = this.findFieldDefinitionInTemplate(fieldName, docTemplate);

		if (!fieldTemplate) {
			return null;
		}

		return this.createField(fieldTemplate);
	},

	createField: function (fieldTemplate, useIdFromTemplate, parentLongId, allowedFields) {
		var thisLongId = !parentLongId ? null : parentLongId + "\\" + fieldTemplate.Id;
		var isAllowed = this.checkFieldIsAllowed(allowedFields, thisLongId);

		if (!fieldTemplate || !fieldTemplate.hasOwnProperty('Type')) {
			return null;
		}

		switch (fieldTemplate.Type) {
			// Group
			case 1:
				return this.createGroup(fieldTemplate, useIdFromTemplate, parentLongId, allowedFields);

				// CheckmarkGroup
			case 2:
				return this.createCheckList(fieldTemplate, useIdFromTemplate, parentLongId, allowedFields);

				// Table
			case 4:
				return this.createTable(fieldTemplate, useIdFromTemplate, parentLongId, allowedFields);
		}

		// ещё может быть секция, но пока не нужна
		// cледовательно simple
		var field = {
			Name: fieldTemplate.Name,
			Caption: fieldTemplate.Caption ? fieldTemplate.Caption : fieldTemplate.Name,
			IsHidden: !isAllowed || fieldTemplate.IsVisible =='false'
		};

		if (useIdFromTemplate) {
			field.Id = fieldTemplate.Id;
		}

		if (isAllowed) {
			// Boolean
			if (fieldTemplate.ValueDef.BasicValueInfo.Type == 0) {
				field.BooleanValue = { Value: false };
			} else {
				field.TextValue = { Text: '' };
			}
		}

		return field;
	},

	findFieldDefinitionInTemplate: function (fieldName, docTemplate, fieldId) {
		if (!fieldName || !docTemplate || !docTemplate.hasOwnProperty('Section')) {
			return null;
		}

		for (var i = 0; i < docTemplate.Section.length; i++) {
			if (docTemplate.Section[i]
				.Name ==
				fieldName &&
				(!fieldId || fieldId === -1 || docTemplate.Section[i].Id == fieldId)) {
				return docTemplate.Section[i];
			}

			var def = this.findFieldDefinition(fieldName, docTemplate.Section[i], fieldId);
			if (def) {
				return def;
			}
		}

		return null;
	},

	findFieldDefinition: function (fieldName, fieldTemplate, fieldId) {
		if (!fieldName || !fieldTemplate || !fieldTemplate.hasOwnProperty('Field') || !fieldTemplate.Field) {
			return null;
		}

		for (var i = 0; i < fieldTemplate.Field.length; i++) {
			if (fieldTemplate.Field[i]
				.Name ==
				fieldName &&
				(!fieldId || fieldId === -1 || fieldTemplate.Field[i].Id == fieldId)) {
				return fieldTemplate.Field[i];
			}

			var def = this.findFieldDefinition(fieldName, fieldTemplate.Field[i], fieldId);
			if (def) {
				return def;
			}
		}

		return null;
	},

	createTableRowFromTemplate: function (fieldName, docTemplate) {
		var fieldTemplate = this.findFieldDefinitionInTemplate(fieldName, docTemplate);
		if (!fieldTemplate) {
			return null;
		}

		var field = { Cell: [] };
		for (var i = 0; i < fieldTemplate.Field.length; i++) {
			field.Cell.push(this.createField(fieldTemplate.Field[i]));
			field.Cell[field.Cell.length - 1].Id = fieldTemplate.Field[i].Id;
		}

		return field;
	},

	createFieldInstanceFromTemplate: function (fieldName, docTemplate) {
		var fieldTemplate = this.findFieldDefinitionInTemplate(fieldName, docTemplate);

		var fieldLongId = this.generateFieldLongId(fieldTemplate, docTemplate);
		return this.createFieldInstanceFromFieldTemplate(fieldTemplate, fieldLongId);
	},

	generateFieldLongIdInField: function (fieldTemplate, parentTemplate) {
		if (!fieldTemplate || !parentTemplate) {
			return null;
		}

		if (!parentTemplate.Field) {
			return null;
		}

		for (var i = 0; i < parentTemplate.Field.length; i++) {
			if (parentTemplate.Field[i].Id == fieldTemplate.Id && parentTemplate.Field[i].Name == fieldTemplate.Name) {
				return fieldTemplate.Id;
			}

			var childId = this.generateFieldLongIdInField(fieldTemplate, parentTemplate.Field[i]);
			if (childId) {
				return parentTemplate.Field[i].Id + "\\" + childId;
			}
		}

		return null;
	},

	generateFieldLongId: function (fieldTemplate, docTemplate) {
		if (!fieldTemplate || !docTemplate) {
			return null;
		}

		for (var i = 0; i < docTemplate.Section.length; i++) {
			if (docTemplate.Section[i].Id == fieldTemplate.Id && docTemplate.Section[i].Name == fieldTemplate.Name) {
				return fieldTemplate.Id;
			}

			var childId = this.generateFieldLongIdInField(fieldTemplate, docTemplate.Section[i]);
			if (childId) {
				return docTemplate.Section[i].Id + "\\" + childId;
			}
		}

		return null;
	},

	checkFieldIsAllowed: function (allowedFields, fieldLongId) {
		if (!allowedFields || allowedFields.AllFields) {
			return true;
		}

		if (!allowedFields.Fields) {
			return false;
		}

		for (var i = 0; i < allowedFields.Fields.length; i++) {
			if (allowedFields.Fields[i].Id == fieldLongId) {
				return true;
			}
		}

		return false;
	},

	createFieldInstanceFromFieldTemplate: function (fieldTemplate, fieldLongId) {
		if (!fieldTemplate) {
			return null;
		}

		var allowedFields = FC.Viewmodels.DocumentEditor.current.currentDocumentFieldSet();

		if (fieldTemplate.Field) {
			var field = {
				Name: fieldTemplate.Name,
				Caption: fieldTemplate.Caption ? fieldTemplate.Caption : fieldTemplate.Name,
				Children: { Field: [], Table: [] }
			};

			if (!fieldTemplate.IsRepeatable) {
				field.Id = fieldTemplate.Id;
			}

			for (var i = 0; i < fieldTemplate.Field.length; i++) {
				if (fieldTemplate.Field[i].Type == 4) {
					field.Children.Table.push(this.createField(fieldTemplate.Field[i], true, fieldLongId, allowedFields));
				} else {
					field.Children.Field.push(this.createField(fieldTemplate.Field[i], true, fieldLongId, allowedFields));
				}
			}

			return field;
		}

		return this.createField(fieldTemplate);
	},

	createTable: function (fieldTemplate, useIdFromTemplate, parentLongId, allowedFields) {
		var thisLongId = !parentLongId ? null : parentLongId + "\\" + fieldTemplate.Id;
		var isAllowed = this.checkFieldIsAllowed(allowedFields, thisLongId);

		var table = {
			Name: fieldTemplate.Name,
			Caption: fieldTemplate.Caption ? fieldTemplate.Caption : fieldTemplate.Name,
			Regions: [{ Region: [] }],
			Id: fieldTemplate.Id,
			IsHidden: !isAllowed
		};

		return table;
	},

	createGroup: function (fieldTemplate, useIdFromTemplate, parentLongId, allowedFields) {
		var thisLongId = !parentLongId ? null : parentLongId + "\\" + fieldTemplate.Id;
		var isAllowed = this.checkFieldIsAllowed(allowedFields, thisLongId);

		if (fieldTemplate.IsRepeatable) {
			return {
				Name: fieldTemplate.Name,
				Caption: fieldTemplate.Caption ? fieldTemplate.Caption : fieldTemplate.Name,
				Id: fieldTemplate.Id,
				IsHidden: !isAllowed
			};
		}

		return this.createFieldInstanceFromFieldTemplate(fieldTemplate, thisLongId);
	},

	createCheckList: function (fieldTemplate, useIdFromTemplate, parentLongId, allowedFields) {
		var thisLongId = !parentLongId ? null : parentLongId + "\\" + fieldTemplate.Id;
		var isAllowed = this.checkFieldIsAllowed(allowedFields, thisLongId);

		var field = {
			Name: fieldTemplate.Name,
			Caption: fieldTemplate.Caption ? fieldTemplate.Caption : fieldTemplate.Name,
			IsHidden: !isAllowed,
			ChoiceValue: isAllowed ? {
				Value: false,
				MinSelectedCount: fieldTemplate.ValueDef.ChoiceFormat.MinSelected,
				MaxSelectedCount: fieldTemplate.ValueDef.ChoiceFormat.MaxSelected,
				MinSelectedCountSpecified: true,
				MaxSelectedCountSpecified: true
			} : null,
			Children: { Field: [] }
		};

		if (useIdFromTemplate) {
			field.Id = fieldTemplate.Id;
		}

		if (isAllowed) {
			for (var i = 0; i < fieldTemplate.Field.length; i++) {
				field.Children.Field.push(this.createField(fieldTemplate.Field[i], useIdFromTemplate));
			}
		}

		return field;
	}
};